/* Generated by Cython 0.28.1 */

/* BEGIN: Cython Metadata
{
    "distutils": {
        "name": "pymarketcap.core",
        "sources": [
            "pymarketcap/core.pyx"
        ]
    },
    "module_name": "pymarketcap.core"
}
END: Cython Metadata */

#define PY_SSIZE_T_CLEAN
#include "Python.h"
#ifndef Py_PYTHON_H
    #error Python headers needed to compile C extensions, please install development version of Python.
#elif PY_VERSION_HEX < 0x02060000 || (0x03000000 <= PY_VERSION_HEX && PY_VERSION_HEX < 0x03030000)
    #error Cython requires Python 2.6+ or Python 3.3+.
#else
#define CYTHON_ABI "0_28_1"
#define CYTHON_FUTURE_DIVISION 0
#include <stddef.h>
#ifndef offsetof
  #define offsetof(type, member) ( (size_t) & ((type*)0) -> member )
#endif
#if !defined(WIN32) && !defined(MS_WINDOWS)
  #ifndef __stdcall
    #define __stdcall
  #endif
  #ifndef __cdecl
    #define __cdecl
  #endif
  #ifndef __fastcall
    #define __fastcall
  #endif
#endif
#ifndef DL_IMPORT
  #define DL_IMPORT(t) t
#endif
#ifndef DL_EXPORT
  #define DL_EXPORT(t) t
#endif
#define __PYX_COMMA ,
#ifndef HAVE_LONG_LONG
  #if PY_VERSION_HEX >= 0x02070000
    #define HAVE_LONG_LONG
  #endif
#endif
#ifndef PY_LONG_LONG
  #define PY_LONG_LONG LONG_LONG
#endif
#ifndef Py_HUGE_VAL
  #define Py_HUGE_VAL HUGE_VAL
#endif
#ifdef PYPY_VERSION
  #define CYTHON_COMPILING_IN_PYPY 1
  #define CYTHON_COMPILING_IN_PYSTON 0
  #define CYTHON_COMPILING_IN_CPYTHON 0
  #undef CYTHON_USE_TYPE_SLOTS
  #define CYTHON_USE_TYPE_SLOTS 0
  #undef CYTHON_USE_PYTYPE_LOOKUP
  #define CYTHON_USE_PYTYPE_LOOKUP 0
  #if PY_VERSION_HEX < 0x03050000
    #undef CYTHON_USE_ASYNC_SLOTS
    #define CYTHON_USE_ASYNC_SLOTS 0
  #elif !defined(CYTHON_USE_ASYNC_SLOTS)
    #define CYTHON_USE_ASYNC_SLOTS 1
  #endif
  #undef CYTHON_USE_PYLIST_INTERNALS
  #define CYTHON_USE_PYLIST_INTERNALS 0
  #undef CYTHON_USE_UNICODE_INTERNALS
  #define CYTHON_USE_UNICODE_INTERNALS 0
  #undef CYTHON_USE_UNICODE_WRITER
  #define CYTHON_USE_UNICODE_WRITER 0
  #undef CYTHON_USE_PYLONG_INTERNALS
  #define CYTHON_USE_PYLONG_INTERNALS 0
  #undef CYTHON_AVOID_BORROWED_REFS
  #define CYTHON_AVOID_BORROWED_REFS 1
  #undef CYTHON_ASSUME_SAFE_MACROS
  #define CYTHON_ASSUME_SAFE_MACROS 0
  #undef CYTHON_UNPACK_METHODS
  #define CYTHON_UNPACK_METHODS 0
  #undef CYTHON_FAST_THREAD_STATE
  #define CYTHON_FAST_THREAD_STATE 0
  #undef CYTHON_FAST_PYCALL
  #define CYTHON_FAST_PYCALL 0
  #undef CYTHON_PEP489_MULTI_PHASE_INIT
  #define CYTHON_PEP489_MULTI_PHASE_INIT 0
  #undef CYTHON_USE_TP_FINALIZE
  #define CYTHON_USE_TP_FINALIZE 0
#elif defined(PYSTON_VERSION)
  #define CYTHON_COMPILING_IN_PYPY 0
  #define CYTHON_COMPILING_IN_PYSTON 1
  #define CYTHON_COMPILING_IN_CPYTHON 0
  #ifndef CYTHON_USE_TYPE_SLOTS
    #define CYTHON_USE_TYPE_SLOTS 1
  #endif
  #undef CYTHON_USE_PYTYPE_LOOKUP
  #define CYTHON_USE_PYTYPE_LOOKUP 0
  #undef CYTHON_USE_ASYNC_SLOTS
  #define CYTHON_USE_ASYNC_SLOTS 0
  #undef CYTHON_USE_PYLIST_INTERNALS
  #define CYTHON_USE_PYLIST_INTERNALS 0
  #ifndef CYTHON_USE_UNICODE_INTERNALS
    #define CYTHON_USE_UNICODE_INTERNALS 1
  #endif
  #undef CYTHON_USE_UNICODE_WRITER
  #define CYTHON_USE_UNICODE_WRITER 0
  #undef CYTHON_USE_PYLONG_INTERNALS
  #define CYTHON_USE_PYLONG_INTERNALS 0
  #ifndef CYTHON_AVOID_BORROWED_REFS
    #define CYTHON_AVOID_BORROWED_REFS 0
  #endif
  #ifndef CYTHON_ASSUME_SAFE_MACROS
    #define CYTHON_ASSUME_SAFE_MACROS 1
  #endif
  #ifndef CYTHON_UNPACK_METHODS
    #define CYTHON_UNPACK_METHODS 1
  #endif
  #undef CYTHON_FAST_THREAD_STATE
  #define CYTHON_FAST_THREAD_STATE 0
  #undef CYTHON_FAST_PYCALL
  #define CYTHON_FAST_PYCALL 0
  #undef CYTHON_PEP489_MULTI_PHASE_INIT
  #define CYTHON_PEP489_MULTI_PHASE_INIT 0
  #undef CYTHON_USE_TP_FINALIZE
  #define CYTHON_USE_TP_FINALIZE 0
#else
  #define CYTHON_COMPILING_IN_PYPY 0
  #define CYTHON_COMPILING_IN_PYSTON 0
  #define CYTHON_COMPILING_IN_CPYTHON 1
  #ifndef CYTHON_USE_TYPE_SLOTS
    #define CYTHON_USE_TYPE_SLOTS 1
  #endif
  #if PY_VERSION_HEX < 0x02070000
    #undef CYTHON_USE_PYTYPE_LOOKUP
    #define CYTHON_USE_PYTYPE_LOOKUP 0
  #elif !defined(CYTHON_USE_PYTYPE_LOOKUP)
    #define CYTHON_USE_PYTYPE_LOOKUP 1
  #endif
  #if PY_MAJOR_VERSION < 3
    #undef CYTHON_USE_ASYNC_SLOTS
    #define CYTHON_USE_ASYNC_SLOTS 0
  #elif !defined(CYTHON_USE_ASYNC_SLOTS)
    #define CYTHON_USE_ASYNC_SLOTS 1
  #endif
  #if PY_VERSION_HEX < 0x02070000
    #undef CYTHON_USE_PYLONG_INTERNALS
    #define CYTHON_USE_PYLONG_INTERNALS 0
  #elif !defined(CYTHON_USE_PYLONG_INTERNALS)
    #define CYTHON_USE_PYLONG_INTERNALS 1
  #endif
  #ifndef CYTHON_USE_PYLIST_INTERNALS
    #define CYTHON_USE_PYLIST_INTERNALS 1
  #endif
  #ifndef CYTHON_USE_UNICODE_INTERNALS
    #define CYTHON_USE_UNICODE_INTERNALS 1
  #endif
  #if PY_VERSION_HEX < 0x030300F0
    #undef CYTHON_USE_UNICODE_WRITER
    #define CYTHON_USE_UNICODE_WRITER 0
  #elif !defined(CYTHON_USE_UNICODE_WRITER)
    #define CYTHON_USE_UNICODE_WRITER 1
  #endif
  #ifndef CYTHON_AVOID_BORROWED_REFS
    #define CYTHON_AVOID_BORROWED_REFS 0
  #endif
  #ifndef CYTHON_ASSUME_SAFE_MACROS
    #define CYTHON_ASSUME_SAFE_MACROS 1
  #endif
  #ifndef CYTHON_UNPACK_METHODS
    #define CYTHON_UNPACK_METHODS 1
  #endif
  #ifndef CYTHON_FAST_THREAD_STATE
    #define CYTHON_FAST_THREAD_STATE 1
  #endif
  #ifndef CYTHON_FAST_PYCALL
    #define CYTHON_FAST_PYCALL 1
  #endif
  #ifndef CYTHON_PEP489_MULTI_PHASE_INIT
    #define CYTHON_PEP489_MULTI_PHASE_INIT (0 && PY_VERSION_HEX >= 0x03050000)
  #endif
  #ifndef CYTHON_USE_TP_FINALIZE
    #define CYTHON_USE_TP_FINALIZE (PY_VERSION_HEX >= 0x030400a1)
  #endif
#endif
#if !defined(CYTHON_FAST_PYCCALL)
#define CYTHON_FAST_PYCCALL  (CYTHON_FAST_PYCALL && PY_VERSION_HEX >= 0x030600B1)
#endif
#if CYTHON_USE_PYLONG_INTERNALS
  #include "longintrepr.h"
  #undef SHIFT
  #undef BASE
  #undef MASK
#endif
#ifndef __has_attribute
  #define __has_attribute(x) 0
#endif
#ifndef __has_cpp_attribute
  #define __has_cpp_attribute(x) 0
#endif
#ifndef CYTHON_RESTRICT
  #if defined(__GNUC__)
    #define CYTHON_RESTRICT __restrict__
  #elif defined(_MSC_VER) && _MSC_VER >= 1400
    #define CYTHON_RESTRICT __restrict
  #elif defined (__STDC_VERSION__) && __STDC_VERSION__ >= 199901L
    #define CYTHON_RESTRICT restrict
  #else
    #define CYTHON_RESTRICT
  #endif
#endif
#ifndef CYTHON_UNUSED
# if defined(__GNUC__)
#   if !(defined(__cplusplus)) || (__GNUC__ > 3 || (__GNUC__ == 3 && __GNUC_MINOR__ >= 4))
#     define CYTHON_UNUSED __attribute__ ((__unused__))
#   else
#     define CYTHON_UNUSED
#   endif
# elif defined(__ICC) || (defined(__INTEL_COMPILER) && !defined(_MSC_VER))
#   define CYTHON_UNUSED __attribute__ ((__unused__))
# else
#   define CYTHON_UNUSED
# endif
#endif
#ifndef CYTHON_MAYBE_UNUSED_VAR
#  if defined(__cplusplus)
     template<class T> void CYTHON_MAYBE_UNUSED_VAR( const T& ) { }
#  else
#    define CYTHON_MAYBE_UNUSED_VAR(x) (void)(x)
#  endif
#endif
#ifndef CYTHON_NCP_UNUSED
# if CYTHON_COMPILING_IN_CPYTHON
#  define CYTHON_NCP_UNUSED
# else
#  define CYTHON_NCP_UNUSED CYTHON_UNUSED
# endif
#endif
#define __Pyx_void_to_None(void_result) ((void)(void_result), Py_INCREF(Py_None), Py_None)
#ifdef _MSC_VER
    #ifndef _MSC_STDINT_H_
        #if _MSC_VER < 1300
           typedef unsigned char     uint8_t;
           typedef unsigned int      uint32_t;
        #else
           typedef unsigned __int8   uint8_t;
           typedef unsigned __int32  uint32_t;
        #endif
    #endif
#else
   #include <stdint.h>
#endif
#ifndef CYTHON_FALLTHROUGH
  #if defined(__cplusplus) && __cplusplus >= 201103L
    #if __has_cpp_attribute(fallthrough)
      #define CYTHON_FALLTHROUGH [[fallthrough]]
    #elif __has_cpp_attribute(clang::fallthrough)
      #define CYTHON_FALLTHROUGH [[clang::fallthrough]]
    #elif __has_cpp_attribute(gnu::fallthrough)
      #define CYTHON_FALLTHROUGH [[gnu::fallthrough]]
    #endif
  #endif
  #ifndef CYTHON_FALLTHROUGH
    #if __has_attribute(fallthrough)
      #define CYTHON_FALLTHROUGH __attribute__((fallthrough))
    #else
      #define CYTHON_FALLTHROUGH
    #endif
  #endif
  #if defined(__clang__ ) && defined(__apple_build_version__)
    #if __apple_build_version__ < 7000000
      #undef  CYTHON_FALLTHROUGH
      #define CYTHON_FALLTHROUGH
    #endif
  #endif
#endif

#ifndef CYTHON_INLINE
  #if defined(__clang__)
    #define CYTHON_INLINE __inline__ __attribute__ ((__unused__))
  #elif defined(__GNUC__)
    #define CYTHON_INLINE __inline__
  #elif defined(_MSC_VER)
    #define CYTHON_INLINE __inline
  #elif defined (__STDC_VERSION__) && __STDC_VERSION__ >= 199901L
    #define CYTHON_INLINE inline
  #else
    #define CYTHON_INLINE
  #endif
#endif

#if CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX < 0x02070600 && !defined(Py_OptimizeFlag)
  #define Py_OptimizeFlag 0
#endif
#define __PYX_BUILD_PY_SSIZE_T "n"
#define CYTHON_FORMAT_SSIZE_T "z"
#if PY_MAJOR_VERSION < 3
  #define __Pyx_BUILTIN_MODULE_NAME "__builtin__"
  #define __Pyx_PyCode_New(a, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)\
          PyCode_New(a+k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)
  #define __Pyx_DefaultClassType PyClass_Type
#else
  #define __Pyx_BUILTIN_MODULE_NAME "builtins"
  #define __Pyx_PyCode_New(a, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)\
          PyCode_New(a, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)
  #define __Pyx_DefaultClassType PyType_Type
#endif
#ifndef Py_TPFLAGS_CHECKTYPES
  #define Py_TPFLAGS_CHECKTYPES 0
#endif
#ifndef Py_TPFLAGS_HAVE_INDEX
  #define Py_TPFLAGS_HAVE_INDEX 0
#endif
#ifndef Py_TPFLAGS_HAVE_NEWBUFFER
  #define Py_TPFLAGS_HAVE_NEWBUFFER 0
#endif
#ifndef Py_TPFLAGS_HAVE_FINALIZE
  #define Py_TPFLAGS_HAVE_FINALIZE 0
#endif
#if PY_VERSION_HEX <= 0x030700A3 || !defined(METH_FASTCALL)
  #ifndef METH_FASTCALL
     #define METH_FASTCALL 0x80
  #endif
  typedef PyObject *(*__Pyx_PyCFunctionFast) (PyObject *self, PyObject *const *args, Py_ssize_t nargs);
  typedef PyObject *(*__Pyx_PyCFunctionFastWithKeywords) (PyObject *self, PyObject *const *args,
                                                          Py_ssize_t nargs, PyObject *kwnames);
#else
  #define __Pyx_PyCFunctionFast _PyCFunctionFast
  #define __Pyx_PyCFunctionFastWithKeywords _PyCFunctionFastWithKeywords
#endif
#if CYTHON_FAST_PYCCALL
#define __Pyx_PyFastCFunction_Check(func)\
    ((PyCFunction_Check(func) && (METH_FASTCALL == (PyCFunction_GET_FLAGS(func) & ~(METH_CLASS | METH_STATIC | METH_COEXIST | METH_KEYWORDS)))))
#else
#define __Pyx_PyFastCFunction_Check(func) 0
#endif
#if CYTHON_COMPILING_IN_PYPY && !defined(PyObject_Malloc)
  #define PyObject_Malloc(s)   PyMem_Malloc(s)
  #define PyObject_Free(p)     PyMem_Free(p)
  #define PyObject_Realloc(p)  PyMem_Realloc(p)
#endif
#if CYTHON_COMPILING_IN_PYSTON
  #define __Pyx_PyCode_HasFreeVars(co)  PyCode_HasFreeVars(co)
  #define __Pyx_PyFrame_SetLineNumber(frame, lineno) PyFrame_SetLineNumber(frame, lineno)
#else
  #define __Pyx_PyCode_HasFreeVars(co)  (PyCode_GetNumFree(co) > 0)
  #define __Pyx_PyFrame_SetLineNumber(frame, lineno)  (frame)->f_lineno = (lineno)
#endif
#if !CYTHON_FAST_THREAD_STATE || PY_VERSION_HEX < 0x02070000
  #define __Pyx_PyThreadState_Current PyThreadState_GET()
#elif PY_VERSION_HEX >= 0x03060000
  #define __Pyx_PyThreadState_Current _PyThreadState_UncheckedGet()
#elif PY_VERSION_HEX >= 0x03000000
  #define __Pyx_PyThreadState_Current PyThreadState_GET()
#else
  #define __Pyx_PyThreadState_Current _PyThreadState_Current
#endif
#if PY_VERSION_HEX < 0x030700A2 && !defined(PyThread_tss_create) && !defined(Py_tss_NEEDS_INIT)
#include "pythread.h"
#define Py_tss_NEEDS_INIT 0
typedef int Py_tss_t;
static CYTHON_INLINE int PyThread_tss_create(Py_tss_t *key) {
  *key = PyThread_create_key();
  return 0; // PyThread_create_key reports success always
}
static CYTHON_INLINE Py_tss_t * PyThread_tss_alloc(void) {
  Py_tss_t *key = (Py_tss_t *)PyObject_Malloc(sizeof(Py_tss_t));
  *key = Py_tss_NEEDS_INIT;
  return key;
}
static CYTHON_INLINE void PyThread_tss_free(Py_tss_t *key) {
  PyObject_Free(key);
}
static CYTHON_INLINE int PyThread_tss_is_created(Py_tss_t *key) {
  return *key != Py_tss_NEEDS_INIT;
}
static CYTHON_INLINE void PyThread_tss_delete(Py_tss_t *key) {
  PyThread_delete_key(*key);
  *key = Py_tss_NEEDS_INIT;
}
static CYTHON_INLINE int PyThread_tss_set(Py_tss_t *key, void *value) {
  return PyThread_set_key_value(*key, value);
}
static CYTHON_INLINE void * PyThread_tss_get(Py_tss_t *key) {
  return PyThread_get_key_value(*key);
}
#endif // TSS (Thread Specific Storage) API
#if CYTHON_COMPILING_IN_CPYTHON || defined(_PyDict_NewPresized)
#define __Pyx_PyDict_NewPresized(n)  ((n <= 8) ? PyDict_New() : _PyDict_NewPresized(n))
#else
#define __Pyx_PyDict_NewPresized(n)  PyDict_New()
#endif
#if PY_MAJOR_VERSION >= 3 || CYTHON_FUTURE_DIVISION
  #define __Pyx_PyNumber_Divide(x,y)         PyNumber_TrueDivide(x,y)
  #define __Pyx_PyNumber_InPlaceDivide(x,y)  PyNumber_InPlaceTrueDivide(x,y)
#else
  #define __Pyx_PyNumber_Divide(x,y)         PyNumber_Divide(x,y)
  #define __Pyx_PyNumber_InPlaceDivide(x,y)  PyNumber_InPlaceDivide(x,y)
#endif
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030500A1 && CYTHON_USE_UNICODE_INTERNALS
#define __Pyx_PyDict_GetItemStr(dict, name)  _PyDict_GetItem_KnownHash(dict, name, ((PyASCIIObject *) name)->hash)
#else
#define __Pyx_PyDict_GetItemStr(dict, name)  PyDict_GetItem(dict, name)
#endif
#if PY_VERSION_HEX > 0x03030000 && defined(PyUnicode_KIND)
  #define CYTHON_PEP393_ENABLED 1
  #define __Pyx_PyUnicode_READY(op)       (likely(PyUnicode_IS_READY(op)) ?\
                                              0 : _PyUnicode_Ready((PyObject *)(op)))
  #define __Pyx_PyUnicode_GET_LENGTH(u)   PyUnicode_GET_LENGTH(u)
  #define __Pyx_PyUnicode_READ_CHAR(u, i) PyUnicode_READ_CHAR(u, i)
  #define __Pyx_PyUnicode_MAX_CHAR_VALUE(u)   PyUnicode_MAX_CHAR_VALUE(u)
  #define __Pyx_PyUnicode_KIND(u)         PyUnicode_KIND(u)
  #define __Pyx_PyUnicode_DATA(u)         PyUnicode_DATA(u)
  #define __Pyx_PyUnicode_READ(k, d, i)   PyUnicode_READ(k, d, i)
  #define __Pyx_PyUnicode_WRITE(k, d, i, ch)  PyUnicode_WRITE(k, d, i, ch)
  #define __Pyx_PyUnicode_IS_TRUE(u)      (0 != (likely(PyUnicode_IS_READY(u)) ? PyUnicode_GET_LENGTH(u) : PyUnicode_GET_SIZE(u)))
#else
  #define CYTHON_PEP393_ENABLED 0
  #define PyUnicode_1BYTE_KIND  1
  #define PyUnicode_2BYTE_KIND  2
  #define PyUnicode_4BYTE_KIND  4
  #define __Pyx_PyUnicode_READY(op)       (0)
  #define __Pyx_PyUnicode_GET_LENGTH(u)   PyUnicode_GET_SIZE(u)
  #define __Pyx_PyUnicode_READ_CHAR(u, i) ((Py_UCS4)(PyUnicode_AS_UNICODE(u)[i]))
  #define __Pyx_PyUnicode_MAX_CHAR_VALUE(u)   ((sizeof(Py_UNICODE) == 2) ? 65535 : 1114111)
  #define __Pyx_PyUnicode_KIND(u)         (sizeof(Py_UNICODE))
  #define __Pyx_PyUnicode_DATA(u)         ((void*)PyUnicode_AS_UNICODE(u))
  #define __Pyx_PyUnicode_READ(k, d, i)   ((void)(k), (Py_UCS4)(((Py_UNICODE*)d)[i]))
  #define __Pyx_PyUnicode_WRITE(k, d, i, ch)  (((void)(k)), ((Py_UNICODE*)d)[i] = ch)
  #define __Pyx_PyUnicode_IS_TRUE(u)      (0 != PyUnicode_GET_SIZE(u))
#endif
#if CYTHON_COMPILING_IN_PYPY
  #define __Pyx_PyUnicode_Concat(a, b)      PyNumber_Add(a, b)
  #define __Pyx_PyUnicode_ConcatSafe(a, b)  PyNumber_Add(a, b)
#else
  #define __Pyx_PyUnicode_Concat(a, b)      PyUnicode_Concat(a, b)
  #define __Pyx_PyUnicode_ConcatSafe(a, b)  ((unlikely((a) == Py_None) || unlikely((b) == Py_None)) ?\
      PyNumber_Add(a, b) : __Pyx_PyUnicode_Concat(a, b))
#endif
#if CYTHON_COMPILING_IN_PYPY && !defined(PyUnicode_Contains)
  #define PyUnicode_Contains(u, s)  PySequence_Contains(u, s)
#endif
#if CYTHON_COMPILING_IN_PYPY && !defined(PyByteArray_Check)
  #define PyByteArray_Check(obj)  PyObject_TypeCheck(obj, &PyByteArray_Type)
#endif
#if CYTHON_COMPILING_IN_PYPY && !defined(PyObject_Format)
  #define PyObject_Format(obj, fmt)  PyObject_CallMethod(obj, "__format__", "O", fmt)
#endif
#define __Pyx_PyString_FormatSafe(a, b)   ((unlikely((a) == Py_None)) ? PyNumber_Remainder(a, b) : __Pyx_PyString_Format(a, b))
#define __Pyx_PyUnicode_FormatSafe(a, b)  ((unlikely((a) == Py_None)) ? PyNumber_Remainder(a, b) : PyUnicode_Format(a, b))
#if PY_MAJOR_VERSION >= 3
  #define __Pyx_PyString_Format(a, b)  PyUnicode_Format(a, b)
#else
  #define __Pyx_PyString_Format(a, b)  PyString_Format(a, b)
#endif
#if PY_MAJOR_VERSION < 3 && !defined(PyObject_ASCII)
  #define PyObject_ASCII(o)            PyObject_Repr(o)
#endif
#if PY_MAJOR_VERSION >= 3
  #define PyBaseString_Type            PyUnicode_Type
  #define PyStringObject               PyUnicodeObject
  #define PyString_Type                PyUnicode_Type
  #define PyString_Check               PyUnicode_Check
  #define PyString_CheckExact          PyUnicode_CheckExact
#endif
#if PY_MAJOR_VERSION >= 3
  #define __Pyx_PyBaseString_Check(obj) PyUnicode_Check(obj)
  #define __Pyx_PyBaseString_CheckExact(obj) PyUnicode_CheckExact(obj)
#else
  #define __Pyx_PyBaseString_Check(obj) (PyString_Check(obj) || PyUnicode_Check(obj))
  #define __Pyx_PyBaseString_CheckExact(obj) (PyString_CheckExact(obj) || PyUnicode_CheckExact(obj))
#endif
#ifndef PySet_CheckExact
  #define PySet_CheckExact(obj)        (Py_TYPE(obj) == &PySet_Type)
#endif
#if CYTHON_ASSUME_SAFE_MACROS
  #define __Pyx_PySequence_SIZE(seq)  Py_SIZE(seq)
#else
  #define __Pyx_PySequence_SIZE(seq)  PySequence_Size(seq)
#endif
#if PY_MAJOR_VERSION >= 3
  #define PyIntObject                  PyLongObject
  #define PyInt_Type                   PyLong_Type
  #define PyInt_Check(op)              PyLong_Check(op)
  #define PyInt_CheckExact(op)         PyLong_CheckExact(op)
  #define PyInt_FromString             PyLong_FromString
  #define PyInt_FromUnicode            PyLong_FromUnicode
  #define PyInt_FromLong               PyLong_FromLong
  #define PyInt_FromSize_t             PyLong_FromSize_t
  #define PyInt_FromSsize_t            PyLong_FromSsize_t
  #define PyInt_AsLong                 PyLong_AsLong
  #define PyInt_AS_LONG                PyLong_AS_LONG
  #define PyInt_AsSsize_t              PyLong_AsSsize_t
  #define PyInt_AsUnsignedLongMask     PyLong_AsUnsignedLongMask
  #define PyInt_AsUnsignedLongLongMask PyLong_AsUnsignedLongLongMask
  #define PyNumber_Int                 PyNumber_Long
#endif
#if PY_MAJOR_VERSION >= 3
  #define PyBoolObject                 PyLongObject
#endif
#if PY_MAJOR_VERSION >= 3 && CYTHON_COMPILING_IN_PYPY
  #ifndef PyUnicode_InternFromString
    #define PyUnicode_InternFromString(s) PyUnicode_FromString(s)
  #endif
#endif
#if PY_VERSION_HEX < 0x030200A4
  typedef long Py_hash_t;
  #define __Pyx_PyInt_FromHash_t PyInt_FromLong
  #define __Pyx_PyInt_AsHash_t   PyInt_AsLong
#else
  #define __Pyx_PyInt_FromHash_t PyInt_FromSsize_t
  #define __Pyx_PyInt_AsHash_t   PyInt_AsSsize_t
#endif
#if PY_MAJOR_VERSION >= 3
  #define __Pyx_PyMethod_New(func, self, klass) ((self) ? PyMethod_New(func, self) : (Py_INCREF(func), func))
#else
  #define __Pyx_PyMethod_New(func, self, klass) PyMethod_New(func, self, klass)
#endif
#if CYTHON_USE_ASYNC_SLOTS
  #if PY_VERSION_HEX >= 0x030500B1
    #define __Pyx_PyAsyncMethodsStruct PyAsyncMethods
    #define __Pyx_PyType_AsAsync(obj) (Py_TYPE(obj)->tp_as_async)
  #else
    #define __Pyx_PyType_AsAsync(obj) ((__Pyx_PyAsyncMethodsStruct*) (Py_TYPE(obj)->tp_reserved))
  #endif
#else
  #define __Pyx_PyType_AsAsync(obj) NULL
#endif
#ifndef __Pyx_PyAsyncMethodsStruct
    typedef struct {
        unaryfunc am_await;
        unaryfunc am_aiter;
        unaryfunc am_anext;
    } __Pyx_PyAsyncMethodsStruct;
#endif

#if defined(WIN32) || defined(MS_WINDOWS)
  #define _USE_MATH_DEFINES
#endif
#include <math.h>
#ifdef NAN
#define __PYX_NAN() ((float) NAN)
#else
static CYTHON_INLINE float __PYX_NAN() {
  float value;
  memset(&value, 0xFF, sizeof(value));
  return value;
}
#endif
#if defined(__CYGWIN__) && defined(_LDBL_EQ_DBL)
#define __Pyx_truncl trunc
#else
#define __Pyx_truncl truncl
#endif


#define __PYX_ERR(f_index, lineno, Ln_error) \
{ \
  __pyx_filename = __pyx_f[f_index]; __pyx_lineno = lineno; __pyx_clineno = __LINE__; goto Ln_error; \
}

#ifndef __PYX_EXTERN_C
  #ifdef __cplusplus
    #define __PYX_EXTERN_C extern "C"
  #else
    #define __PYX_EXTERN_C extern
  #endif
#endif

#define __PYX_HAVE__pymarketcap__core
#define __PYX_HAVE_API__pymarketcap__core
/* Early includes */
#ifdef _OPENMP
#include <omp.h>
#endif /* _OPENMP */

#if defined(PYREX_WITHOUT_ASSERTIONS) && !defined(CYTHON_WITHOUT_ASSERTIONS)
#define CYTHON_WITHOUT_ASSERTIONS
#endif

typedef struct {PyObject **p; const char *s; const Py_ssize_t n; const char* encoding;
                const char is_unicode; const char is_str; const char intern; } __Pyx_StringTabEntry;

#define __PYX_DEFAULT_STRING_ENCODING_IS_ASCII 0
#define __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT 0
#define __PYX_DEFAULT_STRING_ENCODING ""
#define __Pyx_PyObject_FromString __Pyx_PyBytes_FromString
#define __Pyx_PyObject_FromStringAndSize __Pyx_PyBytes_FromStringAndSize
#define __Pyx_uchar_cast(c) ((unsigned char)c)
#define __Pyx_long_cast(x) ((long)x)
#define __Pyx_fits_Py_ssize_t(v, type, is_signed)  (\
    (sizeof(type) < sizeof(Py_ssize_t))  ||\
    (sizeof(type) > sizeof(Py_ssize_t) &&\
          likely(v < (type)PY_SSIZE_T_MAX ||\
                 v == (type)PY_SSIZE_T_MAX)  &&\
          (!is_signed || likely(v > (type)PY_SSIZE_T_MIN ||\
                                v == (type)PY_SSIZE_T_MIN)))  ||\
    (sizeof(type) == sizeof(Py_ssize_t) &&\
          (is_signed || likely(v < (type)PY_SSIZE_T_MAX ||\
                               v == (type)PY_SSIZE_T_MAX)))  )
#if defined (__cplusplus) && __cplusplus >= 201103L
    #include <cstdlib>
    #define __Pyx_sst_abs(value) std::abs(value)
#elif SIZEOF_INT >= SIZEOF_SIZE_T
    #define __Pyx_sst_abs(value) abs(value)
#elif SIZEOF_LONG >= SIZEOF_SIZE_T
    #define __Pyx_sst_abs(value) labs(value)
#elif defined (_MSC_VER)
    #define __Pyx_sst_abs(value) ((Py_ssize_t)_abs64(value))
#elif defined (__STDC_VERSION__) && __STDC_VERSION__ >= 199901L
    #define __Pyx_sst_abs(value) llabs(value)
#elif defined (__GNUC__)
    #define __Pyx_sst_abs(value) __builtin_llabs(value)
#else
    #define __Pyx_sst_abs(value) ((value<0) ? -value : value)
#endif
static CYTHON_INLINE const char* __Pyx_PyObject_AsString(PyObject*);
static CYTHON_INLINE const char* __Pyx_PyObject_AsStringAndSize(PyObject*, Py_ssize_t* length);
#define __Pyx_PyByteArray_FromString(s) PyByteArray_FromStringAndSize((const char*)s, strlen((const char*)s))
#define __Pyx_PyByteArray_FromStringAndSize(s, l) PyByteArray_FromStringAndSize((const char*)s, l)
#define __Pyx_PyBytes_FromString        PyBytes_FromString
#define __Pyx_PyBytes_FromStringAndSize PyBytes_FromStringAndSize
static CYTHON_INLINE PyObject* __Pyx_PyUnicode_FromString(const char*);
#if PY_MAJOR_VERSION < 3
    #define __Pyx_PyStr_FromString        __Pyx_PyBytes_FromString
    #define __Pyx_PyStr_FromStringAndSize __Pyx_PyBytes_FromStringAndSize
#else
    #define __Pyx_PyStr_FromString        __Pyx_PyUnicode_FromString
    #define __Pyx_PyStr_FromStringAndSize __Pyx_PyUnicode_FromStringAndSize
#endif
#define __Pyx_PyBytes_AsWritableString(s)     ((char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsWritableSString(s)    ((signed char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsWritableUString(s)    ((unsigned char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsString(s)     ((const char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsSString(s)    ((const signed char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsUString(s)    ((const unsigned char*) PyBytes_AS_STRING(s))
#define __Pyx_PyObject_AsWritableString(s)    ((char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsWritableSString(s)    ((signed char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsWritableUString(s)    ((unsigned char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsSString(s)    ((const signed char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsUString(s)    ((const unsigned char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_FromCString(s)  __Pyx_PyObject_FromString((const char*)s)
#define __Pyx_PyBytes_FromCString(s)   __Pyx_PyBytes_FromString((const char*)s)
#define __Pyx_PyByteArray_FromCString(s)   __Pyx_PyByteArray_FromString((const char*)s)
#define __Pyx_PyStr_FromCString(s)     __Pyx_PyStr_FromString((const char*)s)
#define __Pyx_PyUnicode_FromCString(s) __Pyx_PyUnicode_FromString((const char*)s)
static CYTHON_INLINE size_t __Pyx_Py_UNICODE_strlen(const Py_UNICODE *u) {
    const Py_UNICODE *u_end = u;
    while (*u_end++) ;
    return (size_t)(u_end - u - 1);
}
#define __Pyx_PyUnicode_FromUnicode(u)       PyUnicode_FromUnicode(u, __Pyx_Py_UNICODE_strlen(u))
#define __Pyx_PyUnicode_FromUnicodeAndLength PyUnicode_FromUnicode
#define __Pyx_PyUnicode_AsUnicode            PyUnicode_AsUnicode
#define __Pyx_NewRef(obj) (Py_INCREF(obj), obj)
#define __Pyx_Owned_Py_None(b) __Pyx_NewRef(Py_None)
#define __Pyx_PyBool_FromLong(b) ((b) ? __Pyx_NewRef(Py_True) : __Pyx_NewRef(Py_False))
static CYTHON_INLINE int __Pyx_PyObject_IsTrue(PyObject*);
static CYTHON_INLINE PyObject* __Pyx_PyNumber_IntOrLong(PyObject* x);
#define __Pyx_PySequence_Tuple(obj)\
    (likely(PyTuple_CheckExact(obj)) ? __Pyx_NewRef(obj) : PySequence_Tuple(obj))
static CYTHON_INLINE Py_ssize_t __Pyx_PyIndex_AsSsize_t(PyObject*);
static CYTHON_INLINE PyObject * __Pyx_PyInt_FromSize_t(size_t);
#if CYTHON_ASSUME_SAFE_MACROS
#define __pyx_PyFloat_AsDouble(x) (PyFloat_CheckExact(x) ? PyFloat_AS_DOUBLE(x) : PyFloat_AsDouble(x))
#else
#define __pyx_PyFloat_AsDouble(x) PyFloat_AsDouble(x)
#endif
#define __pyx_PyFloat_AsFloat(x) ((float) __pyx_PyFloat_AsDouble(x))
#if PY_MAJOR_VERSION >= 3
#define __Pyx_PyNumber_Int(x) (PyLong_CheckExact(x) ? __Pyx_NewRef(x) : PyNumber_Long(x))
#else
#define __Pyx_PyNumber_Int(x) (PyInt_CheckExact(x) ? __Pyx_NewRef(x) : PyNumber_Int(x))
#endif
#define __Pyx_PyNumber_Float(x) (PyFloat_CheckExact(x) ? __Pyx_NewRef(x) : PyNumber_Float(x))
#if PY_MAJOR_VERSION < 3 && __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
static int __Pyx_sys_getdefaultencoding_not_ascii;
static int __Pyx_init_sys_getdefaultencoding_params(void) {
    PyObject* sys;
    PyObject* default_encoding = NULL;
    PyObject* ascii_chars_u = NULL;
    PyObject* ascii_chars_b = NULL;
    const char* default_encoding_c;
    sys = PyImport_ImportModule("sys");
    if (!sys) goto bad;
    default_encoding = PyObject_CallMethod(sys, (char*) "getdefaultencoding", NULL);
    Py_DECREF(sys);
    if (!default_encoding) goto bad;
    default_encoding_c = PyBytes_AsString(default_encoding);
    if (!default_encoding_c) goto bad;
    if (strcmp(default_encoding_c, "ascii") == 0) {
        __Pyx_sys_getdefaultencoding_not_ascii = 0;
    } else {
        char ascii_chars[128];
        int c;
        for (c = 0; c < 128; c++) {
            ascii_chars[c] = c;
        }
        __Pyx_sys_getdefaultencoding_not_ascii = 1;
        ascii_chars_u = PyUnicode_DecodeASCII(ascii_chars, 128, NULL);
        if (!ascii_chars_u) goto bad;
        ascii_chars_b = PyUnicode_AsEncodedString(ascii_chars_u, default_encoding_c, NULL);
        if (!ascii_chars_b || !PyBytes_Check(ascii_chars_b) || memcmp(ascii_chars, PyBytes_AS_STRING(ascii_chars_b), 128) != 0) {
            PyErr_Format(
                PyExc_ValueError,
                "This module compiled with c_string_encoding=ascii, but default encoding '%.200s' is not a superset of ascii.",
                default_encoding_c);
            goto bad;
        }
        Py_DECREF(ascii_chars_u);
        Py_DECREF(ascii_chars_b);
    }
    Py_DECREF(default_encoding);
    return 0;
bad:
    Py_XDECREF(default_encoding);
    Py_XDECREF(ascii_chars_u);
    Py_XDECREF(ascii_chars_b);
    return -1;
}
#endif
#if __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT && PY_MAJOR_VERSION >= 3
#define __Pyx_PyUnicode_FromStringAndSize(c_str, size) PyUnicode_DecodeUTF8(c_str, size, NULL)
#else
#define __Pyx_PyUnicode_FromStringAndSize(c_str, size) PyUnicode_Decode(c_str, size, __PYX_DEFAULT_STRING_ENCODING, NULL)
#if __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT
static char* __PYX_DEFAULT_STRING_ENCODING;
static int __Pyx_init_sys_getdefaultencoding_params(void) {
    PyObject* sys;
    PyObject* default_encoding = NULL;
    char* default_encoding_c;
    sys = PyImport_ImportModule("sys");
    if (!sys) goto bad;
    default_encoding = PyObject_CallMethod(sys, (char*) (const char*) "getdefaultencoding", NULL);
    Py_DECREF(sys);
    if (!default_encoding) goto bad;
    default_encoding_c = PyBytes_AsString(default_encoding);
    if (!default_encoding_c) goto bad;
    __PYX_DEFAULT_STRING_ENCODING = (char*) malloc(strlen(default_encoding_c));
    if (!__PYX_DEFAULT_STRING_ENCODING) goto bad;
    strcpy(__PYX_DEFAULT_STRING_ENCODING, default_encoding_c);
    Py_DECREF(default_encoding);
    return 0;
bad:
    Py_XDECREF(default_encoding);
    return -1;
}
#endif
#endif


/* Test for GCC > 2.95 */
#if defined(__GNUC__)     && (__GNUC__ > 2 || (__GNUC__ == 2 && (__GNUC_MINOR__ > 95)))
  #define likely(x)   __builtin_expect(!!(x), 1)
  #define unlikely(x) __builtin_expect(!!(x), 0)
#else /* !__GNUC__ or GCC < 2.95 */
  #define likely(x)   (x)
  #define unlikely(x) (x)
#endif /* __GNUC__ */
static CYTHON_INLINE void __Pyx_pretend_to_initialize(void* ptr) { (void)ptr; }

static PyObject *__pyx_m = NULL;
static PyObject *__pyx_d;
static PyObject *__pyx_b;
static PyObject *__pyx_cython_runtime;
static PyObject *__pyx_empty_tuple;
static PyObject *__pyx_empty_bytes;
static PyObject *__pyx_empty_unicode;
static int __pyx_lineno;
static int __pyx_clineno = 0;
static const char * __pyx_cfilenm= __FILE__;
static const char *__pyx_filename;


static const char *__pyx_f[] = {
  "pymarketcap/core.pyx",
  "stringsource",
};

/*--- Type declarations ---*/
struct __pyx_obj_11pymarketcap_4core_Pymarketcap;
struct __pyx_opt_args_11pymarketcap_4core_11Pymarketcap__quick_search;
struct __pyx_opt_args_11pymarketcap_4core_11Pymarketcap_stats;
struct __pyx_opt_args_11pymarketcap_4core_11Pymarketcap_ticker;
struct __pyx_opt_args_11pymarketcap_4core_11Pymarketcap_currency;
struct __pyx_opt_args_11pymarketcap_4core_11Pymarketcap_markets;
struct __pyx_opt_args_11pymarketcap_4core_11Pymarketcap_historical;
struct __pyx_opt_args_11pymarketcap_4core_11Pymarketcap_recently;
struct __pyx_opt_args_11pymarketcap_4core_11Pymarketcap_exchange;
struct __pyx_opt_args_11pymarketcap_4core_11Pymarketcap_exchanges;
struct __pyx_opt_args_11pymarketcap_4core_11Pymarketcap_tokens;
struct __pyx_opt_args_11pymarketcap_4core_11Pymarketcap__currency;
struct __pyx_opt_args_11pymarketcap_4core_11Pymarketcap__global_cap;
struct __pyx_opt_args_11pymarketcap_4core_11Pymarketcap__dominance;
struct __pyx_opt_args_11pymarketcap_4core_11Pymarketcap_download_logo;
struct __pyx_opt_args_11pymarketcap_4core_11Pymarketcap_download_exchange_logo;

/* "pymarketcap/core.pyx":132
 *         return response
 * 
 *     cpdef _quick_search(self, exchanges=False):             # <<<<<<<<<<<<<<
 *         """Internal pymarketcap JSON cache files
 *         ``https://s2.coinmarketcap.com/generated/search/quick_search.json``
 */
struct __pyx_opt_args_11pymarketcap_4core_11Pymarketcap__quick_search {
  int __pyx_n;
  PyObject *exchanges;
};

/* "pymarketcap/core.pyx":330
 *                         #######   PUBLIC API   #######
 * 
 *     cpdef stats(self, convert="USD"):             # <<<<<<<<<<<<<<
 *         """ Get global cryptocurrencies statistics.
 * 
 */
struct __pyx_opt_args_11pymarketcap_4core_11Pymarketcap_stats {
  int __pyx_n;
  PyObject *convert;
};

/* "pymarketcap/core.pyx":351
 *         return
 * 
 *     cpdef ticker(self, currency=None, limit=0, start=0, convert="USD"):             # <<<<<<<<<<<<<<
 *         """Get currencies with other aditional data.
 * 
 */
struct __pyx_opt_args_11pymarketcap_4core_11Pymarketcap_ticker {
  int __pyx_n;
  PyObject *currency;
  PyObject *limit;
  PyObject *start;
  PyObject *convert;
};

/* "pymarketcap/core.pyx":450
 *             raise err
 * 
 *     cpdef currency(self, unicode name, convert="USD"):             # <<<<<<<<<<<<<<
 *         """Get currency metadata like total markets capitalization,
 *         websites, source code link, if mineable...
 */
struct __pyx_opt_args_11pymarketcap_4core_11Pymarketcap_currency {
  int __pyx_n;
  PyObject *convert;
};

/* "pymarketcap/core.pyx":490
 *         return response
 * 
 *     cpdef markets(self, unicode name, convert="USD"):             # <<<<<<<<<<<<<<
 *         """Get available coinmarketcap markets data.
 *         It needs a currency as argument.
 */
struct __pyx_opt_args_11pymarketcap_4core_11Pymarketcap_markets {
  int __pyx_n;
  PyObject *convert;
};

/* "pymarketcap/core.pyx":541
 *         return processer.ranks(res)
 * 
 *     cpdef historical(self, unicode name,             # <<<<<<<<<<<<<<
 *                      start=datetime(2008, 8, 18),
 *                      end=datetime.now(),
 */
struct __pyx_opt_args_11pymarketcap_4core_11Pymarketcap_historical {
  int __pyx_n;
  PyObject *start;
  PyObject *end;
  PyObject *revert;
};

/* "pymarketcap/core.pyx":591
 *         return response
 * 
 *     cpdef recently(self, convert="USD"):             # <<<<<<<<<<<<<<
 *         """Get recently added currencies along with other metadata.
 * 
 */
struct __pyx_opt_args_11pymarketcap_4core_11Pymarketcap_recently {
  int __pyx_n;
  PyObject *convert;
};

/* "pymarketcap/core.pyx":608
 *         return list(processer.recently(res, convert))
 * 
 *     cpdef exchange(self, unicode name, convert="USD"):             # <<<<<<<<<<<<<<
 *         """Obtain data from a exchange passed as argument. See ``exchanges_slugs``
 *         property for obtain all posibles values.
 */
struct __pyx_opt_args_11pymarketcap_4core_11Pymarketcap_exchange {
  int __pyx_n;
  PyObject *convert;
};

/* "pymarketcap/core.pyx":639
 *         return response
 * 
 *     cpdef exchanges(self, convert="USD"):             # <<<<<<<<<<<<<<
 *         """Get all the exchanges in coinmarketcap ranked by volumes
 *         along with other metadata.
 */
struct __pyx_opt_args_11pymarketcap_4core_11Pymarketcap_exchanges {
  int __pyx_n;
  PyObject *convert;
};

/* "pymarketcap/core.pyx":656
 *         return processer.exchanges(res, convert)
 * 
 *     cpdef tokens(self, convert="USD"):             # <<<<<<<<<<<<<<
 *         """Get data from platforms tokens
 * 
 */
struct __pyx_opt_args_11pymarketcap_4core_11Pymarketcap_tokens {
  int __pyx_n;
  PyObject *convert;
};

/* "pymarketcap/core.pyx":677
 *                        #######   INTERNAL API   #######
 * 
 *     cpdef _currency(self, unicode name, start=None, end=None):             # <<<<<<<<<<<<<<
 *         """Get graphs data of a currency.
 * 
 */
struct __pyx_opt_args_11pymarketcap_4core_11Pymarketcap__currency {
  int __pyx_n;
  PyObject *start;
  PyObject *end;
};

/* "pymarketcap/core.pyx":703
 *         return processer.graphs(res, start, end)
 * 
 *     cpdef _global_cap(self, bitcoin=True, start=None, end=None):             # <<<<<<<<<<<<<<
 *         """Get global market capitalization graphs, including
 *         or excluding Bitcoin.
 */
struct __pyx_opt_args_11pymarketcap_4core_11Pymarketcap__global_cap {
  int __pyx_n;
  PyObject *bitcoin;
  PyObject *start;
  PyObject *end;
};

/* "pymarketcap/core.pyx":730
 *         return processer.graphs(res, start, end)
 * 
 *     cpdef _dominance(self, start=None, end=None):             # <<<<<<<<<<<<<<
 *         """Get currencies dominance percentage graph
 * 
 */
struct __pyx_opt_args_11pymarketcap_4core_11Pymarketcap__dominance {
  int __pyx_n;
  PyObject *start;
  PyObject *end;
};

/* "pymarketcap/core.pyx":748
 *         return processer.graphs(res, start, end)
 * 
 *     cpdef download_logo(self, unicode name, size=64, filename=None):             # <<<<<<<<<<<<<<
 *         """Download a currency image logo providing their size.
 * 
 */
struct __pyx_opt_args_11pymarketcap_4core_11Pymarketcap_download_logo {
  int __pyx_n;
  PyObject *size;
  PyObject *filename;
};

/* "pymarketcap/core.pyx":793
 *             return filename
 * 
 *     cpdef download_exchange_logo(self, unicode name, size=64, filename=None):             # <<<<<<<<<<<<<<
 *         """Download a exchange logo passing his name or
 *         slug as first parameter and optionally a filename
 */
struct __pyx_opt_args_11pymarketcap_4core_11Pymarketcap_download_exchange_logo {
  int __pyx_n;
  PyObject *size;
  PyObject *filename;
};

/* "pymarketcap/core.pyx":38
 * 
 * 
 * cdef class Pymarketcap:             # <<<<<<<<<<<<<<
 *     """Synchronous class for retrieve data from https://coinmarketcap.com.
 * 
 */
struct __pyx_obj_11pymarketcap_4core_Pymarketcap {
  PyObject_HEAD
  struct __pyx_vtabstruct_11pymarketcap_4core_Pymarketcap *__pyx_vtab;
  PyObject *_correspondences;
  PyObject *_ids_correspondences;
  PyObject *_symbols;
  PyObject *_coins;
  int _total_currencies;
  int _total_exchanges;
  PyObject *_currencies_to_convert;
  PyObject *_converter_cache;
  PyObject *_exchange_names;
  PyObject *_exchange_slugs;
  PyObject *__repeated_symbols;
  PyObject *_exchange_names_slugs_ids;
  long timeout;
  PyObject *graphs;
  int debug;
};



struct __pyx_vtabstruct_11pymarketcap_4core_Pymarketcap {
  PyObject *(*_is_symbol)(struct __pyx_obj_11pymarketcap_4core_Pymarketcap *, PyObject *, int __pyx_skip_dispatch);
  PyObject *(*_quick_search)(struct __pyx_obj_11pymarketcap_4core_Pymarketcap *, int __pyx_skip_dispatch, struct __pyx_opt_args_11pymarketcap_4core_11Pymarketcap__quick_search *__pyx_optional_args);
  PyObject *(*_cache_symbols_ids)(struct __pyx_obj_11pymarketcap_4core_Pymarketcap *, int __pyx_skip_dispatch);
  PyObject *(*_cache_exchanges_ids)(struct __pyx_obj_11pymarketcap_4core_Pymarketcap *, int __pyx_skip_dispatch);
  PyObject *(*__pyx___currencies_to_convert)(struct __pyx_obj_11pymarketcap_4core_Pymarketcap *, int __pyx_skip_dispatch);
  PyObject *(*__pyx___exchange_names_slugs)(struct __pyx_obj_11pymarketcap_4core_Pymarketcap *, int __pyx_skip_dispatch);
  PyObject *(*_get)(struct __pyx_obj_11pymarketcap_4core_Pymarketcap *, char *);
  PyObject *(*stats)(struct __pyx_obj_11pymarketcap_4core_Pymarketcap *, int __pyx_skip_dispatch, struct __pyx_opt_args_11pymarketcap_4core_11Pymarketcap_stats *__pyx_optional_args);
  PyObject *(*ticker)(struct __pyx_obj_11pymarketcap_4core_Pymarketcap *, int __pyx_skip_dispatch, struct __pyx_opt_args_11pymarketcap_4core_11Pymarketcap_ticker *__pyx_optional_args);
  PyObject *(*convert)(struct __pyx_obj_11pymarketcap_4core_Pymarketcap *, PyObject *, PyObject *, PyObject *, int __pyx_skip_dispatch);
  PyObject *(*currency)(struct __pyx_obj_11pymarketcap_4core_Pymarketcap *, PyObject *, int __pyx_skip_dispatch, struct __pyx_opt_args_11pymarketcap_4core_11Pymarketcap_currency *__pyx_optional_args);
  PyObject *(*markets)(struct __pyx_obj_11pymarketcap_4core_Pymarketcap *, PyObject *, int __pyx_skip_dispatch, struct __pyx_opt_args_11pymarketcap_4core_11Pymarketcap_markets *__pyx_optional_args);
  PyObject *(*ranks)(struct __pyx_obj_11pymarketcap_4core_Pymarketcap *, int __pyx_skip_dispatch);
  PyObject *(*historical)(struct __pyx_obj_11pymarketcap_4core_Pymarketcap *, PyObject *, int __pyx_skip_dispatch, struct __pyx_opt_args_11pymarketcap_4core_11Pymarketcap_historical *__pyx_optional_args);
  PyObject *(*recently)(struct __pyx_obj_11pymarketcap_4core_Pymarketcap *, int __pyx_skip_dispatch, struct __pyx_opt_args_11pymarketcap_4core_11Pymarketcap_recently *__pyx_optional_args);
  PyObject *(*exchange)(struct __pyx_obj_11pymarketcap_4core_Pymarketcap *, PyObject *, int __pyx_skip_dispatch, struct __pyx_opt_args_11pymarketcap_4core_11Pymarketcap_exchange *__pyx_optional_args);
  PyObject *(*exchanges)(struct __pyx_obj_11pymarketcap_4core_Pymarketcap *, int __pyx_skip_dispatch, struct __pyx_opt_args_11pymarketcap_4core_11Pymarketcap_exchanges *__pyx_optional_args);
  PyObject *(*tokens)(struct __pyx_obj_11pymarketcap_4core_Pymarketcap *, int __pyx_skip_dispatch, struct __pyx_opt_args_11pymarketcap_4core_11Pymarketcap_tokens *__pyx_optional_args);
  PyObject *(*_currency)(struct __pyx_obj_11pymarketcap_4core_Pymarketcap *, PyObject *, int __pyx_skip_dispatch, struct __pyx_opt_args_11pymarketcap_4core_11Pymarketcap__currency *__pyx_optional_args);
  PyObject *(*_global_cap)(struct __pyx_obj_11pymarketcap_4core_Pymarketcap *, int __pyx_skip_dispatch, struct __pyx_opt_args_11pymarketcap_4core_11Pymarketcap__global_cap *__pyx_optional_args);
  PyObject *(*_dominance)(struct __pyx_obj_11pymarketcap_4core_Pymarketcap *, int __pyx_skip_dispatch, struct __pyx_opt_args_11pymarketcap_4core_11Pymarketcap__dominance *__pyx_optional_args);
  PyObject *(*download_logo)(struct __pyx_obj_11pymarketcap_4core_Pymarketcap *, PyObject *, int __pyx_skip_dispatch, struct __pyx_opt_args_11pymarketcap_4core_11Pymarketcap_download_logo *__pyx_optional_args);
  PyObject *(*download_exchange_logo)(struct __pyx_obj_11pymarketcap_4core_Pymarketcap *, PyObject *, int __pyx_skip_dispatch, struct __pyx_opt_args_11pymarketcap_4core_11Pymarketcap_download_exchange_logo *__pyx_optional_args);
};
static struct __pyx_vtabstruct_11pymarketcap_4core_Pymarketcap *__pyx_vtabptr_11pymarketcap_4core_Pymarketcap;

/* --- Runtime support code (head) --- */
/* Refnanny.proto */
#ifndef CYTHON_REFNANNY
  #define CYTHON_REFNANNY 0
#endif
#if CYTHON_REFNANNY
  typedef struct {
    void (*INCREF)(void*, PyObject*, int);
    void (*DECREF)(void*, PyObject*, int);
    void (*GOTREF)(void*, PyObject*, int);
    void (*GIVEREF)(void*, PyObject*, int);
    void* (*SetupContext)(const char*, int, const char*);
    void (*FinishContext)(void**);
  } __Pyx_RefNannyAPIStruct;
  static __Pyx_RefNannyAPIStruct *__Pyx_RefNanny = NULL;
  static __Pyx_RefNannyAPIStruct *__Pyx_RefNannyImportAPI(const char *modname);
  #define __Pyx_RefNannyDeclarations void *__pyx_refnanny = NULL;
#ifdef WITH_THREAD
  #define __Pyx_RefNannySetupContext(name, acquire_gil)\
          if (acquire_gil) {\
              PyGILState_STATE __pyx_gilstate_save = PyGILState_Ensure();\
              __pyx_refnanny = __Pyx_RefNanny->SetupContext((name), __LINE__, __FILE__);\
              PyGILState_Release(__pyx_gilstate_save);\
          } else {\
              __pyx_refnanny = __Pyx_RefNanny->SetupContext((name), __LINE__, __FILE__);\
          }
#else
  #define __Pyx_RefNannySetupContext(name, acquire_gil)\
          __pyx_refnanny = __Pyx_RefNanny->SetupContext((name), __LINE__, __FILE__)
#endif
  #define __Pyx_RefNannyFinishContext()\
          __Pyx_RefNanny->FinishContext(&__pyx_refnanny)
  #define __Pyx_INCREF(r)  __Pyx_RefNanny->INCREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_DECREF(r)  __Pyx_RefNanny->DECREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_GOTREF(r)  __Pyx_RefNanny->GOTREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_GIVEREF(r) __Pyx_RefNanny->GIVEREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_XINCREF(r)  do { if((r) != NULL) {__Pyx_INCREF(r); }} while(0)
  #define __Pyx_XDECREF(r)  do { if((r) != NULL) {__Pyx_DECREF(r); }} while(0)
  #define __Pyx_XGOTREF(r)  do { if((r) != NULL) {__Pyx_GOTREF(r); }} while(0)
  #define __Pyx_XGIVEREF(r) do { if((r) != NULL) {__Pyx_GIVEREF(r);}} while(0)
#else
  #define __Pyx_RefNannyDeclarations
  #define __Pyx_RefNannySetupContext(name, acquire_gil)
  #define __Pyx_RefNannyFinishContext()
  #define __Pyx_INCREF(r) Py_INCREF(r)
  #define __Pyx_DECREF(r) Py_DECREF(r)
  #define __Pyx_GOTREF(r)
  #define __Pyx_GIVEREF(r)
  #define __Pyx_XINCREF(r) Py_XINCREF(r)
  #define __Pyx_XDECREF(r) Py_XDECREF(r)
  #define __Pyx_XGOTREF(r)
  #define __Pyx_XGIVEREF(r)
#endif
#define __Pyx_XDECREF_SET(r, v) do {\
        PyObject *tmp = (PyObject *) r;\
        r = v; __Pyx_XDECREF(tmp);\
    } while (0)
#define __Pyx_DECREF_SET(r, v) do {\
        PyObject *tmp = (PyObject *) r;\
        r = v; __Pyx_DECREF(tmp);\
    } while (0)
#define __Pyx_CLEAR(r)    do { PyObject* tmp = ((PyObject*)(r)); r = NULL; __Pyx_DECREF(tmp);} while(0)
#define __Pyx_XCLEAR(r)   do { if((r) != NULL) {PyObject* tmp = ((PyObject*)(r)); r = NULL; __Pyx_DECREF(tmp);}} while(0)

/* PyObjectGetAttrStr.proto */
#if CYTHON_USE_TYPE_SLOTS
static CYTHON_INLINE PyObject* __Pyx_PyObject_GetAttrStr(PyObject* obj, PyObject* attr_name);
#else
#define __Pyx_PyObject_GetAttrStr(o,n) PyObject_GetAttr(o,n)
#endif

/* GetBuiltinName.proto */
static PyObject *__Pyx_GetBuiltinName(PyObject *name);

/* RaiseDoubleKeywords.proto */
static void __Pyx_RaiseDoubleKeywordsError(const char* func_name, PyObject* kw_name);

/* ParseKeywords.proto */
static int __Pyx_ParseOptionalKeywords(PyObject *kwds, PyObject **argnames[],\
    PyObject *kwds2, PyObject *values[], Py_ssize_t num_pos_args,\
    const char* function_name);

/* RaiseArgTupleInvalid.proto */
static void __Pyx_RaiseArgtupleInvalid(const char* func_name, int exact,
    Py_ssize_t num_min, Py_ssize_t num_max, Py_ssize_t num_found);

/* PyObjectCall.proto */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_Call(PyObject *func, PyObject *arg, PyObject *kw);
#else
#define __Pyx_PyObject_Call(func, arg, kw) PyObject_Call(func, arg, kw)
#endif

/* GetItemInt.proto */
#define __Pyx_GetItemInt(o, i, type, is_signed, to_py_func, is_list, wraparound, boundscheck)\
    (__Pyx_fits_Py_ssize_t(i, type, is_signed) ?\
    __Pyx_GetItemInt_Fast(o, (Py_ssize_t)i, is_list, wraparound, boundscheck) :\
    (is_list ? (PyErr_SetString(PyExc_IndexError, "list index out of range"), (PyObject*)NULL) :\
               __Pyx_GetItemInt_Generic(o, to_py_func(i))))
#define __Pyx_GetItemInt_List(o, i, type, is_signed, to_py_func, is_list, wraparound, boundscheck)\
    (__Pyx_fits_Py_ssize_t(i, type, is_signed) ?\
    __Pyx_GetItemInt_List_Fast(o, (Py_ssize_t)i, wraparound, boundscheck) :\
    (PyErr_SetString(PyExc_IndexError, "list index out of range"), (PyObject*)NULL))
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_List_Fast(PyObject *o, Py_ssize_t i,
                                                              int wraparound, int boundscheck);
#define __Pyx_GetItemInt_Tuple(o, i, type, is_signed, to_py_func, is_list, wraparound, boundscheck)\
    (__Pyx_fits_Py_ssize_t(i, type, is_signed) ?\
    __Pyx_GetItemInt_Tuple_Fast(o, (Py_ssize_t)i, wraparound, boundscheck) :\
    (PyErr_SetString(PyExc_IndexError, "tuple index out of range"), (PyObject*)NULL))
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Tuple_Fast(PyObject *o, Py_ssize_t i,
                                                              int wraparound, int boundscheck);
static PyObject *__Pyx_GetItemInt_Generic(PyObject *o, PyObject* j);
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Fast(PyObject *o, Py_ssize_t i,
                                                     int is_list, int wraparound, int boundscheck);

/* PyCFunctionFastCall.proto */
#if CYTHON_FAST_PYCCALL
static CYTHON_INLINE PyObject *__Pyx_PyCFunction_FastCall(PyObject *func, PyObject **args, Py_ssize_t nargs);
#else
#define __Pyx_PyCFunction_FastCall(func, args, nargs)  (assert(0), NULL)
#endif

/* PyFunctionFastCall.proto */
#if CYTHON_FAST_PYCALL
#define __Pyx_PyFunction_FastCall(func, args, nargs)\
    __Pyx_PyFunction_FastCallDict((func), (args), (nargs), NULL)
#if 1 || PY_VERSION_HEX < 0x030600B1
static PyObject *__Pyx_PyFunction_FastCallDict(PyObject *func, PyObject **args, int nargs, PyObject *kwargs);
#else
#define __Pyx_PyFunction_FastCallDict(func, args, nargs, kwargs) _PyFunction_FastCallDict(func, args, nargs, kwargs)
#endif
#endif

/* PyObjectCallMethO.proto */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallMethO(PyObject *func, PyObject *arg);
#endif

/* PyObjectCallOneArg.proto */
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallOneArg(PyObject *func, PyObject *arg);

/* PyObjectCallNoArg.proto */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallNoArg(PyObject *func);
#else
#define __Pyx_PyObject_CallNoArg(func) __Pyx_PyObject_Call(func, __pyx_empty_tuple, NULL)
#endif

/* GetModuleGlobalName.proto */
static CYTHON_INLINE PyObject *__Pyx_GetModuleGlobalName(PyObject *name);

/* PySequenceContains.proto */
static CYTHON_INLINE int __Pyx_PySequence_ContainsTF(PyObject* item, PyObject* seq, int eq) {
    int result = PySequence_Contains(seq, item);
    return unlikely(result < 0) ? result : (result == (eq == Py_EQ));
}

/* PyDictContains.proto */
static CYTHON_INLINE int __Pyx_PyDict_ContainsTF(PyObject* item, PyObject* dict, int eq) {
    int result = PyDict_Contains(dict, item);
    return unlikely(result < 0) ? result : (result == (eq == Py_EQ));
}

/* DictGetItem.proto */
#if PY_MAJOR_VERSION >= 3 && !CYTHON_COMPILING_IN_PYPY
static PyObject *__Pyx_PyDict_GetItem(PyObject *d, PyObject* key);
#define __Pyx_PyObject_Dict_GetItem(obj, name)\
    (likely(PyDict_CheckExact(obj)) ?\
     __Pyx_PyDict_GetItem(obj, name) : PyObject_GetItem(obj, name))
#else
#define __Pyx_PyDict_GetItem(d, key) PyObject_GetItem(d, key)
#define __Pyx_PyObject_Dict_GetItem(obj, name)  PyObject_GetItem(obj, name)
#endif

/* PyThreadStateGet.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_PyThreadState_declare  PyThreadState *__pyx_tstate;
#define __Pyx_PyThreadState_assign  __pyx_tstate = __Pyx_PyThreadState_Current;
#define __Pyx_PyErr_Occurred()  __pyx_tstate->curexc_type
#else
#define __Pyx_PyThreadState_declare
#define __Pyx_PyThreadState_assign
#define __Pyx_PyErr_Occurred()  PyErr_Occurred()
#endif

/* PyErrFetchRestore.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_PyErr_Clear() __Pyx_ErrRestore(NULL, NULL, NULL)
#define __Pyx_ErrRestoreWithState(type, value, tb)  __Pyx_ErrRestoreInState(PyThreadState_GET(), type, value, tb)
#define __Pyx_ErrFetchWithState(type, value, tb)    __Pyx_ErrFetchInState(PyThreadState_GET(), type, value, tb)
#define __Pyx_ErrRestore(type, value, tb)  __Pyx_ErrRestoreInState(__pyx_tstate, type, value, tb)
#define __Pyx_ErrFetch(type, value, tb)    __Pyx_ErrFetchInState(__pyx_tstate, type, value, tb)
static CYTHON_INLINE void __Pyx_ErrRestoreInState(PyThreadState *tstate, PyObject *type, PyObject *value, PyObject *tb);
static CYTHON_INLINE void __Pyx_ErrFetchInState(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb);
#if CYTHON_COMPILING_IN_CPYTHON
#define __Pyx_PyErr_SetNone(exc) (Py_INCREF(exc), __Pyx_ErrRestore((exc), NULL, NULL))
#else
#define __Pyx_PyErr_SetNone(exc) PyErr_SetNone(exc)
#endif
#else
#define __Pyx_PyErr_Clear() PyErr_Clear()
#define __Pyx_PyErr_SetNone(exc) PyErr_SetNone(exc)
#define __Pyx_ErrRestoreWithState(type, value, tb)  PyErr_Restore(type, value, tb)
#define __Pyx_ErrFetchWithState(type, value, tb)  PyErr_Fetch(type, value, tb)
#define __Pyx_ErrRestoreInState(tstate, type, value, tb)  PyErr_Restore(type, value, tb)
#define __Pyx_ErrFetchInState(tstate, type, value, tb)  PyErr_Fetch(type, value, tb)
#define __Pyx_ErrRestore(type, value, tb)  PyErr_Restore(type, value, tb)
#define __Pyx_ErrFetch(type, value, tb)  PyErr_Fetch(type, value, tb)
#endif

/* RaiseException.proto */
static void __Pyx_Raise(PyObject *type, PyObject *value, PyObject *tb, PyObject *cause);

/* SaveResetException.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_ExceptionSave(type, value, tb)  __Pyx__ExceptionSave(__pyx_tstate, type, value, tb)
static CYTHON_INLINE void __Pyx__ExceptionSave(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb);
#define __Pyx_ExceptionReset(type, value, tb)  __Pyx__ExceptionReset(__pyx_tstate, type, value, tb)
static CYTHON_INLINE void __Pyx__ExceptionReset(PyThreadState *tstate, PyObject *type, PyObject *value, PyObject *tb);
#else
#define __Pyx_ExceptionSave(type, value, tb)   PyErr_GetExcInfo(type, value, tb)
#define __Pyx_ExceptionReset(type, value, tb)  PyErr_SetExcInfo(type, value, tb)
#endif

/* PyErrExceptionMatches.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_PyErr_ExceptionMatches(err) __Pyx_PyErr_ExceptionMatchesInState(__pyx_tstate, err)
static CYTHON_INLINE int __Pyx_PyErr_ExceptionMatchesInState(PyThreadState* tstate, PyObject* err);
#else
#define __Pyx_PyErr_ExceptionMatches(err)  PyErr_ExceptionMatches(err)
#endif

/* ArgTypeTest.proto */
#define __Pyx_ArgTypeTest(obj, type, none_allowed, name, exact)\
    ((likely((Py_TYPE(obj) == type) | (none_allowed && (obj == Py_None)))) ? 1 :\
        __Pyx__ArgTypeTest(obj, type, name, exact))
static int __Pyx__ArgTypeTest(PyObject *obj, PyTypeObject *type, const char *name, int exact);

/* py_dict_keys.proto */
static CYTHON_INLINE PyObject* __Pyx_PyDict_Keys(PyObject* d);

/* UnpackUnboundCMethod.proto */
typedef struct {
    PyObject *type;
    PyObject **method_name;
    PyCFunction func;
    PyObject *method;
    int flag;
} __Pyx_CachedCFunction;

/* CallUnboundCMethod0.proto */
static PyObject* __Pyx__CallUnboundCMethod0(__Pyx_CachedCFunction* cfunc, PyObject* self);
#if CYTHON_COMPILING_IN_CPYTHON
#define __Pyx_CallUnboundCMethod0(cfunc, self)\
    (likely((cfunc)->func) ?\
        (likely((cfunc)->flag == METH_NOARGS) ?  (*((cfunc)->func))(self, NULL) :\
         (PY_VERSION_HEX >= 0x030600B1 && likely((cfunc)->flag == METH_FASTCALL) ?\
            (PY_VERSION_HEX >= 0x030700A0 ?\
                (*(__Pyx_PyCFunctionFast)(cfunc)->func)(self, &__pyx_empty_tuple, 0) :\
                (*(__Pyx_PyCFunctionFastWithKeywords)(cfunc)->func)(self, &__pyx_empty_tuple, 0, NULL)) :\
          (PY_VERSION_HEX >= 0x030700A0 && (cfunc)->flag == (METH_FASTCALL | METH_KEYWORDS) ?\
            (*(__Pyx_PyCFunctionFastWithKeywords)(cfunc)->func)(self, &__pyx_empty_tuple, 0, NULL) :\
            (likely((cfunc)->flag == (METH_VARARGS | METH_KEYWORDS)) ?  ((*(PyCFunctionWithKeywords)(cfunc)->func)(self, __pyx_empty_tuple, NULL)) :\
               ((cfunc)->flag == METH_VARARGS ?  (*((cfunc)->func))(self, __pyx_empty_tuple) :\
               __Pyx__CallUnboundCMethod0(cfunc, self)))))) :\
        __Pyx__CallUnboundCMethod0(cfunc, self))
#else
#define __Pyx_CallUnboundCMethod0(cfunc, self)  __Pyx__CallUnboundCMethod0(cfunc, self)
#endif

/* ListAppend.proto */
#if CYTHON_USE_PYLIST_INTERNALS && CYTHON_ASSUME_SAFE_MACROS
static CYTHON_INLINE int __Pyx_PyList_Append(PyObject* list, PyObject* x) {
    PyListObject* L = (PyListObject*) list;
    Py_ssize_t len = Py_SIZE(list);
    if (likely(L->allocated > len) & likely(len > (L->allocated >> 1))) {
        Py_INCREF(x);
        PyList_SET_ITEM(list, len, x);
        Py_SIZE(list) = len+1;
        return 0;
    }
    return PyList_Append(list, x);
}
#else
#define __Pyx_PyList_Append(L,x) PyList_Append(L,x)
#endif

/* PyObjectCallMethod1.proto */
static PyObject* __Pyx_PyObject_CallMethod1(PyObject* obj, PyObject* method_name, PyObject* arg);
static PyObject* __Pyx__PyObject_CallMethod1(PyObject* method, PyObject* arg);

/* append.proto */
static CYTHON_INLINE int __Pyx_PyObject_Append(PyObject* L, PyObject* x);

/* RaiseTooManyValuesToUnpack.proto */
static CYTHON_INLINE void __Pyx_RaiseTooManyValuesError(Py_ssize_t expected);

/* RaiseNeedMoreValuesToUnpack.proto */
static CYTHON_INLINE void __Pyx_RaiseNeedMoreValuesError(Py_ssize_t index);

/* IterFinish.proto */
static CYTHON_INLINE int __Pyx_IterFinish(void);

/* UnpackItemEndCheck.proto */
static int __Pyx_IternextUnpackEndCheck(PyObject *retval, Py_ssize_t expected);

/* ListCompAppend.proto */
#if CYTHON_USE_PYLIST_INTERNALS && CYTHON_ASSUME_SAFE_MACROS
static CYTHON_INLINE int __Pyx_ListComp_Append(PyObject* list, PyObject* x) {
    PyListObject* L = (PyListObject*) list;
    Py_ssize_t len = Py_SIZE(list);
    if (likely(L->allocated > len)) {
        Py_INCREF(x);
        PyList_SET_ITEM(list, len, x);
        Py_SIZE(list) = len+1;
        return 0;
    }
    return PyList_Append(list, x);
}
#else
#define __Pyx_ListComp_Append(L,x) PyList_Append(L,x)
#endif

/* PyObjectCallMethod0.proto */
static PyObject* __Pyx_PyObject_CallMethod0(PyObject* obj, PyObject* method_name);

/* RaiseNoneIterError.proto */
static CYTHON_INLINE void __Pyx_RaiseNoneNotIterableError(void);

/* UnpackTupleError.proto */
static void __Pyx_UnpackTupleError(PyObject *, Py_ssize_t index);

/* UnpackTuple2.proto */
#define __Pyx_unpack_tuple2(tuple, value1, value2, is_tuple, has_known_size, decref_tuple)\
    (likely(is_tuple || PyTuple_Check(tuple)) ?\
        (likely(has_known_size || PyTuple_GET_SIZE(tuple) == 2) ?\
            __Pyx_unpack_tuple2_exact(tuple, value1, value2, decref_tuple) :\
            (__Pyx_UnpackTupleError(tuple, 2), -1)) :\
        __Pyx_unpack_tuple2_generic(tuple, value1, value2, has_known_size, decref_tuple))
static CYTHON_INLINE int __Pyx_unpack_tuple2_exact(
    PyObject* tuple, PyObject** value1, PyObject** value2, int decref_tuple);
static int __Pyx_unpack_tuple2_generic(
    PyObject* tuple, PyObject** value1, PyObject** value2, int has_known_size, int decref_tuple);

/* dict_iter.proto */
static CYTHON_INLINE PyObject* __Pyx_dict_iterator(PyObject* dict, int is_dict, PyObject* method_name,
                                                   Py_ssize_t* p_orig_length, int* p_is_dict);
static CYTHON_INLINE int __Pyx_dict_iter_next(PyObject* dict_or_iter, Py_ssize_t orig_length, Py_ssize_t* ppos,
                                              PyObject** pkey, PyObject** pvalue, PyObject** pitem, int is_dict);

/* SliceObject.proto */
static CYTHON_INLINE PyObject* __Pyx_PyObject_GetSlice(
        PyObject* obj, Py_ssize_t cstart, Py_ssize_t cstop,
        PyObject** py_start, PyObject** py_stop, PyObject** py_slice,
        int has_cstart, int has_cstop, int wraparound);

/* ListExtend.proto */
static CYTHON_INLINE int __Pyx_PyList_Extend(PyObject* L, PyObject* v) {
#if CYTHON_COMPILING_IN_CPYTHON
    PyObject* none = _PyList_Extend((PyListObject*)L, v);
    if (unlikely(!none))
        return -1;
    Py_DECREF(none);
    return 0;
#else
    return PyList_SetSlice(L, PY_SSIZE_T_MAX, PY_SSIZE_T_MAX, v);
#endif
}

/* IncludeStringH.proto */
#include <string.h>

/* decode_c_string_utf16.proto */
static CYTHON_INLINE PyObject *__Pyx_PyUnicode_DecodeUTF16(const char *s, Py_ssize_t size, const char *errors) {
    int byteorder = 0;
    return PyUnicode_DecodeUTF16(s, size, errors, &byteorder);
}
static CYTHON_INLINE PyObject *__Pyx_PyUnicode_DecodeUTF16LE(const char *s, Py_ssize_t size, const char *errors) {
    int byteorder = -1;
    return PyUnicode_DecodeUTF16(s, size, errors, &byteorder);
}
static CYTHON_INLINE PyObject *__Pyx_PyUnicode_DecodeUTF16BE(const char *s, Py_ssize_t size, const char *errors) {
    int byteorder = 1;
    return PyUnicode_DecodeUTF16(s, size, errors, &byteorder);
}

/* decode_c_string.proto */
static CYTHON_INLINE PyObject* __Pyx_decode_c_string(
         const char* cstring, Py_ssize_t start, Py_ssize_t stop,
         const char* encoding, const char* errors,
         PyObject* (*decode_func)(const char *s, Py_ssize_t size, const char *errors));

/* ObjectGetItem.proto */
#if CYTHON_USE_TYPE_SLOTS
static CYTHON_INLINE PyObject *__Pyx_PyObject_GetItem(PyObject *obj, PyObject* key);
#else
#define __Pyx_PyObject_GetItem(obj, key)  PyObject_GetItem(obj, key)
#endif

/* GetException.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_GetException(type, value, tb)  __Pyx__GetException(__pyx_tstate, type, value, tb)
static int __Pyx__GetException(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb);
#else
static int __Pyx_GetException(PyObject **type, PyObject **value, PyObject **tb);
#endif

/* SetItemInt.proto */
#define __Pyx_SetItemInt(o, i, v, type, is_signed, to_py_func, is_list, wraparound, boundscheck)\
    (__Pyx_fits_Py_ssize_t(i, type, is_signed) ?\
    __Pyx_SetItemInt_Fast(o, (Py_ssize_t)i, v, is_list, wraparound, boundscheck) :\
    (is_list ? (PyErr_SetString(PyExc_IndexError, "list assignment index out of range"), -1) :\
               __Pyx_SetItemInt_Generic(o, to_py_func(i), v)))
static int __Pyx_SetItemInt_Generic(PyObject *o, PyObject *j, PyObject *v);
static CYTHON_INLINE int __Pyx_SetItemInt_Fast(PyObject *o, Py_ssize_t i, PyObject *v,
                                               int is_list, int wraparound, int boundscheck);

/* BytesEquals.proto */
static CYTHON_INLINE int __Pyx_PyBytes_Equals(PyObject* s1, PyObject* s2, int equals);

/* UnicodeEquals.proto */
static CYTHON_INLINE int __Pyx_PyUnicode_Equals(PyObject* s1, PyObject* s2, int equals);

/* FastTypeChecks.proto */
#if CYTHON_COMPILING_IN_CPYTHON
#define __Pyx_TypeCheck(obj, type) __Pyx_IsSubtype(Py_TYPE(obj), (PyTypeObject *)type)
static CYTHON_INLINE int __Pyx_IsSubtype(PyTypeObject *a, PyTypeObject *b);
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches(PyObject *err, PyObject *type);
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches2(PyObject *err, PyObject *type1, PyObject *type2);
#else
#define __Pyx_TypeCheck(obj, type) PyObject_TypeCheck(obj, (PyTypeObject *)type)
#define __Pyx_PyErr_GivenExceptionMatches(err, type) PyErr_GivenExceptionMatches(err, type)
#define __Pyx_PyErr_GivenExceptionMatches2(err, type1, type2) (PyErr_GivenExceptionMatches(err, type1) || PyErr_GivenExceptionMatches(err, type2))
#endif
#define __Pyx_PyException_Check(obj) __Pyx_TypeCheck(obj, PyExc_Exception)

/* CallUnboundCMethod1.proto */
static PyObject* __Pyx__CallUnboundCMethod1(__Pyx_CachedCFunction* cfunc, PyObject* self, PyObject* arg);
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_CallUnboundCMethod1(__Pyx_CachedCFunction* cfunc, PyObject* self, PyObject* arg);
#else
#define __Pyx_CallUnboundCMethod1(cfunc, self, arg)  __Pyx__CallUnboundCMethod1(cfunc, self, arg)
#endif

/* None.proto */
static CYTHON_INLINE void __Pyx_RaiseUnboundLocalError(const char *varname);

/* PyIntBinop.proto */
#if !CYTHON_COMPILING_IN_PYPY
static PyObject* __Pyx_PyInt_EqObjC(PyObject *op1, PyObject *op2, long intval, int inplace);
#else
#define __Pyx_PyInt_EqObjC(op1, op2, intval, inplace)\
    PyObject_RichCompare(op1, op2, Py_EQ)
    #endif

/* GetAttr.proto */
static CYTHON_INLINE PyObject *__Pyx_GetAttr(PyObject *, PyObject *);

/* GetAttr3.proto */
static CYTHON_INLINE PyObject *__Pyx_GetAttr3(PyObject *, PyObject *, PyObject *);

/* Import.proto */
static PyObject *__Pyx_Import(PyObject *name, PyObject *from_list, int level);

/* ImportFrom.proto */
static PyObject* __Pyx_ImportFrom(PyObject* module, PyObject* name);

/* HasAttr.proto */
static CYTHON_INLINE int __Pyx_HasAttr(PyObject *, PyObject *);

/* PyObject_GenericGetAttrNoDict.proto */
#if CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP && PY_VERSION_HEX < 0x03070000
static CYTHON_INLINE PyObject* __Pyx_PyObject_GenericGetAttrNoDict(PyObject* obj, PyObject* attr_name);
#else
#define __Pyx_PyObject_GenericGetAttrNoDict PyObject_GenericGetAttr
#endif

/* PyObject_GenericGetAttr.proto */
#if CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP && PY_VERSION_HEX < 0x03070000
static PyObject* __Pyx_PyObject_GenericGetAttr(PyObject* obj, PyObject* attr_name);
#else
#define __Pyx_PyObject_GenericGetAttr PyObject_GenericGetAttr
#endif

/* SetVTable.proto */
static int __Pyx_SetVtable(PyObject *dict, void *vtable);

/* SetupReduce.proto */
static int __Pyx_setup_reduce(PyObject* type_obj);

/* CLineInTraceback.proto */
#ifdef CYTHON_CLINE_IN_TRACEBACK
#define __Pyx_CLineForTraceback(tstate, c_line)  (((CYTHON_CLINE_IN_TRACEBACK)) ? c_line : 0)
#else
static int __Pyx_CLineForTraceback(PyThreadState *tstate, int c_line);
#endif

/* CodeObjectCache.proto */
typedef struct {
    PyCodeObject* code_object;
    int code_line;
} __Pyx_CodeObjectCacheEntry;
struct __Pyx_CodeObjectCache {
    int count;
    int max_count;
    __Pyx_CodeObjectCacheEntry* entries;
};
static struct __Pyx_CodeObjectCache __pyx_code_cache = {0,0,NULL};
static int __pyx_bisect_code_objects(__Pyx_CodeObjectCacheEntry* entries, int count, int code_line);
static PyCodeObject *__pyx_find_code_object(int code_line);
static void __pyx_insert_code_object(int code_line, PyCodeObject* code_object);

/* AddTraceback.proto */
static void __Pyx_AddTraceback(const char *funcname, int c_line,
                               int py_line, const char *filename);

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_int(int value);

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_long(long value);

/* Print.proto */
static int __Pyx_Print(PyObject*, PyObject *, int);
#if CYTHON_COMPILING_IN_PYPY || PY_MAJOR_VERSION >= 3
static PyObject* __pyx_print = 0;
static PyObject* __pyx_print_kwargs = 0;
#endif

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_short(short value);

/* CIntFromPy.proto */
static CYTHON_INLINE int __Pyx_PyInt_As_int(PyObject *);

/* CIntFromPy.proto */
static CYTHON_INLINE long __Pyx_PyInt_As_long(PyObject *);

/* CIntFromPy.proto */
static CYTHON_INLINE short __Pyx_PyInt_As_short(PyObject *);

/* PrintOne.proto */
static int __Pyx_PrintOne(PyObject* stream, PyObject *o);

/* CheckBinaryVersion.proto */
static int __Pyx_check_binary_version(void);

/* InitStrings.proto */
static int __Pyx_InitStrings(__Pyx_StringTabEntry *t);

static PyObject *__pyx_f_11pymarketcap_4core_11Pymarketcap__is_symbol(struct __pyx_obj_11pymarketcap_4core_Pymarketcap *__pyx_v_self, PyObject *__pyx_v_currency, int __pyx_skip_dispatch); /* proto*/
static PyObject *__pyx_f_11pymarketcap_4core_11Pymarketcap__quick_search(struct __pyx_obj_11pymarketcap_4core_Pymarketcap *__pyx_v_self, int __pyx_skip_dispatch, struct __pyx_opt_args_11pymarketcap_4core_11Pymarketcap__quick_search *__pyx_optional_args); /* proto*/
static PyObject *__pyx_f_11pymarketcap_4core_11Pymarketcap__cache_symbols_ids(struct __pyx_obj_11pymarketcap_4core_Pymarketcap *__pyx_v_self, int __pyx_skip_dispatch); /* proto*/
static PyObject *__pyx_f_11pymarketcap_4core_11Pymarketcap__cache_exchanges_ids(struct __pyx_obj_11pymarketcap_4core_Pymarketcap *__pyx_v_self, int __pyx_skip_dispatch); /* proto*/
static PyObject *__pyx_f_11pymarketcap_4core_11Pymarketcap___currencies_to_convert(struct __pyx_obj_11pymarketcap_4core_Pymarketcap *__pyx_v_self, int __pyx_skip_dispatch); /* proto*/
static PyObject *__pyx_f_11pymarketcap_4core_11Pymarketcap___exchange_names_slugs(struct __pyx_obj_11pymarketcap_4core_Pymarketcap *__pyx_v_self, int __pyx_skip_dispatch); /* proto*/
static PyObject *__pyx_f_11pymarketcap_4core_11Pymarketcap__get(struct __pyx_obj_11pymarketcap_4core_Pymarketcap *__pyx_v_self, char *__pyx_v_url); /* proto*/
static PyObject *__pyx_f_11pymarketcap_4core_11Pymarketcap_stats(struct __pyx_obj_11pymarketcap_4core_Pymarketcap *__pyx_v_self, int __pyx_skip_dispatch, struct __pyx_opt_args_11pymarketcap_4core_11Pymarketcap_stats *__pyx_optional_args); /* proto*/
static PyObject *__pyx_f_11pymarketcap_4core_11Pymarketcap_ticker(struct __pyx_obj_11pymarketcap_4core_Pymarketcap *__pyx_v_self, int __pyx_skip_dispatch, struct __pyx_opt_args_11pymarketcap_4core_11Pymarketcap_ticker *__pyx_optional_args); /* proto*/
static PyObject *__pyx_f_11pymarketcap_4core_11Pymarketcap_convert(struct __pyx_obj_11pymarketcap_4core_Pymarketcap *__pyx_v_self, PyObject *__pyx_v_value, PyObject *__pyx_v_currency_in, PyObject *__pyx_v_currency_out, int __pyx_skip_dispatch); /* proto*/
static PyObject *__pyx_f_11pymarketcap_4core_11Pymarketcap_currency(struct __pyx_obj_11pymarketcap_4core_Pymarketcap *__pyx_v_self, PyObject *__pyx_v_name, int __pyx_skip_dispatch, struct __pyx_opt_args_11pymarketcap_4core_11Pymarketcap_currency *__pyx_optional_args); /* proto*/
static PyObject *__pyx_f_11pymarketcap_4core_11Pymarketcap_markets(struct __pyx_obj_11pymarketcap_4core_Pymarketcap *__pyx_v_self, PyObject *__pyx_v_name, int __pyx_skip_dispatch, struct __pyx_opt_args_11pymarketcap_4core_11Pymarketcap_markets *__pyx_optional_args); /* proto*/
static PyObject *__pyx_f_11pymarketcap_4core_11Pymarketcap_ranks(struct __pyx_obj_11pymarketcap_4core_Pymarketcap *__pyx_v_self, int __pyx_skip_dispatch); /* proto*/
static PyObject *__pyx_f_11pymarketcap_4core_11Pymarketcap_historical(struct __pyx_obj_11pymarketcap_4core_Pymarketcap *__pyx_v_self, PyObject *__pyx_v_name, int __pyx_skip_dispatch, struct __pyx_opt_args_11pymarketcap_4core_11Pymarketcap_historical *__pyx_optional_args); /* proto*/
static PyObject *__pyx_f_11pymarketcap_4core_11Pymarketcap_recently(struct __pyx_obj_11pymarketcap_4core_Pymarketcap *__pyx_v_self, int __pyx_skip_dispatch, struct __pyx_opt_args_11pymarketcap_4core_11Pymarketcap_recently *__pyx_optional_args); /* proto*/
static PyObject *__pyx_f_11pymarketcap_4core_11Pymarketcap_exchange(struct __pyx_obj_11pymarketcap_4core_Pymarketcap *__pyx_v_self, PyObject *__pyx_v_name, int __pyx_skip_dispatch, struct __pyx_opt_args_11pymarketcap_4core_11Pymarketcap_exchange *__pyx_optional_args); /* proto*/
static PyObject *__pyx_f_11pymarketcap_4core_11Pymarketcap_exchanges(struct __pyx_obj_11pymarketcap_4core_Pymarketcap *__pyx_v_self, int __pyx_skip_dispatch, struct __pyx_opt_args_11pymarketcap_4core_11Pymarketcap_exchanges *__pyx_optional_args); /* proto*/
static PyObject *__pyx_f_11pymarketcap_4core_11Pymarketcap_tokens(struct __pyx_obj_11pymarketcap_4core_Pymarketcap *__pyx_v_self, int __pyx_skip_dispatch, struct __pyx_opt_args_11pymarketcap_4core_11Pymarketcap_tokens *__pyx_optional_args); /* proto*/
static PyObject *__pyx_f_11pymarketcap_4core_11Pymarketcap__currency(struct __pyx_obj_11pymarketcap_4core_Pymarketcap *__pyx_v_self, PyObject *__pyx_v_name, int __pyx_skip_dispatch, struct __pyx_opt_args_11pymarketcap_4core_11Pymarketcap__currency *__pyx_optional_args); /* proto*/
static PyObject *__pyx_f_11pymarketcap_4core_11Pymarketcap__global_cap(struct __pyx_obj_11pymarketcap_4core_Pymarketcap *__pyx_v_self, int __pyx_skip_dispatch, struct __pyx_opt_args_11pymarketcap_4core_11Pymarketcap__global_cap *__pyx_optional_args); /* proto*/
static PyObject *__pyx_f_11pymarketcap_4core_11Pymarketcap__dominance(struct __pyx_obj_11pymarketcap_4core_Pymarketcap *__pyx_v_self, int __pyx_skip_dispatch, struct __pyx_opt_args_11pymarketcap_4core_11Pymarketcap__dominance *__pyx_optional_args); /* proto*/
static PyObject *__pyx_f_11pymarketcap_4core_11Pymarketcap_download_logo(struct __pyx_obj_11pymarketcap_4core_Pymarketcap *__pyx_v_self, PyObject *__pyx_v_name, int __pyx_skip_dispatch, struct __pyx_opt_args_11pymarketcap_4core_11Pymarketcap_download_logo *__pyx_optional_args); /* proto*/
static PyObject *__pyx_f_11pymarketcap_4core_11Pymarketcap_download_exchange_logo(struct __pyx_obj_11pymarketcap_4core_Pymarketcap *__pyx_v_self, PyObject *__pyx_v_name, int __pyx_skip_dispatch, struct __pyx_opt_args_11pymarketcap_4core_11Pymarketcap_download_exchange_logo *__pyx_optional_args); /* proto*/

/* Module declarations from 'pymarketcap.core' */
static PyTypeObject *__pyx_ptype_11pymarketcap_4core_Pymarketcap = 0;
static PyObject *__pyx_f_11pymarketcap_4core___pyx_unpickle_Pymarketcap__set_state(struct __pyx_obj_11pymarketcap_4core_Pymarketcap *, PyObject *); /*proto*/
#define __Pyx_MODULE_NAME "pymarketcap.core"
extern int __pyx_module_is_main_pymarketcap__core;
int __pyx_module_is_main_pymarketcap__core = 0;

/* Implementation of 'pymarketcap.core' */
static PyObject *__pyx_builtin_ValueError;
static PyObject *__pyx_builtin_TypeError;
static PyObject *__pyx_builtin_range;
static PyObject *__pyx_builtin_KeyError;
static PyObject *__pyx_builtin_NotImplementedError;
static const char __pyx_k_1[] = "\\1";
static const char __pyx_k_d[] = "%d";
static const char __pyx_k__2[] = " ";
static const char __pyx_k__3[] = "";
static const char __pyx_k_id[] = "id";
static const char __pyx_k_re[] = "re";
static const char __pyx_k_02d[] = "%02d";
static const char __pyx_k_404[] = "404";
static const char __pyx_k_429[] = "429";
static const char __pyx_k_USD[] = "USD";
static const char __pyx_k__11[] = "&";
static const char __pyx_k_d_d[] = "\"(-*\\d+(?:\\.\\d+)?)\"";
static const char __pyx_k_day[] = "day";
static const char __pyx_k_end[] = "end";
static const char __pyx_k_new[] = "__new__";
static const char __pyx_k_now[] = "now";
static const char __pyx_k_sub[] = "sub";
static const char __pyx_k_url[] = "url";
static const char __pyx_k_code[] = "code";
static const char __pyx_k_dict[] = "__dict__";
static const char __pyx_k_file[] = "file";
static const char __pyx_k_json[] = "json";
static const char __pyx_k_keys[] = "keys";
static const char __pyx_k_main[] = "__main__";
static const char __pyx_k_name[] = "name";
static const char __pyx_k_rank[] = "rank";
static const char __pyx_k_size[] = "size";
static const char __pyx_k_slug[] = "slug";
static const char __pyx_k_test[] = "__test__";
static const char __pyx_k_text[] = "text";
static const char __pyx_k_time[] = "time";
static const char __pyx_k_year[] = "year";
static const char __pyx_k_DEBUG[] = "DEBUG: ";
static const char __pyx_k_coins[] = "coins";
static const char __pyx_k_debug[] = "debug";
static const char __pyx_k_end_s[] = "end=%s";
static const char __pyx_k_exc_s[] = "exc = %s";
static const char __pyx_k_index[] = "index";
static const char __pyx_k_items[] = "items";
static const char __pyx_k_limit[] = "limit";
static const char __pyx_k_loads[] = "loads";
static const char __pyx_k_lower[] = "lower";
static const char __pyx_k_month[] = "month";
static const char __pyx_k_print[] = "print";
static const char __pyx_k_range[] = "range";
static const char __pyx_k_ranks[] = "ranks";
static const char __pyx_k_s_png[] = "%s.png";
static const char __pyx_k_start[] = "start";
static const char __pyx_k_stats[] = "stats";
static const char __pyx_k_upper[] = "upper";
static const char __pyx_k_value[] = "value";
static const char __pyx_k_Graphs[] = "Graphs";
static const char __pyx_k_append[] = "append";
static const char __pyx_k_decode[] = "decode";
static const char __pyx_k_encode[] = "encode";
static const char __pyx_k_graphs[] = "graphs";
static const char __pyx_k_import[] = "__import__";
static const char __pyx_k_name_2[] = "__name__";
static const char __pyx_k_number[] = "number";
static const char __pyx_k_pickle[] = "pickle";
static const char __pyx_k_re_sub[] = "re_sub";
static const char __pyx_k_reduce[] = "__reduce__";
static const char __pyx_k_remove[] = "remove";
static const char __pyx_k_revert[] = "revert";
static const char __pyx_k_symbol[] = "symbol";
static const char __pyx_k_ticker[] = "ticker";
static const char __pyx_k_tokens[] = "tokens";
static const char __pyx_k_update[] = "update";
static const char __pyx_k_values[] = "values";
static const char __pyx_k_bitcoin[] = "bitcoin";
static const char __pyx_k_convert[] = "convert";
static const char __pyx_k_findall[] = "findall";
static const char __pyx_k_history[] = "history";
static const char __pyx_k_islower[] = "islower";
static const char __pyx_k_isupper[] = "isupper";
static const char __pyx_k_markets[] = "markets";
static const char __pyx_k_replace[] = "replace";
static const char __pyx_k_start_d[] = "&start=%d";
static const char __pyx_k_start_s[] = "?start=%s";
static const char __pyx_k_timeout[] = "timeout";
static const char __pyx_k_KeyError[] = "KeyError";
static const char __pyx_k_currency[] = "currency";
static const char __pyx_k_datetime[] = "datetime";
static const char __pyx_k_exchange[] = "exchange";
static const char __pyx_k_filename[] = "filename";
static const char __pyx_k_getstate[] = "__getstate__";
static const char __pyx_k_pyx_type[] = "__pyx_type";
static const char __pyx_k_recently[] = "recently";
static const char __pyx_k_setstate[] = "__setstate__";
static const char __pyx_k_HTTPError[] = "HTTPError";
static const char __pyx_k_TypeError[] = "TypeError";
static const char __pyx_k_convert_s[] = "&convert=%s";
static const char __pyx_k_dominance[] = "dominance";
static const char __pyx_k_exchanges[] = "exchanges";
static const char __pyx_k_is_symbol[] = "_is_symbol";
static const char __pyx_k_price_usd[] = "price_usd";
static const char __pyx_k_processer[] = "processer";
static const char __pyx_k_pyx_state[] = "__pyx_state";
static const char __pyx_k_reduce_ex[] = "__reduce_ex__";
static const char __pyx_k_ValueError[] = "ValueError";
static const char __pyx_k_currency_2[] = "_currency";
static const char __pyx_k_data_a_z_d[] = "data-([a-z]+)=\"\\d";
static const char __pyx_k_global_cap[] = "global_cap";
static const char __pyx_k_historical[] = "historical";
static const char __pyx_k_pyx_result[] = "__pyx_result";
static const char __pyx_k_pyx_vtable[] = "__pyx_vtable__";
static const char __pyx_k_re_findall[] = "re_findall";
static const char __pyx_k_PickleError[] = "PickleError";
static const char __pyx_k_convert_s_2[] = "?convert=%s";
static const char __pyx_k_currency_in[] = "currency_in";
static const char __pyx_k_dominance_2[] = "_dominance";
static const char __pyx_k_pymarketcap[] = "pymarketcap";
static const char __pyx_k_s__dx_d_png[] = "%s_%dx%d.png";
static const char __pyx_k_status_code[] = "status_code";
static const char __pyx_k_urlretrieve[] = "urlretrieve";
static const char __pyx_k_currency_out[] = "currency_out";
static const char __pyx_k_global_cap_2[] = "_global_cap";
static const char __pyx_k_pyx_checksum[] = "__pyx_checksum";
static const char __pyx_k_quick_search[] = "_quick_search";
static const char __pyx_k_stringsource[] = "stringsource";
static const char __pyx_k_urllib_error[] = "urllib.error";
static const char __pyx_k_INVALID_COINS[] = "INVALID_COINS";
static const char __pyx_k_download_logo[] = "download_logo";
static const char __pyx_k_get_to_memory[] = "get_to_memory";
static const char __pyx_k_reduce_cython[] = "__reduce_cython__";
static const char __pyx_k_data_a_z_d_d_e[] = "data-([a-z]+)=\"(\\d+\\.*[\\d|e|-]*)\"";
static const char __pyx_k_exchange_slugs[] = "exchange_slugs";
static const char __pyx_k_urllib_request[] = "urllib.request";
static const char __pyx_k_DEFAULT_TIMEOUT[] = "DEFAULT_TIMEOUT";
static const char __pyx_k_converter_cache[] = "converter_cache";
static const char __pyx_k_correspondences[] = "correspondences";
static const char __pyx_k_http_errors_map[] = "http_errors_map";
static const char __pyx_k_pyx_PickleError[] = "__pyx_PickleError";
static const char __pyx_k_setstate_cython[] = "__setstate_cython__";
static const char __pyx_k_graphs_interface[] = "_graphs_interface";
static const char __pyx_k_pymarketcap_core[] = "pymarketcap.core";
static const char __pyx_k_pymarketcap_curl[] = "pymarketcap.curl";
static const char __pyx_k_cache_symbols_ids[] = "_cache_symbols_ids";
static const char __pyx_k_cline_in_traceback[] = "cline_in_traceback";
static const char __pyx_k_http_error_numbers[] = "http_error_numbers";
static const char __pyx_k_pymarketcap_consts[] = "pymarketcap.consts";
static const char __pyx_k_pymarketcap_errors[] = "pymarketcap.errors";
static const char __pyx_k_NotImplementedError[] = "NotImplementedError";
static const char __pyx_k_Status_code_d_Url_s[] = "Status code -> %d | Url -> %s";
static const char __pyx_k_cache_exchanges_ids[] = "_cache_exchanges_ids";
static const char __pyx_k_ids_correspondences[] = "ids_correspondences";
static const char __pyx_k_Exchange_s_not_found[] = "Exchange %s not found.";
static const char __pyx_k_exchange_names_slugs[] = "__exchange_names_slugs";
static const char __pyx_k_currencies_to_convert[] = "__currencies_to_convert";
static const char __pyx_k_CoinmarketcapHTTPError[] = "CoinmarketcapHTTPError";
static const char __pyx_k_EXCEPTIONAL_COIN_SLUGS[] = "EXCEPTIONAL_COIN_SLUGS";
static const char __pyx_k_download_exchange_logo[] = "download_exchange_logo";
static const char __pyx_k_currencies_to_convert_2[] = "currencies_to_convert";
static const char __pyx_k_currency_exchange_rates[] = "currency_exchange_rates";
static const char __pyx_k_dx_d_is_not_a_valid_size[] = "%dx%d is not a valid size.";
static const char __pyx_k_in_name_index_position_0[] = " in name index position (0).";
static const char __pyx_k_in_slug_index_position_1[] = " in slug index position (1).";
static const char __pyx_k_pyx_unpickle_Pymarketcap[] = "__pyx_unpickle_Pymarketcap";
static const char __pyx_k_CoinmarketcapHTTPError404[] = "CoinmarketcapHTTPError404";
static const char __pyx_k_Slug_catched_but_not_found[] = "Slug catched but not found";
static const char __pyx_k_size_dx_d_Try_another_size[] = "size %dx%d. Try another size.";
static const char __pyx_k_Possible_valid_slug_names_r[] = "\nPossible valid slug names: %r.";
static const char __pyx_k_https_coinmarketcap_com_new[] = "https://coinmarketcap.com/new/";
static const char __pyx_k_s_is_not_a_valid_currency_name[] = "%s is not a valid currency name. See 'symbols' or 'coins'";
static const char __pyx_k_s_is_not_a_valid_exchange_name[] = "%s is not a valid exchange name. See exchange_slugs";
static const char __pyx_k_The_currency_s_is_not_valid_See[] = "The currency %s is not valid. See 'symbols' instance attribute.";
static const char __pyx_k_https_graphs2_coinmarketcap_com[] = "https://graphs2.coinmarketcap.com/currencies/%s/";
static const char __pyx_k_or_coins_properties_for_get_all[] = " or 'coins' properties for get all valid currencies.";
static const char __pyx_k_properties_for_get_all_valid_cu[] = " properties for get all valid currencies.";
static const char __pyx_k_property_for_get_all_valid_exch[] = " property for get all valid exchanges.";
static const char __pyx_k_s_is_not_a_valid_currrency_name[] = "%s is not a valid currrency name. See 'symbols'";
static const char __pyx_k_API_wraper_and_web_scraper_modul[] = "API wraper and web scraper module.";
static const char __pyx_k_CoinmarketcapTooManyRequestsErro[] = "CoinmarketcapTooManyRequestsError";
static const char __pyx_k_Exchange_name_catched_but_not_fo[] = "Exchange name catched but not found";
static const char __pyx_k_Incompatible_checksums_s_vs_0x7d[] = "Incompatible checksums (%s vs 0x7d79c3c = (__repeated_symbols, _coins, _converter_cache, _correspondences, _currencies_to_convert, _exchange_names, _exchange_names_slugs_ids, _exchange_slugs, _ids_correspondences, _symbols, _total_currencies, _total_exchanges, debug, graphs, timeout))";
static const char __pyx_k_Invalid_currency_s_See_currencie[] = "Invalid currency: '%s'. See currencies_to_convert instance attribute.";
static const char __pyx_k_Seems_that_s_currency_doesn_t_al[] = "Seems that %s currency doesn't allows to be downloaded with ";
static const char __pyx_k_Seems_that_s_exchange_doesn_t_al[] = "Seems that %s exchange doesn't allows to be downloaded with ";
static const char __pyx_k_The_symbol_s_has_more_than_one_c[] = "The symbol \"%s\" has more than one correspondence ";
static const char __pyx_k_https_api_coinmarketcap_com_v1_g[] = "https://api.coinmarketcap.com/v1/global/?convert=%s";
static const char __pyx_k_https_api_coinmarketcap_com_v1_t[] = "https://api.coinmarketcap.com/v1/ticker/?limit=%d";
static const char __pyx_k_https_coinmarketcap_com_currenci[] = "https://coinmarketcap.com/currencies/%s/";
static const char __pyx_k_https_coinmarketcap_com_exchange[] = "https://coinmarketcap.com/exchanges/%s/";
static const char __pyx_k_https_coinmarketcap_com_tokens_v[] = "https://coinmarketcap.com/tokens/views/all/";
static const char __pyx_k_https_s2_coinmarketcap_com_gener[] = "https://s2.coinmarketcap.com/generated/search/quick_search_exchanges.json";
static const char __pyx_k_https_s2_coinmarketcap_com_stati[] = "https://s2.coinmarketcap.com/static/img/coins/%dx%d/%d.png";
static const char __pyx_k_with_coin_slugs_in_Coinmarketcap[] = "with coin slugs in Coinmarketcap. Please get this currency as slug. ";
static const char __pyx_k_https_graphs2_coinmarketcap_com_2[] = "https://graphs2.coinmarketcap.com/global/marketcap-total/";
static const char __pyx_k_https_graphs2_coinmarketcap_com_3[] = "https://graphs2.coinmarketcap.com/global/marketcap-altcoin/";
static const char __pyx_k_https_graphs2_coinmarketcap_com_4[] = "https://graphs2.coinmarketcap.com/global/dominance/";
static const char __pyx_k_https_api_coinmarketcap_com_v1_t_2[] = "https://api.coinmarketcap.com/v1/ticker/%s";
static const char __pyx_k_https_coinmarketcap_com_currenci_2[] = "https://coinmarketcap.com/currencies/%s/historical-data/";
static const char __pyx_k_https_coinmarketcap_com_exchange_2[] = "https://coinmarketcap.com/exchanges/volume/24-hour/all/";
static const char __pyx_k_https_s2_coinmarketcap_com_gener_2[] = "https://s2.coinmarketcap.com/generated/search/quick_search.json";
static const char __pyx_k_https_s2_coinmarketcap_com_stati_2[] = "https://s2.coinmarketcap.com/static/img/exchanges/%dx%d/%s.png";
static PyObject *__pyx_kp_s_02d;
static PyObject *__pyx_kp_s_1;
static PyObject *__pyx_kp_s_404;
static PyObject *__pyx_kp_s_429;
static PyObject *__pyx_n_s_CoinmarketcapHTTPError;
static PyObject *__pyx_n_s_CoinmarketcapHTTPError404;
static PyObject *__pyx_n_s_CoinmarketcapTooManyRequestsErro;
static PyObject *__pyx_kp_s_DEBUG;
static PyObject *__pyx_n_s_DEFAULT_TIMEOUT;
static PyObject *__pyx_n_s_EXCEPTIONAL_COIN_SLUGS;
static PyObject *__pyx_kp_s_Exchange_name_catched_but_not_fo;
static PyObject *__pyx_kp_s_Exchange_s_not_found;
static PyObject *__pyx_n_s_Graphs;
static PyObject *__pyx_n_s_HTTPError;
static PyObject *__pyx_n_s_INVALID_COINS;
static PyObject *__pyx_kp_s_Incompatible_checksums_s_vs_0x7d;
static PyObject *__pyx_kp_s_Invalid_currency_s_See_currencie;
static PyObject *__pyx_n_s_KeyError;
static PyObject *__pyx_n_s_NotImplementedError;
static PyObject *__pyx_n_s_PickleError;
static PyObject *__pyx_kp_s_Possible_valid_slug_names_r;
static PyObject *__pyx_kp_s_Seems_that_s_currency_doesn_t_al;
static PyObject *__pyx_kp_s_Seems_that_s_exchange_doesn_t_al;
static PyObject *__pyx_kp_s_Slug_catched_but_not_found;
static PyObject *__pyx_kp_s_Status_code_d_Url_s;
static PyObject *__pyx_kp_s_The_currency_s_is_not_valid_See;
static PyObject *__pyx_kp_s_The_symbol_s_has_more_than_one_c;
static PyObject *__pyx_n_s_TypeError;
static PyObject *__pyx_n_s_USD;
static PyObject *__pyx_n_s_ValueError;
static PyObject *__pyx_kp_s__11;
static PyObject *__pyx_kp_s__2;
static PyObject *__pyx_kp_s__3;
static PyObject *__pyx_n_s_append;
static PyObject *__pyx_n_s_bitcoin;
static PyObject *__pyx_n_s_cache_exchanges_ids;
static PyObject *__pyx_n_s_cache_symbols_ids;
static PyObject *__pyx_n_s_cline_in_traceback;
static PyObject *__pyx_n_s_code;
static PyObject *__pyx_n_s_coins;
static PyObject *__pyx_n_s_convert;
static PyObject *__pyx_kp_s_convert_s;
static PyObject *__pyx_kp_s_convert_s_2;
static PyObject *__pyx_n_s_converter_cache;
static PyObject *__pyx_n_s_correspondences;
static PyObject *__pyx_n_s_currencies_to_convert;
static PyObject *__pyx_n_s_currencies_to_convert_2;
static PyObject *__pyx_n_s_currency;
static PyObject *__pyx_n_s_currency_2;
static PyObject *__pyx_n_s_currency_exchange_rates;
static PyObject *__pyx_n_s_currency_in;
static PyObject *__pyx_n_s_currency_out;
static PyObject *__pyx_kp_s_d;
static PyObject *__pyx_kp_s_d_d;
static PyObject *__pyx_kp_s_data_a_z_d;
static PyObject *__pyx_kp_s_data_a_z_d_d_e;
static PyObject *__pyx_n_s_datetime;
static PyObject *__pyx_n_s_day;
static PyObject *__pyx_n_s_debug;
static PyObject *__pyx_n_s_decode;
static PyObject *__pyx_n_s_dict;
static PyObject *__pyx_n_s_dominance;
static PyObject *__pyx_n_s_dominance_2;
static PyObject *__pyx_n_s_download_exchange_logo;
static PyObject *__pyx_n_s_download_logo;
static PyObject *__pyx_kp_s_dx_d_is_not_a_valid_size;
static PyObject *__pyx_n_s_encode;
static PyObject *__pyx_n_s_end;
static PyObject *__pyx_kp_s_end_s;
static PyObject *__pyx_kp_s_exc_s;
static PyObject *__pyx_n_s_exchange;
static PyObject *__pyx_n_s_exchange_names_slugs;
static PyObject *__pyx_n_s_exchange_slugs;
static PyObject *__pyx_n_s_exchanges;
static PyObject *__pyx_n_s_file;
static PyObject *__pyx_n_s_filename;
static PyObject *__pyx_n_s_findall;
static PyObject *__pyx_n_s_get_to_memory;
static PyObject *__pyx_n_s_getstate;
static PyObject *__pyx_n_s_global_cap;
static PyObject *__pyx_n_s_global_cap_2;
static PyObject *__pyx_n_s_graphs;
static PyObject *__pyx_n_s_graphs_interface;
static PyObject *__pyx_n_s_historical;
static PyObject *__pyx_n_s_history;
static PyObject *__pyx_n_s_http_error_numbers;
static PyObject *__pyx_n_s_http_errors_map;
static PyObject *__pyx_kp_b_https_api_coinmarketcap_com_v1_g;
static PyObject *__pyx_kp_s_https_api_coinmarketcap_com_v1_t;
static PyObject *__pyx_kp_s_https_api_coinmarketcap_com_v1_t_2;
static PyObject *__pyx_kp_b_https_coinmarketcap_com_currenci;
static PyObject *__pyx_kp_s_https_coinmarketcap_com_currenci_2;
static PyObject *__pyx_kp_s_https_coinmarketcap_com_exchange;
static PyObject *__pyx_kp_b_https_coinmarketcap_com_exchange_2;
static PyObject *__pyx_kp_b_https_coinmarketcap_com_new;
static PyObject *__pyx_kp_b_https_coinmarketcap_com_tokens_v;
static PyObject *__pyx_kp_b_https_graphs2_coinmarketcap_com;
static PyObject *__pyx_kp_b_https_graphs2_coinmarketcap_com_2;
static PyObject *__pyx_kp_b_https_graphs2_coinmarketcap_com_3;
static PyObject *__pyx_kp_b_https_graphs2_coinmarketcap_com_4;
static PyObject *__pyx_kp_b_https_s2_coinmarketcap_com_gener;
static PyObject *__pyx_kp_b_https_s2_coinmarketcap_com_gener_2;
static PyObject *__pyx_kp_s_https_s2_coinmarketcap_com_stati;
static PyObject *__pyx_kp_s_https_s2_coinmarketcap_com_stati_2;
static PyObject *__pyx_n_s_id;
static PyObject *__pyx_n_s_ids_correspondences;
static PyObject *__pyx_n_s_import;
static PyObject *__pyx_kp_s_in_name_index_position_0;
static PyObject *__pyx_kp_s_in_slug_index_position_1;
static PyObject *__pyx_n_s_index;
static PyObject *__pyx_n_s_is_symbol;
static PyObject *__pyx_n_s_islower;
static PyObject *__pyx_n_s_isupper;
static PyObject *__pyx_n_s_items;
static PyObject *__pyx_n_s_json;
static PyObject *__pyx_n_s_keys;
static PyObject *__pyx_n_s_limit;
static PyObject *__pyx_n_s_loads;
static PyObject *__pyx_n_s_lower;
static PyObject *__pyx_n_s_main;
static PyObject *__pyx_n_s_markets;
static PyObject *__pyx_n_s_month;
static PyObject *__pyx_n_s_name;
static PyObject *__pyx_n_s_name_2;
static PyObject *__pyx_n_s_new;
static PyObject *__pyx_n_s_now;
static PyObject *__pyx_n_s_number;
static PyObject *__pyx_kp_s_or_coins_properties_for_get_all;
static PyObject *__pyx_n_s_pickle;
static PyObject *__pyx_n_s_price_usd;
static PyObject *__pyx_n_s_print;
static PyObject *__pyx_n_s_processer;
static PyObject *__pyx_kp_s_properties_for_get_all_valid_cu;
static PyObject *__pyx_kp_s_property_for_get_all_valid_exch;
static PyObject *__pyx_n_s_pymarketcap;
static PyObject *__pyx_n_s_pymarketcap_consts;
static PyObject *__pyx_n_s_pymarketcap_core;
static PyObject *__pyx_n_s_pymarketcap_curl;
static PyObject *__pyx_n_s_pymarketcap_errors;
static PyObject *__pyx_n_s_pyx_PickleError;
static PyObject *__pyx_n_s_pyx_checksum;
static PyObject *__pyx_n_s_pyx_result;
static PyObject *__pyx_n_s_pyx_state;
static PyObject *__pyx_n_s_pyx_type;
static PyObject *__pyx_n_s_pyx_unpickle_Pymarketcap;
static PyObject *__pyx_n_s_pyx_vtable;
static PyObject *__pyx_n_s_quick_search;
static PyObject *__pyx_n_s_range;
static PyObject *__pyx_n_s_rank;
static PyObject *__pyx_n_s_ranks;
static PyObject *__pyx_n_s_re;
static PyObject *__pyx_n_s_re_findall;
static PyObject *__pyx_n_s_re_sub;
static PyObject *__pyx_n_s_recently;
static PyObject *__pyx_n_s_reduce;
static PyObject *__pyx_n_s_reduce_cython;
static PyObject *__pyx_n_s_reduce_ex;
static PyObject *__pyx_n_s_remove;
static PyObject *__pyx_n_s_replace;
static PyObject *__pyx_n_s_revert;
static PyObject *__pyx_kp_s_s__dx_d_png;
static PyObject *__pyx_kp_s_s_is_not_a_valid_currency_name;
static PyObject *__pyx_kp_s_s_is_not_a_valid_currrency_name;
static PyObject *__pyx_kp_s_s_is_not_a_valid_exchange_name;
static PyObject *__pyx_kp_s_s_png;
static PyObject *__pyx_n_s_setstate;
static PyObject *__pyx_n_s_setstate_cython;
static PyObject *__pyx_n_s_size;
static PyObject *__pyx_kp_s_size_dx_d_Try_another_size;
static PyObject *__pyx_n_s_slug;
static PyObject *__pyx_n_s_start;
static PyObject *__pyx_kp_s_start_d;
static PyObject *__pyx_kp_s_start_s;
static PyObject *__pyx_n_s_stats;
static PyObject *__pyx_n_s_status_code;
static PyObject *__pyx_kp_s_stringsource;
static PyObject *__pyx_n_s_sub;
static PyObject *__pyx_n_s_symbol;
static PyObject *__pyx_n_s_test;
static PyObject *__pyx_n_s_text;
static PyObject *__pyx_n_s_ticker;
static PyObject *__pyx_n_s_time;
static PyObject *__pyx_n_s_timeout;
static PyObject *__pyx_n_s_tokens;
static PyObject *__pyx_n_s_update;
static PyObject *__pyx_n_s_upper;
static PyObject *__pyx_n_s_url;
static PyObject *__pyx_n_s_urllib_error;
static PyObject *__pyx_n_s_urllib_request;
static PyObject *__pyx_n_s_urlretrieve;
static PyObject *__pyx_n_s_value;
static PyObject *__pyx_n_s_values;
static PyObject *__pyx_kp_s_with_coin_slugs_in_Coinmarketcap;
static PyObject *__pyx_n_s_year;
static int __pyx_pf_11pymarketcap_4core_11Pymarketcap___init__(struct __pyx_obj_11pymarketcap_4core_Pymarketcap *__pyx_v_self, PyObject *__pyx_v_timeout, PyObject *__pyx_v_debug); /* proto */
static PyObject *__pyx_pf_11pymarketcap_4core_11Pymarketcap_17_graphs_interface___get__(struct __pyx_obj_11pymarketcap_4core_Pymarketcap *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_11pymarketcap_4core_11Pymarketcap_15correspondences___get__(struct __pyx_obj_11pymarketcap_4core_Pymarketcap *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_11pymarketcap_4core_11Pymarketcap_19ids_correspondences___get__(struct __pyx_obj_11pymarketcap_4core_Pymarketcap *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_11pymarketcap_4core_11Pymarketcap_2_is_symbol(struct __pyx_obj_11pymarketcap_4core_Pymarketcap *__pyx_v_self, PyObject *__pyx_v_currency); /* proto */
static PyObject *__pyx_pf_11pymarketcap_4core_11Pymarketcap_4_quick_search(struct __pyx_obj_11pymarketcap_4core_Pymarketcap *__pyx_v_self, PyObject *__pyx_v_exchanges); /* proto */
static PyObject *__pyx_pf_11pymarketcap_4core_11Pymarketcap_6_cache_symbols_ids(struct __pyx_obj_11pymarketcap_4core_Pymarketcap *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_11pymarketcap_4core_11Pymarketcap_8_cache_exchanges_ids(struct __pyx_obj_11pymarketcap_4core_Pymarketcap *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_11pymarketcap_4core_11Pymarketcap_7symbols___get__(struct __pyx_obj_11pymarketcap_4core_Pymarketcap *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_11pymarketcap_4core_11Pymarketcap_5coins___get__(struct __pyx_obj_11pymarketcap_4core_Pymarketcap *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_11pymarketcap_4core_11Pymarketcap_16total_currencies___get__(struct __pyx_obj_11pymarketcap_4core_Pymarketcap *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_11pymarketcap_4core_11Pymarketcap_21currencies_to_convert___get__(struct __pyx_obj_11pymarketcap_4core_Pymarketcap *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_11pymarketcap_4core_11Pymarketcap_10__currencies_to_convert(struct __pyx_obj_11pymarketcap_4core_Pymarketcap *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_11pymarketcap_4core_11Pymarketcap_14exchange_names___get__(struct __pyx_obj_11pymarketcap_4core_Pymarketcap *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_11pymarketcap_4core_11Pymarketcap_12__exchange_names_slugs(struct __pyx_obj_11pymarketcap_4core_Pymarketcap *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_11pymarketcap_4core_11Pymarketcap_14exchange_slugs___get__(struct __pyx_obj_11pymarketcap_4core_Pymarketcap *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_11pymarketcap_4core_11Pymarketcap_15total_exchanges___get__(struct __pyx_obj_11pymarketcap_4core_Pymarketcap *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_11pymarketcap_4core_11Pymarketcap_15converter_cache___get__(struct __pyx_obj_11pymarketcap_4core_Pymarketcap *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_11pymarketcap_4core_11Pymarketcap_14stats(struct __pyx_obj_11pymarketcap_4core_Pymarketcap *__pyx_v_self, PyObject *__pyx_v_convert); /* proto */
static PyObject *__pyx_pf_11pymarketcap_4core_11Pymarketcap_13ticker_badges___get__(CYTHON_UNUSED struct __pyx_obj_11pymarketcap_4core_Pymarketcap *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_11pymarketcap_4core_11Pymarketcap_16ticker(struct __pyx_obj_11pymarketcap_4core_Pymarketcap *__pyx_v_self, PyObject *__pyx_v_currency, PyObject *__pyx_v_limit, PyObject *__pyx_v_start, PyObject *__pyx_v_convert); /* proto */
static PyObject *__pyx_pf_11pymarketcap_4core_11Pymarketcap_23currency_exchange_rates___get__(struct __pyx_obj_11pymarketcap_4core_Pymarketcap *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_11pymarketcap_4core_11Pymarketcap_18convert(struct __pyx_obj_11pymarketcap_4core_Pymarketcap *__pyx_v_self, PyObject *__pyx_v_value, PyObject *__pyx_v_currency_in, PyObject *__pyx_v_currency_out); /* proto */
static PyObject *__pyx_pf_11pymarketcap_4core_11Pymarketcap_20currency(struct __pyx_obj_11pymarketcap_4core_Pymarketcap *__pyx_v_self, PyObject *__pyx_v_name, PyObject *__pyx_v_convert); /* proto */
static PyObject *__pyx_pf_11pymarketcap_4core_11Pymarketcap_22markets(struct __pyx_obj_11pymarketcap_4core_Pymarketcap *__pyx_v_self, PyObject *__pyx_v_name, PyObject *__pyx_v_convert); /* proto */
static PyObject *__pyx_pf_11pymarketcap_4core_11Pymarketcap_24ranks(struct __pyx_obj_11pymarketcap_4core_Pymarketcap *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_11pymarketcap_4core_11Pymarketcap_26historical(struct __pyx_obj_11pymarketcap_4core_Pymarketcap *__pyx_v_self, PyObject *__pyx_v_name, PyObject *__pyx_v_start, PyObject *__pyx_v_end, PyObject *__pyx_v_revert); /* proto */
static PyObject *__pyx_pf_11pymarketcap_4core_11Pymarketcap_28recently(struct __pyx_obj_11pymarketcap_4core_Pymarketcap *__pyx_v_self, PyObject *__pyx_v_convert); /* proto */
static PyObject *__pyx_pf_11pymarketcap_4core_11Pymarketcap_30exchange(struct __pyx_obj_11pymarketcap_4core_Pymarketcap *__pyx_v_self, PyObject *__pyx_v_name, PyObject *__pyx_v_convert); /* proto */
static PyObject *__pyx_pf_11pymarketcap_4core_11Pymarketcap_32exchanges(struct __pyx_obj_11pymarketcap_4core_Pymarketcap *__pyx_v_self, PyObject *__pyx_v_convert); /* proto */
static PyObject *__pyx_pf_11pymarketcap_4core_11Pymarketcap_34tokens(struct __pyx_obj_11pymarketcap_4core_Pymarketcap *__pyx_v_self, PyObject *__pyx_v_convert); /* proto */
static PyObject *__pyx_pf_11pymarketcap_4core_11Pymarketcap_36_currency(struct __pyx_obj_11pymarketcap_4core_Pymarketcap *__pyx_v_self, PyObject *__pyx_v_name, PyObject *__pyx_v_start, PyObject *__pyx_v_end); /* proto */
static PyObject *__pyx_pf_11pymarketcap_4core_11Pymarketcap_38_global_cap(struct __pyx_obj_11pymarketcap_4core_Pymarketcap *__pyx_v_self, PyObject *__pyx_v_bitcoin, PyObject *__pyx_v_start, PyObject *__pyx_v_end); /* proto */
static PyObject *__pyx_pf_11pymarketcap_4core_11Pymarketcap_40_dominance(struct __pyx_obj_11pymarketcap_4core_Pymarketcap *__pyx_v_self, PyObject *__pyx_v_start, PyObject *__pyx_v_end); /* proto */
static PyObject *__pyx_pf_11pymarketcap_4core_11Pymarketcap_42download_logo(struct __pyx_obj_11pymarketcap_4core_Pymarketcap *__pyx_v_self, PyObject *__pyx_v_name, PyObject *__pyx_v_size, PyObject *__pyx_v_filename); /* proto */
static PyObject *__pyx_pf_11pymarketcap_4core_11Pymarketcap_44download_exchange_logo(struct __pyx_obj_11pymarketcap_4core_Pymarketcap *__pyx_v_self, PyObject *__pyx_v_name, PyObject *__pyx_v_size, PyObject *__pyx_v_filename); /* proto */
static PyObject *__pyx_pf_11pymarketcap_4core_11Pymarketcap_16_correspondences___get__(struct __pyx_obj_11pymarketcap_4core_Pymarketcap *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_11pymarketcap_4core_11Pymarketcap_20_ids_correspondences___get__(struct __pyx_obj_11pymarketcap_4core_Pymarketcap *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_11pymarketcap_4core_11Pymarketcap_8_symbols___get__(struct __pyx_obj_11pymarketcap_4core_Pymarketcap *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_11pymarketcap_4core_11Pymarketcap_6_coins___get__(struct __pyx_obj_11pymarketcap_4core_Pymarketcap *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_11pymarketcap_4core_11Pymarketcap_17_total_currencies___get__(struct __pyx_obj_11pymarketcap_4core_Pymarketcap *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_11pymarketcap_4core_11Pymarketcap_16_total_exchanges___get__(struct __pyx_obj_11pymarketcap_4core_Pymarketcap *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_11pymarketcap_4core_11Pymarketcap_22_currencies_to_convert___get__(struct __pyx_obj_11pymarketcap_4core_Pymarketcap *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_11pymarketcap_4core_11Pymarketcap_16_converter_cache___get__(struct __pyx_obj_11pymarketcap_4core_Pymarketcap *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_11pymarketcap_4core_11Pymarketcap_15_exchange_names___get__(struct __pyx_obj_11pymarketcap_4core_Pymarketcap *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_11pymarketcap_4core_11Pymarketcap_15_exchange_slugs___get__(struct __pyx_obj_11pymarketcap_4core_Pymarketcap *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_11pymarketcap_4core_11Pymarketcap_18__repeated_symbols___get__(struct __pyx_obj_11pymarketcap_4core_Pymarketcap *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_11pymarketcap_4core_11Pymarketcap_25_exchange_names_slugs_ids___get__(struct __pyx_obj_11pymarketcap_4core_Pymarketcap *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_11pymarketcap_4core_11Pymarketcap_7timeout___get__(struct __pyx_obj_11pymarketcap_4core_Pymarketcap *__pyx_v_self); /* proto */
static int __pyx_pf_11pymarketcap_4core_11Pymarketcap_7timeout_2__set__(struct __pyx_obj_11pymarketcap_4core_Pymarketcap *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static PyObject *__pyx_pf_11pymarketcap_4core_11Pymarketcap_6graphs___get__(struct __pyx_obj_11pymarketcap_4core_Pymarketcap *__pyx_v_self); /* proto */
static int __pyx_pf_11pymarketcap_4core_11Pymarketcap_6graphs_2__set__(struct __pyx_obj_11pymarketcap_4core_Pymarketcap *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_11pymarketcap_4core_11Pymarketcap_6graphs_4__del__(struct __pyx_obj_11pymarketcap_4core_Pymarketcap *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_11pymarketcap_4core_11Pymarketcap_5debug___get__(struct __pyx_obj_11pymarketcap_4core_Pymarketcap *__pyx_v_self); /* proto */
static int __pyx_pf_11pymarketcap_4core_11Pymarketcap_5debug_2__set__(struct __pyx_obj_11pymarketcap_4core_Pymarketcap *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static PyObject *__pyx_pf_11pymarketcap_4core_11Pymarketcap_46__reduce_cython__(struct __pyx_obj_11pymarketcap_4core_Pymarketcap *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_11pymarketcap_4core_11Pymarketcap_48__setstate_cython__(struct __pyx_obj_11pymarketcap_4core_Pymarketcap *__pyx_v_self, PyObject *__pyx_v___pyx_state); /* proto */
static PyObject *__pyx_pf_11pymarketcap_4core___pyx_unpickle_Pymarketcap(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state); /* proto */
static PyObject *__pyx_tp_new_11pymarketcap_4core_Pymarketcap(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static __Pyx_CachedCFunction __pyx_umethod_PyDict_Type_keys = {0, &__pyx_n_s_keys, 0, 0, 0};
static __Pyx_CachedCFunction __pyx_umethod_PyDict_Type_update = {0, &__pyx_n_s_update, 0, 0, 0};
static PyObject *__pyx_int_0;
static PyObject *__pyx_int_1;
static PyObject *__pyx_int_8;
static PyObject *__pyx_int_16;
static PyObject *__pyx_int_18;
static PyObject *__pyx_int_32;
static PyObject *__pyx_int_64;
static PyObject *__pyx_int_128;
static PyObject *__pyx_int_200;
static PyObject *__pyx_int_403;
static PyObject *__pyx_int_600;
static PyObject *__pyx_int_2008;
static PyObject *__pyx_int_20000;
static PyObject *__pyx_int_40000;
static PyObject *__pyx_int_45000;
static PyObject *__pyx_int_50000;
static PyObject *__pyx_int_131570748;
static PyObject *__pyx_int_neg_2000;
static PyObject *__pyx_int_neg_10000;
static PyObject *__pyx_k_;
static PyObject *__pyx_k__9;
static PyObject *__pyx_k__10;
static PyObject *__pyx_slice__5;
static PyObject *__pyx_slice__6;
static PyObject *__pyx_slice__7;
static PyObject *__pyx_slice__8;
static PyObject *__pyx_tuple__4;
static PyObject *__pyx_slice__12;
static PyObject *__pyx_slice__13;
static PyObject *__pyx_slice__14;
static PyObject *__pyx_slice__15;
static PyObject *__pyx_tuple__16;
static PyObject *__pyx_tuple__17;
static PyObject *__pyx_tuple__18;
static PyObject *__pyx_codeobj__19;
/* Late includes */

/* "pymarketcap/core.pyx":64
 *     cdef public bint debug
 * 
 *     def __init__(self, timeout=DEFAULT_TIMEOUT, debug=False):             # <<<<<<<<<<<<<<
 *         self.timeout = timeout
 *         self.debug = debug
 */

/* Python wrapper */
static int __pyx_pw_11pymarketcap_4core_11Pymarketcap_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_11pymarketcap_4core_11Pymarketcap_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_timeout = 0;
  PyObject *__pyx_v_debug = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__init__ (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_timeout,&__pyx_n_s_debug,0};
    PyObject* values[2] = {0,0};
    values[0] = __pyx_k_;
    values[1] = ((PyObject *)Py_False);
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_timeout);
          if (value) { values[0] = value; kw_args--; }
        }
        CYTHON_FALLTHROUGH;
        case  1:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_debug);
          if (value) { values[1] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__init__") < 0)) __PYX_ERR(0, 64, __pyx_L3_error)
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_timeout = values[0];
    __pyx_v_debug = values[1];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__init__", 0, 0, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 64, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("pymarketcap.core.Pymarketcap.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_11pymarketcap_4core_11Pymarketcap___init__(((struct __pyx_obj_11pymarketcap_4core_Pymarketcap *)__pyx_v_self), __pyx_v_timeout, __pyx_v_debug);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_11pymarketcap_4core_11Pymarketcap___init__(struct __pyx_obj_11pymarketcap_4core_Pymarketcap *__pyx_v_self, PyObject *__pyx_v_timeout, PyObject *__pyx_v_debug) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  long __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  __Pyx_RefNannySetupContext("__init__", 0);

  /* "pymarketcap/core.pyx":65
 * 
 *     def __init__(self, timeout=DEFAULT_TIMEOUT, debug=False):
 *         self.timeout = timeout             # <<<<<<<<<<<<<<
 *         self.debug = debug
 * 
 */
  __pyx_t_1 = __Pyx_PyInt_As_long(__pyx_v_timeout); if (unlikely((__pyx_t_1 == (long)-1) && PyErr_Occurred())) __PYX_ERR(0, 65, __pyx_L1_error)
  __pyx_v_self->timeout = __pyx_t_1;

  /* "pymarketcap/core.pyx":66
 *     def __init__(self, timeout=DEFAULT_TIMEOUT, debug=False):
 *         self.timeout = timeout
 *         self.debug = debug             # <<<<<<<<<<<<<<
 * 
 *         #: object: Initialization of graphs internal interface
 */
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_v_debug); if (unlikely((__pyx_t_2 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 66, __pyx_L1_error)
  __pyx_v_self->debug = __pyx_t_2;

  /* "pymarketcap/core.pyx":69
 * 
 *         #: object: Initialization of graphs internal interface
 *         self.graphs = type("Graphs", (), self._graphs_interface)             # <<<<<<<<<<<<<<
 * 
 *     # ====================================================================
 */
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_graphs_interface); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 69, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = PyTuple_New(3); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 69, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_INCREF(__pyx_n_s_Graphs);
  __Pyx_GIVEREF(__pyx_n_s_Graphs);
  PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_n_s_Graphs);
  __Pyx_INCREF(__pyx_empty_tuple);
  __Pyx_GIVEREF(__pyx_empty_tuple);
  PyTuple_SET_ITEM(__pyx_t_4, 1, __pyx_empty_tuple);
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_4, 2, __pyx_t_3);
  __pyx_t_3 = 0;
  __pyx_t_3 = __Pyx_PyObject_Call(((PyObject *)(&PyType_Type)), __pyx_t_4, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 69, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_GIVEREF(__pyx_t_3);
  __Pyx_GOTREF(__pyx_v_self->graphs);
  __Pyx_DECREF(__pyx_v_self->graphs);
  __pyx_v_self->graphs = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "pymarketcap/core.pyx":64
 *     cdef public bint debug
 * 
 *     def __init__(self, timeout=DEFAULT_TIMEOUT, debug=False):             # <<<<<<<<<<<<<<
 *         self.timeout = timeout
 *         self.debug = debug
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("pymarketcap.core.Pymarketcap.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pymarketcap/core.pyx":76
 * 
 *     @property
 *     def _graphs_interface(self):             # <<<<<<<<<<<<<<
 *         return {
 *             "currency": self._currency,
 */

/* Python wrapper */
static PyObject *__pyx_pw_11pymarketcap_4core_11Pymarketcap_17_graphs_interface_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_11pymarketcap_4core_11Pymarketcap_17_graphs_interface_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_11pymarketcap_4core_11Pymarketcap_17_graphs_interface___get__(((struct __pyx_obj_11pymarketcap_4core_Pymarketcap *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_11pymarketcap_4core_11Pymarketcap_17_graphs_interface___get__(struct __pyx_obj_11pymarketcap_4core_Pymarketcap *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "pymarketcap/core.pyx":77
 *     @property
 *     def _graphs_interface(self):
 *         return {             # <<<<<<<<<<<<<<
 *             "currency": self._currency,
 *             "global_cap": self._global_cap,
 */
  __Pyx_XDECREF(__pyx_r);

  /* "pymarketcap/core.pyx":78
 *     def _graphs_interface(self):
 *         return {
 *             "currency": self._currency,             # <<<<<<<<<<<<<<
 *             "global_cap": self._global_cap,
 *             "dominance": self._dominance
 */
  __pyx_t_1 = __Pyx_PyDict_NewPresized(3); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 78, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_currency_2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 78, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_currency, __pyx_t_2) < 0) __PYX_ERR(0, 78, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "pymarketcap/core.pyx":79
 *         return {
 *             "currency": self._currency,
 *             "global_cap": self._global_cap,             # <<<<<<<<<<<<<<
 *             "dominance": self._dominance
 *         }
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_global_cap_2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 79, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_global_cap, __pyx_t_2) < 0) __PYX_ERR(0, 78, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "pymarketcap/core.pyx":80
 *             "currency": self._currency,
 *             "global_cap": self._global_cap,
 *             "dominance": self._dominance             # <<<<<<<<<<<<<<
 *         }
 * 
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_dominance_2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 80, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_dominance, __pyx_t_2) < 0) __PYX_ERR(0, 78, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "pymarketcap/core.pyx":76
 * 
 *     @property
 *     def _graphs_interface(self):             # <<<<<<<<<<<<<<
 *         return {
 *             "currency": self._currency,
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("pymarketcap.core.Pymarketcap._graphs_interface.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pymarketcap/core.pyx":84
 * 
 *     @property
 *     def correspondences(self):             # <<<<<<<<<<<<<<
 *         res = self._correspondences
 *         if res:
 */

/* Python wrapper */
static PyObject *__pyx_pw_11pymarketcap_4core_11Pymarketcap_15correspondences_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_11pymarketcap_4core_11Pymarketcap_15correspondences_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_11pymarketcap_4core_11Pymarketcap_15correspondences___get__(((struct __pyx_obj_11pymarketcap_4core_Pymarketcap *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_11pymarketcap_4core_11Pymarketcap_15correspondences___get__(struct __pyx_obj_11pymarketcap_4core_Pymarketcap *__pyx_v_self) {
  PyObject *__pyx_v_res = NULL;
  PyObject *__pyx_v_main_cache = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "pymarketcap/core.pyx":85
 *     @property
 *     def correspondences(self):
 *         res = self._correspondences             # <<<<<<<<<<<<<<
 *         if res:
 *             return res
 */
  __pyx_t_1 = __pyx_v_self->_correspondences;
  __Pyx_INCREF(__pyx_t_1);
  __pyx_v_res = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "pymarketcap/core.pyx":86
 *     def correspondences(self):
 *         res = self._correspondences
 *         if res:             # <<<<<<<<<<<<<<
 *             return res
 *         else:
 */
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_v_res); if (unlikely(__pyx_t_2 < 0)) __PYX_ERR(0, 86, __pyx_L1_error)
  if (__pyx_t_2) {

    /* "pymarketcap/core.pyx":87
 *         res = self._correspondences
 *         if res:
 *             return res             # <<<<<<<<<<<<<<
 *         else:
 *             main_cache = self._cache_symbols_ids()
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_v_res);
    __pyx_r = __pyx_v_res;
    goto __pyx_L0;

    /* "pymarketcap/core.pyx":86
 *     def correspondences(self):
 *         res = self._correspondences
 *         if res:             # <<<<<<<<<<<<<<
 *             return res
 *         else:
 */
  }

  /* "pymarketcap/core.pyx":89
 *             return res
 *         else:
 *             main_cache = self._cache_symbols_ids()             # <<<<<<<<<<<<<<
 *             self._correspondences = main_cache[0]
 *             self._ids_correspondences = main_cache[1]
 */
  /*else*/ {
    __pyx_t_1 = ((struct __pyx_vtabstruct_11pymarketcap_4core_Pymarketcap *)__pyx_v_self->__pyx_vtab)->_cache_symbols_ids(__pyx_v_self, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 89, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_v_main_cache = __pyx_t_1;
    __pyx_t_1 = 0;

    /* "pymarketcap/core.pyx":90
 *         else:
 *             main_cache = self._cache_symbols_ids()
 *             self._correspondences = main_cache[0]             # <<<<<<<<<<<<<<
 *             self._ids_correspondences = main_cache[1]
 *             return self._correspondences
 */
    __pyx_t_1 = __Pyx_GetItemInt(__pyx_v_main_cache, 0, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 90, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    if (!(likely(PyDict_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "dict", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(0, 90, __pyx_L1_error)
    __Pyx_GIVEREF(__pyx_t_1);
    __Pyx_GOTREF(__pyx_v_self->_correspondences);
    __Pyx_DECREF(__pyx_v_self->_correspondences);
    __pyx_v_self->_correspondences = ((PyObject*)__pyx_t_1);
    __pyx_t_1 = 0;

    /* "pymarketcap/core.pyx":91
 *             main_cache = self._cache_symbols_ids()
 *             self._correspondences = main_cache[0]
 *             self._ids_correspondences = main_cache[1]             # <<<<<<<<<<<<<<
 *             return self._correspondences
 * 
 */
    __pyx_t_1 = __Pyx_GetItemInt(__pyx_v_main_cache, 1, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 91, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    if (!(likely(PyDict_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "dict", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(0, 91, __pyx_L1_error)
    __Pyx_GIVEREF(__pyx_t_1);
    __Pyx_GOTREF(__pyx_v_self->_ids_correspondences);
    __Pyx_DECREF(__pyx_v_self->_ids_correspondences);
    __pyx_v_self->_ids_correspondences = ((PyObject*)__pyx_t_1);
    __pyx_t_1 = 0;

    /* "pymarketcap/core.pyx":92
 *             self._correspondences = main_cache[0]
 *             self._ids_correspondences = main_cache[1]
 *             return self._correspondences             # <<<<<<<<<<<<<<
 * 
 *     @property
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_v_self->_correspondences);
    __pyx_r = __pyx_v_self->_correspondences;
    goto __pyx_L0;
  }

  /* "pymarketcap/core.pyx":84
 * 
 *     @property
 *     def correspondences(self):             # <<<<<<<<<<<<<<
 *         res = self._correspondences
 *         if res:
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pymarketcap.core.Pymarketcap.correspondences.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_res);
  __Pyx_XDECREF(__pyx_v_main_cache);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pymarketcap/core.pyx":95
 * 
 *     @property
 *     def ids_correspondences(self):             # <<<<<<<<<<<<<<
 *         """Get symbols with their correspondient numeric
 *         id (used for debug purposes)."""
 */

/* Python wrapper */
static PyObject *__pyx_pw_11pymarketcap_4core_11Pymarketcap_19ids_correspondences_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_11pymarketcap_4core_11Pymarketcap_19ids_correspondences_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_11pymarketcap_4core_11Pymarketcap_19ids_correspondences___get__(((struct __pyx_obj_11pymarketcap_4core_Pymarketcap *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_11pymarketcap_4core_11Pymarketcap_19ids_correspondences___get__(struct __pyx_obj_11pymarketcap_4core_Pymarketcap *__pyx_v_self) {
  PyObject *__pyx_v_res = NULL;
  PyObject *__pyx_v_main_cache = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "pymarketcap/core.pyx":98
 *         """Get symbols with their correspondient numeric
 *         id (used for debug purposes)."""
 *         res = self._ids_correspondences             # <<<<<<<<<<<<<<
 *         if res:
 *             return res
 */
  __pyx_t_1 = __pyx_v_self->_ids_correspondences;
  __Pyx_INCREF(__pyx_t_1);
  __pyx_v_res = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "pymarketcap/core.pyx":99
 *         id (used for debug purposes)."""
 *         res = self._ids_correspondences
 *         if res:             # <<<<<<<<<<<<<<
 *             return res
 *         else:
 */
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_v_res); if (unlikely(__pyx_t_2 < 0)) __PYX_ERR(0, 99, __pyx_L1_error)
  if (__pyx_t_2) {

    /* "pymarketcap/core.pyx":100
 *         res = self._ids_correspondences
 *         if res:
 *             return res             # <<<<<<<<<<<<<<
 *         else:
 *             main_cache = self._cache_symbols_ids()
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_v_res);
    __pyx_r = __pyx_v_res;
    goto __pyx_L0;

    /* "pymarketcap/core.pyx":99
 *         id (used for debug purposes)."""
 *         res = self._ids_correspondences
 *         if res:             # <<<<<<<<<<<<<<
 *             return res
 *         else:
 */
  }

  /* "pymarketcap/core.pyx":102
 *             return res
 *         else:
 *             main_cache = self._cache_symbols_ids()             # <<<<<<<<<<<<<<
 *             self._correspondences = main_cache[0]
 *             self._ids_correspondences = main_cache[1]
 */
  /*else*/ {
    __pyx_t_1 = ((struct __pyx_vtabstruct_11pymarketcap_4core_Pymarketcap *)__pyx_v_self->__pyx_vtab)->_cache_symbols_ids(__pyx_v_self, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 102, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_v_main_cache = __pyx_t_1;
    __pyx_t_1 = 0;

    /* "pymarketcap/core.pyx":103
 *         else:
 *             main_cache = self._cache_symbols_ids()
 *             self._correspondences = main_cache[0]             # <<<<<<<<<<<<<<
 *             self._ids_correspondences = main_cache[1]
 *             return self._ids_correspondences
 */
    __pyx_t_1 = __Pyx_GetItemInt(__pyx_v_main_cache, 0, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 103, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    if (!(likely(PyDict_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "dict", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(0, 103, __pyx_L1_error)
    __Pyx_GIVEREF(__pyx_t_1);
    __Pyx_GOTREF(__pyx_v_self->_correspondences);
    __Pyx_DECREF(__pyx_v_self->_correspondences);
    __pyx_v_self->_correspondences = ((PyObject*)__pyx_t_1);
    __pyx_t_1 = 0;

    /* "pymarketcap/core.pyx":104
 *             main_cache = self._cache_symbols_ids()
 *             self._correspondences = main_cache[0]
 *             self._ids_correspondences = main_cache[1]             # <<<<<<<<<<<<<<
 *             return self._ids_correspondences
 * 
 */
    __pyx_t_1 = __Pyx_GetItemInt(__pyx_v_main_cache, 1, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 104, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    if (!(likely(PyDict_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "dict", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(0, 104, __pyx_L1_error)
    __Pyx_GIVEREF(__pyx_t_1);
    __Pyx_GOTREF(__pyx_v_self->_ids_correspondences);
    __Pyx_DECREF(__pyx_v_self->_ids_correspondences);
    __pyx_v_self->_ids_correspondences = ((PyObject*)__pyx_t_1);
    __pyx_t_1 = 0;

    /* "pymarketcap/core.pyx":105
 *             self._correspondences = main_cache[0]
 *             self._ids_correspondences = main_cache[1]
 *             return self._ids_correspondences             # <<<<<<<<<<<<<<
 * 
 *     cpdef _is_symbol(self, unicode currency):
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_v_self->_ids_correspondences);
    __pyx_r = __pyx_v_self->_ids_correspondences;
    goto __pyx_L0;
  }

  /* "pymarketcap/core.pyx":95
 * 
 *     @property
 *     def ids_correspondences(self):             # <<<<<<<<<<<<<<
 *         """Get symbols with their correspondient numeric
 *         id (used for debug purposes)."""
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pymarketcap.core.Pymarketcap.ids_correspondences.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_res);
  __Pyx_XDECREF(__pyx_v_main_cache);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pymarketcap/core.pyx":107
 *             return self._ids_correspondences
 * 
 *     cpdef _is_symbol(self, unicode currency):             # <<<<<<<<<<<<<<
 *         """Internal function for check
 *         if a currency string may be a symbol or not.
 */

static PyObject *__pyx_pw_11pymarketcap_4core_11Pymarketcap_3_is_symbol(PyObject *__pyx_v_self, PyObject *__pyx_v_currency); /*proto*/
static PyObject *__pyx_f_11pymarketcap_4core_11Pymarketcap__is_symbol(struct __pyx_obj_11pymarketcap_4core_Pymarketcap *__pyx_v_self, PyObject *__pyx_v_currency, int __pyx_skip_dispatch) {
  int __pyx_v_response;
  PyObject *__pyx_v_msg = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_t_6;
  int __pyx_t_7;
  int __pyx_t_8;
  PyObject *__pyx_t_9 = NULL;
  PyObject *__pyx_t_10 = NULL;
  PyObject *__pyx_t_11 = NULL;
  int __pyx_t_12;
  __Pyx_RefNannySetupContext("_is_symbol", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0)) {
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_is_symbol); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 107, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)__pyx_pw_11pymarketcap_4core_11Pymarketcap_3_is_symbol)) {
      __Pyx_XDECREF(__pyx_r);
      __Pyx_INCREF(__pyx_t_1);
      __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
      if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
        __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
        if (likely(__pyx_t_4)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
          __Pyx_INCREF(__pyx_t_4);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_3, function);
        }
      }
      if (!__pyx_t_4) {
        __pyx_t_2 = __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_v_currency); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 107, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
      } else {
        #if CYTHON_FAST_PYCALL
        if (PyFunction_Check(__pyx_t_3)) {
          PyObject *__pyx_temp[2] = {__pyx_t_4, __pyx_v_currency};
          __pyx_t_2 = __Pyx_PyFunction_FastCall(__pyx_t_3, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 107, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
          __Pyx_GOTREF(__pyx_t_2);
        } else
        #endif
        #if CYTHON_FAST_PYCCALL
        if (__Pyx_PyFastCFunction_Check(__pyx_t_3)) {
          PyObject *__pyx_temp[2] = {__pyx_t_4, __pyx_v_currency};
          __pyx_t_2 = __Pyx_PyCFunction_FastCall(__pyx_t_3, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 107, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
          __Pyx_GOTREF(__pyx_t_2);
        } else
        #endif
        {
          __pyx_t_5 = PyTuple_New(1+1); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 107, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_5);
          __Pyx_GIVEREF(__pyx_t_4); PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_4); __pyx_t_4 = NULL;
          __Pyx_INCREF(__pyx_v_currency);
          __Pyx_GIVEREF(__pyx_v_currency);
          PyTuple_SET_ITEM(__pyx_t_5, 0+1, __pyx_v_currency);
          __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_t_5, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 107, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_2);
          __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
        }
      }
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __pyx_r = __pyx_t_2;
      __pyx_t_2 = 0;
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      goto __pyx_L0;
    }
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  }

  /* "pymarketcap/core.pyx":119
 *         """
 *         cdef bint response
 *         response = 0             # <<<<<<<<<<<<<<
 *         if currency.isupper() or currency in EXCEPTIONAL_COIN_SLUGS:
 *             try:
 */
  __pyx_v_response = 0;

  /* "pymarketcap/core.pyx":120
 *         cdef bint response
 *         response = 0
 *         if currency.isupper() or currency in EXCEPTIONAL_COIN_SLUGS:             # <<<<<<<<<<<<<<
 *             try:
 *                 if currency in self.__repeated_symbols:
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_currency, __pyx_n_s_isupper); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 120, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  if (__pyx_t_3) {
    __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_3); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 120, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  } else {
    __pyx_t_1 = __Pyx_PyObject_CallNoArg(__pyx_t_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 120, __pyx_L1_error)
  }
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_7 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_7 < 0)) __PYX_ERR(0, 120, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (!__pyx_t_7) {
  } else {
    __pyx_t_6 = __pyx_t_7;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_1 = __Pyx_GetModuleGlobalName(__pyx_n_s_EXCEPTIONAL_COIN_SLUGS); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 120, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_7 = (__Pyx_PySequence_ContainsTF(__pyx_v_currency, __pyx_t_1, Py_EQ)); if (unlikely(__pyx_t_7 < 0)) __PYX_ERR(0, 120, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_8 = (__pyx_t_7 != 0);
  __pyx_t_6 = __pyx_t_8;
  __pyx_L4_bool_binop_done:;
  if (__pyx_t_6) {

    /* "pymarketcap/core.pyx":121
 *         response = 0
 *         if currency.isupper() or currency in EXCEPTIONAL_COIN_SLUGS:
 *             try:             # <<<<<<<<<<<<<<
 *                 if currency in self.__repeated_symbols:
 *                     msg = 'The symbol "%s" has more than one correspondence ' % currency \
 */
    {
      __Pyx_PyThreadState_declare
      __Pyx_PyThreadState_assign
      __Pyx_ExceptionSave(&__pyx_t_9, &__pyx_t_10, &__pyx_t_11);
      __Pyx_XGOTREF(__pyx_t_9);
      __Pyx_XGOTREF(__pyx_t_10);
      __Pyx_XGOTREF(__pyx_t_11);
      /*try:*/ {

        /* "pymarketcap/core.pyx":122
 *         if currency.isupper() or currency in EXCEPTIONAL_COIN_SLUGS:
 *             try:
 *                 if currency in self.__repeated_symbols:             # <<<<<<<<<<<<<<
 *                     msg = 'The symbol "%s" has more than one correspondence ' % currency \
 *                         + "with coin slugs in Coinmarketcap. Please get this currency as slug. " \
 */
        if (unlikely(__pyx_v_self->__repeated_symbols == Py_None)) {
          PyErr_SetString(PyExc_TypeError, "'NoneType' object is not iterable");
          __PYX_ERR(0, 122, __pyx_L6_error)
        }
        __pyx_t_6 = (__Pyx_PyDict_ContainsTF(__pyx_v_currency, __pyx_v_self->__repeated_symbols, Py_EQ)); if (unlikely(__pyx_t_6 < 0)) __PYX_ERR(0, 122, __pyx_L6_error)
        __pyx_t_8 = (__pyx_t_6 != 0);
        if (unlikely(__pyx_t_8)) {

          /* "pymarketcap/core.pyx":123
 *             try:
 *                 if currency in self.__repeated_symbols:
 *                     msg = 'The symbol "%s" has more than one correspondence ' % currency \             # <<<<<<<<<<<<<<
 *                         + "with coin slugs in Coinmarketcap. Please get this currency as slug. " \
 *                         + "\nPossible valid slug names: %r." % self.__repeated_symbols[currency]
 */
          __pyx_t_1 = __Pyx_PyString_Format(__pyx_kp_s_The_symbol_s_has_more_than_one_c, __pyx_v_currency); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 123, __pyx_L6_error)
          __Pyx_GOTREF(__pyx_t_1);

          /* "pymarketcap/core.pyx":124
 *                 if currency in self.__repeated_symbols:
 *                     msg = 'The symbol "%s" has more than one correspondence ' % currency \
 *                         + "with coin slugs in Coinmarketcap. Please get this currency as slug. " \             # <<<<<<<<<<<<<<
 *                         + "\nPossible valid slug names: %r." % self.__repeated_symbols[currency]
 *                     raise ValueError(msg)
 */
          __pyx_t_2 = __Pyx_PyUnicode_Concat(__pyx_t_1, __pyx_kp_s_with_coin_slugs_in_Coinmarketcap); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 124, __pyx_L6_error)
          __Pyx_GOTREF(__pyx_t_2);
          __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

          /* "pymarketcap/core.pyx":125
 *                     msg = 'The symbol "%s" has more than one correspondence ' % currency \
 *                         + "with coin slugs in Coinmarketcap. Please get this currency as slug. " \
 *                         + "\nPossible valid slug names: %r." % self.__repeated_symbols[currency]             # <<<<<<<<<<<<<<
 *                     raise ValueError(msg)
 *             except TypeError:
 */
          if (unlikely(__pyx_v_self->__repeated_symbols == Py_None)) {
            PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
            __PYX_ERR(0, 125, __pyx_L6_error)
          }
          __pyx_t_1 = __Pyx_PyDict_GetItem(__pyx_v_self->__repeated_symbols, __pyx_v_currency); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 125, __pyx_L6_error)
          __Pyx_GOTREF(__pyx_t_1);
          __pyx_t_3 = __Pyx_PyString_Format(__pyx_kp_s_Possible_valid_slug_names_r, __pyx_t_1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 125, __pyx_L6_error)
          __Pyx_GOTREF(__pyx_t_3);
          __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
          __pyx_t_1 = __Pyx_PyUnicode_Concat(__pyx_t_2, __pyx_t_3); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 125, __pyx_L6_error)
          __Pyx_GOTREF(__pyx_t_1);
          __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
          __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
          __pyx_v_msg = ((PyObject*)__pyx_t_1);
          __pyx_t_1 = 0;

          /* "pymarketcap/core.pyx":126
 *                         + "with coin slugs in Coinmarketcap. Please get this currency as slug. " \
 *                         + "\nPossible valid slug names: %r." % self.__repeated_symbols[currency]
 *                     raise ValueError(msg)             # <<<<<<<<<<<<<<
 *             except TypeError:
 *                 pass
 */
          __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_builtin_ValueError, __pyx_v_msg); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 126, __pyx_L6_error)
          __Pyx_GOTREF(__pyx_t_1);
          __Pyx_Raise(__pyx_t_1, 0, 0, 0);
          __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
          __PYX_ERR(0, 126, __pyx_L6_error)

          /* "pymarketcap/core.pyx":122
 *         if currency.isupper() or currency in EXCEPTIONAL_COIN_SLUGS:
 *             try:
 *                 if currency in self.__repeated_symbols:             # <<<<<<<<<<<<<<
 *                     msg = 'The symbol "%s" has more than one correspondence ' % currency \
 *                         + "with coin slugs in Coinmarketcap. Please get this currency as slug. " \
 */
        }

        /* "pymarketcap/core.pyx":121
 *         response = 0
 *         if currency.isupper() or currency in EXCEPTIONAL_COIN_SLUGS:
 *             try:             # <<<<<<<<<<<<<<
 *                 if currency in self.__repeated_symbols:
 *                     msg = 'The symbol "%s" has more than one correspondence ' % currency \
 */
      }
      __Pyx_XDECREF(__pyx_t_9); __pyx_t_9 = 0;
      __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
      __Pyx_XDECREF(__pyx_t_11); __pyx_t_11 = 0;
      goto __pyx_L11_try_end;
      __pyx_L6_error:;
      __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
      __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;

      /* "pymarketcap/core.pyx":127
 *                         + "\nPossible valid slug names: %r." % self.__repeated_symbols[currency]
 *                     raise ValueError(msg)
 *             except TypeError:             # <<<<<<<<<<<<<<
 *                 pass
 *             response = 1
 */
      __pyx_t_12 = __Pyx_PyErr_ExceptionMatches(__pyx_builtin_TypeError);
      if (__pyx_t_12) {
        __Pyx_ErrRestore(0,0,0);
        goto __pyx_L7_exception_handled;
      }
      goto __pyx_L8_except_error;
      __pyx_L8_except_error:;

      /* "pymarketcap/core.pyx":121
 *         response = 0
 *         if currency.isupper() or currency in EXCEPTIONAL_COIN_SLUGS:
 *             try:             # <<<<<<<<<<<<<<
 *                 if currency in self.__repeated_symbols:
 *                     msg = 'The symbol "%s" has more than one correspondence ' % currency \
 */
      __Pyx_XGIVEREF(__pyx_t_9);
      __Pyx_XGIVEREF(__pyx_t_10);
      __Pyx_XGIVEREF(__pyx_t_11);
      __Pyx_ExceptionReset(__pyx_t_9, __pyx_t_10, __pyx_t_11);
      goto __pyx_L1_error;
      __pyx_L7_exception_handled:;
      __Pyx_XGIVEREF(__pyx_t_9);
      __Pyx_XGIVEREF(__pyx_t_10);
      __Pyx_XGIVEREF(__pyx_t_11);
      __Pyx_ExceptionReset(__pyx_t_9, __pyx_t_10, __pyx_t_11);
      __pyx_L11_try_end:;
    }

    /* "pymarketcap/core.pyx":129
 *             except TypeError:
 *                 pass
 *             response = 1             # <<<<<<<<<<<<<<
 *         return response
 * 
 */
    __pyx_v_response = 1;

    /* "pymarketcap/core.pyx":120
 *         cdef bint response
 *         response = 0
 *         if currency.isupper() or currency in EXCEPTIONAL_COIN_SLUGS:             # <<<<<<<<<<<<<<
 *             try:
 *                 if currency in self.__repeated_symbols:
 */
  }

  /* "pymarketcap/core.pyx":130
 *                 pass
 *             response = 1
 *         return response             # <<<<<<<<<<<<<<
 * 
 *     cpdef _quick_search(self, exchanges=False):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyBool_FromLong(__pyx_v_response); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 130, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "pymarketcap/core.pyx":107
 *             return self._ids_correspondences
 * 
 *     cpdef _is_symbol(self, unicode currency):             # <<<<<<<<<<<<<<
 *         """Internal function for check
 *         if a currency string may be a symbol or not.
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("pymarketcap.core.Pymarketcap._is_symbol", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_msg);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_11pymarketcap_4core_11Pymarketcap_3_is_symbol(PyObject *__pyx_v_self, PyObject *__pyx_v_currency); /*proto*/
static char __pyx_doc_11pymarketcap_4core_11Pymarketcap_2_is_symbol[] = "Internal function for check\n        if a currency string may be a symbol or not.\n        This function is not strict, so if you pass\n        self._is_symbol(\"OBASDFPAFFFOUASVBF\") will returns True\n        but in this context we don't need to check if\n        the user has introduced a valid symbol.\n\n        Returns bint:\n            1 if True, 0 if False\n        ";
static PyObject *__pyx_pw_11pymarketcap_4core_11Pymarketcap_3_is_symbol(PyObject *__pyx_v_self, PyObject *__pyx_v_currency) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_is_symbol (wrapper)", 0);
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_currency), (&PyUnicode_Type), 1, "currency", 1))) __PYX_ERR(0, 107, __pyx_L1_error)
  __pyx_r = __pyx_pf_11pymarketcap_4core_11Pymarketcap_2_is_symbol(((struct __pyx_obj_11pymarketcap_4core_Pymarketcap *)__pyx_v_self), ((PyObject*)__pyx_v_currency));

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_11pymarketcap_4core_11Pymarketcap_2_is_symbol(struct __pyx_obj_11pymarketcap_4core_Pymarketcap *__pyx_v_self, PyObject *__pyx_v_currency) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("_is_symbol", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_11pymarketcap_4core_11Pymarketcap__is_symbol(__pyx_v_self, __pyx_v_currency, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 107, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pymarketcap.core.Pymarketcap._is_symbol", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pymarketcap/core.pyx":132
 *         return response
 * 
 *     cpdef _quick_search(self, exchanges=False):             # <<<<<<<<<<<<<<
 *         """Internal pymarketcap JSON cache files
 *         ``https://s2.coinmarketcap.com/generated/search/quick_search.json``
 */

static PyObject *__pyx_pw_11pymarketcap_4core_11Pymarketcap_5_quick_search(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyObject *__pyx_f_11pymarketcap_4core_11Pymarketcap__quick_search(struct __pyx_obj_11pymarketcap_4core_Pymarketcap *__pyx_v_self, int __pyx_skip_dispatch, struct __pyx_opt_args_11pymarketcap_4core_11Pymarketcap__quick_search *__pyx_optional_args) {
  PyObject *__pyx_v_exchanges = ((PyObject *)Py_False);
  PyObject *__pyx_v_url = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_t_6;
  char *__pyx_t_7;
  __Pyx_RefNannySetupContext("_quick_search", 0);
  if (__pyx_optional_args) {
    if (__pyx_optional_args->__pyx_n > 0) {
      __pyx_v_exchanges = __pyx_optional_args->exchanges;
    }
  }
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0)) {
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_quick_search); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 132, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)__pyx_pw_11pymarketcap_4core_11Pymarketcap_5_quick_search)) {
      __Pyx_XDECREF(__pyx_r);
      __Pyx_INCREF(__pyx_t_1);
      __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
      if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
        __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
        if (likely(__pyx_t_4)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
          __Pyx_INCREF(__pyx_t_4);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_3, function);
        }
      }
      if (!__pyx_t_4) {
        __pyx_t_2 = __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_v_exchanges); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 132, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
      } else {
        #if CYTHON_FAST_PYCALL
        if (PyFunction_Check(__pyx_t_3)) {
          PyObject *__pyx_temp[2] = {__pyx_t_4, __pyx_v_exchanges};
          __pyx_t_2 = __Pyx_PyFunction_FastCall(__pyx_t_3, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 132, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
          __Pyx_GOTREF(__pyx_t_2);
        } else
        #endif
        #if CYTHON_FAST_PYCCALL
        if (__Pyx_PyFastCFunction_Check(__pyx_t_3)) {
          PyObject *__pyx_temp[2] = {__pyx_t_4, __pyx_v_exchanges};
          __pyx_t_2 = __Pyx_PyCFunction_FastCall(__pyx_t_3, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 132, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
          __Pyx_GOTREF(__pyx_t_2);
        } else
        #endif
        {
          __pyx_t_5 = PyTuple_New(1+1); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 132, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_5);
          __Pyx_GIVEREF(__pyx_t_4); PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_4); __pyx_t_4 = NULL;
          __Pyx_INCREF(__pyx_v_exchanges);
          __Pyx_GIVEREF(__pyx_v_exchanges);
          PyTuple_SET_ITEM(__pyx_t_5, 0+1, __pyx_v_exchanges);
          __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_t_5, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 132, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_2);
          __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
        }
      }
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __pyx_r = __pyx_t_2;
      __pyx_t_2 = 0;
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      goto __pyx_L0;
    }
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  }

  /* "pymarketcap/core.pyx":138
 *         """
 *         cdef bytes url
 *         if exchanges:             # <<<<<<<<<<<<<<
 *             url = b"https://s2.coinmarketcap.com/generated/search/quick_search_exchanges.json"
 *         else:
 */
  __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_v_exchanges); if (unlikely(__pyx_t_6 < 0)) __PYX_ERR(0, 138, __pyx_L1_error)
  if (__pyx_t_6) {

    /* "pymarketcap/core.pyx":139
 *         cdef bytes url
 *         if exchanges:
 *             url = b"https://s2.coinmarketcap.com/generated/search/quick_search_exchanges.json"             # <<<<<<<<<<<<<<
 *         else:
 *             url = b"https://s2.coinmarketcap.com/generated/search/quick_search.json"
 */
    __Pyx_INCREF(__pyx_kp_b_https_s2_coinmarketcap_com_gener);
    __pyx_v_url = __pyx_kp_b_https_s2_coinmarketcap_com_gener;

    /* "pymarketcap/core.pyx":138
 *         """
 *         cdef bytes url
 *         if exchanges:             # <<<<<<<<<<<<<<
 *             url = b"https://s2.coinmarketcap.com/generated/search/quick_search_exchanges.json"
 *         else:
 */
    goto __pyx_L3;
  }

  /* "pymarketcap/core.pyx":141
 *             url = b"https://s2.coinmarketcap.com/generated/search/quick_search_exchanges.json"
 *         else:
 *             url = b"https://s2.coinmarketcap.com/generated/search/quick_search.json"             # <<<<<<<<<<<<<<
 *         return loads(self._get(url))
 * 
 */
  /*else*/ {
    __Pyx_INCREF(__pyx_kp_b_https_s2_coinmarketcap_com_gener_2);
    __pyx_v_url = __pyx_kp_b_https_s2_coinmarketcap_com_gener_2;
  }
  __pyx_L3:;

  /* "pymarketcap/core.pyx":142
 *         else:
 *             url = b"https://s2.coinmarketcap.com/generated/search/quick_search.json"
 *         return loads(self._get(url))             # <<<<<<<<<<<<<<
 * 
 *     cpdef _cache_symbols_ids(self):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_loads); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 142, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_7 = __Pyx_PyBytes_AsWritableString(__pyx_v_url); if (unlikely((!__pyx_t_7) && PyErr_Occurred())) __PYX_ERR(0, 142, __pyx_L1_error)
  __pyx_t_3 = ((struct __pyx_vtabstruct_11pymarketcap_4core_Pymarketcap *)__pyx_v_self->__pyx_vtab)->_get(__pyx_v_self, __pyx_t_7); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 142, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_5 = NULL;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_5)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_5);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  if (!__pyx_t_5) {
    __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_3); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 142, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_GOTREF(__pyx_t_1);
  } else {
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(__pyx_t_2)) {
      PyObject *__pyx_temp[2] = {__pyx_t_5, __pyx_t_3};
      __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 142, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    } else
    #endif
    #if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
      PyObject *__pyx_temp[2] = {__pyx_t_5, __pyx_t_3};
      __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 142, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    } else
    #endif
    {
      __pyx_t_4 = PyTuple_New(1+1); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 142, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_GIVEREF(__pyx_t_5); PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_5); __pyx_t_5 = NULL;
      __Pyx_GIVEREF(__pyx_t_3);
      PyTuple_SET_ITEM(__pyx_t_4, 0+1, __pyx_t_3);
      __pyx_t_3 = 0;
      __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_4, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 142, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    }
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "pymarketcap/core.pyx":132
 *         return response
 * 
 *     cpdef _quick_search(self, exchanges=False):             # <<<<<<<<<<<<<<
 *         """Internal pymarketcap JSON cache files
 *         ``https://s2.coinmarketcap.com/generated/search/quick_search.json``
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("pymarketcap.core.Pymarketcap._quick_search", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_url);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_11pymarketcap_4core_11Pymarketcap_5_quick_search(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_11pymarketcap_4core_11Pymarketcap_4_quick_search[] = "Internal pymarketcap JSON cache files\n        ``https://s2.coinmarketcap.com/generated/search/quick_search.json``\n        ``https://s2.coinmarketcap.com/generated/search/quick_search_exchanges.json``\n        ";
static PyObject *__pyx_pw_11pymarketcap_4core_11Pymarketcap_5_quick_search(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_exchanges = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_quick_search (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_exchanges,0};
    PyObject* values[1] = {0};
    values[0] = ((PyObject *)Py_False);
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_exchanges);
          if (value) { values[0] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "_quick_search") < 0)) __PYX_ERR(0, 132, __pyx_L3_error)
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_exchanges = values[0];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("_quick_search", 0, 0, 1, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 132, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("pymarketcap.core.Pymarketcap._quick_search", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_11pymarketcap_4core_11Pymarketcap_4_quick_search(((struct __pyx_obj_11pymarketcap_4core_Pymarketcap *)__pyx_v_self), __pyx_v_exchanges);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_11pymarketcap_4core_11Pymarketcap_4_quick_search(struct __pyx_obj_11pymarketcap_4core_Pymarketcap *__pyx_v_self, PyObject *__pyx_v_exchanges) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  struct __pyx_opt_args_11pymarketcap_4core_11Pymarketcap__quick_search __pyx_t_2;
  __Pyx_RefNannySetupContext("_quick_search", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2.__pyx_n = 1;
  __pyx_t_2.exchanges = __pyx_v_exchanges;
  __pyx_t_1 = __pyx_vtabptr_11pymarketcap_4core_Pymarketcap->_quick_search(__pyx_v_self, 1, &__pyx_t_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 132, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pymarketcap.core.Pymarketcap._quick_search", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pymarketcap/core.pyx":144
 *         return loads(self._get(url))
 * 
 *     cpdef _cache_symbols_ids(self):             # <<<<<<<<<<<<<<
 *         """Internal function for load in cache all symbols
 *         in coinmarketcap with their respectives currency names."""
 */

static PyObject *__pyx_pw_11pymarketcap_4core_11Pymarketcap_7_cache_symbols_ids(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_f_11pymarketcap_4core_11Pymarketcap__cache_symbols_ids(struct __pyx_obj_11pymarketcap_4core_Pymarketcap *__pyx_v_self, int __pyx_skip_dispatch) {
  PyObject *__pyx_v_symbols_slugs = NULL;
  PyObject *__pyx_v_symbols_ids = NULL;
  PyObject *__pyx_v_currency = NULL;
  PyObject *__pyx_v_symbol = NULL;
  PyObject *__pyx_v_slug = NULL;
  PyObject *__pyx_v_original = NULL;
  PyObject *__pyx_v_correct = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  Py_ssize_t __pyx_t_5;
  PyObject *(*__pyx_t_6)(PyObject *);
  int __pyx_t_7;
  int __pyx_t_8;
  int __pyx_t_9;
  PyObject *__pyx_t_10 = NULL;
  PyObject *(*__pyx_t_11)(PyObject *);
  __Pyx_RefNannySetupContext("_cache_symbols_ids", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0)) {
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_cache_symbols_ids); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 144, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)__pyx_pw_11pymarketcap_4core_11Pymarketcap_7_cache_symbols_ids)) {
      __Pyx_XDECREF(__pyx_r);
      __Pyx_INCREF(__pyx_t_1);
      __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
      if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
        __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
        if (likely(__pyx_t_4)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
          __Pyx_INCREF(__pyx_t_4);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_3, function);
        }
      }
      if (__pyx_t_4) {
        __pyx_t_2 = __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_4); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 144, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      } else {
        __pyx_t_2 = __Pyx_PyObject_CallNoArg(__pyx_t_3); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 144, __pyx_L1_error)
      }
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __pyx_r = __pyx_t_2;
      __pyx_t_2 = 0;
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      goto __pyx_L0;
    }
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  }

  /* "pymarketcap/core.pyx":147
 *         """Internal function for load in cache all symbols
 *         in coinmarketcap with their respectives currency names."""
 *         self.__repeated_symbols = {}             # <<<<<<<<<<<<<<
 *         symbols_slugs, symbols_ids = {}, {}
 *         for currency in self._quick_search():
 */
  __pyx_t_1 = __Pyx_PyDict_NewPresized(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 147, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->__repeated_symbols);
  __Pyx_DECREF(__pyx_v_self->__repeated_symbols);
  __pyx_v_self->__repeated_symbols = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "pymarketcap/core.pyx":148
 *         in coinmarketcap with their respectives currency names."""
 *         self.__repeated_symbols = {}
 *         symbols_slugs, symbols_ids = {}, {}             # <<<<<<<<<<<<<<
 *         for currency in self._quick_search():
 *             symbol = currency["symbol"]
 */
  __pyx_t_1 = __Pyx_PyDict_NewPresized(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 148, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyDict_NewPresized(0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 148, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_v_symbols_slugs = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  __pyx_v_symbols_ids = ((PyObject*)__pyx_t_2);
  __pyx_t_2 = 0;

  /* "pymarketcap/core.pyx":149
 *         self.__repeated_symbols = {}
 *         symbols_slugs, symbols_ids = {}, {}
 *         for currency in self._quick_search():             # <<<<<<<<<<<<<<
 *             symbol = currency["symbol"]
 *             slug = currency["slug"].replace(" ", "")
 */
  __pyx_t_2 = ((struct __pyx_vtabstruct_11pymarketcap_4core_Pymarketcap *)__pyx_v_self->__pyx_vtab)->_quick_search(__pyx_v_self, 0, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 149, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (likely(PyList_CheckExact(__pyx_t_2)) || PyTuple_CheckExact(__pyx_t_2)) {
    __pyx_t_1 = __pyx_t_2; __Pyx_INCREF(__pyx_t_1); __pyx_t_5 = 0;
    __pyx_t_6 = NULL;
  } else {
    __pyx_t_5 = -1; __pyx_t_1 = PyObject_GetIter(__pyx_t_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 149, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_6 = Py_TYPE(__pyx_t_1)->tp_iternext; if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 149, __pyx_L1_error)
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  for (;;) {
    if (likely(!__pyx_t_6)) {
      if (likely(PyList_CheckExact(__pyx_t_1))) {
        if (__pyx_t_5 >= PyList_GET_SIZE(__pyx_t_1)) break;
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_2 = PyList_GET_ITEM(__pyx_t_1, __pyx_t_5); __Pyx_INCREF(__pyx_t_2); __pyx_t_5++; if (unlikely(0 < 0)) __PYX_ERR(0, 149, __pyx_L1_error)
        #else
        __pyx_t_2 = PySequence_ITEM(__pyx_t_1, __pyx_t_5); __pyx_t_5++; if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 149, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        #endif
      } else {
        if (__pyx_t_5 >= PyTuple_GET_SIZE(__pyx_t_1)) break;
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_2 = PyTuple_GET_ITEM(__pyx_t_1, __pyx_t_5); __Pyx_INCREF(__pyx_t_2); __pyx_t_5++; if (unlikely(0 < 0)) __PYX_ERR(0, 149, __pyx_L1_error)
        #else
        __pyx_t_2 = PySequence_ITEM(__pyx_t_1, __pyx_t_5); __pyx_t_5++; if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 149, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        #endif
      }
    } else {
      __pyx_t_2 = __pyx_t_6(__pyx_t_1);
      if (unlikely(!__pyx_t_2)) {
        PyObject* exc_type = PyErr_Occurred();
        if (exc_type) {
          if (likely(__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
          else __PYX_ERR(0, 149, __pyx_L1_error)
        }
        break;
      }
      __Pyx_GOTREF(__pyx_t_2);
    }
    __Pyx_XDECREF_SET(__pyx_v_currency, __pyx_t_2);
    __pyx_t_2 = 0;

    /* "pymarketcap/core.pyx":150
 *         symbols_slugs, symbols_ids = {}, {}
 *         for currency in self._quick_search():
 *             symbol = currency["symbol"]             # <<<<<<<<<<<<<<
 *             slug = currency["slug"].replace(" ", "")
 *             if symbol in symbols_slugs:    # Repeated symbols are stored internally
 */
    __pyx_t_2 = __Pyx_PyObject_Dict_GetItem(__pyx_v_currency, __pyx_n_s_symbol); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 150, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_XDECREF_SET(__pyx_v_symbol, __pyx_t_2);
    __pyx_t_2 = 0;

    /* "pymarketcap/core.pyx":151
 *         for currency in self._quick_search():
 *             symbol = currency["symbol"]
 *             slug = currency["slug"].replace(" ", "")             # <<<<<<<<<<<<<<
 *             if symbol in symbols_slugs:    # Repeated symbols are stored internally
 *                 if symbol in self.__repeated_symbols.keys():
 */
    __pyx_t_2 = __Pyx_PyObject_Dict_GetItem(__pyx_v_currency, __pyx_n_s_slug); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 151, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_replace); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 151, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_tuple__4, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 151, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_XDECREF_SET(__pyx_v_slug, __pyx_t_2);
    __pyx_t_2 = 0;

    /* "pymarketcap/core.pyx":152
 *             symbol = currency["symbol"]
 *             slug = currency["slug"].replace(" ", "")
 *             if symbol in symbols_slugs:    # Repeated symbols are stored internally             # <<<<<<<<<<<<<<
 *                 if symbol in self.__repeated_symbols.keys():
 *                     self.__repeated_symbols[symbol].append(slug)
 */
    __pyx_t_7 = (__Pyx_PyDict_ContainsTF(__pyx_v_symbol, __pyx_v_symbols_slugs, Py_EQ)); if (unlikely(__pyx_t_7 < 0)) __PYX_ERR(0, 152, __pyx_L1_error)
    __pyx_t_8 = (__pyx_t_7 != 0);
    if (__pyx_t_8) {

      /* "pymarketcap/core.pyx":153
 *             slug = currency["slug"].replace(" ", "")
 *             if symbol in symbols_slugs:    # Repeated symbols are stored internally
 *                 if symbol in self.__repeated_symbols.keys():             # <<<<<<<<<<<<<<
 *                     self.__repeated_symbols[symbol].append(slug)
 *                 else:
 */
      if (unlikely(__pyx_v_self->__repeated_symbols == Py_None)) {
        PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "keys");
        __PYX_ERR(0, 153, __pyx_L1_error)
      }
      __pyx_t_2 = __Pyx_PyDict_Keys(__pyx_v_self->__repeated_symbols); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 153, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __pyx_t_8 = (__Pyx_PySequence_ContainsTF(__pyx_v_symbol, __pyx_t_2, Py_EQ)); if (unlikely(__pyx_t_8 < 0)) __PYX_ERR(0, 153, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __pyx_t_7 = (__pyx_t_8 != 0);
      if (__pyx_t_7) {

        /* "pymarketcap/core.pyx":154
 *             if symbol in symbols_slugs:    # Repeated symbols are stored internally
 *                 if symbol in self.__repeated_symbols.keys():
 *                     self.__repeated_symbols[symbol].append(slug)             # <<<<<<<<<<<<<<
 *                 else:
 *                     self.__repeated_symbols[symbol] = [symbols_slugs[symbol], slug]
 */
        if (unlikely(__pyx_v_self->__repeated_symbols == Py_None)) {
          PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
          __PYX_ERR(0, 154, __pyx_L1_error)
        }
        __pyx_t_2 = __Pyx_PyDict_GetItem(__pyx_v_self->__repeated_symbols, __pyx_v_symbol); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 154, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __pyx_t_9 = __Pyx_PyObject_Append(__pyx_t_2, __pyx_v_slug); if (unlikely(__pyx_t_9 == ((int)-1))) __PYX_ERR(0, 154, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

        /* "pymarketcap/core.pyx":153
 *             slug = currency["slug"].replace(" ", "")
 *             if symbol in symbols_slugs:    # Repeated symbols are stored internally
 *                 if symbol in self.__repeated_symbols.keys():             # <<<<<<<<<<<<<<
 *                     self.__repeated_symbols[symbol].append(slug)
 *                 else:
 */
        goto __pyx_L6;
      }

      /* "pymarketcap/core.pyx":156
 *                     self.__repeated_symbols[symbol].append(slug)
 *                 else:
 *                     self.__repeated_symbols[symbol] = [symbols_slugs[symbol], slug]             # <<<<<<<<<<<<<<
 *             else:
 *                 symbols_slugs[symbol] = slug
 */
      /*else*/ {
        __pyx_t_2 = __Pyx_PyDict_GetItem(__pyx_v_symbols_slugs, __pyx_v_symbol); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 156, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __pyx_t_3 = PyList_New(2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 156, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_3);
        __Pyx_GIVEREF(__pyx_t_2);
        PyList_SET_ITEM(__pyx_t_3, 0, __pyx_t_2);
        __Pyx_INCREF(__pyx_v_slug);
        __Pyx_GIVEREF(__pyx_v_slug);
        PyList_SET_ITEM(__pyx_t_3, 1, __pyx_v_slug);
        __pyx_t_2 = 0;
        if (unlikely(__pyx_v_self->__repeated_symbols == Py_None)) {
          PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
          __PYX_ERR(0, 156, __pyx_L1_error)
        }
        if (unlikely(PyDict_SetItem(__pyx_v_self->__repeated_symbols, __pyx_v_symbol, __pyx_t_3) < 0)) __PYX_ERR(0, 156, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      }
      __pyx_L6:;

      /* "pymarketcap/core.pyx":152
 *             symbol = currency["symbol"]
 *             slug = currency["slug"].replace(" ", "")
 *             if symbol in symbols_slugs:    # Repeated symbols are stored internally             # <<<<<<<<<<<<<<
 *                 if symbol in self.__repeated_symbols.keys():
 *                     self.__repeated_symbols[symbol].append(slug)
 */
      goto __pyx_L5;
    }

    /* "pymarketcap/core.pyx":158
 *                     self.__repeated_symbols[symbol] = [symbols_slugs[symbol], slug]
 *             else:
 *                 symbols_slugs[symbol] = slug             # <<<<<<<<<<<<<<
 *             symbols_ids[symbol] = currency["id"]
 *         for original, correct in EXCEPTIONAL_COIN_SLUGS.items():
 */
    /*else*/ {
      if (unlikely(PyDict_SetItem(__pyx_v_symbols_slugs, __pyx_v_symbol, __pyx_v_slug) < 0)) __PYX_ERR(0, 158, __pyx_L1_error)
    }
    __pyx_L5:;

    /* "pymarketcap/core.pyx":159
 *             else:
 *                 symbols_slugs[symbol] = slug
 *             symbols_ids[symbol] = currency["id"]             # <<<<<<<<<<<<<<
 *         for original, correct in EXCEPTIONAL_COIN_SLUGS.items():
 *             symbols_slugs[original] = correct
 */
    __pyx_t_3 = __Pyx_PyObject_Dict_GetItem(__pyx_v_currency, __pyx_n_s_id); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 159, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    if (unlikely(PyDict_SetItem(__pyx_v_symbols_ids, __pyx_v_symbol, __pyx_t_3) < 0)) __PYX_ERR(0, 159, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "pymarketcap/core.pyx":149
 *         self.__repeated_symbols = {}
 *         symbols_slugs, symbols_ids = {}, {}
 *         for currency in self._quick_search():             # <<<<<<<<<<<<<<
 *             symbol = currency["symbol"]
 *             slug = currency["slug"].replace(" ", "")
 */
  }
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "pymarketcap/core.pyx":160
 *                 symbols_slugs[symbol] = slug
 *             symbols_ids[symbol] = currency["id"]
 *         for original, correct in EXCEPTIONAL_COIN_SLUGS.items():             # <<<<<<<<<<<<<<
 *             symbols_slugs[original] = correct
 *         return (symbols_slugs, symbols_ids)
 */
  __pyx_t_3 = __Pyx_GetModuleGlobalName(__pyx_n_s_EXCEPTIONAL_COIN_SLUGS); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 160, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_items); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 160, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = NULL;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  if (__pyx_t_3) {
    __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_3); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 160, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  } else {
    __pyx_t_1 = __Pyx_PyObject_CallNoArg(__pyx_t_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 160, __pyx_L1_error)
  }
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (likely(PyList_CheckExact(__pyx_t_1)) || PyTuple_CheckExact(__pyx_t_1)) {
    __pyx_t_2 = __pyx_t_1; __Pyx_INCREF(__pyx_t_2); __pyx_t_5 = 0;
    __pyx_t_6 = NULL;
  } else {
    __pyx_t_5 = -1; __pyx_t_2 = PyObject_GetIter(__pyx_t_1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 160, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_6 = Py_TYPE(__pyx_t_2)->tp_iternext; if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 160, __pyx_L1_error)
  }
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  for (;;) {
    if (likely(!__pyx_t_6)) {
      if (likely(PyList_CheckExact(__pyx_t_2))) {
        if (__pyx_t_5 >= PyList_GET_SIZE(__pyx_t_2)) break;
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_1 = PyList_GET_ITEM(__pyx_t_2, __pyx_t_5); __Pyx_INCREF(__pyx_t_1); __pyx_t_5++; if (unlikely(0 < 0)) __PYX_ERR(0, 160, __pyx_L1_error)
        #else
        __pyx_t_1 = PySequence_ITEM(__pyx_t_2, __pyx_t_5); __pyx_t_5++; if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 160, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_1);
        #endif
      } else {
        if (__pyx_t_5 >= PyTuple_GET_SIZE(__pyx_t_2)) break;
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_1 = PyTuple_GET_ITEM(__pyx_t_2, __pyx_t_5); __Pyx_INCREF(__pyx_t_1); __pyx_t_5++; if (unlikely(0 < 0)) __PYX_ERR(0, 160, __pyx_L1_error)
        #else
        __pyx_t_1 = PySequence_ITEM(__pyx_t_2, __pyx_t_5); __pyx_t_5++; if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 160, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_1);
        #endif
      }
    } else {
      __pyx_t_1 = __pyx_t_6(__pyx_t_2);
      if (unlikely(!__pyx_t_1)) {
        PyObject* exc_type = PyErr_Occurred();
        if (exc_type) {
          if (likely(__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
          else __PYX_ERR(0, 160, __pyx_L1_error)
        }
        break;
      }
      __Pyx_GOTREF(__pyx_t_1);
    }
    if ((likely(PyTuple_CheckExact(__pyx_t_1))) || (PyList_CheckExact(__pyx_t_1))) {
      PyObject* sequence = __pyx_t_1;
      Py_ssize_t size = __Pyx_PySequence_SIZE(sequence);
      if (unlikely(size != 2)) {
        if (size > 2) __Pyx_RaiseTooManyValuesError(2);
        else if (size >= 0) __Pyx_RaiseNeedMoreValuesError(size);
        __PYX_ERR(0, 160, __pyx_L1_error)
      }
      #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
      if (likely(PyTuple_CheckExact(sequence))) {
        __pyx_t_3 = PyTuple_GET_ITEM(sequence, 0); 
        __pyx_t_4 = PyTuple_GET_ITEM(sequence, 1); 
      } else {
        __pyx_t_3 = PyList_GET_ITEM(sequence, 0); 
        __pyx_t_4 = PyList_GET_ITEM(sequence, 1); 
      }
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_4);
      #else
      __pyx_t_3 = PySequence_ITEM(sequence, 0); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 160, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __pyx_t_4 = PySequence_ITEM(sequence, 1); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 160, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    } else {
      Py_ssize_t index = -1;
      __pyx_t_10 = PyObject_GetIter(__pyx_t_1); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 160, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_10);
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __pyx_t_11 = Py_TYPE(__pyx_t_10)->tp_iternext;
      index = 0; __pyx_t_3 = __pyx_t_11(__pyx_t_10); if (unlikely(!__pyx_t_3)) goto __pyx_L9_unpacking_failed;
      __Pyx_GOTREF(__pyx_t_3);
      index = 1; __pyx_t_4 = __pyx_t_11(__pyx_t_10); if (unlikely(!__pyx_t_4)) goto __pyx_L9_unpacking_failed;
      __Pyx_GOTREF(__pyx_t_4);
      if (__Pyx_IternextUnpackEndCheck(__pyx_t_11(__pyx_t_10), 2) < 0) __PYX_ERR(0, 160, __pyx_L1_error)
      __pyx_t_11 = NULL;
      __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
      goto __pyx_L10_unpacking_done;
      __pyx_L9_unpacking_failed:;
      __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
      __pyx_t_11 = NULL;
      if (__Pyx_IterFinish() == 0) __Pyx_RaiseNeedMoreValuesError(index);
      __PYX_ERR(0, 160, __pyx_L1_error)
      __pyx_L10_unpacking_done:;
    }
    __Pyx_XDECREF_SET(__pyx_v_original, __pyx_t_3);
    __pyx_t_3 = 0;
    __Pyx_XDECREF_SET(__pyx_v_correct, __pyx_t_4);
    __pyx_t_4 = 0;

    /* "pymarketcap/core.pyx":161
 *             symbols_ids[symbol] = currency["id"]
 *         for original, correct in EXCEPTIONAL_COIN_SLUGS.items():
 *             symbols_slugs[original] = correct             # <<<<<<<<<<<<<<
 *         return (symbols_slugs, symbols_ids)
 * 
 */
    if (unlikely(PyDict_SetItem(__pyx_v_symbols_slugs, __pyx_v_original, __pyx_v_correct) < 0)) __PYX_ERR(0, 161, __pyx_L1_error)

    /* "pymarketcap/core.pyx":160
 *                 symbols_slugs[symbol] = slug
 *             symbols_ids[symbol] = currency["id"]
 *         for original, correct in EXCEPTIONAL_COIN_SLUGS.items():             # <<<<<<<<<<<<<<
 *             symbols_slugs[original] = correct
 *         return (symbols_slugs, symbols_ids)
 */
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "pymarketcap/core.pyx":162
 *         for original, correct in EXCEPTIONAL_COIN_SLUGS.items():
 *             symbols_slugs[original] = correct
 *         return (symbols_slugs, symbols_ids)             # <<<<<<<<<<<<<<
 * 
 *     cpdef _cache_exchanges_ids(self):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2 = PyTuple_New(2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 162, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_INCREF(__pyx_v_symbols_slugs);
  __Pyx_GIVEREF(__pyx_v_symbols_slugs);
  PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_v_symbols_slugs);
  __Pyx_INCREF(__pyx_v_symbols_ids);
  __Pyx_GIVEREF(__pyx_v_symbols_ids);
  PyTuple_SET_ITEM(__pyx_t_2, 1, __pyx_v_symbols_ids);
  __pyx_r = __pyx_t_2;
  __pyx_t_2 = 0;
  goto __pyx_L0;

  /* "pymarketcap/core.pyx":144
 *         return loads(self._get(url))
 * 
 *     cpdef _cache_symbols_ids(self):             # <<<<<<<<<<<<<<
 *         """Internal function for load in cache all symbols
 *         in coinmarketcap with their respectives currency names."""
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_10);
  __Pyx_AddTraceback("pymarketcap.core.Pymarketcap._cache_symbols_ids", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_symbols_slugs);
  __Pyx_XDECREF(__pyx_v_symbols_ids);
  __Pyx_XDECREF(__pyx_v_currency);
  __Pyx_XDECREF(__pyx_v_symbol);
  __Pyx_XDECREF(__pyx_v_slug);
  __Pyx_XDECREF(__pyx_v_original);
  __Pyx_XDECREF(__pyx_v_correct);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_11pymarketcap_4core_11Pymarketcap_7_cache_symbols_ids(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_11pymarketcap_4core_11Pymarketcap_6_cache_symbols_ids[] = "Internal function for load in cache all symbols\n        in coinmarketcap with their respectives currency names.";
static PyObject *__pyx_pw_11pymarketcap_4core_11Pymarketcap_7_cache_symbols_ids(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_cache_symbols_ids (wrapper)", 0);
  __pyx_r = __pyx_pf_11pymarketcap_4core_11Pymarketcap_6_cache_symbols_ids(((struct __pyx_obj_11pymarketcap_4core_Pymarketcap *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_11pymarketcap_4core_11Pymarketcap_6_cache_symbols_ids(struct __pyx_obj_11pymarketcap_4core_Pymarketcap *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("_cache_symbols_ids", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_11pymarketcap_4core_11Pymarketcap__cache_symbols_ids(__pyx_v_self, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 144, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pymarketcap.core.Pymarketcap._cache_symbols_ids", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pymarketcap/core.pyx":164
 *         return (symbols_slugs, symbols_ids)
 * 
 *     cpdef _cache_exchanges_ids(self):             # <<<<<<<<<<<<<<
 *         names_slugs_ids = []
 *         for exchange in self._quick_search(exchanges=True):
 */

static PyObject *__pyx_pw_11pymarketcap_4core_11Pymarketcap_9_cache_exchanges_ids(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_f_11pymarketcap_4core_11Pymarketcap__cache_exchanges_ids(struct __pyx_obj_11pymarketcap_4core_Pymarketcap *__pyx_v_self, int __pyx_skip_dispatch) {
  PyObject *__pyx_v_names_slugs_ids = NULL;
  PyObject *__pyx_v_exchange = NULL;
  PyObject *__pyx_v_name_slug_id = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  struct __pyx_opt_args_11pymarketcap_4core_11Pymarketcap__quick_search __pyx_t_5;
  Py_ssize_t __pyx_t_6;
  PyObject *(*__pyx_t_7)(PyObject *);
  int __pyx_t_8;
  int __pyx_t_9;
  int __pyx_t_10;
  __Pyx_RefNannySetupContext("_cache_exchanges_ids", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0)) {
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_cache_exchanges_ids); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 164, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)__pyx_pw_11pymarketcap_4core_11Pymarketcap_9_cache_exchanges_ids)) {
      __Pyx_XDECREF(__pyx_r);
      __Pyx_INCREF(__pyx_t_1);
      __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
      if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
        __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
        if (likely(__pyx_t_4)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
          __Pyx_INCREF(__pyx_t_4);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_3, function);
        }
      }
      if (__pyx_t_4) {
        __pyx_t_2 = __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_4); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 164, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      } else {
        __pyx_t_2 = __Pyx_PyObject_CallNoArg(__pyx_t_3); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 164, __pyx_L1_error)
      }
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __pyx_r = __pyx_t_2;
      __pyx_t_2 = 0;
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      goto __pyx_L0;
    }
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  }

  /* "pymarketcap/core.pyx":165
 * 
 *     cpdef _cache_exchanges_ids(self):
 *         names_slugs_ids = []             # <<<<<<<<<<<<<<
 *         for exchange in self._quick_search(exchanges=True):
 *             name_slug_id = []
 */
  __pyx_t_1 = PyList_New(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 165, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_names_slugs_ids = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "pymarketcap/core.pyx":166
 *     cpdef _cache_exchanges_ids(self):
 *         names_slugs_ids = []
 *         for exchange in self._quick_search(exchanges=True):             # <<<<<<<<<<<<<<
 *             name_slug_id = []
 *             name_slug_id.extend([
 */
  __pyx_t_5.__pyx_n = 1;
  __pyx_t_5.exchanges = Py_True;
  __pyx_t_1 = ((struct __pyx_vtabstruct_11pymarketcap_4core_Pymarketcap *)__pyx_v_self->__pyx_vtab)->_quick_search(__pyx_v_self, 0, &__pyx_t_5); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 166, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (likely(PyList_CheckExact(__pyx_t_1)) || PyTuple_CheckExact(__pyx_t_1)) {
    __pyx_t_2 = __pyx_t_1; __Pyx_INCREF(__pyx_t_2); __pyx_t_6 = 0;
    __pyx_t_7 = NULL;
  } else {
    __pyx_t_6 = -1; __pyx_t_2 = PyObject_GetIter(__pyx_t_1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 166, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_7 = Py_TYPE(__pyx_t_2)->tp_iternext; if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 166, __pyx_L1_error)
  }
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  for (;;) {
    if (likely(!__pyx_t_7)) {
      if (likely(PyList_CheckExact(__pyx_t_2))) {
        if (__pyx_t_6 >= PyList_GET_SIZE(__pyx_t_2)) break;
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_1 = PyList_GET_ITEM(__pyx_t_2, __pyx_t_6); __Pyx_INCREF(__pyx_t_1); __pyx_t_6++; if (unlikely(0 < 0)) __PYX_ERR(0, 166, __pyx_L1_error)
        #else
        __pyx_t_1 = PySequence_ITEM(__pyx_t_2, __pyx_t_6); __pyx_t_6++; if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 166, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_1);
        #endif
      } else {
        if (__pyx_t_6 >= PyTuple_GET_SIZE(__pyx_t_2)) break;
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_1 = PyTuple_GET_ITEM(__pyx_t_2, __pyx_t_6); __Pyx_INCREF(__pyx_t_1); __pyx_t_6++; if (unlikely(0 < 0)) __PYX_ERR(0, 166, __pyx_L1_error)
        #else
        __pyx_t_1 = PySequence_ITEM(__pyx_t_2, __pyx_t_6); __pyx_t_6++; if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 166, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_1);
        #endif
      }
    } else {
      __pyx_t_1 = __pyx_t_7(__pyx_t_2);
      if (unlikely(!__pyx_t_1)) {
        PyObject* exc_type = PyErr_Occurred();
        if (exc_type) {
          if (likely(__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
          else __PYX_ERR(0, 166, __pyx_L1_error)
        }
        break;
      }
      __Pyx_GOTREF(__pyx_t_1);
    }
    __Pyx_XDECREF_SET(__pyx_v_exchange, __pyx_t_1);
    __pyx_t_1 = 0;

    /* "pymarketcap/core.pyx":167
 *         names_slugs_ids = []
 *         for exchange in self._quick_search(exchanges=True):
 *             name_slug_id = []             # <<<<<<<<<<<<<<
 *             name_slug_id.extend([
 *                 exchange["name"], exchange["slug"], exchange["id"]
 */
    __pyx_t_1 = PyList_New(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 167, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_XDECREF_SET(__pyx_v_name_slug_id, ((PyObject*)__pyx_t_1));
    __pyx_t_1 = 0;

    /* "pymarketcap/core.pyx":169
 *             name_slug_id = []
 *             name_slug_id.extend([
 *                 exchange["name"], exchange["slug"], exchange["id"]             # <<<<<<<<<<<<<<
 *             ])
 *             names_slugs_ids.append(name_slug_id)
 */
    __pyx_t_1 = __Pyx_PyObject_Dict_GetItem(__pyx_v_exchange, __pyx_n_s_name); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 169, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_3 = __Pyx_PyObject_Dict_GetItem(__pyx_v_exchange, __pyx_n_s_slug); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 169, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_4 = __Pyx_PyObject_Dict_GetItem(__pyx_v_exchange, __pyx_n_s_id); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 169, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);

    /* "pymarketcap/core.pyx":168
 *         for exchange in self._quick_search(exchanges=True):
 *             name_slug_id = []
 *             name_slug_id.extend([             # <<<<<<<<<<<<<<
 *                 exchange["name"], exchange["slug"], exchange["id"]
 *             ])
 */
    __pyx_t_8 = __Pyx_ListComp_Append(__pyx_v_name_slug_id, __pyx_t_1); if (unlikely(__pyx_t_8 == ((int)-1))) __PYX_ERR(0, 168, __pyx_L1_error)
    __pyx_t_9 = __Pyx_ListComp_Append(__pyx_v_name_slug_id, __pyx_t_3); if (unlikely(__pyx_t_9 == ((int)-1))) __PYX_ERR(0, 168, __pyx_L1_error)
    __pyx_t_10 = __Pyx_PyList_Append(__pyx_v_name_slug_id, __pyx_t_4); if (unlikely(__pyx_t_10 == ((int)-1))) __PYX_ERR(0, 168, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    (void)((__pyx_t_8 | (__pyx_t_9 | __pyx_t_10)));

    /* "pymarketcap/core.pyx":171
 *                 exchange["name"], exchange["slug"], exchange["id"]
 *             ])
 *             names_slugs_ids.append(name_slug_id)             # <<<<<<<<<<<<<<
 *         self._exchange_names_slugs_ids = names_slugs_ids
 *         return self._exchange_names_slugs_ids
 */
    __pyx_t_10 = __Pyx_PyList_Append(__pyx_v_names_slugs_ids, __pyx_v_name_slug_id); if (unlikely(__pyx_t_10 == ((int)-1))) __PYX_ERR(0, 171, __pyx_L1_error)

    /* "pymarketcap/core.pyx":166
 *     cpdef _cache_exchanges_ids(self):
 *         names_slugs_ids = []
 *         for exchange in self._quick_search(exchanges=True):             # <<<<<<<<<<<<<<
 *             name_slug_id = []
 *             name_slug_id.extend([
 */
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "pymarketcap/core.pyx":172
 *             ])
 *             names_slugs_ids.append(name_slug_id)
 *         self._exchange_names_slugs_ids = names_slugs_ids             # <<<<<<<<<<<<<<
 *         return self._exchange_names_slugs_ids
 * 
 */
  __Pyx_INCREF(__pyx_v_names_slugs_ids);
  __Pyx_GIVEREF(__pyx_v_names_slugs_ids);
  __Pyx_GOTREF(__pyx_v_self->_exchange_names_slugs_ids);
  __Pyx_DECREF(__pyx_v_self->_exchange_names_slugs_ids);
  __pyx_v_self->_exchange_names_slugs_ids = __pyx_v_names_slugs_ids;

  /* "pymarketcap/core.pyx":173
 *             names_slugs_ids.append(name_slug_id)
 *         self._exchange_names_slugs_ids = names_slugs_ids
 *         return self._exchange_names_slugs_ids             # <<<<<<<<<<<<<<
 * 
 *     @property
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->_exchange_names_slugs_ids);
  __pyx_r = __pyx_v_self->_exchange_names_slugs_ids;
  goto __pyx_L0;

  /* "pymarketcap/core.pyx":164
 *         return (symbols_slugs, symbols_ids)
 * 
 *     cpdef _cache_exchanges_ids(self):             # <<<<<<<<<<<<<<
 *         names_slugs_ids = []
 *         for exchange in self._quick_search(exchanges=True):
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("pymarketcap.core.Pymarketcap._cache_exchanges_ids", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_names_slugs_ids);
  __Pyx_XDECREF(__pyx_v_exchange);
  __Pyx_XDECREF(__pyx_v_name_slug_id);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_11pymarketcap_4core_11Pymarketcap_9_cache_exchanges_ids(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_11pymarketcap_4core_11Pymarketcap_9_cache_exchanges_ids(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_cache_exchanges_ids (wrapper)", 0);
  __pyx_r = __pyx_pf_11pymarketcap_4core_11Pymarketcap_8_cache_exchanges_ids(((struct __pyx_obj_11pymarketcap_4core_Pymarketcap *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_11pymarketcap_4core_11Pymarketcap_8_cache_exchanges_ids(struct __pyx_obj_11pymarketcap_4core_Pymarketcap *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("_cache_exchanges_ids", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_11pymarketcap_4core_11Pymarketcap__cache_exchanges_ids(__pyx_v_self, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 164, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pymarketcap.core.Pymarketcap._cache_exchanges_ids", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pymarketcap/core.pyx":176
 * 
 *     @property
 *     def symbols(self):             # <<<<<<<<<<<<<<
 *         """Symbols of currencies (in capital letters).
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_11pymarketcap_4core_11Pymarketcap_7symbols_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_11pymarketcap_4core_11Pymarketcap_7symbols_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_11pymarketcap_4core_11Pymarketcap_7symbols___get__(((struct __pyx_obj_11pymarketcap_4core_Pymarketcap *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_11pymarketcap_4core_11Pymarketcap_7symbols___get__(struct __pyx_obj_11pymarketcap_4core_Pymarketcap *__pyx_v_self) {
  PyObject *__pyx_v_res = NULL;
  PyObject *__pyx_v_invalid_symbol = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_t_6;
  Py_ssize_t __pyx_t_7;
  Py_ssize_t __pyx_t_8;
  int __pyx_t_9;
  int __pyx_t_10;
  PyObject *__pyx_t_11 = NULL;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "pymarketcap/core.pyx":182
 *             All currency symbols provided by coinmarketcap.
 *         """
 *         res = self._symbols             # <<<<<<<<<<<<<<
 *         if res:
 *             return res
 */
  __pyx_t_1 = __pyx_v_self->_symbols;
  __Pyx_INCREF(__pyx_t_1);
  __pyx_v_res = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "pymarketcap/core.pyx":183
 *         """
 *         res = self._symbols
 *         if res:             # <<<<<<<<<<<<<<
 *             return res
 *         else:
 */
  __pyx_t_2 = (__pyx_v_res != Py_None)&&(PyList_GET_SIZE(__pyx_v_res) != 0);
  if (__pyx_t_2) {

    /* "pymarketcap/core.pyx":184
 *         res = self._symbols
 *         if res:
 *             return res             # <<<<<<<<<<<<<<
 *         else:
 *             self._symbols = sorted(list(self.correspondences.keys()))
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_v_res);
    __pyx_r = __pyx_v_res;
    goto __pyx_L0;

    /* "pymarketcap/core.pyx":183
 *         """
 *         res = self._symbols
 *         if res:             # <<<<<<<<<<<<<<
 *             return res
 *         else:
 */
  }

  /* "pymarketcap/core.pyx":186
 *             return res
 *         else:
 *             self._symbols = sorted(list(self.correspondences.keys()))             # <<<<<<<<<<<<<<
 *             for invalid_symbol in self.__repeated_symbols:
 *                 self._symbols.remove(invalid_symbol)
 */
  /*else*/ {
    __pyx_t_4 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_correspondences); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 186, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_n_s_keys); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 186, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_t_4 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_5))) {
      __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_5);
      if (likely(__pyx_t_4)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_5);
        __Pyx_INCREF(__pyx_t_4);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_5, function);
      }
    }
    if (__pyx_t_4) {
      __pyx_t_3 = __Pyx_PyObject_CallOneArg(__pyx_t_5, __pyx_t_4); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 186, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    } else {
      __pyx_t_3 = __Pyx_PyObject_CallNoArg(__pyx_t_5); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 186, __pyx_L1_error)
    }
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __pyx_t_5 = PySequence_List(__pyx_t_3); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 186, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_t_3 = PySequence_List(__pyx_t_5); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 186, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __pyx_t_1 = ((PyObject*)__pyx_t_3);
    __pyx_t_3 = 0;
    __pyx_t_6 = PyList_Sort(__pyx_t_1); if (unlikely(__pyx_t_6 == ((int)-1))) __PYX_ERR(0, 186, __pyx_L1_error)
    __Pyx_GIVEREF(__pyx_t_1);
    __Pyx_GOTREF(__pyx_v_self->_symbols);
    __Pyx_DECREF(__pyx_v_self->_symbols);
    __pyx_v_self->_symbols = ((PyObject*)__pyx_t_1);
    __pyx_t_1 = 0;

    /* "pymarketcap/core.pyx":187
 *         else:
 *             self._symbols = sorted(list(self.correspondences.keys()))
 *             for invalid_symbol in self.__repeated_symbols:             # <<<<<<<<<<<<<<
 *                 self._symbols.remove(invalid_symbol)
 *             return self._symbols
 */
    __pyx_t_7 = 0;
    if (unlikely(__pyx_v_self->__repeated_symbols == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not iterable");
      __PYX_ERR(0, 187, __pyx_L1_error)
    }
    __pyx_t_3 = __Pyx_dict_iterator(__pyx_v_self->__repeated_symbols, 1, ((PyObject *)NULL), (&__pyx_t_8), (&__pyx_t_9)); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 187, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_XDECREF(__pyx_t_1);
    __pyx_t_1 = __pyx_t_3;
    __pyx_t_3 = 0;
    while (1) {
      __pyx_t_10 = __Pyx_dict_iter_next(__pyx_t_1, __pyx_t_8, &__pyx_t_7, &__pyx_t_3, NULL, NULL, __pyx_t_9);
      if (unlikely(__pyx_t_10 == 0)) break;
      if (unlikely(__pyx_t_10 == -1)) __PYX_ERR(0, 187, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_XDECREF_SET(__pyx_v_invalid_symbol, __pyx_t_3);
      __pyx_t_3 = 0;

      /* "pymarketcap/core.pyx":188
 *             self._symbols = sorted(list(self.correspondences.keys()))
 *             for invalid_symbol in self.__repeated_symbols:
 *                 self._symbols.remove(invalid_symbol)             # <<<<<<<<<<<<<<
 *             return self._symbols
 * 
 */
      __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_v_self->_symbols, __pyx_n_s_remove); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 188, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
      __pyx_t_4 = NULL;
      if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_5))) {
        __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_5);
        if (likely(__pyx_t_4)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_5);
          __Pyx_INCREF(__pyx_t_4);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_5, function);
        }
      }
      if (!__pyx_t_4) {
        __pyx_t_3 = __Pyx_PyObject_CallOneArg(__pyx_t_5, __pyx_v_invalid_symbol); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 188, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_3);
      } else {
        #if CYTHON_FAST_PYCALL
        if (PyFunction_Check(__pyx_t_5)) {
          PyObject *__pyx_temp[2] = {__pyx_t_4, __pyx_v_invalid_symbol};
          __pyx_t_3 = __Pyx_PyFunction_FastCall(__pyx_t_5, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 188, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
          __Pyx_GOTREF(__pyx_t_3);
        } else
        #endif
        #if CYTHON_FAST_PYCCALL
        if (__Pyx_PyFastCFunction_Check(__pyx_t_5)) {
          PyObject *__pyx_temp[2] = {__pyx_t_4, __pyx_v_invalid_symbol};
          __pyx_t_3 = __Pyx_PyCFunction_FastCall(__pyx_t_5, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 188, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
          __Pyx_GOTREF(__pyx_t_3);
        } else
        #endif
        {
          __pyx_t_11 = PyTuple_New(1+1); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 188, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_11);
          __Pyx_GIVEREF(__pyx_t_4); PyTuple_SET_ITEM(__pyx_t_11, 0, __pyx_t_4); __pyx_t_4 = NULL;
          __Pyx_INCREF(__pyx_v_invalid_symbol);
          __Pyx_GIVEREF(__pyx_v_invalid_symbol);
          PyTuple_SET_ITEM(__pyx_t_11, 0+1, __pyx_v_invalid_symbol);
          __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_5, __pyx_t_11, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 188, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_3);
          __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
        }
      }
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    }
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "pymarketcap/core.pyx":189
 *             for invalid_symbol in self.__repeated_symbols:
 *                 self._symbols.remove(invalid_symbol)
 *             return self._symbols             # <<<<<<<<<<<<<<
 * 
 *     @property
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_v_self->_symbols);
    __pyx_r = __pyx_v_self->_symbols;
    goto __pyx_L0;
  }

  /* "pymarketcap/core.pyx":176
 * 
 *     @property
 *     def symbols(self):             # <<<<<<<<<<<<<<
 *         """Symbols of currencies (in capital letters).
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_11);
  __Pyx_AddTraceback("pymarketcap.core.Pymarketcap.symbols.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_res);
  __Pyx_XDECREF(__pyx_v_invalid_symbol);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pymarketcap/core.pyx":192
 * 
 *     @property
 *     def coins(self):             # <<<<<<<<<<<<<<
 *         """Coins not formatted names for all currencies
 *         (in lowercase letters) used internally by urls.
 */

/* Python wrapper */
static PyObject *__pyx_pw_11pymarketcap_4core_11Pymarketcap_5coins_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_11pymarketcap_4core_11Pymarketcap_5coins_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_11pymarketcap_4core_11Pymarketcap_5coins___get__(((struct __pyx_obj_11pymarketcap_4core_Pymarketcap *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_11pymarketcap_4core_11Pymarketcap_5coins___get__(struct __pyx_obj_11pymarketcap_4core_Pymarketcap *__pyx_v_self) {
  PyObject *__pyx_v_res = NULL;
  PyObject *__pyx_v_coin_or_coins = NULL;
  PyObject *__pyx_v_coin = NULL;
  PyObject *__pyx_v_invalid_coin = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  Py_ssize_t __pyx_t_5;
  int __pyx_t_6;
  Py_ssize_t __pyx_t_7;
  PyObject *(*__pyx_t_8)(PyObject *);
  int __pyx_t_9;
  PyObject *__pyx_t_10 = NULL;
  PyObject *__pyx_t_11 = NULL;
  PyObject *__pyx_t_12 = NULL;
  PyObject *__pyx_t_13 = NULL;
  PyObject *__pyx_t_14 = NULL;
  int __pyx_t_15;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "pymarketcap/core.pyx":199
 *             All currency coins names provided by coinmarketcap.
 *         """
 *         res = self._coins             # <<<<<<<<<<<<<<
 *         if res:
 *             return res
 */
  __pyx_t_1 = __pyx_v_self->_coins;
  __Pyx_INCREF(__pyx_t_1);
  __pyx_v_res = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "pymarketcap/core.pyx":200
 *         """
 *         res = self._coins
 *         if res:             # <<<<<<<<<<<<<<
 *             return res
 *         else:
 */
  __pyx_t_2 = (__pyx_v_res != Py_None)&&(PyList_GET_SIZE(__pyx_v_res) != 0);
  if (__pyx_t_2) {

    /* "pymarketcap/core.pyx":201
 *         res = self._coins
 *         if res:
 *             return res             # <<<<<<<<<<<<<<
 *         else:
 *             self._coins = []
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_v_res);
    __pyx_r = __pyx_v_res;
    goto __pyx_L0;

    /* "pymarketcap/core.pyx":200
 *         """
 *         res = self._coins
 *         if res:             # <<<<<<<<<<<<<<
 *             return res
 *         else:
 */
  }

  /* "pymarketcap/core.pyx":203
 *             return res
 *         else:
 *             self._coins = []             # <<<<<<<<<<<<<<
 *             for coin_or_coins in list(self.correspondences.values()):
 *                 if isinstance(coin_or_coins, list):
 */
  /*else*/ {
    __pyx_t_1 = PyList_New(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 203, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_GIVEREF(__pyx_t_1);
    __Pyx_GOTREF(__pyx_v_self->_coins);
    __Pyx_DECREF(__pyx_v_self->_coins);
    __pyx_v_self->_coins = ((PyObject*)__pyx_t_1);
    __pyx_t_1 = 0;

    /* "pymarketcap/core.pyx":204
 *         else:
 *             self._coins = []
 *             for coin_or_coins in list(self.correspondences.values()):             # <<<<<<<<<<<<<<
 *                 if isinstance(coin_or_coins, list):
 *                     for coin in coin_or_coins:
 */
    __pyx_t_3 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_correspondences); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 204, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_values); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 204, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_t_3 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_4))) {
      __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_4);
      if (likely(__pyx_t_3)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
        __Pyx_INCREF(__pyx_t_3);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_4, function);
      }
    }
    if (__pyx_t_3) {
      __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_4, __pyx_t_3); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 204, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    } else {
      __pyx_t_1 = __Pyx_PyObject_CallNoArg(__pyx_t_4); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 204, __pyx_L1_error)
    }
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_t_4 = PySequence_List(__pyx_t_1); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 204, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_1 = __pyx_t_4; __Pyx_INCREF(__pyx_t_1); __pyx_t_5 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    for (;;) {
      if (__pyx_t_5 >= PyList_GET_SIZE(__pyx_t_1)) break;
      #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
      __pyx_t_4 = PyList_GET_ITEM(__pyx_t_1, __pyx_t_5); __Pyx_INCREF(__pyx_t_4); __pyx_t_5++; if (unlikely(0 < 0)) __PYX_ERR(0, 204, __pyx_L1_error)
      #else
      __pyx_t_4 = PySequence_ITEM(__pyx_t_1, __pyx_t_5); __pyx_t_5++; if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 204, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      #endif
      __Pyx_XDECREF_SET(__pyx_v_coin_or_coins, __pyx_t_4);
      __pyx_t_4 = 0;

      /* "pymarketcap/core.pyx":205
 *             self._coins = []
 *             for coin_or_coins in list(self.correspondences.values()):
 *                 if isinstance(coin_or_coins, list):             # <<<<<<<<<<<<<<
 *                     for coin in coin_or_coins:
 *                         self._coins.append(coin)
 */
      __pyx_t_2 = PyList_Check(__pyx_v_coin_or_coins); 
      __pyx_t_6 = (__pyx_t_2 != 0);
      if (__pyx_t_6) {

        /* "pymarketcap/core.pyx":206
 *             for coin_or_coins in list(self.correspondences.values()):
 *                 if isinstance(coin_or_coins, list):
 *                     for coin in coin_or_coins:             # <<<<<<<<<<<<<<
 *                         self._coins.append(coin)
 *                 else:
 */
        if (likely(PyList_CheckExact(__pyx_v_coin_or_coins)) || PyTuple_CheckExact(__pyx_v_coin_or_coins)) {
          __pyx_t_4 = __pyx_v_coin_or_coins; __Pyx_INCREF(__pyx_t_4); __pyx_t_7 = 0;
          __pyx_t_8 = NULL;
        } else {
          __pyx_t_7 = -1; __pyx_t_4 = PyObject_GetIter(__pyx_v_coin_or_coins); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 206, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_4);
          __pyx_t_8 = Py_TYPE(__pyx_t_4)->tp_iternext; if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 206, __pyx_L1_error)
        }
        for (;;) {
          if (likely(!__pyx_t_8)) {
            if (likely(PyList_CheckExact(__pyx_t_4))) {
              if (__pyx_t_7 >= PyList_GET_SIZE(__pyx_t_4)) break;
              #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
              __pyx_t_3 = PyList_GET_ITEM(__pyx_t_4, __pyx_t_7); __Pyx_INCREF(__pyx_t_3); __pyx_t_7++; if (unlikely(0 < 0)) __PYX_ERR(0, 206, __pyx_L1_error)
              #else
              __pyx_t_3 = PySequence_ITEM(__pyx_t_4, __pyx_t_7); __pyx_t_7++; if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 206, __pyx_L1_error)
              __Pyx_GOTREF(__pyx_t_3);
              #endif
            } else {
              if (__pyx_t_7 >= PyTuple_GET_SIZE(__pyx_t_4)) break;
              #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
              __pyx_t_3 = PyTuple_GET_ITEM(__pyx_t_4, __pyx_t_7); __Pyx_INCREF(__pyx_t_3); __pyx_t_7++; if (unlikely(0 < 0)) __PYX_ERR(0, 206, __pyx_L1_error)
              #else
              __pyx_t_3 = PySequence_ITEM(__pyx_t_4, __pyx_t_7); __pyx_t_7++; if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 206, __pyx_L1_error)
              __Pyx_GOTREF(__pyx_t_3);
              #endif
            }
          } else {
            __pyx_t_3 = __pyx_t_8(__pyx_t_4);
            if (unlikely(!__pyx_t_3)) {
              PyObject* exc_type = PyErr_Occurred();
              if (exc_type) {
                if (likely(__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
                else __PYX_ERR(0, 206, __pyx_L1_error)
              }
              break;
            }
            __Pyx_GOTREF(__pyx_t_3);
          }
          __Pyx_XDECREF_SET(__pyx_v_coin, __pyx_t_3);
          __pyx_t_3 = 0;

          /* "pymarketcap/core.pyx":207
 *                 if isinstance(coin_or_coins, list):
 *                     for coin in coin_or_coins:
 *                         self._coins.append(coin)             # <<<<<<<<<<<<<<
 *                 else:
 *                     self._coins.append(coin_or_coins)
 */
          if (unlikely(__pyx_v_self->_coins == Py_None)) {
            PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "append");
            __PYX_ERR(0, 207, __pyx_L1_error)
          }
          __pyx_t_9 = __Pyx_PyList_Append(__pyx_v_self->_coins, __pyx_v_coin); if (unlikely(__pyx_t_9 == ((int)-1))) __PYX_ERR(0, 207, __pyx_L1_error)

          /* "pymarketcap/core.pyx":206
 *             for coin_or_coins in list(self.correspondences.values()):
 *                 if isinstance(coin_or_coins, list):
 *                     for coin in coin_or_coins:             # <<<<<<<<<<<<<<
 *                         self._coins.append(coin)
 *                 else:
 */
        }
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

        /* "pymarketcap/core.pyx":205
 *             self._coins = []
 *             for coin_or_coins in list(self.correspondences.values()):
 *                 if isinstance(coin_or_coins, list):             # <<<<<<<<<<<<<<
 *                     for coin in coin_or_coins:
 *                         self._coins.append(coin)
 */
        goto __pyx_L6;
      }

      /* "pymarketcap/core.pyx":209
 *                         self._coins.append(coin)
 *                 else:
 *                     self._coins.append(coin_or_coins)             # <<<<<<<<<<<<<<
 *             for invalid_coin in INVALID_COINS:
 *                 try:
 */
      /*else*/ {
        if (unlikely(__pyx_v_self->_coins == Py_None)) {
          PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "append");
          __PYX_ERR(0, 209, __pyx_L1_error)
        }
        __pyx_t_9 = __Pyx_PyList_Append(__pyx_v_self->_coins, __pyx_v_coin_or_coins); if (unlikely(__pyx_t_9 == ((int)-1))) __PYX_ERR(0, 209, __pyx_L1_error)
      }
      __pyx_L6:;

      /* "pymarketcap/core.pyx":204
 *         else:
 *             self._coins = []
 *             for coin_or_coins in list(self.correspondences.values()):             # <<<<<<<<<<<<<<
 *                 if isinstance(coin_or_coins, list):
 *                     for coin in coin_or_coins:
 */
    }
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "pymarketcap/core.pyx":210
 *                 else:
 *                     self._coins.append(coin_or_coins)
 *             for invalid_coin in INVALID_COINS:             # <<<<<<<<<<<<<<
 *                 try:
 *                     self._coins.remove(invalid_coin)
 */
    __pyx_t_1 = __Pyx_GetModuleGlobalName(__pyx_n_s_INVALID_COINS); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 210, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    if (likely(PyList_CheckExact(__pyx_t_1)) || PyTuple_CheckExact(__pyx_t_1)) {
      __pyx_t_4 = __pyx_t_1; __Pyx_INCREF(__pyx_t_4); __pyx_t_5 = 0;
      __pyx_t_8 = NULL;
    } else {
      __pyx_t_5 = -1; __pyx_t_4 = PyObject_GetIter(__pyx_t_1); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 210, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_8 = Py_TYPE(__pyx_t_4)->tp_iternext; if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 210, __pyx_L1_error)
    }
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    for (;;) {
      if (likely(!__pyx_t_8)) {
        if (likely(PyList_CheckExact(__pyx_t_4))) {
          if (__pyx_t_5 >= PyList_GET_SIZE(__pyx_t_4)) break;
          #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
          __pyx_t_1 = PyList_GET_ITEM(__pyx_t_4, __pyx_t_5); __Pyx_INCREF(__pyx_t_1); __pyx_t_5++; if (unlikely(0 < 0)) __PYX_ERR(0, 210, __pyx_L1_error)
          #else
          __pyx_t_1 = PySequence_ITEM(__pyx_t_4, __pyx_t_5); __pyx_t_5++; if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 210, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_1);
          #endif
        } else {
          if (__pyx_t_5 >= PyTuple_GET_SIZE(__pyx_t_4)) break;
          #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
          __pyx_t_1 = PyTuple_GET_ITEM(__pyx_t_4, __pyx_t_5); __Pyx_INCREF(__pyx_t_1); __pyx_t_5++; if (unlikely(0 < 0)) __PYX_ERR(0, 210, __pyx_L1_error)
          #else
          __pyx_t_1 = PySequence_ITEM(__pyx_t_4, __pyx_t_5); __pyx_t_5++; if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 210, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_1);
          #endif
        }
      } else {
        __pyx_t_1 = __pyx_t_8(__pyx_t_4);
        if (unlikely(!__pyx_t_1)) {
          PyObject* exc_type = PyErr_Occurred();
          if (exc_type) {
            if (likely(__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
            else __PYX_ERR(0, 210, __pyx_L1_error)
          }
          break;
        }
        __Pyx_GOTREF(__pyx_t_1);
      }
      __Pyx_XDECREF_SET(__pyx_v_invalid_coin, __pyx_t_1);
      __pyx_t_1 = 0;

      /* "pymarketcap/core.pyx":211
 *                     self._coins.append(coin_or_coins)
 *             for invalid_coin in INVALID_COINS:
 *                 try:             # <<<<<<<<<<<<<<
 *                     self._coins.remove(invalid_coin)
 *                 except ValueError:
 */
      {
        __Pyx_PyThreadState_declare
        __Pyx_PyThreadState_assign
        __Pyx_ExceptionSave(&__pyx_t_10, &__pyx_t_11, &__pyx_t_12);
        __Pyx_XGOTREF(__pyx_t_10);
        __Pyx_XGOTREF(__pyx_t_11);
        __Pyx_XGOTREF(__pyx_t_12);
        /*try:*/ {

          /* "pymarketcap/core.pyx":212
 *             for invalid_coin in INVALID_COINS:
 *                 try:
 *                     self._coins.remove(invalid_coin)             # <<<<<<<<<<<<<<
 *                 except ValueError:
 *                     pass
 */
          __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_self->_coins, __pyx_n_s_remove); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 212, __pyx_L11_error)
          __Pyx_GOTREF(__pyx_t_3);
          __pyx_t_13 = NULL;
          if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_3))) {
            __pyx_t_13 = PyMethod_GET_SELF(__pyx_t_3);
            if (likely(__pyx_t_13)) {
              PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
              __Pyx_INCREF(__pyx_t_13);
              __Pyx_INCREF(function);
              __Pyx_DECREF_SET(__pyx_t_3, function);
            }
          }
          if (!__pyx_t_13) {
            __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_v_invalid_coin); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 212, __pyx_L11_error)
            __Pyx_GOTREF(__pyx_t_1);
          } else {
            #if CYTHON_FAST_PYCALL
            if (PyFunction_Check(__pyx_t_3)) {
              PyObject *__pyx_temp[2] = {__pyx_t_13, __pyx_v_invalid_coin};
              __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_3, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 212, __pyx_L11_error)
              __Pyx_XDECREF(__pyx_t_13); __pyx_t_13 = 0;
              __Pyx_GOTREF(__pyx_t_1);
            } else
            #endif
            #if CYTHON_FAST_PYCCALL
            if (__Pyx_PyFastCFunction_Check(__pyx_t_3)) {
              PyObject *__pyx_temp[2] = {__pyx_t_13, __pyx_v_invalid_coin};
              __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_3, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 212, __pyx_L11_error)
              __Pyx_XDECREF(__pyx_t_13); __pyx_t_13 = 0;
              __Pyx_GOTREF(__pyx_t_1);
            } else
            #endif
            {
              __pyx_t_14 = PyTuple_New(1+1); if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 212, __pyx_L11_error)
              __Pyx_GOTREF(__pyx_t_14);
              __Pyx_GIVEREF(__pyx_t_13); PyTuple_SET_ITEM(__pyx_t_14, 0, __pyx_t_13); __pyx_t_13 = NULL;
              __Pyx_INCREF(__pyx_v_invalid_coin);
              __Pyx_GIVEREF(__pyx_v_invalid_coin);
              PyTuple_SET_ITEM(__pyx_t_14, 0+1, __pyx_v_invalid_coin);
              __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_t_14, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 212, __pyx_L11_error)
              __Pyx_GOTREF(__pyx_t_1);
              __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;
            }
          }
          __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
          __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

          /* "pymarketcap/core.pyx":211
 *                     self._coins.append(coin_or_coins)
 *             for invalid_coin in INVALID_COINS:
 *                 try:             # <<<<<<<<<<<<<<
 *                     self._coins.remove(invalid_coin)
 *                 except ValueError:
 */
        }
        __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
        __Pyx_XDECREF(__pyx_t_11); __pyx_t_11 = 0;
        __Pyx_XDECREF(__pyx_t_12); __pyx_t_12 = 0;
        goto __pyx_L18_try_end;
        __pyx_L11_error:;
        __Pyx_XDECREF(__pyx_t_13); __pyx_t_13 = 0;
        __Pyx_XDECREF(__pyx_t_14); __pyx_t_14 = 0;
        __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
        __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;

        /* "pymarketcap/core.pyx":213
 *                 try:
 *                     self._coins.remove(invalid_coin)
 *                 except ValueError:             # <<<<<<<<<<<<<<
 *                     pass
 *             self._coins = sorted(self._coins)
 */
        __pyx_t_15 = __Pyx_PyErr_ExceptionMatches(__pyx_builtin_ValueError);
        if (__pyx_t_15) {
          __Pyx_ErrRestore(0,0,0);
          goto __pyx_L12_exception_handled;
        }
        goto __pyx_L13_except_error;
        __pyx_L13_except_error:;

        /* "pymarketcap/core.pyx":211
 *                     self._coins.append(coin_or_coins)
 *             for invalid_coin in INVALID_COINS:
 *                 try:             # <<<<<<<<<<<<<<
 *                     self._coins.remove(invalid_coin)
 *                 except ValueError:
 */
        __Pyx_XGIVEREF(__pyx_t_10);
        __Pyx_XGIVEREF(__pyx_t_11);
        __Pyx_XGIVEREF(__pyx_t_12);
        __Pyx_ExceptionReset(__pyx_t_10, __pyx_t_11, __pyx_t_12);
        goto __pyx_L1_error;
        __pyx_L12_exception_handled:;
        __Pyx_XGIVEREF(__pyx_t_10);
        __Pyx_XGIVEREF(__pyx_t_11);
        __Pyx_XGIVEREF(__pyx_t_12);
        __Pyx_ExceptionReset(__pyx_t_10, __pyx_t_11, __pyx_t_12);
        __pyx_L18_try_end:;
      }

      /* "pymarketcap/core.pyx":210
 *                 else:
 *                     self._coins.append(coin_or_coins)
 *             for invalid_coin in INVALID_COINS:             # <<<<<<<<<<<<<<
 *                 try:
 *                     self._coins.remove(invalid_coin)
 */
    }
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

    /* "pymarketcap/core.pyx":215
 *                 except ValueError:
 *                     pass
 *             self._coins = sorted(self._coins)             # <<<<<<<<<<<<<<
 *             return self._coins
 * 
 */
    __pyx_t_1 = __pyx_v_self->_coins;
    __Pyx_INCREF(__pyx_t_1);
    __pyx_t_3 = PySequence_List(__pyx_t_1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 215, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_4 = ((PyObject*)__pyx_t_3);
    __pyx_t_3 = 0;
    __pyx_t_9 = PyList_Sort(__pyx_t_4); if (unlikely(__pyx_t_9 == ((int)-1))) __PYX_ERR(0, 215, __pyx_L1_error)
    __Pyx_GIVEREF(__pyx_t_4);
    __Pyx_GOTREF(__pyx_v_self->_coins);
    __Pyx_DECREF(__pyx_v_self->_coins);
    __pyx_v_self->_coins = ((PyObject*)__pyx_t_4);
    __pyx_t_4 = 0;

    /* "pymarketcap/core.pyx":216
 *                     pass
 *             self._coins = sorted(self._coins)
 *             return self._coins             # <<<<<<<<<<<<<<
 * 
 *     @property
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_v_self->_coins);
    __pyx_r = __pyx_v_self->_coins;
    goto __pyx_L0;
  }

  /* "pymarketcap/core.pyx":192
 * 
 *     @property
 *     def coins(self):             # <<<<<<<<<<<<<<
 *         """Coins not formatted names for all currencies
 *         (in lowercase letters) used internally by urls.
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_13);
  __Pyx_XDECREF(__pyx_t_14);
  __Pyx_AddTraceback("pymarketcap.core.Pymarketcap.coins.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_res);
  __Pyx_XDECREF(__pyx_v_coin_or_coins);
  __Pyx_XDECREF(__pyx_v_coin);
  __Pyx_XDECREF(__pyx_v_invalid_coin);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pymarketcap/core.pyx":219
 * 
 *     @property
 *     def total_currencies(self):             # <<<<<<<<<<<<<<
 *         res = self._total_currencies
 *         if res:
 */

/* Python wrapper */
static PyObject *__pyx_pw_11pymarketcap_4core_11Pymarketcap_16total_currencies_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_11pymarketcap_4core_11Pymarketcap_16total_currencies_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_11pymarketcap_4core_11Pymarketcap_16total_currencies___get__(((struct __pyx_obj_11pymarketcap_4core_Pymarketcap *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_11pymarketcap_4core_11Pymarketcap_16total_currencies___get__(struct __pyx_obj_11pymarketcap_4core_Pymarketcap *__pyx_v_self) {
  int __pyx_v_res;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "pymarketcap/core.pyx":220
 *     @property
 *     def total_currencies(self):
 *         res = self._total_currencies             # <<<<<<<<<<<<<<
 *         if res:
 *             return res
 */
  __pyx_t_1 = __pyx_v_self->_total_currencies;
  __pyx_v_res = __pyx_t_1;

  /* "pymarketcap/core.pyx":221
 *     def total_currencies(self):
 *         res = self._total_currencies
 *         if res:             # <<<<<<<<<<<<<<
 *             return res
 *         else:
 */
  __pyx_t_2 = (__pyx_v_res != 0);
  if (__pyx_t_2) {

    /* "pymarketcap/core.pyx":222
 *         res = self._total_currencies
 *         if res:
 *             return res             # <<<<<<<<<<<<<<
 *         else:
 *             self._total_currencies = self.ticker()[-1]["rank"]
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_3 = __Pyx_PyInt_From_int(__pyx_v_res); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 222, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_r = __pyx_t_3;
    __pyx_t_3 = 0;
    goto __pyx_L0;

    /* "pymarketcap/core.pyx":221
 *     def total_currencies(self):
 *         res = self._total_currencies
 *         if res:             # <<<<<<<<<<<<<<
 *             return res
 *         else:
 */
  }

  /* "pymarketcap/core.pyx":224
 *             return res
 *         else:
 *             self._total_currencies = self.ticker()[-1]["rank"]             # <<<<<<<<<<<<<<
 *             return self._total_currencies
 * 
 */
  /*else*/ {
    __pyx_t_3 = ((struct __pyx_vtabstruct_11pymarketcap_4core_Pymarketcap *)__pyx_v_self->__pyx_vtab)->ticker(__pyx_v_self, 0, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 224, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_4 = __Pyx_GetItemInt(__pyx_t_3, -1L, long, 1, __Pyx_PyInt_From_long, 0, 1, 1); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 224, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_t_3 = __Pyx_PyObject_Dict_GetItem(__pyx_t_4, __pyx_n_s_rank); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 224, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_t_1 = __Pyx_PyInt_As_int(__pyx_t_3); if (unlikely((__pyx_t_1 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 224, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_v_self->_total_currencies = __pyx_t_1;

    /* "pymarketcap/core.pyx":225
 *         else:
 *             self._total_currencies = self.ticker()[-1]["rank"]
 *             return self._total_currencies             # <<<<<<<<<<<<<<
 * 
 *     @property
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_3 = __Pyx_PyInt_From_int(__pyx_v_self->_total_currencies); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 225, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_r = __pyx_t_3;
    __pyx_t_3 = 0;
    goto __pyx_L0;
  }

  /* "pymarketcap/core.pyx":219
 * 
 *     @property
 *     def total_currencies(self):             # <<<<<<<<<<<<<<
 *         res = self._total_currencies
 *         if res:
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("pymarketcap.core.Pymarketcap.total_currencies.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pymarketcap/core.pyx":228
 * 
 *     @property
 *     def currencies_to_convert(self):             # <<<<<<<<<<<<<<
 *         res = self._currencies_to_convert
 *         if res:
 */

/* Python wrapper */
static PyObject *__pyx_pw_11pymarketcap_4core_11Pymarketcap_21currencies_to_convert_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_11pymarketcap_4core_11Pymarketcap_21currencies_to_convert_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_11pymarketcap_4core_11Pymarketcap_21currencies_to_convert___get__(((struct __pyx_obj_11pymarketcap_4core_Pymarketcap *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_11pymarketcap_4core_11Pymarketcap_21currencies_to_convert___get__(struct __pyx_obj_11pymarketcap_4core_Pymarketcap *__pyx_v_self) {
  PyObject *__pyx_v_res = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "pymarketcap/core.pyx":229
 *     @property
 *     def currencies_to_convert(self):
 *         res = self._currencies_to_convert             # <<<<<<<<<<<<<<
 *         if res:
 *             return res
 */
  __pyx_t_1 = __pyx_v_self->_currencies_to_convert;
  __Pyx_INCREF(__pyx_t_1);
  __pyx_v_res = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "pymarketcap/core.pyx":230
 *     def currencies_to_convert(self):
 *         res = self._currencies_to_convert
 *         if res:             # <<<<<<<<<<<<<<
 *             return res
 *         else:
 */
  __pyx_t_2 = (__pyx_v_res != Py_None)&&(PyList_GET_SIZE(__pyx_v_res) != 0);
  if (__pyx_t_2) {

    /* "pymarketcap/core.pyx":231
 *         res = self._currencies_to_convert
 *         if res:
 *             return res             # <<<<<<<<<<<<<<
 *         else:
 *             self._currencies_to_convert = self.__currencies_to_convert()
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_v_res);
    __pyx_r = __pyx_v_res;
    goto __pyx_L0;

    /* "pymarketcap/core.pyx":230
 *     def currencies_to_convert(self):
 *         res = self._currencies_to_convert
 *         if res:             # <<<<<<<<<<<<<<
 *             return res
 *         else:
 */
  }

  /* "pymarketcap/core.pyx":233
 *             return res
 *         else:
 *             self._currencies_to_convert = self.__currencies_to_convert()             # <<<<<<<<<<<<<<
 *             return self._currencies_to_convert
 * 
 */
  /*else*/ {
    __pyx_t_1 = ((struct __pyx_vtabstruct_11pymarketcap_4core_Pymarketcap *)__pyx_v_self->__pyx_vtab)->__pyx___currencies_to_convert(__pyx_v_self, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 233, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    if (!(likely(PyList_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "list", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(0, 233, __pyx_L1_error)
    __Pyx_GIVEREF(__pyx_t_1);
    __Pyx_GOTREF(__pyx_v_self->_currencies_to_convert);
    __Pyx_DECREF(__pyx_v_self->_currencies_to_convert);
    __pyx_v_self->_currencies_to_convert = ((PyObject*)__pyx_t_1);
    __pyx_t_1 = 0;

    /* "pymarketcap/core.pyx":234
 *         else:
 *             self._currencies_to_convert = self.__currencies_to_convert()
 *             return self._currencies_to_convert             # <<<<<<<<<<<<<<
 * 
 *     cpdef __currencies_to_convert(self):
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_v_self->_currencies_to_convert);
    __pyx_r = __pyx_v_self->_currencies_to_convert;
    goto __pyx_L0;
  }

  /* "pymarketcap/core.pyx":228
 * 
 *     @property
 *     def currencies_to_convert(self):             # <<<<<<<<<<<<<<
 *         res = self._currencies_to_convert
 *         if res:
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pymarketcap.core.Pymarketcap.currencies_to_convert.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_res);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pymarketcap/core.pyx":236
 *             return self._currencies_to_convert
 * 
 *     cpdef __currencies_to_convert(self):             # <<<<<<<<<<<<<<
 *         """Internal function for get currencies from and to convert
 *             values in convert() method. Don't use this, but cached
 */

static PyObject *__pyx_pw_11pymarketcap_4core_11Pymarketcap_11__currencies_to_convert(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_f_11pymarketcap_4core_11Pymarketcap___currencies_to_convert(struct __pyx_obj_11pymarketcap_4core_Pymarketcap *__pyx_v_self, int __pyx_skip_dispatch) {
  PyObject *__pyx_v_res = NULL;
  PyObject *__pyx_v_currencies = NULL;
  PyObject *__pyx_v_response = NULL;
  PyObject *__pyx_v_currency = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  Py_ssize_t __pyx_t_7;
  PyObject *(*__pyx_t_8)(PyObject *);
  int __pyx_t_9;
  __Pyx_RefNannySetupContext("__currencies_to_convert", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0)) {
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_currencies_to_convert); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 236, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)__pyx_pw_11pymarketcap_4core_11Pymarketcap_11__currencies_to_convert)) {
      __Pyx_XDECREF(__pyx_r);
      __Pyx_INCREF(__pyx_t_1);
      __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
      if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
        __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
        if (likely(__pyx_t_4)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
          __Pyx_INCREF(__pyx_t_4);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_3, function);
        }
      }
      if (__pyx_t_4) {
        __pyx_t_2 = __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_4); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 236, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      } else {
        __pyx_t_2 = __Pyx_PyObject_CallNoArg(__pyx_t_3); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 236, __pyx_L1_error)
      }
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __pyx_r = __pyx_t_2;
      __pyx_t_2 = 0;
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      goto __pyx_L0;
    }
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  }

  /* "pymarketcap/core.pyx":246
 *             :meth:`~pymarketcap.core.Pymarketcap.convert`.
 *         """
 *         res = self._get(b"https://coinmarketcap.com")             # <<<<<<<<<<<<<<
 *         currencies = re_findall(r'data-([a-z]+)="\d', res[-10000:-2000])
 *         response = [currency.upper() for currency in currencies]
 */
  __pyx_t_1 = ((struct __pyx_vtabstruct_11pymarketcap_4core_Pymarketcap *)__pyx_v_self->__pyx_vtab)->_get(__pyx_v_self, ((char *)"https://coinmarketcap.com")); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 246, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_res = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "pymarketcap/core.pyx":247
 *         """
 *         res = self._get(b"https://coinmarketcap.com")
 *         currencies = re_findall(r'data-([a-z]+)="\d', res[-10000:-2000])             # <<<<<<<<<<<<<<
 *         response = [currency.upper() for currency in currencies]
 *         response.extend([str(currency["symbol"]) for currency in self.ticker()])
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_re_findall); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 247, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyObject_GetSlice(__pyx_v_res, -10000L, -2000L, NULL, NULL, &__pyx_slice__5, 1, 1, 1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 247, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = NULL;
  __pyx_t_5 = 0;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_5 = 1;
    }
  }
  #if CYTHON_FAST_PYCALL
  if (PyFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[3] = {__pyx_t_4, __pyx_kp_s_data_a_z_d, __pyx_t_3};
    __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_5, 2+__pyx_t_5); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 247, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  } else
  #endif
  #if CYTHON_FAST_PYCCALL
  if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[3] = {__pyx_t_4, __pyx_kp_s_data_a_z_d, __pyx_t_3};
    __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_5, 2+__pyx_t_5); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 247, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  } else
  #endif
  {
    __pyx_t_6 = PyTuple_New(2+__pyx_t_5); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 247, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    if (__pyx_t_4) {
      __Pyx_GIVEREF(__pyx_t_4); PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_4); __pyx_t_4 = NULL;
    }
    __Pyx_INCREF(__pyx_kp_s_data_a_z_d);
    __Pyx_GIVEREF(__pyx_kp_s_data_a_z_d);
    PyTuple_SET_ITEM(__pyx_t_6, 0+__pyx_t_5, __pyx_kp_s_data_a_z_d);
    __Pyx_GIVEREF(__pyx_t_3);
    PyTuple_SET_ITEM(__pyx_t_6, 1+__pyx_t_5, __pyx_t_3);
    __pyx_t_3 = 0;
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_6, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 247, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v_currencies = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "pymarketcap/core.pyx":248
 *         res = self._get(b"https://coinmarketcap.com")
 *         currencies = re_findall(r'data-([a-z]+)="\d', res[-10000:-2000])
 *         response = [currency.upper() for currency in currencies]             # <<<<<<<<<<<<<<
 *         response.extend([str(currency["symbol"]) for currency in self.ticker()])
 *         return sorted(response)
 */
  __pyx_t_1 = PyList_New(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 248, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (likely(PyList_CheckExact(__pyx_v_currencies)) || PyTuple_CheckExact(__pyx_v_currencies)) {
    __pyx_t_2 = __pyx_v_currencies; __Pyx_INCREF(__pyx_t_2); __pyx_t_7 = 0;
    __pyx_t_8 = NULL;
  } else {
    __pyx_t_7 = -1; __pyx_t_2 = PyObject_GetIter(__pyx_v_currencies); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 248, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_8 = Py_TYPE(__pyx_t_2)->tp_iternext; if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 248, __pyx_L1_error)
  }
  for (;;) {
    if (likely(!__pyx_t_8)) {
      if (likely(PyList_CheckExact(__pyx_t_2))) {
        if (__pyx_t_7 >= PyList_GET_SIZE(__pyx_t_2)) break;
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_6 = PyList_GET_ITEM(__pyx_t_2, __pyx_t_7); __Pyx_INCREF(__pyx_t_6); __pyx_t_7++; if (unlikely(0 < 0)) __PYX_ERR(0, 248, __pyx_L1_error)
        #else
        __pyx_t_6 = PySequence_ITEM(__pyx_t_2, __pyx_t_7); __pyx_t_7++; if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 248, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_6);
        #endif
      } else {
        if (__pyx_t_7 >= PyTuple_GET_SIZE(__pyx_t_2)) break;
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_6 = PyTuple_GET_ITEM(__pyx_t_2, __pyx_t_7); __Pyx_INCREF(__pyx_t_6); __pyx_t_7++; if (unlikely(0 < 0)) __PYX_ERR(0, 248, __pyx_L1_error)
        #else
        __pyx_t_6 = PySequence_ITEM(__pyx_t_2, __pyx_t_7); __pyx_t_7++; if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 248, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_6);
        #endif
      }
    } else {
      __pyx_t_6 = __pyx_t_8(__pyx_t_2);
      if (unlikely(!__pyx_t_6)) {
        PyObject* exc_type = PyErr_Occurred();
        if (exc_type) {
          if (likely(__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
          else __PYX_ERR(0, 248, __pyx_L1_error)
        }
        break;
      }
      __Pyx_GOTREF(__pyx_t_6);
    }
    __Pyx_XDECREF_SET(__pyx_v_currency, __pyx_t_6);
    __pyx_t_6 = 0;
    __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_currency, __pyx_n_s_upper); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 248, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_4 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_3))) {
      __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
      if (likely(__pyx_t_4)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
        __Pyx_INCREF(__pyx_t_4);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_3, function);
      }
    }
    if (__pyx_t_4) {
      __pyx_t_6 = __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_4); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 248, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    } else {
      __pyx_t_6 = __Pyx_PyObject_CallNoArg(__pyx_t_3); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 248, __pyx_L1_error)
    }
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (unlikely(__Pyx_ListComp_Append(__pyx_t_1, (PyObject*)__pyx_t_6))) __PYX_ERR(0, 248, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v_response = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "pymarketcap/core.pyx":249
 *         currencies = re_findall(r'data-([a-z]+)="\d', res[-10000:-2000])
 *         response = [currency.upper() for currency in currencies]
 *         response.extend([str(currency["symbol"]) for currency in self.ticker()])             # <<<<<<<<<<<<<<
 *         return sorted(response)
 * 
 */
  __pyx_t_1 = PyList_New(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 249, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = ((struct __pyx_vtabstruct_11pymarketcap_4core_Pymarketcap *)__pyx_v_self->__pyx_vtab)->ticker(__pyx_v_self, 0, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 249, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (likely(PyList_CheckExact(__pyx_t_2)) || PyTuple_CheckExact(__pyx_t_2)) {
    __pyx_t_6 = __pyx_t_2; __Pyx_INCREF(__pyx_t_6); __pyx_t_7 = 0;
    __pyx_t_8 = NULL;
  } else {
    __pyx_t_7 = -1; __pyx_t_6 = PyObject_GetIter(__pyx_t_2); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 249, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_8 = Py_TYPE(__pyx_t_6)->tp_iternext; if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 249, __pyx_L1_error)
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  for (;;) {
    if (likely(!__pyx_t_8)) {
      if (likely(PyList_CheckExact(__pyx_t_6))) {
        if (__pyx_t_7 >= PyList_GET_SIZE(__pyx_t_6)) break;
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_2 = PyList_GET_ITEM(__pyx_t_6, __pyx_t_7); __Pyx_INCREF(__pyx_t_2); __pyx_t_7++; if (unlikely(0 < 0)) __PYX_ERR(0, 249, __pyx_L1_error)
        #else
        __pyx_t_2 = PySequence_ITEM(__pyx_t_6, __pyx_t_7); __pyx_t_7++; if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 249, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        #endif
      } else {
        if (__pyx_t_7 >= PyTuple_GET_SIZE(__pyx_t_6)) break;
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_2 = PyTuple_GET_ITEM(__pyx_t_6, __pyx_t_7); __Pyx_INCREF(__pyx_t_2); __pyx_t_7++; if (unlikely(0 < 0)) __PYX_ERR(0, 249, __pyx_L1_error)
        #else
        __pyx_t_2 = PySequence_ITEM(__pyx_t_6, __pyx_t_7); __pyx_t_7++; if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 249, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        #endif
      }
    } else {
      __pyx_t_2 = __pyx_t_8(__pyx_t_6);
      if (unlikely(!__pyx_t_2)) {
        PyObject* exc_type = PyErr_Occurred();
        if (exc_type) {
          if (likely(__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
          else __PYX_ERR(0, 249, __pyx_L1_error)
        }
        break;
      }
      __Pyx_GOTREF(__pyx_t_2);
    }
    __Pyx_XDECREF_SET(__pyx_v_currency, __pyx_t_2);
    __pyx_t_2 = 0;
    __pyx_t_2 = __Pyx_PyObject_Dict_GetItem(__pyx_v_currency, __pyx_n_s_symbol); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 249, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_3 = __Pyx_PyObject_CallOneArg(((PyObject *)(&PyString_Type)), __pyx_t_2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 249, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    if (unlikely(__Pyx_ListComp_Append(__pyx_t_1, (PyObject*)__pyx_t_3))) __PYX_ERR(0, 249, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  }
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  __pyx_t_9 = __Pyx_PyList_Extend(__pyx_v_response, __pyx_t_1); if (unlikely(__pyx_t_9 == ((int)-1))) __PYX_ERR(0, 249, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "pymarketcap/core.pyx":250
 *         response = [currency.upper() for currency in currencies]
 *         response.extend([str(currency["symbol"]) for currency in self.ticker()])
 *         return sorted(response)             # <<<<<<<<<<<<<<
 * 
 *     @property
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_6 = PySequence_List(__pyx_v_response); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 250, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_1 = ((PyObject*)__pyx_t_6);
  __pyx_t_6 = 0;
  __pyx_t_9 = PyList_Sort(__pyx_t_1); if (unlikely(__pyx_t_9 == ((int)-1))) __PYX_ERR(0, 250, __pyx_L1_error)
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "pymarketcap/core.pyx":236
 *             return self._currencies_to_convert
 * 
 *     cpdef __currencies_to_convert(self):             # <<<<<<<<<<<<<<
 *         """Internal function for get currencies from and to convert
 *             values in convert() method. Don't use this, but cached
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_AddTraceback("pymarketcap.core.Pymarketcap.__currencies_to_convert", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_res);
  __Pyx_XDECREF(__pyx_v_currencies);
  __Pyx_XDECREF(__pyx_v_response);
  __Pyx_XDECREF(__pyx_v_currency);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_11pymarketcap_4core_11Pymarketcap_11__currencies_to_convert(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_11pymarketcap_4core_11Pymarketcap_10__currencies_to_convert[] = "Internal function for get currencies from and to convert\n            values in convert() method. Don't use this, but cached\n            :attr:`~pymarketcap.core.Pymarketcap.currencies_to_convert`\n            instance attribute instead.\n\n        Returns (list):\n            All currencies that could be passed to\n            :meth:`~pymarketcap.core.Pymarketcap.convert`.\n        ";
static PyObject *__pyx_pw_11pymarketcap_4core_11Pymarketcap_11__currencies_to_convert(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__currencies_to_convert (wrapper)", 0);
  __pyx_r = __pyx_pf_11pymarketcap_4core_11Pymarketcap_10__currencies_to_convert(((struct __pyx_obj_11pymarketcap_4core_Pymarketcap *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_11pymarketcap_4core_11Pymarketcap_10__currencies_to_convert(struct __pyx_obj_11pymarketcap_4core_Pymarketcap *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__currencies_to_convert", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_11pymarketcap_4core_11Pymarketcap___currencies_to_convert(__pyx_v_self, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 236, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pymarketcap.core.Pymarketcap.__currencies_to_convert", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pymarketcap/core.pyx":253
 * 
 *     @property
 *     def exchange_names(self):             # <<<<<<<<<<<<<<
 *         """Get all exchange formatted names provided by coinmarketcap."""
 *         res = self._exchange_names
 */

/* Python wrapper */
static PyObject *__pyx_pw_11pymarketcap_4core_11Pymarketcap_14exchange_names_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_11pymarketcap_4core_11Pymarketcap_14exchange_names_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_11pymarketcap_4core_11Pymarketcap_14exchange_names___get__(((struct __pyx_obj_11pymarketcap_4core_Pymarketcap *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_11pymarketcap_4core_11Pymarketcap_14exchange_names___get__(struct __pyx_obj_11pymarketcap_4core_Pymarketcap *__pyx_v_self) {
  PyObject *__pyx_v_res = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_t_6;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "pymarketcap/core.pyx":255
 *     def exchange_names(self):
 *         """Get all exchange formatted names provided by coinmarketcap."""
 *         res = self._exchange_names             # <<<<<<<<<<<<<<
 *         if res:
 *             return res
 */
  __pyx_t_1 = __pyx_v_self->_exchange_names;
  __Pyx_INCREF(__pyx_t_1);
  __pyx_v_res = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "pymarketcap/core.pyx":256
 *         """Get all exchange formatted names provided by coinmarketcap."""
 *         res = self._exchange_names
 *         if res:             # <<<<<<<<<<<<<<
 *             return res
 *         else:
 */
  __pyx_t_2 = (__pyx_v_res != Py_None)&&(PyList_GET_SIZE(__pyx_v_res) != 0);
  if (__pyx_t_2) {

    /* "pymarketcap/core.pyx":257
 *         res = self._exchange_names
 *         if res:
 *             return res             # <<<<<<<<<<<<<<
 *         else:
 *             self._exchange_names = sorted(
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_v_res);
    __pyx_r = __pyx_v_res;
    goto __pyx_L0;

    /* "pymarketcap/core.pyx":256
 *         """Get all exchange formatted names provided by coinmarketcap."""
 *         res = self._exchange_names
 *         if res:             # <<<<<<<<<<<<<<
 *             return res
 *         else:
 */
  }

  /* "pymarketcap/core.pyx":259
 *             return res
 *         else:
 *             self._exchange_names = sorted(             # <<<<<<<<<<<<<<
 *                 list(self.__exchange_names_slugs().keys())
 *             )
 */
  /*else*/ {

    /* "pymarketcap/core.pyx":260
 *         else:
 *             self._exchange_names = sorted(
 *                 list(self.__exchange_names_slugs().keys())             # <<<<<<<<<<<<<<
 *             )
 *             return self._exchange_names
 */
    __pyx_t_4 = ((struct __pyx_vtabstruct_11pymarketcap_4core_Pymarketcap *)__pyx_v_self->__pyx_vtab)->__pyx___exchange_names_slugs(__pyx_v_self, 0); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 260, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_n_s_keys); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 260, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_t_4 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_5))) {
      __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_5);
      if (likely(__pyx_t_4)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_5);
        __Pyx_INCREF(__pyx_t_4);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_5, function);
      }
    }
    if (__pyx_t_4) {
      __pyx_t_3 = __Pyx_PyObject_CallOneArg(__pyx_t_5, __pyx_t_4); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 260, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    } else {
      __pyx_t_3 = __Pyx_PyObject_CallNoArg(__pyx_t_5); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 260, __pyx_L1_error)
    }
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __pyx_t_5 = PySequence_List(__pyx_t_3); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 260, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "pymarketcap/core.pyx":259
 *             return res
 *         else:
 *             self._exchange_names = sorted(             # <<<<<<<<<<<<<<
 *                 list(self.__exchange_names_slugs().keys())
 *             )
 */
    __pyx_t_3 = PySequence_List(__pyx_t_5); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 259, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __pyx_t_1 = ((PyObject*)__pyx_t_3);
    __pyx_t_3 = 0;
    __pyx_t_6 = PyList_Sort(__pyx_t_1); if (unlikely(__pyx_t_6 == ((int)-1))) __PYX_ERR(0, 259, __pyx_L1_error)
    __Pyx_GIVEREF(__pyx_t_1);
    __Pyx_GOTREF(__pyx_v_self->_exchange_names);
    __Pyx_DECREF(__pyx_v_self->_exchange_names);
    __pyx_v_self->_exchange_names = ((PyObject*)__pyx_t_1);
    __pyx_t_1 = 0;

    /* "pymarketcap/core.pyx":262
 *                 list(self.__exchange_names_slugs().keys())
 *             )
 *             return self._exchange_names             # <<<<<<<<<<<<<<
 * 
 *     cpdef __exchange_names_slugs(self):
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_v_self->_exchange_names);
    __pyx_r = __pyx_v_self->_exchange_names;
    goto __pyx_L0;
  }

  /* "pymarketcap/core.pyx":253
 * 
 *     @property
 *     def exchange_names(self):             # <<<<<<<<<<<<<<
 *         """Get all exchange formatted names provided by coinmarketcap."""
 *         res = self._exchange_names
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("pymarketcap.core.Pymarketcap.exchange_names.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_res);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pymarketcap/core.pyx":264
 *             return self._exchange_names
 * 
 *     cpdef __exchange_names_slugs(self):             # <<<<<<<<<<<<<<
 *         """Internal function for get all exchange names
 *             available currently in coinmarketcap. Check ``exchange_names``
 */

static PyObject *__pyx_pw_11pymarketcap_4core_11Pymarketcap_13__exchange_names_slugs(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_f_11pymarketcap_4core_11Pymarketcap___exchange_names_slugs(struct __pyx_obj_11pymarketcap_4core_Pymarketcap *__pyx_v_self, int __pyx_skip_dispatch) {
  PyObject *__pyx_v_res = NULL;
  PyObject *__pyx_7genexpr__pyx_v_exc = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  Py_ssize_t __pyx_t_6;
  PyObject *(*__pyx_t_7)(PyObject *);
  __Pyx_RefNannySetupContext("__exchange_names_slugs", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0)) {
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_exchange_names_slugs); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 264, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)__pyx_pw_11pymarketcap_4core_11Pymarketcap_13__exchange_names_slugs)) {
      __Pyx_XDECREF(__pyx_r);
      __Pyx_INCREF(__pyx_t_1);
      __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
      if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
        __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
        if (likely(__pyx_t_4)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
          __Pyx_INCREF(__pyx_t_4);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_3, function);
        }
      }
      if (__pyx_t_4) {
        __pyx_t_2 = __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_4); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 264, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      } else {
        __pyx_t_2 = __Pyx_PyObject_CallNoArg(__pyx_t_3); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 264, __pyx_L1_error)
      }
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __pyx_r = __pyx_t_2;
      __pyx_t_2 = 0;
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      goto __pyx_L0;
    }
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  }

  /* "pymarketcap/core.pyx":272
 *             All exchanges names formatted in coinmarketcap.
 *         """
 *         res = self._get(             # <<<<<<<<<<<<<<
 *             b"https://s2.coinmarketcap.com/generated/search/quick_search_exchanges.json"
 *         )
 */
  __pyx_t_1 = ((struct __pyx_vtabstruct_11pymarketcap_4core_Pymarketcap *)__pyx_v_self->__pyx_vtab)->_get(__pyx_v_self, ((char *)"https://s2.coinmarketcap.com/generated/search/quick_search_exchanges.json")); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 272, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_res = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "pymarketcap/core.pyx":275
 *             b"https://s2.coinmarketcap.com/generated/search/quick_search_exchanges.json"
 *         )
 *         return {exc["name"]: exc["slug"] for exc in loads(res)}             # <<<<<<<<<<<<<<
 * 
 *     @property
 */
  __Pyx_XDECREF(__pyx_r);
  { /* enter inner scope */
    __pyx_t_1 = PyDict_New(); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 275, __pyx_L5_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_3 = __Pyx_GetModuleGlobalName(__pyx_n_s_loads); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 275, __pyx_L5_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_4 = NULL;
    if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
      __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
      if (likely(__pyx_t_4)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
        __Pyx_INCREF(__pyx_t_4);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_3, function);
      }
    }
    if (!__pyx_t_4) {
      __pyx_t_2 = __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_v_res); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 275, __pyx_L5_error)
      __Pyx_GOTREF(__pyx_t_2);
    } else {
      #if CYTHON_FAST_PYCALL
      if (PyFunction_Check(__pyx_t_3)) {
        PyObject *__pyx_temp[2] = {__pyx_t_4, __pyx_v_res};
        __pyx_t_2 = __Pyx_PyFunction_FastCall(__pyx_t_3, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 275, __pyx_L5_error)
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        __Pyx_GOTREF(__pyx_t_2);
      } else
      #endif
      #if CYTHON_FAST_PYCCALL
      if (__Pyx_PyFastCFunction_Check(__pyx_t_3)) {
        PyObject *__pyx_temp[2] = {__pyx_t_4, __pyx_v_res};
        __pyx_t_2 = __Pyx_PyCFunction_FastCall(__pyx_t_3, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 275, __pyx_L5_error)
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        __Pyx_GOTREF(__pyx_t_2);
      } else
      #endif
      {
        __pyx_t_5 = PyTuple_New(1+1); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 275, __pyx_L5_error)
        __Pyx_GOTREF(__pyx_t_5);
        __Pyx_GIVEREF(__pyx_t_4); PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_4); __pyx_t_4 = NULL;
        __Pyx_INCREF(__pyx_v_res);
        __Pyx_GIVEREF(__pyx_v_res);
        PyTuple_SET_ITEM(__pyx_t_5, 0+1, __pyx_v_res);
        __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_t_5, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 275, __pyx_L5_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      }
    }
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (likely(PyList_CheckExact(__pyx_t_2)) || PyTuple_CheckExact(__pyx_t_2)) {
      __pyx_t_3 = __pyx_t_2; __Pyx_INCREF(__pyx_t_3); __pyx_t_6 = 0;
      __pyx_t_7 = NULL;
    } else {
      __pyx_t_6 = -1; __pyx_t_3 = PyObject_GetIter(__pyx_t_2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 275, __pyx_L5_error)
      __Pyx_GOTREF(__pyx_t_3);
      __pyx_t_7 = Py_TYPE(__pyx_t_3)->tp_iternext; if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 275, __pyx_L5_error)
    }
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    for (;;) {
      if (likely(!__pyx_t_7)) {
        if (likely(PyList_CheckExact(__pyx_t_3))) {
          if (__pyx_t_6 >= PyList_GET_SIZE(__pyx_t_3)) break;
          #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
          __pyx_t_2 = PyList_GET_ITEM(__pyx_t_3, __pyx_t_6); __Pyx_INCREF(__pyx_t_2); __pyx_t_6++; if (unlikely(0 < 0)) __PYX_ERR(0, 275, __pyx_L5_error)
          #else
          __pyx_t_2 = PySequence_ITEM(__pyx_t_3, __pyx_t_6); __pyx_t_6++; if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 275, __pyx_L5_error)
          __Pyx_GOTREF(__pyx_t_2);
          #endif
        } else {
          if (__pyx_t_6 >= PyTuple_GET_SIZE(__pyx_t_3)) break;
          #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
          __pyx_t_2 = PyTuple_GET_ITEM(__pyx_t_3, __pyx_t_6); __Pyx_INCREF(__pyx_t_2); __pyx_t_6++; if (unlikely(0 < 0)) __PYX_ERR(0, 275, __pyx_L5_error)
          #else
          __pyx_t_2 = PySequence_ITEM(__pyx_t_3, __pyx_t_6); __pyx_t_6++; if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 275, __pyx_L5_error)
          __Pyx_GOTREF(__pyx_t_2);
          #endif
        }
      } else {
        __pyx_t_2 = __pyx_t_7(__pyx_t_3);
        if (unlikely(!__pyx_t_2)) {
          PyObject* exc_type = PyErr_Occurred();
          if (exc_type) {
            if (likely(__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
            else __PYX_ERR(0, 275, __pyx_L5_error)
          }
          break;
        }
        __Pyx_GOTREF(__pyx_t_2);
      }
      __Pyx_XDECREF_SET(__pyx_7genexpr__pyx_v_exc, __pyx_t_2);
      __pyx_t_2 = 0;
      __pyx_t_2 = __Pyx_PyObject_Dict_GetItem(__pyx_7genexpr__pyx_v_exc, __pyx_n_s_name); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 275, __pyx_L5_error)
      __Pyx_GOTREF(__pyx_t_2);
      __pyx_t_5 = __Pyx_PyObject_Dict_GetItem(__pyx_7genexpr__pyx_v_exc, __pyx_n_s_slug); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 275, __pyx_L5_error)
      __Pyx_GOTREF(__pyx_t_5);
      if (unlikely(PyDict_SetItem(__pyx_t_1, (PyObject*)__pyx_t_2, (PyObject*)__pyx_t_5))) __PYX_ERR(0, 275, __pyx_L5_error)
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    }
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_XDECREF(__pyx_7genexpr__pyx_v_exc); __pyx_7genexpr__pyx_v_exc = 0;
    goto __pyx_L8_exit_scope;
    __pyx_L5_error:;
    __Pyx_XDECREF(__pyx_7genexpr__pyx_v_exc); __pyx_7genexpr__pyx_v_exc = 0;
    goto __pyx_L1_error;
    __pyx_L8_exit_scope:;
  } /* exit inner scope */
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "pymarketcap/core.pyx":264
 *             return self._exchange_names
 * 
 *     cpdef __exchange_names_slugs(self):             # <<<<<<<<<<<<<<
 *         """Internal function for get all exchange names
 *             available currently in coinmarketcap. Check ``exchange_names``
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("pymarketcap.core.Pymarketcap.__exchange_names_slugs", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_res);
  __Pyx_XDECREF(__pyx_7genexpr__pyx_v_exc);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_11pymarketcap_4core_11Pymarketcap_13__exchange_names_slugs(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_11pymarketcap_4core_11Pymarketcap_12__exchange_names_slugs[] = "Internal function for get all exchange names\n            available currently in coinmarketcap. Check ``exchange_names``\n            instance attribute for the cached method counterpart.\n\n        Returns (list):\n            All exchanges names formatted in coinmarketcap.\n        ";
static PyObject *__pyx_pw_11pymarketcap_4core_11Pymarketcap_13__exchange_names_slugs(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__exchange_names_slugs (wrapper)", 0);
  __pyx_r = __pyx_pf_11pymarketcap_4core_11Pymarketcap_12__exchange_names_slugs(((struct __pyx_obj_11pymarketcap_4core_Pymarketcap *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_11pymarketcap_4core_11Pymarketcap_12__exchange_names_slugs(struct __pyx_obj_11pymarketcap_4core_Pymarketcap *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__exchange_names_slugs", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_11pymarketcap_4core_11Pymarketcap___exchange_names_slugs(__pyx_v_self, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 264, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pymarketcap.core.Pymarketcap.__exchange_names_slugs", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pymarketcap/core.pyx":278
 * 
 *     @property
 *     def exchange_slugs(self):             # <<<<<<<<<<<<<<
 *         """Get all exchange raw names provided by coinmarketcap."""
 *         res = self._exchange_slugs
 */

/* Python wrapper */
static PyObject *__pyx_pw_11pymarketcap_4core_11Pymarketcap_14exchange_slugs_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_11pymarketcap_4core_11Pymarketcap_14exchange_slugs_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_11pymarketcap_4core_11Pymarketcap_14exchange_slugs___get__(((struct __pyx_obj_11pymarketcap_4core_Pymarketcap *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_11pymarketcap_4core_11Pymarketcap_14exchange_slugs___get__(struct __pyx_obj_11pymarketcap_4core_Pymarketcap *__pyx_v_self) {
  PyObject *__pyx_v_res = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_t_6;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "pymarketcap/core.pyx":280
 *     def exchange_slugs(self):
 *         """Get all exchange raw names provided by coinmarketcap."""
 *         res = self._exchange_slugs             # <<<<<<<<<<<<<<
 *         if res:
 *             return res
 */
  __pyx_t_1 = __pyx_v_self->_exchange_slugs;
  __Pyx_INCREF(__pyx_t_1);
  __pyx_v_res = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "pymarketcap/core.pyx":281
 *         """Get all exchange raw names provided by coinmarketcap."""
 *         res = self._exchange_slugs
 *         if res:             # <<<<<<<<<<<<<<
 *             return res
 *         else:
 */
  __pyx_t_2 = (__pyx_v_res != Py_None)&&(PyList_GET_SIZE(__pyx_v_res) != 0);
  if (__pyx_t_2) {

    /* "pymarketcap/core.pyx":282
 *         res = self._exchange_slugs
 *         if res:
 *             return res             # <<<<<<<<<<<<<<
 *         else:
 *             self._exchange_slugs = sorted(
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_v_res);
    __pyx_r = __pyx_v_res;
    goto __pyx_L0;

    /* "pymarketcap/core.pyx":281
 *         """Get all exchange raw names provided by coinmarketcap."""
 *         res = self._exchange_slugs
 *         if res:             # <<<<<<<<<<<<<<
 *             return res
 *         else:
 */
  }

  /* "pymarketcap/core.pyx":284
 *             return res
 *         else:
 *             self._exchange_slugs = sorted(             # <<<<<<<<<<<<<<
 *                 list(self.__exchange_names_slugs().values())
 *             )
 */
  /*else*/ {

    /* "pymarketcap/core.pyx":285
 *         else:
 *             self._exchange_slugs = sorted(
 *                 list(self.__exchange_names_slugs().values())             # <<<<<<<<<<<<<<
 *             )
 *             return self._exchange_slugs
 */
    __pyx_t_4 = ((struct __pyx_vtabstruct_11pymarketcap_4core_Pymarketcap *)__pyx_v_self->__pyx_vtab)->__pyx___exchange_names_slugs(__pyx_v_self, 0); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 285, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_n_s_values); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 285, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_t_4 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_5))) {
      __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_5);
      if (likely(__pyx_t_4)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_5);
        __Pyx_INCREF(__pyx_t_4);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_5, function);
      }
    }
    if (__pyx_t_4) {
      __pyx_t_3 = __Pyx_PyObject_CallOneArg(__pyx_t_5, __pyx_t_4); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 285, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    } else {
      __pyx_t_3 = __Pyx_PyObject_CallNoArg(__pyx_t_5); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 285, __pyx_L1_error)
    }
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __pyx_t_5 = PySequence_List(__pyx_t_3); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 285, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "pymarketcap/core.pyx":284
 *             return res
 *         else:
 *             self._exchange_slugs = sorted(             # <<<<<<<<<<<<<<
 *                 list(self.__exchange_names_slugs().values())
 *             )
 */
    __pyx_t_3 = PySequence_List(__pyx_t_5); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 284, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __pyx_t_1 = ((PyObject*)__pyx_t_3);
    __pyx_t_3 = 0;
    __pyx_t_6 = PyList_Sort(__pyx_t_1); if (unlikely(__pyx_t_6 == ((int)-1))) __PYX_ERR(0, 284, __pyx_L1_error)
    __Pyx_GIVEREF(__pyx_t_1);
    __Pyx_GOTREF(__pyx_v_self->_exchange_slugs);
    __Pyx_DECREF(__pyx_v_self->_exchange_slugs);
    __pyx_v_self->_exchange_slugs = ((PyObject*)__pyx_t_1);
    __pyx_t_1 = 0;

    /* "pymarketcap/core.pyx":287
 *                 list(self.__exchange_names_slugs().values())
 *             )
 *             return self._exchange_slugs             # <<<<<<<<<<<<<<
 * 
 *     @property
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_v_self->_exchange_slugs);
    __pyx_r = __pyx_v_self->_exchange_slugs;
    goto __pyx_L0;
  }

  /* "pymarketcap/core.pyx":278
 * 
 *     @property
 *     def exchange_slugs(self):             # <<<<<<<<<<<<<<
 *         """Get all exchange raw names provided by coinmarketcap."""
 *         res = self._exchange_slugs
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("pymarketcap.core.Pymarketcap.exchange_slugs.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_res);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pymarketcap/core.pyx":290
 * 
 *     @property
 *     def total_exchanges(self):             # <<<<<<<<<<<<<<
 *         res = self._total_exchanges
 *         if res:
 */

/* Python wrapper */
static PyObject *__pyx_pw_11pymarketcap_4core_11Pymarketcap_15total_exchanges_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_11pymarketcap_4core_11Pymarketcap_15total_exchanges_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_11pymarketcap_4core_11Pymarketcap_15total_exchanges___get__(((struct __pyx_obj_11pymarketcap_4core_Pymarketcap *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_11pymarketcap_4core_11Pymarketcap_15total_exchanges___get__(struct __pyx_obj_11pymarketcap_4core_Pymarketcap *__pyx_v_self) {
  int __pyx_v_res;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  Py_ssize_t __pyx_t_4;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "pymarketcap/core.pyx":291
 *     @property
 *     def total_exchanges(self):
 *         res = self._total_exchanges             # <<<<<<<<<<<<<<
 *         if res:
 *             return res
 */
  __pyx_t_1 = __pyx_v_self->_total_exchanges;
  __pyx_v_res = __pyx_t_1;

  /* "pymarketcap/core.pyx":292
 *     def total_exchanges(self):
 *         res = self._total_exchanges
 *         if res:             # <<<<<<<<<<<<<<
 *             return res
 *         else:
 */
  __pyx_t_2 = (__pyx_v_res != 0);
  if (__pyx_t_2) {

    /* "pymarketcap/core.pyx":293
 *         res = self._total_exchanges
 *         if res:
 *             return res             # <<<<<<<<<<<<<<
 *         else:
 *             self._total_exchanges = len(self.exchanges())
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_3 = __Pyx_PyInt_From_int(__pyx_v_res); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 293, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_r = __pyx_t_3;
    __pyx_t_3 = 0;
    goto __pyx_L0;

    /* "pymarketcap/core.pyx":292
 *     def total_exchanges(self):
 *         res = self._total_exchanges
 *         if res:             # <<<<<<<<<<<<<<
 *             return res
 *         else:
 */
  }

  /* "pymarketcap/core.pyx":295
 *             return res
 *         else:
 *             self._total_exchanges = len(self.exchanges())             # <<<<<<<<<<<<<<
 *             return self._total_exchanges
 * 
 */
  /*else*/ {
    __pyx_t_3 = ((struct __pyx_vtabstruct_11pymarketcap_4core_Pymarketcap *)__pyx_v_self->__pyx_vtab)->exchanges(__pyx_v_self, 0, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 295, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_4 = PyObject_Length(__pyx_t_3); if (unlikely(__pyx_t_4 == ((Py_ssize_t)-1))) __PYX_ERR(0, 295, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_v_self->_total_exchanges = __pyx_t_4;

    /* "pymarketcap/core.pyx":296
 *         else:
 *             self._total_exchanges = len(self.exchanges())
 *             return self._total_exchanges             # <<<<<<<<<<<<<<
 * 
 *     @property
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_3 = __Pyx_PyInt_From_int(__pyx_v_self->_total_exchanges); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 296, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_r = __pyx_t_3;
    __pyx_t_3 = 0;
    goto __pyx_L0;
  }

  /* "pymarketcap/core.pyx":290
 * 
 *     @property
 *     def total_exchanges(self):             # <<<<<<<<<<<<<<
 *         res = self._total_exchanges
 *         if res:
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("pymarketcap.core.Pymarketcap.total_exchanges.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pymarketcap/core.pyx":299
 * 
 *     @property
 *     def converter_cache(self):             # <<<<<<<<<<<<<<
 *         res = self._converter_cache
 *         if res:
 */

/* Python wrapper */
static PyObject *__pyx_pw_11pymarketcap_4core_11Pymarketcap_15converter_cache_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_11pymarketcap_4core_11Pymarketcap_15converter_cache_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_11pymarketcap_4core_11Pymarketcap_15converter_cache___get__(((struct __pyx_obj_11pymarketcap_4core_Pymarketcap *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_11pymarketcap_4core_11Pymarketcap_15converter_cache___get__(struct __pyx_obj_11pymarketcap_4core_Pymarketcap *__pyx_v_self) {
  PyObject *__pyx_v_res = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "pymarketcap/core.pyx":300
 *     @property
 *     def converter_cache(self):
 *         res = self._converter_cache             # <<<<<<<<<<<<<<
 *         if res:
 *             return res
 */
  __pyx_t_1 = __pyx_v_self->_converter_cache;
  __Pyx_INCREF(__pyx_t_1);
  __pyx_v_res = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "pymarketcap/core.pyx":301
 *     def converter_cache(self):
 *         res = self._converter_cache
 *         if res:             # <<<<<<<<<<<<<<
 *             return res
 *         else:
 */
  __pyx_t_2 = (__pyx_v_res != Py_None)&&(PyList_GET_SIZE(__pyx_v_res) != 0);
  if (__pyx_t_2) {

    /* "pymarketcap/core.pyx":302
 *         res = self._converter_cache
 *         if res:
 *             return res             # <<<<<<<<<<<<<<
 *         else:
 *             self._converter_cache = [self.currency_exchange_rates, time()]
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_v_res);
    __pyx_r = __pyx_v_res;
    goto __pyx_L0;

    /* "pymarketcap/core.pyx":301
 *     def converter_cache(self):
 *         res = self._converter_cache
 *         if res:             # <<<<<<<<<<<<<<
 *             return res
 *         else:
 */
  }

  /* "pymarketcap/core.pyx":304
 *             return res
 *         else:
 *             self._converter_cache = [self.currency_exchange_rates, time()]             # <<<<<<<<<<<<<<
 *             return self._converter_cache
 * 
 */
  /*else*/ {
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_currency_exchange_rates); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 304, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_4 = __Pyx_GetModuleGlobalName(__pyx_n_s_time); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 304, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_5 = NULL;
    if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_4))) {
      __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_4);
      if (likely(__pyx_t_5)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
        __Pyx_INCREF(__pyx_t_5);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_4, function);
      }
    }
    if (__pyx_t_5) {
      __pyx_t_3 = __Pyx_PyObject_CallOneArg(__pyx_t_4, __pyx_t_5); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 304, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    } else {
      __pyx_t_3 = __Pyx_PyObject_CallNoArg(__pyx_t_4); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 304, __pyx_L1_error)
    }
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_t_4 = PyList_New(2); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 304, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_GIVEREF(__pyx_t_1);
    PyList_SET_ITEM(__pyx_t_4, 0, __pyx_t_1);
    __Pyx_GIVEREF(__pyx_t_3);
    PyList_SET_ITEM(__pyx_t_4, 1, __pyx_t_3);
    __pyx_t_1 = 0;
    __pyx_t_3 = 0;
    __Pyx_GIVEREF(__pyx_t_4);
    __Pyx_GOTREF(__pyx_v_self->_converter_cache);
    __Pyx_DECREF(__pyx_v_self->_converter_cache);
    __pyx_v_self->_converter_cache = ((PyObject*)__pyx_t_4);
    __pyx_t_4 = 0;

    /* "pymarketcap/core.pyx":305
 *         else:
 *             self._converter_cache = [self.currency_exchange_rates, time()]
 *             return self._converter_cache             # <<<<<<<<<<<<<<
 * 
 *     cdef _get(self, char *url):
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_v_self->_converter_cache);
    __pyx_r = __pyx_v_self->_converter_cache;
    goto __pyx_L0;
  }

  /* "pymarketcap/core.pyx":299
 * 
 *     @property
 *     def converter_cache(self):             # <<<<<<<<<<<<<<
 *         res = self._converter_cache
 *         if res:
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("pymarketcap.core.Pymarketcap.converter_cache.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_res);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pymarketcap/core.pyx":307
 *             return self._converter_cache
 * 
 *     cdef _get(self, char *url):             # <<<<<<<<<<<<<<
 *         """Internal function to make and HTTP GET request
 *         using the curl Cython bridge to C library or urllib
 */

static PyObject *__pyx_f_11pymarketcap_4core_11Pymarketcap__get(struct __pyx_obj_11pymarketcap_4core_Pymarketcap *__pyx_v_self, char *__pyx_v_url) {
  int __pyx_v_status;
  PyObject *__pyx_v_req = NULL;
  PyObject *__pyx_v_msg = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_t_7;
  PyObject *__pyx_t_8 = NULL;
  int __pyx_t_9;
  int __pyx_t_10;
  __Pyx_RefNannySetupContext("_get", 0);

  /* "pymarketcap/core.pyx":312
 *         standard library, depending on the installation."""
 *         cdef int status
 *         req = get_to_memory(<char *>url, self.timeout, <bint>self.debug)             # <<<<<<<<<<<<<<
 *         status = req.status_code
 *         if status == 200:
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_get_to_memory); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 312, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyBytes_FromString(((char *)__pyx_v_url)); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 312, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = __Pyx_PyInt_From_long(__pyx_v_self->timeout); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 312, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_5 = __Pyx_PyBool_FromLong((__pyx_v_self->debug != 0)); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 312, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_6 = NULL;
  __pyx_t_7 = 0;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_6)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_6);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_7 = 1;
    }
  }
  #if CYTHON_FAST_PYCALL
  if (PyFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[4] = {__pyx_t_6, __pyx_t_3, __pyx_t_4, __pyx_t_5};
    __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_7, 3+__pyx_t_7); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 312, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  } else
  #endif
  #if CYTHON_FAST_PYCCALL
  if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[4] = {__pyx_t_6, __pyx_t_3, __pyx_t_4, __pyx_t_5};
    __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_7, 3+__pyx_t_7); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 312, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  } else
  #endif
  {
    __pyx_t_8 = PyTuple_New(3+__pyx_t_7); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 312, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    if (__pyx_t_6) {
      __Pyx_GIVEREF(__pyx_t_6); PyTuple_SET_ITEM(__pyx_t_8, 0, __pyx_t_6); __pyx_t_6 = NULL;
    }
    __Pyx_GIVEREF(__pyx_t_3);
    PyTuple_SET_ITEM(__pyx_t_8, 0+__pyx_t_7, __pyx_t_3);
    __Pyx_GIVEREF(__pyx_t_4);
    PyTuple_SET_ITEM(__pyx_t_8, 1+__pyx_t_7, __pyx_t_4);
    __Pyx_GIVEREF(__pyx_t_5);
    PyTuple_SET_ITEM(__pyx_t_8, 2+__pyx_t_7, __pyx_t_5);
    __pyx_t_3 = 0;
    __pyx_t_4 = 0;
    __pyx_t_5 = 0;
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_8, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 312, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v_req = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "pymarketcap/core.pyx":313
 *         cdef int status
 *         req = get_to_memory(<char *>url, self.timeout, <bint>self.debug)
 *         status = req.status_code             # <<<<<<<<<<<<<<
 *         if status == 200:
 *             return req.text.decode()
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_req, __pyx_n_s_status_code); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 313, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_7 = __Pyx_PyInt_As_int(__pyx_t_1); if (unlikely((__pyx_t_7 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 313, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_status = __pyx_t_7;

  /* "pymarketcap/core.pyx":314
 *         req = get_to_memory(<char *>url, self.timeout, <bint>self.debug)
 *         status = req.status_code
 *         if status == 200:             # <<<<<<<<<<<<<<
 *             return req.text.decode()
 *         else:
 */
  __pyx_t_9 = ((__pyx_v_status == 0xC8) != 0);
  if (__pyx_t_9) {

    /* "pymarketcap/core.pyx":315
 *         status = req.status_code
 *         if status == 200:
 *             return req.text.decode()             # <<<<<<<<<<<<<<
 *         else:
 *             msg = "Status code -> %d | Url -> %s" % (status, url.decode())
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_req, __pyx_n_s_text); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 315, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_decode); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 315, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_2 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_8))) {
      __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_8);
      if (likely(__pyx_t_2)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_8);
        __Pyx_INCREF(__pyx_t_2);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_8, function);
      }
    }
    if (__pyx_t_2) {
      __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_8, __pyx_t_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 315, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    } else {
      __pyx_t_1 = __Pyx_PyObject_CallNoArg(__pyx_t_8); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 315, __pyx_L1_error)
    }
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    __pyx_r = __pyx_t_1;
    __pyx_t_1 = 0;
    goto __pyx_L0;

    /* "pymarketcap/core.pyx":314
 *         req = get_to_memory(<char *>url, self.timeout, <bint>self.debug)
 *         status = req.status_code
 *         if status == 200:             # <<<<<<<<<<<<<<
 *             return req.text.decode()
 *         else:
 */
  }

  /* "pymarketcap/core.pyx":317
 *             return req.text.decode()
 *         else:
 *             msg = "Status code -> %d | Url -> %s" % (status, url.decode())             # <<<<<<<<<<<<<<
 *             if status in http_error_numbers:
 *                 raise http_errors_map[str(status)](msg)
 */
  /*else*/ {
    __pyx_t_1 = __Pyx_PyInt_From_int(__pyx_v_status); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 317, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_8 = __Pyx_decode_c_string(__pyx_v_url, 0, strlen(__pyx_v_url), NULL, NULL, NULL); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 317, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __pyx_t_2 = PyTuple_New(2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 317, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_GIVEREF(__pyx_t_1);
    PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_t_1);
    __Pyx_GIVEREF(__pyx_t_8);
    PyTuple_SET_ITEM(__pyx_t_2, 1, __pyx_t_8);
    __pyx_t_1 = 0;
    __pyx_t_8 = 0;
    __pyx_t_8 = __Pyx_PyString_Format(__pyx_kp_s_Status_code_d_Url_s, __pyx_t_2); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 317, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_v_msg = ((PyObject*)__pyx_t_8);
    __pyx_t_8 = 0;

    /* "pymarketcap/core.pyx":318
 *         else:
 *             msg = "Status code -> %d | Url -> %s" % (status, url.decode())
 *             if status in http_error_numbers:             # <<<<<<<<<<<<<<
 *                 raise http_errors_map[str(status)](msg)
 *             else:
 */
    __pyx_t_8 = __Pyx_PyInt_From_int(__pyx_v_status); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 318, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_http_error_numbers); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 318, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_9 = (__Pyx_PySequence_ContainsTF(__pyx_t_8, __pyx_t_2, Py_EQ)); if (unlikely(__pyx_t_9 < 0)) __PYX_ERR(0, 318, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_10 = (__pyx_t_9 != 0);
    if (unlikely(__pyx_t_10)) {

      /* "pymarketcap/core.pyx":319
 *             msg = "Status code -> %d | Url -> %s" % (status, url.decode())
 *             if status in http_error_numbers:
 *                 raise http_errors_map[str(status)](msg)             # <<<<<<<<<<<<<<
 *             else:
 *                 print("DEBUG: ")
 */
      __pyx_t_8 = __Pyx_GetModuleGlobalName(__pyx_n_s_http_errors_map); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 319, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_8);
      __pyx_t_1 = __Pyx_PyInt_From_int(__pyx_v_status); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 319, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __pyx_t_5 = __Pyx_PyObject_CallOneArg(((PyObject *)(&PyString_Type)), __pyx_t_1); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 319, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __pyx_t_1 = __Pyx_PyObject_GetItem(__pyx_t_8, __pyx_t_5); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 319, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      __pyx_t_5 = NULL;
      if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_1))) {
        __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_1);
        if (likely(__pyx_t_5)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_1);
          __Pyx_INCREF(__pyx_t_5);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_1, function);
        }
      }
      if (!__pyx_t_5) {
        __pyx_t_2 = __Pyx_PyObject_CallOneArg(__pyx_t_1, __pyx_v_msg); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 319, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
      } else {
        #if CYTHON_FAST_PYCALL
        if (PyFunction_Check(__pyx_t_1)) {
          PyObject *__pyx_temp[2] = {__pyx_t_5, __pyx_v_msg};
          __pyx_t_2 = __Pyx_PyFunction_FastCall(__pyx_t_1, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 319, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
          __Pyx_GOTREF(__pyx_t_2);
        } else
        #endif
        #if CYTHON_FAST_PYCCALL
        if (__Pyx_PyFastCFunction_Check(__pyx_t_1)) {
          PyObject *__pyx_temp[2] = {__pyx_t_5, __pyx_v_msg};
          __pyx_t_2 = __Pyx_PyCFunction_FastCall(__pyx_t_1, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 319, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
          __Pyx_GOTREF(__pyx_t_2);
        } else
        #endif
        {
          __pyx_t_8 = PyTuple_New(1+1); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 319, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_8);
          __Pyx_GIVEREF(__pyx_t_5); PyTuple_SET_ITEM(__pyx_t_8, 0, __pyx_t_5); __pyx_t_5 = NULL;
          __Pyx_INCREF(__pyx_v_msg);
          __Pyx_GIVEREF(__pyx_v_msg);
          PyTuple_SET_ITEM(__pyx_t_8, 0+1, __pyx_v_msg);
          __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_t_8, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 319, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_2);
          __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
        }
      }
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __Pyx_Raise(__pyx_t_2, 0, 0, 0);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __PYX_ERR(0, 319, __pyx_L1_error)

      /* "pymarketcap/core.pyx":318
 *         else:
 *             msg = "Status code -> %d | Url -> %s" % (status, url.decode())
 *             if status in http_error_numbers:             # <<<<<<<<<<<<<<
 *                 raise http_errors_map[str(status)](msg)
 *             else:
 */
    }

    /* "pymarketcap/core.pyx":321
 *                 raise http_errors_map[str(status)](msg)
 *             else:
 *                 print("DEBUG: ")             # <<<<<<<<<<<<<<
 *                 print(req.text)
 *                 print(req.url)
 */
    /*else*/ {
      if (__Pyx_PrintOne(0, __pyx_kp_s_DEBUG) < 0) __PYX_ERR(0, 321, __pyx_L1_error)

      /* "pymarketcap/core.pyx":322
 *             else:
 *                 print("DEBUG: ")
 *                 print(req.text)             # <<<<<<<<<<<<<<
 *                 print(req.url)
 *                 raise CoinmarketcapHTTPError(msg)
 */
      __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_req, __pyx_n_s_text); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 322, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      if (__Pyx_PrintOne(0, __pyx_t_2) < 0) __PYX_ERR(0, 322, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

      /* "pymarketcap/core.pyx":323
 *                 print("DEBUG: ")
 *                 print(req.text)
 *                 print(req.url)             # <<<<<<<<<<<<<<
 *                 raise CoinmarketcapHTTPError(msg)
 * 
 */
      __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_req, __pyx_n_s_url); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 323, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      if (__Pyx_PrintOne(0, __pyx_t_2) < 0) __PYX_ERR(0, 323, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

      /* "pymarketcap/core.pyx":324
 *                 print(req.text)
 *                 print(req.url)
 *                 raise CoinmarketcapHTTPError(msg)             # <<<<<<<<<<<<<<
 * 
 *     # ====================================================================
 */
      __pyx_t_1 = __Pyx_GetModuleGlobalName(__pyx_n_s_CoinmarketcapHTTPError); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 324, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __pyx_t_8 = NULL;
      if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_1))) {
        __pyx_t_8 = PyMethod_GET_SELF(__pyx_t_1);
        if (likely(__pyx_t_8)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_1);
          __Pyx_INCREF(__pyx_t_8);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_1, function);
        }
      }
      if (!__pyx_t_8) {
        __pyx_t_2 = __Pyx_PyObject_CallOneArg(__pyx_t_1, __pyx_v_msg); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 324, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
      } else {
        #if CYTHON_FAST_PYCALL
        if (PyFunction_Check(__pyx_t_1)) {
          PyObject *__pyx_temp[2] = {__pyx_t_8, __pyx_v_msg};
          __pyx_t_2 = __Pyx_PyFunction_FastCall(__pyx_t_1, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 324, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
          __Pyx_GOTREF(__pyx_t_2);
        } else
        #endif
        #if CYTHON_FAST_PYCCALL
        if (__Pyx_PyFastCFunction_Check(__pyx_t_1)) {
          PyObject *__pyx_temp[2] = {__pyx_t_8, __pyx_v_msg};
          __pyx_t_2 = __Pyx_PyCFunction_FastCall(__pyx_t_1, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 324, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
          __Pyx_GOTREF(__pyx_t_2);
        } else
        #endif
        {
          __pyx_t_5 = PyTuple_New(1+1); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 324, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_5);
          __Pyx_GIVEREF(__pyx_t_8); PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_8); __pyx_t_8 = NULL;
          __Pyx_INCREF(__pyx_v_msg);
          __Pyx_GIVEREF(__pyx_v_msg);
          PyTuple_SET_ITEM(__pyx_t_5, 0+1, __pyx_v_msg);
          __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_t_5, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 324, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_2);
          __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
        }
      }
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __Pyx_Raise(__pyx_t_2, 0, 0, 0);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __PYX_ERR(0, 324, __pyx_L1_error)
    }
  }

  /* "pymarketcap/core.pyx":307
 *             return self._converter_cache
 * 
 *     cdef _get(self, char *url):             # <<<<<<<<<<<<<<
 *         """Internal function to make and HTTP GET request
 *         using the curl Cython bridge to C library or urllib
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_AddTraceback("pymarketcap.core.Pymarketcap._get", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_req);
  __Pyx_XDECREF(__pyx_v_msg);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pymarketcap/core.pyx":330
 *                         #######   PUBLIC API   #######
 * 
 *     cpdef stats(self, convert="USD"):             # <<<<<<<<<<<<<<
 *         """ Get global cryptocurrencies statistics.
 * 
 */

static PyObject *__pyx_pw_11pymarketcap_4core_11Pymarketcap_15stats(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyObject *__pyx_f_11pymarketcap_4core_11Pymarketcap_stats(struct __pyx_obj_11pymarketcap_4core_Pymarketcap *__pyx_v_self, int __pyx_skip_dispatch, struct __pyx_opt_args_11pymarketcap_4core_11Pymarketcap_stats *__pyx_optional_args) {
  PyObject *__pyx_v_convert = ((PyObject *)__pyx_n_s_USD);
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  char *__pyx_t_6;
  __Pyx_RefNannySetupContext("stats", 0);
  if (__pyx_optional_args) {
    if (__pyx_optional_args->__pyx_n > 0) {
      __pyx_v_convert = __pyx_optional_args->convert;
    }
  }
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0)) {
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_stats); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 330, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)__pyx_pw_11pymarketcap_4core_11Pymarketcap_15stats)) {
      __Pyx_XDECREF(__pyx_r);
      __Pyx_INCREF(__pyx_t_1);
      __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
      if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
        __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
        if (likely(__pyx_t_4)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
          __Pyx_INCREF(__pyx_t_4);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_3, function);
        }
      }
      if (!__pyx_t_4) {
        __pyx_t_2 = __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_v_convert); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 330, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
      } else {
        #if CYTHON_FAST_PYCALL
        if (PyFunction_Check(__pyx_t_3)) {
          PyObject *__pyx_temp[2] = {__pyx_t_4, __pyx_v_convert};
          __pyx_t_2 = __Pyx_PyFunction_FastCall(__pyx_t_3, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 330, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
          __Pyx_GOTREF(__pyx_t_2);
        } else
        #endif
        #if CYTHON_FAST_PYCCALL
        if (__Pyx_PyFastCFunction_Check(__pyx_t_3)) {
          PyObject *__pyx_temp[2] = {__pyx_t_4, __pyx_v_convert};
          __pyx_t_2 = __Pyx_PyCFunction_FastCall(__pyx_t_3, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 330, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
          __Pyx_GOTREF(__pyx_t_2);
        } else
        #endif
        {
          __pyx_t_5 = PyTuple_New(1+1); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 330, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_5);
          __Pyx_GIVEREF(__pyx_t_4); PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_4); __pyx_t_4 = NULL;
          __Pyx_INCREF(__pyx_v_convert);
          __Pyx_GIVEREF(__pyx_v_convert);
          PyTuple_SET_ITEM(__pyx_t_5, 0+1, __pyx_v_convert);
          __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_t_5, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 330, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_2);
          __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
        }
      }
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __pyx_r = __pyx_t_2;
      __pyx_t_2 = 0;
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      goto __pyx_L0;
    }
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  }

  /* "pymarketcap/core.pyx":342
 *             Global markets statistics.
 *         """
 *         return loads(self._get(             # <<<<<<<<<<<<<<
 *             b"https://api.coinmarketcap.com/v1/global/?convert=%s" % convert.encode()
 *         ))
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_loads); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 342, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);

  /* "pymarketcap/core.pyx":343
 *         """
 *         return loads(self._get(
 *             b"https://api.coinmarketcap.com/v1/global/?convert=%s" % convert.encode()             # <<<<<<<<<<<<<<
 *         ))
 * 
 */
  __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_v_convert, __pyx_n_s_encode); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 343, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_4 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_5))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_5);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_5);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_5, function);
    }
  }
  if (__pyx_t_4) {
    __pyx_t_3 = __Pyx_PyObject_CallOneArg(__pyx_t_5, __pyx_t_4); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 343, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  } else {
    __pyx_t_3 = __Pyx_PyObject_CallNoArg(__pyx_t_5); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 343, __pyx_L1_error)
  }
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __pyx_t_5 = PyNumber_Remainder(__pyx_kp_b_https_api_coinmarketcap_com_v1_g, __pyx_t_3); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 343, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_6 = __Pyx_PyObject_AsWritableString(__pyx_t_5); if (unlikely((!__pyx_t_6) && PyErr_Occurred())) __PYX_ERR(0, 343, __pyx_L1_error)

  /* "pymarketcap/core.pyx":342
 *             Global markets statistics.
 *         """
 *         return loads(self._get(             # <<<<<<<<<<<<<<
 *             b"https://api.coinmarketcap.com/v1/global/?convert=%s" % convert.encode()
 *         ))
 */
  __pyx_t_3 = ((struct __pyx_vtabstruct_11pymarketcap_4core_Pymarketcap *)__pyx_v_self->__pyx_vtab)->_get(__pyx_v_self, __pyx_t_6); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 342, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __pyx_t_5 = NULL;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_5)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_5);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  if (!__pyx_t_5) {
    __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_3); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 342, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_GOTREF(__pyx_t_1);
  } else {
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(__pyx_t_2)) {
      PyObject *__pyx_temp[2] = {__pyx_t_5, __pyx_t_3};
      __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 342, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    } else
    #endif
    #if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
      PyObject *__pyx_temp[2] = {__pyx_t_5, __pyx_t_3};
      __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 342, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    } else
    #endif
    {
      __pyx_t_4 = PyTuple_New(1+1); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 342, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_GIVEREF(__pyx_t_5); PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_5); __pyx_t_5 = NULL;
      __Pyx_GIVEREF(__pyx_t_3);
      PyTuple_SET_ITEM(__pyx_t_4, 0+1, __pyx_t_3);
      __pyx_t_3 = 0;
      __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_4, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 342, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    }
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "pymarketcap/core.pyx":330
 *                         #######   PUBLIC API   #######
 * 
 *     cpdef stats(self, convert="USD"):             # <<<<<<<<<<<<<<
 *         """ Get global cryptocurrencies statistics.
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("pymarketcap.core.Pymarketcap.stats", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_11pymarketcap_4core_11Pymarketcap_15stats(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_11pymarketcap_4core_11Pymarketcap_14stats[] = " Get global cryptocurrencies statistics.\n\n        Args:\n            convert (str, optional): return 24h volume, and\n                market cap in terms of another currency.\n                See ticker_badges property to get valid values.\n                As default ``\"USD\"``.\n\n        Returns (dict):\n            Global markets statistics.\n        ";
static PyObject *__pyx_pw_11pymarketcap_4core_11Pymarketcap_15stats(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_convert = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("stats (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_convert,0};
    PyObject* values[1] = {0};
    values[0] = ((PyObject *)__pyx_n_s_USD);
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_convert);
          if (value) { values[0] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "stats") < 0)) __PYX_ERR(0, 330, __pyx_L3_error)
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_convert = values[0];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("stats", 0, 0, 1, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 330, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("pymarketcap.core.Pymarketcap.stats", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_11pymarketcap_4core_11Pymarketcap_14stats(((struct __pyx_obj_11pymarketcap_4core_Pymarketcap *)__pyx_v_self), __pyx_v_convert);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_11pymarketcap_4core_11Pymarketcap_14stats(struct __pyx_obj_11pymarketcap_4core_Pymarketcap *__pyx_v_self, PyObject *__pyx_v_convert) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  struct __pyx_opt_args_11pymarketcap_4core_11Pymarketcap_stats __pyx_t_2;
  __Pyx_RefNannySetupContext("stats", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2.__pyx_n = 1;
  __pyx_t_2.convert = __pyx_v_convert;
  __pyx_t_1 = __pyx_vtabptr_11pymarketcap_4core_Pymarketcap->stats(__pyx_v_self, 1, &__pyx_t_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 330, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pymarketcap.core.Pymarketcap.stats", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pymarketcap/core.pyx":347
 * 
 *     @property
 *     def ticker_badges(self):             # <<<<<<<<<<<<<<
 *         """Badges in wich you can convert prices in ``ticker()`` method."""
 *         return
 */

/* Python wrapper */
static PyObject *__pyx_pw_11pymarketcap_4core_11Pymarketcap_13ticker_badges_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_11pymarketcap_4core_11Pymarketcap_13ticker_badges_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_11pymarketcap_4core_11Pymarketcap_13ticker_badges___get__(((struct __pyx_obj_11pymarketcap_4core_Pymarketcap *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_11pymarketcap_4core_11Pymarketcap_13ticker_badges___get__(CYTHON_UNUSED struct __pyx_obj_11pymarketcap_4core_Pymarketcap *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "pymarketcap/core.pyx":349
 *     def ticker_badges(self):
 *         """Badges in wich you can convert prices in ``ticker()`` method."""
 *         return             # <<<<<<<<<<<<<<
 * 
 *     cpdef ticker(self, currency=None, limit=0, start=0, convert="USD"):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;

  /* "pymarketcap/core.pyx":347
 * 
 *     @property
 *     def ticker_badges(self):             # <<<<<<<<<<<<<<
 *         """Badges in wich you can convert prices in ``ticker()`` method."""
 *         return
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pymarketcap/core.pyx":351
 *         return
 * 
 *     cpdef ticker(self, currency=None, limit=0, start=0, convert="USD"):             # <<<<<<<<<<<<<<
 *         """Get currencies with other aditional data.
 * 
 */

static PyObject *__pyx_pw_11pymarketcap_4core_11Pymarketcap_17ticker(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyObject *__pyx_f_11pymarketcap_4core_11Pymarketcap_ticker(struct __pyx_obj_11pymarketcap_4core_Pymarketcap *__pyx_v_self, int __pyx_skip_dispatch, struct __pyx_opt_args_11pymarketcap_4core_11Pymarketcap_ticker *__pyx_optional_args) {
  PyObject *__pyx_v_currency = ((PyObject *)Py_None);
  PyObject *__pyx_v_limit = ((PyObject *)__pyx_int_0);
  PyObject *__pyx_v_start = ((PyObject *)__pyx_int_0);
  PyObject *__pyx_v_convert = ((PyObject *)__pyx_n_s_USD);
  short __pyx_v_i;
  short __pyx_v_len_i;
  PyObject *__pyx_v_url = NULL;
  PyObject *__pyx_v_res = NULL;
  PyObject *__pyx_v_response = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_t_7;
  int __pyx_t_8;
  char *__pyx_t_9;
  PyObject *__pyx_t_10 = NULL;
  Py_ssize_t __pyx_t_11;
  short __pyx_t_12;
  short __pyx_t_13;
  short __pyx_t_14;
  __Pyx_RefNannySetupContext("ticker", 0);
  if (__pyx_optional_args) {
    if (__pyx_optional_args->__pyx_n > 0) {
      __pyx_v_currency = __pyx_optional_args->currency;
      if (__pyx_optional_args->__pyx_n > 1) {
        __pyx_v_limit = __pyx_optional_args->limit;
        if (__pyx_optional_args->__pyx_n > 2) {
          __pyx_v_start = __pyx_optional_args->start;
          if (__pyx_optional_args->__pyx_n > 3) {
            __pyx_v_convert = __pyx_optional_args->convert;
          }
        }
      }
    }
  }
  __Pyx_INCREF(__pyx_v_currency);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0)) {
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_ticker); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 351, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)__pyx_pw_11pymarketcap_4core_11Pymarketcap_17ticker)) {
      __Pyx_XDECREF(__pyx_r);
      __Pyx_INCREF(__pyx_t_1);
      __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
      __pyx_t_5 = 0;
      if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
        __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
        if (likely(__pyx_t_4)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
          __Pyx_INCREF(__pyx_t_4);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_3, function);
          __pyx_t_5 = 1;
        }
      }
      #if CYTHON_FAST_PYCALL
      if (PyFunction_Check(__pyx_t_3)) {
        PyObject *__pyx_temp[5] = {__pyx_t_4, __pyx_v_currency, __pyx_v_limit, __pyx_v_start, __pyx_v_convert};
        __pyx_t_2 = __Pyx_PyFunction_FastCall(__pyx_t_3, __pyx_temp+1-__pyx_t_5, 4+__pyx_t_5); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 351, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        __Pyx_GOTREF(__pyx_t_2);
      } else
      #endif
      #if CYTHON_FAST_PYCCALL
      if (__Pyx_PyFastCFunction_Check(__pyx_t_3)) {
        PyObject *__pyx_temp[5] = {__pyx_t_4, __pyx_v_currency, __pyx_v_limit, __pyx_v_start, __pyx_v_convert};
        __pyx_t_2 = __Pyx_PyCFunction_FastCall(__pyx_t_3, __pyx_temp+1-__pyx_t_5, 4+__pyx_t_5); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 351, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        __Pyx_GOTREF(__pyx_t_2);
      } else
      #endif
      {
        __pyx_t_6 = PyTuple_New(4+__pyx_t_5); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 351, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_6);
        if (__pyx_t_4) {
          __Pyx_GIVEREF(__pyx_t_4); PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_4); __pyx_t_4 = NULL;
        }
        __Pyx_INCREF(__pyx_v_currency);
        __Pyx_GIVEREF(__pyx_v_currency);
        PyTuple_SET_ITEM(__pyx_t_6, 0+__pyx_t_5, __pyx_v_currency);
        __Pyx_INCREF(__pyx_v_limit);
        __Pyx_GIVEREF(__pyx_v_limit);
        PyTuple_SET_ITEM(__pyx_t_6, 1+__pyx_t_5, __pyx_v_limit);
        __Pyx_INCREF(__pyx_v_start);
        __Pyx_GIVEREF(__pyx_v_start);
        PyTuple_SET_ITEM(__pyx_t_6, 2+__pyx_t_5, __pyx_v_start);
        __Pyx_INCREF(__pyx_v_convert);
        __Pyx_GIVEREF(__pyx_v_convert);
        PyTuple_SET_ITEM(__pyx_t_6, 3+__pyx_t_5, __pyx_v_convert);
        __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_t_6, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 351, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      }
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __pyx_r = __pyx_t_2;
      __pyx_t_2 = 0;
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      goto __pyx_L0;
    }
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  }

  /* "pymarketcap/core.pyx":375
 *         """
 *         cdef short i, len_i
 *         if not currency:             # <<<<<<<<<<<<<<
 *             url =  "https://api.coinmarketcap.com/v1/ticker/?limit=%d" % limit
 *             url += "&start=%d" % start
 */
  __pyx_t_7 = __Pyx_PyObject_IsTrue(__pyx_v_currency); if (unlikely(__pyx_t_7 < 0)) __PYX_ERR(0, 375, __pyx_L1_error)
  __pyx_t_8 = ((!__pyx_t_7) != 0);
  if (__pyx_t_8) {

    /* "pymarketcap/core.pyx":376
 *         cdef short i, len_i
 *         if not currency:
 *             url =  "https://api.coinmarketcap.com/v1/ticker/?limit=%d" % limit             # <<<<<<<<<<<<<<
 *             url += "&start=%d" % start
 *             url += "&convert=%s" % convert
 */
    __pyx_t_1 = __Pyx_PyString_Format(__pyx_kp_s_https_api_coinmarketcap_com_v1_t, __pyx_v_limit); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 376, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_v_url = __pyx_t_1;
    __pyx_t_1 = 0;

    /* "pymarketcap/core.pyx":377
 *         if not currency:
 *             url =  "https://api.coinmarketcap.com/v1/ticker/?limit=%d" % limit
 *             url += "&start=%d" % start             # <<<<<<<<<<<<<<
 *             url += "&convert=%s" % convert
 *             res = self._get(url.encode())
 */
    __pyx_t_1 = __Pyx_PyString_Format(__pyx_kp_s_start_d, __pyx_v_start); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 377, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_2 = PyNumber_InPlaceAdd(__pyx_v_url, __pyx_t_1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 377, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_DECREF_SET(__pyx_v_url, __pyx_t_2);
    __pyx_t_2 = 0;

    /* "pymarketcap/core.pyx":378
 *             url =  "https://api.coinmarketcap.com/v1/ticker/?limit=%d" % limit
 *             url += "&start=%d" % start
 *             url += "&convert=%s" % convert             # <<<<<<<<<<<<<<
 *             res = self._get(url.encode())
 *             response = loads(re_sub(r'"(-*\d+(?:\.\d+)?)"', r"\1", res))
 */
    __pyx_t_2 = __Pyx_PyString_Format(__pyx_kp_s_convert_s, __pyx_v_convert); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 378, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_1 = PyNumber_InPlaceAdd(__pyx_v_url, __pyx_t_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 378, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF_SET(__pyx_v_url, __pyx_t_1);
    __pyx_t_1 = 0;

    /* "pymarketcap/core.pyx":379
 *             url += "&start=%d" % start
 *             url += "&convert=%s" % convert
 *             res = self._get(url.encode())             # <<<<<<<<<<<<<<
 *             response = loads(re_sub(r'"(-*\d+(?:\.\d+)?)"', r"\1", res))
 *             len_i = len(response)
 */
    __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_url, __pyx_n_s_encode); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 379, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_3 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
      __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
      if (likely(__pyx_t_3)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
        __Pyx_INCREF(__pyx_t_3);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_2, function);
      }
    }
    if (__pyx_t_3) {
      __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_3); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 379, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    } else {
      __pyx_t_1 = __Pyx_PyObject_CallNoArg(__pyx_t_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 379, __pyx_L1_error)
    }
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_9 = __Pyx_PyObject_AsWritableString(__pyx_t_1); if (unlikely((!__pyx_t_9) && PyErr_Occurred())) __PYX_ERR(0, 379, __pyx_L1_error)
    __pyx_t_2 = ((struct __pyx_vtabstruct_11pymarketcap_4core_Pymarketcap *)__pyx_v_self->__pyx_vtab)->_get(__pyx_v_self, __pyx_t_9); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 379, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_v_res = __pyx_t_2;
    __pyx_t_2 = 0;

    /* "pymarketcap/core.pyx":380
 *             url += "&convert=%s" % convert
 *             res = self._get(url.encode())
 *             response = loads(re_sub(r'"(-*\d+(?:\.\d+)?)"', r"\1", res))             # <<<<<<<<<<<<<<
 *             len_i = len(response)
 *             for i in range(len_i):
 */
    __pyx_t_1 = __Pyx_GetModuleGlobalName(__pyx_n_s_loads); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 380, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_6 = __Pyx_GetModuleGlobalName(__pyx_n_s_re_sub); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 380, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_4 = NULL;
    __pyx_t_5 = 0;
    if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_6))) {
      __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_6);
      if (likely(__pyx_t_4)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_6);
        __Pyx_INCREF(__pyx_t_4);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_6, function);
        __pyx_t_5 = 1;
      }
    }
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(__pyx_t_6)) {
      PyObject *__pyx_temp[4] = {__pyx_t_4, __pyx_kp_s_d_d, __pyx_kp_s_1, __pyx_v_res};
      __pyx_t_3 = __Pyx_PyFunction_FastCall(__pyx_t_6, __pyx_temp+1-__pyx_t_5, 3+__pyx_t_5); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 380, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_GOTREF(__pyx_t_3);
    } else
    #endif
    #if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(__pyx_t_6)) {
      PyObject *__pyx_temp[4] = {__pyx_t_4, __pyx_kp_s_d_d, __pyx_kp_s_1, __pyx_v_res};
      __pyx_t_3 = __Pyx_PyCFunction_FastCall(__pyx_t_6, __pyx_temp+1-__pyx_t_5, 3+__pyx_t_5); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 380, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_GOTREF(__pyx_t_3);
    } else
    #endif
    {
      __pyx_t_10 = PyTuple_New(3+__pyx_t_5); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 380, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_10);
      if (__pyx_t_4) {
        __Pyx_GIVEREF(__pyx_t_4); PyTuple_SET_ITEM(__pyx_t_10, 0, __pyx_t_4); __pyx_t_4 = NULL;
      }
      __Pyx_INCREF(__pyx_kp_s_d_d);
      __Pyx_GIVEREF(__pyx_kp_s_d_d);
      PyTuple_SET_ITEM(__pyx_t_10, 0+__pyx_t_5, __pyx_kp_s_d_d);
      __Pyx_INCREF(__pyx_kp_s_1);
      __Pyx_GIVEREF(__pyx_kp_s_1);
      PyTuple_SET_ITEM(__pyx_t_10, 1+__pyx_t_5, __pyx_kp_s_1);
      __Pyx_INCREF(__pyx_v_res);
      __Pyx_GIVEREF(__pyx_v_res);
      PyTuple_SET_ITEM(__pyx_t_10, 2+__pyx_t_5, __pyx_v_res);
      __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_6, __pyx_t_10, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 380, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
    }
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __pyx_t_6 = NULL;
    if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_1))) {
      __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_1);
      if (likely(__pyx_t_6)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_1);
        __Pyx_INCREF(__pyx_t_6);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_1, function);
      }
    }
    if (!__pyx_t_6) {
      __pyx_t_2 = __Pyx_PyObject_CallOneArg(__pyx_t_1, __pyx_t_3); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 380, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_GOTREF(__pyx_t_2);
    } else {
      #if CYTHON_FAST_PYCALL
      if (PyFunction_Check(__pyx_t_1)) {
        PyObject *__pyx_temp[2] = {__pyx_t_6, __pyx_t_3};
        __pyx_t_2 = __Pyx_PyFunction_FastCall(__pyx_t_1, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 380, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      } else
      #endif
      #if CYTHON_FAST_PYCCALL
      if (__Pyx_PyFastCFunction_Check(__pyx_t_1)) {
        PyObject *__pyx_temp[2] = {__pyx_t_6, __pyx_t_3};
        __pyx_t_2 = __Pyx_PyCFunction_FastCall(__pyx_t_1, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 380, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      } else
      #endif
      {
        __pyx_t_10 = PyTuple_New(1+1); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 380, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_10);
        __Pyx_GIVEREF(__pyx_t_6); PyTuple_SET_ITEM(__pyx_t_10, 0, __pyx_t_6); __pyx_t_6 = NULL;
        __Pyx_GIVEREF(__pyx_t_3);
        PyTuple_SET_ITEM(__pyx_t_10, 0+1, __pyx_t_3);
        __pyx_t_3 = 0;
        __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_t_10, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 380, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
      }
    }
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_v_response = __pyx_t_2;
    __pyx_t_2 = 0;

    /* "pymarketcap/core.pyx":381
 *             res = self._get(url.encode())
 *             response = loads(re_sub(r'"(-*\d+(?:\.\d+)?)"', r"\1", res))
 *             len_i = len(response)             # <<<<<<<<<<<<<<
 *             for i in range(len_i):
 *                 response[i]["symbol"] = str(response[i]["symbol"])
 */
    __pyx_t_11 = PyObject_Length(__pyx_v_response); if (unlikely(__pyx_t_11 == ((Py_ssize_t)-1))) __PYX_ERR(0, 381, __pyx_L1_error)
    __pyx_v_len_i = __pyx_t_11;

    /* "pymarketcap/core.pyx":382
 *             response = loads(re_sub(r'"(-*\d+(?:\.\d+)?)"', r"\1", res))
 *             len_i = len(response)
 *             for i in range(len_i):             # <<<<<<<<<<<<<<
 *                 response[i]["symbol"] = str(response[i]["symbol"])
 *         else:
 */
    __pyx_t_12 = __pyx_v_len_i;
    __pyx_t_13 = __pyx_t_12;
    for (__pyx_t_14 = 0; __pyx_t_14 < __pyx_t_13; __pyx_t_14+=1) {
      __pyx_v_i = __pyx_t_14;

      /* "pymarketcap/core.pyx":383
 *             len_i = len(response)
 *             for i in range(len_i):
 *                 response[i]["symbol"] = str(response[i]["symbol"])             # <<<<<<<<<<<<<<
 *         else:
 *             if self._is_symbol(currency):
 */
      __pyx_t_2 = __Pyx_GetItemInt(__pyx_v_response, __pyx_v_i, short, 1, __Pyx_PyInt_From_short, 0, 1, 1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 383, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __pyx_t_1 = __Pyx_PyObject_Dict_GetItem(__pyx_t_2, __pyx_n_s_symbol); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 383, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __pyx_t_2 = __Pyx_PyObject_CallOneArg(((PyObject *)(&PyString_Type)), __pyx_t_1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 383, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __pyx_t_1 = __Pyx_GetItemInt(__pyx_v_response, __pyx_v_i, short, 1, __Pyx_PyInt_From_short, 0, 1, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 383, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (unlikely(PyObject_SetItem(__pyx_t_1, __pyx_n_s_symbol, __pyx_t_2) < 0)) __PYX_ERR(0, 383, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    }

    /* "pymarketcap/core.pyx":375
 *         """
 *         cdef short i, len_i
 *         if not currency:             # <<<<<<<<<<<<<<
 *             url =  "https://api.coinmarketcap.com/v1/ticker/?limit=%d" % limit
 *             url += "&start=%d" % start
 */
    goto __pyx_L3;
  }

  /* "pymarketcap/core.pyx":385
 *                 response[i]["symbol"] = str(response[i]["symbol"])
 *         else:
 *             if self._is_symbol(currency):             # <<<<<<<<<<<<<<
 *                 currency = self.correspondences[currency]
 *             url = "https://api.coinmarketcap.com/v1/ticker/%s" % currency
 */
  /*else*/ {
    if (!(likely(PyUnicode_CheckExact(__pyx_v_currency))||((__pyx_v_currency) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "unicode", Py_TYPE(__pyx_v_currency)->tp_name), 0))) __PYX_ERR(0, 385, __pyx_L1_error)
    __pyx_t_2 = ((struct __pyx_vtabstruct_11pymarketcap_4core_Pymarketcap *)__pyx_v_self->__pyx_vtab)->_is_symbol(__pyx_v_self, ((PyObject*)__pyx_v_currency), 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 385, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_8 = __Pyx_PyObject_IsTrue(__pyx_t_2); if (unlikely(__pyx_t_8 < 0)) __PYX_ERR(0, 385, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    if (__pyx_t_8) {

      /* "pymarketcap/core.pyx":386
 *         else:
 *             if self._is_symbol(currency):
 *                 currency = self.correspondences[currency]             # <<<<<<<<<<<<<<
 *             url = "https://api.coinmarketcap.com/v1/ticker/%s" % currency
 *             url += "?convert=%s" % convert
 */
      __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_correspondences); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 386, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __pyx_t_1 = __Pyx_PyObject_GetItem(__pyx_t_2, __pyx_v_currency); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 386, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __Pyx_DECREF_SET(__pyx_v_currency, __pyx_t_1);
      __pyx_t_1 = 0;

      /* "pymarketcap/core.pyx":385
 *                 response[i]["symbol"] = str(response[i]["symbol"])
 *         else:
 *             if self._is_symbol(currency):             # <<<<<<<<<<<<<<
 *                 currency = self.correspondences[currency]
 *             url = "https://api.coinmarketcap.com/v1/ticker/%s" % currency
 */
    }

    /* "pymarketcap/core.pyx":387
 *             if self._is_symbol(currency):
 *                 currency = self.correspondences[currency]
 *             url = "https://api.coinmarketcap.com/v1/ticker/%s" % currency             # <<<<<<<<<<<<<<
 *             url += "?convert=%s" % convert
 *             res = self._get(url.encode())
 */
    __pyx_t_1 = __Pyx_PyString_Format(__pyx_kp_s_https_api_coinmarketcap_com_v1_t_2, __pyx_v_currency); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 387, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_v_url = __pyx_t_1;
    __pyx_t_1 = 0;

    /* "pymarketcap/core.pyx":388
 *                 currency = self.correspondences[currency]
 *             url = "https://api.coinmarketcap.com/v1/ticker/%s" % currency
 *             url += "?convert=%s" % convert             # <<<<<<<<<<<<<<
 *             res = self._get(url.encode())
 *             response = loads(re_sub(r'"(-*\d+(?:\.\d+)?)"', r"\1", res))[0]
 */
    __pyx_t_1 = __Pyx_PyString_Format(__pyx_kp_s_convert_s_2, __pyx_v_convert); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 388, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_2 = PyNumber_InPlaceAdd(__pyx_v_url, __pyx_t_1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 388, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_DECREF_SET(__pyx_v_url, __pyx_t_2);
    __pyx_t_2 = 0;

    /* "pymarketcap/core.pyx":389
 *             url = "https://api.coinmarketcap.com/v1/ticker/%s" % currency
 *             url += "?convert=%s" % convert
 *             res = self._get(url.encode())             # <<<<<<<<<<<<<<
 *             response = loads(re_sub(r'"(-*\d+(?:\.\d+)?)"', r"\1", res))[0]
 *             response["symbol"] = str(response["symbol"])
 */
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_url, __pyx_n_s_encode); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 389, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_10 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_1))) {
      __pyx_t_10 = PyMethod_GET_SELF(__pyx_t_1);
      if (likely(__pyx_t_10)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_1);
        __Pyx_INCREF(__pyx_t_10);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_1, function);
      }
    }
    if (__pyx_t_10) {
      __pyx_t_2 = __Pyx_PyObject_CallOneArg(__pyx_t_1, __pyx_t_10); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 389, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
    } else {
      __pyx_t_2 = __Pyx_PyObject_CallNoArg(__pyx_t_1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 389, __pyx_L1_error)
    }
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_9 = __Pyx_PyObject_AsWritableString(__pyx_t_2); if (unlikely((!__pyx_t_9) && PyErr_Occurred())) __PYX_ERR(0, 389, __pyx_L1_error)
    __pyx_t_1 = ((struct __pyx_vtabstruct_11pymarketcap_4core_Pymarketcap *)__pyx_v_self->__pyx_vtab)->_get(__pyx_v_self, __pyx_t_9); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 389, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_v_res = __pyx_t_1;
    __pyx_t_1 = 0;

    /* "pymarketcap/core.pyx":390
 *             url += "?convert=%s" % convert
 *             res = self._get(url.encode())
 *             response = loads(re_sub(r'"(-*\d+(?:\.\d+)?)"', r"\1", res))[0]             # <<<<<<<<<<<<<<
 *             response["symbol"] = str(response["symbol"])
 *         return response
 */
    __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_loads); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 390, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_3 = __Pyx_GetModuleGlobalName(__pyx_n_s_re_sub); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 390, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_6 = NULL;
    __pyx_t_5 = 0;
    if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
      __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_3);
      if (likely(__pyx_t_6)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
        __Pyx_INCREF(__pyx_t_6);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_3, function);
        __pyx_t_5 = 1;
      }
    }
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(__pyx_t_3)) {
      PyObject *__pyx_temp[4] = {__pyx_t_6, __pyx_kp_s_d_d, __pyx_kp_s_1, __pyx_v_res};
      __pyx_t_10 = __Pyx_PyFunction_FastCall(__pyx_t_3, __pyx_temp+1-__pyx_t_5, 3+__pyx_t_5); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 390, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
      __Pyx_GOTREF(__pyx_t_10);
    } else
    #endif
    #if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(__pyx_t_3)) {
      PyObject *__pyx_temp[4] = {__pyx_t_6, __pyx_kp_s_d_d, __pyx_kp_s_1, __pyx_v_res};
      __pyx_t_10 = __Pyx_PyCFunction_FastCall(__pyx_t_3, __pyx_temp+1-__pyx_t_5, 3+__pyx_t_5); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 390, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
      __Pyx_GOTREF(__pyx_t_10);
    } else
    #endif
    {
      __pyx_t_4 = PyTuple_New(3+__pyx_t_5); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 390, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      if (__pyx_t_6) {
        __Pyx_GIVEREF(__pyx_t_6); PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_6); __pyx_t_6 = NULL;
      }
      __Pyx_INCREF(__pyx_kp_s_d_d);
      __Pyx_GIVEREF(__pyx_kp_s_d_d);
      PyTuple_SET_ITEM(__pyx_t_4, 0+__pyx_t_5, __pyx_kp_s_d_d);
      __Pyx_INCREF(__pyx_kp_s_1);
      __Pyx_GIVEREF(__pyx_kp_s_1);
      PyTuple_SET_ITEM(__pyx_t_4, 1+__pyx_t_5, __pyx_kp_s_1);
      __Pyx_INCREF(__pyx_v_res);
      __Pyx_GIVEREF(__pyx_v_res);
      PyTuple_SET_ITEM(__pyx_t_4, 2+__pyx_t_5, __pyx_v_res);
      __pyx_t_10 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_t_4, NULL); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 390, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_10);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    }
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_t_3 = NULL;
    if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
      __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
      if (likely(__pyx_t_3)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
        __Pyx_INCREF(__pyx_t_3);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_2, function);
      }
    }
    if (!__pyx_t_3) {
      __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_10); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 390, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
      __Pyx_GOTREF(__pyx_t_1);
    } else {
      #if CYTHON_FAST_PYCALL
      if (PyFunction_Check(__pyx_t_2)) {
        PyObject *__pyx_temp[2] = {__pyx_t_3, __pyx_t_10};
        __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 390, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
        __Pyx_GOTREF(__pyx_t_1);
        __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
      } else
      #endif
      #if CYTHON_FAST_PYCCALL
      if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
        PyObject *__pyx_temp[2] = {__pyx_t_3, __pyx_t_10};
        __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 390, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
        __Pyx_GOTREF(__pyx_t_1);
        __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
      } else
      #endif
      {
        __pyx_t_4 = PyTuple_New(1+1); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 390, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_4);
        __Pyx_GIVEREF(__pyx_t_3); PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_3); __pyx_t_3 = NULL;
        __Pyx_GIVEREF(__pyx_t_10);
        PyTuple_SET_ITEM(__pyx_t_4, 0+1, __pyx_t_10);
        __pyx_t_10 = 0;
        __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_4, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 390, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_1);
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      }
    }
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_2 = __Pyx_GetItemInt(__pyx_t_1, 0, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 390, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_v_response = __pyx_t_2;
    __pyx_t_2 = 0;

    /* "pymarketcap/core.pyx":391
 *             res = self._get(url.encode())
 *             response = loads(re_sub(r'"(-*\d+(?:\.\d+)?)"', r"\1", res))[0]
 *             response["symbol"] = str(response["symbol"])             # <<<<<<<<<<<<<<
 *         return response
 * 
 */
    __pyx_t_2 = __Pyx_PyObject_Dict_GetItem(__pyx_v_response, __pyx_n_s_symbol); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 391, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_1 = __Pyx_PyObject_CallOneArg(((PyObject *)(&PyString_Type)), __pyx_t_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 391, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    if (unlikely(PyObject_SetItem(__pyx_v_response, __pyx_n_s_symbol, __pyx_t_1) < 0)) __PYX_ERR(0, 391, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  }
  __pyx_L3:;

  /* "pymarketcap/core.pyx":392
 *             response = loads(re_sub(r'"(-*\d+(?:\.\d+)?)"', r"\1", res))[0]
 *             response["symbol"] = str(response["symbol"])
 *         return response             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_response);
  __pyx_r = __pyx_v_response;
  goto __pyx_L0;

  /* "pymarketcap/core.pyx":351
 *         return
 * 
 *     cpdef ticker(self, currency=None, limit=0, start=0, convert="USD"):             # <<<<<<<<<<<<<<
 *         """Get currencies with other aditional data.
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_10);
  __Pyx_AddTraceback("pymarketcap.core.Pymarketcap.ticker", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_url);
  __Pyx_XDECREF(__pyx_v_res);
  __Pyx_XDECREF(__pyx_v_response);
  __Pyx_XDECREF(__pyx_v_currency);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_11pymarketcap_4core_11Pymarketcap_17ticker(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_11pymarketcap_4core_11Pymarketcap_16ticker[] = "Get currencies with other aditional data.\n\n        Args:\n            currency (str, optional): Specify a currency to return data,\n                that can be a symbol or coin slug (see ``symbols`` and ``coins``\n                properties). In this case the method returns a dict, otherwise\n                returns a list. If you dont specify a currency,\n                returns data for all in coinmarketcap. As default, ``None``.\n            limit (int, optional): Limit amount of coins on response.\n                If ``limit == 0``, returns all coins in coinmarketcap.\n                Only works if ``currency == None``. As default ``0``.\n            start (int, optional): Rank of first currency to retrieve.\n                The count starts at 0 for the first currency ranked.\n                Only works if ``currency == None``. As default ``0``.\n            convert (str, optional): Allows to convert prices, 24h volumes\n                and market capitalizations in terms of one of badges\n                returned by ``ticker_badges`` property.\n                As default, ``\"USD\"``.\n\n        Returns (dict/list):\n            The type depends if currency param is provided or not.\n        ";
static PyObject *__pyx_pw_11pymarketcap_4core_11Pymarketcap_17ticker(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_currency = 0;
  PyObject *__pyx_v_limit = 0;
  PyObject *__pyx_v_start = 0;
  PyObject *__pyx_v_convert = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("ticker (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_currency,&__pyx_n_s_limit,&__pyx_n_s_start,&__pyx_n_s_convert,0};
    PyObject* values[4] = {0,0,0,0};
    values[0] = ((PyObject *)Py_None);
    values[1] = ((PyObject *)__pyx_int_0);
    values[2] = ((PyObject *)__pyx_int_0);
    values[3] = ((PyObject *)__pyx_n_s_USD);
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        CYTHON_FALLTHROUGH;
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_currency);
          if (value) { values[0] = value; kw_args--; }
        }
        CYTHON_FALLTHROUGH;
        case  1:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_limit);
          if (value) { values[1] = value; kw_args--; }
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_start);
          if (value) { values[2] = value; kw_args--; }
        }
        CYTHON_FALLTHROUGH;
        case  3:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_convert);
          if (value) { values[3] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "ticker") < 0)) __PYX_ERR(0, 351, __pyx_L3_error)
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        CYTHON_FALLTHROUGH;
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_currency = values[0];
    __pyx_v_limit = values[1];
    __pyx_v_start = values[2];
    __pyx_v_convert = values[3];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("ticker", 0, 0, 4, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 351, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("pymarketcap.core.Pymarketcap.ticker", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_11pymarketcap_4core_11Pymarketcap_16ticker(((struct __pyx_obj_11pymarketcap_4core_Pymarketcap *)__pyx_v_self), __pyx_v_currency, __pyx_v_limit, __pyx_v_start, __pyx_v_convert);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_11pymarketcap_4core_11Pymarketcap_16ticker(struct __pyx_obj_11pymarketcap_4core_Pymarketcap *__pyx_v_self, PyObject *__pyx_v_currency, PyObject *__pyx_v_limit, PyObject *__pyx_v_start, PyObject *__pyx_v_convert) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  struct __pyx_opt_args_11pymarketcap_4core_11Pymarketcap_ticker __pyx_t_2;
  __Pyx_RefNannySetupContext("ticker", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2.__pyx_n = 4;
  __pyx_t_2.currency = __pyx_v_currency;
  __pyx_t_2.limit = __pyx_v_limit;
  __pyx_t_2.start = __pyx_v_start;
  __pyx_t_2.convert = __pyx_v_convert;
  __pyx_t_1 = __pyx_vtabptr_11pymarketcap_4core_Pymarketcap->ticker(__pyx_v_self, 1, &__pyx_t_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 351, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pymarketcap.core.Pymarketcap.ticker", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pymarketcap/core.pyx":400
 * 
 *     @property
 *     def currency_exchange_rates(self):             # <<<<<<<<<<<<<<
 *         """Currency exchange rates against $ for all currencies (fiat + crypto).
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_11pymarketcap_4core_11Pymarketcap_23currency_exchange_rates_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_11pymarketcap_4core_11Pymarketcap_23currency_exchange_rates_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_11pymarketcap_4core_11Pymarketcap_23currency_exchange_rates___get__(((struct __pyx_obj_11pymarketcap_4core_Pymarketcap *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_11pymarketcap_4core_11Pymarketcap_23currency_exchange_rates___get__(struct __pyx_obj_11pymarketcap_4core_Pymarketcap *__pyx_v_self) {
  PyObject *__pyx_v_res = NULL;
  PyObject *__pyx_v_rates = NULL;
  PyObject *__pyx_v_response = NULL;
  PyObject *__pyx_v_currency = NULL;
  PyObject *__pyx_8genexpr1__pyx_v_currency = NULL;
  PyObject *__pyx_8genexpr1__pyx_v_rate = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  Py_ssize_t __pyx_t_7;
  PyObject *(*__pyx_t_8)(PyObject *);
  PyObject *__pyx_t_9 = NULL;
  PyObject *(*__pyx_t_10)(PyObject *);
  PyObject *__pyx_t_11 = NULL;
  PyObject *__pyx_t_12 = NULL;
  PyObject *__pyx_t_13 = NULL;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "pymarketcap/core.pyx":407
 *             the prices shown.
 *         """
 *         res = self._get(b"https://coinmarketcap.com")             # <<<<<<<<<<<<<<
 *         rates = re_findall(
 *             r'data-([a-z]+)="(\d+\.*[\d|e|-]*)"', res[-10000:-2000]
 */
  __pyx_t_1 = ((struct __pyx_vtabstruct_11pymarketcap_4core_Pymarketcap *)__pyx_v_self->__pyx_vtab)->_get(__pyx_v_self, ((char *)"https://coinmarketcap.com")); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 407, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_res = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "pymarketcap/core.pyx":408
 *         """
 *         res = self._get(b"https://coinmarketcap.com")
 *         rates = re_findall(             # <<<<<<<<<<<<<<
 *             r'data-([a-z]+)="(\d+\.*[\d|e|-]*)"', res[-10000:-2000]
 *         )
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_re_findall); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 408, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);

  /* "pymarketcap/core.pyx":409
 *         res = self._get(b"https://coinmarketcap.com")
 *         rates = re_findall(
 *             r'data-([a-z]+)="(\d+\.*[\d|e|-]*)"', res[-10000:-2000]             # <<<<<<<<<<<<<<
 *         )
 *         response = {currency.upper(): float(rate) for currency, rate in rates}
 */
  __pyx_t_3 = __Pyx_PyObject_GetSlice(__pyx_v_res, -10000L, -2000L, NULL, NULL, &__pyx_slice__6, 1, 1, 1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 409, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = NULL;
  __pyx_t_5 = 0;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_5 = 1;
    }
  }
  #if CYTHON_FAST_PYCALL
  if (PyFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[3] = {__pyx_t_4, __pyx_kp_s_data_a_z_d_d_e, __pyx_t_3};
    __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_5, 2+__pyx_t_5); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 408, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  } else
  #endif
  #if CYTHON_FAST_PYCCALL
  if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[3] = {__pyx_t_4, __pyx_kp_s_data_a_z_d_d_e, __pyx_t_3};
    __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_5, 2+__pyx_t_5); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 408, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  } else
  #endif
  {
    __pyx_t_6 = PyTuple_New(2+__pyx_t_5); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 408, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    if (__pyx_t_4) {
      __Pyx_GIVEREF(__pyx_t_4); PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_4); __pyx_t_4 = NULL;
    }
    __Pyx_INCREF(__pyx_kp_s_data_a_z_d_d_e);
    __Pyx_GIVEREF(__pyx_kp_s_data_a_z_d_d_e);
    PyTuple_SET_ITEM(__pyx_t_6, 0+__pyx_t_5, __pyx_kp_s_data_a_z_d_d_e);
    __Pyx_GIVEREF(__pyx_t_3);
    PyTuple_SET_ITEM(__pyx_t_6, 1+__pyx_t_5, __pyx_t_3);
    __pyx_t_3 = 0;
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_6, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 408, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v_rates = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "pymarketcap/core.pyx":411
 *             r'data-([a-z]+)="(\d+\.*[\d|e|-]*)"', res[-10000:-2000]
 *         )
 *         response = {currency.upper(): float(rate) for currency, rate in rates}             # <<<<<<<<<<<<<<
 *         for currency in self.ticker():
 *             try:
 */
  { /* enter inner scope */
    __pyx_t_1 = PyDict_New(); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 411, __pyx_L5_error)
    __Pyx_GOTREF(__pyx_t_1);
    if (likely(PyList_CheckExact(__pyx_v_rates)) || PyTuple_CheckExact(__pyx_v_rates)) {
      __pyx_t_2 = __pyx_v_rates; __Pyx_INCREF(__pyx_t_2); __pyx_t_7 = 0;
      __pyx_t_8 = NULL;
    } else {
      __pyx_t_7 = -1; __pyx_t_2 = PyObject_GetIter(__pyx_v_rates); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 411, __pyx_L5_error)
      __Pyx_GOTREF(__pyx_t_2);
      __pyx_t_8 = Py_TYPE(__pyx_t_2)->tp_iternext; if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 411, __pyx_L5_error)
    }
    for (;;) {
      if (likely(!__pyx_t_8)) {
        if (likely(PyList_CheckExact(__pyx_t_2))) {
          if (__pyx_t_7 >= PyList_GET_SIZE(__pyx_t_2)) break;
          #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
          __pyx_t_6 = PyList_GET_ITEM(__pyx_t_2, __pyx_t_7); __Pyx_INCREF(__pyx_t_6); __pyx_t_7++; if (unlikely(0 < 0)) __PYX_ERR(0, 411, __pyx_L5_error)
          #else
          __pyx_t_6 = PySequence_ITEM(__pyx_t_2, __pyx_t_7); __pyx_t_7++; if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 411, __pyx_L5_error)
          __Pyx_GOTREF(__pyx_t_6);
          #endif
        } else {
          if (__pyx_t_7 >= PyTuple_GET_SIZE(__pyx_t_2)) break;
          #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
          __pyx_t_6 = PyTuple_GET_ITEM(__pyx_t_2, __pyx_t_7); __Pyx_INCREF(__pyx_t_6); __pyx_t_7++; if (unlikely(0 < 0)) __PYX_ERR(0, 411, __pyx_L5_error)
          #else
          __pyx_t_6 = PySequence_ITEM(__pyx_t_2, __pyx_t_7); __pyx_t_7++; if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 411, __pyx_L5_error)
          __Pyx_GOTREF(__pyx_t_6);
          #endif
        }
      } else {
        __pyx_t_6 = __pyx_t_8(__pyx_t_2);
        if (unlikely(!__pyx_t_6)) {
          PyObject* exc_type = PyErr_Occurred();
          if (exc_type) {
            if (likely(__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
            else __PYX_ERR(0, 411, __pyx_L5_error)
          }
          break;
        }
        __Pyx_GOTREF(__pyx_t_6);
      }
      if ((likely(PyTuple_CheckExact(__pyx_t_6))) || (PyList_CheckExact(__pyx_t_6))) {
        PyObject* sequence = __pyx_t_6;
        Py_ssize_t size = __Pyx_PySequence_SIZE(sequence);
        if (unlikely(size != 2)) {
          if (size > 2) __Pyx_RaiseTooManyValuesError(2);
          else if (size >= 0) __Pyx_RaiseNeedMoreValuesError(size);
          __PYX_ERR(0, 411, __pyx_L5_error)
        }
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        if (likely(PyTuple_CheckExact(sequence))) {
          __pyx_t_3 = PyTuple_GET_ITEM(sequence, 0); 
          __pyx_t_4 = PyTuple_GET_ITEM(sequence, 1); 
        } else {
          __pyx_t_3 = PyList_GET_ITEM(sequence, 0); 
          __pyx_t_4 = PyList_GET_ITEM(sequence, 1); 
        }
        __Pyx_INCREF(__pyx_t_3);
        __Pyx_INCREF(__pyx_t_4);
        #else
        __pyx_t_3 = PySequence_ITEM(sequence, 0); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 411, __pyx_L5_error)
        __Pyx_GOTREF(__pyx_t_3);
        __pyx_t_4 = PySequence_ITEM(sequence, 1); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 411, __pyx_L5_error)
        __Pyx_GOTREF(__pyx_t_4);
        #endif
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      } else {
        Py_ssize_t index = -1;
        __pyx_t_9 = PyObject_GetIter(__pyx_t_6); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 411, __pyx_L5_error)
        __Pyx_GOTREF(__pyx_t_9);
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
        __pyx_t_10 = Py_TYPE(__pyx_t_9)->tp_iternext;
        index = 0; __pyx_t_3 = __pyx_t_10(__pyx_t_9); if (unlikely(!__pyx_t_3)) goto __pyx_L8_unpacking_failed;
        __Pyx_GOTREF(__pyx_t_3);
        index = 1; __pyx_t_4 = __pyx_t_10(__pyx_t_9); if (unlikely(!__pyx_t_4)) goto __pyx_L8_unpacking_failed;
        __Pyx_GOTREF(__pyx_t_4);
        if (__Pyx_IternextUnpackEndCheck(__pyx_t_10(__pyx_t_9), 2) < 0) __PYX_ERR(0, 411, __pyx_L5_error)
        __pyx_t_10 = NULL;
        __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
        goto __pyx_L9_unpacking_done;
        __pyx_L8_unpacking_failed:;
        __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
        __pyx_t_10 = NULL;
        if (__Pyx_IterFinish() == 0) __Pyx_RaiseNeedMoreValuesError(index);
        __PYX_ERR(0, 411, __pyx_L5_error)
        __pyx_L9_unpacking_done:;
      }
      __Pyx_XDECREF_SET(__pyx_8genexpr1__pyx_v_currency, __pyx_t_3);
      __pyx_t_3 = 0;
      __Pyx_XDECREF_SET(__pyx_8genexpr1__pyx_v_rate, __pyx_t_4);
      __pyx_t_4 = 0;
      __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_8genexpr1__pyx_v_currency, __pyx_n_s_upper); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 411, __pyx_L5_error)
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_3 = NULL;
      if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_4))) {
        __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_4);
        if (likely(__pyx_t_3)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
          __Pyx_INCREF(__pyx_t_3);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_4, function);
        }
      }
      if (__pyx_t_3) {
        __pyx_t_6 = __Pyx_PyObject_CallOneArg(__pyx_t_4, __pyx_t_3); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 411, __pyx_L5_error)
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      } else {
        __pyx_t_6 = __Pyx_PyObject_CallNoArg(__pyx_t_4); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 411, __pyx_L5_error)
      }
      __Pyx_GOTREF(__pyx_t_6);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __pyx_t_4 = __Pyx_PyNumber_Float(__pyx_8genexpr1__pyx_v_rate); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 411, __pyx_L5_error)
      __Pyx_GOTREF(__pyx_t_4);
      if (unlikely(PyDict_SetItem(__pyx_t_1, (PyObject*)__pyx_t_6, (PyObject*)__pyx_t_4))) __PYX_ERR(0, 411, __pyx_L5_error)
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    }
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_XDECREF(__pyx_8genexpr1__pyx_v_currency); __pyx_8genexpr1__pyx_v_currency = 0;
    __Pyx_XDECREF(__pyx_8genexpr1__pyx_v_rate); __pyx_8genexpr1__pyx_v_rate = 0;
    goto __pyx_L10_exit_scope;
    __pyx_L5_error:;
    __Pyx_XDECREF(__pyx_8genexpr1__pyx_v_currency); __pyx_8genexpr1__pyx_v_currency = 0;
    __Pyx_XDECREF(__pyx_8genexpr1__pyx_v_rate); __pyx_8genexpr1__pyx_v_rate = 0;
    goto __pyx_L1_error;
    __pyx_L10_exit_scope:;
  } /* exit inner scope */
  __pyx_v_response = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "pymarketcap/core.pyx":412
 *         )
 *         response = {currency.upper(): float(rate) for currency, rate in rates}
 *         for currency in self.ticker():             # <<<<<<<<<<<<<<
 *             try:
 *                 response[currency["symbol"]] = float(currency["price_usd"])
 */
  __pyx_t_1 = ((struct __pyx_vtabstruct_11pymarketcap_4core_Pymarketcap *)__pyx_v_self->__pyx_vtab)->ticker(__pyx_v_self, 0, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 412, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (likely(PyList_CheckExact(__pyx_t_1)) || PyTuple_CheckExact(__pyx_t_1)) {
    __pyx_t_2 = __pyx_t_1; __Pyx_INCREF(__pyx_t_2); __pyx_t_7 = 0;
    __pyx_t_8 = NULL;
  } else {
    __pyx_t_7 = -1; __pyx_t_2 = PyObject_GetIter(__pyx_t_1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 412, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_8 = Py_TYPE(__pyx_t_2)->tp_iternext; if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 412, __pyx_L1_error)
  }
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  for (;;) {
    if (likely(!__pyx_t_8)) {
      if (likely(PyList_CheckExact(__pyx_t_2))) {
        if (__pyx_t_7 >= PyList_GET_SIZE(__pyx_t_2)) break;
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_1 = PyList_GET_ITEM(__pyx_t_2, __pyx_t_7); __Pyx_INCREF(__pyx_t_1); __pyx_t_7++; if (unlikely(0 < 0)) __PYX_ERR(0, 412, __pyx_L1_error)
        #else
        __pyx_t_1 = PySequence_ITEM(__pyx_t_2, __pyx_t_7); __pyx_t_7++; if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 412, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_1);
        #endif
      } else {
        if (__pyx_t_7 >= PyTuple_GET_SIZE(__pyx_t_2)) break;
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_1 = PyTuple_GET_ITEM(__pyx_t_2, __pyx_t_7); __Pyx_INCREF(__pyx_t_1); __pyx_t_7++; if (unlikely(0 < 0)) __PYX_ERR(0, 412, __pyx_L1_error)
        #else
        __pyx_t_1 = PySequence_ITEM(__pyx_t_2, __pyx_t_7); __pyx_t_7++; if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 412, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_1);
        #endif
      }
    } else {
      __pyx_t_1 = __pyx_t_8(__pyx_t_2);
      if (unlikely(!__pyx_t_1)) {
        PyObject* exc_type = PyErr_Occurred();
        if (exc_type) {
          if (likely(__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
          else __PYX_ERR(0, 412, __pyx_L1_error)
        }
        break;
      }
      __Pyx_GOTREF(__pyx_t_1);
    }
    __Pyx_XDECREF_SET(__pyx_v_currency, __pyx_t_1);
    __pyx_t_1 = 0;

    /* "pymarketcap/core.pyx":413
 *         response = {currency.upper(): float(rate) for currency, rate in rates}
 *         for currency in self.ticker():
 *             try:             # <<<<<<<<<<<<<<
 *                 response[currency["symbol"]] = float(currency["price_usd"])
 *             except TypeError:
 */
    {
      __Pyx_PyThreadState_declare
      __Pyx_PyThreadState_assign
      __Pyx_ExceptionSave(&__pyx_t_11, &__pyx_t_12, &__pyx_t_13);
      __Pyx_XGOTREF(__pyx_t_11);
      __Pyx_XGOTREF(__pyx_t_12);
      __Pyx_XGOTREF(__pyx_t_13);
      /*try:*/ {

        /* "pymarketcap/core.pyx":414
 *         for currency in self.ticker():
 *             try:
 *                 response[currency["symbol"]] = float(currency["price_usd"])             # <<<<<<<<<<<<<<
 *             except TypeError:
 *                 continue
 */
        __pyx_t_1 = __Pyx_PyObject_Dict_GetItem(__pyx_v_currency, __pyx_n_s_price_usd); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 414, __pyx_L13_error)
        __Pyx_GOTREF(__pyx_t_1);
        __pyx_t_4 = __Pyx_PyNumber_Float(__pyx_t_1); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 414, __pyx_L13_error)
        __Pyx_GOTREF(__pyx_t_4);
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        __pyx_t_1 = __Pyx_PyObject_Dict_GetItem(__pyx_v_currency, __pyx_n_s_symbol); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 414, __pyx_L13_error)
        __Pyx_GOTREF(__pyx_t_1);
        if (unlikely(PyDict_SetItem(__pyx_v_response, __pyx_t_1, __pyx_t_4) < 0)) __PYX_ERR(0, 414, __pyx_L13_error)
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

        /* "pymarketcap/core.pyx":413
 *         response = {currency.upper(): float(rate) for currency, rate in rates}
 *         for currency in self.ticker():
 *             try:             # <<<<<<<<<<<<<<
 *                 response[currency["symbol"]] = float(currency["price_usd"])
 *             except TypeError:
 */
      }
      __Pyx_XDECREF(__pyx_t_11); __pyx_t_11 = 0;
      __Pyx_XDECREF(__pyx_t_12); __pyx_t_12 = 0;
      __Pyx_XDECREF(__pyx_t_13); __pyx_t_13 = 0;
      goto __pyx_L20_try_end;
      __pyx_L13_error:;
      __Pyx_XDECREF(__pyx_t_9); __pyx_t_9 = 0;
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
      __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
      __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;

      /* "pymarketcap/core.pyx":415
 *             try:
 *                 response[currency["symbol"]] = float(currency["price_usd"])
 *             except TypeError:             # <<<<<<<<<<<<<<
 *                 continue
 *         return response
 */
      __pyx_t_5 = __Pyx_PyErr_ExceptionMatches(__pyx_builtin_TypeError);
      if (__pyx_t_5) {
        __Pyx_AddTraceback("pymarketcap.core.Pymarketcap.currency_exchange_rates.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
        if (__Pyx_GetException(&__pyx_t_4, &__pyx_t_1, &__pyx_t_6) < 0) __PYX_ERR(0, 415, __pyx_L15_except_error)
        __Pyx_GOTREF(__pyx_t_4);
        __Pyx_GOTREF(__pyx_t_1);
        __Pyx_GOTREF(__pyx_t_6);

        /* "pymarketcap/core.pyx":416
 *                 response[currency["symbol"]] = float(currency["price_usd"])
 *             except TypeError:
 *                 continue             # <<<<<<<<<<<<<<
 *         return response
 * 
 */
        goto __pyx_L22_except_continue;
        __pyx_L22_except_continue:;
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
        goto __pyx_L19_try_continue;
      }
      goto __pyx_L15_except_error;
      __pyx_L15_except_error:;

      /* "pymarketcap/core.pyx":413
 *         response = {currency.upper(): float(rate) for currency, rate in rates}
 *         for currency in self.ticker():
 *             try:             # <<<<<<<<<<<<<<
 *                 response[currency["symbol"]] = float(currency["price_usd"])
 *             except TypeError:
 */
      __Pyx_XGIVEREF(__pyx_t_11);
      __Pyx_XGIVEREF(__pyx_t_12);
      __Pyx_XGIVEREF(__pyx_t_13);
      __Pyx_ExceptionReset(__pyx_t_11, __pyx_t_12, __pyx_t_13);
      goto __pyx_L1_error;
      __pyx_L19_try_continue:;
      __Pyx_XGIVEREF(__pyx_t_11);
      __Pyx_XGIVEREF(__pyx_t_12);
      __Pyx_XGIVEREF(__pyx_t_13);
      __Pyx_ExceptionReset(__pyx_t_11, __pyx_t_12, __pyx_t_13);
      goto __pyx_L11_continue;
      __pyx_L20_try_end:;
    }

    /* "pymarketcap/core.pyx":412
 *         )
 *         response = {currency.upper(): float(rate) for currency, rate in rates}
 *         for currency in self.ticker():             # <<<<<<<<<<<<<<
 *             try:
 *                 response[currency["symbol"]] = float(currency["price_usd"])
 */
    __pyx_L11_continue:;
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "pymarketcap/core.pyx":417
 *             except TypeError:
 *                 continue
 *         return response             # <<<<<<<<<<<<<<
 * 
 *     cpdef convert(self, value, unicode currency_in, unicode currency_out):
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_response);
  __pyx_r = __pyx_v_response;
  goto __pyx_L0;

  /* "pymarketcap/core.pyx":400
 * 
 *     @property
 *     def currency_exchange_rates(self):             # <<<<<<<<<<<<<<
 *         """Currency exchange rates against $ for all currencies (fiat + crypto).
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_AddTraceback("pymarketcap.core.Pymarketcap.currency_exchange_rates.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_res);
  __Pyx_XDECREF(__pyx_v_rates);
  __Pyx_XDECREF(__pyx_v_response);
  __Pyx_XDECREF(__pyx_v_currency);
  __Pyx_XDECREF(__pyx_8genexpr1__pyx_v_currency);
  __Pyx_XDECREF(__pyx_8genexpr1__pyx_v_rate);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pymarketcap/core.pyx":419
 *         return response
 * 
 *     cpdef convert(self, value, unicode currency_in, unicode currency_out):             # <<<<<<<<<<<<<<
 *         """Convert prices between currencies. Provide a value, the currency
 *         of the value and the currency to convert it and get the value in
 */

static PyObject *__pyx_pw_11pymarketcap_4core_11Pymarketcap_19convert(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyObject *__pyx_f_11pymarketcap_4core_11Pymarketcap_convert(struct __pyx_obj_11pymarketcap_4core_Pymarketcap *__pyx_v_self, PyObject *__pyx_v_value, PyObject *__pyx_v_currency_in, PyObject *__pyx_v_currency_out, int __pyx_skip_dispatch) {
  PyObject *__pyx_v_rates = NULL;
  PyObject *__pyx_v_err = NULL;
  PyObject *__pyx_v_msg = NULL;
  PyObject *__pyx_v_param = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_t_7;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  PyObject *__pyx_t_10 = NULL;
  int __pyx_t_11;
  Py_ssize_t __pyx_t_12;
  PyObject *__pyx_t_13 = NULL;
  __Pyx_RefNannySetupContext("convert", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0)) {
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_convert); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 419, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)__pyx_pw_11pymarketcap_4core_11Pymarketcap_19convert)) {
      __Pyx_XDECREF(__pyx_r);
      __Pyx_INCREF(__pyx_t_1);
      __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
      __pyx_t_5 = 0;
      if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
        __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
        if (likely(__pyx_t_4)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
          __Pyx_INCREF(__pyx_t_4);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_3, function);
          __pyx_t_5 = 1;
        }
      }
      #if CYTHON_FAST_PYCALL
      if (PyFunction_Check(__pyx_t_3)) {
        PyObject *__pyx_temp[4] = {__pyx_t_4, __pyx_v_value, __pyx_v_currency_in, __pyx_v_currency_out};
        __pyx_t_2 = __Pyx_PyFunction_FastCall(__pyx_t_3, __pyx_temp+1-__pyx_t_5, 3+__pyx_t_5); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 419, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        __Pyx_GOTREF(__pyx_t_2);
      } else
      #endif
      #if CYTHON_FAST_PYCCALL
      if (__Pyx_PyFastCFunction_Check(__pyx_t_3)) {
        PyObject *__pyx_temp[4] = {__pyx_t_4, __pyx_v_value, __pyx_v_currency_in, __pyx_v_currency_out};
        __pyx_t_2 = __Pyx_PyCFunction_FastCall(__pyx_t_3, __pyx_temp+1-__pyx_t_5, 3+__pyx_t_5); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 419, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        __Pyx_GOTREF(__pyx_t_2);
      } else
      #endif
      {
        __pyx_t_6 = PyTuple_New(3+__pyx_t_5); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 419, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_6);
        if (__pyx_t_4) {
          __Pyx_GIVEREF(__pyx_t_4); PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_4); __pyx_t_4 = NULL;
        }
        __Pyx_INCREF(__pyx_v_value);
        __Pyx_GIVEREF(__pyx_v_value);
        PyTuple_SET_ITEM(__pyx_t_6, 0+__pyx_t_5, __pyx_v_value);
        __Pyx_INCREF(__pyx_v_currency_in);
        __Pyx_GIVEREF(__pyx_v_currency_in);
        PyTuple_SET_ITEM(__pyx_t_6, 1+__pyx_t_5, __pyx_v_currency_in);
        __Pyx_INCREF(__pyx_v_currency_out);
        __Pyx_GIVEREF(__pyx_v_currency_out);
        PyTuple_SET_ITEM(__pyx_t_6, 2+__pyx_t_5, __pyx_v_currency_out);
        __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_t_6, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 419, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      }
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __pyx_r = __pyx_t_2;
      __pyx_t_2 = 0;
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      goto __pyx_L0;
    }
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  }

  /* "pymarketcap/core.pyx":433
 *             Value expressed in currency_out parameter provided.
 *         """
 *         if time() - self.converter_cache[1] > 600:             # <<<<<<<<<<<<<<
 *             self.converter_cache[0] = self.currency_exchange_rates
 *         try:
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_time); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 433, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = NULL;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  if (__pyx_t_3) {
    __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_3); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 433, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  } else {
    __pyx_t_1 = __Pyx_PyObject_CallNoArg(__pyx_t_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 433, __pyx_L1_error)
  }
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_converter_cache); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 433, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_GetItemInt(__pyx_t_2, 1, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 433, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = PyNumber_Subtract(__pyx_t_1, __pyx_t_3); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 433, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = PyObject_RichCompare(__pyx_t_2, __pyx_int_600, Py_GT); __Pyx_XGOTREF(__pyx_t_3); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 433, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_7 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely(__pyx_t_7 < 0)) __PYX_ERR(0, 433, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (__pyx_t_7) {

    /* "pymarketcap/core.pyx":434
 *         """
 *         if time() - self.converter_cache[1] > 600:
 *             self.converter_cache[0] = self.currency_exchange_rates             # <<<<<<<<<<<<<<
 *         try:
 *             if currency_in == "USD":
 */
    __pyx_t_3 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_currency_exchange_rates); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 434, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_converter_cache); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 434, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    if (unlikely(__Pyx_SetItemInt(__pyx_t_2, 0, __pyx_t_3, long, 1, __Pyx_PyInt_From_long, 0, 0, 1) < 0)) __PYX_ERR(0, 434, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "pymarketcap/core.pyx":433
 *             Value expressed in currency_out parameter provided.
 *         """
 *         if time() - self.converter_cache[1] > 600:             # <<<<<<<<<<<<<<
 *             self.converter_cache[0] = self.currency_exchange_rates
 *         try:
 */
  }

  /* "pymarketcap/core.pyx":435
 *         if time() - self.converter_cache[1] > 600:
 *             self.converter_cache[0] = self.currency_exchange_rates
 *         try:             # <<<<<<<<<<<<<<
 *             if currency_in == "USD":
 *                 return value / self.converter_cache[0][currency_out]
 */
  {
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    __Pyx_ExceptionSave(&__pyx_t_8, &__pyx_t_9, &__pyx_t_10);
    __Pyx_XGOTREF(__pyx_t_8);
    __Pyx_XGOTREF(__pyx_t_9);
    __Pyx_XGOTREF(__pyx_t_10);
    /*try:*/ {

      /* "pymarketcap/core.pyx":436
 *             self.converter_cache[0] = self.currency_exchange_rates
 *         try:
 *             if currency_in == "USD":             # <<<<<<<<<<<<<<
 *                 return value / self.converter_cache[0][currency_out]
 *             elif currency_out == "USD":
 */
      __pyx_t_7 = (__Pyx_PyUnicode_Equals(__pyx_v_currency_in, __pyx_n_s_USD, Py_EQ)); if (unlikely(__pyx_t_7 < 0)) __PYX_ERR(0, 436, __pyx_L4_error)
      __pyx_t_11 = (__pyx_t_7 != 0);
      if (__pyx_t_11) {

        /* "pymarketcap/core.pyx":437
 *         try:
 *             if currency_in == "USD":
 *                 return value / self.converter_cache[0][currency_out]             # <<<<<<<<<<<<<<
 *             elif currency_out == "USD":
 *                 return value * self.converter_cache[0][currency_in]
 */
        __Pyx_XDECREF(__pyx_r);
        __pyx_t_3 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_converter_cache); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 437, __pyx_L4_error)
        __Pyx_GOTREF(__pyx_t_3);
        __pyx_t_2 = __Pyx_GetItemInt(__pyx_t_3, 0, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 437, __pyx_L4_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __pyx_t_3 = __Pyx_PyObject_Dict_GetItem(__pyx_t_2, __pyx_v_currency_out); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 437, __pyx_L4_error)
        __Pyx_GOTREF(__pyx_t_3);
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
        __pyx_t_2 = __Pyx_PyNumber_Divide(__pyx_v_value, __pyx_t_3); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 437, __pyx_L4_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __pyx_r = __pyx_t_2;
        __pyx_t_2 = 0;
        goto __pyx_L8_try_return;

        /* "pymarketcap/core.pyx":436
 *             self.converter_cache[0] = self.currency_exchange_rates
 *         try:
 *             if currency_in == "USD":             # <<<<<<<<<<<<<<
 *                 return value / self.converter_cache[0][currency_out]
 *             elif currency_out == "USD":
 */
      }

      /* "pymarketcap/core.pyx":438
 *             if currency_in == "USD":
 *                 return value / self.converter_cache[0][currency_out]
 *             elif currency_out == "USD":             # <<<<<<<<<<<<<<
 *                 return value * self.converter_cache[0][currency_in]
 *             else:
 */
      __pyx_t_11 = (__Pyx_PyUnicode_Equals(__pyx_v_currency_out, __pyx_n_s_USD, Py_EQ)); if (unlikely(__pyx_t_11 < 0)) __PYX_ERR(0, 438, __pyx_L4_error)
      __pyx_t_7 = (__pyx_t_11 != 0);
      if (__pyx_t_7) {

        /* "pymarketcap/core.pyx":439
 *                 return value / self.converter_cache[0][currency_out]
 *             elif currency_out == "USD":
 *                 return value * self.converter_cache[0][currency_in]             # <<<<<<<<<<<<<<
 *             else:
 *                 rates = self.converter_cache[0]
 */
        __Pyx_XDECREF(__pyx_r);
        __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_converter_cache); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 439, __pyx_L4_error)
        __Pyx_GOTREF(__pyx_t_2);
        __pyx_t_3 = __Pyx_GetItemInt(__pyx_t_2, 0, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 439, __pyx_L4_error)
        __Pyx_GOTREF(__pyx_t_3);
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
        __pyx_t_2 = __Pyx_PyObject_Dict_GetItem(__pyx_t_3, __pyx_v_currency_in); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 439, __pyx_L4_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __pyx_t_3 = PyNumber_Multiply(__pyx_v_value, __pyx_t_2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 439, __pyx_L4_error)
        __Pyx_GOTREF(__pyx_t_3);
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
        __pyx_r = __pyx_t_3;
        __pyx_t_3 = 0;
        goto __pyx_L8_try_return;

        /* "pymarketcap/core.pyx":438
 *             if currency_in == "USD":
 *                 return value / self.converter_cache[0][currency_out]
 *             elif currency_out == "USD":             # <<<<<<<<<<<<<<
 *                 return value * self.converter_cache[0][currency_in]
 *             else:
 */
      }

      /* "pymarketcap/core.pyx":441
 *                 return value * self.converter_cache[0][currency_in]
 *             else:
 *                 rates = self.converter_cache[0]             # <<<<<<<<<<<<<<
 *                 return value * rates[currency_in] / rates[currency_out]
 *         except KeyError as err:
 */
      /*else*/ {
        __pyx_t_3 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_converter_cache); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 441, __pyx_L4_error)
        __Pyx_GOTREF(__pyx_t_3);
        __pyx_t_2 = __Pyx_GetItemInt(__pyx_t_3, 0, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 441, __pyx_L4_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __pyx_v_rates = __pyx_t_2;
        __pyx_t_2 = 0;

        /* "pymarketcap/core.pyx":442
 *             else:
 *                 rates = self.converter_cache[0]
 *                 return value * rates[currency_in] / rates[currency_out]             # <<<<<<<<<<<<<<
 *         except KeyError as err:
 *             msg = "Invalid currency: '%s'. See currencies_to_convert instance attribute."
 */
        __Pyx_XDECREF(__pyx_r);
        __pyx_t_2 = __Pyx_PyObject_Dict_GetItem(__pyx_v_rates, __pyx_v_currency_in); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 442, __pyx_L4_error)
        __Pyx_GOTREF(__pyx_t_2);
        __pyx_t_3 = PyNumber_Multiply(__pyx_v_value, __pyx_t_2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 442, __pyx_L4_error)
        __Pyx_GOTREF(__pyx_t_3);
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
        __pyx_t_2 = __Pyx_PyObject_Dict_GetItem(__pyx_v_rates, __pyx_v_currency_out); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 442, __pyx_L4_error)
        __Pyx_GOTREF(__pyx_t_2);
        __pyx_t_1 = __Pyx_PyNumber_Divide(__pyx_t_3, __pyx_t_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 442, __pyx_L4_error)
        __Pyx_GOTREF(__pyx_t_1);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
        __pyx_r = __pyx_t_1;
        __pyx_t_1 = 0;
        goto __pyx_L8_try_return;
      }

      /* "pymarketcap/core.pyx":435
 *         if time() - self.converter_cache[1] > 600:
 *             self.converter_cache[0] = self.currency_exchange_rates
 *         try:             # <<<<<<<<<<<<<<
 *             if currency_in == "USD":
 *                 return value / self.converter_cache[0][currency_out]
 */
    }
    __pyx_L4_error:;
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "pymarketcap/core.pyx":443
 *                 rates = self.converter_cache[0]
 *                 return value * rates[currency_in] / rates[currency_out]
 *         except KeyError as err:             # <<<<<<<<<<<<<<
 *             msg = "Invalid currency: '%s'. See currencies_to_convert instance attribute."
 *             for param in [currency_in, currency_out]:
 */
    __pyx_t_5 = __Pyx_PyErr_ExceptionMatches(__pyx_builtin_KeyError);
    if (__pyx_t_5) {
      __Pyx_AddTraceback("pymarketcap.core.Pymarketcap.convert", __pyx_clineno, __pyx_lineno, __pyx_filename);
      if (__Pyx_GetException(&__pyx_t_1, &__pyx_t_2, &__pyx_t_3) < 0) __PYX_ERR(0, 443, __pyx_L6_except_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_2);
      __pyx_v_err = __pyx_t_2;

      /* "pymarketcap/core.pyx":444
 *                 return value * rates[currency_in] / rates[currency_out]
 *         except KeyError as err:
 *             msg = "Invalid currency: '%s'. See currencies_to_convert instance attribute."             # <<<<<<<<<<<<<<
 *             for param in [currency_in, currency_out]:
 *                 if param not in self.currencies_to_convert:
 */
      __Pyx_INCREF(__pyx_kp_s_Invalid_currency_s_See_currencie);
      __pyx_v_msg = __pyx_kp_s_Invalid_currency_s_See_currencie;

      /* "pymarketcap/core.pyx":445
 *         except KeyError as err:
 *             msg = "Invalid currency: '%s'. See currencies_to_convert instance attribute."
 *             for param in [currency_in, currency_out]:             # <<<<<<<<<<<<<<
 *                 if param not in self.currencies_to_convert:
 *                     raise ValueError(msg % param)
 */
      __pyx_t_6 = PyTuple_New(2); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 445, __pyx_L6_except_error)
      __Pyx_GOTREF(__pyx_t_6);
      __Pyx_INCREF(__pyx_v_currency_in);
      __Pyx_GIVEREF(__pyx_v_currency_in);
      PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_v_currency_in);
      __Pyx_INCREF(__pyx_v_currency_out);
      __Pyx_GIVEREF(__pyx_v_currency_out);
      PyTuple_SET_ITEM(__pyx_t_6, 1, __pyx_v_currency_out);
      __pyx_t_4 = __pyx_t_6; __Pyx_INCREF(__pyx_t_4); __pyx_t_12 = 0;
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      for (;;) {
        if (__pyx_t_12 >= 2) break;
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_6 = PyTuple_GET_ITEM(__pyx_t_4, __pyx_t_12); __Pyx_INCREF(__pyx_t_6); __pyx_t_12++; if (unlikely(0 < 0)) __PYX_ERR(0, 445, __pyx_L6_except_error)
        #else
        __pyx_t_6 = PySequence_ITEM(__pyx_t_4, __pyx_t_12); __pyx_t_12++; if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 445, __pyx_L6_except_error)
        __Pyx_GOTREF(__pyx_t_6);
        #endif
        __Pyx_XDECREF_SET(__pyx_v_param, ((PyObject*)__pyx_t_6));
        __pyx_t_6 = 0;

        /* "pymarketcap/core.pyx":446
 *             msg = "Invalid currency: '%s'. See currencies_to_convert instance attribute."
 *             for param in [currency_in, currency_out]:
 *                 if param not in self.currencies_to_convert:             # <<<<<<<<<<<<<<
 *                     raise ValueError(msg % param)
 *             raise err
 */
        __pyx_t_6 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_currencies_to_convert_2); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 446, __pyx_L6_except_error)
        __Pyx_GOTREF(__pyx_t_6);
        __pyx_t_7 = (__Pyx_PySequence_ContainsTF(__pyx_v_param, __pyx_t_6, Py_NE)); if (unlikely(__pyx_t_7 < 0)) __PYX_ERR(0, 446, __pyx_L6_except_error)
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
        __pyx_t_11 = (__pyx_t_7 != 0);
        if (unlikely(__pyx_t_11)) {

          /* "pymarketcap/core.pyx":447
 *             for param in [currency_in, currency_out]:
 *                 if param not in self.currencies_to_convert:
 *                     raise ValueError(msg % param)             # <<<<<<<<<<<<<<
 *             raise err
 * 
 */
          __pyx_t_6 = PyNumber_Remainder(__pyx_v_msg, __pyx_v_param); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 447, __pyx_L6_except_error)
          __Pyx_GOTREF(__pyx_t_6);
          __pyx_t_13 = __Pyx_PyObject_CallOneArg(__pyx_builtin_ValueError, __pyx_t_6); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 447, __pyx_L6_except_error)
          __Pyx_GOTREF(__pyx_t_13);
          __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
          __Pyx_Raise(__pyx_t_13, 0, 0, 0);
          __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
          __PYX_ERR(0, 447, __pyx_L6_except_error)

          /* "pymarketcap/core.pyx":446
 *             msg = "Invalid currency: '%s'. See currencies_to_convert instance attribute."
 *             for param in [currency_in, currency_out]:
 *                 if param not in self.currencies_to_convert:             # <<<<<<<<<<<<<<
 *                     raise ValueError(msg % param)
 *             raise err
 */
        }

        /* "pymarketcap/core.pyx":445
 *         except KeyError as err:
 *             msg = "Invalid currency: '%s'. See currencies_to_convert instance attribute."
 *             for param in [currency_in, currency_out]:             # <<<<<<<<<<<<<<
 *                 if param not in self.currencies_to_convert:
 *                     raise ValueError(msg % param)
 */
      }
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

      /* "pymarketcap/core.pyx":448
 *                 if param not in self.currencies_to_convert:
 *                     raise ValueError(msg % param)
 *             raise err             # <<<<<<<<<<<<<<
 * 
 *     cpdef currency(self, unicode name, convert="USD"):
 */
      __Pyx_Raise(__pyx_v_err, 0, 0, 0);
      __PYX_ERR(0, 448, __pyx_L6_except_error)
    }
    goto __pyx_L6_except_error;
    __pyx_L6_except_error:;

    /* "pymarketcap/core.pyx":435
 *         if time() - self.converter_cache[1] > 600:
 *             self.converter_cache[0] = self.currency_exchange_rates
 *         try:             # <<<<<<<<<<<<<<
 *             if currency_in == "USD":
 *                 return value / self.converter_cache[0][currency_out]
 */
    __Pyx_XGIVEREF(__pyx_t_8);
    __Pyx_XGIVEREF(__pyx_t_9);
    __Pyx_XGIVEREF(__pyx_t_10);
    __Pyx_ExceptionReset(__pyx_t_8, __pyx_t_9, __pyx_t_10);
    goto __pyx_L1_error;
    __pyx_L8_try_return:;
    __Pyx_XGIVEREF(__pyx_t_8);
    __Pyx_XGIVEREF(__pyx_t_9);
    __Pyx_XGIVEREF(__pyx_t_10);
    __Pyx_ExceptionReset(__pyx_t_8, __pyx_t_9, __pyx_t_10);
    goto __pyx_L0;
  }

  /* "pymarketcap/core.pyx":419
 *         return response
 * 
 *     cpdef convert(self, value, unicode currency_in, unicode currency_out):             # <<<<<<<<<<<<<<
 *         """Convert prices between currencies. Provide a value, the currency
 *         of the value and the currency to convert it and get the value in
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_13);
  __Pyx_AddTraceback("pymarketcap.core.Pymarketcap.convert", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_rates);
  __Pyx_XDECREF(__pyx_v_err);
  __Pyx_XDECREF(__pyx_v_msg);
  __Pyx_XDECREF(__pyx_v_param);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_11pymarketcap_4core_11Pymarketcap_19convert(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_11pymarketcap_4core_11Pymarketcap_18convert[] = "Convert prices between currencies. Provide a value, the currency\n        of the value and the currency to convert it and get the value in\n        currency converted rate. For see all available currencies to convert\n        see ``currencies_to_convert`` property.\n\n        Args:\n            value (int/float): Value to convert betweeen two currencies.\n            currency_in (str): Currency in which is expressed the value passed.\n            currency_out (str): Currency to convert.\n\n        Returns (float):\n            Value expressed in currency_out parameter provided.\n        ";
static PyObject *__pyx_pw_11pymarketcap_4core_11Pymarketcap_19convert(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_value = 0;
  PyObject *__pyx_v_currency_in = 0;
  PyObject *__pyx_v_currency_out = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("convert (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_value,&__pyx_n_s_currency_in,&__pyx_n_s_currency_out,0};
    PyObject* values[3] = {0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_value)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_currency_in)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("convert", 1, 3, 3, 1); __PYX_ERR(0, 419, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_currency_out)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("convert", 1, 3, 3, 2); __PYX_ERR(0, 419, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "convert") < 0)) __PYX_ERR(0, 419, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 3) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
    }
    __pyx_v_value = values[0];
    __pyx_v_currency_in = ((PyObject*)values[1]);
    __pyx_v_currency_out = ((PyObject*)values[2]);
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("convert", 1, 3, 3, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 419, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("pymarketcap.core.Pymarketcap.convert", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_currency_in), (&PyUnicode_Type), 1, "currency_in", 1))) __PYX_ERR(0, 419, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_currency_out), (&PyUnicode_Type), 1, "currency_out", 1))) __PYX_ERR(0, 419, __pyx_L1_error)
  __pyx_r = __pyx_pf_11pymarketcap_4core_11Pymarketcap_18convert(((struct __pyx_obj_11pymarketcap_4core_Pymarketcap *)__pyx_v_self), __pyx_v_value, __pyx_v_currency_in, __pyx_v_currency_out);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_11pymarketcap_4core_11Pymarketcap_18convert(struct __pyx_obj_11pymarketcap_4core_Pymarketcap *__pyx_v_self, PyObject *__pyx_v_value, PyObject *__pyx_v_currency_in, PyObject *__pyx_v_currency_out) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("convert", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_11pymarketcap_4core_11Pymarketcap_convert(__pyx_v_self, __pyx_v_value, __pyx_v_currency_in, __pyx_v_currency_out, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 419, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pymarketcap.core.Pymarketcap.convert", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pymarketcap/core.pyx":450
 *             raise err
 * 
 *     cpdef currency(self, unicode name, convert="USD"):             # <<<<<<<<<<<<<<
 *         """Get currency metadata like total markets capitalization,
 *         websites, source code link, if mineable...
 */

static PyObject *__pyx_pw_11pymarketcap_4core_11Pymarketcap_21currency(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyObject *__pyx_f_11pymarketcap_4core_11Pymarketcap_currency(struct __pyx_obj_11pymarketcap_4core_Pymarketcap *__pyx_v_self, PyObject *__pyx_v_name, int __pyx_skip_dispatch, struct __pyx_opt_args_11pymarketcap_4core_11Pymarketcap_currency *__pyx_optional_args) {
  PyObject *__pyx_v_convert = ((PyObject *)__pyx_n_s_USD);
  PyObject *__pyx_v_response = NULL;
  PyObject *__pyx_v_symbol = NULL;
  PyObject *__pyx_v_slug = NULL;
  PyObject *__pyx_v_res = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_t_7;
  Py_ssize_t __pyx_t_8;
  PyObject *(*__pyx_t_9)(PyObject *);
  PyObject *(*__pyx_t_10)(PyObject *);
  PyObject *__pyx_t_11 = NULL;
  PyObject *__pyx_t_12 = NULL;
  PyObject *__pyx_t_13 = NULL;
  char *__pyx_t_14;
  int __pyx_t_15;
  __Pyx_RefNannySetupContext("currency", 0);
  if (__pyx_optional_args) {
    if (__pyx_optional_args->__pyx_n > 0) {
      __pyx_v_convert = __pyx_optional_args->convert;
    }
  }
  __Pyx_INCREF(__pyx_v_name);
  __Pyx_INCREF(__pyx_v_convert);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0)) {
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_currency); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 450, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)__pyx_pw_11pymarketcap_4core_11Pymarketcap_21currency)) {
      __Pyx_XDECREF(__pyx_r);
      __Pyx_INCREF(__pyx_t_1);
      __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
      __pyx_t_5 = 0;
      if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
        __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
        if (likely(__pyx_t_4)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
          __Pyx_INCREF(__pyx_t_4);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_3, function);
          __pyx_t_5 = 1;
        }
      }
      #if CYTHON_FAST_PYCALL
      if (PyFunction_Check(__pyx_t_3)) {
        PyObject *__pyx_temp[3] = {__pyx_t_4, __pyx_v_name, __pyx_v_convert};
        __pyx_t_2 = __Pyx_PyFunction_FastCall(__pyx_t_3, __pyx_temp+1-__pyx_t_5, 2+__pyx_t_5); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 450, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        __Pyx_GOTREF(__pyx_t_2);
      } else
      #endif
      #if CYTHON_FAST_PYCCALL
      if (__Pyx_PyFastCFunction_Check(__pyx_t_3)) {
        PyObject *__pyx_temp[3] = {__pyx_t_4, __pyx_v_name, __pyx_v_convert};
        __pyx_t_2 = __Pyx_PyCFunction_FastCall(__pyx_t_3, __pyx_temp+1-__pyx_t_5, 2+__pyx_t_5); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 450, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        __Pyx_GOTREF(__pyx_t_2);
      } else
      #endif
      {
        __pyx_t_6 = PyTuple_New(2+__pyx_t_5); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 450, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_6);
        if (__pyx_t_4) {
          __Pyx_GIVEREF(__pyx_t_4); PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_4); __pyx_t_4 = NULL;
        }
        __Pyx_INCREF(__pyx_v_name);
        __Pyx_GIVEREF(__pyx_v_name);
        PyTuple_SET_ITEM(__pyx_t_6, 0+__pyx_t_5, __pyx_v_name);
        __Pyx_INCREF(__pyx_v_convert);
        __Pyx_GIVEREF(__pyx_v_convert);
        PyTuple_SET_ITEM(__pyx_t_6, 1+__pyx_t_5, __pyx_v_convert);
        __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_t_6, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 450, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      }
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __pyx_r = __pyx_t_2;
      __pyx_t_2 = 0;
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      goto __pyx_L0;
    }
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  }

  /* "pymarketcap/core.pyx":463
 *             Aditional general metadata not supported by other methods.
 *         """
 *         response = {}             # <<<<<<<<<<<<<<
 *         if self._is_symbol(name):
 *             response["symbol"] = name
 */
  __pyx_t_1 = __Pyx_PyDict_NewPresized(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 463, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_response = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "pymarketcap/core.pyx":464
 *         """
 *         response = {}
 *         if self._is_symbol(name):             # <<<<<<<<<<<<<<
 *             response["symbol"] = name
 *             name = self.correspondences[name]
 */
  __pyx_t_1 = ((struct __pyx_vtabstruct_11pymarketcap_4core_Pymarketcap *)__pyx_v_self->__pyx_vtab)->_is_symbol(__pyx_v_self, __pyx_v_name, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 464, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_7 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_7 < 0)) __PYX_ERR(0, 464, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (__pyx_t_7) {

    /* "pymarketcap/core.pyx":465
 *         response = {}
 *         if self._is_symbol(name):
 *             response["symbol"] = name             # <<<<<<<<<<<<<<
 *             name = self.correspondences[name]
 *             response["slug"] = name
 */
    if (unlikely(PyDict_SetItem(__pyx_v_response, __pyx_n_s_symbol, __pyx_v_name) < 0)) __PYX_ERR(0, 465, __pyx_L1_error)

    /* "pymarketcap/core.pyx":466
 *         if self._is_symbol(name):
 *             response["symbol"] = name
 *             name = self.correspondences[name]             # <<<<<<<<<<<<<<
 *             response["slug"] = name
 *         else:
 */
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_correspondences); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 466, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_2 = __Pyx_PyObject_Dict_GetItem(__pyx_t_1, __pyx_v_name); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 466, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    if (!(likely(PyUnicode_CheckExact(__pyx_t_2))||((__pyx_t_2) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "unicode", Py_TYPE(__pyx_t_2)->tp_name), 0))) __PYX_ERR(0, 466, __pyx_L1_error)
    __Pyx_DECREF_SET(__pyx_v_name, ((PyObject*)__pyx_t_2));
    __pyx_t_2 = 0;

    /* "pymarketcap/core.pyx":467
 *             response["symbol"] = name
 *             name = self.correspondences[name]
 *             response["slug"] = name             # <<<<<<<<<<<<<<
 *         else:
 *             response["slug"] = name
 */
    if (unlikely(PyDict_SetItem(__pyx_v_response, __pyx_n_s_slug, __pyx_v_name) < 0)) __PYX_ERR(0, 467, __pyx_L1_error)

    /* "pymarketcap/core.pyx":464
 *         """
 *         response = {}
 *         if self._is_symbol(name):             # <<<<<<<<<<<<<<
 *             response["symbol"] = name
 *             name = self.correspondences[name]
 */
    goto __pyx_L3;
  }

  /* "pymarketcap/core.pyx":469
 *             response["slug"] = name
 *         else:
 *             response["slug"] = name             # <<<<<<<<<<<<<<
 *             for symbol, slug in self.correspondences.items():
 *                 if slug == name:
 */
  /*else*/ {
    if (unlikely(PyDict_SetItem(__pyx_v_response, __pyx_n_s_slug, __pyx_v_name) < 0)) __PYX_ERR(0, 469, __pyx_L1_error)

    /* "pymarketcap/core.pyx":470
 *         else:
 *             response["slug"] = name
 *             for symbol, slug in self.correspondences.items():             # <<<<<<<<<<<<<<
 *                 if slug == name:
 *                     response["symbol"] = symbol
 */
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_correspondences); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 470, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_items); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 470, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_1 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_3))) {
      __pyx_t_1 = PyMethod_GET_SELF(__pyx_t_3);
      if (likely(__pyx_t_1)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
        __Pyx_INCREF(__pyx_t_1);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_3, function);
      }
    }
    if (__pyx_t_1) {
      __pyx_t_2 = __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 470, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    } else {
      __pyx_t_2 = __Pyx_PyObject_CallNoArg(__pyx_t_3); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 470, __pyx_L1_error)
    }
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (likely(PyList_CheckExact(__pyx_t_2)) || PyTuple_CheckExact(__pyx_t_2)) {
      __pyx_t_3 = __pyx_t_2; __Pyx_INCREF(__pyx_t_3); __pyx_t_8 = 0;
      __pyx_t_9 = NULL;
    } else {
      __pyx_t_8 = -1; __pyx_t_3 = PyObject_GetIter(__pyx_t_2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 470, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __pyx_t_9 = Py_TYPE(__pyx_t_3)->tp_iternext; if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 470, __pyx_L1_error)
    }
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    for (;;) {
      if (likely(!__pyx_t_9)) {
        if (likely(PyList_CheckExact(__pyx_t_3))) {
          if (__pyx_t_8 >= PyList_GET_SIZE(__pyx_t_3)) break;
          #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
          __pyx_t_2 = PyList_GET_ITEM(__pyx_t_3, __pyx_t_8); __Pyx_INCREF(__pyx_t_2); __pyx_t_8++; if (unlikely(0 < 0)) __PYX_ERR(0, 470, __pyx_L1_error)
          #else
          __pyx_t_2 = PySequence_ITEM(__pyx_t_3, __pyx_t_8); __pyx_t_8++; if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 470, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_2);
          #endif
        } else {
          if (__pyx_t_8 >= PyTuple_GET_SIZE(__pyx_t_3)) break;
          #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
          __pyx_t_2 = PyTuple_GET_ITEM(__pyx_t_3, __pyx_t_8); __Pyx_INCREF(__pyx_t_2); __pyx_t_8++; if (unlikely(0 < 0)) __PYX_ERR(0, 470, __pyx_L1_error)
          #else
          __pyx_t_2 = PySequence_ITEM(__pyx_t_3, __pyx_t_8); __pyx_t_8++; if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 470, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_2);
          #endif
        }
      } else {
        __pyx_t_2 = __pyx_t_9(__pyx_t_3);
        if (unlikely(!__pyx_t_2)) {
          PyObject* exc_type = PyErr_Occurred();
          if (exc_type) {
            if (likely(__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
            else __PYX_ERR(0, 470, __pyx_L1_error)
          }
          break;
        }
        __Pyx_GOTREF(__pyx_t_2);
      }
      if ((likely(PyTuple_CheckExact(__pyx_t_2))) || (PyList_CheckExact(__pyx_t_2))) {
        PyObject* sequence = __pyx_t_2;
        Py_ssize_t size = __Pyx_PySequence_SIZE(sequence);
        if (unlikely(size != 2)) {
          if (size > 2) __Pyx_RaiseTooManyValuesError(2);
          else if (size >= 0) __Pyx_RaiseNeedMoreValuesError(size);
          __PYX_ERR(0, 470, __pyx_L1_error)
        }
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        if (likely(PyTuple_CheckExact(sequence))) {
          __pyx_t_1 = PyTuple_GET_ITEM(sequence, 0); 
          __pyx_t_6 = PyTuple_GET_ITEM(sequence, 1); 
        } else {
          __pyx_t_1 = PyList_GET_ITEM(sequence, 0); 
          __pyx_t_6 = PyList_GET_ITEM(sequence, 1); 
        }
        __Pyx_INCREF(__pyx_t_1);
        __Pyx_INCREF(__pyx_t_6);
        #else
        __pyx_t_1 = PySequence_ITEM(sequence, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 470, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_1);
        __pyx_t_6 = PySequence_ITEM(sequence, 1); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 470, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_6);
        #endif
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      } else {
        Py_ssize_t index = -1;
        __pyx_t_4 = PyObject_GetIter(__pyx_t_2); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 470, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_4);
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
        __pyx_t_10 = Py_TYPE(__pyx_t_4)->tp_iternext;
        index = 0; __pyx_t_1 = __pyx_t_10(__pyx_t_4); if (unlikely(!__pyx_t_1)) goto __pyx_L6_unpacking_failed;
        __Pyx_GOTREF(__pyx_t_1);
        index = 1; __pyx_t_6 = __pyx_t_10(__pyx_t_4); if (unlikely(!__pyx_t_6)) goto __pyx_L6_unpacking_failed;
        __Pyx_GOTREF(__pyx_t_6);
        if (__Pyx_IternextUnpackEndCheck(__pyx_t_10(__pyx_t_4), 2) < 0) __PYX_ERR(0, 470, __pyx_L1_error)
        __pyx_t_10 = NULL;
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
        goto __pyx_L7_unpacking_done;
        __pyx_L6_unpacking_failed:;
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
        __pyx_t_10 = NULL;
        if (__Pyx_IterFinish() == 0) __Pyx_RaiseNeedMoreValuesError(index);
        __PYX_ERR(0, 470, __pyx_L1_error)
        __pyx_L7_unpacking_done:;
      }
      __Pyx_XDECREF_SET(__pyx_v_symbol, __pyx_t_1);
      __pyx_t_1 = 0;
      __Pyx_XDECREF_SET(__pyx_v_slug, __pyx_t_6);
      __pyx_t_6 = 0;

      /* "pymarketcap/core.pyx":471
 *             response["slug"] = name
 *             for symbol, slug in self.correspondences.items():
 *                 if slug == name:             # <<<<<<<<<<<<<<
 *                     response["symbol"] = symbol
 *                     break
 */
      __pyx_t_7 = (__Pyx_PyUnicode_Equals(__pyx_v_slug, __pyx_v_name, Py_EQ)); if (unlikely(__pyx_t_7 < 0)) __PYX_ERR(0, 471, __pyx_L1_error)
      if (__pyx_t_7) {

        /* "pymarketcap/core.pyx":472
 *             for symbol, slug in self.correspondences.items():
 *                 if slug == name:
 *                     response["symbol"] = symbol             # <<<<<<<<<<<<<<
 *                     break
 *         convert = convert.lower()
 */
        if (unlikely(PyDict_SetItem(__pyx_v_response, __pyx_n_s_symbol, __pyx_v_symbol) < 0)) __PYX_ERR(0, 472, __pyx_L1_error)

        /* "pymarketcap/core.pyx":473
 *                 if slug == name:
 *                     response["symbol"] = symbol
 *                     break             # <<<<<<<<<<<<<<
 *         convert = convert.lower()
 * 
 */
        goto __pyx_L5_break;

        /* "pymarketcap/core.pyx":471
 *             response["slug"] = name
 *             for symbol, slug in self.correspondences.items():
 *                 if slug == name:             # <<<<<<<<<<<<<<
 *                     response["symbol"] = symbol
 *                     break
 */
      }

      /* "pymarketcap/core.pyx":470
 *         else:
 *             response["slug"] = name
 *             for symbol, slug in self.correspondences.items():             # <<<<<<<<<<<<<<
 *                 if slug == name:
 *                     response["symbol"] = symbol
 */
    }
    __pyx_L5_break:;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  }
  __pyx_L3:;

  /* "pymarketcap/core.pyx":474
 *                     response["symbol"] = symbol
 *                     break
 *         convert = convert.lower()             # <<<<<<<<<<<<<<
 * 
 *         try:
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_convert, __pyx_n_s_lower); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 474, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_6 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_6)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_6);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  if (__pyx_t_6) {
    __pyx_t_3 = __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_6); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 474, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  } else {
    __pyx_t_3 = __Pyx_PyObject_CallNoArg(__pyx_t_2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 474, __pyx_L1_error)
  }
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF_SET(__pyx_v_convert, __pyx_t_3);
  __pyx_t_3 = 0;

  /* "pymarketcap/core.pyx":476
 *         convert = convert.lower()
 * 
 *         try:             # <<<<<<<<<<<<<<
 *             res = self._get(
 *                 b"https://coinmarketcap.com/currencies/%s/" % name.encode()
 */
  {
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    __Pyx_ExceptionSave(&__pyx_t_11, &__pyx_t_12, &__pyx_t_13);
    __Pyx_XGOTREF(__pyx_t_11);
    __Pyx_XGOTREF(__pyx_t_12);
    __Pyx_XGOTREF(__pyx_t_13);
    /*try:*/ {

      /* "pymarketcap/core.pyx":478
 *         try:
 *             res = self._get(
 *                 b"https://coinmarketcap.com/currencies/%s/" % name.encode()             # <<<<<<<<<<<<<<
 *             )[20000:]
 *         except CoinmarketcapHTTPError404:
 */
      if (unlikely(__pyx_v_name == Py_None)) {
        PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "encode");
        __PYX_ERR(0, 478, __pyx_L9_error)
      }
      __pyx_t_3 = PyUnicode_AsEncodedString(__pyx_v_name, NULL, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 478, __pyx_L9_error)
      __Pyx_GOTREF(__pyx_t_3);
      __pyx_t_2 = PyNumber_Remainder(__pyx_kp_b_https_coinmarketcap_com_currenci, __pyx_t_3); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 478, __pyx_L9_error)
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __pyx_t_14 = __Pyx_PyObject_AsWritableString(__pyx_t_2); if (unlikely((!__pyx_t_14) && PyErr_Occurred())) __PYX_ERR(0, 478, __pyx_L9_error)

      /* "pymarketcap/core.pyx":477
 * 
 *         try:
 *             res = self._get(             # <<<<<<<<<<<<<<
 *                 b"https://coinmarketcap.com/currencies/%s/" % name.encode()
 *             )[20000:]
 */
      __pyx_t_3 = ((struct __pyx_vtabstruct_11pymarketcap_4core_Pymarketcap *)__pyx_v_self->__pyx_vtab)->_get(__pyx_v_self, __pyx_t_14); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 477, __pyx_L9_error)
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

      /* "pymarketcap/core.pyx":479
 *             res = self._get(
 *                 b"https://coinmarketcap.com/currencies/%s/" % name.encode()
 *             )[20000:]             # <<<<<<<<<<<<<<
 *         except CoinmarketcapHTTPError404:
 *             if name not in self.coins:
 */
      __pyx_t_2 = __Pyx_PyObject_GetSlice(__pyx_t_3, 0x4E20, 0, NULL, NULL, &__pyx_slice__7, 1, 0, 1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 479, __pyx_L9_error)
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __pyx_v_res = __pyx_t_2;
      __pyx_t_2 = 0;

      /* "pymarketcap/core.pyx":476
 *         convert = convert.lower()
 * 
 *         try:             # <<<<<<<<<<<<<<
 *             res = self._get(
 *                 b"https://coinmarketcap.com/currencies/%s/" % name.encode()
 */
    }
    __Pyx_XDECREF(__pyx_t_11); __pyx_t_11 = 0;
    __Pyx_XDECREF(__pyx_t_12); __pyx_t_12 = 0;
    __Pyx_XDECREF(__pyx_t_13); __pyx_t_13 = 0;
    goto __pyx_L14_try_end;
    __pyx_L9_error:;
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;

    /* "pymarketcap/core.pyx":480
 *                 b"https://coinmarketcap.com/currencies/%s/" % name.encode()
 *             )[20000:]
 *         except CoinmarketcapHTTPError404:             # <<<<<<<<<<<<<<
 *             if name not in self.coins:
 *                 raise ValueError("%s is not a valid currency name. See 'symbols' or 'coins'" % name \
 */
    __Pyx_ErrFetch(&__pyx_t_2, &__pyx_t_3, &__pyx_t_6);
    __pyx_t_1 = __Pyx_GetModuleGlobalName(__pyx_n_s_CoinmarketcapHTTPError404); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 480, __pyx_L11_except_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_5 = __Pyx_PyErr_GivenExceptionMatches(__pyx_t_2, __pyx_t_1);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_ErrRestore(__pyx_t_2, __pyx_t_3, __pyx_t_6);
    __pyx_t_2 = 0; __pyx_t_3 = 0; __pyx_t_6 = 0;
    if (__pyx_t_5) {
      __Pyx_AddTraceback("pymarketcap.core.Pymarketcap.currency", __pyx_clineno, __pyx_lineno, __pyx_filename);
      if (__Pyx_GetException(&__pyx_t_6, &__pyx_t_3, &__pyx_t_2) < 0) __PYX_ERR(0, 480, __pyx_L11_except_error)
      __Pyx_GOTREF(__pyx_t_6);
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_GOTREF(__pyx_t_2);

      /* "pymarketcap/core.pyx":481
 *             )[20000:]
 *         except CoinmarketcapHTTPError404:
 *             if name not in self.coins:             # <<<<<<<<<<<<<<
 *                 raise ValueError("%s is not a valid currency name. See 'symbols' or 'coins'" % name \
 *                                  + " properties for get all valid currencies.")
 */
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_coins); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 481, __pyx_L11_except_error)
      __Pyx_GOTREF(__pyx_t_1);
      __pyx_t_7 = (__Pyx_PySequence_ContainsTF(__pyx_v_name, __pyx_t_1, Py_NE)); if (unlikely(__pyx_t_7 < 0)) __PYX_ERR(0, 481, __pyx_L11_except_error)
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __pyx_t_15 = (__pyx_t_7 != 0);
      if (unlikely(__pyx_t_15)) {

        /* "pymarketcap/core.pyx":482
 *         except CoinmarketcapHTTPError404:
 *             if name not in self.coins:
 *                 raise ValueError("%s is not a valid currency name. See 'symbols' or 'coins'" % name \             # <<<<<<<<<<<<<<
 *                                  + " properties for get all valid currencies.")
 *             else:
 */
        __pyx_t_1 = __Pyx_PyString_Format(__pyx_kp_s_s_is_not_a_valid_currency_name, __pyx_v_name); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 482, __pyx_L11_except_error)
        __Pyx_GOTREF(__pyx_t_1);

        /* "pymarketcap/core.pyx":483
 *             if name not in self.coins:
 *                 raise ValueError("%s is not a valid currency name. See 'symbols' or 'coins'" % name \
 *                                  + " properties for get all valid currencies.")             # <<<<<<<<<<<<<<
 *             else:
 *                 raise NotImplementedError
 */
        __pyx_t_4 = __Pyx_PyUnicode_Concat(__pyx_t_1, __pyx_kp_s_properties_for_get_all_valid_cu); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 483, __pyx_L11_except_error)
        __Pyx_GOTREF(__pyx_t_4);
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

        /* "pymarketcap/core.pyx":482
 *         except CoinmarketcapHTTPError404:
 *             if name not in self.coins:
 *                 raise ValueError("%s is not a valid currency name. See 'symbols' or 'coins'" % name \             # <<<<<<<<<<<<<<
 *                                  + " properties for get all valid currencies.")
 *             else:
 */
        __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_builtin_ValueError, __pyx_t_4); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 482, __pyx_L11_except_error)
        __Pyx_GOTREF(__pyx_t_1);
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
        __Pyx_Raise(__pyx_t_1, 0, 0, 0);
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        __PYX_ERR(0, 482, __pyx_L11_except_error)

        /* "pymarketcap/core.pyx":481
 *             )[20000:]
 *         except CoinmarketcapHTTPError404:
 *             if name not in self.coins:             # <<<<<<<<<<<<<<
 *                 raise ValueError("%s is not a valid currency name. See 'symbols' or 'coins'" % name \
 *                                  + " properties for get all valid currencies.")
 */
      }

      /* "pymarketcap/core.pyx":485
 *                                  + " properties for get all valid currencies.")
 *             else:
 *                 raise NotImplementedError             # <<<<<<<<<<<<<<
 * 
 *         response.update(processer.currency(res, convert))
 */
      /*else*/ {
        __Pyx_Raise(__pyx_builtin_NotImplementedError, 0, 0, 0);
        __PYX_ERR(0, 485, __pyx_L11_except_error)
      }
    }
    goto __pyx_L11_except_error;
    __pyx_L11_except_error:;

    /* "pymarketcap/core.pyx":476
 *         convert = convert.lower()
 * 
 *         try:             # <<<<<<<<<<<<<<
 *             res = self._get(
 *                 b"https://coinmarketcap.com/currencies/%s/" % name.encode()
 */
    __Pyx_XGIVEREF(__pyx_t_11);
    __Pyx_XGIVEREF(__pyx_t_12);
    __Pyx_XGIVEREF(__pyx_t_13);
    __Pyx_ExceptionReset(__pyx_t_11, __pyx_t_12, __pyx_t_13);
    goto __pyx_L1_error;
    __pyx_L14_try_end:;
  }

  /* "pymarketcap/core.pyx":487
 *                 raise NotImplementedError
 * 
 *         response.update(processer.currency(res, convert))             # <<<<<<<<<<<<<<
 *         return response
 * 
 */
  __pyx_t_3 = __Pyx_GetModuleGlobalName(__pyx_n_s_processer); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 487, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_currency); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 487, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = NULL;
  __pyx_t_5 = 0;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_6))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_6);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_6);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_6, function);
      __pyx_t_5 = 1;
    }
  }
  #if CYTHON_FAST_PYCALL
  if (PyFunction_Check(__pyx_t_6)) {
    PyObject *__pyx_temp[3] = {__pyx_t_3, __pyx_v_res, __pyx_v_convert};
    __pyx_t_2 = __Pyx_PyFunction_FastCall(__pyx_t_6, __pyx_temp+1-__pyx_t_5, 2+__pyx_t_5); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 487, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_GOTREF(__pyx_t_2);
  } else
  #endif
  #if CYTHON_FAST_PYCCALL
  if (__Pyx_PyFastCFunction_Check(__pyx_t_6)) {
    PyObject *__pyx_temp[3] = {__pyx_t_3, __pyx_v_res, __pyx_v_convert};
    __pyx_t_2 = __Pyx_PyCFunction_FastCall(__pyx_t_6, __pyx_temp+1-__pyx_t_5, 2+__pyx_t_5); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 487, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_GOTREF(__pyx_t_2);
  } else
  #endif
  {
    __pyx_t_1 = PyTuple_New(2+__pyx_t_5); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 487, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    if (__pyx_t_3) {
      __Pyx_GIVEREF(__pyx_t_3); PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_t_3); __pyx_t_3 = NULL;
    }
    __Pyx_INCREF(__pyx_v_res);
    __Pyx_GIVEREF(__pyx_v_res);
    PyTuple_SET_ITEM(__pyx_t_1, 0+__pyx_t_5, __pyx_v_res);
    __Pyx_INCREF(__pyx_v_convert);
    __Pyx_GIVEREF(__pyx_v_convert);
    PyTuple_SET_ITEM(__pyx_t_1, 1+__pyx_t_5, __pyx_v_convert);
    __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_6, __pyx_t_1, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 487, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  }
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  __pyx_t_6 = __Pyx_CallUnboundCMethod1(&__pyx_umethod_PyDict_Type_update, __pyx_v_response, __pyx_t_2); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 487, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;

  /* "pymarketcap/core.pyx":488
 * 
 *         response.update(processer.currency(res, convert))
 *         return response             # <<<<<<<<<<<<<<
 * 
 *     cpdef markets(self, unicode name, convert="USD"):
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_response);
  __pyx_r = __pyx_v_response;
  goto __pyx_L0;

  /* "pymarketcap/core.pyx":450
 *             raise err
 * 
 *     cpdef currency(self, unicode name, convert="USD"):             # <<<<<<<<<<<<<<
 *         """Get currency metadata like total markets capitalization,
 *         websites, source code link, if mineable...
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_AddTraceback("pymarketcap.core.Pymarketcap.currency", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_response);
  __Pyx_XDECREF(__pyx_v_symbol);
  __Pyx_XDECREF(__pyx_v_slug);
  __Pyx_XDECREF(__pyx_v_res);
  __Pyx_XDECREF(__pyx_v_name);
  __Pyx_XDECREF(__pyx_v_convert);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_11pymarketcap_4core_11Pymarketcap_21currency(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_11pymarketcap_4core_11Pymarketcap_20currency[] = "Get currency metadata like total markets capitalization,\n        websites, source code link, if mineable...\n\n        Args:\n            currency (str): Currency to get metadata.\n            convert (str, optional): Currency to convert response\n                fields ``total_markets_cap``, ``total_markets_volume_24h``\n                and ``price`` between USD and BTC. As default ``\"USD\"``.\n\n        Returns (dict):\n            Aditional general metadata not supported by other methods.\n        ";
static PyObject *__pyx_pw_11pymarketcap_4core_11Pymarketcap_21currency(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_name = 0;
  PyObject *__pyx_v_convert = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("currency (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_name,&__pyx_n_s_convert,0};
    PyObject* values[2] = {0,0};
    values[1] = ((PyObject *)__pyx_n_s_USD);
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_name)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_convert);
          if (value) { values[1] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "currency") < 0)) __PYX_ERR(0, 450, __pyx_L3_error)
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_name = ((PyObject*)values[0]);
    __pyx_v_convert = values[1];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("currency", 0, 1, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 450, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("pymarketcap.core.Pymarketcap.currency", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_name), (&PyUnicode_Type), 1, "name", 1))) __PYX_ERR(0, 450, __pyx_L1_error)
  __pyx_r = __pyx_pf_11pymarketcap_4core_11Pymarketcap_20currency(((struct __pyx_obj_11pymarketcap_4core_Pymarketcap *)__pyx_v_self), __pyx_v_name, __pyx_v_convert);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_11pymarketcap_4core_11Pymarketcap_20currency(struct __pyx_obj_11pymarketcap_4core_Pymarketcap *__pyx_v_self, PyObject *__pyx_v_name, PyObject *__pyx_v_convert) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  struct __pyx_opt_args_11pymarketcap_4core_11Pymarketcap_currency __pyx_t_2;
  __Pyx_RefNannySetupContext("currency", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2.__pyx_n = 1;
  __pyx_t_2.convert = __pyx_v_convert;
  __pyx_t_1 = __pyx_vtabptr_11pymarketcap_4core_Pymarketcap->currency(__pyx_v_self, __pyx_v_name, 1, &__pyx_t_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 450, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pymarketcap.core.Pymarketcap.currency", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pymarketcap/core.pyx":490
 *         return response
 * 
 *     cpdef markets(self, unicode name, convert="USD"):             # <<<<<<<<<<<<<<
 *         """Get available coinmarketcap markets data.
 *         It needs a currency as argument.
 */

static PyObject *__pyx_pw_11pymarketcap_4core_11Pymarketcap_23markets(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyObject *__pyx_f_11pymarketcap_4core_11Pymarketcap_markets(struct __pyx_obj_11pymarketcap_4core_Pymarketcap *__pyx_v_self, PyObject *__pyx_v_name, int __pyx_skip_dispatch, struct __pyx_opt_args_11pymarketcap_4core_11Pymarketcap_markets *__pyx_optional_args) {
  PyObject *__pyx_v_convert = ((PyObject *)__pyx_n_s_USD);
  PyObject *__pyx_v_response = NULL;
  PyObject *__pyx_v_symbol = NULL;
  PyObject *__pyx_v_slug = NULL;
  PyObject *__pyx_v_res = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_t_7;
  Py_ssize_t __pyx_t_8;
  PyObject *(*__pyx_t_9)(PyObject *);
  PyObject *(*__pyx_t_10)(PyObject *);
  PyObject *__pyx_t_11 = NULL;
  PyObject *__pyx_t_12 = NULL;
  PyObject *__pyx_t_13 = NULL;
  char *__pyx_t_14;
  int __pyx_t_15;
  __Pyx_RefNannySetupContext("markets", 0);
  if (__pyx_optional_args) {
    if (__pyx_optional_args->__pyx_n > 0) {
      __pyx_v_convert = __pyx_optional_args->convert;
    }
  }
  __Pyx_INCREF(__pyx_v_name);
  __Pyx_INCREF(__pyx_v_convert);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0)) {
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_markets); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 490, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)__pyx_pw_11pymarketcap_4core_11Pymarketcap_23markets)) {
      __Pyx_XDECREF(__pyx_r);
      __Pyx_INCREF(__pyx_t_1);
      __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
      __pyx_t_5 = 0;
      if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
        __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
        if (likely(__pyx_t_4)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
          __Pyx_INCREF(__pyx_t_4);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_3, function);
          __pyx_t_5 = 1;
        }
      }
      #if CYTHON_FAST_PYCALL
      if (PyFunction_Check(__pyx_t_3)) {
        PyObject *__pyx_temp[3] = {__pyx_t_4, __pyx_v_name, __pyx_v_convert};
        __pyx_t_2 = __Pyx_PyFunction_FastCall(__pyx_t_3, __pyx_temp+1-__pyx_t_5, 2+__pyx_t_5); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 490, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        __Pyx_GOTREF(__pyx_t_2);
      } else
      #endif
      #if CYTHON_FAST_PYCCALL
      if (__Pyx_PyFastCFunction_Check(__pyx_t_3)) {
        PyObject *__pyx_temp[3] = {__pyx_t_4, __pyx_v_name, __pyx_v_convert};
        __pyx_t_2 = __Pyx_PyCFunction_FastCall(__pyx_t_3, __pyx_temp+1-__pyx_t_5, 2+__pyx_t_5); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 490, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        __Pyx_GOTREF(__pyx_t_2);
      } else
      #endif
      {
        __pyx_t_6 = PyTuple_New(2+__pyx_t_5); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 490, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_6);
        if (__pyx_t_4) {
          __Pyx_GIVEREF(__pyx_t_4); PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_4); __pyx_t_4 = NULL;
        }
        __Pyx_INCREF(__pyx_v_name);
        __Pyx_GIVEREF(__pyx_v_name);
        PyTuple_SET_ITEM(__pyx_t_6, 0+__pyx_t_5, __pyx_v_name);
        __Pyx_INCREF(__pyx_v_convert);
        __Pyx_GIVEREF(__pyx_v_convert);
        PyTuple_SET_ITEM(__pyx_t_6, 1+__pyx_t_5, __pyx_v_convert);
        __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_t_6, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 490, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      }
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __pyx_r = __pyx_t_2;
      __pyx_t_2 = 0;
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      goto __pyx_L0;
    }
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  }

  /* "pymarketcap/core.pyx":503
 *             Markets on wich provided currency is currently tradeable.
 *         """
 *         response = {}             # <<<<<<<<<<<<<<
 *         if self._is_symbol(name):
 *             response["symbol"] = name
 */
  __pyx_t_1 = __Pyx_PyDict_NewPresized(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 503, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_response = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "pymarketcap/core.pyx":504
 *         """
 *         response = {}
 *         if self._is_symbol(name):             # <<<<<<<<<<<<<<
 *             response["symbol"] = name
 *             name = self.correspondences[name]
 */
  __pyx_t_1 = ((struct __pyx_vtabstruct_11pymarketcap_4core_Pymarketcap *)__pyx_v_self->__pyx_vtab)->_is_symbol(__pyx_v_self, __pyx_v_name, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 504, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_7 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_7 < 0)) __PYX_ERR(0, 504, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (__pyx_t_7) {

    /* "pymarketcap/core.pyx":505
 *         response = {}
 *         if self._is_symbol(name):
 *             response["symbol"] = name             # <<<<<<<<<<<<<<
 *             name = self.correspondences[name]
 *             response["slug"] = name
 */
    if (unlikely(PyDict_SetItem(__pyx_v_response, __pyx_n_s_symbol, __pyx_v_name) < 0)) __PYX_ERR(0, 505, __pyx_L1_error)

    /* "pymarketcap/core.pyx":506
 *         if self._is_symbol(name):
 *             response["symbol"] = name
 *             name = self.correspondences[name]             # <<<<<<<<<<<<<<
 *             response["slug"] = name
 *         else:
 */
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_correspondences); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 506, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_2 = __Pyx_PyObject_Dict_GetItem(__pyx_t_1, __pyx_v_name); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 506, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    if (!(likely(PyUnicode_CheckExact(__pyx_t_2))||((__pyx_t_2) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "unicode", Py_TYPE(__pyx_t_2)->tp_name), 0))) __PYX_ERR(0, 506, __pyx_L1_error)
    __Pyx_DECREF_SET(__pyx_v_name, ((PyObject*)__pyx_t_2));
    __pyx_t_2 = 0;

    /* "pymarketcap/core.pyx":507
 *             response["symbol"] = name
 *             name = self.correspondences[name]
 *             response["slug"] = name             # <<<<<<<<<<<<<<
 *         else:
 *             response["slug"] = name
 */
    if (unlikely(PyDict_SetItem(__pyx_v_response, __pyx_n_s_slug, __pyx_v_name) < 0)) __PYX_ERR(0, 507, __pyx_L1_error)

    /* "pymarketcap/core.pyx":504
 *         """
 *         response = {}
 *         if self._is_symbol(name):             # <<<<<<<<<<<<<<
 *             response["symbol"] = name
 *             name = self.correspondences[name]
 */
    goto __pyx_L3;
  }

  /* "pymarketcap/core.pyx":509
 *             response["slug"] = name
 *         else:
 *             response["slug"] = name             # <<<<<<<<<<<<<<
 *             for symbol, slug in self.correspondences.items():
 *                 if slug == name:
 */
  /*else*/ {
    if (unlikely(PyDict_SetItem(__pyx_v_response, __pyx_n_s_slug, __pyx_v_name) < 0)) __PYX_ERR(0, 509, __pyx_L1_error)

    /* "pymarketcap/core.pyx":510
 *         else:
 *             response["slug"] = name
 *             for symbol, slug in self.correspondences.items():             # <<<<<<<<<<<<<<
 *                 if slug == name:
 *                     response["symbol"] = symbol
 */
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_correspondences); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 510, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_items); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 510, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_1 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_3))) {
      __pyx_t_1 = PyMethod_GET_SELF(__pyx_t_3);
      if (likely(__pyx_t_1)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
        __Pyx_INCREF(__pyx_t_1);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_3, function);
      }
    }
    if (__pyx_t_1) {
      __pyx_t_2 = __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 510, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    } else {
      __pyx_t_2 = __Pyx_PyObject_CallNoArg(__pyx_t_3); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 510, __pyx_L1_error)
    }
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (likely(PyList_CheckExact(__pyx_t_2)) || PyTuple_CheckExact(__pyx_t_2)) {
      __pyx_t_3 = __pyx_t_2; __Pyx_INCREF(__pyx_t_3); __pyx_t_8 = 0;
      __pyx_t_9 = NULL;
    } else {
      __pyx_t_8 = -1; __pyx_t_3 = PyObject_GetIter(__pyx_t_2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 510, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __pyx_t_9 = Py_TYPE(__pyx_t_3)->tp_iternext; if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 510, __pyx_L1_error)
    }
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    for (;;) {
      if (likely(!__pyx_t_9)) {
        if (likely(PyList_CheckExact(__pyx_t_3))) {
          if (__pyx_t_8 >= PyList_GET_SIZE(__pyx_t_3)) break;
          #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
          __pyx_t_2 = PyList_GET_ITEM(__pyx_t_3, __pyx_t_8); __Pyx_INCREF(__pyx_t_2); __pyx_t_8++; if (unlikely(0 < 0)) __PYX_ERR(0, 510, __pyx_L1_error)
          #else
          __pyx_t_2 = PySequence_ITEM(__pyx_t_3, __pyx_t_8); __pyx_t_8++; if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 510, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_2);
          #endif
        } else {
          if (__pyx_t_8 >= PyTuple_GET_SIZE(__pyx_t_3)) break;
          #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
          __pyx_t_2 = PyTuple_GET_ITEM(__pyx_t_3, __pyx_t_8); __Pyx_INCREF(__pyx_t_2); __pyx_t_8++; if (unlikely(0 < 0)) __PYX_ERR(0, 510, __pyx_L1_error)
          #else
          __pyx_t_2 = PySequence_ITEM(__pyx_t_3, __pyx_t_8); __pyx_t_8++; if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 510, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_2);
          #endif
        }
      } else {
        __pyx_t_2 = __pyx_t_9(__pyx_t_3);
        if (unlikely(!__pyx_t_2)) {
          PyObject* exc_type = PyErr_Occurred();
          if (exc_type) {
            if (likely(__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
            else __PYX_ERR(0, 510, __pyx_L1_error)
          }
          break;
        }
        __Pyx_GOTREF(__pyx_t_2);
      }
      if ((likely(PyTuple_CheckExact(__pyx_t_2))) || (PyList_CheckExact(__pyx_t_2))) {
        PyObject* sequence = __pyx_t_2;
        Py_ssize_t size = __Pyx_PySequence_SIZE(sequence);
        if (unlikely(size != 2)) {
          if (size > 2) __Pyx_RaiseTooManyValuesError(2);
          else if (size >= 0) __Pyx_RaiseNeedMoreValuesError(size);
          __PYX_ERR(0, 510, __pyx_L1_error)
        }
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        if (likely(PyTuple_CheckExact(sequence))) {
          __pyx_t_1 = PyTuple_GET_ITEM(sequence, 0); 
          __pyx_t_6 = PyTuple_GET_ITEM(sequence, 1); 
        } else {
          __pyx_t_1 = PyList_GET_ITEM(sequence, 0); 
          __pyx_t_6 = PyList_GET_ITEM(sequence, 1); 
        }
        __Pyx_INCREF(__pyx_t_1);
        __Pyx_INCREF(__pyx_t_6);
        #else
        __pyx_t_1 = PySequence_ITEM(sequence, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 510, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_1);
        __pyx_t_6 = PySequence_ITEM(sequence, 1); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 510, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_6);
        #endif
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      } else {
        Py_ssize_t index = -1;
        __pyx_t_4 = PyObject_GetIter(__pyx_t_2); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 510, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_4);
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
        __pyx_t_10 = Py_TYPE(__pyx_t_4)->tp_iternext;
        index = 0; __pyx_t_1 = __pyx_t_10(__pyx_t_4); if (unlikely(!__pyx_t_1)) goto __pyx_L6_unpacking_failed;
        __Pyx_GOTREF(__pyx_t_1);
        index = 1; __pyx_t_6 = __pyx_t_10(__pyx_t_4); if (unlikely(!__pyx_t_6)) goto __pyx_L6_unpacking_failed;
        __Pyx_GOTREF(__pyx_t_6);
        if (__Pyx_IternextUnpackEndCheck(__pyx_t_10(__pyx_t_4), 2) < 0) __PYX_ERR(0, 510, __pyx_L1_error)
        __pyx_t_10 = NULL;
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
        goto __pyx_L7_unpacking_done;
        __pyx_L6_unpacking_failed:;
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
        __pyx_t_10 = NULL;
        if (__Pyx_IterFinish() == 0) __Pyx_RaiseNeedMoreValuesError(index);
        __PYX_ERR(0, 510, __pyx_L1_error)
        __pyx_L7_unpacking_done:;
      }
      __Pyx_XDECREF_SET(__pyx_v_symbol, __pyx_t_1);
      __pyx_t_1 = 0;
      __Pyx_XDECREF_SET(__pyx_v_slug, __pyx_t_6);
      __pyx_t_6 = 0;

      /* "pymarketcap/core.pyx":511
 *             response["slug"] = name
 *             for symbol, slug in self.correspondences.items():
 *                 if slug == name:             # <<<<<<<<<<<<<<
 *                     response["symbol"] = symbol
 *                     break
 */
      __pyx_t_7 = (__Pyx_PyUnicode_Equals(__pyx_v_slug, __pyx_v_name, Py_EQ)); if (unlikely(__pyx_t_7 < 0)) __PYX_ERR(0, 511, __pyx_L1_error)
      if (__pyx_t_7) {

        /* "pymarketcap/core.pyx":512
 *             for symbol, slug in self.correspondences.items():
 *                 if slug == name:
 *                     response["symbol"] = symbol             # <<<<<<<<<<<<<<
 *                     break
 *         convert = convert.lower()
 */
        if (unlikely(PyDict_SetItem(__pyx_v_response, __pyx_n_s_symbol, __pyx_v_symbol) < 0)) __PYX_ERR(0, 512, __pyx_L1_error)

        /* "pymarketcap/core.pyx":513
 *                 if slug == name:
 *                     response["symbol"] = symbol
 *                     break             # <<<<<<<<<<<<<<
 *         convert = convert.lower()
 * 
 */
        goto __pyx_L5_break;

        /* "pymarketcap/core.pyx":511
 *             response["slug"] = name
 *             for symbol, slug in self.correspondences.items():
 *                 if slug == name:             # <<<<<<<<<<<<<<
 *                     response["symbol"] = symbol
 *                     break
 */
      }

      /* "pymarketcap/core.pyx":510
 *         else:
 *             response["slug"] = name
 *             for symbol, slug in self.correspondences.items():             # <<<<<<<<<<<<<<
 *                 if slug == name:
 *                     response["symbol"] = symbol
 */
    }
    __pyx_L5_break:;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  }
  __pyx_L3:;

  /* "pymarketcap/core.pyx":514
 *                     response["symbol"] = symbol
 *                     break
 *         convert = convert.lower()             # <<<<<<<<<<<<<<
 * 
 *         try:
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_convert, __pyx_n_s_lower); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 514, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_6 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_6)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_6);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  if (__pyx_t_6) {
    __pyx_t_3 = __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_6); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 514, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  } else {
    __pyx_t_3 = __Pyx_PyObject_CallNoArg(__pyx_t_2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 514, __pyx_L1_error)
  }
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF_SET(__pyx_v_convert, __pyx_t_3);
  __pyx_t_3 = 0;

  /* "pymarketcap/core.pyx":516
 *         convert = convert.lower()
 * 
 *         try:             # <<<<<<<<<<<<<<
 *             res = self._get(
 *                 b"https://coinmarketcap.com/currencies/%s/" % name.encode()
 */
  {
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    __Pyx_ExceptionSave(&__pyx_t_11, &__pyx_t_12, &__pyx_t_13);
    __Pyx_XGOTREF(__pyx_t_11);
    __Pyx_XGOTREF(__pyx_t_12);
    __Pyx_XGOTREF(__pyx_t_13);
    /*try:*/ {

      /* "pymarketcap/core.pyx":518
 *         try:
 *             res = self._get(
 *                 b"https://coinmarketcap.com/currencies/%s/" % name.encode()             # <<<<<<<<<<<<<<
 *             )[20000:]
 *         except CoinmarketcapHTTPError404:
 */
      if (unlikely(__pyx_v_name == Py_None)) {
        PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "encode");
        __PYX_ERR(0, 518, __pyx_L9_error)
      }
      __pyx_t_3 = PyUnicode_AsEncodedString(__pyx_v_name, NULL, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 518, __pyx_L9_error)
      __Pyx_GOTREF(__pyx_t_3);
      __pyx_t_2 = PyNumber_Remainder(__pyx_kp_b_https_coinmarketcap_com_currenci, __pyx_t_3); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 518, __pyx_L9_error)
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __pyx_t_14 = __Pyx_PyObject_AsWritableString(__pyx_t_2); if (unlikely((!__pyx_t_14) && PyErr_Occurred())) __PYX_ERR(0, 518, __pyx_L9_error)

      /* "pymarketcap/core.pyx":517
 * 
 *         try:
 *             res = self._get(             # <<<<<<<<<<<<<<
 *                 b"https://coinmarketcap.com/currencies/%s/" % name.encode()
 *             )[20000:]
 */
      __pyx_t_3 = ((struct __pyx_vtabstruct_11pymarketcap_4core_Pymarketcap *)__pyx_v_self->__pyx_vtab)->_get(__pyx_v_self, __pyx_t_14); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 517, __pyx_L9_error)
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

      /* "pymarketcap/core.pyx":519
 *             res = self._get(
 *                 b"https://coinmarketcap.com/currencies/%s/" % name.encode()
 *             )[20000:]             # <<<<<<<<<<<<<<
 *         except CoinmarketcapHTTPError404:
 *             if name not in self.coins:
 */
      __pyx_t_2 = __Pyx_PyObject_GetSlice(__pyx_t_3, 0x4E20, 0, NULL, NULL, &__pyx_slice__8, 1, 0, 1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 519, __pyx_L9_error)
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __pyx_v_res = __pyx_t_2;
      __pyx_t_2 = 0;

      /* "pymarketcap/core.pyx":516
 *         convert = convert.lower()
 * 
 *         try:             # <<<<<<<<<<<<<<
 *             res = self._get(
 *                 b"https://coinmarketcap.com/currencies/%s/" % name.encode()
 */
    }
    __Pyx_XDECREF(__pyx_t_11); __pyx_t_11 = 0;
    __Pyx_XDECREF(__pyx_t_12); __pyx_t_12 = 0;
    __Pyx_XDECREF(__pyx_t_13); __pyx_t_13 = 0;
    goto __pyx_L14_try_end;
    __pyx_L9_error:;
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;

    /* "pymarketcap/core.pyx":520
 *                 b"https://coinmarketcap.com/currencies/%s/" % name.encode()
 *             )[20000:]
 *         except CoinmarketcapHTTPError404:             # <<<<<<<<<<<<<<
 *             if name not in self.coins:
 *                 raise ValueError("%s is not a valid currrency name. See 'symbols'" % name \
 */
    __Pyx_ErrFetch(&__pyx_t_2, &__pyx_t_3, &__pyx_t_6);
    __pyx_t_1 = __Pyx_GetModuleGlobalName(__pyx_n_s_CoinmarketcapHTTPError404); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 520, __pyx_L11_except_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_5 = __Pyx_PyErr_GivenExceptionMatches(__pyx_t_2, __pyx_t_1);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_ErrRestore(__pyx_t_2, __pyx_t_3, __pyx_t_6);
    __pyx_t_2 = 0; __pyx_t_3 = 0; __pyx_t_6 = 0;
    if (__pyx_t_5) {
      __Pyx_AddTraceback("pymarketcap.core.Pymarketcap.markets", __pyx_clineno, __pyx_lineno, __pyx_filename);
      if (__Pyx_GetException(&__pyx_t_6, &__pyx_t_3, &__pyx_t_2) < 0) __PYX_ERR(0, 520, __pyx_L11_except_error)
      __Pyx_GOTREF(__pyx_t_6);
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_GOTREF(__pyx_t_2);

      /* "pymarketcap/core.pyx":521
 *             )[20000:]
 *         except CoinmarketcapHTTPError404:
 *             if name not in self.coins:             # <<<<<<<<<<<<<<
 *                 raise ValueError("%s is not a valid currrency name. See 'symbols'" % name \
 *                                  + " or 'coins' properties for get all valid currencies.")
 */
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_coins); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 521, __pyx_L11_except_error)
      __Pyx_GOTREF(__pyx_t_1);
      __pyx_t_7 = (__Pyx_PySequence_ContainsTF(__pyx_v_name, __pyx_t_1, Py_NE)); if (unlikely(__pyx_t_7 < 0)) __PYX_ERR(0, 521, __pyx_L11_except_error)
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __pyx_t_15 = (__pyx_t_7 != 0);
      if (unlikely(__pyx_t_15)) {

        /* "pymarketcap/core.pyx":522
 *         except CoinmarketcapHTTPError404:
 *             if name not in self.coins:
 *                 raise ValueError("%s is not a valid currrency name. See 'symbols'" % name \             # <<<<<<<<<<<<<<
 *                                  + " or 'coins' properties for get all valid currencies.")
 *             else:
 */
        __pyx_t_1 = __Pyx_PyString_Format(__pyx_kp_s_s_is_not_a_valid_currrency_name, __pyx_v_name); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 522, __pyx_L11_except_error)
        __Pyx_GOTREF(__pyx_t_1);

        /* "pymarketcap/core.pyx":523
 *             if name not in self.coins:
 *                 raise ValueError("%s is not a valid currrency name. See 'symbols'" % name \
 *                                  + " or 'coins' properties for get all valid currencies.")             # <<<<<<<<<<<<<<
 *             else:
 *                 raise NotImplementedError
 */
        __pyx_t_4 = __Pyx_PyUnicode_Concat(__pyx_t_1, __pyx_kp_s_or_coins_properties_for_get_all); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 523, __pyx_L11_except_error)
        __Pyx_GOTREF(__pyx_t_4);
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

        /* "pymarketcap/core.pyx":522
 *         except CoinmarketcapHTTPError404:
 *             if name not in self.coins:
 *                 raise ValueError("%s is not a valid currrency name. See 'symbols'" % name \             # <<<<<<<<<<<<<<
 *                                  + " or 'coins' properties for get all valid currencies.")
 *             else:
 */
        __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_builtin_ValueError, __pyx_t_4); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 522, __pyx_L11_except_error)
        __Pyx_GOTREF(__pyx_t_1);
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
        __Pyx_Raise(__pyx_t_1, 0, 0, 0);
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        __PYX_ERR(0, 522, __pyx_L11_except_error)

        /* "pymarketcap/core.pyx":521
 *             )[20000:]
 *         except CoinmarketcapHTTPError404:
 *             if name not in self.coins:             # <<<<<<<<<<<<<<
 *                 raise ValueError("%s is not a valid currrency name. See 'symbols'" % name \
 *                                  + " or 'coins' properties for get all valid currencies.")
 */
      }

      /* "pymarketcap/core.pyx":525
 *                                  + " or 'coins' properties for get all valid currencies.")
 *             else:
 *                 raise NotImplementedError             # <<<<<<<<<<<<<<
 * 
 *         response["markets"] = processer.markets(res, convert)
 */
      /*else*/ {
        __Pyx_Raise(__pyx_builtin_NotImplementedError, 0, 0, 0);
        __PYX_ERR(0, 525, __pyx_L11_except_error)
      }
    }
    goto __pyx_L11_except_error;
    __pyx_L11_except_error:;

    /* "pymarketcap/core.pyx":516
 *         convert = convert.lower()
 * 
 *         try:             # <<<<<<<<<<<<<<
 *             res = self._get(
 *                 b"https://coinmarketcap.com/currencies/%s/" % name.encode()
 */
    __Pyx_XGIVEREF(__pyx_t_11);
    __Pyx_XGIVEREF(__pyx_t_12);
    __Pyx_XGIVEREF(__pyx_t_13);
    __Pyx_ExceptionReset(__pyx_t_11, __pyx_t_12, __pyx_t_13);
    goto __pyx_L1_error;
    __pyx_L14_try_end:;
  }

  /* "pymarketcap/core.pyx":527
 *                 raise NotImplementedError
 * 
 *         response["markets"] = processer.markets(res, convert)             # <<<<<<<<<<<<<<
 *         return response
 * 
 */
  __pyx_t_3 = __Pyx_GetModuleGlobalName(__pyx_n_s_processer); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 527, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_markets); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 527, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = NULL;
  __pyx_t_5 = 0;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_6))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_6);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_6);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_6, function);
      __pyx_t_5 = 1;
    }
  }
  #if CYTHON_FAST_PYCALL
  if (PyFunction_Check(__pyx_t_6)) {
    PyObject *__pyx_temp[3] = {__pyx_t_3, __pyx_v_res, __pyx_v_convert};
    __pyx_t_2 = __Pyx_PyFunction_FastCall(__pyx_t_6, __pyx_temp+1-__pyx_t_5, 2+__pyx_t_5); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 527, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_GOTREF(__pyx_t_2);
  } else
  #endif
  #if CYTHON_FAST_PYCCALL
  if (__Pyx_PyFastCFunction_Check(__pyx_t_6)) {
    PyObject *__pyx_temp[3] = {__pyx_t_3, __pyx_v_res, __pyx_v_convert};
    __pyx_t_2 = __Pyx_PyCFunction_FastCall(__pyx_t_6, __pyx_temp+1-__pyx_t_5, 2+__pyx_t_5); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 527, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_GOTREF(__pyx_t_2);
  } else
  #endif
  {
    __pyx_t_1 = PyTuple_New(2+__pyx_t_5); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 527, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    if (__pyx_t_3) {
      __Pyx_GIVEREF(__pyx_t_3); PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_t_3); __pyx_t_3 = NULL;
    }
    __Pyx_INCREF(__pyx_v_res);
    __Pyx_GIVEREF(__pyx_v_res);
    PyTuple_SET_ITEM(__pyx_t_1, 0+__pyx_t_5, __pyx_v_res);
    __Pyx_INCREF(__pyx_v_convert);
    __Pyx_GIVEREF(__pyx_v_convert);
    PyTuple_SET_ITEM(__pyx_t_1, 1+__pyx_t_5, __pyx_v_convert);
    __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_6, __pyx_t_1, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 527, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  }
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  if (unlikely(PyDict_SetItem(__pyx_v_response, __pyx_n_s_markets, __pyx_t_2) < 0)) __PYX_ERR(0, 527, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "pymarketcap/core.pyx":528
 * 
 *         response["markets"] = processer.markets(res, convert)
 *         return response             # <<<<<<<<<<<<<<
 * 
 *     cpdef ranks(self):
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_response);
  __pyx_r = __pyx_v_response;
  goto __pyx_L0;

  /* "pymarketcap/core.pyx":490
 *         return response
 * 
 *     cpdef markets(self, unicode name, convert="USD"):             # <<<<<<<<<<<<<<
 *         """Get available coinmarketcap markets data.
 *         It needs a currency as argument.
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_AddTraceback("pymarketcap.core.Pymarketcap.markets", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_response);
  __Pyx_XDECREF(__pyx_v_symbol);
  __Pyx_XDECREF(__pyx_v_slug);
  __Pyx_XDECREF(__pyx_v_res);
  __Pyx_XDECREF(__pyx_v_name);
  __Pyx_XDECREF(__pyx_v_convert);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_11pymarketcap_4core_11Pymarketcap_23markets(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_11pymarketcap_4core_11Pymarketcap_22markets[] = "Get available coinmarketcap markets data.\n        It needs a currency as argument.\n\n        Args:\n            currency (str): Currency to get market data.\n            convert (str, optional): Currency to convert response\n                fields ``volume_24h`` and ``price`` between USD\n                and BTC. As default ``\"USD\"``.\n\n        Returns (list):\n            Markets on wich provided currency is currently tradeable.\n        ";
static PyObject *__pyx_pw_11pymarketcap_4core_11Pymarketcap_23markets(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_name = 0;
  PyObject *__pyx_v_convert = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("markets (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_name,&__pyx_n_s_convert,0};
    PyObject* values[2] = {0,0};
    values[1] = ((PyObject *)__pyx_n_s_USD);
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_name)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_convert);
          if (value) { values[1] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "markets") < 0)) __PYX_ERR(0, 490, __pyx_L3_error)
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_name = ((PyObject*)values[0]);
    __pyx_v_convert = values[1];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("markets", 0, 1, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 490, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("pymarketcap.core.Pymarketcap.markets", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_name), (&PyUnicode_Type), 1, "name", 1))) __PYX_ERR(0, 490, __pyx_L1_error)
  __pyx_r = __pyx_pf_11pymarketcap_4core_11Pymarketcap_22markets(((struct __pyx_obj_11pymarketcap_4core_Pymarketcap *)__pyx_v_self), __pyx_v_name, __pyx_v_convert);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_11pymarketcap_4core_11Pymarketcap_22markets(struct __pyx_obj_11pymarketcap_4core_Pymarketcap *__pyx_v_self, PyObject *__pyx_v_name, PyObject *__pyx_v_convert) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  struct __pyx_opt_args_11pymarketcap_4core_11Pymarketcap_markets __pyx_t_2;
  __Pyx_RefNannySetupContext("markets", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2.__pyx_n = 1;
  __pyx_t_2.convert = __pyx_v_convert;
  __pyx_t_1 = __pyx_vtabptr_11pymarketcap_4core_Pymarketcap->markets(__pyx_v_self, __pyx_v_name, 1, &__pyx_t_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 490, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pymarketcap.core.Pymarketcap.markets", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pymarketcap/core.pyx":530
 *         return response
 * 
 *     cpdef ranks(self):             # <<<<<<<<<<<<<<
 *         """Returns gainers and losers for 1 hour, 24 hours and 7 days.
 * 
 */

static PyObject *__pyx_pw_11pymarketcap_4core_11Pymarketcap_25ranks(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_f_11pymarketcap_4core_11Pymarketcap_ranks(struct __pyx_obj_11pymarketcap_4core_Pymarketcap *__pyx_v_self, int __pyx_skip_dispatch) {
  PyObject *__pyx_v_res = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  __Pyx_RefNannySetupContext("ranks", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0)) {
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_ranks); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 530, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)__pyx_pw_11pymarketcap_4core_11Pymarketcap_25ranks)) {
      __Pyx_XDECREF(__pyx_r);
      __Pyx_INCREF(__pyx_t_1);
      __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
      if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
        __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
        if (likely(__pyx_t_4)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
          __Pyx_INCREF(__pyx_t_4);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_3, function);
        }
      }
      if (__pyx_t_4) {
        __pyx_t_2 = __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_4); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 530, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      } else {
        __pyx_t_2 = __Pyx_PyObject_CallNoArg(__pyx_t_3); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 530, __pyx_L1_error)
      }
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __pyx_r = __pyx_t_2;
      __pyx_t_2 = 0;
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      goto __pyx_L0;
    }
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  }

  /* "pymarketcap/core.pyx":537
 *             are the periods ``"1h"``, ``"24h"`` and ``"7d"``.
 *         """
 *         res = self._get(b"https://coinmarketcap.com/gainers-losers/")             # <<<<<<<<<<<<<<
 * 
 *         return processer.ranks(res)
 */
  __pyx_t_1 = ((struct __pyx_vtabstruct_11pymarketcap_4core_Pymarketcap *)__pyx_v_self->__pyx_vtab)->_get(__pyx_v_self, ((char *)"https://coinmarketcap.com/gainers-losers/")); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 537, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_res = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "pymarketcap/core.pyx":539
 *         res = self._get(b"https://coinmarketcap.com/gainers-losers/")
 * 
 *         return processer.ranks(res)             # <<<<<<<<<<<<<<
 * 
 *     cpdef historical(self, unicode name,
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_processer); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 539, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_ranks); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 539, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = NULL;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_3);
    if (likely(__pyx_t_2)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_2);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_3, function);
    }
  }
  if (!__pyx_t_2) {
    __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_v_res); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 539, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
  } else {
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(__pyx_t_3)) {
      PyObject *__pyx_temp[2] = {__pyx_t_2, __pyx_v_res};
      __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_3, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 539, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
      __Pyx_GOTREF(__pyx_t_1);
    } else
    #endif
    #if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(__pyx_t_3)) {
      PyObject *__pyx_temp[2] = {__pyx_t_2, __pyx_v_res};
      __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_3, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 539, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
      __Pyx_GOTREF(__pyx_t_1);
    } else
    #endif
    {
      __pyx_t_4 = PyTuple_New(1+1); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 539, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_GIVEREF(__pyx_t_2); PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_2); __pyx_t_2 = NULL;
      __Pyx_INCREF(__pyx_v_res);
      __Pyx_GIVEREF(__pyx_v_res);
      PyTuple_SET_ITEM(__pyx_t_4, 0+1, __pyx_v_res);
      __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_t_4, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 539, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    }
  }
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "pymarketcap/core.pyx":530
 *         return response
 * 
 *     cpdef ranks(self):             # <<<<<<<<<<<<<<
 *         """Returns gainers and losers for 1 hour, 24 hours and 7 days.
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("pymarketcap.core.Pymarketcap.ranks", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_res);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_11pymarketcap_4core_11Pymarketcap_25ranks(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_11pymarketcap_4core_11Pymarketcap_24ranks[] = "Returns gainers and losers for 1 hour, 24 hours and 7 days.\n\n        Returns (dict):\n            A dictionary with 2 keys (gainers and losers) whose values\n            are the periods ``\"1h\"``, ``\"24h\"`` and ``\"7d\"``.\n        ";
static PyObject *__pyx_pw_11pymarketcap_4core_11Pymarketcap_25ranks(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("ranks (wrapper)", 0);
  __pyx_r = __pyx_pf_11pymarketcap_4core_11Pymarketcap_24ranks(((struct __pyx_obj_11pymarketcap_4core_Pymarketcap *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_11pymarketcap_4core_11Pymarketcap_24ranks(struct __pyx_obj_11pymarketcap_4core_Pymarketcap *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("ranks", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_11pymarketcap_4core_11Pymarketcap_ranks(__pyx_v_self, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 530, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pymarketcap.core.Pymarketcap.ranks", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pymarketcap/core.pyx":541
 *         return processer.ranks(res)
 * 
 *     cpdef historical(self, unicode name,             # <<<<<<<<<<<<<<
 *                      start=datetime(2008, 8, 18),
 *                      end=datetime.now(),
 */

static PyObject *__pyx_pw_11pymarketcap_4core_11Pymarketcap_27historical(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyObject *__pyx_f_11pymarketcap_4core_11Pymarketcap_historical(struct __pyx_obj_11pymarketcap_4core_Pymarketcap *__pyx_v_self, PyObject *__pyx_v_name, int __pyx_skip_dispatch, struct __pyx_opt_args_11pymarketcap_4core_11Pymarketcap_historical *__pyx_optional_args) {
  PyObject *__pyx_v_start = __pyx_k__9;
  PyObject *__pyx_v_end = __pyx_k__10;

  /* "pymarketcap/core.pyx":544
 *                      start=datetime(2008, 8, 18),
 *                      end=datetime.now(),
 *                      revert=False):             # <<<<<<<<<<<<<<
 *         """Get historical data for a currency.
 * 
 */
  PyObject *__pyx_v_revert = ((PyObject *)Py_False);
  PyObject *__pyx_v_response = NULL;
  PyObject *__pyx_v_symbol = NULL;
  PyObject *__pyx_v_slug = NULL;
  PyObject *__pyx_v_url = NULL;
  PyObject *__pyx_v__start = NULL;
  PyObject *__pyx_v__end = NULL;
  PyObject *__pyx_v_res = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_t_7;
  Py_ssize_t __pyx_t_8;
  PyObject *(*__pyx_t_9)(PyObject *);
  PyObject *(*__pyx_t_10)(PyObject *);
  PyObject *__pyx_t_11 = NULL;
  PyObject *__pyx_t_12 = NULL;
  PyObject *__pyx_t_13 = NULL;
  char *__pyx_t_14;
  int __pyx_t_15;
  __Pyx_RefNannySetupContext("historical", 0);
  if (__pyx_optional_args) {
    if (__pyx_optional_args->__pyx_n > 0) {
      __pyx_v_start = __pyx_optional_args->start;
      if (__pyx_optional_args->__pyx_n > 1) {
        __pyx_v_end = __pyx_optional_args->end;
        if (__pyx_optional_args->__pyx_n > 2) {
          __pyx_v_revert = __pyx_optional_args->revert;
        }
      }
    }
  }
  __Pyx_INCREF(__pyx_v_name);

  /* "pymarketcap/core.pyx":541
 *         return processer.ranks(res)
 * 
 *     cpdef historical(self, unicode name,             # <<<<<<<<<<<<<<
 *                      start=datetime(2008, 8, 18),
 *                      end=datetime.now(),
 */
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0)) {
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_historical); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 541, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)__pyx_pw_11pymarketcap_4core_11Pymarketcap_27historical)) {
      __Pyx_XDECREF(__pyx_r);
      __Pyx_INCREF(__pyx_t_1);
      __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
      __pyx_t_5 = 0;
      if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
        __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
        if (likely(__pyx_t_4)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
          __Pyx_INCREF(__pyx_t_4);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_3, function);
          __pyx_t_5 = 1;
        }
      }
      #if CYTHON_FAST_PYCALL
      if (PyFunction_Check(__pyx_t_3)) {
        PyObject *__pyx_temp[5] = {__pyx_t_4, __pyx_v_name, __pyx_v_start, __pyx_v_end, __pyx_v_revert};
        __pyx_t_2 = __Pyx_PyFunction_FastCall(__pyx_t_3, __pyx_temp+1-__pyx_t_5, 4+__pyx_t_5); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 541, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        __Pyx_GOTREF(__pyx_t_2);
      } else
      #endif
      #if CYTHON_FAST_PYCCALL
      if (__Pyx_PyFastCFunction_Check(__pyx_t_3)) {
        PyObject *__pyx_temp[5] = {__pyx_t_4, __pyx_v_name, __pyx_v_start, __pyx_v_end, __pyx_v_revert};
        __pyx_t_2 = __Pyx_PyCFunction_FastCall(__pyx_t_3, __pyx_temp+1-__pyx_t_5, 4+__pyx_t_5); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 541, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        __Pyx_GOTREF(__pyx_t_2);
      } else
      #endif
      {
        __pyx_t_6 = PyTuple_New(4+__pyx_t_5); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 541, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_6);
        if (__pyx_t_4) {
          __Pyx_GIVEREF(__pyx_t_4); PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_4); __pyx_t_4 = NULL;
        }
        __Pyx_INCREF(__pyx_v_name);
        __Pyx_GIVEREF(__pyx_v_name);
        PyTuple_SET_ITEM(__pyx_t_6, 0+__pyx_t_5, __pyx_v_name);
        __Pyx_INCREF(__pyx_v_start);
        __Pyx_GIVEREF(__pyx_v_start);
        PyTuple_SET_ITEM(__pyx_t_6, 1+__pyx_t_5, __pyx_v_start);
        __Pyx_INCREF(__pyx_v_end);
        __Pyx_GIVEREF(__pyx_v_end);
        PyTuple_SET_ITEM(__pyx_t_6, 2+__pyx_t_5, __pyx_v_end);
        __Pyx_INCREF(__pyx_v_revert);
        __Pyx_GIVEREF(__pyx_v_revert);
        PyTuple_SET_ITEM(__pyx_t_6, 3+__pyx_t_5, __pyx_v_revert);
        __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_t_6, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 541, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      }
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __pyx_r = __pyx_t_2;
      __pyx_t_2 = 0;
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      goto __pyx_L0;
    }
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  }

  /* "pymarketcap/core.pyx":561
 *             Historical dayly OHLC for a currency.
 *         """
 *         response = {}             # <<<<<<<<<<<<<<
 * 
 *         if self._is_symbol(name):
 */
  __pyx_t_1 = __Pyx_PyDict_NewPresized(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 561, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_response = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "pymarketcap/core.pyx":563
 *         response = {}
 * 
 *         if self._is_symbol(name):             # <<<<<<<<<<<<<<
 *             response["symbol"] = name
 *             response["slug"] = self.correspondences[name]
 */
  __pyx_t_1 = ((struct __pyx_vtabstruct_11pymarketcap_4core_Pymarketcap *)__pyx_v_self->__pyx_vtab)->_is_symbol(__pyx_v_self, __pyx_v_name, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 563, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_7 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_7 < 0)) __PYX_ERR(0, 563, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (__pyx_t_7) {

    /* "pymarketcap/core.pyx":564
 * 
 *         if self._is_symbol(name):
 *             response["symbol"] = name             # <<<<<<<<<<<<<<
 *             response["slug"] = self.correspondences[name]
 *             name = response["slug"]
 */
    if (unlikely(PyDict_SetItem(__pyx_v_response, __pyx_n_s_symbol, __pyx_v_name) < 0)) __PYX_ERR(0, 564, __pyx_L1_error)

    /* "pymarketcap/core.pyx":565
 *         if self._is_symbol(name):
 *             response["symbol"] = name
 *             response["slug"] = self.correspondences[name]             # <<<<<<<<<<<<<<
 *             name = response["slug"]
 *         else:
 */
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_correspondences); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 565, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_2 = __Pyx_PyObject_Dict_GetItem(__pyx_t_1, __pyx_v_name); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 565, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    if (unlikely(PyDict_SetItem(__pyx_v_response, __pyx_n_s_slug, __pyx_t_2) < 0)) __PYX_ERR(0, 565, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

    /* "pymarketcap/core.pyx":566
 *             response["symbol"] = name
 *             response["slug"] = self.correspondences[name]
 *             name = response["slug"]             # <<<<<<<<<<<<<<
 *         else:
 *             response["slug"] = name
 */
    __pyx_t_2 = __Pyx_PyDict_GetItem(__pyx_v_response, __pyx_n_s_slug); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 566, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    if (!(likely(PyUnicode_CheckExact(__pyx_t_2))||((__pyx_t_2) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "unicode", Py_TYPE(__pyx_t_2)->tp_name), 0))) __PYX_ERR(0, 566, __pyx_L1_error)
    __Pyx_DECREF_SET(__pyx_v_name, ((PyObject*)__pyx_t_2));
    __pyx_t_2 = 0;

    /* "pymarketcap/core.pyx":563
 *         response = {}
 * 
 *         if self._is_symbol(name):             # <<<<<<<<<<<<<<
 *             response["symbol"] = name
 *             response["slug"] = self.correspondences[name]
 */
    goto __pyx_L3;
  }

  /* "pymarketcap/core.pyx":568
 *             name = response["slug"]
 *         else:
 *             response["slug"] = name             # <<<<<<<<<<<<<<
 *             for symbol, slug in self.correspondences.items():
 *                 if slug == name:
 */
  /*else*/ {
    if (unlikely(PyDict_SetItem(__pyx_v_response, __pyx_n_s_slug, __pyx_v_name) < 0)) __PYX_ERR(0, 568, __pyx_L1_error)

    /* "pymarketcap/core.pyx":569
 *         else:
 *             response["slug"] = name
 *             for symbol, slug in self.correspondences.items():             # <<<<<<<<<<<<<<
 *                 if slug == name:
 *                     response["symbol"] = symbol
 */
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_correspondences); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 569, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_items); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 569, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_1 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_3))) {
      __pyx_t_1 = PyMethod_GET_SELF(__pyx_t_3);
      if (likely(__pyx_t_1)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
        __Pyx_INCREF(__pyx_t_1);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_3, function);
      }
    }
    if (__pyx_t_1) {
      __pyx_t_2 = __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 569, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    } else {
      __pyx_t_2 = __Pyx_PyObject_CallNoArg(__pyx_t_3); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 569, __pyx_L1_error)
    }
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (likely(PyList_CheckExact(__pyx_t_2)) || PyTuple_CheckExact(__pyx_t_2)) {
      __pyx_t_3 = __pyx_t_2; __Pyx_INCREF(__pyx_t_3); __pyx_t_8 = 0;
      __pyx_t_9 = NULL;
    } else {
      __pyx_t_8 = -1; __pyx_t_3 = PyObject_GetIter(__pyx_t_2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 569, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __pyx_t_9 = Py_TYPE(__pyx_t_3)->tp_iternext; if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 569, __pyx_L1_error)
    }
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    for (;;) {
      if (likely(!__pyx_t_9)) {
        if (likely(PyList_CheckExact(__pyx_t_3))) {
          if (__pyx_t_8 >= PyList_GET_SIZE(__pyx_t_3)) break;
          #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
          __pyx_t_2 = PyList_GET_ITEM(__pyx_t_3, __pyx_t_8); __Pyx_INCREF(__pyx_t_2); __pyx_t_8++; if (unlikely(0 < 0)) __PYX_ERR(0, 569, __pyx_L1_error)
          #else
          __pyx_t_2 = PySequence_ITEM(__pyx_t_3, __pyx_t_8); __pyx_t_8++; if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 569, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_2);
          #endif
        } else {
          if (__pyx_t_8 >= PyTuple_GET_SIZE(__pyx_t_3)) break;
          #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
          __pyx_t_2 = PyTuple_GET_ITEM(__pyx_t_3, __pyx_t_8); __Pyx_INCREF(__pyx_t_2); __pyx_t_8++; if (unlikely(0 < 0)) __PYX_ERR(0, 569, __pyx_L1_error)
          #else
          __pyx_t_2 = PySequence_ITEM(__pyx_t_3, __pyx_t_8); __pyx_t_8++; if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 569, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_2);
          #endif
        }
      } else {
        __pyx_t_2 = __pyx_t_9(__pyx_t_3);
        if (unlikely(!__pyx_t_2)) {
          PyObject* exc_type = PyErr_Occurred();
          if (exc_type) {
            if (likely(__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
            else __PYX_ERR(0, 569, __pyx_L1_error)
          }
          break;
        }
        __Pyx_GOTREF(__pyx_t_2);
      }
      if ((likely(PyTuple_CheckExact(__pyx_t_2))) || (PyList_CheckExact(__pyx_t_2))) {
        PyObject* sequence = __pyx_t_2;
        Py_ssize_t size = __Pyx_PySequence_SIZE(sequence);
        if (unlikely(size != 2)) {
          if (size > 2) __Pyx_RaiseTooManyValuesError(2);
          else if (size >= 0) __Pyx_RaiseNeedMoreValuesError(size);
          __PYX_ERR(0, 569, __pyx_L1_error)
        }
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        if (likely(PyTuple_CheckExact(sequence))) {
          __pyx_t_1 = PyTuple_GET_ITEM(sequence, 0); 
          __pyx_t_6 = PyTuple_GET_ITEM(sequence, 1); 
        } else {
          __pyx_t_1 = PyList_GET_ITEM(sequence, 0); 
          __pyx_t_6 = PyList_GET_ITEM(sequence, 1); 
        }
        __Pyx_INCREF(__pyx_t_1);
        __Pyx_INCREF(__pyx_t_6);
        #else
        __pyx_t_1 = PySequence_ITEM(sequence, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 569, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_1);
        __pyx_t_6 = PySequence_ITEM(sequence, 1); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 569, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_6);
        #endif
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      } else {
        Py_ssize_t index = -1;
        __pyx_t_4 = PyObject_GetIter(__pyx_t_2); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 569, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_4);
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
        __pyx_t_10 = Py_TYPE(__pyx_t_4)->tp_iternext;
        index = 0; __pyx_t_1 = __pyx_t_10(__pyx_t_4); if (unlikely(!__pyx_t_1)) goto __pyx_L6_unpacking_failed;
        __Pyx_GOTREF(__pyx_t_1);
        index = 1; __pyx_t_6 = __pyx_t_10(__pyx_t_4); if (unlikely(!__pyx_t_6)) goto __pyx_L6_unpacking_failed;
        __Pyx_GOTREF(__pyx_t_6);
        if (__Pyx_IternextUnpackEndCheck(__pyx_t_10(__pyx_t_4), 2) < 0) __PYX_ERR(0, 569, __pyx_L1_error)
        __pyx_t_10 = NULL;
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
        goto __pyx_L7_unpacking_done;
        __pyx_L6_unpacking_failed:;
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
        __pyx_t_10 = NULL;
        if (__Pyx_IterFinish() == 0) __Pyx_RaiseNeedMoreValuesError(index);
        __PYX_ERR(0, 569, __pyx_L1_error)
        __pyx_L7_unpacking_done:;
      }
      __Pyx_XDECREF_SET(__pyx_v_symbol, __pyx_t_1);
      __pyx_t_1 = 0;
      __Pyx_XDECREF_SET(__pyx_v_slug, __pyx_t_6);
      __pyx_t_6 = 0;

      /* "pymarketcap/core.pyx":570
 *             response["slug"] = name
 *             for symbol, slug in self.correspondences.items():
 *                 if slug == name:             # <<<<<<<<<<<<<<
 *                     response["symbol"] = symbol
 *                     break
 */
      __pyx_t_7 = (__Pyx_PyUnicode_Equals(__pyx_v_slug, __pyx_v_name, Py_EQ)); if (unlikely(__pyx_t_7 < 0)) __PYX_ERR(0, 570, __pyx_L1_error)
      if (__pyx_t_7) {

        /* "pymarketcap/core.pyx":571
 *             for symbol, slug in self.correspondences.items():
 *                 if slug == name:
 *                     response["symbol"] = symbol             # <<<<<<<<<<<<<<
 *                     break
 * 
 */
        if (unlikely(PyDict_SetItem(__pyx_v_response, __pyx_n_s_symbol, __pyx_v_symbol) < 0)) __PYX_ERR(0, 571, __pyx_L1_error)

        /* "pymarketcap/core.pyx":572
 *                 if slug == name:
 *                     response["symbol"] = symbol
 *                     break             # <<<<<<<<<<<<<<
 * 
 *         url = "https://coinmarketcap.com/currencies/%s/historical-data/" % name
 */
        goto __pyx_L5_break;

        /* "pymarketcap/core.pyx":570
 *             response["slug"] = name
 *             for symbol, slug in self.correspondences.items():
 *                 if slug == name:             # <<<<<<<<<<<<<<
 *                     response["symbol"] = symbol
 *                     break
 */
      }

      /* "pymarketcap/core.pyx":569
 *         else:
 *             response["slug"] = name
 *             for symbol, slug in self.correspondences.items():             # <<<<<<<<<<<<<<
 *                 if slug == name:
 *                     response["symbol"] = symbol
 */
    }
    __pyx_L5_break:;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  }
  __pyx_L3:;

  /* "pymarketcap/core.pyx":574
 *                     break
 * 
 *         url = "https://coinmarketcap.com/currencies/%s/historical-data/" % name             # <<<<<<<<<<<<<<
 *         _start = "%d" % start.year + "%02d" % start.month + "%02d" % start.day
 *         _end = "%d" % end.year + "%02d" % end.month + "%02d" % end.day
 */
  __pyx_t_3 = __Pyx_PyString_Format(__pyx_kp_s_https_coinmarketcap_com_currenci_2, __pyx_v_name); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 574, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_v_url = ((PyObject*)__pyx_t_3);
  __pyx_t_3 = 0;

  /* "pymarketcap/core.pyx":575
 * 
 *         url = "https://coinmarketcap.com/currencies/%s/historical-data/" % name
 *         _start = "%d" % start.year + "%02d" % start.month + "%02d" % start.day             # <<<<<<<<<<<<<<
 *         _end = "%d" % end.year + "%02d" % end.month + "%02d" % end.day
 *         url += "?start=%s" % _start + "&" + "end=%s" % _end
 */
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_start, __pyx_n_s_year); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 575, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_2 = __Pyx_PyString_Format(__pyx_kp_s_d, __pyx_t_3); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 575, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_start, __pyx_n_s_month); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 575, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_6 = __Pyx_PyString_Format(__pyx_kp_s_02d, __pyx_t_3); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 575, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = PyNumber_Add(__pyx_t_2, __pyx_t_6); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 575, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_v_start, __pyx_n_s_day); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 575, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_2 = __Pyx_PyString_Format(__pyx_kp_s_02d, __pyx_t_6); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 575, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  __pyx_t_6 = PyNumber_Add(__pyx_t_3, __pyx_t_2); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 575, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v__start = __pyx_t_6;
  __pyx_t_6 = 0;

  /* "pymarketcap/core.pyx":576
 *         url = "https://coinmarketcap.com/currencies/%s/historical-data/" % name
 *         _start = "%d" % start.year + "%02d" % start.month + "%02d" % start.day
 *         _end = "%d" % end.year + "%02d" % end.month + "%02d" % end.day             # <<<<<<<<<<<<<<
 *         url += "?start=%s" % _start + "&" + "end=%s" % _end
 * 
 */
  __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_v_end, __pyx_n_s_year); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 576, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_2 = __Pyx_PyString_Format(__pyx_kp_s_d, __pyx_t_6); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 576, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_v_end, __pyx_n_s_month); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 576, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_3 = __Pyx_PyString_Format(__pyx_kp_s_02d, __pyx_t_6); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 576, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  __pyx_t_6 = PyNumber_Add(__pyx_t_2, __pyx_t_3); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 576, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_end, __pyx_n_s_day); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 576, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_2 = __Pyx_PyString_Format(__pyx_kp_s_02d, __pyx_t_3); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 576, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = PyNumber_Add(__pyx_t_6, __pyx_t_2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 576, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v__end = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "pymarketcap/core.pyx":577
 *         _start = "%d" % start.year + "%02d" % start.month + "%02d" % start.day
 *         _end = "%d" % end.year + "%02d" % end.month + "%02d" % end.day
 *         url += "?start=%s" % _start + "&" + "end=%s" % _end             # <<<<<<<<<<<<<<
 * 
 *         try:
 */
  __pyx_t_3 = __Pyx_PyString_Format(__pyx_kp_s_start_s, __pyx_v__start); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 577, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_2 = PyNumber_Add(__pyx_t_3, __pyx_kp_s__11); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 577, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = __Pyx_PyString_Format(__pyx_kp_s_end_s, __pyx_v__end); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 577, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_6 = PyNumber_Add(__pyx_t_2, __pyx_t_3); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 577, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = __Pyx_PyUnicode_Concat(__pyx_v_url, __pyx_t_6); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 577, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  __Pyx_DECREF_SET(__pyx_v_url, ((PyObject*)__pyx_t_3));
  __pyx_t_3 = 0;

  /* "pymarketcap/core.pyx":579
 *         url += "?start=%s" % _start + "&" + "end=%s" % _end
 * 
 *         try:             # <<<<<<<<<<<<<<
 *             res = self._get(url.encode())[50000:]
 *         except CoinmarketcapHTTPError404:
 */
  {
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    __Pyx_ExceptionSave(&__pyx_t_11, &__pyx_t_12, &__pyx_t_13);
    __Pyx_XGOTREF(__pyx_t_11);
    __Pyx_XGOTREF(__pyx_t_12);
    __Pyx_XGOTREF(__pyx_t_13);
    /*try:*/ {

      /* "pymarketcap/core.pyx":580
 * 
 *         try:
 *             res = self._get(url.encode())[50000:]             # <<<<<<<<<<<<<<
 *         except CoinmarketcapHTTPError404:
 *             if name not in self.coins:
 */
      __pyx_t_3 = PyUnicode_AsEncodedString(__pyx_v_url, NULL, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 580, __pyx_L9_error)
      __Pyx_GOTREF(__pyx_t_3);
      __pyx_t_14 = __Pyx_PyBytes_AsWritableString(__pyx_t_3); if (unlikely((!__pyx_t_14) && PyErr_Occurred())) __PYX_ERR(0, 580, __pyx_L9_error)
      __pyx_t_6 = ((struct __pyx_vtabstruct_11pymarketcap_4core_Pymarketcap *)__pyx_v_self->__pyx_vtab)->_get(__pyx_v_self, __pyx_t_14); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 580, __pyx_L9_error)
      __Pyx_GOTREF(__pyx_t_6);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __pyx_t_3 = __Pyx_PyObject_GetSlice(__pyx_t_6, 0xC350, 0, NULL, NULL, &__pyx_slice__12, 1, 0, 1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 580, __pyx_L9_error)
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      __pyx_v_res = __pyx_t_3;
      __pyx_t_3 = 0;

      /* "pymarketcap/core.pyx":579
 *         url += "?start=%s" % _start + "&" + "end=%s" % _end
 * 
 *         try:             # <<<<<<<<<<<<<<
 *             res = self._get(url.encode())[50000:]
 *         except CoinmarketcapHTTPError404:
 */
    }
    __Pyx_XDECREF(__pyx_t_11); __pyx_t_11 = 0;
    __Pyx_XDECREF(__pyx_t_12); __pyx_t_12 = 0;
    __Pyx_XDECREF(__pyx_t_13); __pyx_t_13 = 0;
    goto __pyx_L14_try_end;
    __pyx_L9_error:;
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "pymarketcap/core.pyx":581
 *         try:
 *             res = self._get(url.encode())[50000:]
 *         except CoinmarketcapHTTPError404:             # <<<<<<<<<<<<<<
 *             if name not in self.coins:
 *                 raise ValueError("%s is not a valid currrency name. See 'symbols'" % name \
 */
    __Pyx_ErrFetch(&__pyx_t_3, &__pyx_t_6, &__pyx_t_2);
    __pyx_t_1 = __Pyx_GetModuleGlobalName(__pyx_n_s_CoinmarketcapHTTPError404); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 581, __pyx_L11_except_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_5 = __Pyx_PyErr_GivenExceptionMatches(__pyx_t_3, __pyx_t_1);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_ErrRestore(__pyx_t_3, __pyx_t_6, __pyx_t_2);
    __pyx_t_3 = 0; __pyx_t_6 = 0; __pyx_t_2 = 0;
    if (__pyx_t_5) {
      __Pyx_AddTraceback("pymarketcap.core.Pymarketcap.historical", __pyx_clineno, __pyx_lineno, __pyx_filename);
      if (__Pyx_GetException(&__pyx_t_2, &__pyx_t_6, &__pyx_t_3) < 0) __PYX_ERR(0, 581, __pyx_L11_except_error)
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_GOTREF(__pyx_t_6);
      __Pyx_GOTREF(__pyx_t_3);

      /* "pymarketcap/core.pyx":582
 *             res = self._get(url.encode())[50000:]
 *         except CoinmarketcapHTTPError404:
 *             if name not in self.coins:             # <<<<<<<<<<<<<<
 *                 raise ValueError("%s is not a valid currrency name. See 'symbols'" % name \
 *                                  + " or 'coins' properties for get all valid currencies.")
 */
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_coins); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 582, __pyx_L11_except_error)
      __Pyx_GOTREF(__pyx_t_1);
      __pyx_t_7 = (__Pyx_PySequence_ContainsTF(__pyx_v_name, __pyx_t_1, Py_NE)); if (unlikely(__pyx_t_7 < 0)) __PYX_ERR(0, 582, __pyx_L11_except_error)
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __pyx_t_15 = (__pyx_t_7 != 0);
      if (unlikely(__pyx_t_15)) {

        /* "pymarketcap/core.pyx":583
 *         except CoinmarketcapHTTPError404:
 *             if name not in self.coins:
 *                 raise ValueError("%s is not a valid currrency name. See 'symbols'" % name \             # <<<<<<<<<<<<<<
 *                                  + " or 'coins' properties for get all valid currencies.")
 *             else:
 */
        __pyx_t_1 = __Pyx_PyString_Format(__pyx_kp_s_s_is_not_a_valid_currrency_name, __pyx_v_name); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 583, __pyx_L11_except_error)
        __Pyx_GOTREF(__pyx_t_1);

        /* "pymarketcap/core.pyx":584
 *             if name not in self.coins:
 *                 raise ValueError("%s is not a valid currrency name. See 'symbols'" % name \
 *                                  + " or 'coins' properties for get all valid currencies.")             # <<<<<<<<<<<<<<
 *             else:
 *                 raise NotImplementedError
 */
        __pyx_t_4 = __Pyx_PyUnicode_Concat(__pyx_t_1, __pyx_kp_s_or_coins_properties_for_get_all); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 584, __pyx_L11_except_error)
        __Pyx_GOTREF(__pyx_t_4);
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

        /* "pymarketcap/core.pyx":583
 *         except CoinmarketcapHTTPError404:
 *             if name not in self.coins:
 *                 raise ValueError("%s is not a valid currrency name. See 'symbols'" % name \             # <<<<<<<<<<<<<<
 *                                  + " or 'coins' properties for get all valid currencies.")
 *             else:
 */
        __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_builtin_ValueError, __pyx_t_4); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 583, __pyx_L11_except_error)
        __Pyx_GOTREF(__pyx_t_1);
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
        __Pyx_Raise(__pyx_t_1, 0, 0, 0);
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        __PYX_ERR(0, 583, __pyx_L11_except_error)

        /* "pymarketcap/core.pyx":582
 *             res = self._get(url.encode())[50000:]
 *         except CoinmarketcapHTTPError404:
 *             if name not in self.coins:             # <<<<<<<<<<<<<<
 *                 raise ValueError("%s is not a valid currrency name. See 'symbols'" % name \
 *                                  + " or 'coins' properties for get all valid currencies.")
 */
      }

      /* "pymarketcap/core.pyx":586
 *                                  + " or 'coins' properties for get all valid currencies.")
 *             else:
 *                 raise NotImplementedError             # <<<<<<<<<<<<<<
 * 
 *         response["history"] = processer.historical(res, start, end, revert)
 */
      /*else*/ {
        __Pyx_Raise(__pyx_builtin_NotImplementedError, 0, 0, 0);
        __PYX_ERR(0, 586, __pyx_L11_except_error)
      }
    }
    goto __pyx_L11_except_error;
    __pyx_L11_except_error:;

    /* "pymarketcap/core.pyx":579
 *         url += "?start=%s" % _start + "&" + "end=%s" % _end
 * 
 *         try:             # <<<<<<<<<<<<<<
 *             res = self._get(url.encode())[50000:]
 *         except CoinmarketcapHTTPError404:
 */
    __Pyx_XGIVEREF(__pyx_t_11);
    __Pyx_XGIVEREF(__pyx_t_12);
    __Pyx_XGIVEREF(__pyx_t_13);
    __Pyx_ExceptionReset(__pyx_t_11, __pyx_t_12, __pyx_t_13);
    goto __pyx_L1_error;
    __pyx_L14_try_end:;
  }

  /* "pymarketcap/core.pyx":588
 *                 raise NotImplementedError
 * 
 *         response["history"] = processer.historical(res, start, end, revert)             # <<<<<<<<<<<<<<
 *         return response
 * 
 */
  __pyx_t_6 = __Pyx_GetModuleGlobalName(__pyx_n_s_processer); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 588, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_6, __pyx_n_s_historical); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 588, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  __pyx_t_6 = NULL;
  __pyx_t_5 = 0;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_6)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_6);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_5 = 1;
    }
  }
  #if CYTHON_FAST_PYCALL
  if (PyFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[5] = {__pyx_t_6, __pyx_v_res, __pyx_v_start, __pyx_v_end, __pyx_v_revert};
    __pyx_t_3 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_5, 4+__pyx_t_5); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 588, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_GOTREF(__pyx_t_3);
  } else
  #endif
  #if CYTHON_FAST_PYCCALL
  if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[5] = {__pyx_t_6, __pyx_v_res, __pyx_v_start, __pyx_v_end, __pyx_v_revert};
    __pyx_t_3 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_5, 4+__pyx_t_5); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 588, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_GOTREF(__pyx_t_3);
  } else
  #endif
  {
    __pyx_t_1 = PyTuple_New(4+__pyx_t_5); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 588, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    if (__pyx_t_6) {
      __Pyx_GIVEREF(__pyx_t_6); PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_t_6); __pyx_t_6 = NULL;
    }
    __Pyx_INCREF(__pyx_v_res);
    __Pyx_GIVEREF(__pyx_v_res);
    PyTuple_SET_ITEM(__pyx_t_1, 0+__pyx_t_5, __pyx_v_res);
    __Pyx_INCREF(__pyx_v_start);
    __Pyx_GIVEREF(__pyx_v_start);
    PyTuple_SET_ITEM(__pyx_t_1, 1+__pyx_t_5, __pyx_v_start);
    __Pyx_INCREF(__pyx_v_end);
    __Pyx_GIVEREF(__pyx_v_end);
    PyTuple_SET_ITEM(__pyx_t_1, 2+__pyx_t_5, __pyx_v_end);
    __Pyx_INCREF(__pyx_v_revert);
    __Pyx_GIVEREF(__pyx_v_revert);
    PyTuple_SET_ITEM(__pyx_t_1, 3+__pyx_t_5, __pyx_v_revert);
    __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_1, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 588, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (unlikely(PyDict_SetItem(__pyx_v_response, __pyx_n_s_history, __pyx_t_3) < 0)) __PYX_ERR(0, 588, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "pymarketcap/core.pyx":589
 * 
 *         response["history"] = processer.historical(res, start, end, revert)
 *         return response             # <<<<<<<<<<<<<<
 * 
 *     cpdef recently(self, convert="USD"):
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_response);
  __pyx_r = __pyx_v_response;
  goto __pyx_L0;

  /* "pymarketcap/core.pyx":541
 *         return processer.ranks(res)
 * 
 *     cpdef historical(self, unicode name,             # <<<<<<<<<<<<<<
 *                      start=datetime(2008, 8, 18),
 *                      end=datetime.now(),
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_AddTraceback("pymarketcap.core.Pymarketcap.historical", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_response);
  __Pyx_XDECREF(__pyx_v_symbol);
  __Pyx_XDECREF(__pyx_v_slug);
  __Pyx_XDECREF(__pyx_v_url);
  __Pyx_XDECREF(__pyx_v__start);
  __Pyx_XDECREF(__pyx_v__end);
  __Pyx_XDECREF(__pyx_v_res);
  __Pyx_XDECREF(__pyx_v_name);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_11pymarketcap_4core_11Pymarketcap_27historical(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_11pymarketcap_4core_11Pymarketcap_26historical[] = "Get historical data for a currency.\n\n        Args:\n            name (str): Currency to scrap historical data.\n            start (date, optional): Time to start scraping\n                periods as datetime.datetime type.\n                As default ``datetime(2008, 8, 18)``.\n            end (date, optional): Time to end scraping periods\n                as datetime.datetime type. As default ``datetime.now()``.\n            revert (bool, optional): If ``False``, return first date\n                first, in chronological order, otherwise returns\n                reversed list of periods. As default ``False``.\n\n        Returns (list):\n            Historical dayly OHLC for a currency.\n        ";
static PyObject *__pyx_pw_11pymarketcap_4core_11Pymarketcap_27historical(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_name = 0;
  PyObject *__pyx_v_start = 0;
  PyObject *__pyx_v_end = 0;
  PyObject *__pyx_v_revert = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("historical (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_name,&__pyx_n_s_start,&__pyx_n_s_end,&__pyx_n_s_revert,0};
    PyObject* values[4] = {0,0,0,0};
    values[1] = __pyx_k__9;
    values[2] = __pyx_k__10;

    /* "pymarketcap/core.pyx":544
 *                      start=datetime(2008, 8, 18),
 *                      end=datetime.now(),
 *                      revert=False):             # <<<<<<<<<<<<<<
 *         """Get historical data for a currency.
 * 
 */
    values[3] = ((PyObject *)Py_False);
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        CYTHON_FALLTHROUGH;
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_name)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_start);
          if (value) { values[1] = value; kw_args--; }
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_end);
          if (value) { values[2] = value; kw_args--; }
        }
        CYTHON_FALLTHROUGH;
        case  3:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_revert);
          if (value) { values[3] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "historical") < 0)) __PYX_ERR(0, 541, __pyx_L3_error)
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        CYTHON_FALLTHROUGH;
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_name = ((PyObject*)values[0]);
    __pyx_v_start = values[1];
    __pyx_v_end = values[2];
    __pyx_v_revert = values[3];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("historical", 0, 1, 4, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 541, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("pymarketcap.core.Pymarketcap.historical", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_name), (&PyUnicode_Type), 1, "name", 1))) __PYX_ERR(0, 541, __pyx_L1_error)
  __pyx_r = __pyx_pf_11pymarketcap_4core_11Pymarketcap_26historical(((struct __pyx_obj_11pymarketcap_4core_Pymarketcap *)__pyx_v_self), __pyx_v_name, __pyx_v_start, __pyx_v_end, __pyx_v_revert);

  /* "pymarketcap/core.pyx":541
 *         return processer.ranks(res)
 * 
 *     cpdef historical(self, unicode name,             # <<<<<<<<<<<<<<
 *                      start=datetime(2008, 8, 18),
 *                      end=datetime.now(),
 */

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_11pymarketcap_4core_11Pymarketcap_26historical(struct __pyx_obj_11pymarketcap_4core_Pymarketcap *__pyx_v_self, PyObject *__pyx_v_name, PyObject *__pyx_v_start, PyObject *__pyx_v_end, PyObject *__pyx_v_revert) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  struct __pyx_opt_args_11pymarketcap_4core_11Pymarketcap_historical __pyx_t_2;
  __Pyx_RefNannySetupContext("historical", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2.__pyx_n = 3;
  __pyx_t_2.start = __pyx_v_start;
  __pyx_t_2.end = __pyx_v_end;
  __pyx_t_2.revert = __pyx_v_revert;
  __pyx_t_1 = __pyx_vtabptr_11pymarketcap_4core_Pymarketcap->historical(__pyx_v_self, __pyx_v_name, 1, &__pyx_t_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 541, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pymarketcap.core.Pymarketcap.historical", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pymarketcap/core.pyx":591
 *         return response
 * 
 *     cpdef recently(self, convert="USD"):             # <<<<<<<<<<<<<<
 *         """Get recently added currencies along with other metadata.
 * 
 */

static PyObject *__pyx_pw_11pymarketcap_4core_11Pymarketcap_29recently(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyObject *__pyx_f_11pymarketcap_4core_11Pymarketcap_recently(struct __pyx_obj_11pymarketcap_4core_Pymarketcap *__pyx_v_self, int __pyx_skip_dispatch, struct __pyx_opt_args_11pymarketcap_4core_11Pymarketcap_recently *__pyx_optional_args) {
  PyObject *__pyx_v_convert = ((PyObject *)__pyx_n_s_USD);
  PyObject *__pyx_v_url = NULL;
  PyObject *__pyx_v_res = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  char *__pyx_t_6;
  int __pyx_t_7;
  __Pyx_RefNannySetupContext("recently", 0);
  if (__pyx_optional_args) {
    if (__pyx_optional_args->__pyx_n > 0) {
      __pyx_v_convert = __pyx_optional_args->convert;
    }
  }
  __Pyx_INCREF(__pyx_v_convert);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0)) {
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_recently); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 591, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)__pyx_pw_11pymarketcap_4core_11Pymarketcap_29recently)) {
      __Pyx_XDECREF(__pyx_r);
      __Pyx_INCREF(__pyx_t_1);
      __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
      if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
        __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
        if (likely(__pyx_t_4)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
          __Pyx_INCREF(__pyx_t_4);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_3, function);
        }
      }
      if (!__pyx_t_4) {
        __pyx_t_2 = __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_v_convert); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 591, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
      } else {
        #if CYTHON_FAST_PYCALL
        if (PyFunction_Check(__pyx_t_3)) {
          PyObject *__pyx_temp[2] = {__pyx_t_4, __pyx_v_convert};
          __pyx_t_2 = __Pyx_PyFunction_FastCall(__pyx_t_3, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 591, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
          __Pyx_GOTREF(__pyx_t_2);
        } else
        #endif
        #if CYTHON_FAST_PYCCALL
        if (__Pyx_PyFastCFunction_Check(__pyx_t_3)) {
          PyObject *__pyx_temp[2] = {__pyx_t_4, __pyx_v_convert};
          __pyx_t_2 = __Pyx_PyCFunction_FastCall(__pyx_t_3, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 591, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
          __Pyx_GOTREF(__pyx_t_2);
        } else
        #endif
        {
          __pyx_t_5 = PyTuple_New(1+1); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 591, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_5);
          __Pyx_GIVEREF(__pyx_t_4); PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_4); __pyx_t_4 = NULL;
          __Pyx_INCREF(__pyx_v_convert);
          __Pyx_GIVEREF(__pyx_v_convert);
          PyTuple_SET_ITEM(__pyx_t_5, 0+1, __pyx_v_convert);
          __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_t_5, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 591, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_2);
          __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
        }
      }
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __pyx_r = __pyx_t_2;
      __pyx_t_2 = 0;
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      goto __pyx_L0;
    }
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  }

  /* "pymarketcap/core.pyx":602
 *             Recently added currencies data.
 *         """
 *         convert = convert.lower()             # <<<<<<<<<<<<<<
 *         url = b"https://coinmarketcap.com/new/"
 *         res = self._get(url)
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_convert, __pyx_n_s_lower); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 602, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  if (__pyx_t_3) {
    __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_3); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 602, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  } else {
    __pyx_t_1 = __Pyx_PyObject_CallNoArg(__pyx_t_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 602, __pyx_L1_error)
  }
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF_SET(__pyx_v_convert, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "pymarketcap/core.pyx":603
 *         """
 *         convert = convert.lower()
 *         url = b"https://coinmarketcap.com/new/"             # <<<<<<<<<<<<<<
 *         res = self._get(url)
 * 
 */
  __Pyx_INCREF(__pyx_kp_b_https_coinmarketcap_com_new);
  __pyx_v_url = __pyx_kp_b_https_coinmarketcap_com_new;

  /* "pymarketcap/core.pyx":604
 *         convert = convert.lower()
 *         url = b"https://coinmarketcap.com/new/"
 *         res = self._get(url)             # <<<<<<<<<<<<<<
 * 
 *         return list(processer.recently(res, convert))
 */
  __pyx_t_6 = __Pyx_PyBytes_AsWritableString(__pyx_v_url); if (unlikely((!__pyx_t_6) && PyErr_Occurred())) __PYX_ERR(0, 604, __pyx_L1_error)
  __pyx_t_1 = ((struct __pyx_vtabstruct_11pymarketcap_4core_Pymarketcap *)__pyx_v_self->__pyx_vtab)->_get(__pyx_v_self, __pyx_t_6); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 604, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_res = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "pymarketcap/core.pyx":606
 *         res = self._get(url)
 * 
 *         return list(processer.recently(res, convert))             # <<<<<<<<<<<<<<
 * 
 *     cpdef exchange(self, unicode name, convert="USD"):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_processer); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 606, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_recently); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 606, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = NULL;
  __pyx_t_7 = 0;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_3);
    if (likely(__pyx_t_2)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_2);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_3, function);
      __pyx_t_7 = 1;
    }
  }
  #if CYTHON_FAST_PYCALL
  if (PyFunction_Check(__pyx_t_3)) {
    PyObject *__pyx_temp[3] = {__pyx_t_2, __pyx_v_res, __pyx_v_convert};
    __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_3, __pyx_temp+1-__pyx_t_7, 2+__pyx_t_7); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 606, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_GOTREF(__pyx_t_1);
  } else
  #endif
  #if CYTHON_FAST_PYCCALL
  if (__Pyx_PyFastCFunction_Check(__pyx_t_3)) {
    PyObject *__pyx_temp[3] = {__pyx_t_2, __pyx_v_res, __pyx_v_convert};
    __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_3, __pyx_temp+1-__pyx_t_7, 2+__pyx_t_7); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 606, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_GOTREF(__pyx_t_1);
  } else
  #endif
  {
    __pyx_t_5 = PyTuple_New(2+__pyx_t_7); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 606, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    if (__pyx_t_2) {
      __Pyx_GIVEREF(__pyx_t_2); PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_2); __pyx_t_2 = NULL;
    }
    __Pyx_INCREF(__pyx_v_res);
    __Pyx_GIVEREF(__pyx_v_res);
    PyTuple_SET_ITEM(__pyx_t_5, 0+__pyx_t_7, __pyx_v_res);
    __Pyx_INCREF(__pyx_v_convert);
    __Pyx_GIVEREF(__pyx_v_convert);
    PyTuple_SET_ITEM(__pyx_t_5, 1+__pyx_t_7, __pyx_v_convert);
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_t_5, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 606, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  }
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = PySequence_List(__pyx_t_1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 606, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_r = __pyx_t_3;
  __pyx_t_3 = 0;
  goto __pyx_L0;

  /* "pymarketcap/core.pyx":591
 *         return response
 * 
 *     cpdef recently(self, convert="USD"):             # <<<<<<<<<<<<<<
 *         """Get recently added currencies along with other metadata.
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("pymarketcap.core.Pymarketcap.recently", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_url);
  __Pyx_XDECREF(__pyx_v_res);
  __Pyx_XDECREF(__pyx_v_convert);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_11pymarketcap_4core_11Pymarketcap_29recently(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_11pymarketcap_4core_11Pymarketcap_28recently[] = "Get recently added currencies along with other metadata.\n\n        Args:\n            convert (str, optional): Convert market_caps, prices,\n                volumes and percent_changes between USD and BTC.\n                As default ``\"USD\"``.\n\n        Returns (list):\n            Recently added currencies data.\n        ";
static PyObject *__pyx_pw_11pymarketcap_4core_11Pymarketcap_29recently(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_convert = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("recently (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_convert,0};
    PyObject* values[1] = {0};
    values[0] = ((PyObject *)__pyx_n_s_USD);
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_convert);
          if (value) { values[0] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "recently") < 0)) __PYX_ERR(0, 591, __pyx_L3_error)
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_convert = values[0];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("recently", 0, 0, 1, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 591, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("pymarketcap.core.Pymarketcap.recently", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_11pymarketcap_4core_11Pymarketcap_28recently(((struct __pyx_obj_11pymarketcap_4core_Pymarketcap *)__pyx_v_self), __pyx_v_convert);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_11pymarketcap_4core_11Pymarketcap_28recently(struct __pyx_obj_11pymarketcap_4core_Pymarketcap *__pyx_v_self, PyObject *__pyx_v_convert) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  struct __pyx_opt_args_11pymarketcap_4core_11Pymarketcap_recently __pyx_t_2;
  __Pyx_RefNannySetupContext("recently", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2.__pyx_n = 1;
  __pyx_t_2.convert = __pyx_v_convert;
  __pyx_t_1 = __pyx_vtabptr_11pymarketcap_4core_Pymarketcap->recently(__pyx_v_self, 1, &__pyx_t_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 591, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pymarketcap.core.Pymarketcap.recently", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pymarketcap/core.pyx":608
 *         return list(processer.recently(res, convert))
 * 
 *     cpdef exchange(self, unicode name, convert="USD"):             # <<<<<<<<<<<<<<
 *         """Obtain data from a exchange passed as argument. See ``exchanges_slugs``
 *         property for obtain all posibles values.
 */

static PyObject *__pyx_pw_11pymarketcap_4core_11Pymarketcap_31exchange(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyObject *__pyx_f_11pymarketcap_4core_11Pymarketcap_exchange(struct __pyx_obj_11pymarketcap_4core_Pymarketcap *__pyx_v_self, PyObject *__pyx_v_name, int __pyx_skip_dispatch, struct __pyx_opt_args_11pymarketcap_4core_11Pymarketcap_exchange *__pyx_optional_args) {
  PyObject *__pyx_v_convert = ((PyObject *)__pyx_n_s_USD);
  PyObject *__pyx_v_url = NULL;
  PyObject *__pyx_v_res = NULL;
  PyObject *__pyx_v_response = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  char *__pyx_t_10;
  int __pyx_t_11;
  int __pyx_t_12;
  __Pyx_RefNannySetupContext("exchange", 0);
  if (__pyx_optional_args) {
    if (__pyx_optional_args->__pyx_n > 0) {
      __pyx_v_convert = __pyx_optional_args->convert;
    }
  }
  __Pyx_INCREF(__pyx_v_convert);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0)) {
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_exchange); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 608, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)__pyx_pw_11pymarketcap_4core_11Pymarketcap_31exchange)) {
      __Pyx_XDECREF(__pyx_r);
      __Pyx_INCREF(__pyx_t_1);
      __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
      __pyx_t_5 = 0;
      if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
        __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
        if (likely(__pyx_t_4)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
          __Pyx_INCREF(__pyx_t_4);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_3, function);
          __pyx_t_5 = 1;
        }
      }
      #if CYTHON_FAST_PYCALL
      if (PyFunction_Check(__pyx_t_3)) {
        PyObject *__pyx_temp[3] = {__pyx_t_4, __pyx_v_name, __pyx_v_convert};
        __pyx_t_2 = __Pyx_PyFunction_FastCall(__pyx_t_3, __pyx_temp+1-__pyx_t_5, 2+__pyx_t_5); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 608, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        __Pyx_GOTREF(__pyx_t_2);
      } else
      #endif
      #if CYTHON_FAST_PYCCALL
      if (__Pyx_PyFastCFunction_Check(__pyx_t_3)) {
        PyObject *__pyx_temp[3] = {__pyx_t_4, __pyx_v_name, __pyx_v_convert};
        __pyx_t_2 = __Pyx_PyCFunction_FastCall(__pyx_t_3, __pyx_temp+1-__pyx_t_5, 2+__pyx_t_5); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 608, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        __Pyx_GOTREF(__pyx_t_2);
      } else
      #endif
      {
        __pyx_t_6 = PyTuple_New(2+__pyx_t_5); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 608, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_6);
        if (__pyx_t_4) {
          __Pyx_GIVEREF(__pyx_t_4); PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_4); __pyx_t_4 = NULL;
        }
        __Pyx_INCREF(__pyx_v_name);
        __Pyx_GIVEREF(__pyx_v_name);
        PyTuple_SET_ITEM(__pyx_t_6, 0+__pyx_t_5, __pyx_v_name);
        __Pyx_INCREF(__pyx_v_convert);
        __Pyx_GIVEREF(__pyx_v_convert);
        PyTuple_SET_ITEM(__pyx_t_6, 1+__pyx_t_5, __pyx_v_convert);
        __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_t_6, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 608, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      }
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __pyx_r = __pyx_t_2;
      __pyx_t_2 = 0;
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      goto __pyx_L0;
    }
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  }

  /* "pymarketcap/core.pyx":623
 *             ``"volume"`` (total), ``"social"`` and ``"markets"``.
 *         """
 *         url = "https://coinmarketcap.com/exchanges/%s/" % name             # <<<<<<<<<<<<<<
 *         try:
 *             res = self._get(url.encode())[20000:]
 */
  __pyx_t_1 = __Pyx_PyString_Format(__pyx_kp_s_https_coinmarketcap_com_exchange, __pyx_v_name); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 623, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_url = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "pymarketcap/core.pyx":624
 *         """
 *         url = "https://coinmarketcap.com/exchanges/%s/" % name
 *         try:             # <<<<<<<<<<<<<<
 *             res = self._get(url.encode())[20000:]
 *         except CoinmarketcapHTTPError404:
 */
  {
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    __Pyx_ExceptionSave(&__pyx_t_7, &__pyx_t_8, &__pyx_t_9);
    __Pyx_XGOTREF(__pyx_t_7);
    __Pyx_XGOTREF(__pyx_t_8);
    __Pyx_XGOTREF(__pyx_t_9);
    /*try:*/ {

      /* "pymarketcap/core.pyx":625
 *         url = "https://coinmarketcap.com/exchanges/%s/" % name
 *         try:
 *             res = self._get(url.encode())[20000:]             # <<<<<<<<<<<<<<
 *         except CoinmarketcapHTTPError404:
 *             if name not in self.exchange_slugs:
 */
      __pyx_t_1 = PyUnicode_AsEncodedString(__pyx_v_url, NULL, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 625, __pyx_L3_error)
      __Pyx_GOTREF(__pyx_t_1);
      __pyx_t_10 = __Pyx_PyBytes_AsWritableString(__pyx_t_1); if (unlikely((!__pyx_t_10) && PyErr_Occurred())) __PYX_ERR(0, 625, __pyx_L3_error)
      __pyx_t_2 = ((struct __pyx_vtabstruct_11pymarketcap_4core_Pymarketcap *)__pyx_v_self->__pyx_vtab)->_get(__pyx_v_self, __pyx_t_10); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 625, __pyx_L3_error)
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __pyx_t_1 = __Pyx_PyObject_GetSlice(__pyx_t_2, 0x4E20, 0, NULL, NULL, &__pyx_slice__13, 1, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 625, __pyx_L3_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __pyx_v_res = __pyx_t_1;
      __pyx_t_1 = 0;

      /* "pymarketcap/core.pyx":624
 *         """
 *         url = "https://coinmarketcap.com/exchanges/%s/" % name
 *         try:             # <<<<<<<<<<<<<<
 *             res = self._get(url.encode())[20000:]
 *         except CoinmarketcapHTTPError404:
 */
    }

    /* "pymarketcap/core.pyx":633
 *                 raise NotImplementedError
 *         else:
 *             response = {"slug": name}             # <<<<<<<<<<<<<<
 *         convert = convert.lower()
 * 
 */
    /*else:*/ {
      __pyx_t_1 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 633, __pyx_L5_except_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_slug, __pyx_v_name) < 0) __PYX_ERR(0, 633, __pyx_L5_except_error)
      __pyx_v_response = ((PyObject*)__pyx_t_1);
      __pyx_t_1 = 0;
    }
    __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
    __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
    __Pyx_XDECREF(__pyx_t_9); __pyx_t_9 = 0;
    goto __pyx_L8_try_end;
    __pyx_L3_error:;
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "pymarketcap/core.pyx":626
 *         try:
 *             res = self._get(url.encode())[20000:]
 *         except CoinmarketcapHTTPError404:             # <<<<<<<<<<<<<<
 *             if name not in self.exchange_slugs:
 *                 raise ValueError("%s is not a valid exchange name. See exchange_slugs" % name \
 */
    __Pyx_ErrFetch(&__pyx_t_1, &__pyx_t_2, &__pyx_t_3);
    __pyx_t_6 = __Pyx_GetModuleGlobalName(__pyx_n_s_CoinmarketcapHTTPError404); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 626, __pyx_L5_except_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_5 = __Pyx_PyErr_GivenExceptionMatches(__pyx_t_1, __pyx_t_6);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_ErrRestore(__pyx_t_1, __pyx_t_2, __pyx_t_3);
    __pyx_t_1 = 0; __pyx_t_2 = 0; __pyx_t_3 = 0;
    if (__pyx_t_5) {
      __Pyx_AddTraceback("pymarketcap.core.Pymarketcap.exchange", __pyx_clineno, __pyx_lineno, __pyx_filename);
      if (__Pyx_GetException(&__pyx_t_3, &__pyx_t_2, &__pyx_t_1) < 0) __PYX_ERR(0, 626, __pyx_L5_except_error)
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_GOTREF(__pyx_t_1);

      /* "pymarketcap/core.pyx":627
 *             res = self._get(url.encode())[20000:]
 *         except CoinmarketcapHTTPError404:
 *             if name not in self.exchange_slugs:             # <<<<<<<<<<<<<<
 *                 raise ValueError("%s is not a valid exchange name. See exchange_slugs" % name \
 *                                  + " property for get all valid exchanges.")
 */
      __pyx_t_6 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_exchange_slugs); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 627, __pyx_L5_except_error)
      __Pyx_GOTREF(__pyx_t_6);
      __pyx_t_11 = (__Pyx_PySequence_ContainsTF(__pyx_v_name, __pyx_t_6, Py_NE)); if (unlikely(__pyx_t_11 < 0)) __PYX_ERR(0, 627, __pyx_L5_except_error)
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      __pyx_t_12 = (__pyx_t_11 != 0);
      if (unlikely(__pyx_t_12)) {

        /* "pymarketcap/core.pyx":628
 *         except CoinmarketcapHTTPError404:
 *             if name not in self.exchange_slugs:
 *                 raise ValueError("%s is not a valid exchange name. See exchange_slugs" % name \             # <<<<<<<<<<<<<<
 *                                  + " property for get all valid exchanges.")
 *             else:
 */
        __pyx_t_6 = __Pyx_PyString_Format(__pyx_kp_s_s_is_not_a_valid_exchange_name, __pyx_v_name); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 628, __pyx_L5_except_error)
        __Pyx_GOTREF(__pyx_t_6);

        /* "pymarketcap/core.pyx":629
 *             if name not in self.exchange_slugs:
 *                 raise ValueError("%s is not a valid exchange name. See exchange_slugs" % name \
 *                                  + " property for get all valid exchanges.")             # <<<<<<<<<<<<<<
 *             else:
 *                 raise NotImplementedError
 */
        __pyx_t_4 = __Pyx_PyUnicode_Concat(__pyx_t_6, __pyx_kp_s_property_for_get_all_valid_exch); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 629, __pyx_L5_except_error)
        __Pyx_GOTREF(__pyx_t_4);
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;

        /* "pymarketcap/core.pyx":628
 *         except CoinmarketcapHTTPError404:
 *             if name not in self.exchange_slugs:
 *                 raise ValueError("%s is not a valid exchange name. See exchange_slugs" % name \             # <<<<<<<<<<<<<<
 *                                  + " property for get all valid exchanges.")
 *             else:
 */
        __pyx_t_6 = __Pyx_PyObject_CallOneArg(__pyx_builtin_ValueError, __pyx_t_4); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 628, __pyx_L5_except_error)
        __Pyx_GOTREF(__pyx_t_6);
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
        __Pyx_Raise(__pyx_t_6, 0, 0, 0);
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
        __PYX_ERR(0, 628, __pyx_L5_except_error)

        /* "pymarketcap/core.pyx":627
 *             res = self._get(url.encode())[20000:]
 *         except CoinmarketcapHTTPError404:
 *             if name not in self.exchange_slugs:             # <<<<<<<<<<<<<<
 *                 raise ValueError("%s is not a valid exchange name. See exchange_slugs" % name \
 *                                  + " property for get all valid exchanges.")
 */
      }

      /* "pymarketcap/core.pyx":631
 *                                  + " property for get all valid exchanges.")
 *             else:
 *                 raise NotImplementedError             # <<<<<<<<<<<<<<
 *         else:
 *             response = {"slug": name}
 */
      /*else*/ {
        __Pyx_Raise(__pyx_builtin_NotImplementedError, 0, 0, 0);
        __PYX_ERR(0, 631, __pyx_L5_except_error)
      }
    }
    goto __pyx_L5_except_error;
    __pyx_L5_except_error:;

    /* "pymarketcap/core.pyx":624
 *         """
 *         url = "https://coinmarketcap.com/exchanges/%s/" % name
 *         try:             # <<<<<<<<<<<<<<
 *             res = self._get(url.encode())[20000:]
 *         except CoinmarketcapHTTPError404:
 */
    __Pyx_XGIVEREF(__pyx_t_7);
    __Pyx_XGIVEREF(__pyx_t_8);
    __Pyx_XGIVEREF(__pyx_t_9);
    __Pyx_ExceptionReset(__pyx_t_7, __pyx_t_8, __pyx_t_9);
    goto __pyx_L1_error;
    __pyx_L8_try_end:;
  }

  /* "pymarketcap/core.pyx":634
 *         else:
 *             response = {"slug": name}
 *         convert = convert.lower()             # <<<<<<<<<<<<<<
 * 
 *         response.update(processer.exchange(res, convert))
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_convert, __pyx_n_s_lower); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 634, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  if (__pyx_t_3) {
    __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_3); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 634, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  } else {
    __pyx_t_1 = __Pyx_PyObject_CallNoArg(__pyx_t_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 634, __pyx_L1_error)
  }
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF_SET(__pyx_v_convert, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "pymarketcap/core.pyx":636
 *         convert = convert.lower()
 * 
 *         response.update(processer.exchange(res, convert))             # <<<<<<<<<<<<<<
 *         return response
 * 
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_processer); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 636, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_exchange); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 636, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = NULL;
  __pyx_t_5 = 0;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_3);
    if (likely(__pyx_t_2)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_2);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_3, function);
      __pyx_t_5 = 1;
    }
  }
  #if CYTHON_FAST_PYCALL
  if (PyFunction_Check(__pyx_t_3)) {
    PyObject *__pyx_temp[3] = {__pyx_t_2, __pyx_v_res, __pyx_v_convert};
    __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_3, __pyx_temp+1-__pyx_t_5, 2+__pyx_t_5); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 636, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_GOTREF(__pyx_t_1);
  } else
  #endif
  #if CYTHON_FAST_PYCCALL
  if (__Pyx_PyFastCFunction_Check(__pyx_t_3)) {
    PyObject *__pyx_temp[3] = {__pyx_t_2, __pyx_v_res, __pyx_v_convert};
    __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_3, __pyx_temp+1-__pyx_t_5, 2+__pyx_t_5); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 636, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_GOTREF(__pyx_t_1);
  } else
  #endif
  {
    __pyx_t_6 = PyTuple_New(2+__pyx_t_5); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 636, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    if (__pyx_t_2) {
      __Pyx_GIVEREF(__pyx_t_2); PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_2); __pyx_t_2 = NULL;
    }
    __Pyx_INCREF(__pyx_v_res);
    __Pyx_GIVEREF(__pyx_v_res);
    PyTuple_SET_ITEM(__pyx_t_6, 0+__pyx_t_5, __pyx_v_res);
    __Pyx_INCREF(__pyx_v_convert);
    __Pyx_GIVEREF(__pyx_v_convert);
    PyTuple_SET_ITEM(__pyx_t_6, 1+__pyx_t_5, __pyx_v_convert);
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_t_6, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 636, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  }
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = __Pyx_CallUnboundCMethod1(&__pyx_umethod_PyDict_Type_update, __pyx_v_response, __pyx_t_1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 636, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "pymarketcap/core.pyx":637
 * 
 *         response.update(processer.exchange(res, convert))
 *         return response             # <<<<<<<<<<<<<<
 * 
 *     cpdef exchanges(self, convert="USD"):
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_response);
  __pyx_r = __pyx_v_response;
  goto __pyx_L0;

  /* "pymarketcap/core.pyx":608
 *         return list(processer.recently(res, convert))
 * 
 *     cpdef exchange(self, unicode name, convert="USD"):             # <<<<<<<<<<<<<<
 *         """Obtain data from a exchange passed as argument. See ``exchanges_slugs``
 *         property for obtain all posibles values.
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_AddTraceback("pymarketcap.core.Pymarketcap.exchange", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_url);
  __Pyx_XDECREF(__pyx_v_res);
  __Pyx_XDECREF(__pyx_v_response);
  __Pyx_XDECREF(__pyx_v_convert);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_11pymarketcap_4core_11Pymarketcap_31exchange(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_11pymarketcap_4core_11Pymarketcap_30exchange[] = "Obtain data from a exchange passed as argument. See ``exchanges_slugs``\n        property for obtain all posibles values.\n\n        Args:\n            name (str): Exchange to retrieve data. Check ``exchange_slugs``\n                instance attribute for get all posible values passed\n                in this parameter.\n            convert (str, optional): Convert prices and 24h volumes in\n                return between USD and BTC. As default ``\"USD\"``.\n\n        Returns (dict):\n            Data from a exchange. Keys: ``\"name\"``, ``\"website\"``,\n            ``\"volume\"`` (total), ``\"social\"`` and ``\"markets\"``.\n        ";
static PyObject *__pyx_pw_11pymarketcap_4core_11Pymarketcap_31exchange(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_name = 0;
  PyObject *__pyx_v_convert = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("exchange (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_name,&__pyx_n_s_convert,0};
    PyObject* values[2] = {0,0};
    values[1] = ((PyObject *)__pyx_n_s_USD);
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_name)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_convert);
          if (value) { values[1] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "exchange") < 0)) __PYX_ERR(0, 608, __pyx_L3_error)
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_name = ((PyObject*)values[0]);
    __pyx_v_convert = values[1];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("exchange", 0, 1, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 608, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("pymarketcap.core.Pymarketcap.exchange", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_name), (&PyUnicode_Type), 1, "name", 1))) __PYX_ERR(0, 608, __pyx_L1_error)
  __pyx_r = __pyx_pf_11pymarketcap_4core_11Pymarketcap_30exchange(((struct __pyx_obj_11pymarketcap_4core_Pymarketcap *)__pyx_v_self), __pyx_v_name, __pyx_v_convert);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_11pymarketcap_4core_11Pymarketcap_30exchange(struct __pyx_obj_11pymarketcap_4core_Pymarketcap *__pyx_v_self, PyObject *__pyx_v_name, PyObject *__pyx_v_convert) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  struct __pyx_opt_args_11pymarketcap_4core_11Pymarketcap_exchange __pyx_t_2;
  __Pyx_RefNannySetupContext("exchange", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2.__pyx_n = 1;
  __pyx_t_2.convert = __pyx_v_convert;
  __pyx_t_1 = __pyx_vtabptr_11pymarketcap_4core_Pymarketcap->exchange(__pyx_v_self, __pyx_v_name, 1, &__pyx_t_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 608, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pymarketcap.core.Pymarketcap.exchange", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pymarketcap/core.pyx":639
 *         return response
 * 
 *     cpdef exchanges(self, convert="USD"):             # <<<<<<<<<<<<<<
 *         """Get all the exchanges in coinmarketcap ranked by volumes
 *         along with other metadata.
 */

static PyObject *__pyx_pw_11pymarketcap_4core_11Pymarketcap_33exchanges(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyObject *__pyx_f_11pymarketcap_4core_11Pymarketcap_exchanges(struct __pyx_obj_11pymarketcap_4core_Pymarketcap *__pyx_v_self, int __pyx_skip_dispatch, struct __pyx_opt_args_11pymarketcap_4core_11Pymarketcap_exchanges *__pyx_optional_args) {
  PyObject *__pyx_v_convert = ((PyObject *)__pyx_n_s_USD);
  PyObject *__pyx_v_url = 0;
  PyObject *__pyx_v_res = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  char *__pyx_t_6;
  int __pyx_t_7;
  __Pyx_RefNannySetupContext("exchanges", 0);
  if (__pyx_optional_args) {
    if (__pyx_optional_args->__pyx_n > 0) {
      __pyx_v_convert = __pyx_optional_args->convert;
    }
  }
  __Pyx_INCREF(__pyx_v_convert);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0)) {
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_exchanges); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 639, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)__pyx_pw_11pymarketcap_4core_11Pymarketcap_33exchanges)) {
      __Pyx_XDECREF(__pyx_r);
      __Pyx_INCREF(__pyx_t_1);
      __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
      if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
        __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
        if (likely(__pyx_t_4)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
          __Pyx_INCREF(__pyx_t_4);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_3, function);
        }
      }
      if (!__pyx_t_4) {
        __pyx_t_2 = __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_v_convert); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 639, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
      } else {
        #if CYTHON_FAST_PYCALL
        if (PyFunction_Check(__pyx_t_3)) {
          PyObject *__pyx_temp[2] = {__pyx_t_4, __pyx_v_convert};
          __pyx_t_2 = __Pyx_PyFunction_FastCall(__pyx_t_3, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 639, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
          __Pyx_GOTREF(__pyx_t_2);
        } else
        #endif
        #if CYTHON_FAST_PYCCALL
        if (__Pyx_PyFastCFunction_Check(__pyx_t_3)) {
          PyObject *__pyx_temp[2] = {__pyx_t_4, __pyx_v_convert};
          __pyx_t_2 = __Pyx_PyCFunction_FastCall(__pyx_t_3, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 639, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
          __Pyx_GOTREF(__pyx_t_2);
        } else
        #endif
        {
          __pyx_t_5 = PyTuple_New(1+1); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 639, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_5);
          __Pyx_GIVEREF(__pyx_t_4); PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_4); __pyx_t_4 = NULL;
          __Pyx_INCREF(__pyx_v_convert);
          __Pyx_GIVEREF(__pyx_v_convert);
          PyTuple_SET_ITEM(__pyx_t_5, 0+1, __pyx_v_convert);
          __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_t_5, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 639, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_2);
          __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
        }
      }
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __pyx_r = __pyx_t_2;
      __pyx_t_2 = 0;
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      goto __pyx_L0;
    }
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  }

  /* "pymarketcap/core.pyx":651
 *         """
 *         cdef bytes url
 *         url = b"https://coinmarketcap.com/exchanges/volume/24-hour/all/"             # <<<<<<<<<<<<<<
 *         res = self._get(url)[45000:]
 *         convert = convert.lower()
 */
  __Pyx_INCREF(__pyx_kp_b_https_coinmarketcap_com_exchange_2);
  __pyx_v_url = __pyx_kp_b_https_coinmarketcap_com_exchange_2;

  /* "pymarketcap/core.pyx":652
 *         cdef bytes url
 *         url = b"https://coinmarketcap.com/exchanges/volume/24-hour/all/"
 *         res = self._get(url)[45000:]             # <<<<<<<<<<<<<<
 *         convert = convert.lower()
 *         return processer.exchanges(res, convert)
 */
  __pyx_t_6 = __Pyx_PyBytes_AsWritableString(__pyx_v_url); if (unlikely((!__pyx_t_6) && PyErr_Occurred())) __PYX_ERR(0, 652, __pyx_L1_error)
  __pyx_t_1 = ((struct __pyx_vtabstruct_11pymarketcap_4core_Pymarketcap *)__pyx_v_self->__pyx_vtab)->_get(__pyx_v_self, __pyx_t_6); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 652, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_GetSlice(__pyx_t_1, 0xAFC8, 0, NULL, NULL, &__pyx_slice__14, 1, 0, 1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 652, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_res = __pyx_t_2;
  __pyx_t_2 = 0;

  /* "pymarketcap/core.pyx":653
 *         url = b"https://coinmarketcap.com/exchanges/volume/24-hour/all/"
 *         res = self._get(url)[45000:]
 *         convert = convert.lower()             # <<<<<<<<<<<<<<
 *         return processer.exchanges(res, convert)
 * 
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_convert, __pyx_n_s_lower); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 653, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_1))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_1);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_1);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_1, function);
    }
  }
  if (__pyx_t_3) {
    __pyx_t_2 = __Pyx_PyObject_CallOneArg(__pyx_t_1, __pyx_t_3); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 653, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  } else {
    __pyx_t_2 = __Pyx_PyObject_CallNoArg(__pyx_t_1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 653, __pyx_L1_error)
  }
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF_SET(__pyx_v_convert, __pyx_t_2);
  __pyx_t_2 = 0;

  /* "pymarketcap/core.pyx":654
 *         res = self._get(url)[45000:]
 *         convert = convert.lower()
 *         return processer.exchanges(res, convert)             # <<<<<<<<<<<<<<
 * 
 *     cpdef tokens(self, convert="USD"):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_GetModuleGlobalName(__pyx_n_s_processer); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 654, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_exchanges); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 654, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = NULL;
  __pyx_t_7 = 0;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_1 = PyMethod_GET_SELF(__pyx_t_3);
    if (likely(__pyx_t_1)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_1);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_3, function);
      __pyx_t_7 = 1;
    }
  }
  #if CYTHON_FAST_PYCALL
  if (PyFunction_Check(__pyx_t_3)) {
    PyObject *__pyx_temp[3] = {__pyx_t_1, __pyx_v_res, __pyx_v_convert};
    __pyx_t_2 = __Pyx_PyFunction_FastCall(__pyx_t_3, __pyx_temp+1-__pyx_t_7, 2+__pyx_t_7); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 654, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_GOTREF(__pyx_t_2);
  } else
  #endif
  #if CYTHON_FAST_PYCCALL
  if (__Pyx_PyFastCFunction_Check(__pyx_t_3)) {
    PyObject *__pyx_temp[3] = {__pyx_t_1, __pyx_v_res, __pyx_v_convert};
    __pyx_t_2 = __Pyx_PyCFunction_FastCall(__pyx_t_3, __pyx_temp+1-__pyx_t_7, 2+__pyx_t_7); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 654, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_GOTREF(__pyx_t_2);
  } else
  #endif
  {
    __pyx_t_5 = PyTuple_New(2+__pyx_t_7); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 654, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    if (__pyx_t_1) {
      __Pyx_GIVEREF(__pyx_t_1); PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_1); __pyx_t_1 = NULL;
    }
    __Pyx_INCREF(__pyx_v_res);
    __Pyx_GIVEREF(__pyx_v_res);
    PyTuple_SET_ITEM(__pyx_t_5, 0+__pyx_t_7, __pyx_v_res);
    __Pyx_INCREF(__pyx_v_convert);
    __Pyx_GIVEREF(__pyx_v_convert);
    PyTuple_SET_ITEM(__pyx_t_5, 1+__pyx_t_7, __pyx_v_convert);
    __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_t_5, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 654, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  }
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_r = __pyx_t_2;
  __pyx_t_2 = 0;
  goto __pyx_L0;

  /* "pymarketcap/core.pyx":639
 *         return response
 * 
 *     cpdef exchanges(self, convert="USD"):             # <<<<<<<<<<<<<<
 *         """Get all the exchanges in coinmarketcap ranked by volumes
 *         along with other metadata.
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("pymarketcap.core.Pymarketcap.exchanges", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_url);
  __Pyx_XDECREF(__pyx_v_res);
  __Pyx_XDECREF(__pyx_v_convert);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_11pymarketcap_4core_11Pymarketcap_33exchanges(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_11pymarketcap_4core_11Pymarketcap_32exchanges[] = "Get all the exchanges in coinmarketcap ranked by volumes\n        along with other metadata.\n\n        Args:\n            convert (str, optional): Convert volumes and prices\n                between USD and BTC. As default ``\"USD\"``.\n\n        Returns (list):\n            Exchanges with markets and other data included.\n        ";
static PyObject *__pyx_pw_11pymarketcap_4core_11Pymarketcap_33exchanges(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_convert = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("exchanges (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_convert,0};
    PyObject* values[1] = {0};
    values[0] = ((PyObject *)__pyx_n_s_USD);
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_convert);
          if (value) { values[0] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "exchanges") < 0)) __PYX_ERR(0, 639, __pyx_L3_error)
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_convert = values[0];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("exchanges", 0, 0, 1, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 639, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("pymarketcap.core.Pymarketcap.exchanges", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_11pymarketcap_4core_11Pymarketcap_32exchanges(((struct __pyx_obj_11pymarketcap_4core_Pymarketcap *)__pyx_v_self), __pyx_v_convert);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_11pymarketcap_4core_11Pymarketcap_32exchanges(struct __pyx_obj_11pymarketcap_4core_Pymarketcap *__pyx_v_self, PyObject *__pyx_v_convert) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  struct __pyx_opt_args_11pymarketcap_4core_11Pymarketcap_exchanges __pyx_t_2;
  __Pyx_RefNannySetupContext("exchanges", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2.__pyx_n = 1;
  __pyx_t_2.convert = __pyx_v_convert;
  __pyx_t_1 = __pyx_vtabptr_11pymarketcap_4core_Pymarketcap->exchanges(__pyx_v_self, 1, &__pyx_t_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 639, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pymarketcap.core.Pymarketcap.exchanges", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pymarketcap/core.pyx":656
 *         return processer.exchanges(res, convert)
 * 
 *     cpdef tokens(self, convert="USD"):             # <<<<<<<<<<<<<<
 *         """Get data from platforms tokens
 * 
 */

static PyObject *__pyx_pw_11pymarketcap_4core_11Pymarketcap_35tokens(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyObject *__pyx_f_11pymarketcap_4core_11Pymarketcap_tokens(struct __pyx_obj_11pymarketcap_4core_Pymarketcap *__pyx_v_self, int __pyx_skip_dispatch, struct __pyx_opt_args_11pymarketcap_4core_11Pymarketcap_tokens *__pyx_optional_args) {
  PyObject *__pyx_v_convert = ((PyObject *)__pyx_n_s_USD);
  PyObject *__pyx_v_url = NULL;
  PyObject *__pyx_v_res = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  char *__pyx_t_6;
  int __pyx_t_7;
  __Pyx_RefNannySetupContext("tokens", 0);
  if (__pyx_optional_args) {
    if (__pyx_optional_args->__pyx_n > 0) {
      __pyx_v_convert = __pyx_optional_args->convert;
    }
  }
  __Pyx_INCREF(__pyx_v_convert);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0)) {
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_tokens); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 656, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)__pyx_pw_11pymarketcap_4core_11Pymarketcap_35tokens)) {
      __Pyx_XDECREF(__pyx_r);
      __Pyx_INCREF(__pyx_t_1);
      __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
      if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
        __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
        if (likely(__pyx_t_4)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
          __Pyx_INCREF(__pyx_t_4);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_3, function);
        }
      }
      if (!__pyx_t_4) {
        __pyx_t_2 = __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_v_convert); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 656, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
      } else {
        #if CYTHON_FAST_PYCALL
        if (PyFunction_Check(__pyx_t_3)) {
          PyObject *__pyx_temp[2] = {__pyx_t_4, __pyx_v_convert};
          __pyx_t_2 = __Pyx_PyFunction_FastCall(__pyx_t_3, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 656, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
          __Pyx_GOTREF(__pyx_t_2);
        } else
        #endif
        #if CYTHON_FAST_PYCCALL
        if (__Pyx_PyFastCFunction_Check(__pyx_t_3)) {
          PyObject *__pyx_temp[2] = {__pyx_t_4, __pyx_v_convert};
          __pyx_t_2 = __Pyx_PyCFunction_FastCall(__pyx_t_3, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 656, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
          __Pyx_GOTREF(__pyx_t_2);
        } else
        #endif
        {
          __pyx_t_5 = PyTuple_New(1+1); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 656, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_5);
          __Pyx_GIVEREF(__pyx_t_4); PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_4); __pyx_t_4 = NULL;
          __Pyx_INCREF(__pyx_v_convert);
          __Pyx_GIVEREF(__pyx_v_convert);
          PyTuple_SET_ITEM(__pyx_t_5, 0+1, __pyx_v_convert);
          __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_t_5, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 656, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_2);
          __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
        }
      }
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __pyx_r = __pyx_t_2;
      __pyx_t_2 = 0;
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      goto __pyx_L0;
    }
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  }

  /* "pymarketcap/core.pyx":667
 *             Platforms tokens data.
 *         """
 *         url = b"https://coinmarketcap.com/tokens/views/all/"             # <<<<<<<<<<<<<<
 *         res = self._get(url)[40000:]
 *         convert = convert.lower()
 */
  __Pyx_INCREF(__pyx_kp_b_https_coinmarketcap_com_tokens_v);
  __pyx_v_url = __pyx_kp_b_https_coinmarketcap_com_tokens_v;

  /* "pymarketcap/core.pyx":668
 *         """
 *         url = b"https://coinmarketcap.com/tokens/views/all/"
 *         res = self._get(url)[40000:]             # <<<<<<<<<<<<<<
 *         convert = convert.lower()
 * 
 */
  __pyx_t_6 = __Pyx_PyBytes_AsWritableString(__pyx_v_url); if (unlikely((!__pyx_t_6) && PyErr_Occurred())) __PYX_ERR(0, 668, __pyx_L1_error)
  __pyx_t_1 = ((struct __pyx_vtabstruct_11pymarketcap_4core_Pymarketcap *)__pyx_v_self->__pyx_vtab)->_get(__pyx_v_self, __pyx_t_6); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 668, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_GetSlice(__pyx_t_1, 0x9C40, 0, NULL, NULL, &__pyx_slice__15, 1, 0, 1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 668, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_res = __pyx_t_2;
  __pyx_t_2 = 0;

  /* "pymarketcap/core.pyx":669
 *         url = b"https://coinmarketcap.com/tokens/views/all/"
 *         res = self._get(url)[40000:]
 *         convert = convert.lower()             # <<<<<<<<<<<<<<
 * 
 *         return processer.tokens(res, convert)
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_convert, __pyx_n_s_lower); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 669, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_1))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_1);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_1);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_1, function);
    }
  }
  if (__pyx_t_3) {
    __pyx_t_2 = __Pyx_PyObject_CallOneArg(__pyx_t_1, __pyx_t_3); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 669, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  } else {
    __pyx_t_2 = __Pyx_PyObject_CallNoArg(__pyx_t_1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 669, __pyx_L1_error)
  }
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF_SET(__pyx_v_convert, __pyx_t_2);
  __pyx_t_2 = 0;

  /* "pymarketcap/core.pyx":671
 *         convert = convert.lower()
 * 
 *         return processer.tokens(res, convert)             # <<<<<<<<<<<<<<
 * 
 *     # ====================================================================
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_GetModuleGlobalName(__pyx_n_s_processer); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 671, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_tokens); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 671, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = NULL;
  __pyx_t_7 = 0;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_1 = PyMethod_GET_SELF(__pyx_t_3);
    if (likely(__pyx_t_1)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_1);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_3, function);
      __pyx_t_7 = 1;
    }
  }
  #if CYTHON_FAST_PYCALL
  if (PyFunction_Check(__pyx_t_3)) {
    PyObject *__pyx_temp[3] = {__pyx_t_1, __pyx_v_res, __pyx_v_convert};
    __pyx_t_2 = __Pyx_PyFunction_FastCall(__pyx_t_3, __pyx_temp+1-__pyx_t_7, 2+__pyx_t_7); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 671, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_GOTREF(__pyx_t_2);
  } else
  #endif
  #if CYTHON_FAST_PYCCALL
  if (__Pyx_PyFastCFunction_Check(__pyx_t_3)) {
    PyObject *__pyx_temp[3] = {__pyx_t_1, __pyx_v_res, __pyx_v_convert};
    __pyx_t_2 = __Pyx_PyCFunction_FastCall(__pyx_t_3, __pyx_temp+1-__pyx_t_7, 2+__pyx_t_7); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 671, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_GOTREF(__pyx_t_2);
  } else
  #endif
  {
    __pyx_t_5 = PyTuple_New(2+__pyx_t_7); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 671, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    if (__pyx_t_1) {
      __Pyx_GIVEREF(__pyx_t_1); PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_1); __pyx_t_1 = NULL;
    }
    __Pyx_INCREF(__pyx_v_res);
    __Pyx_GIVEREF(__pyx_v_res);
    PyTuple_SET_ITEM(__pyx_t_5, 0+__pyx_t_7, __pyx_v_res);
    __Pyx_INCREF(__pyx_v_convert);
    __Pyx_GIVEREF(__pyx_v_convert);
    PyTuple_SET_ITEM(__pyx_t_5, 1+__pyx_t_7, __pyx_v_convert);
    __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_t_5, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 671, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  }
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_r = __pyx_t_2;
  __pyx_t_2 = 0;
  goto __pyx_L0;

  /* "pymarketcap/core.pyx":656
 *         return processer.exchanges(res, convert)
 * 
 *     cpdef tokens(self, convert="USD"):             # <<<<<<<<<<<<<<
 *         """Get data from platforms tokens
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("pymarketcap.core.Pymarketcap.tokens", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_url);
  __Pyx_XDECREF(__pyx_v_res);
  __Pyx_XDECREF(__pyx_v_convert);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_11pymarketcap_4core_11Pymarketcap_35tokens(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_11pymarketcap_4core_11Pymarketcap_34tokens[] = "Get data from platforms tokens\n\n        Args:\n            convert (str, optional): Convert ``\"market_cap\"``,\n                ``\"price\"`` and ``\"volume_24h\"`` values between\n                USD and BTC. As default ``\"USD\"``.\n\n        Returns (list):\n            Platforms tokens data.\n        ";
static PyObject *__pyx_pw_11pymarketcap_4core_11Pymarketcap_35tokens(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_convert = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("tokens (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_convert,0};
    PyObject* values[1] = {0};
    values[0] = ((PyObject *)__pyx_n_s_USD);
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_convert);
          if (value) { values[0] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "tokens") < 0)) __PYX_ERR(0, 656, __pyx_L3_error)
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_convert = values[0];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("tokens", 0, 0, 1, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 656, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("pymarketcap.core.Pymarketcap.tokens", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_11pymarketcap_4core_11Pymarketcap_34tokens(((struct __pyx_obj_11pymarketcap_4core_Pymarketcap *)__pyx_v_self), __pyx_v_convert);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_11pymarketcap_4core_11Pymarketcap_34tokens(struct __pyx_obj_11pymarketcap_4core_Pymarketcap *__pyx_v_self, PyObject *__pyx_v_convert) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  struct __pyx_opt_args_11pymarketcap_4core_11Pymarketcap_tokens __pyx_t_2;
  __Pyx_RefNannySetupContext("tokens", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2.__pyx_n = 1;
  __pyx_t_2.convert = __pyx_v_convert;
  __pyx_t_1 = __pyx_vtabptr_11pymarketcap_4core_Pymarketcap->tokens(__pyx_v_self, 1, &__pyx_t_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 656, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pymarketcap.core.Pymarketcap.tokens", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pymarketcap/core.pyx":677
 *                        #######   INTERNAL API   #######
 * 
 *     cpdef _currency(self, unicode name, start=None, end=None):             # <<<<<<<<<<<<<<
 *         """Get graphs data of a currency.
 * 
 */

static PyObject *__pyx_pw_11pymarketcap_4core_11Pymarketcap_37_currency(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyObject *__pyx_f_11pymarketcap_4core_11Pymarketcap__currency(struct __pyx_obj_11pymarketcap_4core_Pymarketcap *__pyx_v_self, PyObject *__pyx_v_name, int __pyx_skip_dispatch, struct __pyx_opt_args_11pymarketcap_4core_11Pymarketcap__currency *__pyx_optional_args) {
  PyObject *__pyx_v_start = ((PyObject *)Py_None);
  PyObject *__pyx_v_end = ((PyObject *)Py_None);
  PyObject *__pyx_v_url = NULL;
  PyObject *__pyx_v_res = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_t_7;
  char *__pyx_t_8;
  __Pyx_RefNannySetupContext("_currency", 0);
  if (__pyx_optional_args) {
    if (__pyx_optional_args->__pyx_n > 0) {
      __pyx_v_start = __pyx_optional_args->start;
      if (__pyx_optional_args->__pyx_n > 1) {
        __pyx_v_end = __pyx_optional_args->end;
      }
    }
  }
  __Pyx_INCREF(__pyx_v_name);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0)) {
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_currency_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 677, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)__pyx_pw_11pymarketcap_4core_11Pymarketcap_37_currency)) {
      __Pyx_XDECREF(__pyx_r);
      __Pyx_INCREF(__pyx_t_1);
      __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
      __pyx_t_5 = 0;
      if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
        __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
        if (likely(__pyx_t_4)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
          __Pyx_INCREF(__pyx_t_4);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_3, function);
          __pyx_t_5 = 1;
        }
      }
      #if CYTHON_FAST_PYCALL
      if (PyFunction_Check(__pyx_t_3)) {
        PyObject *__pyx_temp[4] = {__pyx_t_4, __pyx_v_name, __pyx_v_start, __pyx_v_end};
        __pyx_t_2 = __Pyx_PyFunction_FastCall(__pyx_t_3, __pyx_temp+1-__pyx_t_5, 3+__pyx_t_5); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 677, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        __Pyx_GOTREF(__pyx_t_2);
      } else
      #endif
      #if CYTHON_FAST_PYCCALL
      if (__Pyx_PyFastCFunction_Check(__pyx_t_3)) {
        PyObject *__pyx_temp[4] = {__pyx_t_4, __pyx_v_name, __pyx_v_start, __pyx_v_end};
        __pyx_t_2 = __Pyx_PyCFunction_FastCall(__pyx_t_3, __pyx_temp+1-__pyx_t_5, 3+__pyx_t_5); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 677, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        __Pyx_GOTREF(__pyx_t_2);
      } else
      #endif
      {
        __pyx_t_6 = PyTuple_New(3+__pyx_t_5); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 677, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_6);
        if (__pyx_t_4) {
          __Pyx_GIVEREF(__pyx_t_4); PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_4); __pyx_t_4 = NULL;
        }
        __Pyx_INCREF(__pyx_v_name);
        __Pyx_GIVEREF(__pyx_v_name);
        PyTuple_SET_ITEM(__pyx_t_6, 0+__pyx_t_5, __pyx_v_name);
        __Pyx_INCREF(__pyx_v_start);
        __Pyx_GIVEREF(__pyx_v_start);
        PyTuple_SET_ITEM(__pyx_t_6, 1+__pyx_t_5, __pyx_v_start);
        __Pyx_INCREF(__pyx_v_end);
        __Pyx_GIVEREF(__pyx_v_end);
        PyTuple_SET_ITEM(__pyx_t_6, 2+__pyx_t_5, __pyx_v_end);
        __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_t_6, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 677, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      }
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __pyx_r = __pyx_t_2;
      __pyx_t_2 = 0;
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      goto __pyx_L0;
    }
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  }

  /* "pymarketcap/core.pyx":694
 *             has two values [<datetime>, <value>]
 *         """
 *         if self._is_symbol(name):             # <<<<<<<<<<<<<<
 *             name = self.correspondences[name]
 * 
 */
  __pyx_t_1 = ((struct __pyx_vtabstruct_11pymarketcap_4core_Pymarketcap *)__pyx_v_self->__pyx_vtab)->_is_symbol(__pyx_v_self, __pyx_v_name, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 694, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_7 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_7 < 0)) __PYX_ERR(0, 694, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (__pyx_t_7) {

    /* "pymarketcap/core.pyx":695
 *         """
 *         if self._is_symbol(name):
 *             name = self.correspondences[name]             # <<<<<<<<<<<<<<
 * 
 *         url = b"https://graphs2.coinmarketcap.com/currencies/%s/" \
 */
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_correspondences); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 695, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_2 = __Pyx_PyObject_Dict_GetItem(__pyx_t_1, __pyx_v_name); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 695, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    if (!(likely(PyUnicode_CheckExact(__pyx_t_2))||((__pyx_t_2) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "unicode", Py_TYPE(__pyx_t_2)->tp_name), 0))) __PYX_ERR(0, 695, __pyx_L1_error)
    __Pyx_DECREF_SET(__pyx_v_name, ((PyObject*)__pyx_t_2));
    __pyx_t_2 = 0;

    /* "pymarketcap/core.pyx":694
 *             has two values [<datetime>, <value>]
 *         """
 *         if self._is_symbol(name):             # <<<<<<<<<<<<<<
 *             name = self.correspondences[name]
 * 
 */
  }

  /* "pymarketcap/core.pyx":698
 * 
 *         url = b"https://graphs2.coinmarketcap.com/currencies/%s/" \
 *                   % name.encode()             # <<<<<<<<<<<<<<
 *         res = loads(self._get(url))
 * 
 */
  if (unlikely(__pyx_v_name == Py_None)) {
    PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "encode");
    __PYX_ERR(0, 698, __pyx_L1_error)
  }
  __pyx_t_2 = PyUnicode_AsEncodedString(__pyx_v_name, NULL, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 698, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = PyNumber_Remainder(__pyx_kp_b_https_graphs2_coinmarketcap_com, __pyx_t_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 698, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v_url = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "pymarketcap/core.pyx":699
 *         url = b"https://graphs2.coinmarketcap.com/currencies/%s/" \
 *                   % name.encode()
 *         res = loads(self._get(url))             # <<<<<<<<<<<<<<
 * 
 *         return processer.graphs(res, start, end)
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_loads); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 699, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_8 = __Pyx_PyObject_AsWritableString(__pyx_v_url); if (unlikely((!__pyx_t_8) && PyErr_Occurred())) __PYX_ERR(0, 699, __pyx_L1_error)
  __pyx_t_3 = ((struct __pyx_vtabstruct_11pymarketcap_4core_Pymarketcap *)__pyx_v_self->__pyx_vtab)->_get(__pyx_v_self, __pyx_t_8); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 699, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_6 = NULL;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_6)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_6);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  if (!__pyx_t_6) {
    __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_3); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 699, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_GOTREF(__pyx_t_1);
  } else {
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(__pyx_t_2)) {
      PyObject *__pyx_temp[2] = {__pyx_t_6, __pyx_t_3};
      __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 699, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    } else
    #endif
    #if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
      PyObject *__pyx_temp[2] = {__pyx_t_6, __pyx_t_3};
      __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 699, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    } else
    #endif
    {
      __pyx_t_4 = PyTuple_New(1+1); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 699, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_GIVEREF(__pyx_t_6); PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_6); __pyx_t_6 = NULL;
      __Pyx_GIVEREF(__pyx_t_3);
      PyTuple_SET_ITEM(__pyx_t_4, 0+1, __pyx_t_3);
      __pyx_t_3 = 0;
      __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_4, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 699, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    }
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v_res = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "pymarketcap/core.pyx":701
 *         res = loads(self._get(url))
 * 
 *         return processer.graphs(res, start, end)             # <<<<<<<<<<<<<<
 * 
 *     cpdef _global_cap(self, bitcoin=True, start=None, end=None):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_processer); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 701, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_graphs); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 701, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = NULL;
  __pyx_t_5 = 0;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_4))) {
    __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_4);
    if (likely(__pyx_t_2)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
      __Pyx_INCREF(__pyx_t_2);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_4, function);
      __pyx_t_5 = 1;
    }
  }
  #if CYTHON_FAST_PYCALL
  if (PyFunction_Check(__pyx_t_4)) {
    PyObject *__pyx_temp[4] = {__pyx_t_2, __pyx_v_res, __pyx_v_start, __pyx_v_end};
    __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_4, __pyx_temp+1-__pyx_t_5, 3+__pyx_t_5); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 701, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_GOTREF(__pyx_t_1);
  } else
  #endif
  #if CYTHON_FAST_PYCCALL
  if (__Pyx_PyFastCFunction_Check(__pyx_t_4)) {
    PyObject *__pyx_temp[4] = {__pyx_t_2, __pyx_v_res, __pyx_v_start, __pyx_v_end};
    __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_4, __pyx_temp+1-__pyx_t_5, 3+__pyx_t_5); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 701, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_GOTREF(__pyx_t_1);
  } else
  #endif
  {
    __pyx_t_3 = PyTuple_New(3+__pyx_t_5); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 701, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    if (__pyx_t_2) {
      __Pyx_GIVEREF(__pyx_t_2); PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_2); __pyx_t_2 = NULL;
    }
    __Pyx_INCREF(__pyx_v_res);
    __Pyx_GIVEREF(__pyx_v_res);
    PyTuple_SET_ITEM(__pyx_t_3, 0+__pyx_t_5, __pyx_v_res);
    __Pyx_INCREF(__pyx_v_start);
    __Pyx_GIVEREF(__pyx_v_start);
    PyTuple_SET_ITEM(__pyx_t_3, 1+__pyx_t_5, __pyx_v_start);
    __Pyx_INCREF(__pyx_v_end);
    __Pyx_GIVEREF(__pyx_v_end);
    PyTuple_SET_ITEM(__pyx_t_3, 2+__pyx_t_5, __pyx_v_end);
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_4, __pyx_t_3, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 701, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  }
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "pymarketcap/core.pyx":677
 *                        #######   INTERNAL API   #######
 * 
 *     cpdef _currency(self, unicode name, start=None, end=None):             # <<<<<<<<<<<<<<
 *         """Get graphs data of a currency.
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_AddTraceback("pymarketcap.core.Pymarketcap._currency", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_url);
  __Pyx_XDECREF(__pyx_v_res);
  __Pyx_XDECREF(__pyx_v_name);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_11pymarketcap_4core_11Pymarketcap_37_currency(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_11pymarketcap_4core_11Pymarketcap_36_currency[] = "Get graphs data of a currency.\n\n        Args:\n            currency (str): Currency to retrieve graphs data.\n            start (datetime, optional): Time to start retrieving\n                graphs data in datetime. As default ``None``.\n            end (datetime, optional): Time to end retrieving\n                graphs data in datetime. As default ``None``.\n\n        Returns (dict):\n            Dict info with next keys: ``\"market_cap_by_available_supply\"``,\n            ``\"price_btc\"``, ``\"price_usd\"``, ``\"volume_usd\":``\n            and ``\"price_platform\"``.\n            For each value, a list of lists where each one\n            has two values [<datetime>, <value>]\n        ";
static PyObject *__pyx_pw_11pymarketcap_4core_11Pymarketcap_37_currency(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_name = 0;
  PyObject *__pyx_v_start = 0;
  PyObject *__pyx_v_end = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_currency (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_name,&__pyx_n_s_start,&__pyx_n_s_end,0};
    PyObject* values[3] = {0,0,0};
    values[1] = ((PyObject *)Py_None);
    values[2] = ((PyObject *)Py_None);
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_name)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_start);
          if (value) { values[1] = value; kw_args--; }
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_end);
          if (value) { values[2] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "_currency") < 0)) __PYX_ERR(0, 677, __pyx_L3_error)
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_name = ((PyObject*)values[0]);
    __pyx_v_start = values[1];
    __pyx_v_end = values[2];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("_currency", 0, 1, 3, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 677, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("pymarketcap.core.Pymarketcap._currency", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_name), (&PyUnicode_Type), 1, "name", 1))) __PYX_ERR(0, 677, __pyx_L1_error)
  __pyx_r = __pyx_pf_11pymarketcap_4core_11Pymarketcap_36_currency(((struct __pyx_obj_11pymarketcap_4core_Pymarketcap *)__pyx_v_self), __pyx_v_name, __pyx_v_start, __pyx_v_end);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_11pymarketcap_4core_11Pymarketcap_36_currency(struct __pyx_obj_11pymarketcap_4core_Pymarketcap *__pyx_v_self, PyObject *__pyx_v_name, PyObject *__pyx_v_start, PyObject *__pyx_v_end) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  struct __pyx_opt_args_11pymarketcap_4core_11Pymarketcap__currency __pyx_t_2;
  __Pyx_RefNannySetupContext("_currency", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2.__pyx_n = 2;
  __pyx_t_2.start = __pyx_v_start;
  __pyx_t_2.end = __pyx_v_end;
  __pyx_t_1 = __pyx_vtabptr_11pymarketcap_4core_Pymarketcap->_currency(__pyx_v_self, __pyx_v_name, 1, &__pyx_t_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 677, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pymarketcap.core.Pymarketcap._currency", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pymarketcap/core.pyx":703
 *         return processer.graphs(res, start, end)
 * 
 *     cpdef _global_cap(self, bitcoin=True, start=None, end=None):             # <<<<<<<<<<<<<<
 *         """Get global market capitalization graphs, including
 *         or excluding Bitcoin.
 */

static PyObject *__pyx_pw_11pymarketcap_4core_11Pymarketcap_39_global_cap(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyObject *__pyx_f_11pymarketcap_4core_11Pymarketcap__global_cap(struct __pyx_obj_11pymarketcap_4core_Pymarketcap *__pyx_v_self, int __pyx_skip_dispatch, struct __pyx_opt_args_11pymarketcap_4core_11Pymarketcap__global_cap *__pyx_optional_args) {
  PyObject *__pyx_v_bitcoin = ((PyObject *)Py_True);
  PyObject *__pyx_v_start = ((PyObject *)Py_None);
  PyObject *__pyx_v_end = ((PyObject *)Py_None);
  PyObject *__pyx_v_url = NULL;
  PyObject *__pyx_v_res = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_t_7;
  char *__pyx_t_8;
  __Pyx_RefNannySetupContext("_global_cap", 0);
  if (__pyx_optional_args) {
    if (__pyx_optional_args->__pyx_n > 0) {
      __pyx_v_bitcoin = __pyx_optional_args->bitcoin;
      if (__pyx_optional_args->__pyx_n > 1) {
        __pyx_v_start = __pyx_optional_args->start;
        if (__pyx_optional_args->__pyx_n > 2) {
          __pyx_v_end = __pyx_optional_args->end;
        }
      }
    }
  }
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0)) {
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_global_cap_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 703, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)__pyx_pw_11pymarketcap_4core_11Pymarketcap_39_global_cap)) {
      __Pyx_XDECREF(__pyx_r);
      __Pyx_INCREF(__pyx_t_1);
      __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
      __pyx_t_5 = 0;
      if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
        __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
        if (likely(__pyx_t_4)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
          __Pyx_INCREF(__pyx_t_4);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_3, function);
          __pyx_t_5 = 1;
        }
      }
      #if CYTHON_FAST_PYCALL
      if (PyFunction_Check(__pyx_t_3)) {
        PyObject *__pyx_temp[4] = {__pyx_t_4, __pyx_v_bitcoin, __pyx_v_start, __pyx_v_end};
        __pyx_t_2 = __Pyx_PyFunction_FastCall(__pyx_t_3, __pyx_temp+1-__pyx_t_5, 3+__pyx_t_5); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 703, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        __Pyx_GOTREF(__pyx_t_2);
      } else
      #endif
      #if CYTHON_FAST_PYCCALL
      if (__Pyx_PyFastCFunction_Check(__pyx_t_3)) {
        PyObject *__pyx_temp[4] = {__pyx_t_4, __pyx_v_bitcoin, __pyx_v_start, __pyx_v_end};
        __pyx_t_2 = __Pyx_PyCFunction_FastCall(__pyx_t_3, __pyx_temp+1-__pyx_t_5, 3+__pyx_t_5); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 703, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        __Pyx_GOTREF(__pyx_t_2);
      } else
      #endif
      {
        __pyx_t_6 = PyTuple_New(3+__pyx_t_5); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 703, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_6);
        if (__pyx_t_4) {
          __Pyx_GIVEREF(__pyx_t_4); PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_4); __pyx_t_4 = NULL;
        }
        __Pyx_INCREF(__pyx_v_bitcoin);
        __Pyx_GIVEREF(__pyx_v_bitcoin);
        PyTuple_SET_ITEM(__pyx_t_6, 0+__pyx_t_5, __pyx_v_bitcoin);
        __Pyx_INCREF(__pyx_v_start);
        __Pyx_GIVEREF(__pyx_v_start);
        PyTuple_SET_ITEM(__pyx_t_6, 1+__pyx_t_5, __pyx_v_start);
        __Pyx_INCREF(__pyx_v_end);
        __Pyx_GIVEREF(__pyx_v_end);
        PyTuple_SET_ITEM(__pyx_t_6, 2+__pyx_t_5, __pyx_v_end);
        __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_t_6, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 703, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      }
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __pyx_r = __pyx_t_2;
      __pyx_t_2 = 0;
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      goto __pyx_L0;
    }
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  }

  /* "pymarketcap/core.pyx":721
 *             ``"market_cap_by_available_supply"``.
 *         """
 *         if bitcoin:             # <<<<<<<<<<<<<<
 *             url = b"https://graphs2.coinmarketcap.com/global/marketcap-total/"
 *         else:
 */
  __pyx_t_7 = __Pyx_PyObject_IsTrue(__pyx_v_bitcoin); if (unlikely(__pyx_t_7 < 0)) __PYX_ERR(0, 721, __pyx_L1_error)
  if (__pyx_t_7) {

    /* "pymarketcap/core.pyx":722
 *         """
 *         if bitcoin:
 *             url = b"https://graphs2.coinmarketcap.com/global/marketcap-total/"             # <<<<<<<<<<<<<<
 *         else:
 *             url = b"https://graphs2.coinmarketcap.com/global/marketcap-altcoin/"
 */
    __Pyx_INCREF(__pyx_kp_b_https_graphs2_coinmarketcap_com_2);
    __pyx_v_url = __pyx_kp_b_https_graphs2_coinmarketcap_com_2;

    /* "pymarketcap/core.pyx":721
 *             ``"market_cap_by_available_supply"``.
 *         """
 *         if bitcoin:             # <<<<<<<<<<<<<<
 *             url = b"https://graphs2.coinmarketcap.com/global/marketcap-total/"
 *         else:
 */
    goto __pyx_L3;
  }

  /* "pymarketcap/core.pyx":724
 *             url = b"https://graphs2.coinmarketcap.com/global/marketcap-total/"
 *         else:
 *             url = b"https://graphs2.coinmarketcap.com/global/marketcap-altcoin/"             # <<<<<<<<<<<<<<
 * 
 *         res = loads(self._get(url))
 */
  /*else*/ {
    __Pyx_INCREF(__pyx_kp_b_https_graphs2_coinmarketcap_com_3);
    __pyx_v_url = __pyx_kp_b_https_graphs2_coinmarketcap_com_3;
  }
  __pyx_L3:;

  /* "pymarketcap/core.pyx":726
 *             url = b"https://graphs2.coinmarketcap.com/global/marketcap-altcoin/"
 * 
 *         res = loads(self._get(url))             # <<<<<<<<<<<<<<
 * 
 *         return processer.graphs(res, start, end)
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_loads); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 726, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_8 = __Pyx_PyBytes_AsWritableString(__pyx_v_url); if (unlikely((!__pyx_t_8) && PyErr_Occurred())) __PYX_ERR(0, 726, __pyx_L1_error)
  __pyx_t_3 = ((struct __pyx_vtabstruct_11pymarketcap_4core_Pymarketcap *)__pyx_v_self->__pyx_vtab)->_get(__pyx_v_self, __pyx_t_8); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 726, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_6 = NULL;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_6)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_6);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  if (!__pyx_t_6) {
    __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_3); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 726, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_GOTREF(__pyx_t_1);
  } else {
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(__pyx_t_2)) {
      PyObject *__pyx_temp[2] = {__pyx_t_6, __pyx_t_3};
      __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 726, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    } else
    #endif
    #if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
      PyObject *__pyx_temp[2] = {__pyx_t_6, __pyx_t_3};
      __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 726, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    } else
    #endif
    {
      __pyx_t_4 = PyTuple_New(1+1); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 726, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_GIVEREF(__pyx_t_6); PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_6); __pyx_t_6 = NULL;
      __Pyx_GIVEREF(__pyx_t_3);
      PyTuple_SET_ITEM(__pyx_t_4, 0+1, __pyx_t_3);
      __pyx_t_3 = 0;
      __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_4, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 726, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    }
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v_res = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "pymarketcap/core.pyx":728
 *         res = loads(self._get(url))
 * 
 *         return processer.graphs(res, start, end)             # <<<<<<<<<<<<<<
 * 
 *     cpdef _dominance(self, start=None, end=None):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_processer); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 728, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_graphs); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 728, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = NULL;
  __pyx_t_5 = 0;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_4))) {
    __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_4);
    if (likely(__pyx_t_2)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
      __Pyx_INCREF(__pyx_t_2);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_4, function);
      __pyx_t_5 = 1;
    }
  }
  #if CYTHON_FAST_PYCALL
  if (PyFunction_Check(__pyx_t_4)) {
    PyObject *__pyx_temp[4] = {__pyx_t_2, __pyx_v_res, __pyx_v_start, __pyx_v_end};
    __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_4, __pyx_temp+1-__pyx_t_5, 3+__pyx_t_5); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 728, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_GOTREF(__pyx_t_1);
  } else
  #endif
  #if CYTHON_FAST_PYCCALL
  if (__Pyx_PyFastCFunction_Check(__pyx_t_4)) {
    PyObject *__pyx_temp[4] = {__pyx_t_2, __pyx_v_res, __pyx_v_start, __pyx_v_end};
    __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_4, __pyx_temp+1-__pyx_t_5, 3+__pyx_t_5); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 728, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_GOTREF(__pyx_t_1);
  } else
  #endif
  {
    __pyx_t_3 = PyTuple_New(3+__pyx_t_5); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 728, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    if (__pyx_t_2) {
      __Pyx_GIVEREF(__pyx_t_2); PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_2); __pyx_t_2 = NULL;
    }
    __Pyx_INCREF(__pyx_v_res);
    __Pyx_GIVEREF(__pyx_v_res);
    PyTuple_SET_ITEM(__pyx_t_3, 0+__pyx_t_5, __pyx_v_res);
    __Pyx_INCREF(__pyx_v_start);
    __Pyx_GIVEREF(__pyx_v_start);
    PyTuple_SET_ITEM(__pyx_t_3, 1+__pyx_t_5, __pyx_v_start);
    __Pyx_INCREF(__pyx_v_end);
    __Pyx_GIVEREF(__pyx_v_end);
    PyTuple_SET_ITEM(__pyx_t_3, 2+__pyx_t_5, __pyx_v_end);
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_4, __pyx_t_3, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 728, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  }
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "pymarketcap/core.pyx":703
 *         return processer.graphs(res, start, end)
 * 
 *     cpdef _global_cap(self, bitcoin=True, start=None, end=None):             # <<<<<<<<<<<<<<
 *         """Get global market capitalization graphs, including
 *         or excluding Bitcoin.
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_AddTraceback("pymarketcap.core.Pymarketcap._global_cap", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_url);
  __Pyx_XDECREF(__pyx_v_res);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_11pymarketcap_4core_11Pymarketcap_39_global_cap(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_11pymarketcap_4core_11Pymarketcap_38_global_cap[] = "Get global market capitalization graphs, including\n        or excluding Bitcoin.\n\n        Args:\n            bitcoin (bool, optional): Indicates if Bitcoin will\n                be includedin global market capitalization graph.\n                As default ``True``.\n            start (int, optional): Time to start retrieving\n                graphs data in datetime. As default ``None``.\n            end (optional, datetime): Time to start retrieving\n                graphs data in datetime. As default ``None``.\n\n        Returns (dict):\n            Whose values are lists of lists with timestamp and values,\n            a data structure with the keys: ``\"volume_usd\"`` and\n            ``\"market_cap_by_available_supply\"``.\n        ";
static PyObject *__pyx_pw_11pymarketcap_4core_11Pymarketcap_39_global_cap(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_bitcoin = 0;
  PyObject *__pyx_v_start = 0;
  PyObject *__pyx_v_end = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_global_cap (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_bitcoin,&__pyx_n_s_start,&__pyx_n_s_end,0};
    PyObject* values[3] = {0,0,0};
    values[0] = ((PyObject *)Py_True);
    values[1] = ((PyObject *)Py_None);
    values[2] = ((PyObject *)Py_None);
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_bitcoin);
          if (value) { values[0] = value; kw_args--; }
        }
        CYTHON_FALLTHROUGH;
        case  1:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_start);
          if (value) { values[1] = value; kw_args--; }
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_end);
          if (value) { values[2] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "_global_cap") < 0)) __PYX_ERR(0, 703, __pyx_L3_error)
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_bitcoin = values[0];
    __pyx_v_start = values[1];
    __pyx_v_end = values[2];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("_global_cap", 0, 0, 3, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 703, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("pymarketcap.core.Pymarketcap._global_cap", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_11pymarketcap_4core_11Pymarketcap_38_global_cap(((struct __pyx_obj_11pymarketcap_4core_Pymarketcap *)__pyx_v_self), __pyx_v_bitcoin, __pyx_v_start, __pyx_v_end);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_11pymarketcap_4core_11Pymarketcap_38_global_cap(struct __pyx_obj_11pymarketcap_4core_Pymarketcap *__pyx_v_self, PyObject *__pyx_v_bitcoin, PyObject *__pyx_v_start, PyObject *__pyx_v_end) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  struct __pyx_opt_args_11pymarketcap_4core_11Pymarketcap__global_cap __pyx_t_2;
  __Pyx_RefNannySetupContext("_global_cap", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2.__pyx_n = 3;
  __pyx_t_2.bitcoin = __pyx_v_bitcoin;
  __pyx_t_2.start = __pyx_v_start;
  __pyx_t_2.end = __pyx_v_end;
  __pyx_t_1 = __pyx_vtabptr_11pymarketcap_4core_Pymarketcap->_global_cap(__pyx_v_self, 1, &__pyx_t_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 703, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pymarketcap.core.Pymarketcap._global_cap", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pymarketcap/core.pyx":730
 *         return processer.graphs(res, start, end)
 * 
 *     cpdef _dominance(self, start=None, end=None):             # <<<<<<<<<<<<<<
 *         """Get currencies dominance percentage graph
 * 
 */

static PyObject *__pyx_pw_11pymarketcap_4core_11Pymarketcap_41_dominance(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyObject *__pyx_f_11pymarketcap_4core_11Pymarketcap__dominance(struct __pyx_obj_11pymarketcap_4core_Pymarketcap *__pyx_v_self, int __pyx_skip_dispatch, struct __pyx_opt_args_11pymarketcap_4core_11Pymarketcap__dominance *__pyx_optional_args) {
  PyObject *__pyx_v_start = ((PyObject *)Py_None);
  PyObject *__pyx_v_end = ((PyObject *)Py_None);
  PyObject *__pyx_v_url = NULL;
  PyObject *__pyx_v_res = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  char *__pyx_t_7;
  __Pyx_RefNannySetupContext("_dominance", 0);
  if (__pyx_optional_args) {
    if (__pyx_optional_args->__pyx_n > 0) {
      __pyx_v_start = __pyx_optional_args->start;
      if (__pyx_optional_args->__pyx_n > 1) {
        __pyx_v_end = __pyx_optional_args->end;
      }
    }
  }
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0)) {
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_dominance_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 730, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)__pyx_pw_11pymarketcap_4core_11Pymarketcap_41_dominance)) {
      __Pyx_XDECREF(__pyx_r);
      __Pyx_INCREF(__pyx_t_1);
      __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
      __pyx_t_5 = 0;
      if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
        __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
        if (likely(__pyx_t_4)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
          __Pyx_INCREF(__pyx_t_4);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_3, function);
          __pyx_t_5 = 1;
        }
      }
      #if CYTHON_FAST_PYCALL
      if (PyFunction_Check(__pyx_t_3)) {
        PyObject *__pyx_temp[3] = {__pyx_t_4, __pyx_v_start, __pyx_v_end};
        __pyx_t_2 = __Pyx_PyFunction_FastCall(__pyx_t_3, __pyx_temp+1-__pyx_t_5, 2+__pyx_t_5); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 730, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        __Pyx_GOTREF(__pyx_t_2);
      } else
      #endif
      #if CYTHON_FAST_PYCCALL
      if (__Pyx_PyFastCFunction_Check(__pyx_t_3)) {
        PyObject *__pyx_temp[3] = {__pyx_t_4, __pyx_v_start, __pyx_v_end};
        __pyx_t_2 = __Pyx_PyCFunction_FastCall(__pyx_t_3, __pyx_temp+1-__pyx_t_5, 2+__pyx_t_5); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 730, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        __Pyx_GOTREF(__pyx_t_2);
      } else
      #endif
      {
        __pyx_t_6 = PyTuple_New(2+__pyx_t_5); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 730, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_6);
        if (__pyx_t_4) {
          __Pyx_GIVEREF(__pyx_t_4); PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_4); __pyx_t_4 = NULL;
        }
        __Pyx_INCREF(__pyx_v_start);
        __Pyx_GIVEREF(__pyx_v_start);
        PyTuple_SET_ITEM(__pyx_t_6, 0+__pyx_t_5, __pyx_v_start);
        __Pyx_INCREF(__pyx_v_end);
        __Pyx_GIVEREF(__pyx_v_end);
        PyTuple_SET_ITEM(__pyx_t_6, 1+__pyx_t_5, __pyx_v_end);
        __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_t_6, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 730, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      }
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __pyx_r = __pyx_t_2;
      __pyx_t_2 = 0;
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      goto __pyx_L0;
    }
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  }

  /* "pymarketcap/core.pyx":742
 *             Altcoins and dominance percentage values with timestamps.
 *         """
 *         url = b"https://graphs2.coinmarketcap.com/global/dominance/"             # <<<<<<<<<<<<<<
 * 
 *         res = loads(self._get(url))
 */
  __Pyx_INCREF(__pyx_kp_b_https_graphs2_coinmarketcap_com_4);
  __pyx_v_url = __pyx_kp_b_https_graphs2_coinmarketcap_com_4;

  /* "pymarketcap/core.pyx":744
 *         url = b"https://graphs2.coinmarketcap.com/global/dominance/"
 * 
 *         res = loads(self._get(url))             # <<<<<<<<<<<<<<
 * 
 *         return processer.graphs(res, start, end)
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_loads); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 744, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_7 = __Pyx_PyBytes_AsWritableString(__pyx_v_url); if (unlikely((!__pyx_t_7) && PyErr_Occurred())) __PYX_ERR(0, 744, __pyx_L1_error)
  __pyx_t_3 = ((struct __pyx_vtabstruct_11pymarketcap_4core_Pymarketcap *)__pyx_v_self->__pyx_vtab)->_get(__pyx_v_self, __pyx_t_7); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 744, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_6 = NULL;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_6)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_6);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  if (!__pyx_t_6) {
    __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_3); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 744, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_GOTREF(__pyx_t_1);
  } else {
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(__pyx_t_2)) {
      PyObject *__pyx_temp[2] = {__pyx_t_6, __pyx_t_3};
      __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 744, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    } else
    #endif
    #if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
      PyObject *__pyx_temp[2] = {__pyx_t_6, __pyx_t_3};
      __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 744, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    } else
    #endif
    {
      __pyx_t_4 = PyTuple_New(1+1); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 744, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_GIVEREF(__pyx_t_6); PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_6); __pyx_t_6 = NULL;
      __Pyx_GIVEREF(__pyx_t_3);
      PyTuple_SET_ITEM(__pyx_t_4, 0+1, __pyx_t_3);
      __pyx_t_3 = 0;
      __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_4, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 744, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    }
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v_res = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "pymarketcap/core.pyx":746
 *         res = loads(self._get(url))
 * 
 *         return processer.graphs(res, start, end)             # <<<<<<<<<<<<<<
 * 
 *     cpdef download_logo(self, unicode name, size=64, filename=None):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_processer); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 746, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_graphs); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 746, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = NULL;
  __pyx_t_5 = 0;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_4))) {
    __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_4);
    if (likely(__pyx_t_2)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
      __Pyx_INCREF(__pyx_t_2);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_4, function);
      __pyx_t_5 = 1;
    }
  }
  #if CYTHON_FAST_PYCALL
  if (PyFunction_Check(__pyx_t_4)) {
    PyObject *__pyx_temp[4] = {__pyx_t_2, __pyx_v_res, __pyx_v_start, __pyx_v_end};
    __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_4, __pyx_temp+1-__pyx_t_5, 3+__pyx_t_5); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 746, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_GOTREF(__pyx_t_1);
  } else
  #endif
  #if CYTHON_FAST_PYCCALL
  if (__Pyx_PyFastCFunction_Check(__pyx_t_4)) {
    PyObject *__pyx_temp[4] = {__pyx_t_2, __pyx_v_res, __pyx_v_start, __pyx_v_end};
    __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_4, __pyx_temp+1-__pyx_t_5, 3+__pyx_t_5); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 746, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_GOTREF(__pyx_t_1);
  } else
  #endif
  {
    __pyx_t_3 = PyTuple_New(3+__pyx_t_5); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 746, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    if (__pyx_t_2) {
      __Pyx_GIVEREF(__pyx_t_2); PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_2); __pyx_t_2 = NULL;
    }
    __Pyx_INCREF(__pyx_v_res);
    __Pyx_GIVEREF(__pyx_v_res);
    PyTuple_SET_ITEM(__pyx_t_3, 0+__pyx_t_5, __pyx_v_res);
    __Pyx_INCREF(__pyx_v_start);
    __Pyx_GIVEREF(__pyx_v_start);
    PyTuple_SET_ITEM(__pyx_t_3, 1+__pyx_t_5, __pyx_v_start);
    __Pyx_INCREF(__pyx_v_end);
    __Pyx_GIVEREF(__pyx_v_end);
    PyTuple_SET_ITEM(__pyx_t_3, 2+__pyx_t_5, __pyx_v_end);
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_4, __pyx_t_3, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 746, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  }
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "pymarketcap/core.pyx":730
 *         return processer.graphs(res, start, end)
 * 
 *     cpdef _dominance(self, start=None, end=None):             # <<<<<<<<<<<<<<
 *         """Get currencies dominance percentage graph
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_AddTraceback("pymarketcap.core.Pymarketcap._dominance", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_url);
  __Pyx_XDECREF(__pyx_v_res);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_11pymarketcap_4core_11Pymarketcap_41_dominance(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_11pymarketcap_4core_11Pymarketcap_40_dominance[] = "Get currencies dominance percentage graph\n\n        Args:\n            start (int, optional): Time to start retrieving\n                graphs data in datetime. As default ``None``.\n            end (optional, datetime): Time to start retrieving\n                graphs data in datetime. As default ``None``.\n\n        Returns (dict):\n            Altcoins and dominance percentage values with timestamps.\n        ";
static PyObject *__pyx_pw_11pymarketcap_4core_11Pymarketcap_41_dominance(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_start = 0;
  PyObject *__pyx_v_end = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_dominance (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_start,&__pyx_n_s_end,0};
    PyObject* values[2] = {0,0};
    values[0] = ((PyObject *)Py_None);
    values[1] = ((PyObject *)Py_None);
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_start);
          if (value) { values[0] = value; kw_args--; }
        }
        CYTHON_FALLTHROUGH;
        case  1:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_end);
          if (value) { values[1] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "_dominance") < 0)) __PYX_ERR(0, 730, __pyx_L3_error)
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_start = values[0];
    __pyx_v_end = values[1];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("_dominance", 0, 0, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 730, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("pymarketcap.core.Pymarketcap._dominance", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_11pymarketcap_4core_11Pymarketcap_40_dominance(((struct __pyx_obj_11pymarketcap_4core_Pymarketcap *)__pyx_v_self), __pyx_v_start, __pyx_v_end);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_11pymarketcap_4core_11Pymarketcap_40_dominance(struct __pyx_obj_11pymarketcap_4core_Pymarketcap *__pyx_v_self, PyObject *__pyx_v_start, PyObject *__pyx_v_end) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  struct __pyx_opt_args_11pymarketcap_4core_11Pymarketcap__dominance __pyx_t_2;
  __Pyx_RefNannySetupContext("_dominance", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2.__pyx_n = 2;
  __pyx_t_2.start = __pyx_v_start;
  __pyx_t_2.end = __pyx_v_end;
  __pyx_t_1 = __pyx_vtabptr_11pymarketcap_4core_Pymarketcap->_dominance(__pyx_v_self, 1, &__pyx_t_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 730, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pymarketcap.core.Pymarketcap._dominance", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pymarketcap/core.pyx":748
 *         return processer.graphs(res, start, end)
 * 
 *     cpdef download_logo(self, unicode name, size=64, filename=None):             # <<<<<<<<<<<<<<
 *         """Download a currency image logo providing their size.
 * 
 */

static PyObject *__pyx_pw_11pymarketcap_4core_11Pymarketcap_43download_logo(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyObject *__pyx_f_11pymarketcap_4core_11Pymarketcap_download_logo(struct __pyx_obj_11pymarketcap_4core_Pymarketcap *__pyx_v_self, PyObject *__pyx_v_name, int __pyx_skip_dispatch, struct __pyx_opt_args_11pymarketcap_4core_11Pymarketcap_download_logo *__pyx_optional_args) {
  PyObject *__pyx_v_size = ((PyObject *)__pyx_int_64);
  PyObject *__pyx_v_filename = ((PyObject *)Py_None);
  PyObject *__pyx_v__name = NULL;
  PyObject *__pyx_v_url_schema = NULL;
  PyObject *__pyx_v_url = NULL;
  CYTHON_UNUSED PyObject *__pyx_v_res = NULL;
  PyObject *__pyx_v_e = NULL;
  PyObject *__pyx_v_valid_sizes = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_t_7;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  PyObject *__pyx_t_10 = NULL;
  PyObject *__pyx_t_11 = NULL;
  int __pyx_t_12;
  __Pyx_RefNannySetupContext("download_logo", 0);
  if (__pyx_optional_args) {
    if (__pyx_optional_args->__pyx_n > 0) {
      __pyx_v_size = __pyx_optional_args->size;
      if (__pyx_optional_args->__pyx_n > 1) {
        __pyx_v_filename = __pyx_optional_args->filename;
      }
    }
  }
  __Pyx_INCREF(__pyx_v_filename);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0)) {
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_download_logo); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 748, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)__pyx_pw_11pymarketcap_4core_11Pymarketcap_43download_logo)) {
      __Pyx_XDECREF(__pyx_r);
      __Pyx_INCREF(__pyx_t_1);
      __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
      __pyx_t_5 = 0;
      if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
        __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
        if (likely(__pyx_t_4)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
          __Pyx_INCREF(__pyx_t_4);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_3, function);
          __pyx_t_5 = 1;
        }
      }
      #if CYTHON_FAST_PYCALL
      if (PyFunction_Check(__pyx_t_3)) {
        PyObject *__pyx_temp[4] = {__pyx_t_4, __pyx_v_name, __pyx_v_size, __pyx_v_filename};
        __pyx_t_2 = __Pyx_PyFunction_FastCall(__pyx_t_3, __pyx_temp+1-__pyx_t_5, 3+__pyx_t_5); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 748, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        __Pyx_GOTREF(__pyx_t_2);
      } else
      #endif
      #if CYTHON_FAST_PYCCALL
      if (__Pyx_PyFastCFunction_Check(__pyx_t_3)) {
        PyObject *__pyx_temp[4] = {__pyx_t_4, __pyx_v_name, __pyx_v_size, __pyx_v_filename};
        __pyx_t_2 = __Pyx_PyCFunction_FastCall(__pyx_t_3, __pyx_temp+1-__pyx_t_5, 3+__pyx_t_5); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 748, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        __Pyx_GOTREF(__pyx_t_2);
      } else
      #endif
      {
        __pyx_t_6 = PyTuple_New(3+__pyx_t_5); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 748, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_6);
        if (__pyx_t_4) {
          __Pyx_GIVEREF(__pyx_t_4); PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_4); __pyx_t_4 = NULL;
        }
        __Pyx_INCREF(__pyx_v_name);
        __Pyx_GIVEREF(__pyx_v_name);
        PyTuple_SET_ITEM(__pyx_t_6, 0+__pyx_t_5, __pyx_v_name);
        __Pyx_INCREF(__pyx_v_size);
        __Pyx_GIVEREF(__pyx_v_size);
        PyTuple_SET_ITEM(__pyx_t_6, 1+__pyx_t_5, __pyx_v_size);
        __Pyx_INCREF(__pyx_v_filename);
        __Pyx_GIVEREF(__pyx_v_filename);
        PyTuple_SET_ITEM(__pyx_t_6, 2+__pyx_t_5, __pyx_v_filename);
        __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_t_6, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 748, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      }
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __pyx_r = __pyx_t_2;
      __pyx_t_2 = 0;
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      goto __pyx_L0;
    }
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  }

  /* "pymarketcap/core.pyx":762
 *             Filename of downloaded file if all was correct.
 *         """
 *         if self._is_symbol(name):             # <<<<<<<<<<<<<<
 *             try:
 *                 _name = self.ids_correspondences[name]
 */
  __pyx_t_1 = ((struct __pyx_vtabstruct_11pymarketcap_4core_Pymarketcap *)__pyx_v_self->__pyx_vtab)->_is_symbol(__pyx_v_self, __pyx_v_name, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 762, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_7 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_7 < 0)) __PYX_ERR(0, 762, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (__pyx_t_7) {

    /* "pymarketcap/core.pyx":763
 *         """
 *         if self._is_symbol(name):
 *             try:             # <<<<<<<<<<<<<<
 *                 _name = self.ids_correspondences[name]
 *             except KeyError:
 */
    {
      __Pyx_PyThreadState_declare
      __Pyx_PyThreadState_assign
      __Pyx_ExceptionSave(&__pyx_t_8, &__pyx_t_9, &__pyx_t_10);
      __Pyx_XGOTREF(__pyx_t_8);
      __Pyx_XGOTREF(__pyx_t_9);
      __Pyx_XGOTREF(__pyx_t_10);
      /*try:*/ {

        /* "pymarketcap/core.pyx":764
 *         if self._is_symbol(name):
 *             try:
 *                 _name = self.ids_correspondences[name]             # <<<<<<<<<<<<<<
 *             except KeyError:
 *                 if name not in list(self.ids_correspondences.keys()):
 */
        __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_ids_correspondences); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 764, __pyx_L4_error)
        __Pyx_GOTREF(__pyx_t_1);
        __pyx_t_2 = __Pyx_PyObject_Dict_GetItem(__pyx_t_1, __pyx_v_name); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 764, __pyx_L4_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        __pyx_v__name = __pyx_t_2;
        __pyx_t_2 = 0;

        /* "pymarketcap/core.pyx":763
 *         """
 *         if self._is_symbol(name):
 *             try:             # <<<<<<<<<<<<<<
 *                 _name = self.ids_correspondences[name]
 *             except KeyError:
 */
      }
      __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
      __Pyx_XDECREF(__pyx_t_9); __pyx_t_9 = 0;
      __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
      goto __pyx_L9_try_end;
      __pyx_L4_error:;
      __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
      __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;

      /* "pymarketcap/core.pyx":765
 *             try:
 *                 _name = self.ids_correspondences[name]
 *             except KeyError:             # <<<<<<<<<<<<<<
 *                 if name not in list(self.ids_correspondences.keys()):
 *                     raise ValueError(
 */
      __pyx_t_5 = __Pyx_PyErr_ExceptionMatches(__pyx_builtin_KeyError);
      if (__pyx_t_5) {
        __Pyx_AddTraceback("pymarketcap.core.Pymarketcap.download_logo", __pyx_clineno, __pyx_lineno, __pyx_filename);
        if (__Pyx_GetException(&__pyx_t_2, &__pyx_t_1, &__pyx_t_3) < 0) __PYX_ERR(0, 765, __pyx_L6_except_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_GOTREF(__pyx_t_1);
        __Pyx_GOTREF(__pyx_t_3);

        /* "pymarketcap/core.pyx":766
 *                 _name = self.ids_correspondences[name]
 *             except KeyError:
 *                 if name not in list(self.ids_correspondences.keys()):             # <<<<<<<<<<<<<<
 *                     raise ValueError(
 *                         "The currency %s is not valid. See 'symbols' instance attribute." % name
 */
        __pyx_t_4 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_ids_correspondences); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 766, __pyx_L6_except_error)
        __Pyx_GOTREF(__pyx_t_4);
        __pyx_t_11 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_n_s_keys); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 766, __pyx_L6_except_error)
        __Pyx_GOTREF(__pyx_t_11);
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
        __pyx_t_4 = NULL;
        if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_11))) {
          __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_11);
          if (likely(__pyx_t_4)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_11);
            __Pyx_INCREF(__pyx_t_4);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_11, function);
          }
        }
        if (__pyx_t_4) {
          __pyx_t_6 = __Pyx_PyObject_CallOneArg(__pyx_t_11, __pyx_t_4); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 766, __pyx_L6_except_error)
          __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
        } else {
          __pyx_t_6 = __Pyx_PyObject_CallNoArg(__pyx_t_11); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 766, __pyx_L6_except_error)
        }
        __Pyx_GOTREF(__pyx_t_6);
        __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
        __pyx_t_11 = PySequence_List(__pyx_t_6); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 766, __pyx_L6_except_error)
        __Pyx_GOTREF(__pyx_t_11);
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
        __pyx_t_7 = (__Pyx_PySequence_ContainsTF(__pyx_v_name, __pyx_t_11, Py_NE)); if (unlikely(__pyx_t_7 < 0)) __PYX_ERR(0, 766, __pyx_L6_except_error)
        __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
        __pyx_t_12 = (__pyx_t_7 != 0);
        if (unlikely(__pyx_t_12)) {

          /* "pymarketcap/core.pyx":768
 *                 if name not in list(self.ids_correspondences.keys()):
 *                     raise ValueError(
 *                         "The currency %s is not valid. See 'symbols' instance attribute." % name             # <<<<<<<<<<<<<<
 *                     )
 *         else:
 */
          __pyx_t_11 = __Pyx_PyString_Format(__pyx_kp_s_The_currency_s_is_not_valid_See, __pyx_v_name); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 768, __pyx_L6_except_error)
          __Pyx_GOTREF(__pyx_t_11);

          /* "pymarketcap/core.pyx":767
 *             except KeyError:
 *                 if name not in list(self.ids_correspondences.keys()):
 *                     raise ValueError(             # <<<<<<<<<<<<<<
 *                         "The currency %s is not valid. See 'symbols' instance attribute." % name
 *                     )
 */
          __pyx_t_6 = __Pyx_PyObject_CallOneArg(__pyx_builtin_ValueError, __pyx_t_11); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 767, __pyx_L6_except_error)
          __Pyx_GOTREF(__pyx_t_6);
          __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
          __Pyx_Raise(__pyx_t_6, 0, 0, 0);
          __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
          __PYX_ERR(0, 767, __pyx_L6_except_error)

          /* "pymarketcap/core.pyx":766
 *                 _name = self.ids_correspondences[name]
 *             except KeyError:
 *                 if name not in list(self.ids_correspondences.keys()):             # <<<<<<<<<<<<<<
 *                     raise ValueError(
 *                         "The currency %s is not valid. See 'symbols' instance attribute." % name
 */
        }
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        goto __pyx_L5_exception_handled;
      }
      goto __pyx_L6_except_error;
      __pyx_L6_except_error:;

      /* "pymarketcap/core.pyx":763
 *         """
 *         if self._is_symbol(name):
 *             try:             # <<<<<<<<<<<<<<
 *                 _name = self.ids_correspondences[name]
 *             except KeyError:
 */
      __Pyx_XGIVEREF(__pyx_t_8);
      __Pyx_XGIVEREF(__pyx_t_9);
      __Pyx_XGIVEREF(__pyx_t_10);
      __Pyx_ExceptionReset(__pyx_t_8, __pyx_t_9, __pyx_t_10);
      goto __pyx_L1_error;
      __pyx_L5_exception_handled:;
      __Pyx_XGIVEREF(__pyx_t_8);
      __Pyx_XGIVEREF(__pyx_t_9);
      __Pyx_XGIVEREF(__pyx_t_10);
      __Pyx_ExceptionReset(__pyx_t_8, __pyx_t_9, __pyx_t_10);
      __pyx_L9_try_end:;
    }

    /* "pymarketcap/core.pyx":762
 *             Filename of downloaded file if all was correct.
 *         """
 *         if self._is_symbol(name):             # <<<<<<<<<<<<<<
 *             try:
 *                 _name = self.ids_correspondences[name]
 */
    goto __pyx_L3;
  }

  /* "pymarketcap/core.pyx":771
 *                     )
 *         else:
 *             _name = name             # <<<<<<<<<<<<<<
 * 
 *         url_schema = "https://s2.coinmarketcap.com/static/img/coins/%dx%d/%d.png"
 */
  /*else*/ {
    __Pyx_INCREF(__pyx_v_name);
    __pyx_v__name = __pyx_v_name;
  }
  __pyx_L3:;

  /* "pymarketcap/core.pyx":773
 *             _name = name
 * 
 *         url_schema = "https://s2.coinmarketcap.com/static/img/coins/%dx%d/%d.png"             # <<<<<<<<<<<<<<
 *         url = url_schema % (size, size, _name)
 *         if not filename:
 */
  __Pyx_INCREF(__pyx_kp_s_https_s2_coinmarketcap_com_stati);
  __pyx_v_url_schema = __pyx_kp_s_https_s2_coinmarketcap_com_stati;

  /* "pymarketcap/core.pyx":774
 * 
 *         url_schema = "https://s2.coinmarketcap.com/static/img/coins/%dx%d/%d.png"
 *         url = url_schema % (size, size, _name)             # <<<<<<<<<<<<<<
 *         if not filename:
 *             filename = "%s_%dx%d.png" % (self.correspondences[name], size, size)
 */
  if (unlikely(!__pyx_v__name)) { __Pyx_RaiseUnboundLocalError("_name"); __PYX_ERR(0, 774, __pyx_L1_error) }
  __pyx_t_3 = PyTuple_New(3); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 774, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_INCREF(__pyx_v_size);
  __Pyx_GIVEREF(__pyx_v_size);
  PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_v_size);
  __Pyx_INCREF(__pyx_v_size);
  __Pyx_GIVEREF(__pyx_v_size);
  PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_v_size);
  __Pyx_INCREF(__pyx_v__name);
  __Pyx_GIVEREF(__pyx_v__name);
  PyTuple_SET_ITEM(__pyx_t_3, 2, __pyx_v__name);
  __pyx_t_1 = PyNumber_Remainder(__pyx_v_url_schema, __pyx_t_3); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 774, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_v_url = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "pymarketcap/core.pyx":775
 *         url_schema = "https://s2.coinmarketcap.com/static/img/coins/%dx%d/%d.png"
 *         url = url_schema % (size, size, _name)
 *         if not filename:             # <<<<<<<<<<<<<<
 *             filename = "%s_%dx%d.png" % (self.correspondences[name], size, size)
 *         try:
 */
  __pyx_t_12 = __Pyx_PyObject_IsTrue(__pyx_v_filename); if (unlikely(__pyx_t_12 < 0)) __PYX_ERR(0, 775, __pyx_L1_error)
  __pyx_t_7 = ((!__pyx_t_12) != 0);
  if (__pyx_t_7) {

    /* "pymarketcap/core.pyx":776
 *         url = url_schema % (size, size, _name)
 *         if not filename:
 *             filename = "%s_%dx%d.png" % (self.correspondences[name], size, size)             # <<<<<<<<<<<<<<
 *         try:
 *             res = urlretrieve(url, filename)
 */
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_correspondences); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 776, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_3 = __Pyx_PyObject_Dict_GetItem(__pyx_t_1, __pyx_v_name); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 776, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_1 = PyTuple_New(3); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 776, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_GIVEREF(__pyx_t_3);
    PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_t_3);
    __Pyx_INCREF(__pyx_v_size);
    __Pyx_GIVEREF(__pyx_v_size);
    PyTuple_SET_ITEM(__pyx_t_1, 1, __pyx_v_size);
    __Pyx_INCREF(__pyx_v_size);
    __Pyx_GIVEREF(__pyx_v_size);
    PyTuple_SET_ITEM(__pyx_t_1, 2, __pyx_v_size);
    __pyx_t_3 = 0;
    __pyx_t_3 = __Pyx_PyString_Format(__pyx_kp_s_s__dx_d_png, __pyx_t_1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 776, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_DECREF_SET(__pyx_v_filename, __pyx_t_3);
    __pyx_t_3 = 0;

    /* "pymarketcap/core.pyx":775
 *         url_schema = "https://s2.coinmarketcap.com/static/img/coins/%dx%d/%d.png"
 *         url = url_schema % (size, size, _name)
 *         if not filename:             # <<<<<<<<<<<<<<
 *             filename = "%s_%dx%d.png" % (self.correspondences[name], size, size)
 *         try:
 */
  }

  /* "pymarketcap/core.pyx":777
 *         if not filename:
 *             filename = "%s_%dx%d.png" % (self.correspondences[name], size, size)
 *         try:             # <<<<<<<<<<<<<<
 *             res = urlretrieve(url, filename)
 *         except HTTPError as e:
 */
  {
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    __Pyx_ExceptionSave(&__pyx_t_10, &__pyx_t_9, &__pyx_t_8);
    __Pyx_XGOTREF(__pyx_t_10);
    __Pyx_XGOTREF(__pyx_t_9);
    __Pyx_XGOTREF(__pyx_t_8);
    /*try:*/ {

      /* "pymarketcap/core.pyx":778
 *             filename = "%s_%dx%d.png" % (self.correspondences[name], size, size)
 *         try:
 *             res = urlretrieve(url, filename)             # <<<<<<<<<<<<<<
 *         except HTTPError as e:
 *             if e.code == 403:
 */
      __pyx_t_1 = __Pyx_GetModuleGlobalName(__pyx_n_s_urlretrieve); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 778, __pyx_L14_error)
      __Pyx_GOTREF(__pyx_t_1);
      __pyx_t_2 = NULL;
      __pyx_t_5 = 0;
      if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_1))) {
        __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_1);
        if (likely(__pyx_t_2)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_1);
          __Pyx_INCREF(__pyx_t_2);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_1, function);
          __pyx_t_5 = 1;
        }
      }
      #if CYTHON_FAST_PYCALL
      if (PyFunction_Check(__pyx_t_1)) {
        PyObject *__pyx_temp[3] = {__pyx_t_2, __pyx_v_url, __pyx_v_filename};
        __pyx_t_3 = __Pyx_PyFunction_FastCall(__pyx_t_1, __pyx_temp+1-__pyx_t_5, 2+__pyx_t_5); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 778, __pyx_L14_error)
        __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
        __Pyx_GOTREF(__pyx_t_3);
      } else
      #endif
      #if CYTHON_FAST_PYCCALL
      if (__Pyx_PyFastCFunction_Check(__pyx_t_1)) {
        PyObject *__pyx_temp[3] = {__pyx_t_2, __pyx_v_url, __pyx_v_filename};
        __pyx_t_3 = __Pyx_PyCFunction_FastCall(__pyx_t_1, __pyx_temp+1-__pyx_t_5, 2+__pyx_t_5); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 778, __pyx_L14_error)
        __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
        __Pyx_GOTREF(__pyx_t_3);
      } else
      #endif
      {
        __pyx_t_6 = PyTuple_New(2+__pyx_t_5); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 778, __pyx_L14_error)
        __Pyx_GOTREF(__pyx_t_6);
        if (__pyx_t_2) {
          __Pyx_GIVEREF(__pyx_t_2); PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_2); __pyx_t_2 = NULL;
        }
        __Pyx_INCREF(__pyx_v_url);
        __Pyx_GIVEREF(__pyx_v_url);
        PyTuple_SET_ITEM(__pyx_t_6, 0+__pyx_t_5, __pyx_v_url);
        __Pyx_INCREF(__pyx_v_filename);
        __Pyx_GIVEREF(__pyx_v_filename);
        PyTuple_SET_ITEM(__pyx_t_6, 1+__pyx_t_5, __pyx_v_filename);
        __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_t_6, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 778, __pyx_L14_error)
        __Pyx_GOTREF(__pyx_t_3);
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      }
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __pyx_v_res = __pyx_t_3;
      __pyx_t_3 = 0;

      /* "pymarketcap/core.pyx":777
 *         if not filename:
 *             filename = "%s_%dx%d.png" % (self.correspondences[name], size, size)
 *         try:             # <<<<<<<<<<<<<<
 *             res = urlretrieve(url, filename)
 *         except HTTPError as e:
 */
    }

    /* "pymarketcap/core.pyx":791
 *             raise e
 *         else:
 *             return filename             # <<<<<<<<<<<<<<
 * 
 *     cpdef download_exchange_logo(self, unicode name, size=64, filename=None):
 */
    /*else:*/ {
      __Pyx_XDECREF(__pyx_r);
      __Pyx_INCREF(__pyx_v_filename);
      __pyx_r = __pyx_v_filename;
      goto __pyx_L17_except_return;
    }
    __pyx_L14_error:;
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_XDECREF(__pyx_t_11); __pyx_t_11 = 0;
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "pymarketcap/core.pyx":779
 *         try:
 *             res = urlretrieve(url, filename)
 *         except HTTPError as e:             # <<<<<<<<<<<<<<
 *             if e.code == 403:
 *                 valid_sizes = [16, 32, 64, 128, 200]
 */
    __Pyx_ErrFetch(&__pyx_t_3, &__pyx_t_1, &__pyx_t_6);
    __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_HTTPError); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 779, __pyx_L16_except_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_5 = __Pyx_PyErr_GivenExceptionMatches(__pyx_t_3, __pyx_t_2);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_ErrRestore(__pyx_t_3, __pyx_t_1, __pyx_t_6);
    __pyx_t_3 = 0; __pyx_t_1 = 0; __pyx_t_6 = 0;
    if (__pyx_t_5) {
      __Pyx_AddTraceback("pymarketcap.core.Pymarketcap.download_logo", __pyx_clineno, __pyx_lineno, __pyx_filename);
      if (__Pyx_GetException(&__pyx_t_6, &__pyx_t_1, &__pyx_t_3) < 0) __PYX_ERR(0, 779, __pyx_L16_except_error)
      __Pyx_GOTREF(__pyx_t_6);
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_1);
      __pyx_v_e = __pyx_t_1;

      /* "pymarketcap/core.pyx":780
 *             res = urlretrieve(url, filename)
 *         except HTTPError as e:
 *             if e.code == 403:             # <<<<<<<<<<<<<<
 *                 valid_sizes = [16, 32, 64, 128, 200]
 *                 if size in valid_sizes:
 */
      __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_e, __pyx_n_s_code); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 780, __pyx_L16_except_error)
      __Pyx_GOTREF(__pyx_t_2);
      __pyx_t_11 = __Pyx_PyInt_EqObjC(__pyx_t_2, __pyx_int_403, 0x193, 0); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 780, __pyx_L16_except_error)
      __Pyx_GOTREF(__pyx_t_11);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __pyx_t_7 = __Pyx_PyObject_IsTrue(__pyx_t_11); if (unlikely(__pyx_t_7 < 0)) __PYX_ERR(0, 780, __pyx_L16_except_error)
      __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
      if (__pyx_t_7) {

        /* "pymarketcap/core.pyx":781
 *         except HTTPError as e:
 *             if e.code == 403:
 *                 valid_sizes = [16, 32, 64, 128, 200]             # <<<<<<<<<<<<<<
 *                 if size in valid_sizes:
 *                     raise ValueError(
 */
        __pyx_t_11 = PyList_New(5); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 781, __pyx_L16_except_error)
        __Pyx_GOTREF(__pyx_t_11);
        __Pyx_INCREF(__pyx_int_16);
        __Pyx_GIVEREF(__pyx_int_16);
        PyList_SET_ITEM(__pyx_t_11, 0, __pyx_int_16);
        __Pyx_INCREF(__pyx_int_32);
        __Pyx_GIVEREF(__pyx_int_32);
        PyList_SET_ITEM(__pyx_t_11, 1, __pyx_int_32);
        __Pyx_INCREF(__pyx_int_64);
        __Pyx_GIVEREF(__pyx_int_64);
        PyList_SET_ITEM(__pyx_t_11, 2, __pyx_int_64);
        __Pyx_INCREF(__pyx_int_128);
        __Pyx_GIVEREF(__pyx_int_128);
        PyList_SET_ITEM(__pyx_t_11, 3, __pyx_int_128);
        __Pyx_INCREF(__pyx_int_200);
        __Pyx_GIVEREF(__pyx_int_200);
        PyList_SET_ITEM(__pyx_t_11, 4, __pyx_int_200);
        __pyx_v_valid_sizes = ((PyObject*)__pyx_t_11);
        __pyx_t_11 = 0;

        /* "pymarketcap/core.pyx":782
 *             if e.code == 403:
 *                 valid_sizes = [16, 32, 64, 128, 200]
 *                 if size in valid_sizes:             # <<<<<<<<<<<<<<
 *                     raise ValueError(
 *                         ("Seems that %s currency doesn't allows to be downloaded with " \
 */
        __pyx_t_7 = (__Pyx_PySequence_ContainsTF(__pyx_v_size, __pyx_v_valid_sizes, Py_EQ)); if (unlikely(__pyx_t_7 < 0)) __PYX_ERR(0, 782, __pyx_L16_except_error)
        __pyx_t_12 = (__pyx_t_7 != 0);
        if (unlikely(__pyx_t_12)) {

          /* "pymarketcap/core.pyx":785
 *                     raise ValueError(
 *                         ("Seems that %s currency doesn't allows to be downloaded with " \
 *                         + "size %dx%d. Try another size.") % (name, size, size)             # <<<<<<<<<<<<<<
 *                     )
 *                 else:
 */
          __pyx_t_11 = PyNumber_Add(__pyx_kp_s_Seems_that_s_currency_doesn_t_al, __pyx_kp_s_size_dx_d_Try_another_size); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 785, __pyx_L16_except_error)
          __Pyx_GOTREF(__pyx_t_11);
          __pyx_t_2 = PyTuple_New(3); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 785, __pyx_L16_except_error)
          __Pyx_GOTREF(__pyx_t_2);
          __Pyx_INCREF(__pyx_v_name);
          __Pyx_GIVEREF(__pyx_v_name);
          PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_v_name);
          __Pyx_INCREF(__pyx_v_size);
          __Pyx_GIVEREF(__pyx_v_size);
          PyTuple_SET_ITEM(__pyx_t_2, 1, __pyx_v_size);
          __Pyx_INCREF(__pyx_v_size);
          __Pyx_GIVEREF(__pyx_v_size);
          PyTuple_SET_ITEM(__pyx_t_2, 2, __pyx_v_size);
          __pyx_t_4 = __Pyx_PyString_Format(__pyx_t_11, __pyx_t_2); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 785, __pyx_L16_except_error)
          __Pyx_GOTREF(__pyx_t_4);
          __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
          __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

          /* "pymarketcap/core.pyx":783
 *                 valid_sizes = [16, 32, 64, 128, 200]
 *                 if size in valid_sizes:
 *                     raise ValueError(             # <<<<<<<<<<<<<<
 *                         ("Seems that %s currency doesn't allows to be downloaded with " \
 *                         + "size %dx%d. Try another size.") % (name, size, size)
 */
          __pyx_t_2 = __Pyx_PyObject_CallOneArg(__pyx_builtin_ValueError, __pyx_t_4); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 783, __pyx_L16_except_error)
          __Pyx_GOTREF(__pyx_t_2);
          __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
          __Pyx_Raise(__pyx_t_2, 0, 0, 0);
          __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
          __PYX_ERR(0, 783, __pyx_L16_except_error)

          /* "pymarketcap/core.pyx":782
 *             if e.code == 403:
 *                 valid_sizes = [16, 32, 64, 128, 200]
 *                 if size in valid_sizes:             # <<<<<<<<<<<<<<
 *                     raise ValueError(
 *                         ("Seems that %s currency doesn't allows to be downloaded with " \
 */
        }

        /* "pymarketcap/core.pyx":788
 *                     )
 *                 else:
 *                     raise ValueError("%dx%d is not a valid size." % (size, size))             # <<<<<<<<<<<<<<
 *             raise e
 *         else:
 */
        /*else*/ {
          __pyx_t_2 = PyTuple_New(2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 788, __pyx_L16_except_error)
          __Pyx_GOTREF(__pyx_t_2);
          __Pyx_INCREF(__pyx_v_size);
          __Pyx_GIVEREF(__pyx_v_size);
          PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_v_size);
          __Pyx_INCREF(__pyx_v_size);
          __Pyx_GIVEREF(__pyx_v_size);
          PyTuple_SET_ITEM(__pyx_t_2, 1, __pyx_v_size);
          __pyx_t_4 = __Pyx_PyString_Format(__pyx_kp_s_dx_d_is_not_a_valid_size, __pyx_t_2); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 788, __pyx_L16_except_error)
          __Pyx_GOTREF(__pyx_t_4);
          __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
          __pyx_t_2 = __Pyx_PyObject_CallOneArg(__pyx_builtin_ValueError, __pyx_t_4); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 788, __pyx_L16_except_error)
          __Pyx_GOTREF(__pyx_t_2);
          __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
          __Pyx_Raise(__pyx_t_2, 0, 0, 0);
          __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
          __PYX_ERR(0, 788, __pyx_L16_except_error)
        }

        /* "pymarketcap/core.pyx":780
 *             res = urlretrieve(url, filename)
 *         except HTTPError as e:
 *             if e.code == 403:             # <<<<<<<<<<<<<<
 *                 valid_sizes = [16, 32, 64, 128, 200]
 *                 if size in valid_sizes:
 */
      }

      /* "pymarketcap/core.pyx":789
 *                 else:
 *                     raise ValueError("%dx%d is not a valid size." % (size, size))
 *             raise e             # <<<<<<<<<<<<<<
 *         else:
 *             return filename
 */
      __Pyx_Raise(__pyx_v_e, 0, 0, 0);
      __PYX_ERR(0, 789, __pyx_L16_except_error)
    }
    goto __pyx_L16_except_error;
    __pyx_L16_except_error:;

    /* "pymarketcap/core.pyx":777
 *         if not filename:
 *             filename = "%s_%dx%d.png" % (self.correspondences[name], size, size)
 *         try:             # <<<<<<<<<<<<<<
 *             res = urlretrieve(url, filename)
 *         except HTTPError as e:
 */
    __Pyx_XGIVEREF(__pyx_t_10);
    __Pyx_XGIVEREF(__pyx_t_9);
    __Pyx_XGIVEREF(__pyx_t_8);
    __Pyx_ExceptionReset(__pyx_t_10, __pyx_t_9, __pyx_t_8);
    goto __pyx_L1_error;
    __pyx_L17_except_return:;
    __Pyx_XGIVEREF(__pyx_t_10);
    __Pyx_XGIVEREF(__pyx_t_9);
    __Pyx_XGIVEREF(__pyx_t_8);
    __Pyx_ExceptionReset(__pyx_t_10, __pyx_t_9, __pyx_t_8);
    goto __pyx_L0;
  }

  /* "pymarketcap/core.pyx":748
 *         return processer.graphs(res, start, end)
 * 
 *     cpdef download_logo(self, unicode name, size=64, filename=None):             # <<<<<<<<<<<<<<
 *         """Download a currency image logo providing their size.
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_11);
  __Pyx_AddTraceback("pymarketcap.core.Pymarketcap.download_logo", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v__name);
  __Pyx_XDECREF(__pyx_v_url_schema);
  __Pyx_XDECREF(__pyx_v_url);
  __Pyx_XDECREF(__pyx_v_res);
  __Pyx_XDECREF(__pyx_v_e);
  __Pyx_XDECREF(__pyx_v_valid_sizes);
  __Pyx_XDECREF(__pyx_v_filename);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_11pymarketcap_4core_11Pymarketcap_43download_logo(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_11pymarketcap_4core_11Pymarketcap_42download_logo[] = "Download a currency image logo providing their size.\n\n        Args:\n            currency (str): Currency name or symbol to download.\n            size (int, optional): Size in pixels. Valid sizes are:\n                ``[16, 32, 64, 128, 200]``. As default ``128``.\n            filename (str, optional): Filename for store the logo.\n                Doesn't include the extension (will be \".png\").\n                As default ``None``.\n\n        Returns (str):\n            Filename of downloaded file if all was correct.\n        ";
static PyObject *__pyx_pw_11pymarketcap_4core_11Pymarketcap_43download_logo(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_name = 0;
  PyObject *__pyx_v_size = 0;
  PyObject *__pyx_v_filename = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("download_logo (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_name,&__pyx_n_s_size,&__pyx_n_s_filename,0};
    PyObject* values[3] = {0,0,0};
    values[1] = ((PyObject *)__pyx_int_64);
    values[2] = ((PyObject *)Py_None);
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_name)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_size);
          if (value) { values[1] = value; kw_args--; }
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_filename);
          if (value) { values[2] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "download_logo") < 0)) __PYX_ERR(0, 748, __pyx_L3_error)
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_name = ((PyObject*)values[0]);
    __pyx_v_size = values[1];
    __pyx_v_filename = values[2];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("download_logo", 0, 1, 3, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 748, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("pymarketcap.core.Pymarketcap.download_logo", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_name), (&PyUnicode_Type), 1, "name", 1))) __PYX_ERR(0, 748, __pyx_L1_error)
  __pyx_r = __pyx_pf_11pymarketcap_4core_11Pymarketcap_42download_logo(((struct __pyx_obj_11pymarketcap_4core_Pymarketcap *)__pyx_v_self), __pyx_v_name, __pyx_v_size, __pyx_v_filename);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_11pymarketcap_4core_11Pymarketcap_42download_logo(struct __pyx_obj_11pymarketcap_4core_Pymarketcap *__pyx_v_self, PyObject *__pyx_v_name, PyObject *__pyx_v_size, PyObject *__pyx_v_filename) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  struct __pyx_opt_args_11pymarketcap_4core_11Pymarketcap_download_logo __pyx_t_2;
  __Pyx_RefNannySetupContext("download_logo", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2.__pyx_n = 2;
  __pyx_t_2.size = __pyx_v_size;
  __pyx_t_2.filename = __pyx_v_filename;
  __pyx_t_1 = __pyx_vtabptr_11pymarketcap_4core_Pymarketcap->download_logo(__pyx_v_self, __pyx_v_name, 1, &__pyx_t_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 748, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pymarketcap.core.Pymarketcap.download_logo", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pymarketcap/core.pyx":793
 *             return filename
 * 
 *     cpdef download_exchange_logo(self, unicode name, size=64, filename=None):             # <<<<<<<<<<<<<<
 *         """Download a exchange logo passing his name or
 *         slug as first parameter and optionally a filename
 */

static PyObject *__pyx_pw_11pymarketcap_4core_11Pymarketcap_45download_exchange_logo(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyObject *__pyx_f_11pymarketcap_4core_11Pymarketcap_download_exchange_logo(struct __pyx_obj_11pymarketcap_4core_Pymarketcap *__pyx_v_self, PyObject *__pyx_v_name, int __pyx_skip_dispatch, struct __pyx_opt_args_11pymarketcap_4core_11Pymarketcap_download_exchange_logo *__pyx_optional_args) {
  PyObject *__pyx_v_size = ((PyObject *)__pyx_int_64);
  PyObject *__pyx_v_filename = ((PyObject *)Py_None);
  PyObject *__pyx_v_exc_names_slugs_ids = NULL;
  int __pyx_v_exc_found;
  PyObject *__pyx_v_exc = NULL;
  PyObject *__pyx_v__id = NULL;
  PyObject *__pyx_v__slug = NULL;
  CYTHON_UNUSED PyObject *__pyx_v__name = NULL;
  PyObject *__pyx_v_url = NULL;
  CYTHON_UNUSED PyObject *__pyx_v_res = NULL;
  PyObject *__pyx_v_e = NULL;
  PyObject *__pyx_v_valid_sizes = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_t_7;
  int __pyx_t_8;
  Py_ssize_t __pyx_t_9;
  PyObject *__pyx_t_10 = NULL;
  PyObject *__pyx_t_11 = NULL;
  PyObject *__pyx_t_12 = NULL;
  PyObject *__pyx_t_13 = NULL;
  __Pyx_RefNannySetupContext("download_exchange_logo", 0);
  if (__pyx_optional_args) {
    if (__pyx_optional_args->__pyx_n > 0) {
      __pyx_v_size = __pyx_optional_args->size;
      if (__pyx_optional_args->__pyx_n > 1) {
        __pyx_v_filename = __pyx_optional_args->filename;
      }
    }
  }
  __Pyx_INCREF(__pyx_v_filename);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0)) {
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_download_exchange_logo); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 793, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)__pyx_pw_11pymarketcap_4core_11Pymarketcap_45download_exchange_logo)) {
      __Pyx_XDECREF(__pyx_r);
      __Pyx_INCREF(__pyx_t_1);
      __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
      __pyx_t_5 = 0;
      if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
        __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
        if (likely(__pyx_t_4)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
          __Pyx_INCREF(__pyx_t_4);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_3, function);
          __pyx_t_5 = 1;
        }
      }
      #if CYTHON_FAST_PYCALL
      if (PyFunction_Check(__pyx_t_3)) {
        PyObject *__pyx_temp[4] = {__pyx_t_4, __pyx_v_name, __pyx_v_size, __pyx_v_filename};
        __pyx_t_2 = __Pyx_PyFunction_FastCall(__pyx_t_3, __pyx_temp+1-__pyx_t_5, 3+__pyx_t_5); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 793, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        __Pyx_GOTREF(__pyx_t_2);
      } else
      #endif
      #if CYTHON_FAST_PYCCALL
      if (__Pyx_PyFastCFunction_Check(__pyx_t_3)) {
        PyObject *__pyx_temp[4] = {__pyx_t_4, __pyx_v_name, __pyx_v_size, __pyx_v_filename};
        __pyx_t_2 = __Pyx_PyCFunction_FastCall(__pyx_t_3, __pyx_temp+1-__pyx_t_5, 3+__pyx_t_5); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 793, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        __Pyx_GOTREF(__pyx_t_2);
      } else
      #endif
      {
        __pyx_t_6 = PyTuple_New(3+__pyx_t_5); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 793, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_6);
        if (__pyx_t_4) {
          __Pyx_GIVEREF(__pyx_t_4); PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_4); __pyx_t_4 = NULL;
        }
        __Pyx_INCREF(__pyx_v_name);
        __Pyx_GIVEREF(__pyx_v_name);
        PyTuple_SET_ITEM(__pyx_t_6, 0+__pyx_t_5, __pyx_v_name);
        __Pyx_INCREF(__pyx_v_size);
        __Pyx_GIVEREF(__pyx_v_size);
        PyTuple_SET_ITEM(__pyx_t_6, 1+__pyx_t_5, __pyx_v_size);
        __Pyx_INCREF(__pyx_v_filename);
        __Pyx_GIVEREF(__pyx_v_filename);
        PyTuple_SET_ITEM(__pyx_t_6, 2+__pyx_t_5, __pyx_v_filename);
        __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_t_6, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 793, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      }
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __pyx_r = __pyx_t_2;
      __pyx_t_2 = 0;
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      goto __pyx_L0;
    }
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  }

  /* "pymarketcap/core.pyx":810
 *         """
 *         # Ensure exchanges - names - ids correspondences are cached
 *         exc_names_slugs_ids = self._exchange_names_slugs_ids             # <<<<<<<<<<<<<<
 *         if not exc_names_slugs_ids:  # Not cached yet
 *             self._cache_exchanges_ids()
 */
  __pyx_t_1 = __pyx_v_self->_exchange_names_slugs_ids;
  __Pyx_INCREF(__pyx_t_1);
  __pyx_v_exc_names_slugs_ids = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "pymarketcap/core.pyx":811
 *         # Ensure exchanges - names - ids correspondences are cached
 *         exc_names_slugs_ids = self._exchange_names_slugs_ids
 *         if not exc_names_slugs_ids:  # Not cached yet             # <<<<<<<<<<<<<<
 *             self._cache_exchanges_ids()
 * 
 */
  __pyx_t_7 = (__pyx_v_exc_names_slugs_ids != Py_None)&&(PyList_GET_SIZE(__pyx_v_exc_names_slugs_ids) != 0);
  __pyx_t_8 = ((!__pyx_t_7) != 0);
  if (__pyx_t_8) {

    /* "pymarketcap/core.pyx":812
 *         exc_names_slugs_ids = self._exchange_names_slugs_ids
 *         if not exc_names_slugs_ids:  # Not cached yet
 *             self._cache_exchanges_ids()             # <<<<<<<<<<<<<<
 * 
 *         exc_found = False
 */
    __pyx_t_1 = ((struct __pyx_vtabstruct_11pymarketcap_4core_Pymarketcap *)__pyx_v_self->__pyx_vtab)->_cache_exchanges_ids(__pyx_v_self, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 812, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "pymarketcap/core.pyx":811
 *         # Ensure exchanges - names - ids correspondences are cached
 *         exc_names_slugs_ids = self._exchange_names_slugs_ids
 *         if not exc_names_slugs_ids:  # Not cached yet             # <<<<<<<<<<<<<<
 *             self._cache_exchanges_ids()
 * 
 */
  }

  /* "pymarketcap/core.pyx":814
 *             self._cache_exchanges_ids()
 * 
 *         exc_found = False             # <<<<<<<<<<<<<<
 *         if name.islower():  # Is slug
 *             for exc in self._exchange_names_slugs_ids:
 */
  __pyx_v_exc_found = 0;

  /* "pymarketcap/core.pyx":815
 * 
 *         exc_found = False
 *         if name.islower():  # Is slug             # <<<<<<<<<<<<<<
 *             for exc in self._exchange_names_slugs_ids:
 *                 if name in exc:
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_name, __pyx_n_s_islower); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 815, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  if (__pyx_t_3) {
    __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_3); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 815, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  } else {
    __pyx_t_1 = __Pyx_PyObject_CallNoArg(__pyx_t_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 815, __pyx_L1_error)
  }
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_8 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_8 < 0)) __PYX_ERR(0, 815, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (__pyx_t_8) {

    /* "pymarketcap/core.pyx":816
 *         exc_found = False
 *         if name.islower():  # Is slug
 *             for exc in self._exchange_names_slugs_ids:             # <<<<<<<<<<<<<<
 *                 if name in exc:
 *                     exc_found = True
 */
    if (unlikely(__pyx_v_self->_exchange_names_slugs_ids == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not iterable");
      __PYX_ERR(0, 816, __pyx_L1_error)
    }
    __pyx_t_1 = __pyx_v_self->_exchange_names_slugs_ids; __Pyx_INCREF(__pyx_t_1); __pyx_t_9 = 0;
    for (;;) {
      if (__pyx_t_9 >= PyList_GET_SIZE(__pyx_t_1)) break;
      #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
      __pyx_t_2 = PyList_GET_ITEM(__pyx_t_1, __pyx_t_9); __Pyx_INCREF(__pyx_t_2); __pyx_t_9++; if (unlikely(0 < 0)) __PYX_ERR(0, 816, __pyx_L1_error)
      #else
      __pyx_t_2 = PySequence_ITEM(__pyx_t_1, __pyx_t_9); __pyx_t_9++; if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 816, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      #endif
      __Pyx_XDECREF_SET(__pyx_v_exc, __pyx_t_2);
      __pyx_t_2 = 0;

      /* "pymarketcap/core.pyx":817
 *         if name.islower():  # Is slug
 *             for exc in self._exchange_names_slugs_ids:
 *                 if name in exc:             # <<<<<<<<<<<<<<
 *                     exc_found = True
 *                     if exc.index(name) <= 1:
 */
      __pyx_t_8 = (__Pyx_PySequence_ContainsTF(__pyx_v_name, __pyx_v_exc, Py_EQ)); if (unlikely(__pyx_t_8 < 0)) __PYX_ERR(0, 817, __pyx_L1_error)
      __pyx_t_7 = (__pyx_t_8 != 0);
      if (__pyx_t_7) {

        /* "pymarketcap/core.pyx":818
 *             for exc in self._exchange_names_slugs_ids:
 *                 if name in exc:
 *                     exc_found = True             # <<<<<<<<<<<<<<
 *                     if exc.index(name) <= 1:
 *                         _id = exc[2]
 */
        __pyx_v_exc_found = 1;

        /* "pymarketcap/core.pyx":819
 *                 if name in exc:
 *                     exc_found = True
 *                     if exc.index(name) <= 1:             # <<<<<<<<<<<<<<
 *                         _id = exc[2]
 *                         _slug = exc[1]
 */
        __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_exc, __pyx_n_s_index); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 819, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_3);
        __pyx_t_6 = NULL;
        if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_3))) {
          __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_3);
          if (likely(__pyx_t_6)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
            __Pyx_INCREF(__pyx_t_6);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_3, function);
          }
        }
        if (!__pyx_t_6) {
          __pyx_t_2 = __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_v_name); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 819, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_2);
        } else {
          #if CYTHON_FAST_PYCALL
          if (PyFunction_Check(__pyx_t_3)) {
            PyObject *__pyx_temp[2] = {__pyx_t_6, __pyx_v_name};
            __pyx_t_2 = __Pyx_PyFunction_FastCall(__pyx_t_3, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 819, __pyx_L1_error)
            __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
            __Pyx_GOTREF(__pyx_t_2);
          } else
          #endif
          #if CYTHON_FAST_PYCCALL
          if (__Pyx_PyFastCFunction_Check(__pyx_t_3)) {
            PyObject *__pyx_temp[2] = {__pyx_t_6, __pyx_v_name};
            __pyx_t_2 = __Pyx_PyCFunction_FastCall(__pyx_t_3, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 819, __pyx_L1_error)
            __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
            __Pyx_GOTREF(__pyx_t_2);
          } else
          #endif
          {
            __pyx_t_4 = PyTuple_New(1+1); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 819, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_4);
            __Pyx_GIVEREF(__pyx_t_6); PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_6); __pyx_t_6 = NULL;
            __Pyx_INCREF(__pyx_v_name);
            __Pyx_GIVEREF(__pyx_v_name);
            PyTuple_SET_ITEM(__pyx_t_4, 0+1, __pyx_v_name);
            __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_t_4, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 819, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_2);
            __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
          }
        }
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __pyx_t_3 = PyObject_RichCompare(__pyx_t_2, __pyx_int_1, Py_LE); __Pyx_XGOTREF(__pyx_t_3); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 819, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
        __pyx_t_7 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely(__pyx_t_7 < 0)) __PYX_ERR(0, 819, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        if (__pyx_t_7) {

          /* "pymarketcap/core.pyx":820
 *                     exc_found = True
 *                     if exc.index(name) <= 1:
 *                         _id = exc[2]             # <<<<<<<<<<<<<<
 *                         _slug = exc[1]
 *                         _name = exc[0]
 */
          __pyx_t_3 = __Pyx_GetItemInt(__pyx_v_exc, 2, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 820, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_3);
          __pyx_v__id = __pyx_t_3;
          __pyx_t_3 = 0;

          /* "pymarketcap/core.pyx":821
 *                     if exc.index(name) <= 1:
 *                         _id = exc[2]
 *                         _slug = exc[1]             # <<<<<<<<<<<<<<
 *                         _name = exc[0]
 *                         break
 */
          __pyx_t_3 = __Pyx_GetItemInt(__pyx_v_exc, 1, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 821, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_3);
          __pyx_v__slug = __pyx_t_3;
          __pyx_t_3 = 0;

          /* "pymarketcap/core.pyx":822
 *                         _id = exc[2]
 *                         _slug = exc[1]
 *                         _name = exc[0]             # <<<<<<<<<<<<<<
 *                         break
 *                     else:
 */
          __pyx_t_3 = __Pyx_GetItemInt(__pyx_v_exc, 0, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 822, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_3);
          __pyx_v__name = __pyx_t_3;
          __pyx_t_3 = 0;

          /* "pymarketcap/core.pyx":823
 *                         _slug = exc[1]
 *                         _name = exc[0]
 *                         break             # <<<<<<<<<<<<<<
 *                     else:
 *                         print("exc = %s" % exc)
 */
          goto __pyx_L6_break;

          /* "pymarketcap/core.pyx":819
 *                 if name in exc:
 *                     exc_found = True
 *                     if exc.index(name) <= 1:             # <<<<<<<<<<<<<<
 *                         _id = exc[2]
 *                         _slug = exc[1]
 */
        }

        /* "pymarketcap/core.pyx":825
 *                         break
 *                     else:
 *                         print("exc = %s" % exc)             # <<<<<<<<<<<<<<
 *                         raise NotImplementedError(
 *                             "Slug catched but not found" \
 */
        /*else*/ {
          __pyx_t_3 = __Pyx_PyString_Format(__pyx_kp_s_exc_s, __pyx_v_exc); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 825, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_3);
          if (__Pyx_PrintOne(0, __pyx_t_3) < 0) __PYX_ERR(0, 825, __pyx_L1_error)
          __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

          /* "pymarketcap/core.pyx":828
 *                         raise NotImplementedError(
 *                             "Slug catched but not found" \
 *                             + " in slug index position (1)."             # <<<<<<<<<<<<<<
 *                         )
 *         else:
 */
          __pyx_t_3 = PyNumber_Add(__pyx_kp_s_Slug_catched_but_not_found, __pyx_kp_s_in_slug_index_position_1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 828, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_3);

          /* "pymarketcap/core.pyx":826
 *                     else:
 *                         print("exc = %s" % exc)
 *                         raise NotImplementedError(             # <<<<<<<<<<<<<<
 *                             "Slug catched but not found" \
 *                             + " in slug index position (1)."
 */
          __pyx_t_2 = __Pyx_PyObject_CallOneArg(__pyx_builtin_NotImplementedError, __pyx_t_3); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 826, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_2);
          __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
          __Pyx_Raise(__pyx_t_2, 0, 0, 0);
          __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
          __PYX_ERR(0, 826, __pyx_L1_error)
        }

        /* "pymarketcap/core.pyx":817
 *         if name.islower():  # Is slug
 *             for exc in self._exchange_names_slugs_ids:
 *                 if name in exc:             # <<<<<<<<<<<<<<
 *                     exc_found = True
 *                     if exc.index(name) <= 1:
 */
      }

      /* "pymarketcap/core.pyx":816
 *         exc_found = False
 *         if name.islower():  # Is slug
 *             for exc in self._exchange_names_slugs_ids:             # <<<<<<<<<<<<<<
 *                 if name in exc:
 *                     exc_found = True
 */
    }
    __pyx_L6_break:;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "pymarketcap/core.pyx":815
 * 
 *         exc_found = False
 *         if name.islower():  # Is slug             # <<<<<<<<<<<<<<
 *             for exc in self._exchange_names_slugs_ids:
 *                 if name in exc:
 */
    goto __pyx_L4;
  }

  /* "pymarketcap/core.pyx":831
 *                         )
 *         else:
 *             for exc in self._exchange_names_slugs_ids:             # <<<<<<<<<<<<<<
 *                 if name in exc:
 *                     exc_found = True
 */
  /*else*/ {
    if (unlikely(__pyx_v_self->_exchange_names_slugs_ids == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not iterable");
      __PYX_ERR(0, 831, __pyx_L1_error)
    }
    __pyx_t_1 = __pyx_v_self->_exchange_names_slugs_ids; __Pyx_INCREF(__pyx_t_1); __pyx_t_9 = 0;
    for (;;) {
      if (__pyx_t_9 >= PyList_GET_SIZE(__pyx_t_1)) break;
      #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
      __pyx_t_2 = PyList_GET_ITEM(__pyx_t_1, __pyx_t_9); __Pyx_INCREF(__pyx_t_2); __pyx_t_9++; if (unlikely(0 < 0)) __PYX_ERR(0, 831, __pyx_L1_error)
      #else
      __pyx_t_2 = PySequence_ITEM(__pyx_t_1, __pyx_t_9); __pyx_t_9++; if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 831, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      #endif
      __Pyx_XDECREF_SET(__pyx_v_exc, __pyx_t_2);
      __pyx_t_2 = 0;

      /* "pymarketcap/core.pyx":832
 *         else:
 *             for exc in self._exchange_names_slugs_ids:
 *                 if name in exc:             # <<<<<<<<<<<<<<
 *                     exc_found = True
 *                     if exc.index(name) == 0:
 */
      __pyx_t_7 = (__Pyx_PySequence_ContainsTF(__pyx_v_name, __pyx_v_exc, Py_EQ)); if (unlikely(__pyx_t_7 < 0)) __PYX_ERR(0, 832, __pyx_L1_error)
      __pyx_t_8 = (__pyx_t_7 != 0);
      if (__pyx_t_8) {

        /* "pymarketcap/core.pyx":833
 *             for exc in self._exchange_names_slugs_ids:
 *                 if name in exc:
 *                     exc_found = True             # <<<<<<<<<<<<<<
 *                     if exc.index(name) == 0:
 *                         _id = exc[2]
 */
        __pyx_v_exc_found = 1;

        /* "pymarketcap/core.pyx":834
 *                 if name in exc:
 *                     exc_found = True
 *                     if exc.index(name) == 0:             # <<<<<<<<<<<<<<
 *                         _id = exc[2]
 *                         _slug = exc[1]
 */
        __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_exc, __pyx_n_s_index); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 834, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_3);
        __pyx_t_4 = NULL;
        if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_3))) {
          __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
          if (likely(__pyx_t_4)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
            __Pyx_INCREF(__pyx_t_4);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_3, function);
          }
        }
        if (!__pyx_t_4) {
          __pyx_t_2 = __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_v_name); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 834, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_2);
        } else {
          #if CYTHON_FAST_PYCALL
          if (PyFunction_Check(__pyx_t_3)) {
            PyObject *__pyx_temp[2] = {__pyx_t_4, __pyx_v_name};
            __pyx_t_2 = __Pyx_PyFunction_FastCall(__pyx_t_3, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 834, __pyx_L1_error)
            __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
            __Pyx_GOTREF(__pyx_t_2);
          } else
          #endif
          #if CYTHON_FAST_PYCCALL
          if (__Pyx_PyFastCFunction_Check(__pyx_t_3)) {
            PyObject *__pyx_temp[2] = {__pyx_t_4, __pyx_v_name};
            __pyx_t_2 = __Pyx_PyCFunction_FastCall(__pyx_t_3, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 834, __pyx_L1_error)
            __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
            __Pyx_GOTREF(__pyx_t_2);
          } else
          #endif
          {
            __pyx_t_6 = PyTuple_New(1+1); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 834, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_6);
            __Pyx_GIVEREF(__pyx_t_4); PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_4); __pyx_t_4 = NULL;
            __Pyx_INCREF(__pyx_v_name);
            __Pyx_GIVEREF(__pyx_v_name);
            PyTuple_SET_ITEM(__pyx_t_6, 0+1, __pyx_v_name);
            __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_t_6, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 834, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_2);
            __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
          }
        }
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __pyx_t_3 = __Pyx_PyInt_EqObjC(__pyx_t_2, __pyx_int_0, 0, 0); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 834, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_3);
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
        __pyx_t_8 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely(__pyx_t_8 < 0)) __PYX_ERR(0, 834, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        if (__pyx_t_8) {

          /* "pymarketcap/core.pyx":835
 *                     exc_found = True
 *                     if exc.index(name) == 0:
 *                         _id = exc[2]             # <<<<<<<<<<<<<<
 *                         _slug = exc[1]
 *                         _name = exc[0]
 */
          __pyx_t_3 = __Pyx_GetItemInt(__pyx_v_exc, 2, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 835, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_3);
          __pyx_v__id = __pyx_t_3;
          __pyx_t_3 = 0;

          /* "pymarketcap/core.pyx":836
 *                     if exc.index(name) == 0:
 *                         _id = exc[2]
 *                         _slug = exc[1]             # <<<<<<<<<<<<<<
 *                         _name = exc[0]
 *                         break
 */
          __pyx_t_3 = __Pyx_GetItemInt(__pyx_v_exc, 1, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 836, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_3);
          __pyx_v__slug = __pyx_t_3;
          __pyx_t_3 = 0;

          /* "pymarketcap/core.pyx":837
 *                         _id = exc[2]
 *                         _slug = exc[1]
 *                         _name = exc[0]             # <<<<<<<<<<<<<<
 *                         break
 *                     else:
 */
          __pyx_t_3 = __Pyx_GetItemInt(__pyx_v_exc, 0, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 837, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_3);
          __pyx_v__name = __pyx_t_3;
          __pyx_t_3 = 0;

          /* "pymarketcap/core.pyx":838
 *                         _slug = exc[1]
 *                         _name = exc[0]
 *                         break             # <<<<<<<<<<<<<<
 *                     else:
 *                         print("exc = %s" % exc)
 */
          goto __pyx_L10_break;

          /* "pymarketcap/core.pyx":834
 *                 if name in exc:
 *                     exc_found = True
 *                     if exc.index(name) == 0:             # <<<<<<<<<<<<<<
 *                         _id = exc[2]
 *                         _slug = exc[1]
 */
        }

        /* "pymarketcap/core.pyx":840
 *                         break
 *                     else:
 *                         print("exc = %s" % exc)             # <<<<<<<<<<<<<<
 *                         raise NotImplementedError(
 *                             "Exchange name catched but not found" \
 */
        /*else*/ {
          __pyx_t_3 = __Pyx_PyString_Format(__pyx_kp_s_exc_s, __pyx_v_exc); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 840, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_3);
          if (__Pyx_PrintOne(0, __pyx_t_3) < 0) __PYX_ERR(0, 840, __pyx_L1_error)
          __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

          /* "pymarketcap/core.pyx":843
 *                         raise NotImplementedError(
 *                             "Exchange name catched but not found" \
 *                             + " in name index position (0)."             # <<<<<<<<<<<<<<
 *                         )
 * 
 */
          __pyx_t_3 = PyNumber_Add(__pyx_kp_s_Exchange_name_catched_but_not_fo, __pyx_kp_s_in_name_index_position_0); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 843, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_3);

          /* "pymarketcap/core.pyx":841
 *                     else:
 *                         print("exc = %s" % exc)
 *                         raise NotImplementedError(             # <<<<<<<<<<<<<<
 *                             "Exchange name catched but not found" \
 *                             + " in name index position (0)."
 */
          __pyx_t_2 = __Pyx_PyObject_CallOneArg(__pyx_builtin_NotImplementedError, __pyx_t_3); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 841, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_2);
          __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
          __Pyx_Raise(__pyx_t_2, 0, 0, 0);
          __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
          __PYX_ERR(0, 841, __pyx_L1_error)
        }

        /* "pymarketcap/core.pyx":832
 *         else:
 *             for exc in self._exchange_names_slugs_ids:
 *                 if name in exc:             # <<<<<<<<<<<<<<
 *                     exc_found = True
 *                     if exc.index(name) == 0:
 */
      }

      /* "pymarketcap/core.pyx":831
 *                         )
 *         else:
 *             for exc in self._exchange_names_slugs_ids:             # <<<<<<<<<<<<<<
 *                 if name in exc:
 *                     exc_found = True
 */
    }
    __pyx_L10_break:;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  }
  __pyx_L4:;

  /* "pymarketcap/core.pyx":846
 *                         )
 * 
 *         if not exc_found:             # <<<<<<<<<<<<<<
 *             raise ValueError("Exchange %s not found." % name)
 * 
 */
  __pyx_t_8 = ((!(__pyx_v_exc_found != 0)) != 0);
  if (unlikely(__pyx_t_8)) {

    /* "pymarketcap/core.pyx":847
 * 
 *         if not exc_found:
 *             raise ValueError("Exchange %s not found." % name)             # <<<<<<<<<<<<<<
 * 
 *         filename = "%s_%dx%d.png" % (_slug, size, size) if not filename \
 */
    __pyx_t_1 = __Pyx_PyString_Format(__pyx_kp_s_Exchange_s_not_found, __pyx_v_name); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 847, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_2 = __Pyx_PyObject_CallOneArg(__pyx_builtin_ValueError, __pyx_t_1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 847, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_Raise(__pyx_t_2, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __PYX_ERR(0, 847, __pyx_L1_error)

    /* "pymarketcap/core.pyx":846
 *                         )
 * 
 *         if not exc_found:             # <<<<<<<<<<<<<<
 *             raise ValueError("Exchange %s not found." % name)
 * 
 */
  }

  /* "pymarketcap/core.pyx":849
 *             raise ValueError("Exchange %s not found." % name)
 * 
 *         filename = "%s_%dx%d.png" % (_slug, size, size) if not filename \             # <<<<<<<<<<<<<<
 *             else "%s.png" % filename
 * 
 */
  __pyx_t_8 = __Pyx_PyObject_IsTrue(__pyx_v_filename); if (unlikely(__pyx_t_8 < 0)) __PYX_ERR(0, 849, __pyx_L1_error)
  if (((!__pyx_t_8) != 0)) {
    if (unlikely(!__pyx_v__slug)) { __Pyx_RaiseUnboundLocalError("_slug"); __PYX_ERR(0, 849, __pyx_L1_error) }
    __pyx_t_1 = PyTuple_New(3); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 849, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_INCREF(__pyx_v__slug);
    __Pyx_GIVEREF(__pyx_v__slug);
    PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_v__slug);
    __Pyx_INCREF(__pyx_v_size);
    __Pyx_GIVEREF(__pyx_v_size);
    PyTuple_SET_ITEM(__pyx_t_1, 1, __pyx_v_size);
    __Pyx_INCREF(__pyx_v_size);
    __Pyx_GIVEREF(__pyx_v_size);
    PyTuple_SET_ITEM(__pyx_t_1, 2, __pyx_v_size);
    __pyx_t_3 = __Pyx_PyString_Format(__pyx_kp_s_s__dx_d_png, __pyx_t_1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 849, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_2 = __pyx_t_3;
    __pyx_t_3 = 0;
  } else {

    /* "pymarketcap/core.pyx":850
 * 
 *         filename = "%s_%dx%d.png" % (_slug, size, size) if not filename \
 *             else "%s.png" % filename             # <<<<<<<<<<<<<<
 * 
 *         url = "https://s2.coinmarketcap.com/static/img/exchanges/%dx%d/%s.png" % \
 */
    __pyx_t_3 = __Pyx_PyString_Format(__pyx_kp_s_s_png, __pyx_v_filename); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 850, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_2 = __pyx_t_3;
    __pyx_t_3 = 0;
  }
  __Pyx_DECREF_SET(__pyx_v_filename, __pyx_t_2);
  __pyx_t_2 = 0;

  /* "pymarketcap/core.pyx":853
 * 
 *         url = "https://s2.coinmarketcap.com/static/img/exchanges/%dx%d/%s.png" % \
 *              (size, size, _id)             # <<<<<<<<<<<<<<
 * 
 *         try:
 */
  if (unlikely(!__pyx_v__id)) { __Pyx_RaiseUnboundLocalError("_id"); __PYX_ERR(0, 853, __pyx_L1_error) }
  __pyx_t_2 = PyTuple_New(3); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 853, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_INCREF(__pyx_v_size);
  __Pyx_GIVEREF(__pyx_v_size);
  PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_v_size);
  __Pyx_INCREF(__pyx_v_size);
  __Pyx_GIVEREF(__pyx_v_size);
  PyTuple_SET_ITEM(__pyx_t_2, 1, __pyx_v_size);
  __Pyx_INCREF(__pyx_v__id);
  __Pyx_GIVEREF(__pyx_v__id);
  PyTuple_SET_ITEM(__pyx_t_2, 2, __pyx_v__id);

  /* "pymarketcap/core.pyx":852
 *             else "%s.png" % filename
 * 
 *         url = "https://s2.coinmarketcap.com/static/img/exchanges/%dx%d/%s.png" % \             # <<<<<<<<<<<<<<
 *              (size, size, _id)
 * 
 */
  __pyx_t_3 = __Pyx_PyString_Format(__pyx_kp_s_https_s2_coinmarketcap_com_stati_2, __pyx_t_2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 852, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v_url = ((PyObject*)__pyx_t_3);
  __pyx_t_3 = 0;

  /* "pymarketcap/core.pyx":855
 *              (size, size, _id)
 * 
 *         try:             # <<<<<<<<<<<<<<
 *             res = urlretrieve(url, filename)
 *         except HTTPError as e:
 */
  {
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    __Pyx_ExceptionSave(&__pyx_t_10, &__pyx_t_11, &__pyx_t_12);
    __Pyx_XGOTREF(__pyx_t_10);
    __Pyx_XGOTREF(__pyx_t_11);
    __Pyx_XGOTREF(__pyx_t_12);
    /*try:*/ {

      /* "pymarketcap/core.pyx":856
 * 
 *         try:
 *             res = urlretrieve(url, filename)             # <<<<<<<<<<<<<<
 *         except HTTPError as e:
 *             if e.code == 403:
 */
      __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_urlretrieve); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 856, __pyx_L14_error)
      __Pyx_GOTREF(__pyx_t_2);
      __pyx_t_1 = NULL;
      __pyx_t_5 = 0;
      if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
        __pyx_t_1 = PyMethod_GET_SELF(__pyx_t_2);
        if (likely(__pyx_t_1)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
          __Pyx_INCREF(__pyx_t_1);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_2, function);
          __pyx_t_5 = 1;
        }
      }
      #if CYTHON_FAST_PYCALL
      if (PyFunction_Check(__pyx_t_2)) {
        PyObject *__pyx_temp[3] = {__pyx_t_1, __pyx_v_url, __pyx_v_filename};
        __pyx_t_3 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_5, 2+__pyx_t_5); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 856, __pyx_L14_error)
        __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
        __Pyx_GOTREF(__pyx_t_3);
      } else
      #endif
      #if CYTHON_FAST_PYCCALL
      if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
        PyObject *__pyx_temp[3] = {__pyx_t_1, __pyx_v_url, __pyx_v_filename};
        __pyx_t_3 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_5, 2+__pyx_t_5); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 856, __pyx_L14_error)
        __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
        __Pyx_GOTREF(__pyx_t_3);
      } else
      #endif
      {
        __pyx_t_6 = PyTuple_New(2+__pyx_t_5); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 856, __pyx_L14_error)
        __Pyx_GOTREF(__pyx_t_6);
        if (__pyx_t_1) {
          __Pyx_GIVEREF(__pyx_t_1); PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_1); __pyx_t_1 = NULL;
        }
        __Pyx_INCREF(__pyx_v_url);
        __Pyx_GIVEREF(__pyx_v_url);
        PyTuple_SET_ITEM(__pyx_t_6, 0+__pyx_t_5, __pyx_v_url);
        __Pyx_INCREF(__pyx_v_filename);
        __Pyx_GIVEREF(__pyx_v_filename);
        PyTuple_SET_ITEM(__pyx_t_6, 1+__pyx_t_5, __pyx_v_filename);
        __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_6, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 856, __pyx_L14_error)
        __Pyx_GOTREF(__pyx_t_3);
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      }
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __pyx_v_res = __pyx_t_3;
      __pyx_t_3 = 0;

      /* "pymarketcap/core.pyx":855
 *              (size, size, _id)
 * 
 *         try:             # <<<<<<<<<<<<<<
 *             res = urlretrieve(url, filename)
 *         except HTTPError as e:
 */
    }

    /* "pymarketcap/core.pyx":869
 *             raise e
 *         else:
 *             return filename             # <<<<<<<<<<<<<<
 * 
 * 
 */
    /*else:*/ {
      __Pyx_XDECREF(__pyx_r);
      __Pyx_INCREF(__pyx_v_filename);
      __pyx_r = __pyx_v_filename;
      goto __pyx_L17_except_return;
    }
    __pyx_L14_error:;
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "pymarketcap/core.pyx":857
 *         try:
 *             res = urlretrieve(url, filename)
 *         except HTTPError as e:             # <<<<<<<<<<<<<<
 *             if e.code == 403:
 *                 valid_sizes = [16, 32, 64, 128, 200]
 */
    __Pyx_ErrFetch(&__pyx_t_3, &__pyx_t_2, &__pyx_t_6);
    __pyx_t_1 = __Pyx_GetModuleGlobalName(__pyx_n_s_HTTPError); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 857, __pyx_L16_except_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_5 = __Pyx_PyErr_GivenExceptionMatches(__pyx_t_3, __pyx_t_1);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_ErrRestore(__pyx_t_3, __pyx_t_2, __pyx_t_6);
    __pyx_t_3 = 0; __pyx_t_2 = 0; __pyx_t_6 = 0;
    if (__pyx_t_5) {
      __Pyx_AddTraceback("pymarketcap.core.Pymarketcap.download_exchange_logo", __pyx_clineno, __pyx_lineno, __pyx_filename);
      if (__Pyx_GetException(&__pyx_t_6, &__pyx_t_2, &__pyx_t_3) < 0) __PYX_ERR(0, 857, __pyx_L16_except_error)
      __Pyx_GOTREF(__pyx_t_6);
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_2);
      __pyx_v_e = __pyx_t_2;

      /* "pymarketcap/core.pyx":858
 *             res = urlretrieve(url, filename)
 *         except HTTPError as e:
 *             if e.code == 403:             # <<<<<<<<<<<<<<
 *                 valid_sizes = [16, 32, 64, 128, 200]
 *                 if size in valid_sizes:
 */
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_e, __pyx_n_s_code); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 858, __pyx_L16_except_error)
      __Pyx_GOTREF(__pyx_t_1);
      __pyx_t_4 = __Pyx_PyInt_EqObjC(__pyx_t_1, __pyx_int_403, 0x193, 0); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 858, __pyx_L16_except_error)
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __pyx_t_8 = __Pyx_PyObject_IsTrue(__pyx_t_4); if (unlikely(__pyx_t_8 < 0)) __PYX_ERR(0, 858, __pyx_L16_except_error)
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (__pyx_t_8) {

        /* "pymarketcap/core.pyx":859
 *         except HTTPError as e:
 *             if e.code == 403:
 *                 valid_sizes = [16, 32, 64, 128, 200]             # <<<<<<<<<<<<<<
 *                 if size in valid_sizes:
 *                     raise ValueError(
 */
        __pyx_t_4 = PyList_New(5); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 859, __pyx_L16_except_error)
        __Pyx_GOTREF(__pyx_t_4);
        __Pyx_INCREF(__pyx_int_16);
        __Pyx_GIVEREF(__pyx_int_16);
        PyList_SET_ITEM(__pyx_t_4, 0, __pyx_int_16);
        __Pyx_INCREF(__pyx_int_32);
        __Pyx_GIVEREF(__pyx_int_32);
        PyList_SET_ITEM(__pyx_t_4, 1, __pyx_int_32);
        __Pyx_INCREF(__pyx_int_64);
        __Pyx_GIVEREF(__pyx_int_64);
        PyList_SET_ITEM(__pyx_t_4, 2, __pyx_int_64);
        __Pyx_INCREF(__pyx_int_128);
        __Pyx_GIVEREF(__pyx_int_128);
        PyList_SET_ITEM(__pyx_t_4, 3, __pyx_int_128);
        __Pyx_INCREF(__pyx_int_200);
        __Pyx_GIVEREF(__pyx_int_200);
        PyList_SET_ITEM(__pyx_t_4, 4, __pyx_int_200);
        __pyx_v_valid_sizes = ((PyObject*)__pyx_t_4);
        __pyx_t_4 = 0;

        /* "pymarketcap/core.pyx":860
 *             if e.code == 403:
 *                 valid_sizes = [16, 32, 64, 128, 200]
 *                 if size in valid_sizes:             # <<<<<<<<<<<<<<
 *                     raise ValueError(
 *                         ("Seems that %s exchange doesn't allows to be downloaded with " \
 */
        __pyx_t_8 = (__Pyx_PySequence_ContainsTF(__pyx_v_size, __pyx_v_valid_sizes, Py_EQ)); if (unlikely(__pyx_t_8 < 0)) __PYX_ERR(0, 860, __pyx_L16_except_error)
        __pyx_t_7 = (__pyx_t_8 != 0);
        if (unlikely(__pyx_t_7)) {

          /* "pymarketcap/core.pyx":863
 *                     raise ValueError(
 *                         ("Seems that %s exchange doesn't allows to be downloaded with " \
 *                         + "size %dx%d. Try another size.") % (name, size, size)             # <<<<<<<<<<<<<<
 *                     )
 *                 else:
 */
          __pyx_t_4 = PyNumber_Add(__pyx_kp_s_Seems_that_s_exchange_doesn_t_al, __pyx_kp_s_size_dx_d_Try_another_size); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 863, __pyx_L16_except_error)
          __Pyx_GOTREF(__pyx_t_4);
          __pyx_t_1 = PyTuple_New(3); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 863, __pyx_L16_except_error)
          __Pyx_GOTREF(__pyx_t_1);
          __Pyx_INCREF(__pyx_v_name);
          __Pyx_GIVEREF(__pyx_v_name);
          PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_v_name);
          __Pyx_INCREF(__pyx_v_size);
          __Pyx_GIVEREF(__pyx_v_size);
          PyTuple_SET_ITEM(__pyx_t_1, 1, __pyx_v_size);
          __Pyx_INCREF(__pyx_v_size);
          __Pyx_GIVEREF(__pyx_v_size);
          PyTuple_SET_ITEM(__pyx_t_1, 2, __pyx_v_size);
          __pyx_t_13 = __Pyx_PyString_Format(__pyx_t_4, __pyx_t_1); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 863, __pyx_L16_except_error)
          __Pyx_GOTREF(__pyx_t_13);
          __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
          __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

          /* "pymarketcap/core.pyx":861
 *                 valid_sizes = [16, 32, 64, 128, 200]
 *                 if size in valid_sizes:
 *                     raise ValueError(             # <<<<<<<<<<<<<<
 *                         ("Seems that %s exchange doesn't allows to be downloaded with " \
 *                         + "size %dx%d. Try another size.") % (name, size, size)
 */
          __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_builtin_ValueError, __pyx_t_13); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 861, __pyx_L16_except_error)
          __Pyx_GOTREF(__pyx_t_1);
          __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
          __Pyx_Raise(__pyx_t_1, 0, 0, 0);
          __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
          __PYX_ERR(0, 861, __pyx_L16_except_error)

          /* "pymarketcap/core.pyx":860
 *             if e.code == 403:
 *                 valid_sizes = [16, 32, 64, 128, 200]
 *                 if size in valid_sizes:             # <<<<<<<<<<<<<<
 *                     raise ValueError(
 *                         ("Seems that %s exchange doesn't allows to be downloaded with " \
 */
        }

        /* "pymarketcap/core.pyx":866
 *                     )
 *                 else:
 *                     raise ValueError("%dx%d is not a valid size." % (size, size))             # <<<<<<<<<<<<<<
 *             raise e
 *         else:
 */
        /*else*/ {
          __pyx_t_1 = PyTuple_New(2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 866, __pyx_L16_except_error)
          __Pyx_GOTREF(__pyx_t_1);
          __Pyx_INCREF(__pyx_v_size);
          __Pyx_GIVEREF(__pyx_v_size);
          PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_v_size);
          __Pyx_INCREF(__pyx_v_size);
          __Pyx_GIVEREF(__pyx_v_size);
          PyTuple_SET_ITEM(__pyx_t_1, 1, __pyx_v_size);
          __pyx_t_13 = __Pyx_PyString_Format(__pyx_kp_s_dx_d_is_not_a_valid_size, __pyx_t_1); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 866, __pyx_L16_except_error)
          __Pyx_GOTREF(__pyx_t_13);
          __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
          __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_builtin_ValueError, __pyx_t_13); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 866, __pyx_L16_except_error)
          __Pyx_GOTREF(__pyx_t_1);
          __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
          __Pyx_Raise(__pyx_t_1, 0, 0, 0);
          __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
          __PYX_ERR(0, 866, __pyx_L16_except_error)
        }

        /* "pymarketcap/core.pyx":858
 *             res = urlretrieve(url, filename)
 *         except HTTPError as e:
 *             if e.code == 403:             # <<<<<<<<<<<<<<
 *                 valid_sizes = [16, 32, 64, 128, 200]
 *                 if size in valid_sizes:
 */
      }

      /* "pymarketcap/core.pyx":867
 *                 else:
 *                     raise ValueError("%dx%d is not a valid size." % (size, size))
 *             raise e             # <<<<<<<<<<<<<<
 *         else:
 *             return filename
 */
      __Pyx_Raise(__pyx_v_e, 0, 0, 0);
      __PYX_ERR(0, 867, __pyx_L16_except_error)
    }
    goto __pyx_L16_except_error;
    __pyx_L16_except_error:;

    /* "pymarketcap/core.pyx":855
 *              (size, size, _id)
 * 
 *         try:             # <<<<<<<<<<<<<<
 *             res = urlretrieve(url, filename)
 *         except HTTPError as e:
 */
    __Pyx_XGIVEREF(__pyx_t_10);
    __Pyx_XGIVEREF(__pyx_t_11);
    __Pyx_XGIVEREF(__pyx_t_12);
    __Pyx_ExceptionReset(__pyx_t_10, __pyx_t_11, __pyx_t_12);
    goto __pyx_L1_error;
    __pyx_L17_except_return:;
    __Pyx_XGIVEREF(__pyx_t_10);
    __Pyx_XGIVEREF(__pyx_t_11);
    __Pyx_XGIVEREF(__pyx_t_12);
    __Pyx_ExceptionReset(__pyx_t_10, __pyx_t_11, __pyx_t_12);
    goto __pyx_L0;
  }

  /* "pymarketcap/core.pyx":793
 *             return filename
 * 
 *     cpdef download_exchange_logo(self, unicode name, size=64, filename=None):             # <<<<<<<<<<<<<<
 *         """Download a exchange logo passing his name or
 *         slug as first parameter and optionally a filename
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_13);
  __Pyx_AddTraceback("pymarketcap.core.Pymarketcap.download_exchange_logo", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_exc_names_slugs_ids);
  __Pyx_XDECREF(__pyx_v_exc);
  __Pyx_XDECREF(__pyx_v__id);
  __Pyx_XDECREF(__pyx_v__slug);
  __Pyx_XDECREF(__pyx_v__name);
  __Pyx_XDECREF(__pyx_v_url);
  __Pyx_XDECREF(__pyx_v_res);
  __Pyx_XDECREF(__pyx_v_e);
  __Pyx_XDECREF(__pyx_v_valid_sizes);
  __Pyx_XDECREF(__pyx_v_filename);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_11pymarketcap_4core_11Pymarketcap_45download_exchange_logo(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_11pymarketcap_4core_11Pymarketcap_44download_exchange_logo[] = "Download a exchange logo passing his name or\n        slug as first parameter and optionally a filename\n        without extension.\n\n        Args:\n            name (str): Exchange slug to download logo.\n            size (int): Size in pixels. Valid values are:\n                ``[16, 32, 64, 128, 200]``.\n            filename (str, optional): Filename for store the logo.\n                Doesn't include the extension (will be \".png\").\n                As default ``None``.\n\n        Returns (str):\n            Filename of downloaded file if all was correct.\n        ";
static PyObject *__pyx_pw_11pymarketcap_4core_11Pymarketcap_45download_exchange_logo(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_name = 0;
  PyObject *__pyx_v_size = 0;
  PyObject *__pyx_v_filename = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("download_exchange_logo (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_name,&__pyx_n_s_size,&__pyx_n_s_filename,0};
    PyObject* values[3] = {0,0,0};
    values[1] = ((PyObject *)__pyx_int_64);
    values[2] = ((PyObject *)Py_None);
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_name)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_size);
          if (value) { values[1] = value; kw_args--; }
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_filename);
          if (value) { values[2] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "download_exchange_logo") < 0)) __PYX_ERR(0, 793, __pyx_L3_error)
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_name = ((PyObject*)values[0]);
    __pyx_v_size = values[1];
    __pyx_v_filename = values[2];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("download_exchange_logo", 0, 1, 3, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 793, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("pymarketcap.core.Pymarketcap.download_exchange_logo", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_name), (&PyUnicode_Type), 1, "name", 1))) __PYX_ERR(0, 793, __pyx_L1_error)
  __pyx_r = __pyx_pf_11pymarketcap_4core_11Pymarketcap_44download_exchange_logo(((struct __pyx_obj_11pymarketcap_4core_Pymarketcap *)__pyx_v_self), __pyx_v_name, __pyx_v_size, __pyx_v_filename);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_11pymarketcap_4core_11Pymarketcap_44download_exchange_logo(struct __pyx_obj_11pymarketcap_4core_Pymarketcap *__pyx_v_self, PyObject *__pyx_v_name, PyObject *__pyx_v_size, PyObject *__pyx_v_filename) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  struct __pyx_opt_args_11pymarketcap_4core_11Pymarketcap_download_exchange_logo __pyx_t_2;
  __Pyx_RefNannySetupContext("download_exchange_logo", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2.__pyx_n = 2;
  __pyx_t_2.size = __pyx_v_size;
  __pyx_t_2.filename = __pyx_v_filename;
  __pyx_t_1 = __pyx_vtabptr_11pymarketcap_4core_Pymarketcap->download_exchange_logo(__pyx_v_self, __pyx_v_name, 1, &__pyx_t_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 793, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pymarketcap.core.Pymarketcap.download_exchange_logo", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pymarketcap/core.pyx":47
 *             As default, ``False``.
 *     """
 *     cdef readonly dict _correspondences             # <<<<<<<<<<<<<<
 *     cdef readonly dict _ids_correspondences
 *     cdef readonly list _symbols
 */

/* Python wrapper */
static PyObject *__pyx_pw_11pymarketcap_4core_11Pymarketcap_16_correspondences_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_11pymarketcap_4core_11Pymarketcap_16_correspondences_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_11pymarketcap_4core_11Pymarketcap_16_correspondences___get__(((struct __pyx_obj_11pymarketcap_4core_Pymarketcap *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_11pymarketcap_4core_11Pymarketcap_16_correspondences___get__(struct __pyx_obj_11pymarketcap_4core_Pymarketcap *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->_correspondences);
  __pyx_r = __pyx_v_self->_correspondences;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pymarketcap/core.pyx":48
 *     """
 *     cdef readonly dict _correspondences
 *     cdef readonly dict _ids_correspondences             # <<<<<<<<<<<<<<
 *     cdef readonly list _symbols
 *     cdef readonly list _coins
 */

/* Python wrapper */
static PyObject *__pyx_pw_11pymarketcap_4core_11Pymarketcap_20_ids_correspondences_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_11pymarketcap_4core_11Pymarketcap_20_ids_correspondences_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_11pymarketcap_4core_11Pymarketcap_20_ids_correspondences___get__(((struct __pyx_obj_11pymarketcap_4core_Pymarketcap *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_11pymarketcap_4core_11Pymarketcap_20_ids_correspondences___get__(struct __pyx_obj_11pymarketcap_4core_Pymarketcap *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->_ids_correspondences);
  __pyx_r = __pyx_v_self->_ids_correspondences;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pymarketcap/core.pyx":49
 *     cdef readonly dict _correspondences
 *     cdef readonly dict _ids_correspondences
 *     cdef readonly list _symbols             # <<<<<<<<<<<<<<
 *     cdef readonly list _coins
 *     cdef readonly int  _total_currencies
 */

/* Python wrapper */
static PyObject *__pyx_pw_11pymarketcap_4core_11Pymarketcap_8_symbols_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_11pymarketcap_4core_11Pymarketcap_8_symbols_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_11pymarketcap_4core_11Pymarketcap_8_symbols___get__(((struct __pyx_obj_11pymarketcap_4core_Pymarketcap *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_11pymarketcap_4core_11Pymarketcap_8_symbols___get__(struct __pyx_obj_11pymarketcap_4core_Pymarketcap *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->_symbols);
  __pyx_r = __pyx_v_self->_symbols;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pymarketcap/core.pyx":50
 *     cdef readonly dict _ids_correspondences
 *     cdef readonly list _symbols
 *     cdef readonly list _coins             # <<<<<<<<<<<<<<
 *     cdef readonly int  _total_currencies
 *     cdef readonly int  _total_exchanges
 */

/* Python wrapper */
static PyObject *__pyx_pw_11pymarketcap_4core_11Pymarketcap_6_coins_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_11pymarketcap_4core_11Pymarketcap_6_coins_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_11pymarketcap_4core_11Pymarketcap_6_coins___get__(((struct __pyx_obj_11pymarketcap_4core_Pymarketcap *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_11pymarketcap_4core_11Pymarketcap_6_coins___get__(struct __pyx_obj_11pymarketcap_4core_Pymarketcap *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->_coins);
  __pyx_r = __pyx_v_self->_coins;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pymarketcap/core.pyx":51
 *     cdef readonly list _symbols
 *     cdef readonly list _coins
 *     cdef readonly int  _total_currencies             # <<<<<<<<<<<<<<
 *     cdef readonly int  _total_exchanges
 *     cdef readonly list _currencies_to_convert
 */

/* Python wrapper */
static PyObject *__pyx_pw_11pymarketcap_4core_11Pymarketcap_17_total_currencies_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_11pymarketcap_4core_11Pymarketcap_17_total_currencies_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_11pymarketcap_4core_11Pymarketcap_17_total_currencies___get__(((struct __pyx_obj_11pymarketcap_4core_Pymarketcap *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_11pymarketcap_4core_11Pymarketcap_17_total_currencies___get__(struct __pyx_obj_11pymarketcap_4core_Pymarketcap *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_From_int(__pyx_v_self->_total_currencies); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 51, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pymarketcap.core.Pymarketcap._total_currencies.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pymarketcap/core.pyx":52
 *     cdef readonly list _coins
 *     cdef readonly int  _total_currencies
 *     cdef readonly int  _total_exchanges             # <<<<<<<<<<<<<<
 *     cdef readonly list _currencies_to_convert
 *     cdef readonly list _converter_cache
 */

/* Python wrapper */
static PyObject *__pyx_pw_11pymarketcap_4core_11Pymarketcap_16_total_exchanges_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_11pymarketcap_4core_11Pymarketcap_16_total_exchanges_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_11pymarketcap_4core_11Pymarketcap_16_total_exchanges___get__(((struct __pyx_obj_11pymarketcap_4core_Pymarketcap *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_11pymarketcap_4core_11Pymarketcap_16_total_exchanges___get__(struct __pyx_obj_11pymarketcap_4core_Pymarketcap *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_From_int(__pyx_v_self->_total_exchanges); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 52, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pymarketcap.core.Pymarketcap._total_exchanges.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pymarketcap/core.pyx":53
 *     cdef readonly int  _total_currencies
 *     cdef readonly int  _total_exchanges
 *     cdef readonly list _currencies_to_convert             # <<<<<<<<<<<<<<
 *     cdef readonly list _converter_cache
 *     cdef readonly list _exchange_names
 */

/* Python wrapper */
static PyObject *__pyx_pw_11pymarketcap_4core_11Pymarketcap_22_currencies_to_convert_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_11pymarketcap_4core_11Pymarketcap_22_currencies_to_convert_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_11pymarketcap_4core_11Pymarketcap_22_currencies_to_convert___get__(((struct __pyx_obj_11pymarketcap_4core_Pymarketcap *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_11pymarketcap_4core_11Pymarketcap_22_currencies_to_convert___get__(struct __pyx_obj_11pymarketcap_4core_Pymarketcap *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->_currencies_to_convert);
  __pyx_r = __pyx_v_self->_currencies_to_convert;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pymarketcap/core.pyx":54
 *     cdef readonly int  _total_exchanges
 *     cdef readonly list _currencies_to_convert
 *     cdef readonly list _converter_cache             # <<<<<<<<<<<<<<
 *     cdef readonly list _exchange_names
 *     cdef readonly list _exchange_slugs
 */

/* Python wrapper */
static PyObject *__pyx_pw_11pymarketcap_4core_11Pymarketcap_16_converter_cache_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_11pymarketcap_4core_11Pymarketcap_16_converter_cache_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_11pymarketcap_4core_11Pymarketcap_16_converter_cache___get__(((struct __pyx_obj_11pymarketcap_4core_Pymarketcap *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_11pymarketcap_4core_11Pymarketcap_16_converter_cache___get__(struct __pyx_obj_11pymarketcap_4core_Pymarketcap *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->_converter_cache);
  __pyx_r = __pyx_v_self->_converter_cache;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pymarketcap/core.pyx":55
 *     cdef readonly list _currencies_to_convert
 *     cdef readonly list _converter_cache
 *     cdef readonly list _exchange_names             # <<<<<<<<<<<<<<
 *     cdef readonly list _exchange_slugs
 *     cdef readonly dict __repeated_symbols
 */

/* Python wrapper */
static PyObject *__pyx_pw_11pymarketcap_4core_11Pymarketcap_15_exchange_names_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_11pymarketcap_4core_11Pymarketcap_15_exchange_names_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_11pymarketcap_4core_11Pymarketcap_15_exchange_names___get__(((struct __pyx_obj_11pymarketcap_4core_Pymarketcap *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_11pymarketcap_4core_11Pymarketcap_15_exchange_names___get__(struct __pyx_obj_11pymarketcap_4core_Pymarketcap *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->_exchange_names);
  __pyx_r = __pyx_v_self->_exchange_names;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pymarketcap/core.pyx":56
 *     cdef readonly list _converter_cache
 *     cdef readonly list _exchange_names
 *     cdef readonly list _exchange_slugs             # <<<<<<<<<<<<<<
 *     cdef readonly dict __repeated_symbols
 *     cdef readonly list _exchange_names_slugs_ids
 */

/* Python wrapper */
static PyObject *__pyx_pw_11pymarketcap_4core_11Pymarketcap_15_exchange_slugs_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_11pymarketcap_4core_11Pymarketcap_15_exchange_slugs_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_11pymarketcap_4core_11Pymarketcap_15_exchange_slugs___get__(((struct __pyx_obj_11pymarketcap_4core_Pymarketcap *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_11pymarketcap_4core_11Pymarketcap_15_exchange_slugs___get__(struct __pyx_obj_11pymarketcap_4core_Pymarketcap *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->_exchange_slugs);
  __pyx_r = __pyx_v_self->_exchange_slugs;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pymarketcap/core.pyx":57
 *     cdef readonly list _exchange_names
 *     cdef readonly list _exchange_slugs
 *     cdef readonly dict __repeated_symbols             # <<<<<<<<<<<<<<
 *     cdef readonly list _exchange_names_slugs_ids
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_11pymarketcap_4core_11Pymarketcap_18__repeated_symbols_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_11pymarketcap_4core_11Pymarketcap_18__repeated_symbols_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_11pymarketcap_4core_11Pymarketcap_18__repeated_symbols___get__(((struct __pyx_obj_11pymarketcap_4core_Pymarketcap *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_11pymarketcap_4core_11Pymarketcap_18__repeated_symbols___get__(struct __pyx_obj_11pymarketcap_4core_Pymarketcap *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->__repeated_symbols);
  __pyx_r = __pyx_v_self->__repeated_symbols;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pymarketcap/core.pyx":58
 *     cdef readonly list _exchange_slugs
 *     cdef readonly dict __repeated_symbols
 *     cdef readonly list _exchange_names_slugs_ids             # <<<<<<<<<<<<<<
 * 
 *     cdef public long timeout
 */

/* Python wrapper */
static PyObject *__pyx_pw_11pymarketcap_4core_11Pymarketcap_25_exchange_names_slugs_ids_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_11pymarketcap_4core_11Pymarketcap_25_exchange_names_slugs_ids_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_11pymarketcap_4core_11Pymarketcap_25_exchange_names_slugs_ids___get__(((struct __pyx_obj_11pymarketcap_4core_Pymarketcap *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_11pymarketcap_4core_11Pymarketcap_25_exchange_names_slugs_ids___get__(struct __pyx_obj_11pymarketcap_4core_Pymarketcap *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->_exchange_names_slugs_ids);
  __pyx_r = __pyx_v_self->_exchange_names_slugs_ids;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pymarketcap/core.pyx":60
 *     cdef readonly list _exchange_names_slugs_ids
 * 
 *     cdef public long timeout             # <<<<<<<<<<<<<<
 *     cdef public object graphs
 *     cdef public bint debug
 */

/* Python wrapper */
static PyObject *__pyx_pw_11pymarketcap_4core_11Pymarketcap_7timeout_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_11pymarketcap_4core_11Pymarketcap_7timeout_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_11pymarketcap_4core_11Pymarketcap_7timeout___get__(((struct __pyx_obj_11pymarketcap_4core_Pymarketcap *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_11pymarketcap_4core_11Pymarketcap_7timeout___get__(struct __pyx_obj_11pymarketcap_4core_Pymarketcap *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_From_long(__pyx_v_self->timeout); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 60, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pymarketcap.core.Pymarketcap.timeout.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_11pymarketcap_4core_11Pymarketcap_7timeout_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_11pymarketcap_4core_11Pymarketcap_7timeout_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_11pymarketcap_4core_11Pymarketcap_7timeout_2__set__(((struct __pyx_obj_11pymarketcap_4core_Pymarketcap *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_11pymarketcap_4core_11Pymarketcap_7timeout_2__set__(struct __pyx_obj_11pymarketcap_4core_Pymarketcap *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  long __pyx_t_1;
  __Pyx_RefNannySetupContext("__set__", 0);
  __pyx_t_1 = __Pyx_PyInt_As_long(__pyx_v_value); if (unlikely((__pyx_t_1 == (long)-1) && PyErr_Occurred())) __PYX_ERR(0, 60, __pyx_L1_error)
  __pyx_v_self->timeout = __pyx_t_1;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("pymarketcap.core.Pymarketcap.timeout.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pymarketcap/core.pyx":61
 * 
 *     cdef public long timeout
 *     cdef public object graphs             # <<<<<<<<<<<<<<
 *     cdef public bint debug
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_11pymarketcap_4core_11Pymarketcap_6graphs_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_11pymarketcap_4core_11Pymarketcap_6graphs_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_11pymarketcap_4core_11Pymarketcap_6graphs___get__(((struct __pyx_obj_11pymarketcap_4core_Pymarketcap *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_11pymarketcap_4core_11Pymarketcap_6graphs___get__(struct __pyx_obj_11pymarketcap_4core_Pymarketcap *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->graphs);
  __pyx_r = __pyx_v_self->graphs;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_11pymarketcap_4core_11Pymarketcap_6graphs_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_11pymarketcap_4core_11Pymarketcap_6graphs_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_11pymarketcap_4core_11Pymarketcap_6graphs_2__set__(((struct __pyx_obj_11pymarketcap_4core_Pymarketcap *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_11pymarketcap_4core_11Pymarketcap_6graphs_2__set__(struct __pyx_obj_11pymarketcap_4core_Pymarketcap *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__", 0);
  __Pyx_INCREF(__pyx_v_value);
  __Pyx_GIVEREF(__pyx_v_value);
  __Pyx_GOTREF(__pyx_v_self->graphs);
  __Pyx_DECREF(__pyx_v_self->graphs);
  __pyx_v_self->graphs = __pyx_v_value;

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_11pymarketcap_4core_11Pymarketcap_6graphs_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_11pymarketcap_4core_11Pymarketcap_6graphs_5__del__(PyObject *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_11pymarketcap_4core_11Pymarketcap_6graphs_4__del__(((struct __pyx_obj_11pymarketcap_4core_Pymarketcap *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_11pymarketcap_4core_11Pymarketcap_6graphs_4__del__(struct __pyx_obj_11pymarketcap_4core_Pymarketcap *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__", 0);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->graphs);
  __Pyx_DECREF(__pyx_v_self->graphs);
  __pyx_v_self->graphs = Py_None;

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "pymarketcap/core.pyx":62
 *     cdef public long timeout
 *     cdef public object graphs
 *     cdef public bint debug             # <<<<<<<<<<<<<<
 * 
 *     def __init__(self, timeout=DEFAULT_TIMEOUT, debug=False):
 */

/* Python wrapper */
static PyObject *__pyx_pw_11pymarketcap_4core_11Pymarketcap_5debug_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_11pymarketcap_4core_11Pymarketcap_5debug_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_11pymarketcap_4core_11Pymarketcap_5debug___get__(((struct __pyx_obj_11pymarketcap_4core_Pymarketcap *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_11pymarketcap_4core_11Pymarketcap_5debug___get__(struct __pyx_obj_11pymarketcap_4core_Pymarketcap *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyBool_FromLong(__pyx_v_self->debug); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 62, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pymarketcap.core.Pymarketcap.debug.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_11pymarketcap_4core_11Pymarketcap_5debug_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_11pymarketcap_4core_11Pymarketcap_5debug_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_11pymarketcap_4core_11Pymarketcap_5debug_2__set__(((struct __pyx_obj_11pymarketcap_4core_Pymarketcap *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_11pymarketcap_4core_11Pymarketcap_5debug_2__set__(struct __pyx_obj_11pymarketcap_4core_Pymarketcap *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  __Pyx_RefNannySetupContext("__set__", 0);
  __pyx_t_1 = __Pyx_PyObject_IsTrue(__pyx_v_value); if (unlikely((__pyx_t_1 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 62, __pyx_L1_error)
  __pyx_v_self->debug = __pyx_t_1;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("pymarketcap.core.Pymarketcap.debug.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef bint use_setstate
 *     state = (self.__repeated_symbols, self._coins, self._converter_cache, self._correspondences, self._currencies_to_convert, self._exchange_names, self._exchange_names_slugs_ids, self._exchange_slugs, self._ids_correspondences, self._symbols, self._total_currencies, self._total_exchanges, self.debug, self.graphs, self.timeout)
 */

/* Python wrapper */
static PyObject *__pyx_pw_11pymarketcap_4core_11Pymarketcap_47__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_11pymarketcap_4core_11Pymarketcap_47__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_11pymarketcap_4core_11Pymarketcap_46__reduce_cython__(((struct __pyx_obj_11pymarketcap_4core_Pymarketcap *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_11pymarketcap_4core_11Pymarketcap_46__reduce_cython__(struct __pyx_obj_11pymarketcap_4core_Pymarketcap *__pyx_v_self) {
  int __pyx_v_use_setstate;
  PyObject *__pyx_v_state = NULL;
  PyObject *__pyx_v__dict = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_t_6;
  int __pyx_t_7;
  int __pyx_t_8;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     cdef bint use_setstate
 *     state = (self.__repeated_symbols, self._coins, self._converter_cache, self._correspondences, self._currencies_to_convert, self._exchange_names, self._exchange_names_slugs_ids, self._exchange_slugs, self._ids_correspondences, self._symbols, self._total_currencies, self._total_exchanges, self.debug, self.graphs, self.timeout)             # <<<<<<<<<<<<<<
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(__pyx_v_self->_total_currencies); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 3, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyInt_From_int(__pyx_v_self->_total_exchanges); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 3, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyBool_FromLong(__pyx_v_self->debug); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 3, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = __Pyx_PyInt_From_long(__pyx_v_self->timeout); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 3, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_5 = PyTuple_New(15); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 3, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_INCREF(__pyx_v_self->__repeated_symbols);
  __Pyx_GIVEREF(__pyx_v_self->__repeated_symbols);
  PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_v_self->__repeated_symbols);
  __Pyx_INCREF(__pyx_v_self->_coins);
  __Pyx_GIVEREF(__pyx_v_self->_coins);
  PyTuple_SET_ITEM(__pyx_t_5, 1, __pyx_v_self->_coins);
  __Pyx_INCREF(__pyx_v_self->_converter_cache);
  __Pyx_GIVEREF(__pyx_v_self->_converter_cache);
  PyTuple_SET_ITEM(__pyx_t_5, 2, __pyx_v_self->_converter_cache);
  __Pyx_INCREF(__pyx_v_self->_correspondences);
  __Pyx_GIVEREF(__pyx_v_self->_correspondences);
  PyTuple_SET_ITEM(__pyx_t_5, 3, __pyx_v_self->_correspondences);
  __Pyx_INCREF(__pyx_v_self->_currencies_to_convert);
  __Pyx_GIVEREF(__pyx_v_self->_currencies_to_convert);
  PyTuple_SET_ITEM(__pyx_t_5, 4, __pyx_v_self->_currencies_to_convert);
  __Pyx_INCREF(__pyx_v_self->_exchange_names);
  __Pyx_GIVEREF(__pyx_v_self->_exchange_names);
  PyTuple_SET_ITEM(__pyx_t_5, 5, __pyx_v_self->_exchange_names);
  __Pyx_INCREF(__pyx_v_self->_exchange_names_slugs_ids);
  __Pyx_GIVEREF(__pyx_v_self->_exchange_names_slugs_ids);
  PyTuple_SET_ITEM(__pyx_t_5, 6, __pyx_v_self->_exchange_names_slugs_ids);
  __Pyx_INCREF(__pyx_v_self->_exchange_slugs);
  __Pyx_GIVEREF(__pyx_v_self->_exchange_slugs);
  PyTuple_SET_ITEM(__pyx_t_5, 7, __pyx_v_self->_exchange_slugs);
  __Pyx_INCREF(__pyx_v_self->_ids_correspondences);
  __Pyx_GIVEREF(__pyx_v_self->_ids_correspondences);
  PyTuple_SET_ITEM(__pyx_t_5, 8, __pyx_v_self->_ids_correspondences);
  __Pyx_INCREF(__pyx_v_self->_symbols);
  __Pyx_GIVEREF(__pyx_v_self->_symbols);
  PyTuple_SET_ITEM(__pyx_t_5, 9, __pyx_v_self->_symbols);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_5, 10, __pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_5, 11, __pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_5, 12, __pyx_t_3);
  __Pyx_INCREF(__pyx_v_self->graphs);
  __Pyx_GIVEREF(__pyx_v_self->graphs);
  PyTuple_SET_ITEM(__pyx_t_5, 13, __pyx_v_self->graphs);
  __Pyx_GIVEREF(__pyx_t_4);
  PyTuple_SET_ITEM(__pyx_t_5, 14, __pyx_t_4);
  __pyx_t_1 = 0;
  __pyx_t_2 = 0;
  __pyx_t_3 = 0;
  __pyx_t_4 = 0;
  __pyx_v_state = ((PyObject*)__pyx_t_5);
  __pyx_t_5 = 0;

  /* "(tree fragment)":4
 *     cdef bint use_setstate
 *     state = (self.__repeated_symbols, self._coins, self._converter_cache, self._correspondences, self._currencies_to_convert, self._exchange_names, self._exchange_names_slugs_ids, self._exchange_slugs, self._ids_correspondences, self._symbols, self._total_currencies, self._total_exchanges, self.debug, self.graphs, self.timeout)
 *     _dict = getattr(self, '__dict__', None)             # <<<<<<<<<<<<<<
 *     if _dict is not None:
 *         state += (_dict,)
 */
  __pyx_t_5 = __Pyx_GetAttr3(((PyObject *)__pyx_v_self), __pyx_n_s_dict, Py_None); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_v__dict = __pyx_t_5;
  __pyx_t_5 = 0;

  /* "(tree fragment)":5
 *     state = (self.__repeated_symbols, self._coins, self._converter_cache, self._correspondences, self._currencies_to_convert, self._exchange_names, self._exchange_names_slugs_ids, self._exchange_slugs, self._ids_correspondences, self._symbols, self._total_currencies, self._total_exchanges, self.debug, self.graphs, self.timeout)
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
 */
  __pyx_t_6 = (__pyx_v__dict != Py_None);
  __pyx_t_7 = (__pyx_t_6 != 0);
  if (__pyx_t_7) {

    /* "(tree fragment)":6
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:
 *         state += (_dict,)             # <<<<<<<<<<<<<<
 *         use_setstate = True
 *     else:
 */
    __pyx_t_5 = PyTuple_New(1); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_INCREF(__pyx_v__dict);
    __Pyx_GIVEREF(__pyx_v__dict);
    PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_v__dict);
    __pyx_t_4 = PyNumber_InPlaceAdd(__pyx_v_state, __pyx_t_5); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_DECREF_SET(__pyx_v_state, ((PyObject*)__pyx_t_4));
    __pyx_t_4 = 0;

    /* "(tree fragment)":7
 *     if _dict is not None:
 *         state += (_dict,)
 *         use_setstate = True             # <<<<<<<<<<<<<<
 *     else:
 *         use_setstate = self.__repeated_symbols is not None or self._coins is not None or self._converter_cache is not None or self._correspondences is not None or self._currencies_to_convert is not None or self._exchange_names is not None or self._exchange_names_slugs_ids is not None or self._exchange_slugs is not None or self._ids_correspondences is not None or self._symbols is not None or self.graphs is not None
 */
    __pyx_v_use_setstate = 1;

    /* "(tree fragment)":5
 *     state = (self.__repeated_symbols, self._coins, self._converter_cache, self._correspondences, self._currencies_to_convert, self._exchange_names, self._exchange_names_slugs_ids, self._exchange_slugs, self._ids_correspondences, self._symbols, self._total_currencies, self._total_exchanges, self.debug, self.graphs, self.timeout)
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
 */
    goto __pyx_L3;
  }

  /* "(tree fragment)":9
 *         use_setstate = True
 *     else:
 *         use_setstate = self.__repeated_symbols is not None or self._coins is not None or self._converter_cache is not None or self._correspondences is not None or self._currencies_to_convert is not None or self._exchange_names is not None or self._exchange_names_slugs_ids is not None or self._exchange_slugs is not None or self._ids_correspondences is not None or self._symbols is not None or self.graphs is not None             # <<<<<<<<<<<<<<
 *     if use_setstate:
 *         return __pyx_unpickle_Pymarketcap, (type(self), 0x7d79c3c, None), state
 */
  /*else*/ {
    __pyx_t_6 = (__pyx_v_self->__repeated_symbols != ((PyObject*)Py_None));
    __pyx_t_8 = (__pyx_t_6 != 0);
    if (!__pyx_t_8) {
    } else {
      __pyx_t_7 = __pyx_t_8;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_8 = (__pyx_v_self->_coins != ((PyObject*)Py_None));
    __pyx_t_6 = (__pyx_t_8 != 0);
    if (!__pyx_t_6) {
    } else {
      __pyx_t_7 = __pyx_t_6;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_6 = (__pyx_v_self->_converter_cache != ((PyObject*)Py_None));
    __pyx_t_8 = (__pyx_t_6 != 0);
    if (!__pyx_t_8) {
    } else {
      __pyx_t_7 = __pyx_t_8;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_8 = (__pyx_v_self->_correspondences != ((PyObject*)Py_None));
    __pyx_t_6 = (__pyx_t_8 != 0);
    if (!__pyx_t_6) {
    } else {
      __pyx_t_7 = __pyx_t_6;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_6 = (__pyx_v_self->_currencies_to_convert != ((PyObject*)Py_None));
    __pyx_t_8 = (__pyx_t_6 != 0);
    if (!__pyx_t_8) {
    } else {
      __pyx_t_7 = __pyx_t_8;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_8 = (__pyx_v_self->_exchange_names != ((PyObject*)Py_None));
    __pyx_t_6 = (__pyx_t_8 != 0);
    if (!__pyx_t_6) {
    } else {
      __pyx_t_7 = __pyx_t_6;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_6 = (__pyx_v_self->_exchange_names_slugs_ids != ((PyObject*)Py_None));
    __pyx_t_8 = (__pyx_t_6 != 0);
    if (!__pyx_t_8) {
    } else {
      __pyx_t_7 = __pyx_t_8;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_8 = (__pyx_v_self->_exchange_slugs != ((PyObject*)Py_None));
    __pyx_t_6 = (__pyx_t_8 != 0);
    if (!__pyx_t_6) {
    } else {
      __pyx_t_7 = __pyx_t_6;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_6 = (__pyx_v_self->_ids_correspondences != ((PyObject*)Py_None));
    __pyx_t_8 = (__pyx_t_6 != 0);
    if (!__pyx_t_8) {
    } else {
      __pyx_t_7 = __pyx_t_8;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_8 = (__pyx_v_self->_symbols != ((PyObject*)Py_None));
    __pyx_t_6 = (__pyx_t_8 != 0);
    if (!__pyx_t_6) {
    } else {
      __pyx_t_7 = __pyx_t_6;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_6 = (__pyx_v_self->graphs != Py_None);
    __pyx_t_8 = (__pyx_t_6 != 0);
    __pyx_t_7 = __pyx_t_8;
    __pyx_L4_bool_binop_done:;
    __pyx_v_use_setstate = __pyx_t_7;
  }
  __pyx_L3:;

  /* "(tree fragment)":10
 *     else:
 *         use_setstate = self.__repeated_symbols is not None or self._coins is not None or self._converter_cache is not None or self._correspondences is not None or self._currencies_to_convert is not None or self._exchange_names is not None or self._exchange_names_slugs_ids is not None or self._exchange_slugs is not None or self._ids_correspondences is not None or self._symbols is not None or self.graphs is not None
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle_Pymarketcap, (type(self), 0x7d79c3c, None), state
 *     else:
 */
  __pyx_t_7 = (__pyx_v_use_setstate != 0);
  if (__pyx_t_7) {

    /* "(tree fragment)":11
 *         use_setstate = self.__repeated_symbols is not None or self._coins is not None or self._converter_cache is not None or self._correspondences is not None or self._currencies_to_convert is not None or self._exchange_names is not None or self._exchange_names_slugs_ids is not None or self._exchange_slugs is not None or self._ids_correspondences is not None or self._symbols is not None or self.graphs is not None
 *     if use_setstate:
 *         return __pyx_unpickle_Pymarketcap, (type(self), 0x7d79c3c, None), state             # <<<<<<<<<<<<<<
 *     else:
 *         return __pyx_unpickle_Pymarketcap, (type(self), 0x7d79c3c, state)
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_4 = __Pyx_GetModuleGlobalName(__pyx_n_s_pyx_unpickle_Pymarketcap); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 11, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_5 = PyTuple_New(3); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 11, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    PyTuple_SET_ITEM(__pyx_t_5, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_INCREF(__pyx_int_131570748);
    __Pyx_GIVEREF(__pyx_int_131570748);
    PyTuple_SET_ITEM(__pyx_t_5, 1, __pyx_int_131570748);
    __Pyx_INCREF(Py_None);
    __Pyx_GIVEREF(Py_None);
    PyTuple_SET_ITEM(__pyx_t_5, 2, Py_None);
    __pyx_t_3 = PyTuple_New(3); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 11, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_GIVEREF(__pyx_t_4);
    PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_4);
    __Pyx_GIVEREF(__pyx_t_5);
    PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_t_5);
    __Pyx_INCREF(__pyx_v_state);
    __Pyx_GIVEREF(__pyx_v_state);
    PyTuple_SET_ITEM(__pyx_t_3, 2, __pyx_v_state);
    __pyx_t_4 = 0;
    __pyx_t_5 = 0;
    __pyx_r = __pyx_t_3;
    __pyx_t_3 = 0;
    goto __pyx_L0;

    /* "(tree fragment)":10
 *     else:
 *         use_setstate = self.__repeated_symbols is not None or self._coins is not None or self._converter_cache is not None or self._correspondences is not None or self._currencies_to_convert is not None or self._exchange_names is not None or self._exchange_names_slugs_ids is not None or self._exchange_slugs is not None or self._ids_correspondences is not None or self._symbols is not None or self.graphs is not None
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle_Pymarketcap, (type(self), 0x7d79c3c, None), state
 *     else:
 */
  }

  /* "(tree fragment)":13
 *         return __pyx_unpickle_Pymarketcap, (type(self), 0x7d79c3c, None), state
 *     else:
 *         return __pyx_unpickle_Pymarketcap, (type(self), 0x7d79c3c, state)             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle_Pymarketcap__set_state(self, __pyx_state)
 */
  /*else*/ {
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_3 = __Pyx_GetModuleGlobalName(__pyx_n_s_pyx_unpickle_Pymarketcap); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_5 = PyTuple_New(3); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    PyTuple_SET_ITEM(__pyx_t_5, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_INCREF(__pyx_int_131570748);
    __Pyx_GIVEREF(__pyx_int_131570748);
    PyTuple_SET_ITEM(__pyx_t_5, 1, __pyx_int_131570748);
    __Pyx_INCREF(__pyx_v_state);
    __Pyx_GIVEREF(__pyx_v_state);
    PyTuple_SET_ITEM(__pyx_t_5, 2, __pyx_v_state);
    __pyx_t_4 = PyTuple_New(2); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_GIVEREF(__pyx_t_3);
    PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_3);
    __Pyx_GIVEREF(__pyx_t_5);
    PyTuple_SET_ITEM(__pyx_t_4, 1, __pyx_t_5);
    __pyx_t_3 = 0;
    __pyx_t_5 = 0;
    __pyx_r = __pyx_t_4;
    __pyx_t_4 = 0;
    goto __pyx_L0;
  }

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef bint use_setstate
 *     state = (self.__repeated_symbols, self._coins, self._converter_cache, self._correspondences, self._currencies_to_convert, self._exchange_names, self._exchange_names_slugs_ids, self._exchange_slugs, self._ids_correspondences, self._symbols, self._total_currencies, self._total_exchanges, self.debug, self.graphs, self.timeout)
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("pymarketcap.core.Pymarketcap.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_state);
  __Pyx_XDECREF(__pyx_v__dict);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":14
 *     else:
 *         return __pyx_unpickle_Pymarketcap, (type(self), 0x7d79c3c, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_Pymarketcap__set_state(self, __pyx_state)
 */

/* Python wrapper */
static PyObject *__pyx_pw_11pymarketcap_4core_11Pymarketcap_49__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state); /*proto*/
static PyObject *__pyx_pw_11pymarketcap_4core_11Pymarketcap_49__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_11pymarketcap_4core_11Pymarketcap_48__setstate_cython__(((struct __pyx_obj_11pymarketcap_4core_Pymarketcap *)__pyx_v_self), ((PyObject *)__pyx_v___pyx_state));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_11pymarketcap_4core_11Pymarketcap_48__setstate_cython__(struct __pyx_obj_11pymarketcap_4core_Pymarketcap *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":15
 *         return __pyx_unpickle_Pymarketcap, (type(self), 0x7d79c3c, state)
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle_Pymarketcap__set_state(self, __pyx_state)             # <<<<<<<<<<<<<<
 */
  if (!(likely(PyTuple_CheckExact(__pyx_v___pyx_state))||((__pyx_v___pyx_state) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "tuple", Py_TYPE(__pyx_v___pyx_state)->tp_name), 0))) __PYX_ERR(1, 15, __pyx_L1_error)
  __pyx_t_1 = __pyx_f_11pymarketcap_4core___pyx_unpickle_Pymarketcap__set_state(__pyx_v_self, ((PyObject*)__pyx_v___pyx_state)); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 15, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "(tree fragment)":14
 *     else:
 *         return __pyx_unpickle_Pymarketcap, (type(self), 0x7d79c3c, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_Pymarketcap__set_state(self, __pyx_state)
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("pymarketcap.core.Pymarketcap.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __pyx_unpickle_Pymarketcap(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     if __pyx_checksum != 0x7d79c3c:
 *         from pickle import PickleError as __pyx_PickleError
 */

/* Python wrapper */
static PyObject *__pyx_pw_11pymarketcap_4core_1__pyx_unpickle_Pymarketcap(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_mdef_11pymarketcap_4core_1__pyx_unpickle_Pymarketcap = {"__pyx_unpickle_Pymarketcap", (PyCFunction)__pyx_pw_11pymarketcap_4core_1__pyx_unpickle_Pymarketcap, METH_VARARGS|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_11pymarketcap_4core_1__pyx_unpickle_Pymarketcap(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v___pyx_type = 0;
  long __pyx_v___pyx_checksum;
  PyObject *__pyx_v___pyx_state = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__pyx_unpickle_Pymarketcap (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_pyx_type,&__pyx_n_s_pyx_checksum,&__pyx_n_s_pyx_state,0};
    PyObject* values[3] = {0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_type)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_checksum)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_Pymarketcap", 1, 3, 3, 1); __PYX_ERR(1, 1, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_state)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_Pymarketcap", 1, 3, 3, 2); __PYX_ERR(1, 1, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__pyx_unpickle_Pymarketcap") < 0)) __PYX_ERR(1, 1, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 3) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
    }
    __pyx_v___pyx_type = values[0];
    __pyx_v___pyx_checksum = __Pyx_PyInt_As_long(values[1]); if (unlikely((__pyx_v___pyx_checksum == (long)-1) && PyErr_Occurred())) __PYX_ERR(1, 1, __pyx_L3_error)
    __pyx_v___pyx_state = values[2];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_Pymarketcap", 1, 3, 3, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(1, 1, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("pymarketcap.core.__pyx_unpickle_Pymarketcap", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_11pymarketcap_4core___pyx_unpickle_Pymarketcap(__pyx_self, __pyx_v___pyx_type, __pyx_v___pyx_checksum, __pyx_v___pyx_state);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_11pymarketcap_4core___pyx_unpickle_Pymarketcap(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_v___pyx_PickleError = NULL;
  PyObject *__pyx_v___pyx_result = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_t_7;
  __Pyx_RefNannySetupContext("__pyx_unpickle_Pymarketcap", 0);

  /* "(tree fragment)":2
 * def __pyx_unpickle_Pymarketcap(__pyx_type, long __pyx_checksum, __pyx_state):
 *     if __pyx_checksum != 0x7d79c3c:             # <<<<<<<<<<<<<<
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x7d79c3c = (__repeated_symbols, _coins, _converter_cache, _correspondences, _currencies_to_convert, _exchange_names, _exchange_names_slugs_ids, _exchange_slugs, _ids_correspondences, _symbols, _total_currencies, _total_exchanges, debug, graphs, timeout))" % __pyx_checksum)
 */
  __pyx_t_1 = ((__pyx_v___pyx_checksum != 0x7d79c3c) != 0);
  if (__pyx_t_1) {

    /* "(tree fragment)":3
 * def __pyx_unpickle_Pymarketcap(__pyx_type, long __pyx_checksum, __pyx_state):
 *     if __pyx_checksum != 0x7d79c3c:
 *         from pickle import PickleError as __pyx_PickleError             # <<<<<<<<<<<<<<
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x7d79c3c = (__repeated_symbols, _coins, _converter_cache, _correspondences, _currencies_to_convert, _exchange_names, _exchange_names_slugs_ids, _exchange_slugs, _ids_correspondences, _symbols, _total_currencies, _total_exchanges, debug, graphs, timeout))" % __pyx_checksum)
 *     __pyx_result = Pymarketcap.__new__(__pyx_type)
 */
    __pyx_t_2 = PyList_New(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 3, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_n_s_PickleError);
    __Pyx_GIVEREF(__pyx_n_s_PickleError);
    PyList_SET_ITEM(__pyx_t_2, 0, __pyx_n_s_PickleError);
    __pyx_t_3 = __Pyx_Import(__pyx_n_s_pickle, __pyx_t_2, -1); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 3, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_2 = __Pyx_ImportFrom(__pyx_t_3, __pyx_n_s_PickleError); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 3, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_t_2);
    __pyx_v___pyx_PickleError = __pyx_t_2;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "(tree fragment)":4
 *     if __pyx_checksum != 0x7d79c3c:
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x7d79c3c = (__repeated_symbols, _coins, _converter_cache, _correspondences, _currencies_to_convert, _exchange_names, _exchange_names_slugs_ids, _exchange_slugs, _ids_correspondences, _symbols, _total_currencies, _total_exchanges, debug, graphs, timeout))" % __pyx_checksum)             # <<<<<<<<<<<<<<
 *     __pyx_result = Pymarketcap.__new__(__pyx_type)
 *     if __pyx_state is not None:
 */
    __pyx_t_2 = __Pyx_PyInt_From_long(__pyx_v___pyx_checksum); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 4, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_4 = __Pyx_PyString_Format(__pyx_kp_s_Incompatible_checksums_s_vs_0x7d, __pyx_t_2); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 4, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_INCREF(__pyx_v___pyx_PickleError);
    __pyx_t_2 = __pyx_v___pyx_PickleError; __pyx_t_5 = NULL;
    if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
      __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_2);
      if (likely(__pyx_t_5)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
        __Pyx_INCREF(__pyx_t_5);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_2, function);
      }
    }
    if (!__pyx_t_5) {
      __pyx_t_3 = __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_4); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 4, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_GOTREF(__pyx_t_3);
    } else {
      #if CYTHON_FAST_PYCALL
      if (PyFunction_Check(__pyx_t_2)) {
        PyObject *__pyx_temp[2] = {__pyx_t_5, __pyx_t_4};
        __pyx_t_3 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 4, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
        __Pyx_GOTREF(__pyx_t_3);
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      } else
      #endif
      #if CYTHON_FAST_PYCCALL
      if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
        PyObject *__pyx_temp[2] = {__pyx_t_5, __pyx_t_4};
        __pyx_t_3 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 4, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
        __Pyx_GOTREF(__pyx_t_3);
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      } else
      #endif
      {
        __pyx_t_6 = PyTuple_New(1+1); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 4, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_6);
        __Pyx_GIVEREF(__pyx_t_5); PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_5); __pyx_t_5 = NULL;
        __Pyx_GIVEREF(__pyx_t_4);
        PyTuple_SET_ITEM(__pyx_t_6, 0+1, __pyx_t_4);
        __pyx_t_4 = 0;
        __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_6, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 4, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_3);
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      }
    }
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_Raise(__pyx_t_3, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __PYX_ERR(1, 4, __pyx_L1_error)

    /* "(tree fragment)":2
 * def __pyx_unpickle_Pymarketcap(__pyx_type, long __pyx_checksum, __pyx_state):
 *     if __pyx_checksum != 0x7d79c3c:             # <<<<<<<<<<<<<<
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x7d79c3c = (__repeated_symbols, _coins, _converter_cache, _correspondences, _currencies_to_convert, _exchange_names, _exchange_names_slugs_ids, _exchange_slugs, _ids_correspondences, _symbols, _total_currencies, _total_exchanges, debug, graphs, timeout))" % __pyx_checksum)
 */
  }

  /* "(tree fragment)":5
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x7d79c3c = (__repeated_symbols, _coins, _converter_cache, _correspondences, _currencies_to_convert, _exchange_names, _exchange_names_slugs_ids, _exchange_slugs, _ids_correspondences, _symbols, _total_currencies, _total_exchanges, debug, graphs, timeout))" % __pyx_checksum)
 *     __pyx_result = Pymarketcap.__new__(__pyx_type)             # <<<<<<<<<<<<<<
 *     if __pyx_state is not None:
 *         __pyx_unpickle_Pymarketcap__set_state(<Pymarketcap> __pyx_result, __pyx_state)
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_ptype_11pymarketcap_4core_Pymarketcap), __pyx_n_s_new); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_6 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_6)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_6);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  if (!__pyx_t_6) {
    __pyx_t_3 = __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_v___pyx_type); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
  } else {
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(__pyx_t_2)) {
      PyObject *__pyx_temp[2] = {__pyx_t_6, __pyx_v___pyx_type};
      __pyx_t_3 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 5, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
      __Pyx_GOTREF(__pyx_t_3);
    } else
    #endif
    #if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
      PyObject *__pyx_temp[2] = {__pyx_t_6, __pyx_v___pyx_type};
      __pyx_t_3 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 5, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
      __Pyx_GOTREF(__pyx_t_3);
    } else
    #endif
    {
      __pyx_t_4 = PyTuple_New(1+1); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 5, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_GIVEREF(__pyx_t_6); PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_6); __pyx_t_6 = NULL;
      __Pyx_INCREF(__pyx_v___pyx_type);
      __Pyx_GIVEREF(__pyx_v___pyx_type);
      PyTuple_SET_ITEM(__pyx_t_4, 0+1, __pyx_v___pyx_type);
      __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_4, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 5, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    }
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v___pyx_result = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "(tree fragment)":6
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x7d79c3c = (__repeated_symbols, _coins, _converter_cache, _correspondences, _currencies_to_convert, _exchange_names, _exchange_names_slugs_ids, _exchange_slugs, _ids_correspondences, _symbols, _total_currencies, _total_exchanges, debug, graphs, timeout))" % __pyx_checksum)
 *     __pyx_result = Pymarketcap.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle_Pymarketcap__set_state(<Pymarketcap> __pyx_result, __pyx_state)
 *     return __pyx_result
 */
  __pyx_t_1 = (__pyx_v___pyx_state != Py_None);
  __pyx_t_7 = (__pyx_t_1 != 0);
  if (__pyx_t_7) {

    /* "(tree fragment)":7
 *     __pyx_result = Pymarketcap.__new__(__pyx_type)
 *     if __pyx_state is not None:
 *         __pyx_unpickle_Pymarketcap__set_state(<Pymarketcap> __pyx_result, __pyx_state)             # <<<<<<<<<<<<<<
 *     return __pyx_result
 * cdef __pyx_unpickle_Pymarketcap__set_state(Pymarketcap __pyx_result, tuple __pyx_state):
 */
    if (!(likely(PyTuple_CheckExact(__pyx_v___pyx_state))||((__pyx_v___pyx_state) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "tuple", Py_TYPE(__pyx_v___pyx_state)->tp_name), 0))) __PYX_ERR(1, 7, __pyx_L1_error)
    __pyx_t_3 = __pyx_f_11pymarketcap_4core___pyx_unpickle_Pymarketcap__set_state(((struct __pyx_obj_11pymarketcap_4core_Pymarketcap *)__pyx_v___pyx_result), ((PyObject*)__pyx_v___pyx_state)); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 7, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "(tree fragment)":6
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x7d79c3c = (__repeated_symbols, _coins, _converter_cache, _correspondences, _currencies_to_convert, _exchange_names, _exchange_names_slugs_ids, _exchange_slugs, _ids_correspondences, _symbols, _total_currencies, _total_exchanges, debug, graphs, timeout))" % __pyx_checksum)
 *     __pyx_result = Pymarketcap.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle_Pymarketcap__set_state(<Pymarketcap> __pyx_result, __pyx_state)
 *     return __pyx_result
 */
  }

  /* "(tree fragment)":8
 *     if __pyx_state is not None:
 *         __pyx_unpickle_Pymarketcap__set_state(<Pymarketcap> __pyx_result, __pyx_state)
 *     return __pyx_result             # <<<<<<<<<<<<<<
 * cdef __pyx_unpickle_Pymarketcap__set_state(Pymarketcap __pyx_result, tuple __pyx_state):
 *     __pyx_result.__repeated_symbols = __pyx_state[0]; __pyx_result._coins = __pyx_state[1]; __pyx_result._converter_cache = __pyx_state[2]; __pyx_result._correspondences = __pyx_state[3]; __pyx_result._currencies_to_convert = __pyx_state[4]; __pyx_result._exchange_names = __pyx_state[5]; __pyx_result._exchange_names_slugs_ids = __pyx_state[6]; __pyx_result._exchange_slugs = __pyx_state[7]; __pyx_result._ids_correspondences = __pyx_state[8]; __pyx_result._symbols = __pyx_state[9]; __pyx_result._total_currencies = __pyx_state[10]; __pyx_result._total_exchanges = __pyx_state[11]; __pyx_result.debug = __pyx_state[12]; __pyx_result.graphs = __pyx_state[13]; __pyx_result.timeout = __pyx_state[14]
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v___pyx_result);
  __pyx_r = __pyx_v___pyx_result;
  goto __pyx_L0;

  /* "(tree fragment)":1
 * def __pyx_unpickle_Pymarketcap(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     if __pyx_checksum != 0x7d79c3c:
 *         from pickle import PickleError as __pyx_PickleError
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_AddTraceback("pymarketcap.core.__pyx_unpickle_Pymarketcap", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v___pyx_PickleError);
  __Pyx_XDECREF(__pyx_v___pyx_result);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":9
 *         __pyx_unpickle_Pymarketcap__set_state(<Pymarketcap> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_Pymarketcap__set_state(Pymarketcap __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result.__repeated_symbols = __pyx_state[0]; __pyx_result._coins = __pyx_state[1]; __pyx_result._converter_cache = __pyx_state[2]; __pyx_result._correspondences = __pyx_state[3]; __pyx_result._currencies_to_convert = __pyx_state[4]; __pyx_result._exchange_names = __pyx_state[5]; __pyx_result._exchange_names_slugs_ids = __pyx_state[6]; __pyx_result._exchange_slugs = __pyx_state[7]; __pyx_result._ids_correspondences = __pyx_state[8]; __pyx_result._symbols = __pyx_state[9]; __pyx_result._total_currencies = __pyx_state[10]; __pyx_result._total_exchanges = __pyx_state[11]; __pyx_result.debug = __pyx_state[12]; __pyx_result.graphs = __pyx_state[13]; __pyx_result.timeout = __pyx_state[14]
 *     if len(__pyx_state) > 15 and hasattr(__pyx_result, '__dict__'):
 */

static PyObject *__pyx_f_11pymarketcap_4core___pyx_unpickle_Pymarketcap__set_state(struct __pyx_obj_11pymarketcap_4core_Pymarketcap *__pyx_v___pyx_result, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  int __pyx_t_3;
  long __pyx_t_4;
  Py_ssize_t __pyx_t_5;
  int __pyx_t_6;
  int __pyx_t_7;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  PyObject *__pyx_t_10 = NULL;
  PyObject *__pyx_t_11 = NULL;
  __Pyx_RefNannySetupContext("__pyx_unpickle_Pymarketcap__set_state", 0);

  /* "(tree fragment)":10
 *     return __pyx_result
 * cdef __pyx_unpickle_Pymarketcap__set_state(Pymarketcap __pyx_result, tuple __pyx_state):
 *     __pyx_result.__repeated_symbols = __pyx_state[0]; __pyx_result._coins = __pyx_state[1]; __pyx_result._converter_cache = __pyx_state[2]; __pyx_result._correspondences = __pyx_state[3]; __pyx_result._currencies_to_convert = __pyx_state[4]; __pyx_result._exchange_names = __pyx_state[5]; __pyx_result._exchange_names_slugs_ids = __pyx_state[6]; __pyx_result._exchange_slugs = __pyx_state[7]; __pyx_result._ids_correspondences = __pyx_state[8]; __pyx_result._symbols = __pyx_state[9]; __pyx_result._total_currencies = __pyx_state[10]; __pyx_result._total_exchanges = __pyx_state[11]; __pyx_result.debug = __pyx_state[12]; __pyx_result.graphs = __pyx_state[13]; __pyx_result.timeout = __pyx_state[14]             # <<<<<<<<<<<<<<
 *     if len(__pyx_state) > 15 and hasattr(__pyx_result, '__dict__'):
 *         __pyx_result.__dict__.update(__pyx_state[15])
 */
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 10, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 0, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 10, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyDict_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "dict", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(1, 10, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__repeated_symbols);
  __Pyx_DECREF(__pyx_v___pyx_result->__repeated_symbols);
  __pyx_v___pyx_result->__repeated_symbols = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 10, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 1, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 10, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyList_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "list", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(1, 10, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->_coins);
  __Pyx_DECREF(__pyx_v___pyx_result->_coins);
  __pyx_v___pyx_result->_coins = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 10, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 2, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 10, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyList_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "list", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(1, 10, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->_converter_cache);
  __Pyx_DECREF(__pyx_v___pyx_result->_converter_cache);
  __pyx_v___pyx_result->_converter_cache = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 10, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 3, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 10, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyDict_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "dict", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(1, 10, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->_correspondences);
  __Pyx_DECREF(__pyx_v___pyx_result->_correspondences);
  __pyx_v___pyx_result->_correspondences = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 10, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 4, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 10, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyList_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "list", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(1, 10, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->_currencies_to_convert);
  __Pyx_DECREF(__pyx_v___pyx_result->_currencies_to_convert);
  __pyx_v___pyx_result->_currencies_to_convert = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 10, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 5, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 10, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyList_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "list", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(1, 10, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->_exchange_names);
  __Pyx_DECREF(__pyx_v___pyx_result->_exchange_names);
  __pyx_v___pyx_result->_exchange_names = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 10, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 6, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 10, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyList_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "list", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(1, 10, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->_exchange_names_slugs_ids);
  __Pyx_DECREF(__pyx_v___pyx_result->_exchange_names_slugs_ids);
  __pyx_v___pyx_result->_exchange_names_slugs_ids = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 10, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 7, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 10, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyList_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "list", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(1, 10, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->_exchange_slugs);
  __Pyx_DECREF(__pyx_v___pyx_result->_exchange_slugs);
  __pyx_v___pyx_result->_exchange_slugs = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 10, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 8, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 10, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyDict_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "dict", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(1, 10, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->_ids_correspondences);
  __Pyx_DECREF(__pyx_v___pyx_result->_ids_correspondences);
  __pyx_v___pyx_result->_ids_correspondences = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 10, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 9, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 10, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyList_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "list", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(1, 10, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->_symbols);
  __Pyx_DECREF(__pyx_v___pyx_result->_symbols);
  __pyx_v___pyx_result->_symbols = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 10, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 10, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 10, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyInt_As_int(__pyx_t_1); if (unlikely((__pyx_t_2 == (int)-1) && PyErr_Occurred())) __PYX_ERR(1, 10, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->_total_currencies = __pyx_t_2;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 10, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 11, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 10, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyInt_As_int(__pyx_t_1); if (unlikely((__pyx_t_2 == (int)-1) && PyErr_Occurred())) __PYX_ERR(1, 10, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->_total_exchanges = __pyx_t_2;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 10, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 12, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 10, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely((__pyx_t_3 == (int)-1) && PyErr_Occurred())) __PYX_ERR(1, 10, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->debug = __pyx_t_3;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 10, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 13, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 10, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->graphs);
  __Pyx_DECREF(__pyx_v___pyx_result->graphs);
  __pyx_v___pyx_result->graphs = __pyx_t_1;
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 10, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 14, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 10, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_4 = __Pyx_PyInt_As_long(__pyx_t_1); if (unlikely((__pyx_t_4 == (long)-1) && PyErr_Occurred())) __PYX_ERR(1, 10, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->timeout = __pyx_t_4;

  /* "(tree fragment)":11
 * cdef __pyx_unpickle_Pymarketcap__set_state(Pymarketcap __pyx_result, tuple __pyx_state):
 *     __pyx_result.__repeated_symbols = __pyx_state[0]; __pyx_result._coins = __pyx_state[1]; __pyx_result._converter_cache = __pyx_state[2]; __pyx_result._correspondences = __pyx_state[3]; __pyx_result._currencies_to_convert = __pyx_state[4]; __pyx_result._exchange_names = __pyx_state[5]; __pyx_result._exchange_names_slugs_ids = __pyx_state[6]; __pyx_result._exchange_slugs = __pyx_state[7]; __pyx_result._ids_correspondences = __pyx_state[8]; __pyx_result._symbols = __pyx_state[9]; __pyx_result._total_currencies = __pyx_state[10]; __pyx_result._total_exchanges = __pyx_state[11]; __pyx_result.debug = __pyx_state[12]; __pyx_result.graphs = __pyx_state[13]; __pyx_result.timeout = __pyx_state[14]
 *     if len(__pyx_state) > 15 and hasattr(__pyx_result, '__dict__'):             # <<<<<<<<<<<<<<
 *         __pyx_result.__dict__.update(__pyx_state[15])
 */
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
    __PYX_ERR(1, 11, __pyx_L1_error)
  }
  __pyx_t_5 = PyTuple_GET_SIZE(__pyx_v___pyx_state); if (unlikely(__pyx_t_5 == ((Py_ssize_t)-1))) __PYX_ERR(1, 11, __pyx_L1_error)
  __pyx_t_6 = ((__pyx_t_5 > 15) != 0);
  if (__pyx_t_6) {
  } else {
    __pyx_t_3 = __pyx_t_6;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_6 = __Pyx_HasAttr(((PyObject *)__pyx_v___pyx_result), __pyx_n_s_dict); if (unlikely(__pyx_t_6 == ((int)-1))) __PYX_ERR(1, 11, __pyx_L1_error)
  __pyx_t_7 = (__pyx_t_6 != 0);
  __pyx_t_3 = __pyx_t_7;
  __pyx_L4_bool_binop_done:;
  if (__pyx_t_3) {

    /* "(tree fragment)":12
 *     __pyx_result.__repeated_symbols = __pyx_state[0]; __pyx_result._coins = __pyx_state[1]; __pyx_result._converter_cache = __pyx_state[2]; __pyx_result._correspondences = __pyx_state[3]; __pyx_result._currencies_to_convert = __pyx_state[4]; __pyx_result._exchange_names = __pyx_state[5]; __pyx_result._exchange_names_slugs_ids = __pyx_state[6]; __pyx_result._exchange_slugs = __pyx_state[7]; __pyx_result._ids_correspondences = __pyx_state[8]; __pyx_result._symbols = __pyx_state[9]; __pyx_result._total_currencies = __pyx_state[10]; __pyx_result._total_exchanges = __pyx_state[11]; __pyx_result.debug = __pyx_state[12]; __pyx_result.graphs = __pyx_state[13]; __pyx_result.timeout = __pyx_state[14]
 *     if len(__pyx_state) > 15 and hasattr(__pyx_result, '__dict__'):
 *         __pyx_result.__dict__.update(__pyx_state[15])             # <<<<<<<<<<<<<<
 */
    __pyx_t_8 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v___pyx_result), __pyx_n_s_dict); if (unlikely(!__pyx_t_8)) __PYX_ERR(1, 12, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __pyx_t_9 = __Pyx_PyObject_GetAttrStr(__pyx_t_8, __pyx_n_s_update); if (unlikely(!__pyx_t_9)) __PYX_ERR(1, 12, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_9);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    if (unlikely(__pyx_v___pyx_state == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      __PYX_ERR(1, 12, __pyx_L1_error)
    }
    __pyx_t_8 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 15, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_8)) __PYX_ERR(1, 12, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __pyx_t_10 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_9))) {
      __pyx_t_10 = PyMethod_GET_SELF(__pyx_t_9);
      if (likely(__pyx_t_10)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_9);
        __Pyx_INCREF(__pyx_t_10);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_9, function);
      }
    }
    if (!__pyx_t_10) {
      __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_9, __pyx_t_8); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      __Pyx_GOTREF(__pyx_t_1);
    } else {
      #if CYTHON_FAST_PYCALL
      if (PyFunction_Check(__pyx_t_9)) {
        PyObject *__pyx_temp[2] = {__pyx_t_10, __pyx_t_8};
        __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_9, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
        __Pyx_GOTREF(__pyx_t_1);
        __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      } else
      #endif
      #if CYTHON_FAST_PYCCALL
      if (__Pyx_PyFastCFunction_Check(__pyx_t_9)) {
        PyObject *__pyx_temp[2] = {__pyx_t_10, __pyx_t_8};
        __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_9, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
        __Pyx_GOTREF(__pyx_t_1);
        __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      } else
      #endif
      {
        __pyx_t_11 = PyTuple_New(1+1); if (unlikely(!__pyx_t_11)) __PYX_ERR(1, 12, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_11);
        __Pyx_GIVEREF(__pyx_t_10); PyTuple_SET_ITEM(__pyx_t_11, 0, __pyx_t_10); __pyx_t_10 = NULL;
        __Pyx_GIVEREF(__pyx_t_8);
        PyTuple_SET_ITEM(__pyx_t_11, 0+1, __pyx_t_8);
        __pyx_t_8 = 0;
        __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_9, __pyx_t_11, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_1);
        __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
      }
    }
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "(tree fragment)":11
 * cdef __pyx_unpickle_Pymarketcap__set_state(Pymarketcap __pyx_result, tuple __pyx_state):
 *     __pyx_result.__repeated_symbols = __pyx_state[0]; __pyx_result._coins = __pyx_state[1]; __pyx_result._converter_cache = __pyx_state[2]; __pyx_result._correspondences = __pyx_state[3]; __pyx_result._currencies_to_convert = __pyx_state[4]; __pyx_result._exchange_names = __pyx_state[5]; __pyx_result._exchange_names_slugs_ids = __pyx_state[6]; __pyx_result._exchange_slugs = __pyx_state[7]; __pyx_result._ids_correspondences = __pyx_state[8]; __pyx_result._symbols = __pyx_state[9]; __pyx_result._total_currencies = __pyx_state[10]; __pyx_result._total_exchanges = __pyx_state[11]; __pyx_result.debug = __pyx_state[12]; __pyx_result.graphs = __pyx_state[13]; __pyx_result.timeout = __pyx_state[14]
 *     if len(__pyx_state) > 15 and hasattr(__pyx_result, '__dict__'):             # <<<<<<<<<<<<<<
 *         __pyx_result.__dict__.update(__pyx_state[15])
 */
  }

  /* "(tree fragment)":9
 *         __pyx_unpickle_Pymarketcap__set_state(<Pymarketcap> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_Pymarketcap__set_state(Pymarketcap __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result.__repeated_symbols = __pyx_state[0]; __pyx_result._coins = __pyx_state[1]; __pyx_result._converter_cache = __pyx_state[2]; __pyx_result._correspondences = __pyx_state[3]; __pyx_result._currencies_to_convert = __pyx_state[4]; __pyx_result._exchange_names = __pyx_state[5]; __pyx_result._exchange_names_slugs_ids = __pyx_state[6]; __pyx_result._exchange_slugs = __pyx_state[7]; __pyx_result._ids_correspondences = __pyx_state[8]; __pyx_result._symbols = __pyx_state[9]; __pyx_result._total_currencies = __pyx_state[10]; __pyx_result._total_exchanges = __pyx_state[11]; __pyx_result.debug = __pyx_state[12]; __pyx_result.graphs = __pyx_state[13]; __pyx_result.timeout = __pyx_state[14]
 *     if len(__pyx_state) > 15 and hasattr(__pyx_result, '__dict__'):
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_XDECREF(__pyx_t_10);
  __Pyx_XDECREF(__pyx_t_11);
  __Pyx_AddTraceback("pymarketcap.core.__pyx_unpickle_Pymarketcap__set_state", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}
static struct __pyx_vtabstruct_11pymarketcap_4core_Pymarketcap __pyx_vtable_11pymarketcap_4core_Pymarketcap;

static PyObject *__pyx_tp_new_11pymarketcap_4core_Pymarketcap(PyTypeObject *t, CYTHON_UNUSED PyObject *a, CYTHON_UNUSED PyObject *k) {
  struct __pyx_obj_11pymarketcap_4core_Pymarketcap *p;
  PyObject *o;
  if (likely((t->tp_flags & Py_TPFLAGS_IS_ABSTRACT) == 0)) {
    o = (*t->tp_alloc)(t, 0);
  } else {
    o = (PyObject *) PyBaseObject_Type.tp_new(t, __pyx_empty_tuple, 0);
  }
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_11pymarketcap_4core_Pymarketcap *)o);
  p->__pyx_vtab = __pyx_vtabptr_11pymarketcap_4core_Pymarketcap;
  p->_correspondences = ((PyObject*)Py_None); Py_INCREF(Py_None);
  p->_ids_correspondences = ((PyObject*)Py_None); Py_INCREF(Py_None);
  p->_symbols = ((PyObject*)Py_None); Py_INCREF(Py_None);
  p->_coins = ((PyObject*)Py_None); Py_INCREF(Py_None);
  p->_currencies_to_convert = ((PyObject*)Py_None); Py_INCREF(Py_None);
  p->_converter_cache = ((PyObject*)Py_None); Py_INCREF(Py_None);
  p->_exchange_names = ((PyObject*)Py_None); Py_INCREF(Py_None);
  p->_exchange_slugs = ((PyObject*)Py_None); Py_INCREF(Py_None);
  p->__repeated_symbols = ((PyObject*)Py_None); Py_INCREF(Py_None);
  p->_exchange_names_slugs_ids = ((PyObject*)Py_None); Py_INCREF(Py_None);
  p->graphs = Py_None; Py_INCREF(Py_None);
  return o;
}

static void __pyx_tp_dealloc_11pymarketcap_4core_Pymarketcap(PyObject *o) {
  struct __pyx_obj_11pymarketcap_4core_Pymarketcap *p = (struct __pyx_obj_11pymarketcap_4core_Pymarketcap *)o;
  #if CYTHON_USE_TP_FINALIZE
  if (unlikely(PyType_HasFeature(Py_TYPE(o), Py_TPFLAGS_HAVE_FINALIZE) && Py_TYPE(o)->tp_finalize) && !_PyGC_FINALIZED(o)) {
    if (PyObject_CallFinalizerFromDealloc(o)) return;
  }
  #endif
  PyObject_GC_UnTrack(o);
  Py_CLEAR(p->_correspondences);
  Py_CLEAR(p->_ids_correspondences);
  Py_CLEAR(p->_symbols);
  Py_CLEAR(p->_coins);
  Py_CLEAR(p->_currencies_to_convert);
  Py_CLEAR(p->_converter_cache);
  Py_CLEAR(p->_exchange_names);
  Py_CLEAR(p->_exchange_slugs);
  Py_CLEAR(p->__repeated_symbols);
  Py_CLEAR(p->_exchange_names_slugs_ids);
  Py_CLEAR(p->graphs);
  (*Py_TYPE(o)->tp_free)(o);
}

static int __pyx_tp_traverse_11pymarketcap_4core_Pymarketcap(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_11pymarketcap_4core_Pymarketcap *p = (struct __pyx_obj_11pymarketcap_4core_Pymarketcap *)o;
  if (p->_correspondences) {
    e = (*v)(p->_correspondences, a); if (e) return e;
  }
  if (p->_ids_correspondences) {
    e = (*v)(p->_ids_correspondences, a); if (e) return e;
  }
  if (p->_symbols) {
    e = (*v)(p->_symbols, a); if (e) return e;
  }
  if (p->_coins) {
    e = (*v)(p->_coins, a); if (e) return e;
  }
  if (p->_currencies_to_convert) {
    e = (*v)(p->_currencies_to_convert, a); if (e) return e;
  }
  if (p->_converter_cache) {
    e = (*v)(p->_converter_cache, a); if (e) return e;
  }
  if (p->_exchange_names) {
    e = (*v)(p->_exchange_names, a); if (e) return e;
  }
  if (p->_exchange_slugs) {
    e = (*v)(p->_exchange_slugs, a); if (e) return e;
  }
  if (p->__repeated_symbols) {
    e = (*v)(p->__repeated_symbols, a); if (e) return e;
  }
  if (p->_exchange_names_slugs_ids) {
    e = (*v)(p->_exchange_names_slugs_ids, a); if (e) return e;
  }
  if (p->graphs) {
    e = (*v)(p->graphs, a); if (e) return e;
  }
  return 0;
}

static int __pyx_tp_clear_11pymarketcap_4core_Pymarketcap(PyObject *o) {
  PyObject* tmp;
  struct __pyx_obj_11pymarketcap_4core_Pymarketcap *p = (struct __pyx_obj_11pymarketcap_4core_Pymarketcap *)o;
  tmp = ((PyObject*)p->_correspondences);
  p->_correspondences = ((PyObject*)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->_ids_correspondences);
  p->_ids_correspondences = ((PyObject*)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->_symbols);
  p->_symbols = ((PyObject*)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->_coins);
  p->_coins = ((PyObject*)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->_currencies_to_convert);
  p->_currencies_to_convert = ((PyObject*)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->_converter_cache);
  p->_converter_cache = ((PyObject*)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->_exchange_names);
  p->_exchange_names = ((PyObject*)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->_exchange_slugs);
  p->_exchange_slugs = ((PyObject*)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->__repeated_symbols);
  p->__repeated_symbols = ((PyObject*)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->_exchange_names_slugs_ids);
  p->_exchange_names_slugs_ids = ((PyObject*)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->graphs);
  p->graphs = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  return 0;
}

static PyObject *__pyx_getprop_11pymarketcap_4core_11Pymarketcap__graphs_interface(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_11pymarketcap_4core_11Pymarketcap_17_graphs_interface_1__get__(o);
}

static PyObject *__pyx_getprop_11pymarketcap_4core_11Pymarketcap_correspondences(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_11pymarketcap_4core_11Pymarketcap_15correspondences_1__get__(o);
}

static PyObject *__pyx_getprop_11pymarketcap_4core_11Pymarketcap_ids_correspondences(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_11pymarketcap_4core_11Pymarketcap_19ids_correspondences_1__get__(o);
}

static PyObject *__pyx_getprop_11pymarketcap_4core_11Pymarketcap_symbols(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_11pymarketcap_4core_11Pymarketcap_7symbols_1__get__(o);
}

static PyObject *__pyx_getprop_11pymarketcap_4core_11Pymarketcap_coins(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_11pymarketcap_4core_11Pymarketcap_5coins_1__get__(o);
}

static PyObject *__pyx_getprop_11pymarketcap_4core_11Pymarketcap_total_currencies(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_11pymarketcap_4core_11Pymarketcap_16total_currencies_1__get__(o);
}

static PyObject *__pyx_getprop_11pymarketcap_4core_11Pymarketcap_currencies_to_convert(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_11pymarketcap_4core_11Pymarketcap_21currencies_to_convert_1__get__(o);
}

static PyObject *__pyx_getprop_11pymarketcap_4core_11Pymarketcap_exchange_names(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_11pymarketcap_4core_11Pymarketcap_14exchange_names_1__get__(o);
}

static PyObject *__pyx_getprop_11pymarketcap_4core_11Pymarketcap_exchange_slugs(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_11pymarketcap_4core_11Pymarketcap_14exchange_slugs_1__get__(o);
}

static PyObject *__pyx_getprop_11pymarketcap_4core_11Pymarketcap_total_exchanges(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_11pymarketcap_4core_11Pymarketcap_15total_exchanges_1__get__(o);
}

static PyObject *__pyx_getprop_11pymarketcap_4core_11Pymarketcap_converter_cache(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_11pymarketcap_4core_11Pymarketcap_15converter_cache_1__get__(o);
}

static PyObject *__pyx_getprop_11pymarketcap_4core_11Pymarketcap_ticker_badges(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_11pymarketcap_4core_11Pymarketcap_13ticker_badges_1__get__(o);
}

static PyObject *__pyx_getprop_11pymarketcap_4core_11Pymarketcap_currency_exchange_rates(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_11pymarketcap_4core_11Pymarketcap_23currency_exchange_rates_1__get__(o);
}

static PyObject *__pyx_getprop_11pymarketcap_4core_11Pymarketcap__correspondences(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_11pymarketcap_4core_11Pymarketcap_16_correspondences_1__get__(o);
}

static PyObject *__pyx_getprop_11pymarketcap_4core_11Pymarketcap__ids_correspondences(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_11pymarketcap_4core_11Pymarketcap_20_ids_correspondences_1__get__(o);
}

static PyObject *__pyx_getprop_11pymarketcap_4core_11Pymarketcap__symbols(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_11pymarketcap_4core_11Pymarketcap_8_symbols_1__get__(o);
}

static PyObject *__pyx_getprop_11pymarketcap_4core_11Pymarketcap__coins(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_11pymarketcap_4core_11Pymarketcap_6_coins_1__get__(o);
}

static PyObject *__pyx_getprop_11pymarketcap_4core_11Pymarketcap__total_currencies(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_11pymarketcap_4core_11Pymarketcap_17_total_currencies_1__get__(o);
}

static PyObject *__pyx_getprop_11pymarketcap_4core_11Pymarketcap__total_exchanges(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_11pymarketcap_4core_11Pymarketcap_16_total_exchanges_1__get__(o);
}

static PyObject *__pyx_getprop_11pymarketcap_4core_11Pymarketcap__currencies_to_convert(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_11pymarketcap_4core_11Pymarketcap_22_currencies_to_convert_1__get__(o);
}

static PyObject *__pyx_getprop_11pymarketcap_4core_11Pymarketcap__converter_cache(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_11pymarketcap_4core_11Pymarketcap_16_converter_cache_1__get__(o);
}

static PyObject *__pyx_getprop_11pymarketcap_4core_11Pymarketcap__exchange_names(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_11pymarketcap_4core_11Pymarketcap_15_exchange_names_1__get__(o);
}

static PyObject *__pyx_getprop_11pymarketcap_4core_11Pymarketcap__exchange_slugs(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_11pymarketcap_4core_11Pymarketcap_15_exchange_slugs_1__get__(o);
}

static PyObject *__pyx_getprop_11pymarketcap_4core_11Pymarketcap___repeated_symbols(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_11pymarketcap_4core_11Pymarketcap_18__repeated_symbols_1__get__(o);
}

static PyObject *__pyx_getprop_11pymarketcap_4core_11Pymarketcap__exchange_names_slugs_ids(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_11pymarketcap_4core_11Pymarketcap_25_exchange_names_slugs_ids_1__get__(o);
}

static PyObject *__pyx_getprop_11pymarketcap_4core_11Pymarketcap_timeout(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_11pymarketcap_4core_11Pymarketcap_7timeout_1__get__(o);
}

static int __pyx_setprop_11pymarketcap_4core_11Pymarketcap_timeout(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_11pymarketcap_4core_11Pymarketcap_7timeout_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyObject *__pyx_getprop_11pymarketcap_4core_11Pymarketcap_graphs(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_11pymarketcap_4core_11Pymarketcap_6graphs_1__get__(o);
}

static int __pyx_setprop_11pymarketcap_4core_11Pymarketcap_graphs(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_11pymarketcap_4core_11Pymarketcap_6graphs_3__set__(o, v);
  }
  else {
    return __pyx_pw_11pymarketcap_4core_11Pymarketcap_6graphs_5__del__(o);
  }
}

static PyObject *__pyx_getprop_11pymarketcap_4core_11Pymarketcap_debug(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_11pymarketcap_4core_11Pymarketcap_5debug_1__get__(o);
}

static int __pyx_setprop_11pymarketcap_4core_11Pymarketcap_debug(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_11pymarketcap_4core_11Pymarketcap_5debug_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyMethodDef __pyx_methods_11pymarketcap_4core_Pymarketcap[] = {
  {"_is_symbol", (PyCFunction)__pyx_pw_11pymarketcap_4core_11Pymarketcap_3_is_symbol, METH_O, __pyx_doc_11pymarketcap_4core_11Pymarketcap_2_is_symbol},
  {"_quick_search", (PyCFunction)__pyx_pw_11pymarketcap_4core_11Pymarketcap_5_quick_search, METH_VARARGS|METH_KEYWORDS, __pyx_doc_11pymarketcap_4core_11Pymarketcap_4_quick_search},
  {"_cache_symbols_ids", (PyCFunction)__pyx_pw_11pymarketcap_4core_11Pymarketcap_7_cache_symbols_ids, METH_NOARGS, __pyx_doc_11pymarketcap_4core_11Pymarketcap_6_cache_symbols_ids},
  {"_cache_exchanges_ids", (PyCFunction)__pyx_pw_11pymarketcap_4core_11Pymarketcap_9_cache_exchanges_ids, METH_NOARGS, 0},
  {"__currencies_to_convert", (PyCFunction)__pyx_pw_11pymarketcap_4core_11Pymarketcap_11__currencies_to_convert, METH_NOARGS, __pyx_doc_11pymarketcap_4core_11Pymarketcap_10__currencies_to_convert},
  {"__exchange_names_slugs", (PyCFunction)__pyx_pw_11pymarketcap_4core_11Pymarketcap_13__exchange_names_slugs, METH_NOARGS, __pyx_doc_11pymarketcap_4core_11Pymarketcap_12__exchange_names_slugs},
  {"stats", (PyCFunction)__pyx_pw_11pymarketcap_4core_11Pymarketcap_15stats, METH_VARARGS|METH_KEYWORDS, __pyx_doc_11pymarketcap_4core_11Pymarketcap_14stats},
  {"ticker", (PyCFunction)__pyx_pw_11pymarketcap_4core_11Pymarketcap_17ticker, METH_VARARGS|METH_KEYWORDS, __pyx_doc_11pymarketcap_4core_11Pymarketcap_16ticker},
  {"convert", (PyCFunction)__pyx_pw_11pymarketcap_4core_11Pymarketcap_19convert, METH_VARARGS|METH_KEYWORDS, __pyx_doc_11pymarketcap_4core_11Pymarketcap_18convert},
  {"currency", (PyCFunction)__pyx_pw_11pymarketcap_4core_11Pymarketcap_21currency, METH_VARARGS|METH_KEYWORDS, __pyx_doc_11pymarketcap_4core_11Pymarketcap_20currency},
  {"markets", (PyCFunction)__pyx_pw_11pymarketcap_4core_11Pymarketcap_23markets, METH_VARARGS|METH_KEYWORDS, __pyx_doc_11pymarketcap_4core_11Pymarketcap_22markets},
  {"ranks", (PyCFunction)__pyx_pw_11pymarketcap_4core_11Pymarketcap_25ranks, METH_NOARGS, __pyx_doc_11pymarketcap_4core_11Pymarketcap_24ranks},
  {"historical", (PyCFunction)__pyx_pw_11pymarketcap_4core_11Pymarketcap_27historical, METH_VARARGS|METH_KEYWORDS, __pyx_doc_11pymarketcap_4core_11Pymarketcap_26historical},
  {"recently", (PyCFunction)__pyx_pw_11pymarketcap_4core_11Pymarketcap_29recently, METH_VARARGS|METH_KEYWORDS, __pyx_doc_11pymarketcap_4core_11Pymarketcap_28recently},
  {"exchange", (PyCFunction)__pyx_pw_11pymarketcap_4core_11Pymarketcap_31exchange, METH_VARARGS|METH_KEYWORDS, __pyx_doc_11pymarketcap_4core_11Pymarketcap_30exchange},
  {"exchanges", (PyCFunction)__pyx_pw_11pymarketcap_4core_11Pymarketcap_33exchanges, METH_VARARGS|METH_KEYWORDS, __pyx_doc_11pymarketcap_4core_11Pymarketcap_32exchanges},
  {"tokens", (PyCFunction)__pyx_pw_11pymarketcap_4core_11Pymarketcap_35tokens, METH_VARARGS|METH_KEYWORDS, __pyx_doc_11pymarketcap_4core_11Pymarketcap_34tokens},
  {"_currency", (PyCFunction)__pyx_pw_11pymarketcap_4core_11Pymarketcap_37_currency, METH_VARARGS|METH_KEYWORDS, __pyx_doc_11pymarketcap_4core_11Pymarketcap_36_currency},
  {"_global_cap", (PyCFunction)__pyx_pw_11pymarketcap_4core_11Pymarketcap_39_global_cap, METH_VARARGS|METH_KEYWORDS, __pyx_doc_11pymarketcap_4core_11Pymarketcap_38_global_cap},
  {"_dominance", (PyCFunction)__pyx_pw_11pymarketcap_4core_11Pymarketcap_41_dominance, METH_VARARGS|METH_KEYWORDS, __pyx_doc_11pymarketcap_4core_11Pymarketcap_40_dominance},
  {"download_logo", (PyCFunction)__pyx_pw_11pymarketcap_4core_11Pymarketcap_43download_logo, METH_VARARGS|METH_KEYWORDS, __pyx_doc_11pymarketcap_4core_11Pymarketcap_42download_logo},
  {"download_exchange_logo", (PyCFunction)__pyx_pw_11pymarketcap_4core_11Pymarketcap_45download_exchange_logo, METH_VARARGS|METH_KEYWORDS, __pyx_doc_11pymarketcap_4core_11Pymarketcap_44download_exchange_logo},
  {"__reduce_cython__", (PyCFunction)__pyx_pw_11pymarketcap_4core_11Pymarketcap_47__reduce_cython__, METH_NOARGS, 0},
  {"__setstate_cython__", (PyCFunction)__pyx_pw_11pymarketcap_4core_11Pymarketcap_49__setstate_cython__, METH_O, 0},
  {0, 0, 0, 0}
};

static struct PyGetSetDef __pyx_getsets_11pymarketcap_4core_Pymarketcap[] = {
  {(char *)"_graphs_interface", __pyx_getprop_11pymarketcap_4core_11Pymarketcap__graphs_interface, 0, (char *)0, 0},
  {(char *)"correspondences", __pyx_getprop_11pymarketcap_4core_11Pymarketcap_correspondences, 0, (char *)0, 0},
  {(char *)"ids_correspondences", __pyx_getprop_11pymarketcap_4core_11Pymarketcap_ids_correspondences, 0, (char *)"Get symbols with their correspondient numeric\n        id (used for debug purposes).", 0},
  {(char *)"symbols", __pyx_getprop_11pymarketcap_4core_11Pymarketcap_symbols, 0, (char *)"Symbols of currencies (in capital letters).\n\n        Returns (list):\n            All currency symbols provided by coinmarketcap.\n        ", 0},
  {(char *)"coins", __pyx_getprop_11pymarketcap_4core_11Pymarketcap_coins, 0, (char *)"Coins not formatted names for all currencies\n        (in lowercase letters) used internally by urls.\n\n        Returns (list):\n            All currency coins names provided by coinmarketcap.\n        ", 0},
  {(char *)"total_currencies", __pyx_getprop_11pymarketcap_4core_11Pymarketcap_total_currencies, 0, (char *)0, 0},
  {(char *)"currencies_to_convert", __pyx_getprop_11pymarketcap_4core_11Pymarketcap_currencies_to_convert, 0, (char *)0, 0},
  {(char *)"exchange_names", __pyx_getprop_11pymarketcap_4core_11Pymarketcap_exchange_names, 0, (char *)"Get all exchange formatted names provided by coinmarketcap.", 0},
  {(char *)"exchange_slugs", __pyx_getprop_11pymarketcap_4core_11Pymarketcap_exchange_slugs, 0, (char *)"Get all exchange raw names provided by coinmarketcap.", 0},
  {(char *)"total_exchanges", __pyx_getprop_11pymarketcap_4core_11Pymarketcap_total_exchanges, 0, (char *)0, 0},
  {(char *)"converter_cache", __pyx_getprop_11pymarketcap_4core_11Pymarketcap_converter_cache, 0, (char *)0, 0},
  {(char *)"ticker_badges", __pyx_getprop_11pymarketcap_4core_11Pymarketcap_ticker_badges, 0, (char *)"Badges in wich you can convert prices in ``ticker()`` method.", 0},
  {(char *)"currency_exchange_rates", __pyx_getprop_11pymarketcap_4core_11Pymarketcap_currency_exchange_rates, 0, (char *)"Currency exchange rates against $ for all currencies (fiat + crypto).\n\n        Returns (dict):\n            All currencies rates used internally by coinmarketcap to calculate\n            the prices shown.\n        ", 0},
  {(char *)"_correspondences", __pyx_getprop_11pymarketcap_4core_11Pymarketcap__correspondences, 0, (char *)0, 0},
  {(char *)"_ids_correspondences", __pyx_getprop_11pymarketcap_4core_11Pymarketcap__ids_correspondences, 0, (char *)0, 0},
  {(char *)"_symbols", __pyx_getprop_11pymarketcap_4core_11Pymarketcap__symbols, 0, (char *)0, 0},
  {(char *)"_coins", __pyx_getprop_11pymarketcap_4core_11Pymarketcap__coins, 0, (char *)0, 0},
  {(char *)"_total_currencies", __pyx_getprop_11pymarketcap_4core_11Pymarketcap__total_currencies, 0, (char *)0, 0},
  {(char *)"_total_exchanges", __pyx_getprop_11pymarketcap_4core_11Pymarketcap__total_exchanges, 0, (char *)0, 0},
  {(char *)"_currencies_to_convert", __pyx_getprop_11pymarketcap_4core_11Pymarketcap__currencies_to_convert, 0, (char *)0, 0},
  {(char *)"_converter_cache", __pyx_getprop_11pymarketcap_4core_11Pymarketcap__converter_cache, 0, (char *)0, 0},
  {(char *)"_exchange_names", __pyx_getprop_11pymarketcap_4core_11Pymarketcap__exchange_names, 0, (char *)0, 0},
  {(char *)"_exchange_slugs", __pyx_getprop_11pymarketcap_4core_11Pymarketcap__exchange_slugs, 0, (char *)0, 0},
  {(char *)"__repeated_symbols", __pyx_getprop_11pymarketcap_4core_11Pymarketcap___repeated_symbols, 0, (char *)0, 0},
  {(char *)"_exchange_names_slugs_ids", __pyx_getprop_11pymarketcap_4core_11Pymarketcap__exchange_names_slugs_ids, 0, (char *)0, 0},
  {(char *)"timeout", __pyx_getprop_11pymarketcap_4core_11Pymarketcap_timeout, __pyx_setprop_11pymarketcap_4core_11Pymarketcap_timeout, (char *)0, 0},
  {(char *)"graphs", __pyx_getprop_11pymarketcap_4core_11Pymarketcap_graphs, __pyx_setprop_11pymarketcap_4core_11Pymarketcap_graphs, (char *)0, 0},
  {(char *)"debug", __pyx_getprop_11pymarketcap_4core_11Pymarketcap_debug, __pyx_setprop_11pymarketcap_4core_11Pymarketcap_debug, (char *)0, 0},
  {0, 0, 0, 0, 0}
};

static PyTypeObject __pyx_type_11pymarketcap_4core_Pymarketcap = {
  PyVarObject_HEAD_INIT(0, 0)
  "pymarketcap.core.Pymarketcap", /*tp_name*/
  sizeof(struct __pyx_obj_11pymarketcap_4core_Pymarketcap), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_11pymarketcap_4core_Pymarketcap, /*tp_dealloc*/
  0, /*tp_print*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  "Synchronous class for retrieve data from https://coinmarketcap.com.\n\n    Args:\n        timeout (int, optional): Set timeout value for requests.\n            As default ``20``.\n        debug: (bool, optional): Show low level data in get requests.\n            As default, ``False``.\n    ", /*tp_doc*/
  __pyx_tp_traverse_11pymarketcap_4core_Pymarketcap, /*tp_traverse*/
  __pyx_tp_clear_11pymarketcap_4core_Pymarketcap, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_11pymarketcap_4core_Pymarketcap, /*tp_methods*/
  0, /*tp_members*/
  __pyx_getsets_11pymarketcap_4core_Pymarketcap, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  __pyx_pw_11pymarketcap_4core_11Pymarketcap_1__init__, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_11pymarketcap_4core_Pymarketcap, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
};

static PyMethodDef __pyx_methods[] = {
  {0, 0, 0, 0}
};

#if PY_MAJOR_VERSION >= 3
#if CYTHON_PEP489_MULTI_PHASE_INIT
static PyObject* __pyx_pymod_create(PyObject *spec, PyModuleDef *def); /*proto*/
static int __pyx_pymod_exec_core(PyObject* module); /*proto*/
static PyModuleDef_Slot __pyx_moduledef_slots[] = {
  {Py_mod_create, (void*)__pyx_pymod_create},
  {Py_mod_exec, (void*)__pyx_pymod_exec_core},
  {0, NULL}
};
#endif

static struct PyModuleDef __pyx_moduledef = {
    PyModuleDef_HEAD_INIT,
    "core",
    __pyx_k_API_wraper_and_web_scraper_modul, /* m_doc */
  #if CYTHON_PEP489_MULTI_PHASE_INIT
    0, /* m_size */
  #else
    -1, /* m_size */
  #endif
    __pyx_methods /* m_methods */,
  #if CYTHON_PEP489_MULTI_PHASE_INIT
    __pyx_moduledef_slots, /* m_slots */
  #else
    NULL, /* m_reload */
  #endif
    NULL, /* m_traverse */
    NULL, /* m_clear */
    NULL /* m_free */
};
#endif

static __Pyx_StringTabEntry __pyx_string_tab[] = {
  {&__pyx_kp_s_02d, __pyx_k_02d, sizeof(__pyx_k_02d), 0, 0, 1, 0},
  {&__pyx_kp_s_1, __pyx_k_1, sizeof(__pyx_k_1), 0, 0, 1, 0},
  {&__pyx_kp_s_404, __pyx_k_404, sizeof(__pyx_k_404), 0, 0, 1, 0},
  {&__pyx_kp_s_429, __pyx_k_429, sizeof(__pyx_k_429), 0, 0, 1, 0},
  {&__pyx_n_s_CoinmarketcapHTTPError, __pyx_k_CoinmarketcapHTTPError, sizeof(__pyx_k_CoinmarketcapHTTPError), 0, 0, 1, 1},
  {&__pyx_n_s_CoinmarketcapHTTPError404, __pyx_k_CoinmarketcapHTTPError404, sizeof(__pyx_k_CoinmarketcapHTTPError404), 0, 0, 1, 1},
  {&__pyx_n_s_CoinmarketcapTooManyRequestsErro, __pyx_k_CoinmarketcapTooManyRequestsErro, sizeof(__pyx_k_CoinmarketcapTooManyRequestsErro), 0, 0, 1, 1},
  {&__pyx_kp_s_DEBUG, __pyx_k_DEBUG, sizeof(__pyx_k_DEBUG), 0, 0, 1, 0},
  {&__pyx_n_s_DEFAULT_TIMEOUT, __pyx_k_DEFAULT_TIMEOUT, sizeof(__pyx_k_DEFAULT_TIMEOUT), 0, 0, 1, 1},
  {&__pyx_n_s_EXCEPTIONAL_COIN_SLUGS, __pyx_k_EXCEPTIONAL_COIN_SLUGS, sizeof(__pyx_k_EXCEPTIONAL_COIN_SLUGS), 0, 0, 1, 1},
  {&__pyx_kp_s_Exchange_name_catched_but_not_fo, __pyx_k_Exchange_name_catched_but_not_fo, sizeof(__pyx_k_Exchange_name_catched_but_not_fo), 0, 0, 1, 0},
  {&__pyx_kp_s_Exchange_s_not_found, __pyx_k_Exchange_s_not_found, sizeof(__pyx_k_Exchange_s_not_found), 0, 0, 1, 0},
  {&__pyx_n_s_Graphs, __pyx_k_Graphs, sizeof(__pyx_k_Graphs), 0, 0, 1, 1},
  {&__pyx_n_s_HTTPError, __pyx_k_HTTPError, sizeof(__pyx_k_HTTPError), 0, 0, 1, 1},
  {&__pyx_n_s_INVALID_COINS, __pyx_k_INVALID_COINS, sizeof(__pyx_k_INVALID_COINS), 0, 0, 1, 1},
  {&__pyx_kp_s_Incompatible_checksums_s_vs_0x7d, __pyx_k_Incompatible_checksums_s_vs_0x7d, sizeof(__pyx_k_Incompatible_checksums_s_vs_0x7d), 0, 0, 1, 0},
  {&__pyx_kp_s_Invalid_currency_s_See_currencie, __pyx_k_Invalid_currency_s_See_currencie, sizeof(__pyx_k_Invalid_currency_s_See_currencie), 0, 0, 1, 0},
  {&__pyx_n_s_KeyError, __pyx_k_KeyError, sizeof(__pyx_k_KeyError), 0, 0, 1, 1},
  {&__pyx_n_s_NotImplementedError, __pyx_k_NotImplementedError, sizeof(__pyx_k_NotImplementedError), 0, 0, 1, 1},
  {&__pyx_n_s_PickleError, __pyx_k_PickleError, sizeof(__pyx_k_PickleError), 0, 0, 1, 1},
  {&__pyx_kp_s_Possible_valid_slug_names_r, __pyx_k_Possible_valid_slug_names_r, sizeof(__pyx_k_Possible_valid_slug_names_r), 0, 0, 1, 0},
  {&__pyx_kp_s_Seems_that_s_currency_doesn_t_al, __pyx_k_Seems_that_s_currency_doesn_t_al, sizeof(__pyx_k_Seems_that_s_currency_doesn_t_al), 0, 0, 1, 0},
  {&__pyx_kp_s_Seems_that_s_exchange_doesn_t_al, __pyx_k_Seems_that_s_exchange_doesn_t_al, sizeof(__pyx_k_Seems_that_s_exchange_doesn_t_al), 0, 0, 1, 0},
  {&__pyx_kp_s_Slug_catched_but_not_found, __pyx_k_Slug_catched_but_not_found, sizeof(__pyx_k_Slug_catched_but_not_found), 0, 0, 1, 0},
  {&__pyx_kp_s_Status_code_d_Url_s, __pyx_k_Status_code_d_Url_s, sizeof(__pyx_k_Status_code_d_Url_s), 0, 0, 1, 0},
  {&__pyx_kp_s_The_currency_s_is_not_valid_See, __pyx_k_The_currency_s_is_not_valid_See, sizeof(__pyx_k_The_currency_s_is_not_valid_See), 0, 0, 1, 0},
  {&__pyx_kp_s_The_symbol_s_has_more_than_one_c, __pyx_k_The_symbol_s_has_more_than_one_c, sizeof(__pyx_k_The_symbol_s_has_more_than_one_c), 0, 0, 1, 0},
  {&__pyx_n_s_TypeError, __pyx_k_TypeError, sizeof(__pyx_k_TypeError), 0, 0, 1, 1},
  {&__pyx_n_s_USD, __pyx_k_USD, sizeof(__pyx_k_USD), 0, 0, 1, 1},
  {&__pyx_n_s_ValueError, __pyx_k_ValueError, sizeof(__pyx_k_ValueError), 0, 0, 1, 1},
  {&__pyx_kp_s__11, __pyx_k__11, sizeof(__pyx_k__11), 0, 0, 1, 0},
  {&__pyx_kp_s__2, __pyx_k__2, sizeof(__pyx_k__2), 0, 0, 1, 0},
  {&__pyx_kp_s__3, __pyx_k__3, sizeof(__pyx_k__3), 0, 0, 1, 0},
  {&__pyx_n_s_append, __pyx_k_append, sizeof(__pyx_k_append), 0, 0, 1, 1},
  {&__pyx_n_s_bitcoin, __pyx_k_bitcoin, sizeof(__pyx_k_bitcoin), 0, 0, 1, 1},
  {&__pyx_n_s_cache_exchanges_ids, __pyx_k_cache_exchanges_ids, sizeof(__pyx_k_cache_exchanges_ids), 0, 0, 1, 1},
  {&__pyx_n_s_cache_symbols_ids, __pyx_k_cache_symbols_ids, sizeof(__pyx_k_cache_symbols_ids), 0, 0, 1, 1},
  {&__pyx_n_s_cline_in_traceback, __pyx_k_cline_in_traceback, sizeof(__pyx_k_cline_in_traceback), 0, 0, 1, 1},
  {&__pyx_n_s_code, __pyx_k_code, sizeof(__pyx_k_code), 0, 0, 1, 1},
  {&__pyx_n_s_coins, __pyx_k_coins, sizeof(__pyx_k_coins), 0, 0, 1, 1},
  {&__pyx_n_s_convert, __pyx_k_convert, sizeof(__pyx_k_convert), 0, 0, 1, 1},
  {&__pyx_kp_s_convert_s, __pyx_k_convert_s, sizeof(__pyx_k_convert_s), 0, 0, 1, 0},
  {&__pyx_kp_s_convert_s_2, __pyx_k_convert_s_2, sizeof(__pyx_k_convert_s_2), 0, 0, 1, 0},
  {&__pyx_n_s_converter_cache, __pyx_k_converter_cache, sizeof(__pyx_k_converter_cache), 0, 0, 1, 1},
  {&__pyx_n_s_correspondences, __pyx_k_correspondences, sizeof(__pyx_k_correspondences), 0, 0, 1, 1},
  {&__pyx_n_s_currencies_to_convert, __pyx_k_currencies_to_convert, sizeof(__pyx_k_currencies_to_convert), 0, 0, 1, 1},
  {&__pyx_n_s_currencies_to_convert_2, __pyx_k_currencies_to_convert_2, sizeof(__pyx_k_currencies_to_convert_2), 0, 0, 1, 1},
  {&__pyx_n_s_currency, __pyx_k_currency, sizeof(__pyx_k_currency), 0, 0, 1, 1},
  {&__pyx_n_s_currency_2, __pyx_k_currency_2, sizeof(__pyx_k_currency_2), 0, 0, 1, 1},
  {&__pyx_n_s_currency_exchange_rates, __pyx_k_currency_exchange_rates, sizeof(__pyx_k_currency_exchange_rates), 0, 0, 1, 1},
  {&__pyx_n_s_currency_in, __pyx_k_currency_in, sizeof(__pyx_k_currency_in), 0, 0, 1, 1},
  {&__pyx_n_s_currency_out, __pyx_k_currency_out, sizeof(__pyx_k_currency_out), 0, 0, 1, 1},
  {&__pyx_kp_s_d, __pyx_k_d, sizeof(__pyx_k_d), 0, 0, 1, 0},
  {&__pyx_kp_s_d_d, __pyx_k_d_d, sizeof(__pyx_k_d_d), 0, 0, 1, 0},
  {&__pyx_kp_s_data_a_z_d, __pyx_k_data_a_z_d, sizeof(__pyx_k_data_a_z_d), 0, 0, 1, 0},
  {&__pyx_kp_s_data_a_z_d_d_e, __pyx_k_data_a_z_d_d_e, sizeof(__pyx_k_data_a_z_d_d_e), 0, 0, 1, 0},
  {&__pyx_n_s_datetime, __pyx_k_datetime, sizeof(__pyx_k_datetime), 0, 0, 1, 1},
  {&__pyx_n_s_day, __pyx_k_day, sizeof(__pyx_k_day), 0, 0, 1, 1},
  {&__pyx_n_s_debug, __pyx_k_debug, sizeof(__pyx_k_debug), 0, 0, 1, 1},
  {&__pyx_n_s_decode, __pyx_k_decode, sizeof(__pyx_k_decode), 0, 0, 1, 1},
  {&__pyx_n_s_dict, __pyx_k_dict, sizeof(__pyx_k_dict), 0, 0, 1, 1},
  {&__pyx_n_s_dominance, __pyx_k_dominance, sizeof(__pyx_k_dominance), 0, 0, 1, 1},
  {&__pyx_n_s_dominance_2, __pyx_k_dominance_2, sizeof(__pyx_k_dominance_2), 0, 0, 1, 1},
  {&__pyx_n_s_download_exchange_logo, __pyx_k_download_exchange_logo, sizeof(__pyx_k_download_exchange_logo), 0, 0, 1, 1},
  {&__pyx_n_s_download_logo, __pyx_k_download_logo, sizeof(__pyx_k_download_logo), 0, 0, 1, 1},
  {&__pyx_kp_s_dx_d_is_not_a_valid_size, __pyx_k_dx_d_is_not_a_valid_size, sizeof(__pyx_k_dx_d_is_not_a_valid_size), 0, 0, 1, 0},
  {&__pyx_n_s_encode, __pyx_k_encode, sizeof(__pyx_k_encode), 0, 0, 1, 1},
  {&__pyx_n_s_end, __pyx_k_end, sizeof(__pyx_k_end), 0, 0, 1, 1},
  {&__pyx_kp_s_end_s, __pyx_k_end_s, sizeof(__pyx_k_end_s), 0, 0, 1, 0},
  {&__pyx_kp_s_exc_s, __pyx_k_exc_s, sizeof(__pyx_k_exc_s), 0, 0, 1, 0},
  {&__pyx_n_s_exchange, __pyx_k_exchange, sizeof(__pyx_k_exchange), 0, 0, 1, 1},
  {&__pyx_n_s_exchange_names_slugs, __pyx_k_exchange_names_slugs, sizeof(__pyx_k_exchange_names_slugs), 0, 0, 1, 1},
  {&__pyx_n_s_exchange_slugs, __pyx_k_exchange_slugs, sizeof(__pyx_k_exchange_slugs), 0, 0, 1, 1},
  {&__pyx_n_s_exchanges, __pyx_k_exchanges, sizeof(__pyx_k_exchanges), 0, 0, 1, 1},
  {&__pyx_n_s_file, __pyx_k_file, sizeof(__pyx_k_file), 0, 0, 1, 1},
  {&__pyx_n_s_filename, __pyx_k_filename, sizeof(__pyx_k_filename), 0, 0, 1, 1},
  {&__pyx_n_s_findall, __pyx_k_findall, sizeof(__pyx_k_findall), 0, 0, 1, 1},
  {&__pyx_n_s_get_to_memory, __pyx_k_get_to_memory, sizeof(__pyx_k_get_to_memory), 0, 0, 1, 1},
  {&__pyx_n_s_getstate, __pyx_k_getstate, sizeof(__pyx_k_getstate), 0, 0, 1, 1},
  {&__pyx_n_s_global_cap, __pyx_k_global_cap, sizeof(__pyx_k_global_cap), 0, 0, 1, 1},
  {&__pyx_n_s_global_cap_2, __pyx_k_global_cap_2, sizeof(__pyx_k_global_cap_2), 0, 0, 1, 1},
  {&__pyx_n_s_graphs, __pyx_k_graphs, sizeof(__pyx_k_graphs), 0, 0, 1, 1},
  {&__pyx_n_s_graphs_interface, __pyx_k_graphs_interface, sizeof(__pyx_k_graphs_interface), 0, 0, 1, 1},
  {&__pyx_n_s_historical, __pyx_k_historical, sizeof(__pyx_k_historical), 0, 0, 1, 1},
  {&__pyx_n_s_history, __pyx_k_history, sizeof(__pyx_k_history), 0, 0, 1, 1},
  {&__pyx_n_s_http_error_numbers, __pyx_k_http_error_numbers, sizeof(__pyx_k_http_error_numbers), 0, 0, 1, 1},
  {&__pyx_n_s_http_errors_map, __pyx_k_http_errors_map, sizeof(__pyx_k_http_errors_map), 0, 0, 1, 1},
  {&__pyx_kp_b_https_api_coinmarketcap_com_v1_g, __pyx_k_https_api_coinmarketcap_com_v1_g, sizeof(__pyx_k_https_api_coinmarketcap_com_v1_g), 0, 0, 0, 0},
  {&__pyx_kp_s_https_api_coinmarketcap_com_v1_t, __pyx_k_https_api_coinmarketcap_com_v1_t, sizeof(__pyx_k_https_api_coinmarketcap_com_v1_t), 0, 0, 1, 0},
  {&__pyx_kp_s_https_api_coinmarketcap_com_v1_t_2, __pyx_k_https_api_coinmarketcap_com_v1_t_2, sizeof(__pyx_k_https_api_coinmarketcap_com_v1_t_2), 0, 0, 1, 0},
  {&__pyx_kp_b_https_coinmarketcap_com_currenci, __pyx_k_https_coinmarketcap_com_currenci, sizeof(__pyx_k_https_coinmarketcap_com_currenci), 0, 0, 0, 0},
  {&__pyx_kp_s_https_coinmarketcap_com_currenci_2, __pyx_k_https_coinmarketcap_com_currenci_2, sizeof(__pyx_k_https_coinmarketcap_com_currenci_2), 0, 0, 1, 0},
  {&__pyx_kp_s_https_coinmarketcap_com_exchange, __pyx_k_https_coinmarketcap_com_exchange, sizeof(__pyx_k_https_coinmarketcap_com_exchange), 0, 0, 1, 0},
  {&__pyx_kp_b_https_coinmarketcap_com_exchange_2, __pyx_k_https_coinmarketcap_com_exchange_2, sizeof(__pyx_k_https_coinmarketcap_com_exchange_2), 0, 0, 0, 0},
  {&__pyx_kp_b_https_coinmarketcap_com_new, __pyx_k_https_coinmarketcap_com_new, sizeof(__pyx_k_https_coinmarketcap_com_new), 0, 0, 0, 0},
  {&__pyx_kp_b_https_coinmarketcap_com_tokens_v, __pyx_k_https_coinmarketcap_com_tokens_v, sizeof(__pyx_k_https_coinmarketcap_com_tokens_v), 0, 0, 0, 0},
  {&__pyx_kp_b_https_graphs2_coinmarketcap_com, __pyx_k_https_graphs2_coinmarketcap_com, sizeof(__pyx_k_https_graphs2_coinmarketcap_com), 0, 0, 0, 0},
  {&__pyx_kp_b_https_graphs2_coinmarketcap_com_2, __pyx_k_https_graphs2_coinmarketcap_com_2, sizeof(__pyx_k_https_graphs2_coinmarketcap_com_2), 0, 0, 0, 0},
  {&__pyx_kp_b_https_graphs2_coinmarketcap_com_3, __pyx_k_https_graphs2_coinmarketcap_com_3, sizeof(__pyx_k_https_graphs2_coinmarketcap_com_3), 0, 0, 0, 0},
  {&__pyx_kp_b_https_graphs2_coinmarketcap_com_4, __pyx_k_https_graphs2_coinmarketcap_com_4, sizeof(__pyx_k_https_graphs2_coinmarketcap_com_4), 0, 0, 0, 0},
  {&__pyx_kp_b_https_s2_coinmarketcap_com_gener, __pyx_k_https_s2_coinmarketcap_com_gener, sizeof(__pyx_k_https_s2_coinmarketcap_com_gener), 0, 0, 0, 0},
  {&__pyx_kp_b_https_s2_coinmarketcap_com_gener_2, __pyx_k_https_s2_coinmarketcap_com_gener_2, sizeof(__pyx_k_https_s2_coinmarketcap_com_gener_2), 0, 0, 0, 0},
  {&__pyx_kp_s_https_s2_coinmarketcap_com_stati, __pyx_k_https_s2_coinmarketcap_com_stati, sizeof(__pyx_k_https_s2_coinmarketcap_com_stati), 0, 0, 1, 0},
  {&__pyx_kp_s_https_s2_coinmarketcap_com_stati_2, __pyx_k_https_s2_coinmarketcap_com_stati_2, sizeof(__pyx_k_https_s2_coinmarketcap_com_stati_2), 0, 0, 1, 0},
  {&__pyx_n_s_id, __pyx_k_id, sizeof(__pyx_k_id), 0, 0, 1, 1},
  {&__pyx_n_s_ids_correspondences, __pyx_k_ids_correspondences, sizeof(__pyx_k_ids_correspondences), 0, 0, 1, 1},
  {&__pyx_n_s_import, __pyx_k_import, sizeof(__pyx_k_import), 0, 0, 1, 1},
  {&__pyx_kp_s_in_name_index_position_0, __pyx_k_in_name_index_position_0, sizeof(__pyx_k_in_name_index_position_0), 0, 0, 1, 0},
  {&__pyx_kp_s_in_slug_index_position_1, __pyx_k_in_slug_index_position_1, sizeof(__pyx_k_in_slug_index_position_1), 0, 0, 1, 0},
  {&__pyx_n_s_index, __pyx_k_index, sizeof(__pyx_k_index), 0, 0, 1, 1},
  {&__pyx_n_s_is_symbol, __pyx_k_is_symbol, sizeof(__pyx_k_is_symbol), 0, 0, 1, 1},
  {&__pyx_n_s_islower, __pyx_k_islower, sizeof(__pyx_k_islower), 0, 0, 1, 1},
  {&__pyx_n_s_isupper, __pyx_k_isupper, sizeof(__pyx_k_isupper), 0, 0, 1, 1},
  {&__pyx_n_s_items, __pyx_k_items, sizeof(__pyx_k_items), 0, 0, 1, 1},
  {&__pyx_n_s_json, __pyx_k_json, sizeof(__pyx_k_json), 0, 0, 1, 1},
  {&__pyx_n_s_keys, __pyx_k_keys, sizeof(__pyx_k_keys), 0, 0, 1, 1},
  {&__pyx_n_s_limit, __pyx_k_limit, sizeof(__pyx_k_limit), 0, 0, 1, 1},
  {&__pyx_n_s_loads, __pyx_k_loads, sizeof(__pyx_k_loads), 0, 0, 1, 1},
  {&__pyx_n_s_lower, __pyx_k_lower, sizeof(__pyx_k_lower), 0, 0, 1, 1},
  {&__pyx_n_s_main, __pyx_k_main, sizeof(__pyx_k_main), 0, 0, 1, 1},
  {&__pyx_n_s_markets, __pyx_k_markets, sizeof(__pyx_k_markets), 0, 0, 1, 1},
  {&__pyx_n_s_month, __pyx_k_month, sizeof(__pyx_k_month), 0, 0, 1, 1},
  {&__pyx_n_s_name, __pyx_k_name, sizeof(__pyx_k_name), 0, 0, 1, 1},
  {&__pyx_n_s_name_2, __pyx_k_name_2, sizeof(__pyx_k_name_2), 0, 0, 1, 1},
  {&__pyx_n_s_new, __pyx_k_new, sizeof(__pyx_k_new), 0, 0, 1, 1},
  {&__pyx_n_s_now, __pyx_k_now, sizeof(__pyx_k_now), 0, 0, 1, 1},
  {&__pyx_n_s_number, __pyx_k_number, sizeof(__pyx_k_number), 0, 0, 1, 1},
  {&__pyx_kp_s_or_coins_properties_for_get_all, __pyx_k_or_coins_properties_for_get_all, sizeof(__pyx_k_or_coins_properties_for_get_all), 0, 0, 1, 0},
  {&__pyx_n_s_pickle, __pyx_k_pickle, sizeof(__pyx_k_pickle), 0, 0, 1, 1},
  {&__pyx_n_s_price_usd, __pyx_k_price_usd, sizeof(__pyx_k_price_usd), 0, 0, 1, 1},
  {&__pyx_n_s_print, __pyx_k_print, sizeof(__pyx_k_print), 0, 0, 1, 1},
  {&__pyx_n_s_processer, __pyx_k_processer, sizeof(__pyx_k_processer), 0, 0, 1, 1},
  {&__pyx_kp_s_properties_for_get_all_valid_cu, __pyx_k_properties_for_get_all_valid_cu, sizeof(__pyx_k_properties_for_get_all_valid_cu), 0, 0, 1, 0},
  {&__pyx_kp_s_property_for_get_all_valid_exch, __pyx_k_property_for_get_all_valid_exch, sizeof(__pyx_k_property_for_get_all_valid_exch), 0, 0, 1, 0},
  {&__pyx_n_s_pymarketcap, __pyx_k_pymarketcap, sizeof(__pyx_k_pymarketcap), 0, 0, 1, 1},
  {&__pyx_n_s_pymarketcap_consts, __pyx_k_pymarketcap_consts, sizeof(__pyx_k_pymarketcap_consts), 0, 0, 1, 1},
  {&__pyx_n_s_pymarketcap_core, __pyx_k_pymarketcap_core, sizeof(__pyx_k_pymarketcap_core), 0, 0, 1, 1},
  {&__pyx_n_s_pymarketcap_curl, __pyx_k_pymarketcap_curl, sizeof(__pyx_k_pymarketcap_curl), 0, 0, 1, 1},
  {&__pyx_n_s_pymarketcap_errors, __pyx_k_pymarketcap_errors, sizeof(__pyx_k_pymarketcap_errors), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_PickleError, __pyx_k_pyx_PickleError, sizeof(__pyx_k_pyx_PickleError), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_checksum, __pyx_k_pyx_checksum, sizeof(__pyx_k_pyx_checksum), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_result, __pyx_k_pyx_result, sizeof(__pyx_k_pyx_result), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_state, __pyx_k_pyx_state, sizeof(__pyx_k_pyx_state), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_type, __pyx_k_pyx_type, sizeof(__pyx_k_pyx_type), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_unpickle_Pymarketcap, __pyx_k_pyx_unpickle_Pymarketcap, sizeof(__pyx_k_pyx_unpickle_Pymarketcap), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_vtable, __pyx_k_pyx_vtable, sizeof(__pyx_k_pyx_vtable), 0, 0, 1, 1},
  {&__pyx_n_s_quick_search, __pyx_k_quick_search, sizeof(__pyx_k_quick_search), 0, 0, 1, 1},
  {&__pyx_n_s_range, __pyx_k_range, sizeof(__pyx_k_range), 0, 0, 1, 1},
  {&__pyx_n_s_rank, __pyx_k_rank, sizeof(__pyx_k_rank), 0, 0, 1, 1},
  {&__pyx_n_s_ranks, __pyx_k_ranks, sizeof(__pyx_k_ranks), 0, 0, 1, 1},
  {&__pyx_n_s_re, __pyx_k_re, sizeof(__pyx_k_re), 0, 0, 1, 1},
  {&__pyx_n_s_re_findall, __pyx_k_re_findall, sizeof(__pyx_k_re_findall), 0, 0, 1, 1},
  {&__pyx_n_s_re_sub, __pyx_k_re_sub, sizeof(__pyx_k_re_sub), 0, 0, 1, 1},
  {&__pyx_n_s_recently, __pyx_k_recently, sizeof(__pyx_k_recently), 0, 0, 1, 1},
  {&__pyx_n_s_reduce, __pyx_k_reduce, sizeof(__pyx_k_reduce), 0, 0, 1, 1},
  {&__pyx_n_s_reduce_cython, __pyx_k_reduce_cython, sizeof(__pyx_k_reduce_cython), 0, 0, 1, 1},
  {&__pyx_n_s_reduce_ex, __pyx_k_reduce_ex, sizeof(__pyx_k_reduce_ex), 0, 0, 1, 1},
  {&__pyx_n_s_remove, __pyx_k_remove, sizeof(__pyx_k_remove), 0, 0, 1, 1},
  {&__pyx_n_s_replace, __pyx_k_replace, sizeof(__pyx_k_replace), 0, 0, 1, 1},
  {&__pyx_n_s_revert, __pyx_k_revert, sizeof(__pyx_k_revert), 0, 0, 1, 1},
  {&__pyx_kp_s_s__dx_d_png, __pyx_k_s__dx_d_png, sizeof(__pyx_k_s__dx_d_png), 0, 0, 1, 0},
  {&__pyx_kp_s_s_is_not_a_valid_currency_name, __pyx_k_s_is_not_a_valid_currency_name, sizeof(__pyx_k_s_is_not_a_valid_currency_name), 0, 0, 1, 0},
  {&__pyx_kp_s_s_is_not_a_valid_currrency_name, __pyx_k_s_is_not_a_valid_currrency_name, sizeof(__pyx_k_s_is_not_a_valid_currrency_name), 0, 0, 1, 0},
  {&__pyx_kp_s_s_is_not_a_valid_exchange_name, __pyx_k_s_is_not_a_valid_exchange_name, sizeof(__pyx_k_s_is_not_a_valid_exchange_name), 0, 0, 1, 0},
  {&__pyx_kp_s_s_png, __pyx_k_s_png, sizeof(__pyx_k_s_png), 0, 0, 1, 0},
  {&__pyx_n_s_setstate, __pyx_k_setstate, sizeof(__pyx_k_setstate), 0, 0, 1, 1},
  {&__pyx_n_s_setstate_cython, __pyx_k_setstate_cython, sizeof(__pyx_k_setstate_cython), 0, 0, 1, 1},
  {&__pyx_n_s_size, __pyx_k_size, sizeof(__pyx_k_size), 0, 0, 1, 1},
  {&__pyx_kp_s_size_dx_d_Try_another_size, __pyx_k_size_dx_d_Try_another_size, sizeof(__pyx_k_size_dx_d_Try_another_size), 0, 0, 1, 0},
  {&__pyx_n_s_slug, __pyx_k_slug, sizeof(__pyx_k_slug), 0, 0, 1, 1},
  {&__pyx_n_s_start, __pyx_k_start, sizeof(__pyx_k_start), 0, 0, 1, 1},
  {&__pyx_kp_s_start_d, __pyx_k_start_d, sizeof(__pyx_k_start_d), 0, 0, 1, 0},
  {&__pyx_kp_s_start_s, __pyx_k_start_s, sizeof(__pyx_k_start_s), 0, 0, 1, 0},
  {&__pyx_n_s_stats, __pyx_k_stats, sizeof(__pyx_k_stats), 0, 0, 1, 1},
  {&__pyx_n_s_status_code, __pyx_k_status_code, sizeof(__pyx_k_status_code), 0, 0, 1, 1},
  {&__pyx_kp_s_stringsource, __pyx_k_stringsource, sizeof(__pyx_k_stringsource), 0, 0, 1, 0},
  {&__pyx_n_s_sub, __pyx_k_sub, sizeof(__pyx_k_sub), 0, 0, 1, 1},
  {&__pyx_n_s_symbol, __pyx_k_symbol, sizeof(__pyx_k_symbol), 0, 0, 1, 1},
  {&__pyx_n_s_test, __pyx_k_test, sizeof(__pyx_k_test), 0, 0, 1, 1},
  {&__pyx_n_s_text, __pyx_k_text, sizeof(__pyx_k_text), 0, 0, 1, 1},
  {&__pyx_n_s_ticker, __pyx_k_ticker, sizeof(__pyx_k_ticker), 0, 0, 1, 1},
  {&__pyx_n_s_time, __pyx_k_time, sizeof(__pyx_k_time), 0, 0, 1, 1},
  {&__pyx_n_s_timeout, __pyx_k_timeout, sizeof(__pyx_k_timeout), 0, 0, 1, 1},
  {&__pyx_n_s_tokens, __pyx_k_tokens, sizeof(__pyx_k_tokens), 0, 0, 1, 1},
  {&__pyx_n_s_update, __pyx_k_update, sizeof(__pyx_k_update), 0, 0, 1, 1},
  {&__pyx_n_s_upper, __pyx_k_upper, sizeof(__pyx_k_upper), 0, 0, 1, 1},
  {&__pyx_n_s_url, __pyx_k_url, sizeof(__pyx_k_url), 0, 0, 1, 1},
  {&__pyx_n_s_urllib_error, __pyx_k_urllib_error, sizeof(__pyx_k_urllib_error), 0, 0, 1, 1},
  {&__pyx_n_s_urllib_request, __pyx_k_urllib_request, sizeof(__pyx_k_urllib_request), 0, 0, 1, 1},
  {&__pyx_n_s_urlretrieve, __pyx_k_urlretrieve, sizeof(__pyx_k_urlretrieve), 0, 0, 1, 1},
  {&__pyx_n_s_value, __pyx_k_value, sizeof(__pyx_k_value), 0, 0, 1, 1},
  {&__pyx_n_s_values, __pyx_k_values, sizeof(__pyx_k_values), 0, 0, 1, 1},
  {&__pyx_kp_s_with_coin_slugs_in_Coinmarketcap, __pyx_k_with_coin_slugs_in_Coinmarketcap, sizeof(__pyx_k_with_coin_slugs_in_Coinmarketcap), 0, 0, 1, 0},
  {&__pyx_n_s_year, __pyx_k_year, sizeof(__pyx_k_year), 0, 0, 1, 1},
  {0, 0, 0, 0, 0, 0, 0}
};
static int __Pyx_InitCachedBuiltins(void) {
  __pyx_builtin_ValueError = __Pyx_GetBuiltinName(__pyx_n_s_ValueError); if (!__pyx_builtin_ValueError) __PYX_ERR(0, 126, __pyx_L1_error)
  __pyx_builtin_TypeError = __Pyx_GetBuiltinName(__pyx_n_s_TypeError); if (!__pyx_builtin_TypeError) __PYX_ERR(0, 127, __pyx_L1_error)
  __pyx_builtin_range = __Pyx_GetBuiltinName(__pyx_n_s_range); if (!__pyx_builtin_range) __PYX_ERR(0, 382, __pyx_L1_error)
  __pyx_builtin_KeyError = __Pyx_GetBuiltinName(__pyx_n_s_KeyError); if (!__pyx_builtin_KeyError) __PYX_ERR(0, 443, __pyx_L1_error)
  __pyx_builtin_NotImplementedError = __Pyx_GetBuiltinName(__pyx_n_s_NotImplementedError); if (!__pyx_builtin_NotImplementedError) __PYX_ERR(0, 485, __pyx_L1_error)
  return 0;
  __pyx_L1_error:;
  return -1;
}

static int __Pyx_InitCachedConstants(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_InitCachedConstants", 0);

  /* "pymarketcap/core.pyx":151
 *         for currency in self._quick_search():
 *             symbol = currency["symbol"]
 *             slug = currency["slug"].replace(" ", "")             # <<<<<<<<<<<<<<
 *             if symbol in symbols_slugs:    # Repeated symbols are stored internally
 *                 if symbol in self.__repeated_symbols.keys():
 */
  __pyx_tuple__4 = PyTuple_Pack(2, __pyx_kp_s__2, __pyx_kp_s__3); if (unlikely(!__pyx_tuple__4)) __PYX_ERR(0, 151, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__4);
  __Pyx_GIVEREF(__pyx_tuple__4);

  /* "pymarketcap/core.pyx":247
 *         """
 *         res = self._get(b"https://coinmarketcap.com")
 *         currencies = re_findall(r'data-([a-z]+)="\d', res[-10000:-2000])             # <<<<<<<<<<<<<<
 *         response = [currency.upper() for currency in currencies]
 *         response.extend([str(currency["symbol"]) for currency in self.ticker()])
 */
  __pyx_slice__5 = PySlice_New(__pyx_int_neg_10000, __pyx_int_neg_2000, Py_None); if (unlikely(!__pyx_slice__5)) __PYX_ERR(0, 247, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_slice__5);
  __Pyx_GIVEREF(__pyx_slice__5);

  /* "pymarketcap/core.pyx":409
 *         res = self._get(b"https://coinmarketcap.com")
 *         rates = re_findall(
 *             r'data-([a-z]+)="(\d+\.*[\d|e|-]*)"', res[-10000:-2000]             # <<<<<<<<<<<<<<
 *         )
 *         response = {currency.upper(): float(rate) for currency, rate in rates}
 */
  __pyx_slice__6 = PySlice_New(__pyx_int_neg_10000, __pyx_int_neg_2000, Py_None); if (unlikely(!__pyx_slice__6)) __PYX_ERR(0, 409, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_slice__6);
  __Pyx_GIVEREF(__pyx_slice__6);

  /* "pymarketcap/core.pyx":479
 *             res = self._get(
 *                 b"https://coinmarketcap.com/currencies/%s/" % name.encode()
 *             )[20000:]             # <<<<<<<<<<<<<<
 *         except CoinmarketcapHTTPError404:
 *             if name not in self.coins:
 */
  __pyx_slice__7 = PySlice_New(__pyx_int_20000, Py_None, Py_None); if (unlikely(!__pyx_slice__7)) __PYX_ERR(0, 479, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_slice__7);
  __Pyx_GIVEREF(__pyx_slice__7);

  /* "pymarketcap/core.pyx":519
 *             res = self._get(
 *                 b"https://coinmarketcap.com/currencies/%s/" % name.encode()
 *             )[20000:]             # <<<<<<<<<<<<<<
 *         except CoinmarketcapHTTPError404:
 *             if name not in self.coins:
 */
  __pyx_slice__8 = PySlice_New(__pyx_int_20000, Py_None, Py_None); if (unlikely(!__pyx_slice__8)) __PYX_ERR(0, 519, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_slice__8);
  __Pyx_GIVEREF(__pyx_slice__8);

  /* "pymarketcap/core.pyx":580
 * 
 *         try:
 *             res = self._get(url.encode())[50000:]             # <<<<<<<<<<<<<<
 *         except CoinmarketcapHTTPError404:
 *             if name not in self.coins:
 */
  __pyx_slice__12 = PySlice_New(__pyx_int_50000, Py_None, Py_None); if (unlikely(!__pyx_slice__12)) __PYX_ERR(0, 580, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_slice__12);
  __Pyx_GIVEREF(__pyx_slice__12);

  /* "pymarketcap/core.pyx":625
 *         url = "https://coinmarketcap.com/exchanges/%s/" % name
 *         try:
 *             res = self._get(url.encode())[20000:]             # <<<<<<<<<<<<<<
 *         except CoinmarketcapHTTPError404:
 *             if name not in self.exchange_slugs:
 */
  __pyx_slice__13 = PySlice_New(__pyx_int_20000, Py_None, Py_None); if (unlikely(!__pyx_slice__13)) __PYX_ERR(0, 625, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_slice__13);
  __Pyx_GIVEREF(__pyx_slice__13);

  /* "pymarketcap/core.pyx":652
 *         cdef bytes url
 *         url = b"https://coinmarketcap.com/exchanges/volume/24-hour/all/"
 *         res = self._get(url)[45000:]             # <<<<<<<<<<<<<<
 *         convert = convert.lower()
 *         return processer.exchanges(res, convert)
 */
  __pyx_slice__14 = PySlice_New(__pyx_int_45000, Py_None, Py_None); if (unlikely(!__pyx_slice__14)) __PYX_ERR(0, 652, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_slice__14);
  __Pyx_GIVEREF(__pyx_slice__14);

  /* "pymarketcap/core.pyx":668
 *         """
 *         url = b"https://coinmarketcap.com/tokens/views/all/"
 *         res = self._get(url)[40000:]             # <<<<<<<<<<<<<<
 *         convert = convert.lower()
 * 
 */
  __pyx_slice__15 = PySlice_New(__pyx_int_40000, Py_None, Py_None); if (unlikely(!__pyx_slice__15)) __PYX_ERR(0, 668, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_slice__15);
  __Pyx_GIVEREF(__pyx_slice__15);

  /* "pymarketcap/core.pyx":542
 * 
 *     cpdef historical(self, unicode name,
 *                      start=datetime(2008, 8, 18),             # <<<<<<<<<<<<<<
 *                      end=datetime.now(),
 *                      revert=False):
 */
  __pyx_tuple__16 = PyTuple_Pack(3, __pyx_int_2008, __pyx_int_8, __pyx_int_18); if (unlikely(!__pyx_tuple__16)) __PYX_ERR(0, 542, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__16);
  __Pyx_GIVEREF(__pyx_tuple__16);
  __pyx_tuple__17 = PyTuple_Pack(3, __pyx_int_2008, __pyx_int_8, __pyx_int_18); if (unlikely(!__pyx_tuple__17)) __PYX_ERR(0, 542, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__17);
  __Pyx_GIVEREF(__pyx_tuple__17);

  /* "(tree fragment)":1
 * def __pyx_unpickle_Pymarketcap(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     if __pyx_checksum != 0x7d79c3c:
 *         from pickle import PickleError as __pyx_PickleError
 */
  __pyx_tuple__18 = PyTuple_Pack(5, __pyx_n_s_pyx_type, __pyx_n_s_pyx_checksum, __pyx_n_s_pyx_state, __pyx_n_s_pyx_PickleError, __pyx_n_s_pyx_result); if (unlikely(!__pyx_tuple__18)) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__18);
  __Pyx_GIVEREF(__pyx_tuple__18);
  __pyx_codeobj__19 = (PyObject*)__Pyx_PyCode_New(3, 0, 5, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__18, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_stringsource, __pyx_n_s_pyx_unpickle_Pymarketcap, 1, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__19)) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_RefNannyFinishContext();
  return 0;
  __pyx_L1_error:;
  __Pyx_RefNannyFinishContext();
  return -1;
}

static int __Pyx_InitGlobals(void) {
  __pyx_umethod_PyDict_Type_keys.type = (PyObject*)&PyDict_Type;
  __pyx_umethod_PyDict_Type_update.type = (PyObject*)&PyDict_Type;
  if (__Pyx_InitStrings(__pyx_string_tab) < 0) __PYX_ERR(0, 2, __pyx_L1_error);
  __pyx_int_0 = PyInt_FromLong(0); if (unlikely(!__pyx_int_0)) __PYX_ERR(0, 2, __pyx_L1_error)
  __pyx_int_1 = PyInt_FromLong(1); if (unlikely(!__pyx_int_1)) __PYX_ERR(0, 2, __pyx_L1_error)
  __pyx_int_8 = PyInt_FromLong(8); if (unlikely(!__pyx_int_8)) __PYX_ERR(0, 2, __pyx_L1_error)
  __pyx_int_16 = PyInt_FromLong(16); if (unlikely(!__pyx_int_16)) __PYX_ERR(0, 2, __pyx_L1_error)
  __pyx_int_18 = PyInt_FromLong(18); if (unlikely(!__pyx_int_18)) __PYX_ERR(0, 2, __pyx_L1_error)
  __pyx_int_32 = PyInt_FromLong(32); if (unlikely(!__pyx_int_32)) __PYX_ERR(0, 2, __pyx_L1_error)
  __pyx_int_64 = PyInt_FromLong(64); if (unlikely(!__pyx_int_64)) __PYX_ERR(0, 2, __pyx_L1_error)
  __pyx_int_128 = PyInt_FromLong(128); if (unlikely(!__pyx_int_128)) __PYX_ERR(0, 2, __pyx_L1_error)
  __pyx_int_200 = PyInt_FromLong(200); if (unlikely(!__pyx_int_200)) __PYX_ERR(0, 2, __pyx_L1_error)
  __pyx_int_403 = PyInt_FromLong(403); if (unlikely(!__pyx_int_403)) __PYX_ERR(0, 2, __pyx_L1_error)
  __pyx_int_600 = PyInt_FromLong(600); if (unlikely(!__pyx_int_600)) __PYX_ERR(0, 2, __pyx_L1_error)
  __pyx_int_2008 = PyInt_FromLong(2008); if (unlikely(!__pyx_int_2008)) __PYX_ERR(0, 2, __pyx_L1_error)
  __pyx_int_20000 = PyInt_FromLong(20000L); if (unlikely(!__pyx_int_20000)) __PYX_ERR(0, 2, __pyx_L1_error)
  __pyx_int_40000 = PyInt_FromLong(40000L); if (unlikely(!__pyx_int_40000)) __PYX_ERR(0, 2, __pyx_L1_error)
  __pyx_int_45000 = PyInt_FromLong(45000L); if (unlikely(!__pyx_int_45000)) __PYX_ERR(0, 2, __pyx_L1_error)
  __pyx_int_50000 = PyInt_FromLong(50000L); if (unlikely(!__pyx_int_50000)) __PYX_ERR(0, 2, __pyx_L1_error)
  __pyx_int_131570748 = PyInt_FromLong(131570748L); if (unlikely(!__pyx_int_131570748)) __PYX_ERR(0, 2, __pyx_L1_error)
  __pyx_int_neg_2000 = PyInt_FromLong(-2000); if (unlikely(!__pyx_int_neg_2000)) __PYX_ERR(0, 2, __pyx_L1_error)
  __pyx_int_neg_10000 = PyInt_FromLong(-10000L); if (unlikely(!__pyx_int_neg_10000)) __PYX_ERR(0, 2, __pyx_L1_error)
  return 0;
  __pyx_L1_error:;
  return -1;
}

static int __Pyx_modinit_global_init_code(void); /*proto*/
static int __Pyx_modinit_variable_export_code(void); /*proto*/
static int __Pyx_modinit_function_export_code(void); /*proto*/
static int __Pyx_modinit_type_init_code(void); /*proto*/
static int __Pyx_modinit_type_import_code(void); /*proto*/
static int __Pyx_modinit_variable_import_code(void); /*proto*/
static int __Pyx_modinit_function_import_code(void); /*proto*/

static int __Pyx_modinit_global_init_code(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_modinit_global_init_code", 0);
  /*--- Global init code ---*/
  __Pyx_RefNannyFinishContext();
  return 0;
}

static int __Pyx_modinit_variable_export_code(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_modinit_variable_export_code", 0);
  /*--- Variable export code ---*/
  __Pyx_RefNannyFinishContext();
  return 0;
}

static int __Pyx_modinit_function_export_code(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_modinit_function_export_code", 0);
  /*--- Function export code ---*/
  __Pyx_RefNannyFinishContext();
  return 0;
}

static int __Pyx_modinit_type_init_code(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_modinit_type_init_code", 0);
  /*--- Type init code ---*/
  __pyx_vtabptr_11pymarketcap_4core_Pymarketcap = &__pyx_vtable_11pymarketcap_4core_Pymarketcap;
  __pyx_vtable_11pymarketcap_4core_Pymarketcap._is_symbol = (PyObject *(*)(struct __pyx_obj_11pymarketcap_4core_Pymarketcap *, PyObject *, int __pyx_skip_dispatch))__pyx_f_11pymarketcap_4core_11Pymarketcap__is_symbol;
  __pyx_vtable_11pymarketcap_4core_Pymarketcap._quick_search = (PyObject *(*)(struct __pyx_obj_11pymarketcap_4core_Pymarketcap *, int __pyx_skip_dispatch, struct __pyx_opt_args_11pymarketcap_4core_11Pymarketcap__quick_search *__pyx_optional_args))__pyx_f_11pymarketcap_4core_11Pymarketcap__quick_search;
  __pyx_vtable_11pymarketcap_4core_Pymarketcap._cache_symbols_ids = (PyObject *(*)(struct __pyx_obj_11pymarketcap_4core_Pymarketcap *, int __pyx_skip_dispatch))__pyx_f_11pymarketcap_4core_11Pymarketcap__cache_symbols_ids;
  __pyx_vtable_11pymarketcap_4core_Pymarketcap._cache_exchanges_ids = (PyObject *(*)(struct __pyx_obj_11pymarketcap_4core_Pymarketcap *, int __pyx_skip_dispatch))__pyx_f_11pymarketcap_4core_11Pymarketcap__cache_exchanges_ids;
  __pyx_vtable_11pymarketcap_4core_Pymarketcap.__pyx___currencies_to_convert = (PyObject *(*)(struct __pyx_obj_11pymarketcap_4core_Pymarketcap *, int __pyx_skip_dispatch))__pyx_f_11pymarketcap_4core_11Pymarketcap___currencies_to_convert;
  __pyx_vtable_11pymarketcap_4core_Pymarketcap.__pyx___exchange_names_slugs = (PyObject *(*)(struct __pyx_obj_11pymarketcap_4core_Pymarketcap *, int __pyx_skip_dispatch))__pyx_f_11pymarketcap_4core_11Pymarketcap___exchange_names_slugs;
  __pyx_vtable_11pymarketcap_4core_Pymarketcap._get = (PyObject *(*)(struct __pyx_obj_11pymarketcap_4core_Pymarketcap *, char *))__pyx_f_11pymarketcap_4core_11Pymarketcap__get;
  __pyx_vtable_11pymarketcap_4core_Pymarketcap.stats = (PyObject *(*)(struct __pyx_obj_11pymarketcap_4core_Pymarketcap *, int __pyx_skip_dispatch, struct __pyx_opt_args_11pymarketcap_4core_11Pymarketcap_stats *__pyx_optional_args))__pyx_f_11pymarketcap_4core_11Pymarketcap_stats;
  __pyx_vtable_11pymarketcap_4core_Pymarketcap.ticker = (PyObject *(*)(struct __pyx_obj_11pymarketcap_4core_Pymarketcap *, int __pyx_skip_dispatch, struct __pyx_opt_args_11pymarketcap_4core_11Pymarketcap_ticker *__pyx_optional_args))__pyx_f_11pymarketcap_4core_11Pymarketcap_ticker;
  __pyx_vtable_11pymarketcap_4core_Pymarketcap.convert = (PyObject *(*)(struct __pyx_obj_11pymarketcap_4core_Pymarketcap *, PyObject *, PyObject *, PyObject *, int __pyx_skip_dispatch))__pyx_f_11pymarketcap_4core_11Pymarketcap_convert;
  __pyx_vtable_11pymarketcap_4core_Pymarketcap.currency = (PyObject *(*)(struct __pyx_obj_11pymarketcap_4core_Pymarketcap *, PyObject *, int __pyx_skip_dispatch, struct __pyx_opt_args_11pymarketcap_4core_11Pymarketcap_currency *__pyx_optional_args))__pyx_f_11pymarketcap_4core_11Pymarketcap_currency;
  __pyx_vtable_11pymarketcap_4core_Pymarketcap.markets = (PyObject *(*)(struct __pyx_obj_11pymarketcap_4core_Pymarketcap *, PyObject *, int __pyx_skip_dispatch, struct __pyx_opt_args_11pymarketcap_4core_11Pymarketcap_markets *__pyx_optional_args))__pyx_f_11pymarketcap_4core_11Pymarketcap_markets;
  __pyx_vtable_11pymarketcap_4core_Pymarketcap.ranks = (PyObject *(*)(struct __pyx_obj_11pymarketcap_4core_Pymarketcap *, int __pyx_skip_dispatch))__pyx_f_11pymarketcap_4core_11Pymarketcap_ranks;
  __pyx_vtable_11pymarketcap_4core_Pymarketcap.historical = (PyObject *(*)(struct __pyx_obj_11pymarketcap_4core_Pymarketcap *, PyObject *, int __pyx_skip_dispatch, struct __pyx_opt_args_11pymarketcap_4core_11Pymarketcap_historical *__pyx_optional_args))__pyx_f_11pymarketcap_4core_11Pymarketcap_historical;
  __pyx_vtable_11pymarketcap_4core_Pymarketcap.recently = (PyObject *(*)(struct __pyx_obj_11pymarketcap_4core_Pymarketcap *, int __pyx_skip_dispatch, struct __pyx_opt_args_11pymarketcap_4core_11Pymarketcap_recently *__pyx_optional_args))__pyx_f_11pymarketcap_4core_11Pymarketcap_recently;
  __pyx_vtable_11pymarketcap_4core_Pymarketcap.exchange = (PyObject *(*)(struct __pyx_obj_11pymarketcap_4core_Pymarketcap *, PyObject *, int __pyx_skip_dispatch, struct __pyx_opt_args_11pymarketcap_4core_11Pymarketcap_exchange *__pyx_optional_args))__pyx_f_11pymarketcap_4core_11Pymarketcap_exchange;
  __pyx_vtable_11pymarketcap_4core_Pymarketcap.exchanges = (PyObject *(*)(struct __pyx_obj_11pymarketcap_4core_Pymarketcap *, int __pyx_skip_dispatch, struct __pyx_opt_args_11pymarketcap_4core_11Pymarketcap_exchanges *__pyx_optional_args))__pyx_f_11pymarketcap_4core_11Pymarketcap_exchanges;
  __pyx_vtable_11pymarketcap_4core_Pymarketcap.tokens = (PyObject *(*)(struct __pyx_obj_11pymarketcap_4core_Pymarketcap *, int __pyx_skip_dispatch, struct __pyx_opt_args_11pymarketcap_4core_11Pymarketcap_tokens *__pyx_optional_args))__pyx_f_11pymarketcap_4core_11Pymarketcap_tokens;
  __pyx_vtable_11pymarketcap_4core_Pymarketcap._currency = (PyObject *(*)(struct __pyx_obj_11pymarketcap_4core_Pymarketcap *, PyObject *, int __pyx_skip_dispatch, struct __pyx_opt_args_11pymarketcap_4core_11Pymarketcap__currency *__pyx_optional_args))__pyx_f_11pymarketcap_4core_11Pymarketcap__currency;
  __pyx_vtable_11pymarketcap_4core_Pymarketcap._global_cap = (PyObject *(*)(struct __pyx_obj_11pymarketcap_4core_Pymarketcap *, int __pyx_skip_dispatch, struct __pyx_opt_args_11pymarketcap_4core_11Pymarketcap__global_cap *__pyx_optional_args))__pyx_f_11pymarketcap_4core_11Pymarketcap__global_cap;
  __pyx_vtable_11pymarketcap_4core_Pymarketcap._dominance = (PyObject *(*)(struct __pyx_obj_11pymarketcap_4core_Pymarketcap *, int __pyx_skip_dispatch, struct __pyx_opt_args_11pymarketcap_4core_11Pymarketcap__dominance *__pyx_optional_args))__pyx_f_11pymarketcap_4core_11Pymarketcap__dominance;
  __pyx_vtable_11pymarketcap_4core_Pymarketcap.download_logo = (PyObject *(*)(struct __pyx_obj_11pymarketcap_4core_Pymarketcap *, PyObject *, int __pyx_skip_dispatch, struct __pyx_opt_args_11pymarketcap_4core_11Pymarketcap_download_logo *__pyx_optional_args))__pyx_f_11pymarketcap_4core_11Pymarketcap_download_logo;
  __pyx_vtable_11pymarketcap_4core_Pymarketcap.download_exchange_logo = (PyObject *(*)(struct __pyx_obj_11pymarketcap_4core_Pymarketcap *, PyObject *, int __pyx_skip_dispatch, struct __pyx_opt_args_11pymarketcap_4core_11Pymarketcap_download_exchange_logo *__pyx_optional_args))__pyx_f_11pymarketcap_4core_11Pymarketcap_download_exchange_logo;
  if (PyType_Ready(&__pyx_type_11pymarketcap_4core_Pymarketcap) < 0) __PYX_ERR(0, 38, __pyx_L1_error)
  __pyx_type_11pymarketcap_4core_Pymarketcap.tp_print = 0;
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_type_11pymarketcap_4core_Pymarketcap.tp_dictoffset && __pyx_type_11pymarketcap_4core_Pymarketcap.tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_type_11pymarketcap_4core_Pymarketcap.tp_getattro = __Pyx_PyObject_GenericGetAttr;
  }
  if (__Pyx_SetVtable(__pyx_type_11pymarketcap_4core_Pymarketcap.tp_dict, __pyx_vtabptr_11pymarketcap_4core_Pymarketcap) < 0) __PYX_ERR(0, 38, __pyx_L1_error)
  if (PyObject_SetAttrString(__pyx_m, "Pymarketcap", (PyObject *)&__pyx_type_11pymarketcap_4core_Pymarketcap) < 0) __PYX_ERR(0, 38, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject*)&__pyx_type_11pymarketcap_4core_Pymarketcap) < 0) __PYX_ERR(0, 38, __pyx_L1_error)
  __pyx_ptype_11pymarketcap_4core_Pymarketcap = &__pyx_type_11pymarketcap_4core_Pymarketcap;
  __Pyx_RefNannyFinishContext();
  return 0;
  __pyx_L1_error:;
  __Pyx_RefNannyFinishContext();
  return -1;
}

static int __Pyx_modinit_type_import_code(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_modinit_type_import_code", 0);
  /*--- Type import code ---*/
  __Pyx_RefNannyFinishContext();
  return 0;
}

static int __Pyx_modinit_variable_import_code(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_modinit_variable_import_code", 0);
  /*--- Variable import code ---*/
  __Pyx_RefNannyFinishContext();
  return 0;
}

static int __Pyx_modinit_function_import_code(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_modinit_function_import_code", 0);
  /*--- Function import code ---*/
  __Pyx_RefNannyFinishContext();
  return 0;
}


#if PY_MAJOR_VERSION < 3
#ifdef CYTHON_NO_PYINIT_EXPORT
#define __Pyx_PyMODINIT_FUNC void
#else
#define __Pyx_PyMODINIT_FUNC PyMODINIT_FUNC
#endif
#else
#ifdef CYTHON_NO_PYINIT_EXPORT
#define __Pyx_PyMODINIT_FUNC PyObject *
#else
#define __Pyx_PyMODINIT_FUNC PyMODINIT_FUNC
#endif
#endif
#ifndef CYTHON_SMALL_CODE
#if defined(__clang__)
    #define CYTHON_SMALL_CODE
#elif defined(__GNUC__)
    #define CYTHON_SMALL_CODE __attribute__((optimize("Os")))
#else
    #define CYTHON_SMALL_CODE
#endif
#endif


#if PY_MAJOR_VERSION < 3
__Pyx_PyMODINIT_FUNC initcore(void) CYTHON_SMALL_CODE; /*proto*/
__Pyx_PyMODINIT_FUNC initcore(void)
#else
__Pyx_PyMODINIT_FUNC PyInit_core(void) CYTHON_SMALL_CODE; /*proto*/
__Pyx_PyMODINIT_FUNC PyInit_core(void)
#if CYTHON_PEP489_MULTI_PHASE_INIT
{
  return PyModuleDef_Init(&__pyx_moduledef);
}
static int __Pyx_copy_spec_to_module(PyObject *spec, PyObject *moddict, const char* from_name, const char* to_name) {
    PyObject *value = PyObject_GetAttrString(spec, from_name);
    int result = 0;
    if (likely(value)) {
        result = PyDict_SetItemString(moddict, to_name, value);
        Py_DECREF(value);
    } else if (PyErr_ExceptionMatches(PyExc_AttributeError)) {
        PyErr_Clear();
    } else {
        result = -1;
    }
    return result;
}
static PyObject* __pyx_pymod_create(PyObject *spec, CYTHON_UNUSED PyModuleDef *def) {
    PyObject *module = NULL, *moddict, *modname;
    if (__pyx_m)
        return __Pyx_NewRef(__pyx_m);
    modname = PyObject_GetAttrString(spec, "name");
    if (unlikely(!modname)) goto bad;
    module = PyModule_NewObject(modname);
    Py_DECREF(modname);
    if (unlikely(!module)) goto bad;
    moddict = PyModule_GetDict(module);
    if (unlikely(!moddict)) goto bad;
    if (unlikely(__Pyx_copy_spec_to_module(spec, moddict, "loader", "__loader__") < 0)) goto bad;
    if (unlikely(__Pyx_copy_spec_to_module(spec, moddict, "origin", "__file__") < 0)) goto bad;
    if (unlikely(__Pyx_copy_spec_to_module(spec, moddict, "parent", "__package__") < 0)) goto bad;
    if (unlikely(__Pyx_copy_spec_to_module(spec, moddict, "submodule_search_locations", "__path__") < 0)) goto bad;
    return module;
bad:
    Py_XDECREF(module);
    return NULL;
}


static int __pyx_pymod_exec_core(PyObject *__pyx_pyinit_module)
#endif
#endif
{
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  Py_ssize_t __pyx_t_5;
  PyObject *(*__pyx_t_6)(PyObject *);
  __Pyx_RefNannyDeclarations
  #if CYTHON_PEP489_MULTI_PHASE_INIT
  if (__pyx_m && __pyx_m == __pyx_pyinit_module) return 0;
  #elif PY_MAJOR_VERSION >= 3
  if (__pyx_m) return __Pyx_NewRef(__pyx_m);
  #endif
  #if CYTHON_REFNANNY
__Pyx_RefNanny = __Pyx_RefNannyImportAPI("refnanny");
if (!__Pyx_RefNanny) {
  PyErr_Clear();
  __Pyx_RefNanny = __Pyx_RefNannyImportAPI("Cython.Runtime.refnanny");
  if (!__Pyx_RefNanny)
      Py_FatalError("failed to import 'refnanny' module");
}
#endif
  __Pyx_RefNannySetupContext("__Pyx_PyMODINIT_FUNC PyInit_core(void)", 0);
  if (__Pyx_check_binary_version() < 0) __PYX_ERR(0, 2, __pyx_L1_error)
  __pyx_empty_tuple = PyTuple_New(0); if (unlikely(!__pyx_empty_tuple)) __PYX_ERR(0, 2, __pyx_L1_error)
  __pyx_empty_bytes = PyBytes_FromStringAndSize("", 0); if (unlikely(!__pyx_empty_bytes)) __PYX_ERR(0, 2, __pyx_L1_error)
  __pyx_empty_unicode = PyUnicode_FromStringAndSize("", 0); if (unlikely(!__pyx_empty_unicode)) __PYX_ERR(0, 2, __pyx_L1_error)
  #ifdef __Pyx_CyFunction_USED
  if (__pyx_CyFunction_init() < 0) __PYX_ERR(0, 2, __pyx_L1_error)
  #endif
  #ifdef __Pyx_FusedFunction_USED
  if (__pyx_FusedFunction_init() < 0) __PYX_ERR(0, 2, __pyx_L1_error)
  #endif
  #ifdef __Pyx_Coroutine_USED
  if (__pyx_Coroutine_init() < 0) __PYX_ERR(0, 2, __pyx_L1_error)
  #endif
  #ifdef __Pyx_Generator_USED
  if (__pyx_Generator_init() < 0) __PYX_ERR(0, 2, __pyx_L1_error)
  #endif
  #ifdef __Pyx_AsyncGen_USED
  if (__pyx_AsyncGen_init() < 0) __PYX_ERR(0, 2, __pyx_L1_error)
  #endif
  #ifdef __Pyx_StopAsyncIteration_USED
  if (__pyx_StopAsyncIteration_init() < 0) __PYX_ERR(0, 2, __pyx_L1_error)
  #endif
  /*--- Library function declarations ---*/
  /*--- Threads initialization code ---*/
  #if defined(__PYX_FORCE_INIT_THREADS) && __PYX_FORCE_INIT_THREADS
  #ifdef WITH_THREAD /* Python build with threading support? */
  PyEval_InitThreads();
  #endif
  #endif
  /*--- Module creation code ---*/
  #if CYTHON_PEP489_MULTI_PHASE_INIT
  __pyx_m = __pyx_pyinit_module;
  Py_INCREF(__pyx_m);
  #else
  #if PY_MAJOR_VERSION < 3
  __pyx_m = Py_InitModule4("core", __pyx_methods, __pyx_k_API_wraper_and_web_scraper_modul, 0, PYTHON_API_VERSION); Py_XINCREF(__pyx_m);
  #else
  __pyx_m = PyModule_Create(&__pyx_moduledef);
  #endif
  if (unlikely(!__pyx_m)) __PYX_ERR(0, 2, __pyx_L1_error)
  #endif
  __pyx_d = PyModule_GetDict(__pyx_m); if (unlikely(!__pyx_d)) __PYX_ERR(0, 2, __pyx_L1_error)
  Py_INCREF(__pyx_d);
  __pyx_b = PyImport_AddModule(__Pyx_BUILTIN_MODULE_NAME); if (unlikely(!__pyx_b)) __PYX_ERR(0, 2, __pyx_L1_error)
  __pyx_cython_runtime = PyImport_AddModule((char *) "cython_runtime"); if (unlikely(!__pyx_cython_runtime)) __PYX_ERR(0, 2, __pyx_L1_error)
  #if CYTHON_COMPILING_IN_PYPY
  Py_INCREF(__pyx_b);
  #endif
  if (PyObject_SetAttrString(__pyx_m, "__builtins__", __pyx_b) < 0) __PYX_ERR(0, 2, __pyx_L1_error);
  /*--- Initialize various global constants etc. ---*/
  if (__Pyx_InitGlobals() < 0) __PYX_ERR(0, 2, __pyx_L1_error)
  #if PY_MAJOR_VERSION < 3 && (__PYX_DEFAULT_STRING_ENCODING_IS_ASCII || __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT)
  if (__Pyx_init_sys_getdefaultencoding_params() < 0) __PYX_ERR(0, 2, __pyx_L1_error)
  #endif
  if (__pyx_module_is_main_pymarketcap__core) {
    if (PyObject_SetAttrString(__pyx_m, "__name__", __pyx_n_s_main) < 0) __PYX_ERR(0, 2, __pyx_L1_error)
  }
  #if PY_MAJOR_VERSION >= 3
  {
    PyObject *modules = PyImport_GetModuleDict(); if (unlikely(!modules)) __PYX_ERR(0, 2, __pyx_L1_error)
    if (!PyDict_GetItemString(modules, "pymarketcap.core")) {
      if (unlikely(PyDict_SetItemString(modules, "pymarketcap.core", __pyx_m) < 0)) __PYX_ERR(0, 2, __pyx_L1_error)
    }
  }
  #endif
  /*--- Builtin init code ---*/
  if (__Pyx_InitCachedBuiltins() < 0) __PYX_ERR(0, 2, __pyx_L1_error)
  /*--- Constants init code ---*/
  if (__Pyx_InitCachedConstants() < 0) __PYX_ERR(0, 2, __pyx_L1_error)
  /*--- Global type/function init code ---*/
  (void)__Pyx_modinit_global_init_code();
  (void)__Pyx_modinit_variable_export_code();
  (void)__Pyx_modinit_function_export_code();
  if (unlikely(__Pyx_modinit_type_init_code() != 0)) goto __pyx_L1_error;
  (void)__Pyx_modinit_type_import_code();
  (void)__Pyx_modinit_variable_import_code();
  (void)__Pyx_modinit_function_import_code();
  /*--- Execution code ---*/
  #if defined(__Pyx_Generator_USED) || defined(__Pyx_Coroutine_USED)
  if (__Pyx_patch_abc() < 0) __PYX_ERR(0, 2, __pyx_L1_error)
  #endif

  /* "pymarketcap/core.pyx":5
 * 
 * # Standard Python modules
 * from re import sub as re_sub             # <<<<<<<<<<<<<<
 * from re import findall as re_findall
 * from json import loads
 */
  __pyx_t_1 = PyList_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_n_s_sub);
  __Pyx_GIVEREF(__pyx_n_s_sub);
  PyList_SET_ITEM(__pyx_t_1, 0, __pyx_n_s_sub);
  __pyx_t_2 = __Pyx_Import(__pyx_n_s_re, __pyx_t_1, -1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_ImportFrom(__pyx_t_2, __pyx_n_s_sub); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_re_sub, __pyx_t_1) < 0) __PYX_ERR(0, 5, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "pymarketcap/core.pyx":6
 * # Standard Python modules
 * from re import sub as re_sub
 * from re import findall as re_findall             # <<<<<<<<<<<<<<
 * from json import loads
 * from datetime import datetime
 */
  __pyx_t_2 = PyList_New(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 6, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_INCREF(__pyx_n_s_findall);
  __Pyx_GIVEREF(__pyx_n_s_findall);
  PyList_SET_ITEM(__pyx_t_2, 0, __pyx_n_s_findall);
  __pyx_t_1 = __Pyx_Import(__pyx_n_s_re, __pyx_t_2, -1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 6, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_ImportFrom(__pyx_t_1, __pyx_n_s_findall); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 6, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_re_findall, __pyx_t_2) < 0) __PYX_ERR(0, 6, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "pymarketcap/core.pyx":7
 * from re import sub as re_sub
 * from re import findall as re_findall
 * from json import loads             # <<<<<<<<<<<<<<
 * from datetime import datetime
 * from time import time
 */
  __pyx_t_1 = PyList_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 7, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_n_s_loads);
  __Pyx_GIVEREF(__pyx_n_s_loads);
  PyList_SET_ITEM(__pyx_t_1, 0, __pyx_n_s_loads);
  __pyx_t_2 = __Pyx_Import(__pyx_n_s_json, __pyx_t_1, -1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 7, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_ImportFrom(__pyx_t_2, __pyx_n_s_loads); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 7, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_loads, __pyx_t_1) < 0) __PYX_ERR(0, 7, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "pymarketcap/core.pyx":8
 * from re import findall as re_findall
 * from json import loads
 * from datetime import datetime             # <<<<<<<<<<<<<<
 * from time import time
 * from urllib.request import urlretrieve
 */
  __pyx_t_2 = PyList_New(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 8, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_INCREF(__pyx_n_s_datetime);
  __Pyx_GIVEREF(__pyx_n_s_datetime);
  PyList_SET_ITEM(__pyx_t_2, 0, __pyx_n_s_datetime);
  __pyx_t_1 = __Pyx_Import(__pyx_n_s_datetime, __pyx_t_2, -1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 8, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_ImportFrom(__pyx_t_1, __pyx_n_s_datetime); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 8, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_datetime, __pyx_t_2) < 0) __PYX_ERR(0, 8, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "pymarketcap/core.pyx":9
 * from json import loads
 * from datetime import datetime
 * from time import time             # <<<<<<<<<<<<<<
 * from urllib.request import urlretrieve
 * from urllib.error import HTTPError
 */
  __pyx_t_1 = PyList_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 9, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_n_s_time);
  __Pyx_GIVEREF(__pyx_n_s_time);
  PyList_SET_ITEM(__pyx_t_1, 0, __pyx_n_s_time);
  __pyx_t_2 = __Pyx_Import(__pyx_n_s_time, __pyx_t_1, -1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 9, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_ImportFrom(__pyx_t_2, __pyx_n_s_time); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 9, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_time, __pyx_t_1) < 0) __PYX_ERR(0, 9, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "pymarketcap/core.pyx":10
 * from datetime import datetime
 * from time import time
 * from urllib.request import urlretrieve             # <<<<<<<<<<<<<<
 * from urllib.error import HTTPError
 * 
 */
  __pyx_t_2 = PyList_New(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 10, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_INCREF(__pyx_n_s_urlretrieve);
  __Pyx_GIVEREF(__pyx_n_s_urlretrieve);
  PyList_SET_ITEM(__pyx_t_2, 0, __pyx_n_s_urlretrieve);
  __pyx_t_1 = __Pyx_Import(__pyx_n_s_urllib_request, __pyx_t_2, -1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 10, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_ImportFrom(__pyx_t_1, __pyx_n_s_urlretrieve); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 10, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_urlretrieve, __pyx_t_2) < 0) __PYX_ERR(0, 10, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "pymarketcap/core.pyx":11
 * from time import time
 * from urllib.request import urlretrieve
 * from urllib.error import HTTPError             # <<<<<<<<<<<<<<
 * 
 * # Internal Cython modules
 */
  __pyx_t_1 = PyList_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 11, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_n_s_HTTPError);
  __Pyx_GIVEREF(__pyx_n_s_HTTPError);
  PyList_SET_ITEM(__pyx_t_1, 0, __pyx_n_s_HTTPError);
  __pyx_t_2 = __Pyx_Import(__pyx_n_s_urllib_error, __pyx_t_1, -1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 11, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_ImportFrom(__pyx_t_2, __pyx_n_s_HTTPError); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 11, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_HTTPError, __pyx_t_1) < 0) __PYX_ERR(0, 11, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "pymarketcap/core.pyx":14
 * 
 * # Internal Cython modules
 * from pymarketcap.curl import get_to_memory             # <<<<<<<<<<<<<<
 * from pymarketcap import processer
 * 
 */
  __pyx_t_2 = PyList_New(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 14, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_INCREF(__pyx_n_s_get_to_memory);
  __Pyx_GIVEREF(__pyx_n_s_get_to_memory);
  PyList_SET_ITEM(__pyx_t_2, 0, __pyx_n_s_get_to_memory);
  __pyx_t_1 = __Pyx_Import(__pyx_n_s_pymarketcap_curl, __pyx_t_2, -1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 14, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_ImportFrom(__pyx_t_1, __pyx_n_s_get_to_memory); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 14, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_get_to_memory, __pyx_t_2) < 0) __PYX_ERR(0, 14, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "pymarketcap/core.pyx":15
 * # Internal Cython modules
 * from pymarketcap.curl import get_to_memory
 * from pymarketcap import processer             # <<<<<<<<<<<<<<
 * 
 * # Internal Python modules
 */
  __pyx_t_1 = PyList_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 15, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_n_s_processer);
  __Pyx_GIVEREF(__pyx_n_s_processer);
  PyList_SET_ITEM(__pyx_t_1, 0, __pyx_n_s_processer);
  __pyx_t_2 = __Pyx_Import(__pyx_n_s_pymarketcap, __pyx_t_1, -1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 15, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_ImportFrom(__pyx_t_2, __pyx_n_s_processer); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 15, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_processer, __pyx_t_1) < 0) __PYX_ERR(0, 15, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "pymarketcap/core.pyx":19
 * # Internal Python modules
 * from pymarketcap.consts import (
 *     DEFAULT_TIMEOUT,             # <<<<<<<<<<<<<<
 *     EXCEPTIONAL_COIN_SLUGS,
 *     INVALID_COINS
 */
  __pyx_t_2 = PyList_New(3); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 19, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_INCREF(__pyx_n_s_DEFAULT_TIMEOUT);
  __Pyx_GIVEREF(__pyx_n_s_DEFAULT_TIMEOUT);
  PyList_SET_ITEM(__pyx_t_2, 0, __pyx_n_s_DEFAULT_TIMEOUT);
  __Pyx_INCREF(__pyx_n_s_EXCEPTIONAL_COIN_SLUGS);
  __Pyx_GIVEREF(__pyx_n_s_EXCEPTIONAL_COIN_SLUGS);
  PyList_SET_ITEM(__pyx_t_2, 1, __pyx_n_s_EXCEPTIONAL_COIN_SLUGS);
  __Pyx_INCREF(__pyx_n_s_INVALID_COINS);
  __Pyx_GIVEREF(__pyx_n_s_INVALID_COINS);
  PyList_SET_ITEM(__pyx_t_2, 2, __pyx_n_s_INVALID_COINS);

  /* "pymarketcap/core.pyx":18
 * 
 * # Internal Python modules
 * from pymarketcap.consts import (             # <<<<<<<<<<<<<<
 *     DEFAULT_TIMEOUT,
 *     EXCEPTIONAL_COIN_SLUGS,
 */
  __pyx_t_1 = __Pyx_Import(__pyx_n_s_pymarketcap_consts, __pyx_t_2, -1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 18, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_ImportFrom(__pyx_t_1, __pyx_n_s_DEFAULT_TIMEOUT); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 18, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_DEFAULT_TIMEOUT, __pyx_t_2) < 0) __PYX_ERR(0, 19, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_ImportFrom(__pyx_t_1, __pyx_n_s_EXCEPTIONAL_COIN_SLUGS); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 18, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_EXCEPTIONAL_COIN_SLUGS, __pyx_t_2) < 0) __PYX_ERR(0, 20, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_ImportFrom(__pyx_t_1, __pyx_n_s_INVALID_COINS); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 18, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_INVALID_COINS, __pyx_t_2) < 0) __PYX_ERR(0, 21, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "pymarketcap/core.pyx":24
 * )
 * from pymarketcap.errors import (
 *     CoinmarketcapHTTPError,             # <<<<<<<<<<<<<<
 *     CoinmarketcapHTTPError404,
 *     CoinmarketcapTooManyRequestsError
 */
  __pyx_t_1 = PyList_New(3); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 24, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_n_s_CoinmarketcapHTTPError);
  __Pyx_GIVEREF(__pyx_n_s_CoinmarketcapHTTPError);
  PyList_SET_ITEM(__pyx_t_1, 0, __pyx_n_s_CoinmarketcapHTTPError);
  __Pyx_INCREF(__pyx_n_s_CoinmarketcapHTTPError404);
  __Pyx_GIVEREF(__pyx_n_s_CoinmarketcapHTTPError404);
  PyList_SET_ITEM(__pyx_t_1, 1, __pyx_n_s_CoinmarketcapHTTPError404);
  __Pyx_INCREF(__pyx_n_s_CoinmarketcapTooManyRequestsErro);
  __Pyx_GIVEREF(__pyx_n_s_CoinmarketcapTooManyRequestsErro);
  PyList_SET_ITEM(__pyx_t_1, 2, __pyx_n_s_CoinmarketcapTooManyRequestsErro);

  /* "pymarketcap/core.pyx":23
 *     INVALID_COINS
 * )
 * from pymarketcap.errors import (             # <<<<<<<<<<<<<<
 *     CoinmarketcapHTTPError,
 *     CoinmarketcapHTTPError404,
 */
  __pyx_t_2 = __Pyx_Import(__pyx_n_s_pymarketcap_errors, __pyx_t_1, -1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 23, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_ImportFrom(__pyx_t_2, __pyx_n_s_CoinmarketcapHTTPError); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 23, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_CoinmarketcapHTTPError, __pyx_t_1) < 0) __PYX_ERR(0, 24, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_ImportFrom(__pyx_t_2, __pyx_n_s_CoinmarketcapHTTPError404); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 23, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_CoinmarketcapHTTPError404, __pyx_t_1) < 0) __PYX_ERR(0, 25, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_ImportFrom(__pyx_t_2, __pyx_n_s_CoinmarketcapTooManyRequestsErro); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 23, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_CoinmarketcapTooManyRequestsErro, __pyx_t_1) < 0) __PYX_ERR(0, 26, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "pymarketcap/core.pyx":31
 * # HTTP errors mapper
 * http_errors_map = {
 *     "429": CoinmarketcapTooManyRequestsError,             # <<<<<<<<<<<<<<
 *     "404": CoinmarketcapHTTPError404,
 * }
 */
  __pyx_t_2 = __Pyx_PyDict_NewPresized(2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 31, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = __Pyx_GetModuleGlobalName(__pyx_n_s_CoinmarketcapTooManyRequestsErro); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 31, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_t_2, __pyx_kp_s_429, __pyx_t_1) < 0) __PYX_ERR(0, 31, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "pymarketcap/core.pyx":32
 * http_errors_map = {
 *     "429": CoinmarketcapTooManyRequestsError,
 *     "404": CoinmarketcapHTTPError404,             # <<<<<<<<<<<<<<
 * }
 * 
 */
  __pyx_t_1 = __Pyx_GetModuleGlobalName(__pyx_n_s_CoinmarketcapHTTPError404); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 32, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_t_2, __pyx_kp_s_404, __pyx_t_1) < 0) __PYX_ERR(0, 31, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_http_errors_map, __pyx_t_2) < 0) __PYX_ERR(0, 30, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "pymarketcap/core.pyx":35
 * }
 * 
 * http_error_numbers = [int(number) for number in http_errors_map.keys()]             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_t_2 = PyList_New(0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 35, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_GetModuleGlobalName(__pyx_n_s_http_errors_map); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 35, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_keys); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 35, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = NULL;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_4))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_4);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_4, function);
    }
  }
  if (__pyx_t_3) {
    __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_4, __pyx_t_3); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 35, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  } else {
    __pyx_t_1 = __Pyx_PyObject_CallNoArg(__pyx_t_4); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 35, __pyx_L1_error)
  }
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (likely(PyList_CheckExact(__pyx_t_1)) || PyTuple_CheckExact(__pyx_t_1)) {
    __pyx_t_4 = __pyx_t_1; __Pyx_INCREF(__pyx_t_4); __pyx_t_5 = 0;
    __pyx_t_6 = NULL;
  } else {
    __pyx_t_5 = -1; __pyx_t_4 = PyObject_GetIter(__pyx_t_1); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 35, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_6 = Py_TYPE(__pyx_t_4)->tp_iternext; if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 35, __pyx_L1_error)
  }
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  for (;;) {
    if (likely(!__pyx_t_6)) {
      if (likely(PyList_CheckExact(__pyx_t_4))) {
        if (__pyx_t_5 >= PyList_GET_SIZE(__pyx_t_4)) break;
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_1 = PyList_GET_ITEM(__pyx_t_4, __pyx_t_5); __Pyx_INCREF(__pyx_t_1); __pyx_t_5++; if (unlikely(0 < 0)) __PYX_ERR(0, 35, __pyx_L1_error)
        #else
        __pyx_t_1 = PySequence_ITEM(__pyx_t_4, __pyx_t_5); __pyx_t_5++; if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 35, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_1);
        #endif
      } else {
        if (__pyx_t_5 >= PyTuple_GET_SIZE(__pyx_t_4)) break;
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_1 = PyTuple_GET_ITEM(__pyx_t_4, __pyx_t_5); __Pyx_INCREF(__pyx_t_1); __pyx_t_5++; if (unlikely(0 < 0)) __PYX_ERR(0, 35, __pyx_L1_error)
        #else
        __pyx_t_1 = PySequence_ITEM(__pyx_t_4, __pyx_t_5); __pyx_t_5++; if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 35, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_1);
        #endif
      }
    } else {
      __pyx_t_1 = __pyx_t_6(__pyx_t_4);
      if (unlikely(!__pyx_t_1)) {
        PyObject* exc_type = PyErr_Occurred();
        if (exc_type) {
          if (likely(__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
          else __PYX_ERR(0, 35, __pyx_L1_error)
        }
        break;
      }
      __Pyx_GOTREF(__pyx_t_1);
    }
    if (PyDict_SetItem(__pyx_d, __pyx_n_s_number, __pyx_t_1) < 0) __PYX_ERR(0, 35, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_1 = __Pyx_GetModuleGlobalName(__pyx_n_s_number); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 35, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_3 = __Pyx_PyNumber_Int(__pyx_t_1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 35, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    if (unlikely(__Pyx_ListComp_Append(__pyx_t_2, (PyObject*)__pyx_t_3))) __PYX_ERR(0, 35, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  }
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_http_error_numbers, __pyx_t_2) < 0) __PYX_ERR(0, 35, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "pymarketcap/core.pyx":64
 *     cdef public bint debug
 * 
 *     def __init__(self, timeout=DEFAULT_TIMEOUT, debug=False):             # <<<<<<<<<<<<<<
 *         self.timeout = timeout
 *         self.debug = debug
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_DEFAULT_TIMEOUT); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 64, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_k_ = __pyx_t_2;
  __Pyx_GIVEREF(__pyx_t_2);
  __pyx_t_2 = 0;

  /* "pymarketcap/core.pyx":542
 * 
 *     cpdef historical(self, unicode name,
 *                      start=datetime(2008, 8, 18),             # <<<<<<<<<<<<<<
 *                      end=datetime.now(),
 *                      revert=False):
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_datetime); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 542, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_tuple__16, NULL); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 542, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_k__9 = __pyx_t_4;
  __Pyx_GIVEREF(__pyx_t_4);
  __pyx_t_4 = 0;

  /* "pymarketcap/core.pyx":543
 *     cpdef historical(self, unicode name,
 *                      start=datetime(2008, 8, 18),
 *                      end=datetime.now(),             # <<<<<<<<<<<<<<
 *                      revert=False):
 *         """Get historical data for a currency.
 */
  __pyx_t_4 = __Pyx_GetModuleGlobalName(__pyx_n_s_datetime); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 543, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_n_s_now); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 543, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_4 = __Pyx_PyObject_CallNoArg(__pyx_t_2); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 543, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_k__10 = __pyx_t_4;
  __Pyx_GIVEREF(__pyx_t_4);
  __pyx_t_4 = 0;

  /* "pymarketcap/core.pyx":542
 * 
 *     cpdef historical(self, unicode name,
 *                      start=datetime(2008, 8, 18),             # <<<<<<<<<<<<<<
 *                      end=datetime.now(),
 *                      revert=False):
 */
  __pyx_t_4 = __Pyx_GetModuleGlobalName(__pyx_n_s_datetime); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 542, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_4, __pyx_tuple__17, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 542, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_k__9 = __pyx_t_2;
  __Pyx_GIVEREF(__pyx_t_2);
  __pyx_t_2 = 0;

  /* "pymarketcap/core.pyx":543
 *     cpdef historical(self, unicode name,
 *                      start=datetime(2008, 8, 18),
 *                      end=datetime.now(),             # <<<<<<<<<<<<<<
 *                      revert=False):
 *         """Get historical data for a currency.
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_datetime); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 543, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_now); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 543, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_PyObject_CallNoArg(__pyx_t_4); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 543, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_k__10 = __pyx_t_2;
  __Pyx_GIVEREF(__pyx_t_2);
  __pyx_t_2 = 0;

  /* "(tree fragment)":1
 * def __pyx_unpickle_Pymarketcap(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     if __pyx_checksum != 0x7d79c3c:
 *         from pickle import PickleError as __pyx_PickleError
 */
  __pyx_t_2 = PyCFunction_NewEx(&__pyx_mdef_11pymarketcap_4core_1__pyx_unpickle_Pymarketcap, NULL, __pyx_n_s_pymarketcap_core); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_pyx_unpickle_Pymarketcap, __pyx_t_2) < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "pymarketcap/core.pyx":2
 * 
 * """API wraper and web scraper module."""             # <<<<<<<<<<<<<<
 * 
 * # Standard Python modules
 */
  __pyx_t_2 = __Pyx_PyDict_NewPresized(0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_test, __pyx_t_2) < 0) __PYX_ERR(0, 2, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /*--- Wrapped vars code ---*/

  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  if (__pyx_m) {
    if (__pyx_d) {
      __Pyx_AddTraceback("init pymarketcap.core", 0, __pyx_lineno, __pyx_filename);
    }
    Py_DECREF(__pyx_m); __pyx_m = 0;
  } else if (!PyErr_Occurred()) {
    PyErr_SetString(PyExc_ImportError, "init pymarketcap.core");
  }
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  #if CYTHON_PEP489_MULTI_PHASE_INIT
  return (__pyx_m != NULL) ? 0 : -1;
  #elif PY_MAJOR_VERSION >= 3
  return __pyx_m;
  #else
  return;
  #endif
}

/* --- Runtime support code --- */
/* Refnanny */
#if CYTHON_REFNANNY
static __Pyx_RefNannyAPIStruct *__Pyx_RefNannyImportAPI(const char *modname) {
    PyObject *m = NULL, *p = NULL;
    void *r = NULL;
    m = PyImport_ImportModule((char *)modname);
    if (!m) goto end;
    p = PyObject_GetAttrString(m, (char *)"RefNannyAPI");
    if (!p) goto end;
    r = PyLong_AsVoidPtr(p);
end:
    Py_XDECREF(p);
    Py_XDECREF(m);
    return (__Pyx_RefNannyAPIStruct *)r;
}
#endif

/* PyObjectGetAttrStr */
#if CYTHON_USE_TYPE_SLOTS
static CYTHON_INLINE PyObject* __Pyx_PyObject_GetAttrStr(PyObject* obj, PyObject* attr_name) {
    PyTypeObject* tp = Py_TYPE(obj);
    if (likely(tp->tp_getattro))
        return tp->tp_getattro(obj, attr_name);
#if PY_MAJOR_VERSION < 3
    if (likely(tp->tp_getattr))
        return tp->tp_getattr(obj, PyString_AS_STRING(attr_name));
#endif
    return PyObject_GetAttr(obj, attr_name);
}
#endif

/* GetBuiltinName */
static PyObject *__Pyx_GetBuiltinName(PyObject *name) {
    PyObject* result = __Pyx_PyObject_GetAttrStr(__pyx_b, name);
    if (unlikely(!result)) {
        PyErr_Format(PyExc_NameError,
#if PY_MAJOR_VERSION >= 3
            "name '%U' is not defined", name);
#else
            "name '%.200s' is not defined", PyString_AS_STRING(name));
#endif
    }
    return result;
}

/* RaiseDoubleKeywords */
static void __Pyx_RaiseDoubleKeywordsError(
    const char* func_name,
    PyObject* kw_name)
{
    PyErr_Format(PyExc_TypeError,
        #if PY_MAJOR_VERSION >= 3
        "%s() got multiple values for keyword argument '%U'", func_name, kw_name);
        #else
        "%s() got multiple values for keyword argument '%s'", func_name,
        PyString_AsString(kw_name));
        #endif
}

/* ParseKeywords */
static int __Pyx_ParseOptionalKeywords(
    PyObject *kwds,
    PyObject **argnames[],
    PyObject *kwds2,
    PyObject *values[],
    Py_ssize_t num_pos_args,
    const char* function_name)
{
    PyObject *key = 0, *value = 0;
    Py_ssize_t pos = 0;
    PyObject*** name;
    PyObject*** first_kw_arg = argnames + num_pos_args;
    while (PyDict_Next(kwds, &pos, &key, &value)) {
        name = first_kw_arg;
        while (*name && (**name != key)) name++;
        if (*name) {
            values[name-argnames] = value;
            continue;
        }
        name = first_kw_arg;
        #if PY_MAJOR_VERSION < 3
        if (likely(PyString_CheckExact(key)) || likely(PyString_Check(key))) {
            while (*name) {
                if ((CYTHON_COMPILING_IN_PYPY || PyString_GET_SIZE(**name) == PyString_GET_SIZE(key))
                        && _PyString_Eq(**name, key)) {
                    values[name-argnames] = value;
                    break;
                }
                name++;
            }
            if (*name) continue;
            else {
                PyObject*** argname = argnames;
                while (argname != first_kw_arg) {
                    if ((**argname == key) || (
                            (CYTHON_COMPILING_IN_PYPY || PyString_GET_SIZE(**argname) == PyString_GET_SIZE(key))
                             && _PyString_Eq(**argname, key))) {
                        goto arg_passed_twice;
                    }
                    argname++;
                }
            }
        } else
        #endif
        if (likely(PyUnicode_Check(key))) {
            while (*name) {
                int cmp = (**name == key) ? 0 :
                #if !CYTHON_COMPILING_IN_PYPY && PY_MAJOR_VERSION >= 3
                    (PyUnicode_GET_SIZE(**name) != PyUnicode_GET_SIZE(key)) ? 1 :
                #endif
                    PyUnicode_Compare(**name, key);
                if (cmp < 0 && unlikely(PyErr_Occurred())) goto bad;
                if (cmp == 0) {
                    values[name-argnames] = value;
                    break;
                }
                name++;
            }
            if (*name) continue;
            else {
                PyObject*** argname = argnames;
                while (argname != first_kw_arg) {
                    int cmp = (**argname == key) ? 0 :
                    #if !CYTHON_COMPILING_IN_PYPY && PY_MAJOR_VERSION >= 3
                        (PyUnicode_GET_SIZE(**argname) != PyUnicode_GET_SIZE(key)) ? 1 :
                    #endif
                        PyUnicode_Compare(**argname, key);
                    if (cmp < 0 && unlikely(PyErr_Occurred())) goto bad;
                    if (cmp == 0) goto arg_passed_twice;
                    argname++;
                }
            }
        } else
            goto invalid_keyword_type;
        if (kwds2) {
            if (unlikely(PyDict_SetItem(kwds2, key, value))) goto bad;
        } else {
            goto invalid_keyword;
        }
    }
    return 0;
arg_passed_twice:
    __Pyx_RaiseDoubleKeywordsError(function_name, key);
    goto bad;
invalid_keyword_type:
    PyErr_Format(PyExc_TypeError,
        "%.200s() keywords must be strings", function_name);
    goto bad;
invalid_keyword:
    PyErr_Format(PyExc_TypeError,
    #if PY_MAJOR_VERSION < 3
        "%.200s() got an unexpected keyword argument '%.200s'",
        function_name, PyString_AsString(key));
    #else
        "%s() got an unexpected keyword argument '%U'",
        function_name, key);
    #endif
bad:
    return -1;
}

/* RaiseArgTupleInvalid */
static void __Pyx_RaiseArgtupleInvalid(
    const char* func_name,
    int exact,
    Py_ssize_t num_min,
    Py_ssize_t num_max,
    Py_ssize_t num_found)
{
    Py_ssize_t num_expected;
    const char *more_or_less;
    if (num_found < num_min) {
        num_expected = num_min;
        more_or_less = "at least";
    } else {
        num_expected = num_max;
        more_or_less = "at most";
    }
    if (exact) {
        more_or_less = "exactly";
    }
    PyErr_Format(PyExc_TypeError,
                 "%.200s() takes %.8s %" CYTHON_FORMAT_SSIZE_T "d positional argument%.1s (%" CYTHON_FORMAT_SSIZE_T "d given)",
                 func_name, more_or_less, num_expected,
                 (num_expected == 1) ? "" : "s", num_found);
}

/* PyObjectCall */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_Call(PyObject *func, PyObject *arg, PyObject *kw) {
    PyObject *result;
    ternaryfunc call = func->ob_type->tp_call;
    if (unlikely(!call))
        return PyObject_Call(func, arg, kw);
    if (unlikely(Py_EnterRecursiveCall((char*)" while calling a Python object")))
        return NULL;
    result = (*call)(func, arg, kw);
    Py_LeaveRecursiveCall();
    if (unlikely(!result) && unlikely(!PyErr_Occurred())) {
        PyErr_SetString(
            PyExc_SystemError,
            "NULL result without error in PyObject_Call");
    }
    return result;
}
#endif

/* GetItemInt */
static PyObject *__Pyx_GetItemInt_Generic(PyObject *o, PyObject* j) {
    PyObject *r;
    if (!j) return NULL;
    r = PyObject_GetItem(o, j);
    Py_DECREF(j);
    return r;
}
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_List_Fast(PyObject *o, Py_ssize_t i,
                                                              CYTHON_NCP_UNUSED int wraparound,
                                                              CYTHON_NCP_UNUSED int boundscheck) {
#if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    Py_ssize_t wrapped_i = i;
    if (wraparound & unlikely(i < 0)) {
        wrapped_i += PyList_GET_SIZE(o);
    }
    if ((!boundscheck) || likely((0 <= wrapped_i) & (wrapped_i < PyList_GET_SIZE(o)))) {
        PyObject *r = PyList_GET_ITEM(o, wrapped_i);
        Py_INCREF(r);
        return r;
    }
    return __Pyx_GetItemInt_Generic(o, PyInt_FromSsize_t(i));
#else
    return PySequence_GetItem(o, i);
#endif
}
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Tuple_Fast(PyObject *o, Py_ssize_t i,
                                                              CYTHON_NCP_UNUSED int wraparound,
                                                              CYTHON_NCP_UNUSED int boundscheck) {
#if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    Py_ssize_t wrapped_i = i;
    if (wraparound & unlikely(i < 0)) {
        wrapped_i += PyTuple_GET_SIZE(o);
    }
    if ((!boundscheck) || likely((0 <= wrapped_i) & (wrapped_i < PyTuple_GET_SIZE(o)))) {
        PyObject *r = PyTuple_GET_ITEM(o, wrapped_i);
        Py_INCREF(r);
        return r;
    }
    return __Pyx_GetItemInt_Generic(o, PyInt_FromSsize_t(i));
#else
    return PySequence_GetItem(o, i);
#endif
}
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Fast(PyObject *o, Py_ssize_t i, int is_list,
                                                     CYTHON_NCP_UNUSED int wraparound,
                                                     CYTHON_NCP_UNUSED int boundscheck) {
#if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS && CYTHON_USE_TYPE_SLOTS
    if (is_list || PyList_CheckExact(o)) {
        Py_ssize_t n = ((!wraparound) | likely(i >= 0)) ? i : i + PyList_GET_SIZE(o);
        if ((!boundscheck) || (likely((n >= 0) & (n < PyList_GET_SIZE(o))))) {
            PyObject *r = PyList_GET_ITEM(o, n);
            Py_INCREF(r);
            return r;
        }
    }
    else if (PyTuple_CheckExact(o)) {
        Py_ssize_t n = ((!wraparound) | likely(i >= 0)) ? i : i + PyTuple_GET_SIZE(o);
        if ((!boundscheck) || likely((n >= 0) & (n < PyTuple_GET_SIZE(o)))) {
            PyObject *r = PyTuple_GET_ITEM(o, n);
            Py_INCREF(r);
            return r;
        }
    } else {
        PySequenceMethods *m = Py_TYPE(o)->tp_as_sequence;
        if (likely(m && m->sq_item)) {
            if (wraparound && unlikely(i < 0) && likely(m->sq_length)) {
                Py_ssize_t l = m->sq_length(o);
                if (likely(l >= 0)) {
                    i += l;
                } else {
                    if (!PyErr_ExceptionMatches(PyExc_OverflowError))
                        return NULL;
                    PyErr_Clear();
                }
            }
            return m->sq_item(o, i);
        }
    }
#else
    if (is_list || PySequence_Check(o)) {
        return PySequence_GetItem(o, i);
    }
#endif
    return __Pyx_GetItemInt_Generic(o, PyInt_FromSsize_t(i));
}

/* PyCFunctionFastCall */
#if CYTHON_FAST_PYCCALL
static CYTHON_INLINE PyObject * __Pyx_PyCFunction_FastCall(PyObject *func_obj, PyObject **args, Py_ssize_t nargs) {
    PyCFunctionObject *func = (PyCFunctionObject*)func_obj;
    PyCFunction meth = PyCFunction_GET_FUNCTION(func);
    PyObject *self = PyCFunction_GET_SELF(func);
    int flags = PyCFunction_GET_FLAGS(func);
    assert(PyCFunction_Check(func));
    assert(METH_FASTCALL == (flags & ~(METH_CLASS | METH_STATIC | METH_COEXIST | METH_KEYWORDS)));
    assert(nargs >= 0);
    assert(nargs == 0 || args != NULL);
    /* _PyCFunction_FastCallDict() must not be called with an exception set,
       because it may clear it (directly or indirectly) and so the
       caller loses its exception */
    assert(!PyErr_Occurred());
    if ((PY_VERSION_HEX < 0x030700A0) || unlikely(flags & METH_KEYWORDS)) {
        return (*((__Pyx_PyCFunctionFastWithKeywords)meth)) (self, args, nargs, NULL);
    } else {
        return (*((__Pyx_PyCFunctionFast)meth)) (self, args, nargs);
    }
}
#endif

/* PyFunctionFastCall */
#if CYTHON_FAST_PYCALL
#include "frameobject.h"
static PyObject* __Pyx_PyFunction_FastCallNoKw(PyCodeObject *co, PyObject **args, Py_ssize_t na,
                                               PyObject *globals) {
    PyFrameObject *f;
    PyThreadState *tstate = __Pyx_PyThreadState_Current;
    PyObject **fastlocals;
    Py_ssize_t i;
    PyObject *result;
    assert(globals != NULL);
    /* XXX Perhaps we should create a specialized
       PyFrame_New() that doesn't take locals, but does
       take builtins without sanity checking them.
       */
    assert(tstate != NULL);
    f = PyFrame_New(tstate, co, globals, NULL);
    if (f == NULL) {
        return NULL;
    }
    fastlocals = f->f_localsplus;
    for (i = 0; i < na; i++) {
        Py_INCREF(*args);
        fastlocals[i] = *args++;
    }
    result = PyEval_EvalFrameEx(f,0);
    ++tstate->recursion_depth;
    Py_DECREF(f);
    --tstate->recursion_depth;
    return result;
}
#if 1 || PY_VERSION_HEX < 0x030600B1
static PyObject *__Pyx_PyFunction_FastCallDict(PyObject *func, PyObject **args, int nargs, PyObject *kwargs) {
    PyCodeObject *co = (PyCodeObject *)PyFunction_GET_CODE(func);
    PyObject *globals = PyFunction_GET_GLOBALS(func);
    PyObject *argdefs = PyFunction_GET_DEFAULTS(func);
    PyObject *closure;
#if PY_MAJOR_VERSION >= 3
    PyObject *kwdefs;
#endif
    PyObject *kwtuple, **k;
    PyObject **d;
    Py_ssize_t nd;
    Py_ssize_t nk;
    PyObject *result;
    assert(kwargs == NULL || PyDict_Check(kwargs));
    nk = kwargs ? PyDict_Size(kwargs) : 0;
    if (Py_EnterRecursiveCall((char*)" while calling a Python object")) {
        return NULL;
    }
    if (
#if PY_MAJOR_VERSION >= 3
            co->co_kwonlyargcount == 0 &&
#endif
            likely(kwargs == NULL || nk == 0) &&
            co->co_flags == (CO_OPTIMIZED | CO_NEWLOCALS | CO_NOFREE)) {
        if (argdefs == NULL && co->co_argcount == nargs) {
            result = __Pyx_PyFunction_FastCallNoKw(co, args, nargs, globals);
            goto done;
        }
        else if (nargs == 0 && argdefs != NULL
                 && co->co_argcount == Py_SIZE(argdefs)) {
            /* function called with no arguments, but all parameters have
               a default value: use default values as arguments .*/
            args = &PyTuple_GET_ITEM(argdefs, 0);
            result =__Pyx_PyFunction_FastCallNoKw(co, args, Py_SIZE(argdefs), globals);
            goto done;
        }
    }
    if (kwargs != NULL) {
        Py_ssize_t pos, i;
        kwtuple = PyTuple_New(2 * nk);
        if (kwtuple == NULL) {
            result = NULL;
            goto done;
        }
        k = &PyTuple_GET_ITEM(kwtuple, 0);
        pos = i = 0;
        while (PyDict_Next(kwargs, &pos, &k[i], &k[i+1])) {
            Py_INCREF(k[i]);
            Py_INCREF(k[i+1]);
            i += 2;
        }
        nk = i / 2;
    }
    else {
        kwtuple = NULL;
        k = NULL;
    }
    closure = PyFunction_GET_CLOSURE(func);
#if PY_MAJOR_VERSION >= 3
    kwdefs = PyFunction_GET_KW_DEFAULTS(func);
#endif
    if (argdefs != NULL) {
        d = &PyTuple_GET_ITEM(argdefs, 0);
        nd = Py_SIZE(argdefs);
    }
    else {
        d = NULL;
        nd = 0;
    }
#if PY_MAJOR_VERSION >= 3
    result = PyEval_EvalCodeEx((PyObject*)co, globals, (PyObject *)NULL,
                               args, nargs,
                               k, (int)nk,
                               d, (int)nd, kwdefs, closure);
#else
    result = PyEval_EvalCodeEx(co, globals, (PyObject *)NULL,
                               args, nargs,
                               k, (int)nk,
                               d, (int)nd, closure);
#endif
    Py_XDECREF(kwtuple);
done:
    Py_LeaveRecursiveCall();
    return result;
}
#endif
#endif

/* PyObjectCallMethO */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallMethO(PyObject *func, PyObject *arg) {
    PyObject *self, *result;
    PyCFunction cfunc;
    cfunc = PyCFunction_GET_FUNCTION(func);
    self = PyCFunction_GET_SELF(func);
    if (unlikely(Py_EnterRecursiveCall((char*)" while calling a Python object")))
        return NULL;
    result = cfunc(self, arg);
    Py_LeaveRecursiveCall();
    if (unlikely(!result) && unlikely(!PyErr_Occurred())) {
        PyErr_SetString(
            PyExc_SystemError,
            "NULL result without error in PyObject_Call");
    }
    return result;
}
#endif

/* PyObjectCallOneArg */
#if CYTHON_COMPILING_IN_CPYTHON
static PyObject* __Pyx__PyObject_CallOneArg(PyObject *func, PyObject *arg) {
    PyObject *result;
    PyObject *args = PyTuple_New(1);
    if (unlikely(!args)) return NULL;
    Py_INCREF(arg);
    PyTuple_SET_ITEM(args, 0, arg);
    result = __Pyx_PyObject_Call(func, args, NULL);
    Py_DECREF(args);
    return result;
}
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallOneArg(PyObject *func, PyObject *arg) {
#if CYTHON_FAST_PYCALL
    if (PyFunction_Check(func)) {
        return __Pyx_PyFunction_FastCall(func, &arg, 1);
    }
#endif
    if (likely(PyCFunction_Check(func))) {
        if (likely(PyCFunction_GET_FLAGS(func) & METH_O)) {
            return __Pyx_PyObject_CallMethO(func, arg);
#if CYTHON_FAST_PYCCALL
        } else if (PyCFunction_GET_FLAGS(func) & METH_FASTCALL) {
            return __Pyx_PyCFunction_FastCall(func, &arg, 1);
#endif
        }
    }
    return __Pyx__PyObject_CallOneArg(func, arg);
}
#else
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallOneArg(PyObject *func, PyObject *arg) {
    PyObject *result;
    PyObject *args = PyTuple_Pack(1, arg);
    if (unlikely(!args)) return NULL;
    result = __Pyx_PyObject_Call(func, args, NULL);
    Py_DECREF(args);
    return result;
}
#endif

/* PyObjectCallNoArg */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallNoArg(PyObject *func) {
#if CYTHON_FAST_PYCALL
    if (PyFunction_Check(func)) {
        return __Pyx_PyFunction_FastCall(func, NULL, 0);
    }
#endif
#ifdef __Pyx_CyFunction_USED
    if (likely(PyCFunction_Check(func) || __Pyx_TypeCheck(func, __pyx_CyFunctionType))) {
#else
    if (likely(PyCFunction_Check(func))) {
#endif
        if (likely(PyCFunction_GET_FLAGS(func) & METH_NOARGS)) {
            return __Pyx_PyObject_CallMethO(func, NULL);
        }
    }
    return __Pyx_PyObject_Call(func, __pyx_empty_tuple, NULL);
}
#endif

/* GetModuleGlobalName */
  static CYTHON_INLINE PyObject *__Pyx_GetModuleGlobalName(PyObject *name) {
    PyObject *result;
#if !CYTHON_AVOID_BORROWED_REFS
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030500A1
    result = _PyDict_GetItem_KnownHash(__pyx_d, name, ((PyASCIIObject *) name)->hash);
    if (likely(result)) {
        Py_INCREF(result);
    } else if (unlikely(PyErr_Occurred())) {
        result = NULL;
    } else {
#else
    result = PyDict_GetItem(__pyx_d, name);
    if (likely(result)) {
        Py_INCREF(result);
    } else {
#endif
#else
    result = PyObject_GetItem(__pyx_d, name);
    if (!result) {
        PyErr_Clear();
#endif
        result = __Pyx_GetBuiltinName(name);
    }
    return result;
}

/* DictGetItem */
      #if PY_MAJOR_VERSION >= 3 && !CYTHON_COMPILING_IN_PYPY
static PyObject *__Pyx_PyDict_GetItem(PyObject *d, PyObject* key) {
    PyObject *value;
    value = PyDict_GetItemWithError(d, key);
    if (unlikely(!value)) {
        if (!PyErr_Occurred()) {
            PyObject* args = PyTuple_Pack(1, key);
            if (likely(args))
                PyErr_SetObject(PyExc_KeyError, args);
            Py_XDECREF(args);
        }
        return NULL;
    }
    Py_INCREF(value);
    return value;
}
#endif

/* PyErrFetchRestore */
      #if CYTHON_FAST_THREAD_STATE
static CYTHON_INLINE void __Pyx_ErrRestoreInState(PyThreadState *tstate, PyObject *type, PyObject *value, PyObject *tb) {
    PyObject *tmp_type, *tmp_value, *tmp_tb;
    tmp_type = tstate->curexc_type;
    tmp_value = tstate->curexc_value;
    tmp_tb = tstate->curexc_traceback;
    tstate->curexc_type = type;
    tstate->curexc_value = value;
    tstate->curexc_traceback = tb;
    Py_XDECREF(tmp_type);
    Py_XDECREF(tmp_value);
    Py_XDECREF(tmp_tb);
}
static CYTHON_INLINE void __Pyx_ErrFetchInState(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb) {
    *type = tstate->curexc_type;
    *value = tstate->curexc_value;
    *tb = tstate->curexc_traceback;
    tstate->curexc_type = 0;
    tstate->curexc_value = 0;
    tstate->curexc_traceback = 0;
}
#endif

/* RaiseException */
      #if PY_MAJOR_VERSION < 3
static void __Pyx_Raise(PyObject *type, PyObject *value, PyObject *tb,
                        CYTHON_UNUSED PyObject *cause) {
    __Pyx_PyThreadState_declare
    Py_XINCREF(type);
    if (!value || value == Py_None)
        value = NULL;
    else
        Py_INCREF(value);
    if (!tb || tb == Py_None)
        tb = NULL;
    else {
        Py_INCREF(tb);
        if (!PyTraceBack_Check(tb)) {
            PyErr_SetString(PyExc_TypeError,
                "raise: arg 3 must be a traceback or None");
            goto raise_error;
        }
    }
    if (PyType_Check(type)) {
#if CYTHON_COMPILING_IN_PYPY
        if (!value) {
            Py_INCREF(Py_None);
            value = Py_None;
        }
#endif
        PyErr_NormalizeException(&type, &value, &tb);
    } else {
        if (value) {
            PyErr_SetString(PyExc_TypeError,
                "instance exception may not have a separate value");
            goto raise_error;
        }
        value = type;
        type = (PyObject*) Py_TYPE(type);
        Py_INCREF(type);
        if (!PyType_IsSubtype((PyTypeObject *)type, (PyTypeObject *)PyExc_BaseException)) {
            PyErr_SetString(PyExc_TypeError,
                "raise: exception class must be a subclass of BaseException");
            goto raise_error;
        }
    }
    __Pyx_PyThreadState_assign
    __Pyx_ErrRestore(type, value, tb);
    return;
raise_error:
    Py_XDECREF(value);
    Py_XDECREF(type);
    Py_XDECREF(tb);
    return;
}
#else
static void __Pyx_Raise(PyObject *type, PyObject *value, PyObject *tb, PyObject *cause) {
    PyObject* owned_instance = NULL;
    if (tb == Py_None) {
        tb = 0;
    } else if (tb && !PyTraceBack_Check(tb)) {
        PyErr_SetString(PyExc_TypeError,
            "raise: arg 3 must be a traceback or None");
        goto bad;
    }
    if (value == Py_None)
        value = 0;
    if (PyExceptionInstance_Check(type)) {
        if (value) {
            PyErr_SetString(PyExc_TypeError,
                "instance exception may not have a separate value");
            goto bad;
        }
        value = type;
        type = (PyObject*) Py_TYPE(value);
    } else if (PyExceptionClass_Check(type)) {
        PyObject *instance_class = NULL;
        if (value && PyExceptionInstance_Check(value)) {
            instance_class = (PyObject*) Py_TYPE(value);
            if (instance_class != type) {
                int is_subclass = PyObject_IsSubclass(instance_class, type);
                if (!is_subclass) {
                    instance_class = NULL;
                } else if (unlikely(is_subclass == -1)) {
                    goto bad;
                } else {
                    type = instance_class;
                }
            }
        }
        if (!instance_class) {
            PyObject *args;
            if (!value)
                args = PyTuple_New(0);
            else if (PyTuple_Check(value)) {
                Py_INCREF(value);
                args = value;
            } else
                args = PyTuple_Pack(1, value);
            if (!args)
                goto bad;
            owned_instance = PyObject_Call(type, args, NULL);
            Py_DECREF(args);
            if (!owned_instance)
                goto bad;
            value = owned_instance;
            if (!PyExceptionInstance_Check(value)) {
                PyErr_Format(PyExc_TypeError,
                             "calling %R should have returned an instance of "
                             "BaseException, not %R",
                             type, Py_TYPE(value));
                goto bad;
            }
        }
    } else {
        PyErr_SetString(PyExc_TypeError,
            "raise: exception class must be a subclass of BaseException");
        goto bad;
    }
    if (cause) {
        PyObject *fixed_cause;
        if (cause == Py_None) {
            fixed_cause = NULL;
        } else if (PyExceptionClass_Check(cause)) {
            fixed_cause = PyObject_CallObject(cause, NULL);
            if (fixed_cause == NULL)
                goto bad;
        } else if (PyExceptionInstance_Check(cause)) {
            fixed_cause = cause;
            Py_INCREF(fixed_cause);
        } else {
            PyErr_SetString(PyExc_TypeError,
                            "exception causes must derive from "
                            "BaseException");
            goto bad;
        }
        PyException_SetCause(value, fixed_cause);
    }
    PyErr_SetObject(type, value);
    if (tb) {
#if CYTHON_COMPILING_IN_PYPY
        PyObject *tmp_type, *tmp_value, *tmp_tb;
        PyErr_Fetch(&tmp_type, &tmp_value, &tmp_tb);
        Py_INCREF(tb);
        PyErr_Restore(tmp_type, tmp_value, tb);
        Py_XDECREF(tmp_tb);
#else
        PyThreadState *tstate = __Pyx_PyThreadState_Current;
        PyObject* tmp_tb = tstate->curexc_traceback;
        if (tb != tmp_tb) {
            Py_INCREF(tb);
            tstate->curexc_traceback = tb;
            Py_XDECREF(tmp_tb);
        }
#endif
    }
bad:
    Py_XDECREF(owned_instance);
    return;
}
#endif

/* SaveResetException */
      #if CYTHON_FAST_THREAD_STATE
static CYTHON_INLINE void __Pyx__ExceptionSave(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb) {
    #if PY_VERSION_HEX >= 0x030700A2
    *type = tstate->exc_state.exc_type;
    *value = tstate->exc_state.exc_value;
    *tb = tstate->exc_state.exc_traceback;
    #else
    *type = tstate->exc_type;
    *value = tstate->exc_value;
    *tb = tstate->exc_traceback;
    #endif
    Py_XINCREF(*type);
    Py_XINCREF(*value);
    Py_XINCREF(*tb);
}
static CYTHON_INLINE void __Pyx__ExceptionReset(PyThreadState *tstate, PyObject *type, PyObject *value, PyObject *tb) {
    PyObject *tmp_type, *tmp_value, *tmp_tb;
    #if PY_VERSION_HEX >= 0x030700A2
    tmp_type = tstate->exc_state.exc_type;
    tmp_value = tstate->exc_state.exc_value;
    tmp_tb = tstate->exc_state.exc_traceback;
    tstate->exc_state.exc_type = type;
    tstate->exc_state.exc_value = value;
    tstate->exc_state.exc_traceback = tb;
    #else
    tmp_type = tstate->exc_type;
    tmp_value = tstate->exc_value;
    tmp_tb = tstate->exc_traceback;
    tstate->exc_type = type;
    tstate->exc_value = value;
    tstate->exc_traceback = tb;
    #endif
    Py_XDECREF(tmp_type);
    Py_XDECREF(tmp_value);
    Py_XDECREF(tmp_tb);
}
#endif

/* PyErrExceptionMatches */
      #if CYTHON_FAST_THREAD_STATE
static int __Pyx_PyErr_ExceptionMatchesTuple(PyObject *exc_type, PyObject *tuple) {
    Py_ssize_t i, n;
    n = PyTuple_GET_SIZE(tuple);
#if PY_MAJOR_VERSION >= 3
    for (i=0; i<n; i++) {
        if (exc_type == PyTuple_GET_ITEM(tuple, i)) return 1;
    }
#endif
    for (i=0; i<n; i++) {
        if (__Pyx_PyErr_GivenExceptionMatches(exc_type, PyTuple_GET_ITEM(tuple, i))) return 1;
    }
    return 0;
}
static CYTHON_INLINE int __Pyx_PyErr_ExceptionMatchesInState(PyThreadState* tstate, PyObject* err) {
    PyObject *exc_type = tstate->curexc_type;
    if (exc_type == err) return 1;
    if (unlikely(!exc_type)) return 0;
    if (unlikely(PyTuple_Check(err)))
        return __Pyx_PyErr_ExceptionMatchesTuple(exc_type, err);
    return __Pyx_PyErr_GivenExceptionMatches(exc_type, err);
}
#endif

/* ArgTypeTest */
      static int __Pyx__ArgTypeTest(PyObject *obj, PyTypeObject *type, const char *name, int exact)
{
    if (unlikely(!type)) {
        PyErr_SetString(PyExc_SystemError, "Missing type object");
        return 0;
    }
    else if (exact) {
        #if PY_MAJOR_VERSION == 2
        if ((type == &PyBaseString_Type) && likely(__Pyx_PyBaseString_CheckExact(obj))) return 1;
        #endif
    }
    else {
        if (likely(__Pyx_TypeCheck(obj, type))) return 1;
    }
    PyErr_Format(PyExc_TypeError,
        "Argument '%.200s' has incorrect type (expected %.200s, got %.200s)",
        name, type->tp_name, Py_TYPE(obj)->tp_name);
    return 0;
}

/* UnpackUnboundCMethod */
      static int __Pyx_TryUnpackUnboundCMethod(__Pyx_CachedCFunction* target) {
    PyObject *method;
    method = __Pyx_PyObject_GetAttrStr(target->type, *target->method_name);
    if (unlikely(!method))
        return -1;
    target->method = method;
#if CYTHON_COMPILING_IN_CPYTHON
    #if PY_MAJOR_VERSION >= 3
    if (likely(__Pyx_TypeCheck(method, &PyMethodDescr_Type)))
    #endif
    {
        PyMethodDescrObject *descr = (PyMethodDescrObject*) method;
        target->func = descr->d_method->ml_meth;
        target->flag = descr->d_method->ml_flags & ~(METH_CLASS | METH_STATIC | METH_COEXIST);
    }
#endif
    return 0;
}

/* CallUnboundCMethod0 */
      static PyObject* __Pyx__CallUnboundCMethod0(__Pyx_CachedCFunction* cfunc, PyObject* self) {
    PyObject *args, *result = NULL;
    if (unlikely(!cfunc->method) && unlikely(__Pyx_TryUnpackUnboundCMethod(cfunc) < 0)) return NULL;
#if CYTHON_ASSUME_SAFE_MACROS
    args = PyTuple_New(1);
    if (unlikely(!args)) goto bad;
    Py_INCREF(self);
    PyTuple_SET_ITEM(args, 0, self);
#else
    args = PyTuple_Pack(1, self);
    if (unlikely(!args)) goto bad;
#endif
    result = __Pyx_PyObject_Call(cfunc->method, args, NULL);
    Py_DECREF(args);
bad:
    return result;
}

/* py_dict_keys */
      static CYTHON_INLINE PyObject* __Pyx_PyDict_Keys(PyObject* d) {
    if (PY_MAJOR_VERSION >= 3)
        return __Pyx_CallUnboundCMethod0(&__pyx_umethod_PyDict_Type_keys, d);
    else
        return PyDict_Keys(d);
}

/* PyObjectCallMethod1 */
      static PyObject* __Pyx__PyObject_CallMethod1(PyObject* method, PyObject* arg) {
    PyObject *result = NULL;
#if CYTHON_UNPACK_METHODS
    if (likely(PyMethod_Check(method))) {
        PyObject *self = PyMethod_GET_SELF(method);
        if (likely(self)) {
            PyObject *args;
            PyObject *function = PyMethod_GET_FUNCTION(method);
            #if CYTHON_FAST_PYCALL
            if (PyFunction_Check(function)) {
                PyObject *args[2] = {self, arg};
                result = __Pyx_PyFunction_FastCall(function, args, 2);
                goto done;
            }
            #endif
            #if CYTHON_FAST_PYCCALL
            if (__Pyx_PyFastCFunction_Check(function)) {
                PyObject *args[2] = {self, arg};
                result = __Pyx_PyCFunction_FastCall(function, args, 2);
                goto done;
            }
            #endif
            args = PyTuple_New(2);
            if (unlikely(!args)) goto done;
            Py_INCREF(self);
            PyTuple_SET_ITEM(args, 0, self);
            Py_INCREF(arg);
            PyTuple_SET_ITEM(args, 1, arg);
            Py_INCREF(function);
            result = __Pyx_PyObject_Call(function, args, NULL);
            Py_DECREF(args);
            Py_DECREF(function);
            return result;
        }
    }
#endif
    result = __Pyx_PyObject_CallOneArg(method, arg);
    goto done;
done:
    return result;
}
static PyObject* __Pyx_PyObject_CallMethod1(PyObject* obj, PyObject* method_name, PyObject* arg) {
    PyObject *method, *result;
    method = __Pyx_PyObject_GetAttrStr(obj, method_name);
    if (unlikely(!method)) return NULL;
    result = __Pyx__PyObject_CallMethod1(method, arg);
    Py_DECREF(method);
    return result;
}

/* append */
      static CYTHON_INLINE int __Pyx_PyObject_Append(PyObject* L, PyObject* x) {
    if (likely(PyList_CheckExact(L))) {
        if (unlikely(__Pyx_PyList_Append(L, x) < 0)) return -1;
    } else {
        PyObject* retval = __Pyx_PyObject_CallMethod1(L, __pyx_n_s_append, x);
        if (unlikely(!retval))
            return -1;
        Py_DECREF(retval);
    }
    return 0;
}

/* RaiseTooManyValuesToUnpack */
      static CYTHON_INLINE void __Pyx_RaiseTooManyValuesError(Py_ssize_t expected) {
    PyErr_Format(PyExc_ValueError,
                 "too many values to unpack (expected %" CYTHON_FORMAT_SSIZE_T "d)", expected);
}

/* RaiseNeedMoreValuesToUnpack */
      static CYTHON_INLINE void __Pyx_RaiseNeedMoreValuesError(Py_ssize_t index) {
    PyErr_Format(PyExc_ValueError,
                 "need more than %" CYTHON_FORMAT_SSIZE_T "d value%.1s to unpack",
                 index, (index == 1) ? "" : "s");
}

/* IterFinish */
      static CYTHON_INLINE int __Pyx_IterFinish(void) {
#if CYTHON_FAST_THREAD_STATE
    PyThreadState *tstate = __Pyx_PyThreadState_Current;
    PyObject* exc_type = tstate->curexc_type;
    if (unlikely(exc_type)) {
        if (likely(__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) {
            PyObject *exc_value, *exc_tb;
            exc_value = tstate->curexc_value;
            exc_tb = tstate->curexc_traceback;
            tstate->curexc_type = 0;
            tstate->curexc_value = 0;
            tstate->curexc_traceback = 0;
            Py_DECREF(exc_type);
            Py_XDECREF(exc_value);
            Py_XDECREF(exc_tb);
            return 0;
        } else {
            return -1;
        }
    }
    return 0;
#else
    if (unlikely(PyErr_Occurred())) {
        if (likely(PyErr_ExceptionMatches(PyExc_StopIteration))) {
            PyErr_Clear();
            return 0;
        } else {
            return -1;
        }
    }
    return 0;
#endif
}

/* UnpackItemEndCheck */
      static int __Pyx_IternextUnpackEndCheck(PyObject *retval, Py_ssize_t expected) {
    if (unlikely(retval)) {
        Py_DECREF(retval);
        __Pyx_RaiseTooManyValuesError(expected);
        return -1;
    } else {
        return __Pyx_IterFinish();
    }
    return 0;
}

/* PyObjectCallMethod0 */
      static PyObject* __Pyx_PyObject_CallMethod0(PyObject* obj, PyObject* method_name) {
    PyObject *method, *result = NULL;
    method = __Pyx_PyObject_GetAttrStr(obj, method_name);
    if (unlikely(!method)) goto bad;
#if CYTHON_UNPACK_METHODS
    if (likely(PyMethod_Check(method))) {
        PyObject *self = PyMethod_GET_SELF(method);
        if (likely(self)) {
            PyObject *function = PyMethod_GET_FUNCTION(method);
            result = __Pyx_PyObject_CallOneArg(function, self);
            Py_DECREF(method);
            return result;
        }
    }
#endif
    result = __Pyx_PyObject_CallNoArg(method);
    Py_DECREF(method);
bad:
    return result;
}

/* RaiseNoneIterError */
      static CYTHON_INLINE void __Pyx_RaiseNoneNotIterableError(void) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not iterable");
}

/* UnpackTupleError */
      static void __Pyx_UnpackTupleError(PyObject *t, Py_ssize_t index) {
    if (t == Py_None) {
      __Pyx_RaiseNoneNotIterableError();
    } else if (PyTuple_GET_SIZE(t) < index) {
      __Pyx_RaiseNeedMoreValuesError(PyTuple_GET_SIZE(t));
    } else {
      __Pyx_RaiseTooManyValuesError(index);
    }
}

/* UnpackTuple2 */
      static CYTHON_INLINE int __Pyx_unpack_tuple2_exact(
        PyObject* tuple, PyObject** pvalue1, PyObject** pvalue2, int decref_tuple) {
    PyObject *value1 = NULL, *value2 = NULL;
#if CYTHON_COMPILING_IN_PYPY
    value1 = PySequence_ITEM(tuple, 0);  if (unlikely(!value1)) goto bad;
    value2 = PySequence_ITEM(tuple, 1);  if (unlikely(!value2)) goto bad;
#else
    value1 = PyTuple_GET_ITEM(tuple, 0);  Py_INCREF(value1);
    value2 = PyTuple_GET_ITEM(tuple, 1);  Py_INCREF(value2);
#endif
    if (decref_tuple) {
        Py_DECREF(tuple);
    }
    *pvalue1 = value1;
    *pvalue2 = value2;
    return 0;
#if CYTHON_COMPILING_IN_PYPY
bad:
    Py_XDECREF(value1);
    Py_XDECREF(value2);
    if (decref_tuple) { Py_XDECREF(tuple); }
    return -1;
#endif
}
static int __Pyx_unpack_tuple2_generic(PyObject* tuple, PyObject** pvalue1, PyObject** pvalue2,
                                       int has_known_size, int decref_tuple) {
    Py_ssize_t index;
    PyObject *value1 = NULL, *value2 = NULL, *iter = NULL;
    iternextfunc iternext;
    iter = PyObject_GetIter(tuple);
    if (unlikely(!iter)) goto bad;
    if (decref_tuple) { Py_DECREF(tuple); tuple = NULL; }
    iternext = Py_TYPE(iter)->tp_iternext;
    value1 = iternext(iter); if (unlikely(!value1)) { index = 0; goto unpacking_failed; }
    value2 = iternext(iter); if (unlikely(!value2)) { index = 1; goto unpacking_failed; }
    if (!has_known_size && unlikely(__Pyx_IternextUnpackEndCheck(iternext(iter), 2))) goto bad;
    Py_DECREF(iter);
    *pvalue1 = value1;
    *pvalue2 = value2;
    return 0;
unpacking_failed:
    if (!has_known_size && __Pyx_IterFinish() == 0)
        __Pyx_RaiseNeedMoreValuesError(index);
bad:
    Py_XDECREF(iter);
    Py_XDECREF(value1);
    Py_XDECREF(value2);
    if (decref_tuple) { Py_XDECREF(tuple); }
    return -1;
}

/* dict_iter */
      static CYTHON_INLINE PyObject* __Pyx_dict_iterator(PyObject* iterable, int is_dict, PyObject* method_name,
                                                   Py_ssize_t* p_orig_length, int* p_source_is_dict) {
    is_dict = is_dict || likely(PyDict_CheckExact(iterable));
    *p_source_is_dict = is_dict;
    if (is_dict) {
#if !CYTHON_COMPILING_IN_PYPY
        *p_orig_length = PyDict_Size(iterable);
        Py_INCREF(iterable);
        return iterable;
#elif PY_MAJOR_VERSION >= 3
        static PyObject *py_items = NULL, *py_keys = NULL, *py_values = NULL;
        PyObject **pp = NULL;
        if (method_name) {
            const char *name = PyUnicode_AsUTF8(method_name);
            if (strcmp(name, "iteritems") == 0) pp = &py_items;
            else if (strcmp(name, "iterkeys") == 0) pp = &py_keys;
            else if (strcmp(name, "itervalues") == 0) pp = &py_values;
            if (pp) {
                if (!*pp) {
                    *pp = PyUnicode_FromString(name + 4);
                    if (!*pp)
                        return NULL;
                }
                method_name = *pp;
            }
        }
#endif
    }
    *p_orig_length = 0;
    if (method_name) {
        PyObject* iter;
        iterable = __Pyx_PyObject_CallMethod0(iterable, method_name);
        if (!iterable)
            return NULL;
#if !CYTHON_COMPILING_IN_PYPY
        if (PyTuple_CheckExact(iterable) || PyList_CheckExact(iterable))
            return iterable;
#endif
        iter = PyObject_GetIter(iterable);
        Py_DECREF(iterable);
        return iter;
    }
    return PyObject_GetIter(iterable);
}
static CYTHON_INLINE int __Pyx_dict_iter_next(
        PyObject* iter_obj, CYTHON_NCP_UNUSED Py_ssize_t orig_length, CYTHON_NCP_UNUSED Py_ssize_t* ppos,
        PyObject** pkey, PyObject** pvalue, PyObject** pitem, int source_is_dict) {
    PyObject* next_item;
#if !CYTHON_COMPILING_IN_PYPY
    if (source_is_dict) {
        PyObject *key, *value;
        if (unlikely(orig_length != PyDict_Size(iter_obj))) {
            PyErr_SetString(PyExc_RuntimeError, "dictionary changed size during iteration");
            return -1;
        }
        if (unlikely(!PyDict_Next(iter_obj, ppos, &key, &value))) {
            return 0;
        }
        if (pitem) {
            PyObject* tuple = PyTuple_New(2);
            if (unlikely(!tuple)) {
                return -1;
            }
            Py_INCREF(key);
            Py_INCREF(value);
            PyTuple_SET_ITEM(tuple, 0, key);
            PyTuple_SET_ITEM(tuple, 1, value);
            *pitem = tuple;
        } else {
            if (pkey) {
                Py_INCREF(key);
                *pkey = key;
            }
            if (pvalue) {
                Py_INCREF(value);
                *pvalue = value;
            }
        }
        return 1;
    } else if (PyTuple_CheckExact(iter_obj)) {
        Py_ssize_t pos = *ppos;
        if (unlikely(pos >= PyTuple_GET_SIZE(iter_obj))) return 0;
        *ppos = pos + 1;
        next_item = PyTuple_GET_ITEM(iter_obj, pos);
        Py_INCREF(next_item);
    } else if (PyList_CheckExact(iter_obj)) {
        Py_ssize_t pos = *ppos;
        if (unlikely(pos >= PyList_GET_SIZE(iter_obj))) return 0;
        *ppos = pos + 1;
        next_item = PyList_GET_ITEM(iter_obj, pos);
        Py_INCREF(next_item);
    } else
#endif
    {
        next_item = PyIter_Next(iter_obj);
        if (unlikely(!next_item)) {
            return __Pyx_IterFinish();
        }
    }
    if (pitem) {
        *pitem = next_item;
    } else if (pkey && pvalue) {
        if (__Pyx_unpack_tuple2(next_item, pkey, pvalue, source_is_dict, source_is_dict, 1))
            return -1;
    } else if (pkey) {
        *pkey = next_item;
    } else {
        *pvalue = next_item;
    }
    return 1;
}

/* SliceObject */
      static CYTHON_INLINE PyObject* __Pyx_PyObject_GetSlice(PyObject* obj,
        Py_ssize_t cstart, Py_ssize_t cstop,
        PyObject** _py_start, PyObject** _py_stop, PyObject** _py_slice,
        int has_cstart, int has_cstop, CYTHON_UNUSED int wraparound) {
#if CYTHON_USE_TYPE_SLOTS
    PyMappingMethods* mp;
#if PY_MAJOR_VERSION < 3
    PySequenceMethods* ms = Py_TYPE(obj)->tp_as_sequence;
    if (likely(ms && ms->sq_slice)) {
        if (!has_cstart) {
            if (_py_start && (*_py_start != Py_None)) {
                cstart = __Pyx_PyIndex_AsSsize_t(*_py_start);
                if ((cstart == (Py_ssize_t)-1) && PyErr_Occurred()) goto bad;
            } else
                cstart = 0;
        }
        if (!has_cstop) {
            if (_py_stop && (*_py_stop != Py_None)) {
                cstop = __Pyx_PyIndex_AsSsize_t(*_py_stop);
                if ((cstop == (Py_ssize_t)-1) && PyErr_Occurred()) goto bad;
            } else
                cstop = PY_SSIZE_T_MAX;
        }
        if (wraparound && unlikely((cstart < 0) | (cstop < 0)) && likely(ms->sq_length)) {
            Py_ssize_t l = ms->sq_length(obj);
            if (likely(l >= 0)) {
                if (cstop < 0) {
                    cstop += l;
                    if (cstop < 0) cstop = 0;
                }
                if (cstart < 0) {
                    cstart += l;
                    if (cstart < 0) cstart = 0;
                }
            } else {
                if (!PyErr_ExceptionMatches(PyExc_OverflowError))
                    goto bad;
                PyErr_Clear();
            }
        }
        return ms->sq_slice(obj, cstart, cstop);
    }
#endif
    mp = Py_TYPE(obj)->tp_as_mapping;
    if (likely(mp && mp->mp_subscript))
#endif
    {
        PyObject* result;
        PyObject *py_slice, *py_start, *py_stop;
        if (_py_slice) {
            py_slice = *_py_slice;
        } else {
            PyObject* owned_start = NULL;
            PyObject* owned_stop = NULL;
            if (_py_start) {
                py_start = *_py_start;
            } else {
                if (has_cstart) {
                    owned_start = py_start = PyInt_FromSsize_t(cstart);
                    if (unlikely(!py_start)) goto bad;
                } else
                    py_start = Py_None;
            }
            if (_py_stop) {
                py_stop = *_py_stop;
            } else {
                if (has_cstop) {
                    owned_stop = py_stop = PyInt_FromSsize_t(cstop);
                    if (unlikely(!py_stop)) {
                        Py_XDECREF(owned_start);
                        goto bad;
                    }
                } else
                    py_stop = Py_None;
            }
            py_slice = PySlice_New(py_start, py_stop, Py_None);
            Py_XDECREF(owned_start);
            Py_XDECREF(owned_stop);
            if (unlikely(!py_slice)) goto bad;
        }
#if CYTHON_USE_TYPE_SLOTS
        result = mp->mp_subscript(obj, py_slice);
#else
        result = PyObject_GetItem(obj, py_slice);
#endif
        if (!_py_slice) {
            Py_DECREF(py_slice);
        }
        return result;
    }
    PyErr_Format(PyExc_TypeError,
        "'%.200s' object is unsliceable", Py_TYPE(obj)->tp_name);
bad:
    return NULL;
}

/* decode_c_string */
      static CYTHON_INLINE PyObject* __Pyx_decode_c_string(
         const char* cstring, Py_ssize_t start, Py_ssize_t stop,
         const char* encoding, const char* errors,
         PyObject* (*decode_func)(const char *s, Py_ssize_t size, const char *errors)) {
    Py_ssize_t length;
    if (unlikely((start < 0) | (stop < 0))) {
        size_t slen = strlen(cstring);
        if (unlikely(slen > (size_t) PY_SSIZE_T_MAX)) {
            PyErr_SetString(PyExc_OverflowError,
                            "c-string too long to convert to Python");
            return NULL;
        }
        length = (Py_ssize_t) slen;
        if (start < 0) {
            start += length;
            if (start < 0)
                start = 0;
        }
        if (stop < 0)
            stop += length;
    }
    length = stop - start;
    if (unlikely(length <= 0))
        return PyUnicode_FromUnicode(NULL, 0);
    cstring += start;
    if (decode_func) {
        return decode_func(cstring, length, errors);
    } else {
        return PyUnicode_Decode(cstring, length, encoding, errors);
    }
}

/* ObjectGetItem */
      #if CYTHON_USE_TYPE_SLOTS
static PyObject *__Pyx_PyObject_GetIndex(PyObject *obj, PyObject* index) {
    PyObject *runerr;
    Py_ssize_t key_value;
    PySequenceMethods *m = Py_TYPE(obj)->tp_as_sequence;
    if (unlikely(!(m && m->sq_item))) {
        PyErr_Format(PyExc_TypeError, "'%.200s' object is not subscriptable", Py_TYPE(obj)->tp_name);
        return NULL;
    }
    key_value = __Pyx_PyIndex_AsSsize_t(index);
    if (likely(key_value != -1 || !(runerr = PyErr_Occurred()))) {
        return __Pyx_GetItemInt_Fast(obj, key_value, 0, 1, 1);
    }
    if (PyErr_GivenExceptionMatches(runerr, PyExc_OverflowError)) {
        PyErr_Clear();
        PyErr_Format(PyExc_IndexError, "cannot fit '%.200s' into an index-sized integer", Py_TYPE(index)->tp_name);
    }
    return NULL;
}
static PyObject *__Pyx_PyObject_GetItem(PyObject *obj, PyObject* key) {
    PyMappingMethods *m = Py_TYPE(obj)->tp_as_mapping;
    if (likely(m && m->mp_subscript)) {
        return m->mp_subscript(obj, key);
    }
    return __Pyx_PyObject_GetIndex(obj, key);
}
#endif

/* GetException */
      #if CYTHON_FAST_THREAD_STATE
static int __Pyx__GetException(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb) {
#else
static int __Pyx_GetException(PyObject **type, PyObject **value, PyObject **tb) {
#endif
    PyObject *local_type, *local_value, *local_tb;
#if CYTHON_FAST_THREAD_STATE
    PyObject *tmp_type, *tmp_value, *tmp_tb;
    local_type = tstate->curexc_type;
    local_value = tstate->curexc_value;
    local_tb = tstate->curexc_traceback;
    tstate->curexc_type = 0;
    tstate->curexc_value = 0;
    tstate->curexc_traceback = 0;
#else
    PyErr_Fetch(&local_type, &local_value, &local_tb);
#endif
    PyErr_NormalizeException(&local_type, &local_value, &local_tb);
#if CYTHON_FAST_THREAD_STATE
    if (unlikely(tstate->curexc_type))
#else
    if (unlikely(PyErr_Occurred()))
#endif
        goto bad;
    #if PY_MAJOR_VERSION >= 3
    if (local_tb) {
        if (unlikely(PyException_SetTraceback(local_value, local_tb) < 0))
            goto bad;
    }
    #endif
    Py_XINCREF(local_tb);
    Py_XINCREF(local_type);
    Py_XINCREF(local_value);
    *type = local_type;
    *value = local_value;
    *tb = local_tb;
#if CYTHON_FAST_THREAD_STATE
    #if PY_VERSION_HEX >= 0x030700A2
    tmp_type = tstate->exc_state.exc_type;
    tmp_value = tstate->exc_state.exc_value;
    tmp_tb = tstate->exc_state.exc_traceback;
    tstate->exc_state.exc_type = local_type;
    tstate->exc_state.exc_value = local_value;
    tstate->exc_state.exc_traceback = local_tb;
    #else
    tmp_type = tstate->exc_type;
    tmp_value = tstate->exc_value;
    tmp_tb = tstate->exc_traceback;
    tstate->exc_type = local_type;
    tstate->exc_value = local_value;
    tstate->exc_traceback = local_tb;
    #endif
    Py_XDECREF(tmp_type);
    Py_XDECREF(tmp_value);
    Py_XDECREF(tmp_tb);
#else
    PyErr_SetExcInfo(local_type, local_value, local_tb);
#endif
    return 0;
bad:
    *type = 0;
    *value = 0;
    *tb = 0;
    Py_XDECREF(local_type);
    Py_XDECREF(local_value);
    Py_XDECREF(local_tb);
    return -1;
}

/* SetItemInt */
        static int __Pyx_SetItemInt_Generic(PyObject *o, PyObject *j, PyObject *v) {
    int r;
    if (!j) return -1;
    r = PyObject_SetItem(o, j, v);
    Py_DECREF(j);
    return r;
}
static CYTHON_INLINE int __Pyx_SetItemInt_Fast(PyObject *o, Py_ssize_t i, PyObject *v, int is_list,
                                               CYTHON_NCP_UNUSED int wraparound, CYTHON_NCP_UNUSED int boundscheck) {
#if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS && CYTHON_USE_TYPE_SLOTS
    if (is_list || PyList_CheckExact(o)) {
        Py_ssize_t n = (!wraparound) ? i : ((likely(i >= 0)) ? i : i + PyList_GET_SIZE(o));
        if ((!boundscheck) || likely((n >= 0) & (n < PyList_GET_SIZE(o)))) {
            PyObject* old = PyList_GET_ITEM(o, n);
            Py_INCREF(v);
            PyList_SET_ITEM(o, n, v);
            Py_DECREF(old);
            return 1;
        }
    } else {
        PySequenceMethods *m = Py_TYPE(o)->tp_as_sequence;
        if (likely(m && m->sq_ass_item)) {
            if (wraparound && unlikely(i < 0) && likely(m->sq_length)) {
                Py_ssize_t l = m->sq_length(o);
                if (likely(l >= 0)) {
                    i += l;
                } else {
                    if (!PyErr_ExceptionMatches(PyExc_OverflowError))
                        return -1;
                    PyErr_Clear();
                }
            }
            return m->sq_ass_item(o, i, v);
        }
    }
#else
#if CYTHON_COMPILING_IN_PYPY
    if (is_list || (PySequence_Check(o) && !PyDict_Check(o))) {
#else
    if (is_list || PySequence_Check(o)) {
#endif
        return PySequence_SetItem(o, i, v);
    }
#endif
    return __Pyx_SetItemInt_Generic(o, PyInt_FromSsize_t(i), v);
}

/* BytesEquals */
          static CYTHON_INLINE int __Pyx_PyBytes_Equals(PyObject* s1, PyObject* s2, int equals) {
#if CYTHON_COMPILING_IN_PYPY
    return PyObject_RichCompareBool(s1, s2, equals);
#else
    if (s1 == s2) {
        return (equals == Py_EQ);
    } else if (PyBytes_CheckExact(s1) & PyBytes_CheckExact(s2)) {
        const char *ps1, *ps2;
        Py_ssize_t length = PyBytes_GET_SIZE(s1);
        if (length != PyBytes_GET_SIZE(s2))
            return (equals == Py_NE);
        ps1 = PyBytes_AS_STRING(s1);
        ps2 = PyBytes_AS_STRING(s2);
        if (ps1[0] != ps2[0]) {
            return (equals == Py_NE);
        } else if (length == 1) {
            return (equals == Py_EQ);
        } else {
            int result;
#if CYTHON_USE_UNICODE_INTERNALS
            Py_hash_t hash1, hash2;
            hash1 = ((PyBytesObject*)s1)->ob_shash;
            hash2 = ((PyBytesObject*)s2)->ob_shash;
            if (hash1 != hash2 && hash1 != -1 && hash2 != -1) {
                return (equals == Py_NE);
            }
#endif
            result = memcmp(ps1, ps2, (size_t)length);
            return (equals == Py_EQ) ? (result == 0) : (result != 0);
        }
    } else if ((s1 == Py_None) & PyBytes_CheckExact(s2)) {
        return (equals == Py_NE);
    } else if ((s2 == Py_None) & PyBytes_CheckExact(s1)) {
        return (equals == Py_NE);
    } else {
        int result;
        PyObject* py_result = PyObject_RichCompare(s1, s2, equals);
        if (!py_result)
            return -1;
        result = __Pyx_PyObject_IsTrue(py_result);
        Py_DECREF(py_result);
        return result;
    }
#endif
}

/* UnicodeEquals */
          static CYTHON_INLINE int __Pyx_PyUnicode_Equals(PyObject* s1, PyObject* s2, int equals) {
#if CYTHON_COMPILING_IN_PYPY
    return PyObject_RichCompareBool(s1, s2, equals);
#else
#if PY_MAJOR_VERSION < 3
    PyObject* owned_ref = NULL;
#endif
    int s1_is_unicode, s2_is_unicode;
    if (s1 == s2) {
        goto return_eq;
    }
    s1_is_unicode = PyUnicode_CheckExact(s1);
    s2_is_unicode = PyUnicode_CheckExact(s2);
#if PY_MAJOR_VERSION < 3
    if ((s1_is_unicode & (!s2_is_unicode)) && PyString_CheckExact(s2)) {
        owned_ref = PyUnicode_FromObject(s2);
        if (unlikely(!owned_ref))
            return -1;
        s2 = owned_ref;
        s2_is_unicode = 1;
    } else if ((s2_is_unicode & (!s1_is_unicode)) && PyString_CheckExact(s1)) {
        owned_ref = PyUnicode_FromObject(s1);
        if (unlikely(!owned_ref))
            return -1;
        s1 = owned_ref;
        s1_is_unicode = 1;
    } else if (((!s2_is_unicode) & (!s1_is_unicode))) {
        return __Pyx_PyBytes_Equals(s1, s2, equals);
    }
#endif
    if (s1_is_unicode & s2_is_unicode) {
        Py_ssize_t length;
        int kind;
        void *data1, *data2;
        if (unlikely(__Pyx_PyUnicode_READY(s1) < 0) || unlikely(__Pyx_PyUnicode_READY(s2) < 0))
            return -1;
        length = __Pyx_PyUnicode_GET_LENGTH(s1);
        if (length != __Pyx_PyUnicode_GET_LENGTH(s2)) {
            goto return_ne;
        }
#if CYTHON_USE_UNICODE_INTERNALS
        {
            Py_hash_t hash1, hash2;
        #if CYTHON_PEP393_ENABLED
            hash1 = ((PyASCIIObject*)s1)->hash;
            hash2 = ((PyASCIIObject*)s2)->hash;
        #else
            hash1 = ((PyUnicodeObject*)s1)->hash;
            hash2 = ((PyUnicodeObject*)s2)->hash;
        #endif
            if (hash1 != hash2 && hash1 != -1 && hash2 != -1) {
                goto return_ne;
            }
        }
#endif
        kind = __Pyx_PyUnicode_KIND(s1);
        if (kind != __Pyx_PyUnicode_KIND(s2)) {
            goto return_ne;
        }
        data1 = __Pyx_PyUnicode_DATA(s1);
        data2 = __Pyx_PyUnicode_DATA(s2);
        if (__Pyx_PyUnicode_READ(kind, data1, 0) != __Pyx_PyUnicode_READ(kind, data2, 0)) {
            goto return_ne;
        } else if (length == 1) {
            goto return_eq;
        } else {
            int result = memcmp(data1, data2, (size_t)(length * kind));
            #if PY_MAJOR_VERSION < 3
            Py_XDECREF(owned_ref);
            #endif
            return (equals == Py_EQ) ? (result == 0) : (result != 0);
        }
    } else if ((s1 == Py_None) & s2_is_unicode) {
        goto return_ne;
    } else if ((s2 == Py_None) & s1_is_unicode) {
        goto return_ne;
    } else {
        int result;
        PyObject* py_result = PyObject_RichCompare(s1, s2, equals);
        if (!py_result)
            return -1;
        result = __Pyx_PyObject_IsTrue(py_result);
        Py_DECREF(py_result);
        return result;
    }
return_eq:
    #if PY_MAJOR_VERSION < 3
    Py_XDECREF(owned_ref);
    #endif
    return (equals == Py_EQ);
return_ne:
    #if PY_MAJOR_VERSION < 3
    Py_XDECREF(owned_ref);
    #endif
    return (equals == Py_NE);
#endif
}

/* FastTypeChecks */
          #if CYTHON_COMPILING_IN_CPYTHON
static int __Pyx_InBases(PyTypeObject *a, PyTypeObject *b) {
    while (a) {
        a = a->tp_base;
        if (a == b)
            return 1;
    }
    return b == &PyBaseObject_Type;
}
static CYTHON_INLINE int __Pyx_IsSubtype(PyTypeObject *a, PyTypeObject *b) {
    PyObject *mro;
    if (a == b) return 1;
    mro = a->tp_mro;
    if (likely(mro)) {
        Py_ssize_t i, n;
        n = PyTuple_GET_SIZE(mro);
        for (i = 0; i < n; i++) {
            if (PyTuple_GET_ITEM(mro, i) == (PyObject *)b)
                return 1;
        }
        return 0;
    }
    return __Pyx_InBases(a, b);
}
#if PY_MAJOR_VERSION == 2
static int __Pyx_inner_PyErr_GivenExceptionMatches2(PyObject *err, PyObject* exc_type1, PyObject* exc_type2) {
    PyObject *exception, *value, *tb;
    int res;
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    __Pyx_ErrFetch(&exception, &value, &tb);
    res = exc_type1 ? PyObject_IsSubclass(err, exc_type1) : 0;
    if (unlikely(res == -1)) {
        PyErr_WriteUnraisable(err);
        res = 0;
    }
    if (!res) {
        res = PyObject_IsSubclass(err, exc_type2);
        if (unlikely(res == -1)) {
            PyErr_WriteUnraisable(err);
            res = 0;
        }
    }
    __Pyx_ErrRestore(exception, value, tb);
    return res;
}
#else
static CYTHON_INLINE int __Pyx_inner_PyErr_GivenExceptionMatches2(PyObject *err, PyObject* exc_type1, PyObject *exc_type2) {
    int res = exc_type1 ? __Pyx_IsSubtype((PyTypeObject*)err, (PyTypeObject*)exc_type1) : 0;
    if (!res) {
        res = __Pyx_IsSubtype((PyTypeObject*)err, (PyTypeObject*)exc_type2);
    }
    return res;
}
#endif
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches(PyObject *err, PyObject* exc_type) {
    if (likely(err == exc_type)) return 1;
    if (likely(PyExceptionClass_Check(err))) {
        return __Pyx_inner_PyErr_GivenExceptionMatches2(err, NULL, exc_type);
    }
    return PyErr_GivenExceptionMatches(err, exc_type);
}
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches2(PyObject *err, PyObject *exc_type1, PyObject *exc_type2) {
    if (likely(err == exc_type1 || err == exc_type2)) return 1;
    if (likely(PyExceptionClass_Check(err))) {
        return __Pyx_inner_PyErr_GivenExceptionMatches2(err, exc_type1, exc_type2);
    }
    return (PyErr_GivenExceptionMatches(err, exc_type1) || PyErr_GivenExceptionMatches(err, exc_type2));
}
#endif

/* CallUnboundCMethod1 */
          #if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_CallUnboundCMethod1(__Pyx_CachedCFunction* cfunc, PyObject* self, PyObject* arg) {
    if (likely(cfunc->func)) {
        int flag = cfunc->flag;
        if (flag == METH_O) {
            return (*(cfunc->func))(self, arg);
        } else if (PY_VERSION_HEX >= 0x030600B1 && flag == METH_FASTCALL) {
            if (PY_VERSION_HEX >= 0x030700A0) {
                return (*(__Pyx_PyCFunctionFast)cfunc->func)(self, &arg, 1);
            } else {
                return (*(__Pyx_PyCFunctionFastWithKeywords)cfunc->func)(self, &arg, 1, NULL);
            }
        } else if (PY_VERSION_HEX >= 0x030700A0 && flag == (METH_FASTCALL | METH_KEYWORDS)) {
            return (*(__Pyx_PyCFunctionFastWithKeywords)cfunc->func)(self, &arg, 1, NULL);
        }
    }
    return __Pyx__CallUnboundCMethod1(cfunc, self, arg);
}
#endif
static PyObject* __Pyx__CallUnboundCMethod1(__Pyx_CachedCFunction* cfunc, PyObject* self, PyObject* arg){
    PyObject *args, *result = NULL;
    if (unlikely(!cfunc->func && !cfunc->method) && unlikely(__Pyx_TryUnpackUnboundCMethod(cfunc) < 0)) return NULL;
#if CYTHON_COMPILING_IN_CPYTHON
    if (cfunc->func && (cfunc->flag & METH_VARARGS)) {
        args = PyTuple_New(1);
        if (unlikely(!args)) goto bad;
        Py_INCREF(arg);
        PyTuple_SET_ITEM(args, 0, arg);
        if (cfunc->flag & METH_KEYWORDS)
            result = (*(PyCFunctionWithKeywords)cfunc->func)(self, args, NULL);
        else
            result = (*cfunc->func)(self, args);
    } else {
        args = PyTuple_New(2);
        if (unlikely(!args)) goto bad;
        Py_INCREF(self);
        PyTuple_SET_ITEM(args, 0, self);
        Py_INCREF(arg);
        PyTuple_SET_ITEM(args, 1, arg);
        result = __Pyx_PyObject_Call(cfunc->method, args, NULL);
    }
#else
    args = PyTuple_Pack(2, self, arg);
    if (unlikely(!args)) goto bad;
    result = __Pyx_PyObject_Call(cfunc->method, args, NULL);
#endif
bad:
    Py_XDECREF(args);
    return result;
}

/* None */
          static CYTHON_INLINE void __Pyx_RaiseUnboundLocalError(const char *varname) {
    PyErr_Format(PyExc_UnboundLocalError, "local variable '%s' referenced before assignment", varname);
}

/* PyIntBinop */
          #if !CYTHON_COMPILING_IN_PYPY
static PyObject* __Pyx_PyInt_EqObjC(PyObject *op1, PyObject *op2, CYTHON_UNUSED long intval, CYTHON_UNUSED int inplace) {
    if (op1 == op2) {
        Py_RETURN_TRUE;
    }
    #if PY_MAJOR_VERSION < 3
    if (likely(PyInt_CheckExact(op1))) {
        const long b = intval;
        long a = PyInt_AS_LONG(op1);
        if (a == b) {
            Py_RETURN_TRUE;
        } else {
            Py_RETURN_FALSE;
        }
    }
    #endif
    #if CYTHON_USE_PYLONG_INTERNALS
    if (likely(PyLong_CheckExact(op1))) {
        const long b = intval;
        long a;
        const digit* digits = ((PyLongObject*)op1)->ob_digit;
        const Py_ssize_t size = Py_SIZE(op1);
        if (likely(__Pyx_sst_abs(size) <= 1)) {
            a = likely(size) ? digits[0] : 0;
            if (size == -1) a = -a;
        } else {
            switch (size) {
                case -2:
                    if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                        a = -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
                    }
                    CYTHON_FALLTHROUGH;
                case 2:
                    if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                        a = (long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
                    }
                    CYTHON_FALLTHROUGH;
                case -3:
                    if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                        a = -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
                    }
                    CYTHON_FALLTHROUGH;
                case 3:
                    if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                        a = (long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
                    }
                    CYTHON_FALLTHROUGH;
                case -4:
                    if (8 * sizeof(long) - 1 > 4 * PyLong_SHIFT) {
                        a = -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
                    }
                    CYTHON_FALLTHROUGH;
                case 4:
                    if (8 * sizeof(long) - 1 > 4 * PyLong_SHIFT) {
                        a = (long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
                    }
                    CYTHON_FALLTHROUGH;
                #if PyLong_SHIFT < 30 && PyLong_SHIFT != 15
                default: return PyLong_Type.tp_richcompare(op1, op2, Py_EQ);
                #else
                default: Py_RETURN_FALSE;
                #endif
            }
        }
            if (a == b) {
                Py_RETURN_TRUE;
            } else {
                Py_RETURN_FALSE;
            }
    }
    #endif
    if (PyFloat_CheckExact(op1)) {
        const long b = intval;
        double a = PyFloat_AS_DOUBLE(op1);
            if ((double)a == (double)b) {
                Py_RETURN_TRUE;
            } else {
                Py_RETURN_FALSE;
            }
    }
    return PyObject_RichCompare(op1, op2, Py_EQ);
}
#endif

/* GetAttr */
          static CYTHON_INLINE PyObject *__Pyx_GetAttr(PyObject *o, PyObject *n) {
#if CYTHON_USE_TYPE_SLOTS
#if PY_MAJOR_VERSION >= 3
    if (likely(PyUnicode_Check(n)))
#else
    if (likely(PyString_Check(n)))
#endif
        return __Pyx_PyObject_GetAttrStr(o, n);
#endif
    return PyObject_GetAttr(o, n);
}

/* GetAttr3 */
          static PyObject *__Pyx_GetAttr3Default(PyObject *d) {
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    if (unlikely(!__Pyx_PyErr_ExceptionMatches(PyExc_AttributeError)))
        return NULL;
    __Pyx_PyErr_Clear();
    Py_INCREF(d);
    return d;
}
static CYTHON_INLINE PyObject *__Pyx_GetAttr3(PyObject *o, PyObject *n, PyObject *d) {
    PyObject *r = __Pyx_GetAttr(o, n);
    return (likely(r)) ? r : __Pyx_GetAttr3Default(d);
}

/* Import */
          static PyObject *__Pyx_Import(PyObject *name, PyObject *from_list, int level) {
    PyObject *empty_list = 0;
    PyObject *module = 0;
    PyObject *global_dict = 0;
    PyObject *empty_dict = 0;
    PyObject *list;
    #if PY_MAJOR_VERSION < 3
    PyObject *py_import;
    py_import = __Pyx_PyObject_GetAttrStr(__pyx_b, __pyx_n_s_import);
    if (!py_import)
        goto bad;
    #endif
    if (from_list)
        list = from_list;
    else {
        empty_list = PyList_New(0);
        if (!empty_list)
            goto bad;
        list = empty_list;
    }
    global_dict = PyModule_GetDict(__pyx_m);
    if (!global_dict)
        goto bad;
    empty_dict = PyDict_New();
    if (!empty_dict)
        goto bad;
    {
        #if PY_MAJOR_VERSION >= 3
        if (level == -1) {
            if (strchr(__Pyx_MODULE_NAME, '.')) {
                module = PyImport_ImportModuleLevelObject(
                    name, global_dict, empty_dict, list, 1);
                if (!module) {
                    if (!PyErr_ExceptionMatches(PyExc_ImportError))
                        goto bad;
                    PyErr_Clear();
                }
            }
            level = 0;
        }
        #endif
        if (!module) {
            #if PY_MAJOR_VERSION < 3
            PyObject *py_level = PyInt_FromLong(level);
            if (!py_level)
                goto bad;
            module = PyObject_CallFunctionObjArgs(py_import,
                name, global_dict, empty_dict, list, py_level, NULL);
            Py_DECREF(py_level);
            #else
            module = PyImport_ImportModuleLevelObject(
                name, global_dict, empty_dict, list, level);
            #endif
        }
    }
bad:
    #if PY_MAJOR_VERSION < 3
    Py_XDECREF(py_import);
    #endif
    Py_XDECREF(empty_list);
    Py_XDECREF(empty_dict);
    return module;
}

/* ImportFrom */
          static PyObject* __Pyx_ImportFrom(PyObject* module, PyObject* name) {
    PyObject* value = __Pyx_PyObject_GetAttrStr(module, name);
    if (unlikely(!value) && PyErr_ExceptionMatches(PyExc_AttributeError)) {
        PyErr_Format(PyExc_ImportError,
        #if PY_MAJOR_VERSION < 3
            "cannot import name %.230s", PyString_AS_STRING(name));
        #else
            "cannot import name %S", name);
        #endif
    }
    return value;
}

/* HasAttr */
          static CYTHON_INLINE int __Pyx_HasAttr(PyObject *o, PyObject *n) {
    PyObject *r;
    if (unlikely(!__Pyx_PyBaseString_Check(n))) {
        PyErr_SetString(PyExc_TypeError,
                        "hasattr(): attribute name must be string");
        return -1;
    }
    r = __Pyx_GetAttr(o, n);
    if (unlikely(!r)) {
        PyErr_Clear();
        return 0;
    } else {
        Py_DECREF(r);
        return 1;
    }
}

/* PyObject_GenericGetAttrNoDict */
          #if CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP && PY_VERSION_HEX < 0x03070000
static PyObject *__Pyx_RaiseGenericGetAttributeError(PyTypeObject *tp, PyObject *attr_name) {
    PyErr_Format(PyExc_AttributeError,
#if PY_MAJOR_VERSION >= 3
                 "'%.50s' object has no attribute '%U'",
                 tp->tp_name, attr_name);
#else
                 "'%.50s' object has no attribute '%.400s'",
                 tp->tp_name, PyString_AS_STRING(attr_name));
#endif
    return NULL;
}
static CYTHON_INLINE PyObject* __Pyx_PyObject_GenericGetAttrNoDict(PyObject* obj, PyObject* attr_name) {
    PyObject *descr;
    PyTypeObject *tp = Py_TYPE(obj);
    if (unlikely(!PyString_Check(attr_name))) {
        return PyObject_GenericGetAttr(obj, attr_name);
    }
    assert(!tp->tp_dictoffset);
    descr = _PyType_Lookup(tp, attr_name);
    if (unlikely(!descr)) {
        return __Pyx_RaiseGenericGetAttributeError(tp, attr_name);
    }
    Py_INCREF(descr);
    #if PY_MAJOR_VERSION < 3
    if (likely(PyType_HasFeature(Py_TYPE(descr), Py_TPFLAGS_HAVE_CLASS)))
    #endif
    {
        descrgetfunc f = Py_TYPE(descr)->tp_descr_get;
        if (unlikely(f)) {
            PyObject *res = f(descr, obj, (PyObject *)tp);
            Py_DECREF(descr);
            return res;
        }
    }
    return descr;
}
#endif

/* PyObject_GenericGetAttr */
          #if CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP && PY_VERSION_HEX < 0x03070000
static PyObject* __Pyx_PyObject_GenericGetAttr(PyObject* obj, PyObject* attr_name) {
    if (unlikely(Py_TYPE(obj)->tp_dictoffset)) {
        return PyObject_GenericGetAttr(obj, attr_name);
    }
    return __Pyx_PyObject_GenericGetAttrNoDict(obj, attr_name);
}
#endif

/* SetVTable */
          static int __Pyx_SetVtable(PyObject *dict, void *vtable) {
#if PY_VERSION_HEX >= 0x02070000
    PyObject *ob = PyCapsule_New(vtable, 0, 0);
#else
    PyObject *ob = PyCObject_FromVoidPtr(vtable, 0);
#endif
    if (!ob)
        goto bad;
    if (PyDict_SetItem(dict, __pyx_n_s_pyx_vtable, ob) < 0)
        goto bad;
    Py_DECREF(ob);
    return 0;
bad:
    Py_XDECREF(ob);
    return -1;
}

/* SetupReduce */
          static int __Pyx_setup_reduce_is_named(PyObject* meth, PyObject* name) {
  int ret;
  PyObject *name_attr;
  name_attr = __Pyx_PyObject_GetAttrStr(meth, __pyx_n_s_name_2);
  if (likely(name_attr)) {
      ret = PyObject_RichCompareBool(name_attr, name, Py_EQ);
  } else {
      ret = -1;
  }
  if (unlikely(ret < 0)) {
      PyErr_Clear();
      ret = 0;
  }
  Py_XDECREF(name_attr);
  return ret;
}
static int __Pyx_setup_reduce(PyObject* type_obj) {
    int ret = 0;
    PyObject *object_reduce = NULL;
    PyObject *object_reduce_ex = NULL;
    PyObject *reduce = NULL;
    PyObject *reduce_ex = NULL;
    PyObject *reduce_cython = NULL;
    PyObject *setstate = NULL;
    PyObject *setstate_cython = NULL;
#if CYTHON_USE_PYTYPE_LOOKUP
    if (_PyType_Lookup((PyTypeObject*)type_obj, __pyx_n_s_getstate)) goto GOOD;
#else
    if (PyObject_HasAttr(type_obj, __pyx_n_s_getstate)) goto GOOD;
#endif
#if CYTHON_USE_PYTYPE_LOOKUP
    object_reduce_ex = _PyType_Lookup(&PyBaseObject_Type, __pyx_n_s_reduce_ex); if (!object_reduce_ex) goto BAD;
#else
    object_reduce_ex = __Pyx_PyObject_GetAttrStr((PyObject*)&PyBaseObject_Type, __pyx_n_s_reduce_ex); if (!object_reduce_ex) goto BAD;
#endif
    reduce_ex = __Pyx_PyObject_GetAttrStr(type_obj, __pyx_n_s_reduce_ex); if (unlikely(!reduce_ex)) goto BAD;
    if (reduce_ex == object_reduce_ex) {
#if CYTHON_USE_PYTYPE_LOOKUP
        object_reduce = _PyType_Lookup(&PyBaseObject_Type, __pyx_n_s_reduce); if (!object_reduce) goto BAD;
#else
        object_reduce = __Pyx_PyObject_GetAttrStr((PyObject*)&PyBaseObject_Type, __pyx_n_s_reduce); if (!object_reduce) goto BAD;
#endif
        reduce = __Pyx_PyObject_GetAttrStr(type_obj, __pyx_n_s_reduce); if (unlikely(!reduce)) goto BAD;
        if (reduce == object_reduce || __Pyx_setup_reduce_is_named(reduce, __pyx_n_s_reduce_cython)) {
            reduce_cython = __Pyx_PyObject_GetAttrStr(type_obj, __pyx_n_s_reduce_cython); if (unlikely(!reduce_cython)) goto BAD;
            ret = PyDict_SetItem(((PyTypeObject*)type_obj)->tp_dict, __pyx_n_s_reduce, reduce_cython); if (unlikely(ret < 0)) goto BAD;
            ret = PyDict_DelItem(((PyTypeObject*)type_obj)->tp_dict, __pyx_n_s_reduce_cython); if (unlikely(ret < 0)) goto BAD;
            setstate = __Pyx_PyObject_GetAttrStr(type_obj, __pyx_n_s_setstate);
            if (!setstate) PyErr_Clear();
            if (!setstate || __Pyx_setup_reduce_is_named(setstate, __pyx_n_s_setstate_cython)) {
                setstate_cython = __Pyx_PyObject_GetAttrStr(type_obj, __pyx_n_s_setstate_cython); if (unlikely(!setstate_cython)) goto BAD;
                ret = PyDict_SetItem(((PyTypeObject*)type_obj)->tp_dict, __pyx_n_s_setstate, setstate_cython); if (unlikely(ret < 0)) goto BAD;
                ret = PyDict_DelItem(((PyTypeObject*)type_obj)->tp_dict, __pyx_n_s_setstate_cython); if (unlikely(ret < 0)) goto BAD;
            }
            PyType_Modified((PyTypeObject*)type_obj);
        }
    }
    goto GOOD;
BAD:
    if (!PyErr_Occurred())
        PyErr_Format(PyExc_RuntimeError, "Unable to initialize pickling for %s", ((PyTypeObject*)type_obj)->tp_name);
    ret = -1;
GOOD:
#if !CYTHON_USE_PYTYPE_LOOKUP
    Py_XDECREF(object_reduce);
    Py_XDECREF(object_reduce_ex);
#endif
    Py_XDECREF(reduce);
    Py_XDECREF(reduce_ex);
    Py_XDECREF(reduce_cython);
    Py_XDECREF(setstate);
    Py_XDECREF(setstate_cython);
    return ret;
}

/* CLineInTraceback */
          #ifndef CYTHON_CLINE_IN_TRACEBACK
static int __Pyx_CLineForTraceback(CYTHON_UNUSED PyThreadState *tstate, int c_line) {
    PyObject *use_cline;
    PyObject *ptype, *pvalue, *ptraceback;
#if CYTHON_COMPILING_IN_CPYTHON
    PyObject **cython_runtime_dict;
#endif
    __Pyx_ErrFetchInState(tstate, &ptype, &pvalue, &ptraceback);
#if CYTHON_COMPILING_IN_CPYTHON
    cython_runtime_dict = _PyObject_GetDictPtr(__pyx_cython_runtime);
    if (likely(cython_runtime_dict)) {
      use_cline = __Pyx_PyDict_GetItemStr(*cython_runtime_dict, __pyx_n_s_cline_in_traceback);
    } else
#endif
    {
      PyObject *use_cline_obj = __Pyx_PyObject_GetAttrStr(__pyx_cython_runtime, __pyx_n_s_cline_in_traceback);
      if (use_cline_obj) {
        use_cline = PyObject_Not(use_cline_obj) ? Py_False : Py_True;
        Py_DECREF(use_cline_obj);
      } else {
        PyErr_Clear();
        use_cline = NULL;
      }
    }
    if (!use_cline) {
        c_line = 0;
        PyObject_SetAttr(__pyx_cython_runtime, __pyx_n_s_cline_in_traceback, Py_False);
    }
    else if (PyObject_Not(use_cline) != 0) {
        c_line = 0;
    }
    __Pyx_ErrRestoreInState(tstate, ptype, pvalue, ptraceback);
    return c_line;
}
#endif

/* CodeObjectCache */
          static int __pyx_bisect_code_objects(__Pyx_CodeObjectCacheEntry* entries, int count, int code_line) {
    int start = 0, mid = 0, end = count - 1;
    if (end >= 0 && code_line > entries[end].code_line) {
        return count;
    }
    while (start < end) {
        mid = start + (end - start) / 2;
        if (code_line < entries[mid].code_line) {
            end = mid;
        } else if (code_line > entries[mid].code_line) {
             start = mid + 1;
        } else {
            return mid;
        }
    }
    if (code_line <= entries[mid].code_line) {
        return mid;
    } else {
        return mid + 1;
    }
}
static PyCodeObject *__pyx_find_code_object(int code_line) {
    PyCodeObject* code_object;
    int pos;
    if (unlikely(!code_line) || unlikely(!__pyx_code_cache.entries)) {
        return NULL;
    }
    pos = __pyx_bisect_code_objects(__pyx_code_cache.entries, __pyx_code_cache.count, code_line);
    if (unlikely(pos >= __pyx_code_cache.count) || unlikely(__pyx_code_cache.entries[pos].code_line != code_line)) {
        return NULL;
    }
    code_object = __pyx_code_cache.entries[pos].code_object;
    Py_INCREF(code_object);
    return code_object;
}
static void __pyx_insert_code_object(int code_line, PyCodeObject* code_object) {
    int pos, i;
    __Pyx_CodeObjectCacheEntry* entries = __pyx_code_cache.entries;
    if (unlikely(!code_line)) {
        return;
    }
    if (unlikely(!entries)) {
        entries = (__Pyx_CodeObjectCacheEntry*)PyMem_Malloc(64*sizeof(__Pyx_CodeObjectCacheEntry));
        if (likely(entries)) {
            __pyx_code_cache.entries = entries;
            __pyx_code_cache.max_count = 64;
            __pyx_code_cache.count = 1;
            entries[0].code_line = code_line;
            entries[0].code_object = code_object;
            Py_INCREF(code_object);
        }
        return;
    }
    pos = __pyx_bisect_code_objects(__pyx_code_cache.entries, __pyx_code_cache.count, code_line);
    if ((pos < __pyx_code_cache.count) && unlikely(__pyx_code_cache.entries[pos].code_line == code_line)) {
        PyCodeObject* tmp = entries[pos].code_object;
        entries[pos].code_object = code_object;
        Py_DECREF(tmp);
        return;
    }
    if (__pyx_code_cache.count == __pyx_code_cache.max_count) {
        int new_max = __pyx_code_cache.max_count + 64;
        entries = (__Pyx_CodeObjectCacheEntry*)PyMem_Realloc(
            __pyx_code_cache.entries, (size_t)new_max*sizeof(__Pyx_CodeObjectCacheEntry));
        if (unlikely(!entries)) {
            return;
        }
        __pyx_code_cache.entries = entries;
        __pyx_code_cache.max_count = new_max;
    }
    for (i=__pyx_code_cache.count; i>pos; i--) {
        entries[i] = entries[i-1];
    }
    entries[pos].code_line = code_line;
    entries[pos].code_object = code_object;
    __pyx_code_cache.count++;
    Py_INCREF(code_object);
}

/* AddTraceback */
          #include "compile.h"
#include "frameobject.h"
#include "traceback.h"
static PyCodeObject* __Pyx_CreateCodeObjectForTraceback(
            const char *funcname, int c_line,
            int py_line, const char *filename) {
    PyCodeObject *py_code = 0;
    PyObject *py_srcfile = 0;
    PyObject *py_funcname = 0;
    #if PY_MAJOR_VERSION < 3
    py_srcfile = PyString_FromString(filename);
    #else
    py_srcfile = PyUnicode_FromString(filename);
    #endif
    if (!py_srcfile) goto bad;
    if (c_line) {
        #if PY_MAJOR_VERSION < 3
        py_funcname = PyString_FromFormat( "%s (%s:%d)", funcname, __pyx_cfilenm, c_line);
        #else
        py_funcname = PyUnicode_FromFormat( "%s (%s:%d)", funcname, __pyx_cfilenm, c_line);
        #endif
    }
    else {
        #if PY_MAJOR_VERSION < 3
        py_funcname = PyString_FromString(funcname);
        #else
        py_funcname = PyUnicode_FromString(funcname);
        #endif
    }
    if (!py_funcname) goto bad;
    py_code = __Pyx_PyCode_New(
        0,
        0,
        0,
        0,
        0,
        __pyx_empty_bytes, /*PyObject *code,*/
        __pyx_empty_tuple, /*PyObject *consts,*/
        __pyx_empty_tuple, /*PyObject *names,*/
        __pyx_empty_tuple, /*PyObject *varnames,*/
        __pyx_empty_tuple, /*PyObject *freevars,*/
        __pyx_empty_tuple, /*PyObject *cellvars,*/
        py_srcfile,   /*PyObject *filename,*/
        py_funcname,  /*PyObject *name,*/
        py_line,
        __pyx_empty_bytes  /*PyObject *lnotab*/
    );
    Py_DECREF(py_srcfile);
    Py_DECREF(py_funcname);
    return py_code;
bad:
    Py_XDECREF(py_srcfile);
    Py_XDECREF(py_funcname);
    return NULL;
}
static void __Pyx_AddTraceback(const char *funcname, int c_line,
                               int py_line, const char *filename) {
    PyCodeObject *py_code = 0;
    PyFrameObject *py_frame = 0;
    PyThreadState *tstate = __Pyx_PyThreadState_Current;
    if (c_line) {
        c_line = __Pyx_CLineForTraceback(tstate, c_line);
    }
    py_code = __pyx_find_code_object(c_line ? -c_line : py_line);
    if (!py_code) {
        py_code = __Pyx_CreateCodeObjectForTraceback(
            funcname, c_line, py_line, filename);
        if (!py_code) goto bad;
        __pyx_insert_code_object(c_line ? -c_line : py_line, py_code);
    }
    py_frame = PyFrame_New(
        tstate,            /*PyThreadState *tstate,*/
        py_code,           /*PyCodeObject *code,*/
        __pyx_d,    /*PyObject *globals,*/
        0                  /*PyObject *locals*/
    );
    if (!py_frame) goto bad;
    __Pyx_PyFrame_SetLineNumber(py_frame, py_line);
    PyTraceBack_Here(py_frame);
bad:
    Py_XDECREF(py_code);
    Py_XDECREF(py_frame);
}

/* CIntToPy */
          static CYTHON_INLINE PyObject* __Pyx_PyInt_From_int(int value) {
    const int neg_one = (int) -1, const_zero = (int) 0;
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(int) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(int) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(int) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(int) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(int) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
#endif
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(int),
                                     little, !is_unsigned);
    }
}

/* CIntFromPyVerify */
          #define __PYX_VERIFY_RETURN_INT(target_type, func_type, func_value)\
    __PYX__VERIFY_RETURN_INT(target_type, func_type, func_value, 0)
#define __PYX_VERIFY_RETURN_INT_EXC(target_type, func_type, func_value)\
    __PYX__VERIFY_RETURN_INT(target_type, func_type, func_value, 1)
#define __PYX__VERIFY_RETURN_INT(target_type, func_type, func_value, exc)\
    {\
        func_type value = func_value;\
        if (sizeof(target_type) < sizeof(func_type)) {\
            if (unlikely(value != (func_type) (target_type) value)) {\
                func_type zero = 0;\
                if (exc && unlikely(value == (func_type)-1 && PyErr_Occurred()))\
                    return (target_type) -1;\
                if (is_unsigned && unlikely(value < zero))\
                    goto raise_neg_overflow;\
                else\
                    goto raise_overflow;\
            }\
        }\
        return (target_type) value;\
    }

/* CIntToPy */
          static CYTHON_INLINE PyObject* __Pyx_PyInt_From_long(long value) {
    const long neg_one = (long) -1, const_zero = (long) 0;
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(long) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(long) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(long) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(long) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(long) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
#endif
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(long),
                                     little, !is_unsigned);
    }
}

/* Print */
          #if !CYTHON_COMPILING_IN_PYPY && PY_MAJOR_VERSION < 3
static PyObject *__Pyx_GetStdout(void) {
    PyObject *f = PySys_GetObject((char *)"stdout");
    if (!f) {
        PyErr_SetString(PyExc_RuntimeError, "lost sys.stdout");
    }
    return f;
}
static int __Pyx_Print(PyObject* f, PyObject *arg_tuple, int newline) {
    int i;
    if (!f) {
        if (!(f = __Pyx_GetStdout()))
            return -1;
    }
    Py_INCREF(f);
    for (i=0; i < PyTuple_GET_SIZE(arg_tuple); i++) {
        PyObject* v;
        if (PyFile_SoftSpace(f, 1)) {
            if (PyFile_WriteString(" ", f) < 0)
                goto error;
        }
        v = PyTuple_GET_ITEM(arg_tuple, i);
        if (PyFile_WriteObject(v, f, Py_PRINT_RAW) < 0)
            goto error;
        if (PyString_Check(v)) {
            char *s = PyString_AsString(v);
            Py_ssize_t len = PyString_Size(v);
            if (len > 0) {
                switch (s[len-1]) {
                    case ' ': break;
                    case '\f': case '\r': case '\n': case '\t': case '\v':
                        PyFile_SoftSpace(f, 0);
                        break;
                    default:  break;
                }
            }
        }
    }
    if (newline) {
        if (PyFile_WriteString("\n", f) < 0)
            goto error;
        PyFile_SoftSpace(f, 0);
    }
    Py_DECREF(f);
    return 0;
error:
    Py_DECREF(f);
    return -1;
}
#else
static int __Pyx_Print(PyObject* stream, PyObject *arg_tuple, int newline) {
    PyObject* kwargs = 0;
    PyObject* result = 0;
    PyObject* end_string;
    if (unlikely(!__pyx_print)) {
        __pyx_print = PyObject_GetAttr(__pyx_b, __pyx_n_s_print);
        if (!__pyx_print)
            return -1;
    }
    if (stream) {
        kwargs = PyDict_New();
        if (unlikely(!kwargs))
            return -1;
        if (unlikely(PyDict_SetItem(kwargs, __pyx_n_s_file, stream) < 0))
            goto bad;
        if (!newline) {
            end_string = PyUnicode_FromStringAndSize(" ", 1);
            if (unlikely(!end_string))
                goto bad;
            if (PyDict_SetItem(kwargs, __pyx_n_s_end, end_string) < 0) {
                Py_DECREF(end_string);
                goto bad;
            }
            Py_DECREF(end_string);
        }
    } else if (!newline) {
        if (unlikely(!__pyx_print_kwargs)) {
            __pyx_print_kwargs = PyDict_New();
            if (unlikely(!__pyx_print_kwargs))
                return -1;
            end_string = PyUnicode_FromStringAndSize(" ", 1);
            if (unlikely(!end_string))
                return -1;
            if (PyDict_SetItem(__pyx_print_kwargs, __pyx_n_s_end, end_string) < 0) {
                Py_DECREF(end_string);
                return -1;
            }
            Py_DECREF(end_string);
        }
        kwargs = __pyx_print_kwargs;
    }
    result = PyObject_Call(__pyx_print, arg_tuple, kwargs);
    if (unlikely(kwargs) && (kwargs != __pyx_print_kwargs))
        Py_DECREF(kwargs);
    if (!result)
        return -1;
    Py_DECREF(result);
    return 0;
bad:
    if (kwargs != __pyx_print_kwargs)
        Py_XDECREF(kwargs);
    return -1;
}
#endif

/* CIntToPy */
          static CYTHON_INLINE PyObject* __Pyx_PyInt_From_short(short value) {
    const short neg_one = (short) -1, const_zero = (short) 0;
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(short) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(short) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(short) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(short) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(short) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
#endif
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(short),
                                     little, !is_unsigned);
    }
}

/* CIntFromPy */
          static CYTHON_INLINE int __Pyx_PyInt_As_int(PyObject *x) {
    const int neg_one = (int) -1, const_zero = (int) 0;
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(int) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(int, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (int) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (int) 0;
                case  1: __PYX_VERIFY_RETURN_INT(int, digit, digits[0])
                case 2:
                    if (8 * sizeof(int) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) >= 2 * PyLong_SHIFT) {
                            return (int) (((((int)digits[1]) << PyLong_SHIFT) | (int)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(int) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) >= 3 * PyLong_SHIFT) {
                            return (int) (((((((int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(int) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) >= 4 * PyLong_SHIFT) {
                            return (int) (((((((((int)digits[3]) << PyLong_SHIFT) | (int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0]));
                        }
                    }
                    break;
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (int) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if (sizeof(int) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT_EXC(int, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(int) <= sizeof(unsigned PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(int, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (int) 0;
                case -1: __PYX_VERIFY_RETURN_INT(int, sdigit, (sdigit) (-(sdigit)digits[0]))
                case  1: __PYX_VERIFY_RETURN_INT(int,  digit, +digits[0])
                case -2:
                    if (8 * sizeof(int) - 1 > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 2 * PyLong_SHIFT) {
                            return (int) (((int)-1)*(((((int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if (8 * sizeof(int) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 2 * PyLong_SHIFT) {
                            return (int) ((((((int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if (8 * sizeof(int) - 1 > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 3 * PyLong_SHIFT) {
                            return (int) (((int)-1)*(((((((int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(int) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 3 * PyLong_SHIFT) {
                            return (int) ((((((((int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if (8 * sizeof(int) - 1 > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 4 * PyLong_SHIFT) {
                            return (int) (((int)-1)*(((((((((int)digits[3]) << PyLong_SHIFT) | (int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(int) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 4 * PyLong_SHIFT) {
                            return (int) ((((((((((int)digits[3]) << PyLong_SHIFT) | (int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
            }
#endif
            if (sizeof(int) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT_EXC(int, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(int) <= sizeof(PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(int, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            int val;
            PyObject *v = __Pyx_PyNumber_IntOrLong(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (int) -1;
        }
    } else {
        int val;
        PyObject *tmp = __Pyx_PyNumber_IntOrLong(x);
        if (!tmp) return (int) -1;
        val = __Pyx_PyInt_As_int(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to int");
    return (int) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to int");
    return (int) -1;
}

/* CIntFromPy */
          static CYTHON_INLINE long __Pyx_PyInt_As_long(PyObject *x) {
    const long neg_one = (long) -1, const_zero = (long) 0;
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(long) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(long, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (long) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (long) 0;
                case  1: __PYX_VERIFY_RETURN_INT(long, digit, digits[0])
                case 2:
                    if (8 * sizeof(long) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) >= 2 * PyLong_SHIFT) {
                            return (long) (((((long)digits[1]) << PyLong_SHIFT) | (long)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(long) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) >= 3 * PyLong_SHIFT) {
                            return (long) (((((((long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(long) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) >= 4 * PyLong_SHIFT) {
                            return (long) (((((((((long)digits[3]) << PyLong_SHIFT) | (long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0]));
                        }
                    }
                    break;
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (long) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if (sizeof(long) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT_EXC(long, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(long) <= sizeof(unsigned PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(long, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (long) 0;
                case -1: __PYX_VERIFY_RETURN_INT(long, sdigit, (sdigit) (-(sdigit)digits[0]))
                case  1: __PYX_VERIFY_RETURN_INT(long,  digit, +digits[0])
                case -2:
                    if (8 * sizeof(long) - 1 > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                            return (long) (((long)-1)*(((((long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if (8 * sizeof(long) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                            return (long) ((((((long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                            return (long) (((long)-1)*(((((((long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(long) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                            return (long) ((((((((long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 4 * PyLong_SHIFT) {
                            return (long) (((long)-1)*(((((((((long)digits[3]) << PyLong_SHIFT) | (long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(long) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 4 * PyLong_SHIFT) {
                            return (long) ((((((((((long)digits[3]) << PyLong_SHIFT) | (long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
            }
#endif
            if (sizeof(long) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT_EXC(long, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(long) <= sizeof(PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(long, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            long val;
            PyObject *v = __Pyx_PyNumber_IntOrLong(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (long) -1;
        }
    } else {
        long val;
        PyObject *tmp = __Pyx_PyNumber_IntOrLong(x);
        if (!tmp) return (long) -1;
        val = __Pyx_PyInt_As_long(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to long");
    return (long) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to long");
    return (long) -1;
}

/* CIntFromPy */
          static CYTHON_INLINE short __Pyx_PyInt_As_short(PyObject *x) {
    const short neg_one = (short) -1, const_zero = (short) 0;
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(short) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(short, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (short) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (short) 0;
                case  1: __PYX_VERIFY_RETURN_INT(short, digit, digits[0])
                case 2:
                    if (8 * sizeof(short) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(short, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(short) >= 2 * PyLong_SHIFT) {
                            return (short) (((((short)digits[1]) << PyLong_SHIFT) | (short)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(short) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(short, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(short) >= 3 * PyLong_SHIFT) {
                            return (short) (((((((short)digits[2]) << PyLong_SHIFT) | (short)digits[1]) << PyLong_SHIFT) | (short)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(short) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(short, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(short) >= 4 * PyLong_SHIFT) {
                            return (short) (((((((((short)digits[3]) << PyLong_SHIFT) | (short)digits[2]) << PyLong_SHIFT) | (short)digits[1]) << PyLong_SHIFT) | (short)digits[0]));
                        }
                    }
                    break;
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (short) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if (sizeof(short) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT_EXC(short, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(short) <= sizeof(unsigned PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(short, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (short) 0;
                case -1: __PYX_VERIFY_RETURN_INT(short, sdigit, (sdigit) (-(sdigit)digits[0]))
                case  1: __PYX_VERIFY_RETURN_INT(short,  digit, +digits[0])
                case -2:
                    if (8 * sizeof(short) - 1 > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(short, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(short) - 1 > 2 * PyLong_SHIFT) {
                            return (short) (((short)-1)*(((((short)digits[1]) << PyLong_SHIFT) | (short)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if (8 * sizeof(short) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(short, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(short) - 1 > 2 * PyLong_SHIFT) {
                            return (short) ((((((short)digits[1]) << PyLong_SHIFT) | (short)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if (8 * sizeof(short) - 1 > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(short, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(short) - 1 > 3 * PyLong_SHIFT) {
                            return (short) (((short)-1)*(((((((short)digits[2]) << PyLong_SHIFT) | (short)digits[1]) << PyLong_SHIFT) | (short)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(short) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(short, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(short) - 1 > 3 * PyLong_SHIFT) {
                            return (short) ((((((((short)digits[2]) << PyLong_SHIFT) | (short)digits[1]) << PyLong_SHIFT) | (short)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if (8 * sizeof(short) - 1 > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(short, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(short) - 1 > 4 * PyLong_SHIFT) {
                            return (short) (((short)-1)*(((((((((short)digits[3]) << PyLong_SHIFT) | (short)digits[2]) << PyLong_SHIFT) | (short)digits[1]) << PyLong_SHIFT) | (short)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(short) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(short, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(short) - 1 > 4 * PyLong_SHIFT) {
                            return (short) ((((((((((short)digits[3]) << PyLong_SHIFT) | (short)digits[2]) << PyLong_SHIFT) | (short)digits[1]) << PyLong_SHIFT) | (short)digits[0])));
                        }
                    }
                    break;
            }
#endif
            if (sizeof(short) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT_EXC(short, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(short) <= sizeof(PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(short, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            short val;
            PyObject *v = __Pyx_PyNumber_IntOrLong(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (short) -1;
        }
    } else {
        short val;
        PyObject *tmp = __Pyx_PyNumber_IntOrLong(x);
        if (!tmp) return (short) -1;
        val = __Pyx_PyInt_As_short(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to short");
    return (short) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to short");
    return (short) -1;
}

/* PrintOne */
          #if !CYTHON_COMPILING_IN_PYPY && PY_MAJOR_VERSION < 3
static int __Pyx_PrintOne(PyObject* f, PyObject *o) {
    if (!f) {
        if (!(f = __Pyx_GetStdout()))
            return -1;
    }
    Py_INCREF(f);
    if (PyFile_SoftSpace(f, 0)) {
        if (PyFile_WriteString(" ", f) < 0)
            goto error;
    }
    if (PyFile_WriteObject(o, f, Py_PRINT_RAW) < 0)
        goto error;
    if (PyFile_WriteString("\n", f) < 0)
        goto error;
    Py_DECREF(f);
    return 0;
error:
    Py_DECREF(f);
    return -1;
    /* the line below is just to avoid C compiler
     * warnings about unused functions */
    return __Pyx_Print(f, NULL, 0);
}
#else
static int __Pyx_PrintOne(PyObject* stream, PyObject *o) {
    int res;
    PyObject* arg_tuple = PyTuple_Pack(1, o);
    if (unlikely(!arg_tuple))
        return -1;
    res = __Pyx_Print(stream, arg_tuple, 1);
    Py_DECREF(arg_tuple);
    return res;
}
#endif

/* CheckBinaryVersion */
          static int __Pyx_check_binary_version(void) {
    char ctversion[4], rtversion[4];
    PyOS_snprintf(ctversion, 4, "%d.%d", PY_MAJOR_VERSION, PY_MINOR_VERSION);
    PyOS_snprintf(rtversion, 4, "%s", Py_GetVersion());
    if (ctversion[0] != rtversion[0] || ctversion[2] != rtversion[2]) {
        char message[200];
        PyOS_snprintf(message, sizeof(message),
                      "compiletime version %s of module '%.100s' "
                      "does not match runtime version %s",
                      ctversion, __Pyx_MODULE_NAME, rtversion);
        return PyErr_WarnEx(NULL, message, 1);
    }
    return 0;
}

/* InitStrings */
          static int __Pyx_InitStrings(__Pyx_StringTabEntry *t) {
    while (t->p) {
        #if PY_MAJOR_VERSION < 3
        if (t->is_unicode) {
            *t->p = PyUnicode_DecodeUTF8(t->s, t->n - 1, NULL);
        } else if (t->intern) {
            *t->p = PyString_InternFromString(t->s);
        } else {
            *t->p = PyString_FromStringAndSize(t->s, t->n - 1);
        }
        #else
        if (t->is_unicode | t->is_str) {
            if (t->intern) {
                *t->p = PyUnicode_InternFromString(t->s);
            } else if (t->encoding) {
                *t->p = PyUnicode_Decode(t->s, t->n - 1, t->encoding, NULL);
            } else {
                *t->p = PyUnicode_FromStringAndSize(t->s, t->n - 1);
            }
        } else {
            *t->p = PyBytes_FromStringAndSize(t->s, t->n - 1);
        }
        #endif
        if (!*t->p)
            return -1;
        if (PyObject_Hash(*t->p) == -1)
            return -1;
        ++t;
    }
    return 0;
}

static CYTHON_INLINE PyObject* __Pyx_PyUnicode_FromString(const char* c_str) {
    return __Pyx_PyUnicode_FromStringAndSize(c_str, (Py_ssize_t)strlen(c_str));
}
static CYTHON_INLINE const char* __Pyx_PyObject_AsString(PyObject* o) {
    Py_ssize_t ignore;
    return __Pyx_PyObject_AsStringAndSize(o, &ignore);
}
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII || __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT
#if !CYTHON_PEP393_ENABLED
static const char* __Pyx_PyUnicode_AsStringAndSize(PyObject* o, Py_ssize_t *length) {
    char* defenc_c;
    PyObject* defenc = _PyUnicode_AsDefaultEncodedString(o, NULL);
    if (!defenc) return NULL;
    defenc_c = PyBytes_AS_STRING(defenc);
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
    {
        char* end = defenc_c + PyBytes_GET_SIZE(defenc);
        char* c;
        for (c = defenc_c; c < end; c++) {
            if ((unsigned char) (*c) >= 128) {
                PyUnicode_AsASCIIString(o);
                return NULL;
            }
        }
    }
#endif
    *length = PyBytes_GET_SIZE(defenc);
    return defenc_c;
}
#else
static CYTHON_INLINE const char* __Pyx_PyUnicode_AsStringAndSize(PyObject* o, Py_ssize_t *length) {
    if (unlikely(__Pyx_PyUnicode_READY(o) == -1)) return NULL;
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
    if (likely(PyUnicode_IS_ASCII(o))) {
        *length = PyUnicode_GET_LENGTH(o);
        return PyUnicode_AsUTF8(o);
    } else {
        PyUnicode_AsASCIIString(o);
        return NULL;
    }
#else
    return PyUnicode_AsUTF8AndSize(o, length);
#endif
}
#endif
#endif
static CYTHON_INLINE const char* __Pyx_PyObject_AsStringAndSize(PyObject* o, Py_ssize_t *length) {
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII || __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT
    if (
#if PY_MAJOR_VERSION < 3 && __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
            __Pyx_sys_getdefaultencoding_not_ascii &&
#endif
            PyUnicode_Check(o)) {
        return __Pyx_PyUnicode_AsStringAndSize(o, length);
    } else
#endif
#if (!CYTHON_COMPILING_IN_PYPY) || (defined(PyByteArray_AS_STRING) && defined(PyByteArray_GET_SIZE))
    if (PyByteArray_Check(o)) {
        *length = PyByteArray_GET_SIZE(o);
        return PyByteArray_AS_STRING(o);
    } else
#endif
    {
        char* result;
        int r = PyBytes_AsStringAndSize(o, &result, length);
        if (unlikely(r < 0)) {
            return NULL;
        } else {
            return result;
        }
    }
}
static CYTHON_INLINE int __Pyx_PyObject_IsTrue(PyObject* x) {
   int is_true = x == Py_True;
   if (is_true | (x == Py_False) | (x == Py_None)) return is_true;
   else return PyObject_IsTrue(x);
}
static PyObject* __Pyx_PyNumber_IntOrLongWrongResultType(PyObject* result, const char* type_name) {
#if PY_MAJOR_VERSION >= 3
    if (PyLong_Check(result)) {
        if (PyErr_WarnFormat(PyExc_DeprecationWarning, 1,
                "__int__ returned non-int (type %.200s).  "
                "The ability to return an instance of a strict subclass of int "
                "is deprecated, and may be removed in a future version of Python.",
                Py_TYPE(result)->tp_name)) {
            Py_DECREF(result);
            return NULL;
        }
        return result;
    }
#endif
    PyErr_Format(PyExc_TypeError,
                 "__%.4s__ returned non-%.4s (type %.200s)",
                 type_name, type_name, Py_TYPE(result)->tp_name);
    Py_DECREF(result);
    return NULL;
}
static CYTHON_INLINE PyObject* __Pyx_PyNumber_IntOrLong(PyObject* x) {
#if CYTHON_USE_TYPE_SLOTS
  PyNumberMethods *m;
#endif
  const char *name = NULL;
  PyObject *res = NULL;
#if PY_MAJOR_VERSION < 3
  if (likely(PyInt_Check(x) || PyLong_Check(x)))
#else
  if (likely(PyLong_Check(x)))
#endif
    return __Pyx_NewRef(x);
#if CYTHON_USE_TYPE_SLOTS
  m = Py_TYPE(x)->tp_as_number;
  #if PY_MAJOR_VERSION < 3
  if (m && m->nb_int) {
    name = "int";
    res = m->nb_int(x);
  }
  else if (m && m->nb_long) {
    name = "long";
    res = m->nb_long(x);
  }
  #else
  if (likely(m && m->nb_int)) {
    name = "int";
    res = m->nb_int(x);
  }
  #endif
#else
  if (!PyBytes_CheckExact(x) && !PyUnicode_CheckExact(x)) {
    res = PyNumber_Int(x);
  }
#endif
  if (likely(res)) {
#if PY_MAJOR_VERSION < 3
    if (unlikely(!PyInt_Check(res) && !PyLong_Check(res))) {
#else
    if (unlikely(!PyLong_CheckExact(res))) {
#endif
        return __Pyx_PyNumber_IntOrLongWrongResultType(res, name);
    }
  }
  else if (!PyErr_Occurred()) {
    PyErr_SetString(PyExc_TypeError,
                    "an integer is required");
  }
  return res;
}
static CYTHON_INLINE Py_ssize_t __Pyx_PyIndex_AsSsize_t(PyObject* b) {
  Py_ssize_t ival;
  PyObject *x;
#if PY_MAJOR_VERSION < 3
  if (likely(PyInt_CheckExact(b))) {
    if (sizeof(Py_ssize_t) >= sizeof(long))
        return PyInt_AS_LONG(b);
    else
        return PyInt_AsSsize_t(x);
  }
#endif
  if (likely(PyLong_CheckExact(b))) {
    #if CYTHON_USE_PYLONG_INTERNALS
    const digit* digits = ((PyLongObject*)b)->ob_digit;
    const Py_ssize_t size = Py_SIZE(b);
    if (likely(__Pyx_sst_abs(size) <= 1)) {
        ival = likely(size) ? digits[0] : 0;
        if (size == -1) ival = -ival;
        return ival;
    } else {
      switch (size) {
         case 2:
           if (8 * sizeof(Py_ssize_t) > 2 * PyLong_SHIFT) {
             return (Py_ssize_t) (((((size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case -2:
           if (8 * sizeof(Py_ssize_t) > 2 * PyLong_SHIFT) {
             return -(Py_ssize_t) (((((size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case 3:
           if (8 * sizeof(Py_ssize_t) > 3 * PyLong_SHIFT) {
             return (Py_ssize_t) (((((((size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case -3:
           if (8 * sizeof(Py_ssize_t) > 3 * PyLong_SHIFT) {
             return -(Py_ssize_t) (((((((size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case 4:
           if (8 * sizeof(Py_ssize_t) > 4 * PyLong_SHIFT) {
             return (Py_ssize_t) (((((((((size_t)digits[3]) << PyLong_SHIFT) | (size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case -4:
           if (8 * sizeof(Py_ssize_t) > 4 * PyLong_SHIFT) {
             return -(Py_ssize_t) (((((((((size_t)digits[3]) << PyLong_SHIFT) | (size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
      }
    }
    #endif
    return PyLong_AsSsize_t(b);
  }
  x = PyNumber_Index(b);
  if (!x) return -1;
  ival = PyInt_AsSsize_t(x);
  Py_DECREF(x);
  return ival;
}
static CYTHON_INLINE PyObject * __Pyx_PyInt_FromSize_t(size_t ival) {
    return PyInt_FromSize_t(ival);
}


#endif /* Py_PYTHON_H */
